// MIT License

// Copyright (c) 2024 Joao Chrisostomo, Kacper Michajłow

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

//!DESC ArtCNN C4F32 DS (Conv2D)
//!COMPUTE 48 32 12 16
//!HOOK LUMA
//!BIND LUMA
//!SAVE conv2d
//!WIDTH LUMA.w 4.0 *
//!HEIGHT LUMA.h 2.0 *
//!COMPONENTS 4
//!WHEN OUTPUT.w LUMA.w / 1.3 > OUTPUT.h LUMA.h / 1.3 > *
#extension GL_EXT_shader_explicit_arithmetic_types_float16 : enable
#ifdef GL_EXT_shader_explicit_arithmetic_types_float16
#	define V4 f16vec4
#	define M4 f16mat4
#	define F float16_t
#else
#	define V4 vec4
#	define M4 mat4
#	define F float
#endif

const ivec2 ksize = ivec2(3, 3);
const ivec2 offset = ksize / 2;
const ivec2 wg_size = ivec2(gl_WorkGroupSize);
const ivec2 isize = wg_size + ksize - 1;
shared F inp[1][isize.y][isize.x];
void hook() {
    const uvec2 local_xy = gl_LocalInvocationID.xy;
    ivec2 base = ivec2(gl_WorkGroupID) * wg_size;
    for (uint y = local_xy.y; y < isize.y; y += wg_size.y) {
        for (uint x = local_xy.x; x < isize.x; x += wg_size.x) {
            const ivec2 input_base = (base + ivec2(x,y) - offset) * ivec2(1, 1);
            inp[0][y][x] = F(LUMA_mul * texelFetch(LUMA_raw, input_base + ivec2(0, 0), 0).x);
        }
    }

    barrier();
    V4 result0 = V4(0.00089311425, 0.002499194, -0.004786538, -0.08860893);
    V4 result1 = V4(-0.0015083052, 0.0019296855, -0.0014606133, 0.0006201888);
    V4 result2 = V4(0.001752575, 0.0041350685, -0.0011413988, 0.004784151);
    V4 result3 = V4(-0.0035739949, -0.082944125, 4.586902e-05, 0.0013953715);
    V4 result4 = V4(0.0002710992, 0.0008978144, 0.0004965891, -0.00094145624);
    V4 result5 = V4(-0.0010284873, 0.0013667014, 0.00038797423, 0.00029513543);
    V4 result6 = V4(-0.002057877, 0.00054591236, 0.030937938, 0.040469307);
    V4 result7 = V4(-0.003917487, -0.00083294045, 0.022597028, -0.0010706473);
    const F inp_0_0_0 = inp[0][local_xy.y + 0][local_xy.x + 0];
    const F inp_0_1_0 = inp[0][local_xy.y + 0][local_xy.x + 1];
    const F inp_0_2_0 = inp[0][local_xy.y + 0][local_xy.x + 2];
    const F inp_0_0_1 = inp[0][local_xy.y + 1][local_xy.x + 0];
    const F inp_0_1_1 = inp[0][local_xy.y + 1][local_xy.x + 1];
    const F inp_0_2_1 = inp[0][local_xy.y + 1][local_xy.x + 2];
    const F inp_0_0_2 = inp[0][local_xy.y + 2][local_xy.x + 0];
    const F inp_0_1_2 = inp[0][local_xy.y + 2][local_xy.x + 1];
    const F inp_0_2_2 = inp[0][local_xy.y + 2][local_xy.x + 2];
    result0 += V4(-0.035958596, -0.060004253, 0.03943403, -0.045566887) * inp_0_0_0;
    result0 += V4(0.044507273, 0.17763752, -0.04503674, -0.016751116) * inp_0_1_0;
    result0 += V4(-0.010687862, -0.12798516, 0.025843065, 0.03014903) * inp_0_2_0;
    result0 += V4(0.030451322, 0.0077890065, -0.014686611, -0.23188043) * inp_0_0_1;
    result0 += V4(-0.030106366, -0.041448865, -0.00767549, 0.13158405) * inp_0_1_1;
    result0 += V4(0.0051620533, 0.0389417, 0.0060597523, 0.14171156) * inp_0_2_1;
    result0 += V4(-0.0046348735, 0.066254914, -0.020437688, -0.0009821887) * inp_0_0_2;
    result0 += V4(-0.011564222, -0.13914865, 0.05816083, 0.07022447) * inp_0_1_2;
    result0 += V4(0.011584058, 0.07486852, -0.03373219, 0.083574474) * inp_0_2_2;
    result1 += V4(0.00062077807, 0.024263378, 0.089257665, 0.10184684) * inp_0_0_0;
    result1 += V4(0.038053896, -0.028899275, 0.051053822, -0.09761824) * inp_0_1_0;
    result1 += V4(-0.03485628, 0.0067617865, -0.14220814, -0.013733605) * inp_0_2_0;
    result1 += V4(0.009204052, 0.0018324523, -0.07947752, -0.15909244) * inp_0_0_1;
    result1 += V4(-0.047955345, 0.03217239, 0.055164095, 0.12852746) * inp_0_1_1;
    result1 += V4(0.02498112, -0.049751177, 0.03457846, 0.044998176) * inp_0_2_1;
    result1 += V4(-0.017058387, -0.031415854, -0.0035229074, 0.06362083) * inp_0_0_2;
    result1 += V4(0.016933544, 3.808678e-05, -0.11192203, -0.042893015) * inp_0_1_2;
    result1 += V4(0.013478469, 0.04132511, 0.10584697, -0.026422672) * inp_0_2_2;
    result2 += V4(-0.015578481, 0.02504455, 0.07044277, -0.007802456) * inp_0_0_0;
    result2 += V4(0.04809971, 0.067445114, -0.15689819, 0.0034548934) * inp_0_1_0;
    result2 += V4(-0.038173147, -0.014147258, 0.073628984, -0.042763058) * inp_0_2_0;
    result2 += V4(-0.08515715, -0.23991378, -0.0955237, -0.08558465) * inp_0_0_1;
    result2 += V4(-0.3974349, 0.61468863, 0.32536703, 0.11435247) * inp_0_1_1;
    result2 += V4(0.005842903, -0.3842624, -0.20692533, 0.11462825) * inp_0_2_1;
    result2 += V4(0.13793384, -0.06795837, 0.014168254, 0.03054699) * inp_0_0_2;
    result2 += V4(0.30806887, -0.011660746, -0.16237725, -0.15453142) * inp_0_1_2;
    result2 += V4(0.044988763, 0.0038763757, 0.13178241, 0.014963309) * inp_0_2_2;
    result3 += V4(-0.003683686, 0.014270647, 0.0043344423, -0.015633006) * inp_0_0_0;
    result3 += V4(-0.030564636, 0.21247868, -0.021809936, 0.013817947) * inp_0_1_0;
    result3 += V4(0.02399825, 0.004406272, 0.015059249, 0.0010144329) * inp_0_2_0;
    result3 += V4(0.00010378581, 0.10598455, -0.006707862, 0.011136183) * inp_0_0_1;
    result3 += V4(0.013539149, 0.2749906, 0.021540713, 0.025244687) * inp_0_1_1;
    result3 += V4(0.020217163, 0.19998224, -0.012846466, -0.030140227) * inp_0_2_1;
    result3 += V4(0.027657423, 0.11470645, 0.010029715, 0.010141564) * inp_0_0_2;
    result3 += V4(-0.07298885, -0.008780641, -0.0040592775, -0.037318952) * inp_0_1_2;
    result3 += V4(0.026665544, 0.07994509, -0.0056807166, 0.019316245) * inp_0_2_2;
    result4 += V4(0.002948534, 0.033499982, 0.0032091131, -0.0027556971) * inp_0_0_0;
    result4 += V4(0.0075665102, -0.034180634, -0.04215804, -0.0071719103) * inp_0_1_0;
    result4 += V4(-0.011019996, -0.009231276, 0.048820637, 0.014189916) * inp_0_2_0;
    result4 += V4(-0.005161535, -0.025562594, 0.0035650614, -0.01925894) * inp_0_0_1;
    result4 += V4(-0.016090712, 0.020798631, 0.058718156, 0.0100623425) * inp_0_1_1;
    result4 += V4(0.021310624, 0.019072615, -0.06790249, -0.00041809396) * inp_0_2_1;
    result4 += V4(0.010523118, -0.0024980404, 0.0003207803, 0.018543378) * inp_0_0_2;
    result4 += V4(0.010756684, 0.0045863953, -0.017695986, 0.010115866) * inp_0_1_2;
    result4 += V4(-0.021126758, -0.0074445037, 0.011682463, -0.021380486) * inp_0_2_2;
    result5 += V4(-0.0049991603, 0.0019236456, 0.007169694, 0.06452039) * inp_0_0_0;
    result5 += V4(0.006957834, -0.013417244, 0.019883526, -0.01858016) * inp_0_1_0;
    result5 += V4(0.006814296, 0.007543322, -0.026768891, -0.06367549) * inp_0_2_0;
    result5 += V4(-0.021478062, -0.0066295904, 0.009691604, -0.09153566) * inp_0_0_1;
    result5 += V4(0.009004609, 0.024502916, -0.031367555, -0.022727493) * inp_0_1_1;
    result5 += V4(0.0074311667, -0.011627917, 0.027571302, 0.12401726) * inp_0_2_1;
    result5 += V4(0.028863866, 0.007811565, -0.019657925, 0.036058005) * inp_0_0_2;
    result5 += V4(-0.0017962893, -0.010928022, 0.0157012, 0.028995067) * inp_0_1_2;
    result5 += V4(-0.029693393, -0.0015881241, -0.003247556, -0.058890972) * inp_0_2_2;
    result6 += V4(-0.032331463, 0.0065363524, 0.049518257, -0.012501104) * inp_0_0_0;
    result6 += V4(-0.035301298, -0.013548508, 0.29885164, -0.13313596) * inp_0_1_0;
    result6 += V4(0.07043573, -0.0018750599, 0.13343722, 0.015572058) * inp_0_2_0;
    result6 += V4(-0.13376155, -0.008068636, -0.2967271, -0.008516732) * inp_0_0_1;
    result6 += V4(0.18386544, 0.010009506, -0.42981163, -0.21265282) * inp_0_1_1;
    result6 += V4(-0.03643452, 0.0058413832, 0.23180154, 0.03535132) * inp_0_2_1;
    result6 += V4(0.17461592, 0.0063286256, -0.04102188, 0.022001417) * inp_0_0_2;
    result6 += V4(-0.16066682, -0.0007505177, 0.010841749, 0.100822635) * inp_0_1_2;
    result6 += V4(-0.029538872, -0.005307859, -0.029856786, 0.12387307) * inp_0_2_2;
    result7 += V4(0.006816583, -0.19523832, -0.025473615, -0.022553278) * inp_0_0_0;
    result7 += V4(-0.015773071, -0.23995522, -0.009343216, 0.041813616) * inp_0_1_0;
    result7 += V4(0.016404342, -0.031855572, -0.00558089, -0.022974888) * inp_0_2_0;
    result7 += V4(-0.004554481, -0.25763816, 0.006497755, 0.025481649) * inp_0_0_1;
    result7 += V4(-0.029135285, 0.5982863, 0.03992608, -0.031649347) * inp_0_1_1;
    result7 += V4(0.014662961, 0.11318362, -0.009431781, 0.008744289) * inp_0_2_1;
    result7 += V4(0.003189722, -0.03648215, -0.0072179674, -0.008473718) * inp_0_0_2;
    result7 += V4(0.041432086, 0.09411732, -0.017917186, -0.0045331274) * inp_0_1_2;
    result7 += V4(-0.027795214, -0.024455419, -0.01216437, 0.016004749) * inp_0_2_2;
    const ivec2 output_base = ivec2(gl_GlobalInvocationID) * ivec2(4, 2);
    imageStore(out_image, output_base + ivec2(0, 0), result0);
    imageStore(out_image, output_base + ivec2(1, 0), result1);
    imageStore(out_image, output_base + ivec2(2, 0), result2);
    imageStore(out_image, output_base + ivec2(3, 0), result3);
    imageStore(out_image, output_base + ivec2(0, 1), result4);
    imageStore(out_image, output_base + ivec2(1, 1), result5);
    imageStore(out_image, output_base + ivec2(2, 1), result6);
    imageStore(out_image, output_base + ivec2(3, 1), result7);
}

//!DESC ArtCNN C4F32 DS (Conv2D-1-ReLU)
//!COMPUTE 48 32 12 16
//!HOOK LUMA
//!BIND conv2d
//!SAVE conv2d_1
//!WIDTH LUMA.w 4.0 *
//!HEIGHT LUMA.h 2.0 *
//!COMPONENTS 4
//!WHEN OUTPUT.w LUMA.w / 1.3 > OUTPUT.h LUMA.h / 1.3 > *
#extension GL_EXT_shader_explicit_arithmetic_types_float16 : enable
#ifdef GL_EXT_shader_explicit_arithmetic_types_float16
#	define V4 f16vec4
#	define M4 f16mat4
#	define F float16_t
#else
#	define V4 vec4
#	define M4 mat4
#	define F float
#endif

const ivec2 ksize = ivec2(3, 3);
const ivec2 offset = ksize / 2;
const ivec2 wg_size = ivec2(gl_WorkGroupSize);
const ivec2 isize = wg_size + ksize - 1;
shared V4 inp[8][isize.y][isize.x];
void hook() {
    const uvec2 local_xy = gl_LocalInvocationID.xy;
    ivec2 base = ivec2(gl_WorkGroupID) * wg_size;
    for (uint y = local_xy.y; y < isize.y; y += wg_size.y) {
        for (uint x = local_xy.x; x < isize.x; x += wg_size.x) {
            const ivec2 input_base = (base + ivec2(x,y) - offset) * ivec2(4, 2);
            inp[0][y][x] = V4(conv2d_mul * texelFetch(conv2d_raw, input_base + ivec2(0, 0), 0));
            inp[1][y][x] = V4(conv2d_mul * texelFetch(conv2d_raw, input_base + ivec2(1, 0), 0));
            inp[2][y][x] = V4(conv2d_mul * texelFetch(conv2d_raw, input_base + ivec2(2, 0), 0));
            inp[3][y][x] = V4(conv2d_mul * texelFetch(conv2d_raw, input_base + ivec2(3, 0), 0));
            inp[4][y][x] = V4(conv2d_mul * texelFetch(conv2d_raw, input_base + ivec2(0, 1), 0));
            inp[5][y][x] = V4(conv2d_mul * texelFetch(conv2d_raw, input_base + ivec2(1, 1), 0));
            inp[6][y][x] = V4(conv2d_mul * texelFetch(conv2d_raw, input_base + ivec2(2, 1), 0));
            inp[7][y][x] = V4(conv2d_mul * texelFetch(conv2d_raw, input_base + ivec2(3, 1), 0));
        }
    }

    barrier();
    V4 result0 = V4(-0.04467844, 0.0049022418, 0.13805263, 0.030450424);
    V4 result1 = V4(0.009601072, -0.46114153, -0.024348829, -0.019869762);
    V4 result2 = V4(0.003637924, 0.040468704, 0.044098776, 0.04335378);
    V4 result3 = V4(-0.020247545, 0.04669445, -0.03225563, -0.024919005);
    V4 result4 = V4(0.01737375, -5.7217916e-05, -0.012391514, -1.4343712);
    V4 result5 = V4(0.024462264, 0.0150317475, 0.022890186, 0.010359794);
    V4 result6 = V4(-0.0034673586, 0.037262075, 0.012298364, -0.043889463);
    V4 result7 = V4(0.028333688, -0.015527652, -0.0011932984, 0.0016915953);
    const V4 inp_0_0_0 = inp[0][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_0_1_0 = inp[0][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_0_2_0 = inp[0][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_0_0_1 = inp[0][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_0_1_1 = inp[0][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_0_2_1 = inp[0][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_0_0_2 = inp[0][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_0_1_2 = inp[0][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_0_2_2 = inp[0][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.012476379, 0.16010484, 0.034310848, -0.056875646, -0.021604264, 0.24270451, 0.09700147, 0.12055584, -0.03644585, -0.40869, -0.05495135, 0.029706802, 0.10518179, 0.20404455, -0.13103698, -0.070853636) * inp_0_0_0;
    result0 += M4(0.037537426, -0.03647798, 0.01530499, -0.051454574, -0.04416546, -0.083710164, -0.033194892, 0.11395831, 0.11050255, 0.20698757, -0.011844064, 0.014948316, -0.046072826, 0.11208794, 0.07785686, 0.076611884) * inp_0_1_0;
    result0 += M4(-0.072894216, -0.06441451, -0.05774543, 0.15381762, -0.031631187, 0.0346974, 0.045475416, 0.08089255, -0.08508116, 0.06651949, 0.04707502, -0.15071931, 0.0829612, -0.10414661, 0.059735604, -0.02422098) * inp_0_2_0;
    result0 += M4(0.045969576, 0.2631346, -0.0048640547, -0.10304488, 0.164873, 0.22959359, -0.0075043063, -0.08314451, -0.06560714, -0.017811678, 0.029763248, 0.066473626, -0.13026278, 0.24492739, 0.09381853, 0.040828954) * inp_0_0_1;
    result0 += M4(-0.067587785, -0.32194465, -0.019337254, -0.008639831, 0.067846864, -0.20848557, -0.03221332, -0.123152606, 0.06928918, -0.028279053, 0.05741726, 0.032882016, -0.09418902, 0.26343894, -0.054905217, 0.2817886) * inp_0_1_1;
    result0 += M4(0.06070925, 0.027280724, 0.044727612, 0.030780334, 0.10029746, 0.09299838, -0.0414605, -0.036044385, -0.052984867, 0.057353646, -0.030598046, -0.03433491, -0.32283732, -0.09604272, 0.17944786, -0.11335495) * inp_0_2_1;
    result0 += M4(-0.0610565, -0.36309803, 0.025884535, 0.14746907, -0.05050082, -0.54964375, 0.08109557, -0.044789318, 0.049387585, 0.44988832, -0.01812837, -0.09586797, -0.023542495, 0.23870507, -0.06575153, -0.14744054) * inp_0_0_2;
    result0 += M4(0.00022142928, 0.36096507, 0.009643361, -0.015302071, -0.17164908, 0.18442212, -0.09389947, -0.02499582, -0.027357096, -0.27527344, -0.03425672, 0.065544434, 0.050887298, 0.08956457, 0.14305551, 0.038377512) * inp_0_1_2;
    result0 += M4(0.06381955, -0.023469426, -0.038757868, -0.09701053, -0.010048118, 0.036415868, 0.041446816, 0.05584906, 0.043436483, -0.05571012, -0.008116062, 0.0707133, 0.023365268, -0.030535812, -0.037904207, -0.06883219) * inp_0_2_2;
    result1 += M4(-0.11100155, -0.019128513, 0.027479757, -0.013570839, -0.11550077, -0.32631448, 0.100484684, 0.09046397, 0.07207261, 0.18175566, -0.14849976, 0.013359751, -0.17028649, 0.22833112, -0.027653255, 0.01317994) * inp_0_0_0;
    result1 += M4(-0.0738779, 0.019135077, -0.013964845, 0.048138183, 0.24521379, 0.22989906, 0.14874054, 0.032249056, -0.06554244, -0.009309696, -0.23526369, -0.00023031681, -0.04659464, 0.17558765, 0.05373698, -0.003878701) * inp_0_1_0;
    result1 += M4(0.25949183, 0.010694582, 0.031014454, -0.08960627, 0.12192091, -0.046603333, 0.02204166, -0.07997773, -0.16575159, 0.008809328, 0.19471288, 0.039545525, -0.17903352, 0.124228075, -0.18464, 0.17297497) * inp_0_2_0;
    result1 += M4(0.23200586, -0.24151738, -0.014088263, -0.0059408965, -0.0829041, 0.0068598897, -0.23174596, 0.07675318, 0.08959873, 0.012726359, 0.36888096, -0.050350882, 0.06643468, 0.22702922, -0.015331179, -0.004550539) * inp_0_0_1;
    result1 += M4(-0.021651397, 0.24366957, 0.040342506, -0.24721362, -0.005902784, 0.14867117, -0.033994425, -0.039739825, -0.05845934, -0.063340485, -0.07133739, 0.17629945, -0.12145087, -0.06868309, 0.029150063, -0.06576643) * inp_0_1_1;
    result1 += M4(-0.32693225, -0.04200751, -0.10659509, 0.36584485, -0.36981782, 0.033772863, -0.25479138, 0.30821982, 0.23569852, -0.053151924, -0.03967065, -0.19290408, -0.2813252, 0.01379474, -0.15260714, -0.37712798) * inp_0_2_1;
    result1 += M4(-0.12821548, 0.25275564, -0.018821824, 0.051261876, -0.01923859, 0.2523838, -0.0013737879, -0.014045371, -0.06406133, -0.23169409, -0.12686376, -0.046836633, 0.02293076, 0.15417558, 0.047473818, -0.18602899) * inp_0_0_2;
    result1 += M4(0.08506662, -0.35125327, 0.033777177, 0.22353902, -0.006683731, -0.25178406, 0.21563014, 0.20351326, 0.02679889, 0.152316, 0.06841809, -0.18339412, -0.06537388, -0.055875134, -0.14664769, 0.21581966) * inp_0_1_2;
    result1 += M4(0.10185095, 0.09008758, 0.00696842, -0.29809582, 0.21360777, 0.032124124, -0.037651017, -0.49623758, -0.09352008, 0.0352857, 0.014847121, 0.19101319, 0.19197558, -0.09515366, 0.15020458, -0.118004985) * inp_0_2_2;
    result2 += M4(0.014629263, 0.08245461, 0.14595643, -0.17556569, 0.06385132, 0.1419754, -0.011333805, 0.03958886, -0.23655912, -0.13346797, -0.12795845, 0.32802743, 0.205912, 0.06961516, 0.040190138, -0.1926467) * inp_0_0_0;
    result2 += M4(-0.012703785, -0.080728285, -0.18361735, 0.3070895, -0.08344303, 0.0014483038, -0.027769895, 0.26244068, 0.12592074, 0.037135795, 0.05237845, -0.26279292, -0.23068699, 0.08898937, -0.005921998, -0.025242371) * inp_0_1_0;
    result2 += M4(0.07262395, -0.024050396, -0.0034278086, -0.20275311, 0.032771394, 0.189799, 0.18321598, -0.31710908, -0.0034721764, 0.017403912, -0.010297318, 0.056015905, 0.27239415, -0.055846464, 0.036127254, -0.04250254) * inp_0_2_0;
    result2 += M4(0.11564553, 0.05183159, -0.11971902, -0.21618709, 0.09731858, 0.24438748, -0.080142714, -0.08203317, 0.10826372, -0.052179452, 0.119175844, 0.024188846, 0.015448769, 0.010815456, 0.0733774, -0.07060369) * inp_0_0_1;
    result2 += M4(-0.09228986, 0.10522772, 0.2727991, -0.06376762, 0.13179892, 0.012362693, 0.05784434, -0.32129714, -0.22848202, -0.17429459, -0.07267329, 0.15565768, -0.22041139, -0.03245926, 0.13742937, 0.36633345) * inp_0_1_1;
    result2 += M4(-0.06198087, -0.1175674, -0.09913351, 0.31343797, -0.038735192, -0.1873892, 0.009993588, 0.14528956, 0.056053206, 0.16658154, 0.02907715, -0.15691331, 0.19988059, -0.01834252, 0.46962515, -0.064187154) * inp_0_2_1;
    result2 += M4(-0.11719899, -0.16999686, 0.02257641, 0.3223083, -0.15903072, -0.40030316, -0.03589585, 0.45637497, 0.12901607, 0.32938957, 0.014032652, -0.41644755, 0.33290324, 0.17553867, 0.012034421, -0.18931937) * inp_0_0_2;
    result2 += M4(0.11842574, 0.040019833, -0.16621087, -0.11499885, -0.09518712, 0.13208286, -0.21081607, -0.06069021, 0.06143568, -0.15429604, 0.18529588, 0.14195079, -0.15420824, 0.0716563, 0.12249702, 0.09005742) * inp_0_1_2;
    result2 += M4(-0.059616882, 0.1138857, 0.12431743, -0.19244297, 0.0720559, -0.107711434, 0.07433882, -0.052495923, -0.019477533, -0.045064177, -0.16710162, 0.13274139, 0.0026548936, 0.11099447, -0.03197866, -0.11518836) * inp_0_2_2;
    result3 += M4(0.032534212, -0.07685322, -0.15637842, 0.025097046, 0.015236837, -0.2805893, -0.3159124, -0.00041563212, -0.08622452, 0.24220231, 0.14404196, -0.071027406, 0.07522184, 0.0011687896, 0.11932766, -0.024047557) * inp_0_0_0;
    result3 += M4(-0.029998705, -0.0009586352, 0.08241694, -0.00026669362, 0.07162797, 0.3430884, 0.06875453, 0.25899032, 0.20517887, -0.34067324, 0.09039775, -0.008306875, 0.06430712, -0.089556865, 0.16515936, 0.0713199) * inp_0_1_0;
    result3 += M4(0.006993019, 0.075032696, 0.19676167, 0.10859143, -0.10486299, -0.18222243, 0.105399325, -0.1505508, -0.11092601, 0.045593288, -0.23229752, -0.118107155, -0.0153102875, -0.15978232, 0.07414685, 0.09085846) * inp_0_2_0;
    result3 += M4(0.17027707, 0.1391393, 0.105213016, 0.054292865, -0.17626949, 0.005493701, 0.15917695, 0.070628, -0.019625878, -0.08552467, -0.05571983, -0.12984659, -0.026558846, 0.12788464, -0.07357135, -0.025131753) * inp_0_0_1;
    result3 += M4(-0.12393617, -0.036976904, -0.24064167, -0.15856177, 0.04281509, 0.17204553, -0.0661299, 0.009044231, -0.12967792, 0.123816594, 0.08468006, 0.08911857, -0.14901564, -0.025398143, -0.10142638, -0.056037974) * inp_0_1_1;
    result3 += M4(0.020925289, -0.08589934, -0.11643164, -0.16396794, 0.0134169115, -0.07244595, -0.090754285, -0.03479365, 0.04067377, -0.026124647, 0.13660558, -0.016326347, -0.21360819, 0.09955633, -0.2419808, 0.010979515) * inp_0_2_1;
    result3 += M4(-0.21563676, -0.091879666, 0.10492369, -0.22455989, 0.021112233, -0.08441514, -0.06186469, -0.4093106, 0.17764647, -0.011290547, -0.06689858, 0.24968916, -0.108101144, -0.034365285, 0.107189916, 0.07326495) * inp_0_0_2;
    result3 += M4(0.24719702, 0.06461297, 0.03784562, 0.2647736, 0.44071993, 0.029137414, 0.09055576, 0.15784428, -0.23389187, 0.028132996, -0.11332246, 0.07233078, -0.077102736, 0.03028702, 0.017126702, -0.11108468) * inp_0_1_2;
    result3 += M4(-0.12288439, 0.0062769116, -0.017691111, 0.09528079, -0.32696974, 0.015363019, 0.021106089, -0.0046059084, 0.16812147, 0.04387978, 0.03293634, -0.023743365, 0.30163994, 0.051049452, 0.076907255, -0.0380604) * inp_0_2_2;
    result4 += M4(-0.07838978, 0.028612126, 0.07348161, -0.028761562, 0.008525737, 0.08415331, 0.13152717, -0.16551343, 0.043476887, -0.12979946, -0.16894357, -0.03648245, -0.17242903, 0.24919662, 0.15903752, -0.084688626) * inp_0_0_0;
    result4 += M4(-0.04739004, 0.06921299, -0.016344976, 0.014119498, 0.21757004, -0.2614843, -0.49765137, 0.15622579, -0.24795724, 0.14603397, 0.27785406, -0.19244938, 0.18468656, -0.11979078, 0.14491226, 0.10552778) * inp_0_1_0;
    result4 += M4(0.12422068, -0.167178, -0.01860196, -0.034110762, 0.002149093, 0.19076334, -0.11347088, -0.123145625, 0.026780589, 0.07095608, -0.03387897, 0.14458896, 0.056529354, 0.15001257, 0.03863959, 0.3942185) * inp_0_2_0;
    result4 += M4(0.25833586, -0.026738508, -0.1303547, -0.15713316, -0.20352028, -0.006826321, 0.19108644, 0.060279682, -0.16828959, 0.06868258, 0.037054494, 0.044498675, 0.09924341, -0.1178955, -0.13358697, -0.06691196) * inp_0_0_1;
    result4 += M4(-0.15118182, -0.08796379, 0.27049968, 0.19887371, 0.19522509, -0.12180497, 0.18916601, 0.13574517, 0.10993147, -0.011310563, -0.21117137, -0.02890744, -0.1467781, 0.19209619, -0.045861524, 0.23935856) * inp_0_1_1;
    result4 += M4(-0.12562373, 0.15131769, -0.030763004, -0.06642105, -0.101053506, 0.0883601, 0.12666841, -0.053451743, 0.10440258, -0.10090532, -0.046381865, -0.043777727, -0.00192042, -0.038428757, -0.18604071, 0.6348329) * inp_0_2_1;
    result4 += M4(-0.26621053, -0.0009860593, 0.16671081, 0.14714198, -0.5278073, 0.09916401, 0.13803464, 0.12989873, 0.4056124, -0.0013267854, -0.14884634, -0.08460469, 0.091025054, -0.104601145, 0.04286637, -0.14586782) * inp_0_0_2;
    result4 += M4(0.35099834, 0.007102794, -0.22229737, -0.22965796, 0.5147435, -0.07441457, -0.0024696041, -0.08683364, -0.3202245, 0.011796412, 0.098135434, 0.1472355, -0.17119142, 0.015596345, -0.12376362, 0.017371092) * inp_0_1_2;
    result4 += M4(-0.094284914, 0.025709534, -0.039909486, 0.09203921, -0.21215943, -0.010714028, -0.20425268, 0.14158505, 0.084774345, -0.03609831, 0.19025882, -0.17930739, 0.14083056, -0.10798088, 0.020472994, 0.2192234) * inp_0_2_2;
    result5 += M4(-0.0008000042, 0.038407337, 0.13816875, -0.039427508, 0.015808055, 0.2795672, 0.049372997, 0.06091239, 0.010971081, -0.17450957, 0.030572439, -0.006890102, 0.046513334, -0.10370144, -0.08065253, 7.048361e-05) * inp_0_0_0;
    result5 += M4(-0.027297264, -0.22097585, -0.03225178, 0.015152229, -0.09807998, -0.5524079, -0.12855892, 0.15604335, 0.11956049, 0.50090796, 0.06540699, -0.22107905, -0.16297925, 0.13236201, 0.03551786, -0.116825595) * inp_0_1_0;
    result5 += M4(0.013785604, 0.28211528, -0.13625407, 0.010024515, 0.0988568, 0.34211683, -0.064453006, 0.05764747, -0.099175476, -0.4050886, -0.003359295, 0.084971726, 0.034398846, 0.074138686, 0.043082237, -0.0049443236) * inp_0_2_0;
    result5 += M4(0.023930376, -0.15015331, -0.02656651, 0.013332934, -0.09726202, -0.39969844, -0.30077493, 0.056498833, 0.10068795, 0.2210851, 0.21558507, 0.015926206, -0.01162051, 0.036911394, -0.11862244, -0.08344909) * inp_0_0_1;
    result5 += M4(-0.10757797, 0.2793768, 0.1524726, -0.049981453, 0.082571395, -0.053748023, 0.3300356, 0.06862572, -0.062724665, -0.13647196, -0.33152902, 0.041283503, 0.0059613064, 0.11067001, -0.17685866, -0.060903884) * inp_0_1_1;
    result5 += M4(0.13337377, -0.33635977, -0.10037772, 0.015884405, 0.044559564, -0.38200134, -0.09768941, -0.04693317, -0.10116736, 0.40970784, 0.06666501, -0.011164121, -0.16942228, -0.05908055, 0.15768959, -0.12873632) * inp_0_2_1;
    result5 += M4(-0.011551388, 0.022928033, -0.07545851, -0.01947196, 0.09232493, 0.38010514, 0.47068447, -0.15316454, -0.15780632, -0.2829537, -0.2242694, 0.063013166, -0.010571721, -0.07957418, -0.14062683, 0.031018283) * inp_0_0_2;
    result5 += M4(0.112418346, -0.1842562, -0.19619858, 0.04301123, 0.12616394, -0.37492257, -0.44571495, -0.31482506, -0.06880175, 0.41055626, 0.2655091, 0.12954302, -0.14754769, 0.09494228, 0.062052526, 0.10925992) * inp_0_1_2;
    result5 += M4(-0.11685395, 0.31982002, 0.25853613, -0.00011290956, -0.23274507, 0.84622824, 0.102370545, 0.07372836, 0.23103483, -0.5966417, -0.05081907, -0.09835096, 0.08647884, 0.04958592, 0.1365349, 0.02105821) * inp_0_2_2;
    result6 += M4(-0.06137326, -0.05751124, 0.0076942933, -0.094282836, 0.23956716, 0.15953112, 0.06249994, -0.5292439, -0.01604201, -0.065193, -0.12487887, -0.21154858, -0.026964514, 0.112398855, 0.12681447, 0.07796009) * inp_0_0_0;
    result6 += M4(-0.07133012, -0.02728201, 0.0077056875, -0.054551516, -0.32229257, -0.21736954, -0.05522625, 0.10198827, 0.2593545, 0.14521861, 0.051098313, 0.4448661, -0.081150636, 0.013555089, 0.012497802, -0.108372174) * inp_0_1_0;
    result6 += M4(0.09238161, 0.044398423, 0.043347146, 0.1443533, 0.51191765, -0.049967125, 0.17621061, -0.02587045, -0.2009257, -0.058428533, 0.0332718, -0.09498776, 0.027913868, -0.06311284, 0.039818637, 0.29705036) * inp_0_2_0;
    result6 += M4(0.2076069, 0.15684958, -0.124930404, 0.06414177, 0.14258055, 0.033104505, 0.022616928, 0.22212118, -0.34679863, -0.020894082, 0.10385539, 0.29819396, -0.004497999, -0.04102552, -0.0035917996, 0.013004418) * inp_0_0_1;
    result6 += M4(-0.020969821, 0.1778961, -0.02090504, -0.009210482, -0.09468077, 0.20082138, 0.03529127, 0.14752069, 0.03739664, -0.19953212, 0.014599309, -0.22669794, 0.065938875, -0.044280622, 0.16111043, -0.031342328) * inp_0_1_1;
    result6 += M4(0.15281385, -0.22557463, 0.08047147, -0.04797425, -0.023924889, -0.11499638, -0.052220743, 0.22205336, 0.008815763, 0.1725549, -0.053253856, -0.014478057, -0.14807229, 0.16329765, -0.09802161, -0.018543856) * inp_0_2_1;
    result6 += M4(-0.112908, -0.07324629, 0.09709477, 0.24838887, -0.2022791, 0.066073805, -0.12320426, 0.075550124, 0.29294693, -0.04175848, 0.052824266, -0.50257576, -0.06619184, 0.053450778, -0.09120521, 0.35930964) * inp_0_0_2;
    result6 += M4(-0.013811215, -0.16521634, -0.0032390924, -0.051966038, -0.12673458, -0.41126445, 0.049083363, -0.12238471, 0.017674427, 0.1543551, -0.09116496, 0.22545585, 0.0646296, -0.109819934, -0.039788492, -0.10248821) * inp_0_1_2;
    result6 += M4(-0.16474281, 0.15209548, -0.07012364, -0.16893724, -0.1451151, 0.29374197, -0.1927315, 0.03713139, -0.052724726, -0.10448133, 0.033324067, 0.016668538, 0.06687186, -0.08086538, 0.02388471, -0.32185557) * inp_0_2_2;
    result7 += M4(-0.039662734, 0.44383052, -0.20161961, -0.08882638, -0.016595656, 0.5589564, 0.025454672, -0.014690094, -0.06500511, -0.48825505, 0.1879182, 0.23027903, -0.061789576, -0.10505195, -0.07535154, 0.03454654) * inp_0_0_0;
    result7 += M4(0.06296442, -0.27413386, -0.018103061, 0.119976245, -0.07784182, -0.22344096, 0.08796147, -0.023586905, 0.12135997, 0.18701448, 0.23790869, -0.014557308, 0.19082057, -0.0057992693, -0.18310264, -0.023800656) * inp_0_1_0;
    result7 += M4(-0.05203187, -0.11101147, 0.23129576, -0.0057628704, -0.04408878, -0.19063832, 0.4499639, 0.09586061, -0.038601067, -0.007388081, -0.43557703, -0.059237137, -0.14226647, 0.07841109, 0.06843994, -0.07287038) * inp_0_2_0;
    result7 += M4(-0.08217525, -0.50320625, 0.018502457, 0.18121296, -0.03123458, -0.3618317, -0.31232235, 0.24958493, 0.053864483, 0.6841272, -0.09427892, -0.38369578, -0.08790775, 0.12446816, -0.10653659, 0.20534627) * inp_0_0_1;
    result7 += M4(-0.09934337, -0.0760638, -0.020223297, -0.13623865, -0.00014531374, 0.11544176, -0.12960517, -0.16332789, -0.08225344, -0.09632516, 0.066712715, 0.07721042, -0.10233232, 0.040252615, 0.08264773, -0.038671933) * inp_0_1_1;
    result7 += M4(0.18698329, 0.33864796, -0.11731727, -0.0976833, 0.05016272, 0.13366434, 0.042030085, -0.137447, -0.065422036, -0.20866382, 0.040400177, 0.11880914, -0.1537183, -0.15991189, 0.16162673, 0.20311818) * inp_0_2_1;
    result7 += M4(0.096021265, 0.14392109, 0.20454735, 0.12194475, -0.20166257, 0.41190207, -0.17872716, 0.24613844, 0.09619643, -0.49238986, -0.21062289, -0.06493853, 0.22718894, 0.13749272, 0.099183306, 0.3362956) * inp_0_0_2;
    result7 += M4(0.12019309, 0.3091686, -0.015021784, -0.18055113, 0.53572774, 0.04715107, -0.010879809, -0.42181915, -0.18832584, -0.042092398, 0.08507512, 0.1944622, 0.06918366, -0.2554239, -0.14499606, 0.16999748) * inp_0_1_2;
    result7 += M4(-0.19241048, -0.28178588, -0.09085262, 0.08329617, -0.21543424, -0.3984169, 0.117114894, 0.098369226, 0.1830067, 0.43262973, 0.09548312, -0.07239261, 0.30002746, 0.1369142, -0.19316457, -0.0076390193) * inp_0_2_2;
    const V4 inp_1_0_0 = inp[1][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_1_1_0 = inp[1][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_1_2_0 = inp[1][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_1_0_1 = inp[1][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_1_1_1 = inp[1][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_1_2_1 = inp[1][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_1_0_2 = inp[1][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_1_1_2 = inp[1][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_1_2_2 = inp[1][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.08574931, 0.046119012, 0.020954598, -0.037963476, 0.018834094, 0.23751381, 0.02999579, -0.11077994, 0.13854688, 0.37359157, 0.13068838, -0.048836906, -0.01243482, -0.14289413, -0.060592104, 0.070388235) * inp_1_0_0;
    result0 += M4(-0.06528283, 0.1183093, 0.04305256, 0.002475696, -0.12651098, -0.08638781, -0.06358072, 0.092261314, -0.050544776, -0.08688598, 0.10708776, 0.09300388, -0.0029727116, -0.17009182, 0.07710906, 0.00774956) * inp_1_1_0;
    result0 += M4(0.0004482572, -0.05328342, -0.108019546, -0.008226592, 0.070789844, -0.038273063, -0.07937051, 0.026227172, -0.082204424, 0.0785013, 0.04617875, -0.0008129718, -0.06798322, 0.0057495036, 0.15905924, -0.071027935) * inp_1_2_0;
    result0 += M4(-0.08157971, 0.51732063, -0.04956653, 0.017764878, 0.11964033, -0.5099303, 0.093349956, 0.09073065, 0.11438579, -0.040996328, -0.22976847, 0.09492253, -0.06420875, 0.10458285, -0.014820482, -0.0049525863) * inp_1_0_1;
    result0 += M4(0.08996965, -0.35326537, -0.014035315, -0.11001226, -0.10920351, 0.288606, -0.07739453, -0.037711725, -0.09168751, 0.15427455, -0.050164964, -0.060057536, -0.07634462, 0.30105263, 0.026316546, 0.2008122) * inp_1_1_1;
    result0 += M4(-0.02282385, -0.056226756, 0.04704554, 0.038283117, 0.0043623215, 0.04538535, 0.058691386, -0.040914185, 0.05729335, -0.087185994, -0.023449987, -0.11386003, 0.05842814, -0.10828237, -0.09265681, -0.094098285) * inp_1_2_1;
    result0 += M4(0.08713917, -0.5133243, 0.05745826, 0.017201738, -0.07506585, 0.25101408, -0.068967745, -0.059569623, 0.34928724, -0.33552662, 0.12537596, 0.040018965, 0.21130194, 0.026493914, -0.044443917, 0.022968296) * inp_1_0_2;
    result0 += M4(-0.09974505, 0.29625472, -0.08284343, 0.10048285, 0.19497599, -0.09511008, 0.15121377, -0.0671518, -0.15665631, 0.038787745, -0.15164608, -0.0039708996, 0.09527161, -0.030612426, -0.012772264, -0.12556618) * inp_1_1_2;
    result0 += M4(0.0052476027, 0.055192772, 0.056826845, 0.02826825, -0.118776925, -0.09234367, -0.07163074, 0.06316459, -0.24958646, -0.106207415, 0.081393994, 0.08678538, -0.108548135, 0.0055485414, 0.022092631, -0.009685952) * inp_1_2_2;
    result1 += M4(0.0321491, 0.021805024, -0.010523085, 0.08631526, -0.0017704265, -0.0010345684, 0.113861695, -0.039950226, -0.0072899153, -0.05358501, 0.13188832, -0.037800197, -0.0002283867, -0.051761586, -0.008290752, -0.019323856) * inp_1_0_0;
    result1 += M4(-0.16980807, -0.013802133, -0.02957213, -0.055998623, -0.11816309, 0.07748291, -0.012276806, 0.07482442, -0.08065824, -0.09694578, 0.11080198, 0.13901159, 0.2657676, 0.22563814, 0.15561837, 0.061890688) * inp_1_1_0;
    result1 += M4(0.12891048, -0.019125782, 0.003982004, -0.054757793, 0.13721251, -0.11796853, -0.078444995, -0.07522106, -0.08874099, -0.100907475, -0.08691511, -0.041108787, -0.29799354, -0.018441258, -0.00074196106, -0.0073287864) * inp_1_2_0;
    result1 += M4(-0.0023615018, -0.004466802, -0.07680671, 0.03577486, -0.016012585, -0.020165779, -0.049634263, 0.038900305, -0.2515303, 0.26373756, -0.27043033, 0.1405178, 0.08176101, -0.06907637, 0.26379254, 0.036395032) * inp_1_0_1;
    result1 += M4(-0.11221885, -0.050818175, 0.028118284, -0.05663893, 0.1042001, -0.09262266, 0.026762936, -0.12738106, 0.053750657, -0.06593969, -0.017118264, -0.21959095, -0.094345614, 0.052861296, -0.28331548, -0.24607615) * inp_1_1_1;
    result1 += M4(0.06508235, 0.039127585, 0.12323445, 0.0075195883, -0.063816145, 0.035835486, 0.029013732, 0.07395974, 0.27144292, -0.04694524, 0.17408025, -0.36001265, 0.0366541, -0.18577601, 0.02741173, -0.20826125) * inp_1_2_1;
    result1 += M4(-0.030580726, 0.1848571, -0.009810806, -0.034985285, -0.003627176, 0.052203946, 0.019690514, 0.0061428472, 0.034399945, 0.048220914, 0.024746055, 0.020764248, -0.024136145, -0.1085185, -0.030791724, -0.06408201) * inp_1_0_2;
    result1 += M4(0.1881339, -0.029948886, 0.067958474, 0.06697623, 0.0006943492, 0.04812094, -0.05307135, 0.12693827, 0.2553739, 0.29609585, -0.036814354, 0.1794815, 0.046307098, 0.19943592, -0.14510977, 0.34722927) * inp_1_1_2;
    result1 += M4(-0.11491229, -0.030783666, -0.08621489, -0.033378977, -0.057646547, 0.015805308, 0.025795864, -0.096047625, -0.15989435, -0.13394319, -0.05973718, 0.20315811, 0.020505836, -0.07974688, 0.043592013, 0.08474713) * inp_1_2_2;
    result2 += M4(0.15523103, 0.028478509, 0.04105635, -0.20228796, 0.13227354, 0.053736523, 0.119671084, -0.2127464, 0.40008676, 0.12707932, 0.1050505, -0.3584657, -0.08015611, -0.09118849, -0.08432585, 0.2492181) * inp_1_0_0;
    result2 += M4(-0.011617511, -0.049681693, -0.17354977, 0.25620294, -0.1918878, -0.109878495, -0.0708639, 0.26782212, -0.18135867, -0.021055145, -0.10017032, 0.38087356, -0.2192616, 0.008333105, 0.008666713, -0.20650959) * inp_1_1_0;
    result2 += M4(-0.05640768, 0.04659588, 0.047719937, -0.03132368, 0.0539905, 0.1375343, 0.040263705, -0.08937565, 0.05703962, -0.0063495394, 0.10587751, 0.04631117, -0.080019005, -0.13612463, 0.014259479, 0.10772893) * inp_1_2_0;
    result2 += M4(-0.11171598, 0.32819664, -0.030752115, -0.04746112, -0.14447165, -0.16487236, -0.09416045, 0.29229045, -0.38795862, 0.19850379, -0.21713622, 0.23340008, 0.24737294, 0.010070532, -0.04280738, -0.2266256) * inp_1_0_1;
    result2 += M4(-0.07834498, -0.013677633, 0.23162697, -0.23171139, 0.35781196, 0.26779866, 0.013411939, -0.071976304, 0.27195856, 0.011532933, 0.06420967, -0.31647936, 0.1352284, -0.028804695, -0.17347507, 0.38072187) * inp_1_1_1;
    result2 += M4(-0.017979477, -0.18013494, -0.10270775, 0.19119504, -0.10386336, -0.16578773, -0.11051131, 0.003517049, 0.13276383, -0.0030411447, 0.08773117, -0.102050014, 0.23029284, 0.1466224, 0.3260689, -0.33013913) * inp_1_2_1;
    result2 += M4(-0.002814933, -0.32968277, -0.019855238, 0.18034016, -0.040919013, 0.12480793, -0.06126709, -0.05204221, 0.0023848622, -0.41126576, -0.011893205, 0.47549984, -0.14025038, 0.15945561, -0.09022637, 0.13698553) * inp_1_0_2;
    result2 += M4(-0.026653355, 0.22123325, -0.016331326, -0.015128683, -0.1083284, -0.14733271, 0.018588688, -0.14167576, -0.26519227, 0.06620403, -0.1390749, -0.1700693, -0.112189926, -0.30117342, 0.05452324, -0.015392895) * inp_1_1_2;
    result2 += M4(0.08213919, -0.038829014, 0.14417891, -0.12125593, 0.006388008, -0.013683165, 0.15099323, -0.0144394785, 0.0988488, 0.04552092, 0.05471711, -0.08628729, 0.13044108, 0.22620185, -0.018823931, -0.07179031) * inp_1_2_2;
    result3 += M4(0.010085075, -0.08273988, 0.0669952, 0.038478468, 0.05114215, -0.2267201, -0.06747502, -0.19664267, -0.00504256, -0.34897727, -0.07483643, 0.088111095, 0.11590281, 0.14444594, 0.06021992, -0.029173216) * inp_1_0_0;
    result3 += M4(-0.0014854956, 0.04024376, -0.035733417, 0.02358302, -0.076695055, 0.18767709, 0.0053490484, 0.05264421, -0.13163841, 0.34689024, -0.24520406, -0.16012901, 0.11458966, 0.15876415, -0.0964449, 0.032863297) * inp_1_1_0;
    result3 += M4(-0.05246891, -0.028553436, 0.0020784412, -0.1993373, -0.08019684, 0.053851508, -0.013588629, 0.059410833, -0.049483433, -0.047252174, -0.20861742, 0.13431038, -0.01883628, -0.23383309, -0.14801608, 0.071896546) * inp_1_2_0;
    result3 += M4(-0.00023797514, -0.0090008415, 0.12150159, -0.06517781, -0.14026988, 0.20308848, -0.33220103, 0.09943322, -0.072733395, -0.20566021, 0.19597657, -0.026968818, 0.06888126, -0.022148028, 0.37623814, 0.2505778) * inp_1_0_1;
    result3 += M4(-0.09857941, 0.009038963, -0.2381985, -0.22246929, 0.25685543, -0.19117014, 0.27624875, 0.024117488, 0.33860093, 0.21367572, 0.24617882, -0.15471032, 0.07015589, -0.17979616, 0.4517521, 0.27315417) * inp_1_1_1;
    result3 += M4(0.13177012, 0.03457181, 0.06913809, 0.11484792, 0.0005283079, -0.028241461, 0.018554205, -0.09827939, -0.17514093, -0.014591834, 0.26389632, 0.40957195, -0.5734528, 0.03618209, 0.052483544, 0.32115594) * inp_1_2_1;
    result3 += M4(-0.02592137, -0.008782552, 0.066065095, 0.0009207073, 0.0953715, 0.030192807, 0.36660382, 0.13959815, -0.12815492, 0.053058002, 0.0845464, 0.08182, -0.091465995, 0.07445252, -0.28693622, 0.20897591) * inp_1_0_2;
    result3 += M4(0.2725273, -0.04394502, 0.033780154, 0.0943544, -0.24068138, 0.033059448, -0.24339917, -0.09444974, 0.33537617, -0.040065534, 0.037516013, -0.69145054, -0.23028664, 0.047348276, -0.22458333, -0.7587233) * inp_1_1_2;
    result3 += M4(-0.24783657, 0.09600403, -0.08303815, 0.2632958, 0.07781936, -0.053915493, 0.031153522, 0.047211315, -0.10873064, 0.0068784403, -0.35249618, 0.32131657, 0.5942259, -0.0037521468, -0.20362103, -0.31184834) * inp_1_2_2;
    result4 += M4(-0.073927335, 0.039118838, 0.011866287, -0.0048890547, -0.08208442, 0.08996661, 0.11544066, -0.10439287, -0.07372439, 0.055127203, 0.15122382, 0.019693775, 0.047560025, -0.29471314, -0.10660698, 0.14410214) * inp_1_0_0;
    result4 += M4(-0.037954193, 0.18029143, 0.009807185, 0.033830144, 0.22922832, -0.0035890662, -0.110000975, 0.273785, 0.27799937, -0.019786412, -0.3769301, 0.19510779, 0.061215892, -0.09820875, 0.005866155, -0.116745435) * inp_1_1_0;
    result4 += M4(0.015307555, -0.19876374, -0.022143494, -0.04151592, -0.19983307, -0.01978673, 0.006996651, -0.04637906, 0.07055613, -0.035412423, 0.056756545, -0.19867693, 0.1432836, 0.2910714, 0.06626114, -0.08228163) * inp_1_2_0;
    result4 += M4(0.13842049, 0.104437485, -0.028804729, -0.043916173, -0.024756636, -0.06236875, -0.04901829, 0.17438368, 0.060556017, -0.0050281337, -0.06790126, 0.15917602, 0.37883118, 0.07494883, -0.082579516, -0.12580325) * inp_1_0_1;
    result4 += M4(-0.01095327, -0.25760198, -0.003590159, 0.18646783, -0.118214145, 0.009566247, -0.08332226, -0.30838218, -0.15403298, -0.0008127171, -0.012494576, -0.12906352, -0.13486333, 0.2771379, -0.18822528, -0.011363474) * inp_1_1_1;
    result4 += M4(-0.07196802, 0.1331734, 0.031958222, -0.10119373, 0.113899276, -0.005558482, 0.14026071, 0.097824976, 0.09906921, -0.020112824, 0.048134558, 0.08454568, -0.019386586, -0.13530004, -0.10631985, 0.37467676) * inp_1_2_1;
    result4 += M4(-0.27301347, -0.005966319, 0.044025406, 0.06049721, 0.13004349, -0.014802673, -0.09317759, 0.033549823, -0.3728264, 0.10539046, 0.033984855, -0.00959702, -0.018767532, -0.04914667, -0.020138053, 0.06281365) * inp_1_0_2;
    result4 += M4(0.19435543, 0.09796175, 0.012069981, -0.22960426, -0.10108765, 0.028759126, 0.11125609, 0.026187904, 0.07455131, -0.027699342, 0.13108131, -0.1988953, -0.49404347, -0.04983165, -0.025327252, -0.18104582) * inp_1_1_2;
    result4 += M4(0.12228214, -0.06941695, -0.056143254, 0.15348594, 0.060938302, -0.021824585, -0.05306777, -0.045544185, 0.023197114, -0.03840524, -0.017227441, 0.21013664, 0.03667773, -0.0018215064, 0.44044235, -0.2534644) * inp_1_2_2;
    result5 += M4(-0.040756766, -0.080762096, -0.0049218624, -0.024108728, 0.08998148, 0.06985498, -0.027022155, -0.22197172, 0.0341577, 0.07389703, -0.071012095, -0.017138297, -0.116702326, 0.020409549, -0.0525858, 0.22263718) * inp_1_0_0;
    result5 += M4(0.10699896, 0.044498842, 0.010055746, 0.017340625, -0.01578182, -0.1386136, -0.042876218, 0.21596523, -0.16034828, -0.33798686, 0.0031534894, 0.12122412, 0.060232278, 0.14491849, -0.17810991, -0.035931304) * inp_1_1_0;
    result5 += M4(-0.1598519, 0.08654293, 0.025728472, -0.014619213, -0.07952838, 0.092982896, 0.03143556, -0.0008807353, -0.019850863, 0.02587728, 0.051167656, 0.045845397, 0.07888695, -0.36632967, 0.111387245, -0.20123541) * inp_1_2_0;
    result5 += M4(0.005830316, -0.07157508, -0.12777932, -0.014374555, -0.100706175, -0.18734251, 0.061001733, 0.1827039, -0.21763462, -0.19760424, -0.37425548, 0.053339936, 0.14082512, -0.020139322, -0.09360297, -0.27422395) * inp_1_0_1;
    result5 += M4(-0.22483984, 0.09207488, 0.029112678, 0.03576301, 0.036353726, 0.021101713, -0.018743822, -0.19240125, 0.19563903, 0.036352184, 0.07956482, 0.15098672, 0.07476694, 0.25194246, -0.056585547, 0.11765578) * inp_1_1_1;
    result5 += M4(0.23452108, -0.09901171, 0.026834877, -0.007953325, 0.08165642, 0.090021305, -0.0067408504, 0.015735148, 0.08172352, -0.067515306, 0.21273877, -0.019923307, -0.26892048, 0.3878925, -0.020895122, 0.13106538) * inp_1_2_1;
    result5 += M4(-0.03882831, 0.03691786, 0.13011302, -0.035606105, 0.021109594, 0.29718626, -0.12419472, 0.0048599727, 0.1298759, 0.715122, 0.37739468, -0.0217872, -0.042816486, 0.20628247, 0.053024314, 0.34838006) * inp_1_0_2;
    result5 += M4(0.12444867, -0.16623838, -0.1861748, 0.007836394, -0.0022297804, -0.14715388, 0.09513178, 0.035051193, 0.06603966, -0.8383844, -0.16994108, -0.15656011, -0.08545113, -0.19795841, 0.4011993, -0.036730915) * inp_1_1_2;
    result5 += M4(-0.10883912, 0.18596072, 0.061363313, -0.011135692, -0.025366183, -0.10636424, 0.047833566, -0.040319677, -0.030681416, 0.67921007, -0.16853434, -0.19930968, 0.18908837, -0.4051701, -0.12647615, -0.24116567) * inp_1_2_2;
    result6 += M4(0.102941066, 0.0013523927, 0.102058716, 0.406925, -0.034666996, 0.13984837, -0.034201782, 0.017713785, 0.35405, 0.17410833, 0.04889266, 1.1272304, 0.14500073, -0.012419583, -0.12336538, 0.14190999) * inp_1_0_0;
    result6 += M4(-0.0051482865, 0.014756921, 0.021474963, 0.0026971812, 0.02990448, -0.17780462, 0.17119178, -0.099039234, -0.314709, -0.17635727, 0.04427993, -0.46377707, 0.086625636, 0.09816523, -0.08924386, 0.013632795) * inp_1_1_0;
    result6 += M4(-0.21829039, -0.01677249, -0.12165999, 0.047532983, 0.05389769, -0.007123228, -0.11432193, -0.12454678, -0.11901217, 0.002103641, -0.15515429, -0.2787537, 0.02284135, -0.00032842092, 0.10963464, 0.034328483) * inp_1_2_0;
    result6 += M4(0.12476719, 0.0033219284, -0.025862543, -0.26161426, -0.10377172, -0.19586359, -0.0022512595, -0.11292683, 0.16553764, -0.12120338, 0.09512054, -0.9664559, -0.2757104, 0.17536083, 0.06960417, -0.22852023) * inp_1_0_1;
    result6 += M4(-0.05337549, 0.04137789, -0.06283511, -0.022294795, -0.0042120605, 0.11502807, -0.18435436, 0.29227355, 0.18864873, 0.044518415, -0.18433616, 0.0065808897, 0.00855716, -0.14005974, 0.09050006, -0.046336677) * inp_1_1_1;
    result6 += M4(0.26538166, -0.033322994, 0.05448341, -0.20392519, -0.051943097, 0.08372591, 0.09748659, -0.22193591, -0.3299761, 0.13206795, 0.060412847, -0.06899449, -0.20251848, -0.001605812, -0.14349239, -0.10846266) * inp_1_2_1;
    result6 += M4(-0.04840006, -0.041287977, -0.077647164, 0.11850927, 0.013275551, 0.030838681, 0.039178044, -0.062292106, -0.10273876, 0.1687487, -0.09918537, 0.8757821, 0.065918915, -0.12908606, 0.009265899, -0.015337381) * inp_1_0_2;
    result6 += M4(-0.07587807, -0.08099356, 0.09854129, -0.24765737, 0.07954571, -0.011765395, -0.02604946, -0.025223993, -0.005086039, -0.1407015, 0.024433857, -0.80496216, -0.031708382, 0.05946459, -0.045870747, 0.017038776) * inp_1_1_2;
    result6 += M4(-0.09291104, 0.03915578, 0.034814447, 0.28117064, -0.026955916, 0.008131043, 0.046918288, 0.38911128, 0.054639746, -0.16244864, 0.15913315, 0.8118539, 0.21244885, -0.015722089, 0.13611871, 0.15597977) * inp_1_2_2;
    result7 += M4(-0.08811035, 0.20961332, 0.043805182, -0.09171073, -0.062867306, 0.31783062, -0.03932886, 0.102558605, -0.056088865, 0.22779377, 0.23413637, -0.36373082, -0.016316433, -0.7218059, 0.21676157, 0.02662746) * inp_1_0_0;
    result7 += M4(0.055972725, -0.07707781, 0.0030206814, -0.054069463, 0.04698476, -0.18267463, 0.1082284, -0.18012989, -0.12521768, -0.18854925, -0.35279155, -0.011622362, 0.026911862, 0.09045731, 0.108385764, -0.0040569333) * inp_1_1_0;
    result7 += M4(0.048058335, -0.018780923, 0.070564955, 0.05256818, 0.07273224, 0.03335361, -0.09678063, -0.048124127, 0.05264172, 0.37404713, 0.49034005, -0.17783774, -0.023760479, -0.008406288, 0.13747461, -0.09548085) * inp_1_2_0;
    result7 += M4(0.21156628, -0.1715816, -0.06315947, 0.37885797, -0.12056943, -0.0796504, 0.19817011, -0.0120966155, 0.020814296, -0.53419787, -0.14854543, 0.71862996, 0.036247108, 0.14240651, -0.19438009, -0.13628107) * inp_1_0_1;
    result7 += M4(-0.3852832, -0.2539996, 0.19302005, -0.2443196, 0.17698999, -0.005411701, -0.13167202, 0.285723, 0.09776202, -0.044077825, 0.050374154, -0.1285509, 0.35980687, 0.5795425, -0.13977218, 0.23171923) * inp_1_1_1;
    result7 += M4(0.035990525, 0.15255032, 0.007972157, -0.051908147, -0.104554206, 0.14555484, -0.06104286, -0.18685639, -0.120113224, -0.3215943, 0.046485584, 0.1841854, -0.10450712, -0.08062763, -0.09123315, -0.113001846) * inp_1_2_1;
    result7 += M4(-0.18556476, -0.010947291, -0.09433011, 0.16707145, 0.21669869, -0.33069137, -0.18022358, -0.23611239, -0.35732126, 0.5596981, -0.051232312, -0.044858817, 0.027454788, -0.10936752, -0.40957195, -0.18655065) * inp_1_0_2;
    result7 += M4(0.42236233, 0.19221388, -0.19636822, -0.020675713, -0.26386678, 0.32583892, -0.033730365, -0.029015461, 0.22806683, 0.2034699, -0.1365019, 0.10225159, -0.73015153, -0.11852831, 0.3797903, 0.3213426) * inp_1_1_2;
    result7 += M4(-0.08587759, -0.059673034, 0.10762104, -0.08051981, 0.05787773, -0.19492052, 0.30376947, 0.29709262, 0.28393966, -0.2629357, 0.09823903, -0.2836761, 0.44475958, 0.15889728, 0.007276111, -0.024843667) * inp_1_2_2;
    const V4 inp_2_0_0 = inp[2][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_2_1_0 = inp[2][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_2_2_0 = inp[2][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_2_0_1 = inp[2][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_2_1_1 = inp[2][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_2_2_1 = inp[2][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_2_0_2 = inp[2][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_2_1_2 = inp[2][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_2_2_2 = inp[2][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.056262627, -0.120091505, -0.0394741, -0.21015897, -0.0028062228, -0.555141, -0.022049367, -0.12143185, 0.0060306736, 0.5249525, -0.032032117, 0.15975793, -0.13457072, 0.24440652, -0.0042279996, 0.18491724) * inp_2_0_0;
    result0 += M4(0.024050638, -0.15881386, -0.25241175, -0.009917868, -0.010181417, 0.090908706, 0.007031999, -0.08366394, -0.04153045, -0.13920939, 0.10489166, -0.07325257, -0.13626494, -0.12108292, 0.23408714, 0.05409461) * inp_2_1_0;
    result0 += M4(0.009776347, 0.0043413355, 0.08681197, -0.009173229, 0.105244756, -0.010035704, 0.07004577, -0.005230554, -0.13834922, -0.019199599, -0.054893807, 0.026840376, -0.09739593, -0.12588318, -0.11372026, 0.22213714) * inp_2_2_0;
    result0 += M4(0.023274723, 0.43909514, 0.06880877, -0.034627315, -0.037740078, -1.6495128, 0.25376892, 0.17181663, 0.22727823, -1.2335284, -0.053442292, 0.016903266, 0.04399073, 0.39892426, 0.008131126, -0.16557522) * inp_2_0_1;
    result0 += M4(0.20439608, -0.3044595, 0.17660199, -0.11179806, -0.3429613, 0.60792786, -0.1275554, 0.2447349, -0.28771684, 0.6885402, -0.07019849, 0.1598983, -0.054622643, 0.15813884, 0.0056008114, 0.15435389) * inp_2_1_1;
    result0 += M4(-0.06291005, -0.22571751, -0.0136052, -0.0022390515, 0.03202772, 0.008872484, 0.23785365, -0.109599836, -0.05892117, -0.085315764, -0.046227053, -0.12281602, -0.02666804, 0.073034056, 0.06236496, 0.033287562) * inp_2_2_1;
    result0 += M4(0.10448842, -0.07015032, 0.019036144, 0.0373919, -0.23232675, -0.2210121, -0.057296243, 0.0046015917, -0.009649588, 0.38426068, -0.04436723, 0.08849047, 0.05475419, -0.03430461, -0.00197634, -0.10086484) * inp_2_0_2;
    result0 += M4(-0.37837815, -0.0124347545, -0.019573353, -0.058534484, 0.24496683, 0.18797229, -0.012113022, 0.051206056, 0.3284624, -0.06757448, 0.082276605, -0.19071254, 0.110901766, 0.098412454, 0.10042103, -0.013673172) * inp_2_1_2;
    result0 += M4(0.046657402, -0.03003201, -0.027478449, 0.15333948, -0.062413637, 0.05632152, -0.048239596, -0.03323799, -0.052456114, -0.034965135, 0.050573, -0.14014673, 0.11521139, -0.031151926, -0.06598732, 0.02933929) * inp_2_2_2;
    result1 += M4(-0.14907318, 0.21655872, -0.05447498, -0.06050015, -0.16697474, -0.18743953, -0.25136387, 0.016454129, -0.12174726, -0.11324483, 0.11853852, 0.017615994, 0.029493513, -0.27575198, 0.13176043, 0.07877295) * inp_2_0_0;
    result1 += M4(-0.9630119, 0.3590198, -0.5903436, 0.077519774, 0.09418416, -0.15129092, 0.2333055, 0.030682804, 0.0621497, 0.020074725, 0.27796507, -0.08044228, 0.3174451, -0.1017919, 0.2219028, 0.16505244) * inp_2_1_0;
    result1 += M4(-0.82431436, -0.10680173, -0.20730054, 0.021736879, -0.42835253, 0.008032411, -0.25394067, -0.068184786, 0.12252377, -0.055227652, -0.11482357, 0.021264475, 0.3222033, 0.1201656, -0.067605525, -0.017368957) * inp_2_2_0;
    result1 += M4(0.09606009, -0.09918996, 0.061694033, -0.067704484, -0.05403101, 0.40342993, -0.10458237, -0.08201592, -0.19958632, 0.084273815, -0.19246328, -0.07985164, -0.06528409, 0.15964693, 0.040423468, 0.14335974) * inp_2_0_1;
    result1 += M4(-0.009779892, -0.2667726, 0.07639251, 0.47742772, 0.37280366, -0.12851582, 0.07263907, 0.033441212, 0.25391766, -0.113268524, 0.13468456, -0.15849815, -0.09331333, -0.036685262, -0.12601791, -0.18033881) * inp_2_1_1;
    result1 += M4(0.04606251, 0.009647454, -0.19929941, 0.4864382, -0.36745456, 0.012115451, -0.26760098, -0.8078474, 0.013283229, -0.060418606, 0.1205209, -0.37943593, -0.3466796, 0.06823674, -0.22884123, -0.5351986) * inp_2_2_1;
    result1 += M4(0.03683549, 0.061676163, 0.017906427, 0.011659635, -0.07462267, -0.07974469, -0.15836494, 0.12465921, -0.04001797, -0.04694436, 0.05780844, 0.0035512147, -0.02974517, 0.03763326, 0.14391673, 0.15917476) * inp_2_0_2;
    result1 += M4(-0.0054733646, 0.07263643, 0.037775997, -0.0839341, 0.07018293, -0.03195326, -0.02439077, 0.11775236, -0.004578805, 0.12776618, -0.31248224, 0.41912243, 0.039626777, 0.11221147, -0.090069614, 0.32341605) * inp_2_1_2;
    result1 += M4(0.03318654, -0.00037346288, 0.07595474, -0.29545853, 0.09850959, 0.09271038, -0.055692878, -0.41876775, -0.12090302, 0.021860281, -0.0485132, 0.25869316, 0.12551478, 0.11601167, 0.08036788, -0.22235693) * inp_2_2_2;
    result2 += M4(0.057724476, 0.001412806, 0.08581889, -0.23139188, -0.22039379, -0.2413023, -0.06627071, -0.016611796, 0.026109941, 0.14607431, 0.2469793, -0.11156607, 0.09840208, 0.15298077, -0.103305064, -0.1698864) * inp_2_0_0;
    result2 += M4(0.021000588, -0.03695654, -0.008774686, -0.12411673, 0.017505057, -0.1034351, -0.19600812, 0.04622594, -0.13292445, -0.11881531, 0.022501854, -0.06973899, -0.18078333, -0.12878609, 0.06356907, 0.0011069226) * inp_2_1_0;
    result2 += M4(0.47079045, -0.03309746, 0.045801003, -0.46056712, 0.07276125, -0.063882455, 0.18092608, -0.24902092, 0.1299754, -0.034312617, 0.2717586, -0.1372073, -0.16525201, -0.035831418, 0.08252455, 0.19915064) * inp_2_2_0;
    result2 += M4(-0.038180724, -0.123354964, 0.08969119, -0.007823074, -0.52864975, -0.43687066, -0.24947357, 0.3062412, -0.2821293, -0.37231222, -0.29444325, 0.30432, 0.029004663, 0.16786546, 0.14748505, -0.15503979) * inp_2_0_1;
    result2 += M4(-0.35730043, -0.26344103, -0.06993687, -0.45399222, 0.30368653, 0.38176364, -0.3048732, 0.55001813, 0.49270493, 0.35821933, -0.29239672, 0.26077515, -0.07670548, 0.20555444, 0.06793258, 0.26207882) * inp_2_1_1;
    result2 += M4(0.25442517, -0.3969137, -0.055527277, -0.16578002, 0.39195567, -0.065995716, 0.36755094, -0.028058624, 0.12227609, 0.016397579, 0.15453194, 0.07383472, -0.15106998, 0.05135229, 0.41270995, -0.15833637) * inp_2_2_1;
    result2 += M4(-0.10872487, -0.10342029, 0.057473995, 0.05288436, -0.36462107, -0.28009877, -0.10015184, 0.13036962, -0.018235588, -0.012834135, 0.020822058, 0.11575629, 0.25098822, 0.059997242, -0.17898306, -0.025334937) * inp_2_0_2;
    result2 += M4(-0.0073537226, 0.18807274, -0.07365404, 0.04858543, -0.010181402, -0.04178177, -0.23090595, 0.023130689, -0.1839737, -0.24239574, 0.059900083, -0.37247744, -0.033257786, -0.0057081827, -0.0727641, 0.10221611) * inp_2_1_2;
    result2 += M4(0.3194853, 0.014371719, -0.13260032, -0.044141047, 0.3220972, -0.10681125, 0.33147326, -0.17377193, -0.09510941, 0.17351691, -0.10594851, -0.065722, 0.08665872, 0.035880663, 0.29743567, 0.11018781) * inp_2_2_2;
    result3 += M4(-0.052927013, -0.5174794, -0.57640785, -0.054410987, 0.07248968, -0.17889194, -0.5400645, 0.0025586726, 0.0022557883, -0.45381483, -0.009867949, -0.115258195, -0.024773354, 0.12259194, 0.38769826, 0.0821551) * inp_2_0_0;
    result3 += M4(0.047797814, -1.132801, 0.23379998, 0.071528, 0.18533687, 0.06287021, 0.044287737, -0.003747413, -0.09610298, 0.6049399, -0.12367218, -0.20490903, 0.019250553, 0.5681266, -0.17649947, 0.22593169) * inp_2_1_0;
    result3 += M4(-0.04651503, -0.42470142, 0.14213495, -0.10224324, -0.02598372, 0.09212613, -0.09562424, 0.024036912, 0.086655475, -0.09309862, -0.23810424, 0.20251946, -0.0035155497, 0.16785572, -0.04239578, 0.07893037) * inp_2_2_0;
    result3 += M4(-0.06738507, 0.04612074, 0.18183503, 0.15344405, 0.011471549, -0.060696967, -0.8046626, -0.14718351, -0.066696785, 0.1562699, -0.28052536, 0.28606227, 0.027543148, -0.23327029, 0.20010807, -0.06536306) * inp_2_0_1;
    result3 += M4(-0.45907757, 0.16505389, -0.17519484, -0.18293017, 0.49241808, -0.020531196, 0.17354171, 0.23284785, 0.6548785, -0.25636014, 0.42952624, 0.14822266, -0.009240976, -0.07170324, 0.036269523, 0.12483055) * inp_2_1_1;
    result3 += M4(0.35353604, -0.055675123, -0.14708294, -0.042513866, -0.26392066, 0.0059200623, -0.0194938, 0.2284739, -0.2859118, 0.002964937, -0.010807703, 0.2636482, -0.39808747, -0.14613093, -0.35213697, 0.1719062) * inp_2_2_1;
    result3 += M4(-0.13939838, 0.025461996, -0.2207263, 0.0124531705, -0.03222462, -0.04793651, -0.13931951, 0.18809874, 0.17101718, 0.07797488, 0.12283034, 0.20291652, 0.081040464, -0.089602254, 0.4752905, -0.1251515) * inp_2_0_2;
    result3 += M4(0.43533564, -0.019357897, -0.14939448, 0.1896254, -0.4240308, -0.07118417, -0.028115949, 0.03331889, -0.76518387, 0.006083566, 0.15403745, -0.6948592, 0.17139912, 0.19854018, -0.025002103, -0.25610456) * inp_2_1_2;
    result3 += M4(-0.19286261, -0.010696067, 0.11392321, 0.023400566, 0.10416773, -0.093117505, -0.034862753, 0.1401014, 0.28115955, 0.0021862655, -0.042950924, -0.011018465, 0.25624534, -0.23166557, -0.14128067, -0.1395417) * inp_2_2_2;
    result4 += M4(-0.3739177, 0.23330803, 0.26339749, -0.005807815, 0.014868444, 0.089745484, 0.011422298, 0.08804699, 0.051226474, 0.3062516, 0.27061826, 0.26266497, 0.19366117, -0.1391938, -0.058357712, -0.1332936) * inp_2_0_0;
    result4 += M4(-0.6463295, 0.89011353, 0.8705065, 0.12015008, 0.25911883, -0.3981861, -0.054021873, 0.016178887, 0.32113722, -0.4164801, -0.37367934, 0.14944674, 0.3820476, -0.52520585, -0.54008466, -0.06908891) * inp_2_1_0;
    result4 += M4(0.022441948, 0.38176483, 0.34903094, 0.34774145, -0.023569208, 0.0063026706, -0.19421843, 0.13009152, 0.041146595, 0.0662253, 0.1181367, -0.27358693, 0.007935552, 0.03906263, -0.29328123, -0.3363809) * inp_2_2_0;
    result4 += M4(0.027838843, 0.02480065, 0.091410756, -0.05110536, -0.08037859, -0.13254218, -0.17506385, 0.097757384, 0.08842489, -0.22406933, -0.2550929, 0.019010257, -0.04249905, 0.05231701, 0.15345895, -0.13609427) * inp_2_0_1;
    result4 += M4(-0.06773401, -0.27124503, -0.14987823, -0.12925452, 0.04106765, 0.27270445, 0.21549943, -0.4805376, -0.2621179, 0.23363186, 0.024485655, -0.4257533, -0.016881868, 0.11049424, 0.12801866, 0.3607362) * inp_2_1_1;
    result4 += M4(0.06861509, -0.034282163, -0.1294472, -0.52650577, 0.18782115, -0.019438887, -0.15997823, 0.45722812, -0.01650927, -0.0018689133, -0.014349947, 0.25020573, -0.14025036, 0.2823825, 0.040148314, 0.49252912) * inp_2_2_1;
    result4 += M4(-0.07088465, 0.067930296, -0.0044128657, 0.063494004, -0.17670815, 0.08101386, 0.04568007, 0.0110607045, 0.35073176, -0.015810331, -0.037067015, -0.026630554, -0.10834112, -0.034264255, 0.23092791, -0.094318025) * inp_2_0_2;
    result4 += M4(0.28764957, 0.044370633, 0.101890974, -0.06420341, -0.19596781, 0.045037124, -0.03133305, -0.11785714, -0.5315873, 0.10500109, 0.09125529, 0.03932123, -0.007044694, -0.10311018, -0.022669163, -0.06344478) * inp_2_1_2;
    result4 += M4(-0.04636875, 0.08645628, -0.11097084, 0.1290511, 0.036433734, 0.07333574, -0.07168435, 0.13854054, 0.03046853, -0.03847721, 0.20378336, 0.033737183, -0.23585765, 0.0008267129, 0.06618149, -0.026402768) * inp_2_2_2;
    result5 += M4(0.07688099, 0.11621531, -0.03596591, -0.45984262, 0.079646796, 0.38535038, 0.06384355, -0.163427, 0.09629532, 0.17752475, -0.22604202, -0.15011443, -0.103282325, 0.0027606708, -0.1709319, 0.23072939) * inp_2_0_0;
    result5 += M4(0.13948068, 0.13010399, 0.08875485, -0.798808, 0.06619676, 0.003719729, 0.048301727, -0.02218738, -0.15188186, -0.33060336, -0.17521581, 0.26503626, -0.05535515, -0.40065056, -0.19203906, 0.41081664) * inp_2_1_0;
    result5 += M4(-0.1267752, 0.26698646, -0.028298615, -0.15904306, -0.023901265, 0.13773313, 0.031336673, 0.069330774, -0.058091264, -0.31386793, -0.05117996, -0.019965187, -0.11004113, 0.26474836, -0.0075633787, 0.12242906) * inp_2_2_0;
    result5 += M4(0.001315123, -0.08967905, -0.21683984, 0.16917078, -0.1243254, 0.33556706, 0.51739424, 0.05696327, -0.07826989, 0.35487178, -0.04188437, -0.10135021, 0.051972646, -0.25284332, -0.15185288, -0.18534786) * inp_2_0_1;
    result5 += M4(-0.28163013, 0.087246954, 0.013366833, -0.09257271, 0.57224613, 0.2181369, 0.034347124, -0.18253, 0.43985444, -0.16947658, 0.054141775, -0.094207056, -0.012949717, -0.011820775, 0.11144185, -0.031586997) * inp_2_1_1;
    result5 += M4(0.20259786, 0.08894934, -0.15689865, 0.049813192, -0.48923564, -0.2358704, 0.42517582, -0.050615128, -0.2307858, 0.0611569, 0.12879305, -0.11441994, -0.2443859, -0.0522228, 0.14179628, -0.25914487) * inp_2_2_1;
    result5 += M4(0.05764494, 0.33921456, 0.044625428, -0.017751582, 0.01690225, 0.17960005, 0.3137845, -0.06704236, 0.06732017, 1.1360378, -0.11092488, 0.018677473, -0.06955155, 0.020209415, -0.015863458, 0.05880711) * inp_2_0_2;
    result5 += M4(-0.12520139, -0.08618236, 0.020716468, -0.18261905, 0.15017317, 0.07853614, -0.19012818, -0.098162495, -0.22217709, -0.6105832, 0.4226759, 0.27255252, -0.048310935, -0.15282944, 0.014004568, 0.019936146) * inp_2_1_2;
    result5 += M4(0.035561547, 0.29872948, -0.039099842, -0.010471976, -0.2012462, 0.18738073, 0.104550734, -0.056581337, 0.13778536, -0.28153682, 0.053283658, -0.043497737, -0.074606776, 0.44355187, 0.14303862, -0.021337084) * inp_2_2_2;
    result6 += M4(0.112634696, 0.16953748, 0.052379683, 0.5584071, 0.5865797, 0.0064157387, 0.019492876, -0.3526775, 0.041345783, 0.336151, 0.03695398, 0.37217444, -0.17506213, -0.22187193, 0.12915136, -0.30016297) * inp_2_0_0;
    result6 += M4(-0.16295612, 0.22111064, 0.010594432, 0.78683, -0.3082541, 0.05513776, -0.10596833, -0.17383496, -0.10626953, -0.2966693, 0.0102795875, -0.6132398, -0.13209526, 0.011474233, 0.014162682, -0.36527586) * inp_2_1_0;
    result6 += M4(-0.041698817, -0.03230435, 0.028562762, 0.021288363, 0.45049953, 0.06517588, -0.24255545, -0.12276394, 0.08234611, 0.19750723, 0.052154616, -0.11375956, 0.30351156, -0.040281896, 0.09971311, 0.10196472) * inp_2_2_0;
    result6 += M4(0.20120919, -0.0050901426, 0.054440852, 0.19490218, -0.4307951, 0.031755667, -0.089258574, -0.22510026, -0.33001775, -0.3055703, -0.0021069834, -0.74051565, 0.03540687, -0.03866121, 0.027312024, 0.49167573) * inp_2_0_1;
    result6 += M4(-0.09725064, -0.22178878, -0.22075537, -0.16873322, 0.41188037, 0.20165943, 0.09593906, 0.17181072, 0.123869486, -0.00292663, -0.18012603, 0.16711923, -0.07558325, -0.014392749, 0.052088805, -0.07674485) * inp_2_1_1;
    result6 += M4(-0.084349856, 0.06602209, -0.00033241825, -0.102612965, -0.5197442, 0.1326572, -0.13953003, 0.1788996, -0.39011675, -0.00557521, -0.1807771, -0.1899016, 0.11786232, 0.07557294, 0.007957731, -0.013600629) * inp_2_2_1;
    result6 += M4(0.04867372, -0.052597906, -0.069009244, 0.08898578, -0.009354843, -0.06426343, 0.081881054, -0.2694366, -0.004480465, 0.07505591, 0.019822828, 0.29468566, 0.0317458, 0.012437868, 0.07382326, 0.17835645) * inp_2_0_2;
    result6 += M4(-0.023986168, -0.05843806, 0.12213107, -0.21878517, -0.13765971, 0.039790604, 0.047566082, -0.06877765, 0.25280216, 0.1649335, 0.009922982, 0.12748815, -0.0043473807, 0.06912965, 0.012035092, 0.06945373) * inp_2_1_2;
    result6 += M4(-0.11432987, 0.048129637, -0.002010731, 0.16498773, -0.14913662, 0.121214986, 0.0035933736, 0.14037101, 0.17240964, -0.13321064, 0.2232337, 0.63483125, -0.12734045, 0.030908737, -0.10541092, -0.13146289) * inp_2_2_2;
    result7 += M4(-0.0153750675, 0.16340362, 0.1549973, -0.026151787, -0.03565179, -0.08433068, -0.06934278, -0.28113946, -0.025983544, -0.18679847, 0.09776474, -0.29104713, -0.08410125, 0.081313066, -0.21002433, -0.12308652) * inp_2_0_0;
    result7 += M4(0.02475679, 0.048625994, -0.021352543, -0.11060805, 0.24802865, 0.10500063, -0.023609456, -0.1212433, -0.04837286, 0.010227467, -0.28921923, 0.07676356, -0.1279043, -0.12993357, 0.06325884, 0.06843497) * inp_2_1_0;
    result7 += M4(-0.118155316, -0.08497501, -0.2778288, 0.005133327, -0.019045567, 0.18918684, 0.29344165, -0.054198094, -0.007141581, 0.17824881, -0.048405986, 0.01927543, -0.030169345, -0.01645672, 0.70765394, -0.22126992) * inp_2_2_0;
    result7 += M4(-0.20475274, 0.3196863, -0.08956707, 0.30072972, -0.014788609, -0.2992024, 0.085849166, -0.5443602, -0.34682554, -0.24297374, 0.091763005, -0.4006719, 0.13680574, -0.11139741, -0.24142939, 0.3756479) * inp_2_0_1;
    result7 += M4(-0.6938271, -0.32799092, 0.09303433, -0.2370136, 0.25698695, 0.3298745, -0.13991725, 0.20185243, 0.73119485, 0.328759, -0.13318634, 0.30523822, 0.3239923, 0.40927178, -0.19340658, 0.16107222) * inp_2_1_1;
    result7 += M4(-0.21665075, -0.05512588, -0.014360347, -0.051036727, 0.07715817, -0.1988644, 0.016061177, 0.20529053, -0.11264627, -0.029451434, -0.18438397, 0.33249667, 0.1497252, 0.0744202, 0.42392698, -0.11500206) * inp_2_2_1;
    result7 += M4(0.08813388, -0.3217021, -0.24371512, 0.2237141, -0.0828113, 0.16869392, 0.58881855, -0.57096016, 0.27723303, -0.07348669, 0.1782379, -0.52130604, -0.36056453, 0.13806035, -0.27184355, 0.40185755) * inp_2_0_2;
    result7 += M4(0.3056401, 0.17291003, 0.1084398, -0.15465526, -0.12150846, -0.20386326, 0.11828008, -0.22114685, -0.8182182, 0.11132604, 0.24497974, 0.4008387, 0.09207106, -0.2891656, -0.45780572, -0.12221976) * inp_2_1_2;
    result7 += M4(0.19135028, -0.11964563, 0.03740506, -0.046266373, -0.023875643, -0.3071676, 0.0796005, 0.01661964, 0.33135328, -0.0649288, 0.059491508, 0.06498407, -0.06439623, -0.26991135, -0.045774728, 0.15992664) * inp_2_2_2;
    const V4 inp_3_0_0 = inp[3][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_3_1_0 = inp[3][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_3_2_0 = inp[3][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_3_0_1 = inp[3][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_3_1_1 = inp[3][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_3_2_1 = inp[3][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_3_0_2 = inp[3][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_3_1_2 = inp[3][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_3_2_2 = inp[3][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.18898425, 0.291575, -0.061722476, 0.14693774, -0.022125702, 0.021088704, 0.023908699, 0.010248642, -0.04080439, 0.018247878, 0.021257216, -0.0031792796, -0.048703745, 0.276557, 0.06829409, -0.07498463) * inp_3_0_0;
    result0 += M4(-0.13271399, -0.28317246, 0.22144566, -0.004917188, -0.13270383, -0.0947697, 0.069102064, -0.13574633, -0.015218175, -0.040410858, 0.022689894, -0.09377845, -0.03589918, -0.1384443, -0.043065343, 0.07834419) * inp_3_1_0;
    result0 += M4(-0.011964419, -0.061483707, -0.052917298, 0.12573245, 0.018006086, 0.058913715, -0.019590048, -0.013268406, 0.042956408, -0.0073637413, -0.06635184, 0.052598212, 0.061013427, -0.06284602, -0.024052273, 0.11623441) * inp_3_2_0;
    result0 += M4(0.1487832, 0.6688938, -0.11193207, 0.03581165, 0.103470236, -0.04635191, -0.0795634, -0.03428114, -0.018823467, -0.040830474, -0.0055451635, -0.037857134, 0.07081023, -0.38651276, -0.0324409, 0.016321894) * inp_3_0_1;
    result0 += M4(-0.061237346, -0.33768037, -0.028849373, -0.07214201, -0.10321171, -0.013108828, -0.1279186, 0.059050985, 0.08737682, 0.05861349, 0.03129082, 0.12232065, 0.061487924, 0.14658585, 0.04831864, -0.018338196) * inp_3_1_1;
    result0 += M4(0.16864146, -0.22925776, -0.11393271, 0.021888772, 0.16317365, 0.13284361, -0.044329714, 0.026397178, 0.00562982, -0.014000551, -0.005146578, -0.036523018, -0.059099518, 0.13842571, -0.023233395, -0.08331662) * inp_3_2_1;
    result0 += M4(0.05234591, 0.30753034, -0.099197984, -0.15620501, -0.036455173, -0.14343107, 0.02639039, 0.06132225, 0.027379315, 0.032852717, -0.018570095, -0.033086866, -0.011217479, -0.017946001, -0.014154288, 0.06270892) * inp_3_0_2;
    result0 += M4(0.0049735378, -0.16602385, 0.12996367, -0.031539943, -0.032842774, -0.053054806, -0.090486415, -0.01076484, -0.059575863, 0.030861413, 0.0060979063, -0.0057128286, -0.063221954, 0.09743514, 0.026264075, -0.14427833) * inp_3_1_2;
    result0 += M4(0.10558327, -0.012665829, -0.05515791, -0.050036557, 0.05740738, 0.13066679, 0.05681485, 0.07314771, -0.019621221, -0.029415315, 0.009767939, 0.03805756, 0.009099535, -0.047612138, 0.01670276, 0.043647967) * inp_3_2_2;
    result1 += M4(0.00456792, -0.23990019, 0.11986005, -0.020280862, -0.03892396, -0.01824271, -0.04225369, -0.040559087, -0.041293796, 0.011075429, 0.02580548, -0.027978873, -0.011511918, -0.21014334, 0.07495654, -0.014282626) * inp_3_0_0;
    result1 += M4(0.012663989, -0.06305359, -0.005730604, 0.14834055, -0.088035814, 0.07813173, 0.076148406, -0.21165636, 0.07579863, 0.039044358, 0.03360743, -0.030678209, 0.023705235, 0.21957953, 0.08990768, -0.054248534) * inp_3_1_0;
    result1 += M4(0.24986957, 0.15844785, -0.1016871, -0.102435805, 0.03505946, 0.0135202, -0.0104242265, -0.027614385, -0.03970909, -0.07471214, -0.075802974, 0.051321603, 0.08343938, -0.119302556, -0.054541994, 0.06839875) * inp_3_2_0;
    result1 += M4(-0.09865286, -0.0855772, 0.15371707, -0.07880625, 0.063628785, 0.14560711, 0.018285954, 0.16305149, 0.02371025, 0.0018474398, -0.014267369, -0.0051176585, -0.048461035, -0.051086944, -0.1125831, -0.075111106) * inp_3_0_1;
    result1 += M4(-0.16403013, 0.030175457, -0.025496654, 0.11332837, 0.09955438, 0.14093259, -0.0007396667, -0.15423453, -0.070228085, -0.08039371, -0.03863463, 0.105587356, 0.18635885, -0.042819854, 0.05563064, -0.10247018) * inp_3_1_1;
    result1 += M4(0.11314658, 0.11475463, 0.0033028952, -0.028300948, 0.10385804, -0.033111624, 0.0005002298, 0.043353584, 0.07315249, 0.095554926, 0.08963826, -0.09079311, -0.2570769, 0.13410723, -0.13701375, 0.21545015) * inp_3_2_1;
    result1 += M4(-0.029019676, 0.03845977, 0.003149422, -0.009275528, 0.003849309, 0.04264313, 0.046974223, 0.1459158, -0.0072650714, -0.095452, -0.0043309676, 0.028062286, 0.009724209, 0.19761232, 0.025261559, 0.10360836) * inp_3_0_2;
    result1 += M4(-0.058368005, 0.012107906, 0.04968661, 0.12188344, -0.08943931, 0.07957856, -0.0021639587, 0.045702443, 0.014235795, 0.06364201, 0.040949255, -0.056683008, -0.1679991, -0.18332006, -0.028061653, 0.21650492) * inp_3_1_2;
    result1 += M4(0.00014916724, 0.00943583, -0.05990642, 0.01680546, -0.08850472, 0.0039096368, -0.006874372, 0.065508574, -0.028935244, -0.052228305, -0.036107484, 0.016058315, 0.17812766, 0.030521104, 0.068151474, -0.3358205) * inp_3_2_2;
    result2 += M4(0.11476918, 0.12219196, 0.16418898, -0.66710335, 0.035750408, -0.002784234, -0.02181527, -0.10429968, 0.0041955886, -0.00018118808, 0.003594439, 0.06858864, 0.09336084, 0.09015931, 0.124352835, -0.12684177) * inp_3_0_0;
    result2 += M4(-0.11766873, -0.10751612, -0.12846822, 0.37174696, -0.028493369, 0.05460859, 0.042917136, 0.0124626, -0.060918253, -0.034215156, 0.016110107, -0.04717236, -0.062953815, -0.06725237, -0.0038987754, 0.21732345) * inp_3_1_0;
    result2 += M4(-0.2131502, -0.098095015, -0.024533644, 0.22987323, -0.08996307, -0.044345886, -0.080055274, -0.07436349, 0.038980205, -0.006087008, -0.0074494225, -0.06267265, 0.019817626, 0.0011141625, -0.023195228, -0.2417616) * inp_3_2_0;
    result2 += M4(0.1531615, 0.3747804, 0.14962557, -0.18928313, 0.12924314, 0.092626795, 0.048440978, -0.17279796, -0.049772467, 0.00020447923, -0.009873994, -0.1591585, -0.08793431, -0.07346416, -0.1738986, -0.042440582) * inp_3_0_1;
    result2 += M4(-0.16217856, -0.07140058, 0.07640337, -0.06457851, 0.02719563, 0.19784553, 0.019423094, 0.192794, 0.0805526, 0.07173286, -0.014318049, 0.19705434, 0.32921526, 0.1456285, 0.04064316, -0.069415316) * inp_3_1_1;
    result2 += M4(-0.064924344, -0.0504818, -0.0733605, 0.019870961, -0.16896923, 0.076375425, -0.01618084, 0.11858032, -0.023817504, -0.050634444, 0.0031132428, -0.036045123, -0.13305505, -0.080176055, -0.01263651, 0.27672407) * inp_3_2_1;
    result2 += M4(0.42034805, 0.20536813, -0.040567454, -0.35557795, 0.018582698, -0.16705757, -0.03261912, -0.07779998, 0.005941442, 0.0036671523, -0.028199088, 0.103401385, -0.08413099, -0.13199575, 0.06300005, 0.21444835) * inp_3_0_2;
    result2 += M4(-0.042118825, -0.0133740185, 0.12999201, 0.15468611, 0.012985766, -0.12173105, 0.036345564, 0.034626957, -0.006896667, 0.017636385, 0.026484223, -0.07366979, -0.11947948, 0.06689162, -0.1785031, -0.15233819) * inp_3_1_2;
    result2 += M4(-0.16532758, -0.31511045, -0.2317241, 0.44619024, 0.02689671, -0.07670926, -0.06947053, -0.041049127, 0.024975395, -0.0007117158, 0.026667755, 0.013615376, 0.050792653, 0.04710853, 0.13138746, -0.08191431) * inp_3_2_2;
    result3 += M4(-0.00047281553, -0.009867511, 0.38717154, -0.085363165, 0.050538328, -0.15022695, -0.0994756, -0.04180447, 0.035130676, 0.056177106, -0.010576, -0.011156313, 0.04320468, -0.22672321, -0.5257049, -0.077716716) * inp_3_0_0;
    result3 += M4(-0.13208178, 0.12477081, -0.21931244, 0.16052003, -0.10047274, 0.08195659, -0.0664801, -0.061796766, -0.06447204, -0.06332469, -0.026440825, 0.11732688, -0.14196949, 0.26690507, 0.20109898, 0.015952885) * inp_3_1_0;
    result3 += M4(0.04810481, 0.07723198, -0.19662032, 0.011374177, 0.07468607, 0.06761588, -0.049966905, 0.018628774, 0.0009902745, -0.005393279, 0.0079566585, -0.09793498, 0.051823072, -0.023763776, 0.27408338, 0.15832107) * inp_3_2_0;
    result3 += M4(-0.074422486, -0.044656634, 0.2898587, 0.06298712, -0.01518616, 0.06406038, 0.057199273, -0.012180423, -0.023158178, 0.0005442024, 0.04728296, -0.078961916, -0.027104966, 0.12010583, 0.06029023, 0.17390285) * inp_3_0_1;
    result3 += M4(0.005386401, -0.083401315, -0.13327897, 0.007947267, -0.038257908, 0.15271744, 0.12504464, 0.073065676, 0.043758076, 0.06223881, -0.033510327, -0.10013652, 0.11710908, -0.035505135, -0.0009008535, 0.0046008406) * inp_3_1_1;
    result3 += M4(0.06633504, -0.18508743, -0.079345986, 0.059515253, -0.042905908, -0.005960921, 0.14241229, -0.0071819886, -0.0008865221, -0.005838218, 0.04973574, 0.081774995, 0.028309753, -0.06582205, -0.12040953, -0.16185874) * inp_3_2_1;
    result3 += M4(0.051608335, 0.017378084, 0.3089071, -0.19371884, 0.038767867, -0.16759078, 0.032621928, 0.025578445, -0.00023755356, -0.030640995, 0.015046336, 0.09735791, -0.03521915, 0.051206395, 0.19995798, -0.22498097) * inp_3_0_2;
    result3 += M4(0.10374095, -0.04885578, -0.07870265, -0.27914393, 0.09294524, -0.062048715, -0.08732918, 0.105123796, 0.025347658, -0.015904367, -0.022560203, -0.020714136, 0.07217001, -0.113494664, -0.078100346, 0.15815341) * inp_3_1_2;
    result3 += M4(-0.09524351, -0.06394484, -0.1617012, 0.04229062, -0.02166816, -0.008166774, 0.032111544, -0.00756445, 0.0050624455, 0.027077613, -0.03567809, 0.010427402, -0.12921853, 0.016202807, -0.03196146, -0.071933806) * inp_3_2_2;
    result4 += M4(0.12242533, -0.07674831, 0.054159198, -0.16273373, -0.06003675, 0.008656486, 0.06178957, 0.1359734, 0.07196773, 0.0022787363, 0.02366818, -0.06125349, 0.04108278, 0.0969364, 0.14692457, -0.063847385) * inp_3_0_0;
    result4 += M4(-0.048062768, -0.042662997, -0.26525763, 0.07260901, 0.041845076, -0.12547067, -0.07991441, 0.26612192, 0.023196585, -0.074596085, 0.0075986045, -0.016512932, 0.108508766, -0.1932214, -0.21672018, 0.15656923) * inp_3_1_0;
    result4 += M4(0.05382612, -0.0033462597, 0.13519815, -0.22031572, -0.105710514, -0.0095332395, -0.063694276, 0.21047534, -0.15197092, 0.09995336, -0.0068561016, -0.040798478, -0.02298197, 0.043114647, 0.03574411, -0.11977219) * inp_3_2_0;
    result4 += M4(-0.14801845, 0.039097633, 0.17756791, 0.05060088, -0.0110762445, 0.0007288805, -0.0293652, 0.002042276, -0.015578498, -0.010527784, -0.0024964584, -0.014171364, -0.059836768, -0.07666803, -0.10455488, 0.043944575) * inp_3_0_1;
    result4 += M4(0.031037306, -0.030817393, 0.02831715, 0.075475894, 0.116013356, 0.033402704, -0.077723905, 0.20610738, -0.06733321, 0.10708126, -0.00020568825, 0.049512833, -0.041350495, 0.07109068, 0.16513824, -0.12339117) * inp_3_1_1;
    result4 += M4(-0.20280594, 0.15320116, 0.11488065, -0.014950845, 0.00971464, -0.026365321, 0.027397748, 0.21373214, 0.11723963, -0.11702759, 0.005081529, -0.06213027, 0.0026627167, -0.0016529203, 0.08459672, -0.007704027) * inp_3_2_1;
    result4 += M4(0.26501402, -0.10359216, -0.09785157, -0.008028277, -0.05003541, -0.0064222706, 0.06561313, -0.08026002, -0.044000268, -0.0025128254, -0.025534391, 0.025565675, -0.10883505, 0.013206194, 0.16096134, -0.0127850035) * inp_3_0_2;
    result4 += M4(0.08631148, 0.055833764, -0.01145649, 0.11213601, 0.07163718, 0.023215257, 0.067192666, 0.18573873, 0.08101908, -0.028066676, 0.007949582, -0.08111778, 0.2253773, -0.011821263, -0.0948351, -0.047216892) * inp_3_1_2;
    result4 += M4(-0.046617974, 0.021306306, 0.01531973, -0.1439318, -0.0004477247, 0.054696914, 0.08097244, 0.078228444, -0.00022891925, 0.012469684, -0.009363124, 0.052688416, -0.16335802, 0.049735136, -0.17021139, 0.07950753) * inp_3_2_2;
    result5 += M4(-0.10241755, -0.08017574, -0.25171524, -0.25954306, -0.033294957, -0.09173958, -0.11190315, -0.09979342, 0.029529339, 0.07855416, 0.05884228, 0.026005927, 0.04528758, 0.13671516, 0.07038915, -0.12426077) * inp_3_0_0;
    result5 += M4(-0.07543016, -0.29333854, 0.11205348, 0.17395023, 0.046305012, -0.0076197204, 0.12884322, -0.07109099, 0.017455548, -0.032654677, -0.08157761, 0.011856915, -0.1640484, -0.31875846, -0.04518671, 0.11148263) * inp_3_1_0;
    result5 += M4(0.079700276, -0.10423748, -0.037925187, 0.019203082, -0.12918551, -0.057582404, -0.091268584, -0.01754391, -0.051827304, -0.080260314, 0.04165195, -0.013490427, 0.146066, 0.26714754, -0.12784432, 0.07092448) * inp_3_2_0;
    result5 += M4(0.14208613, -0.1469741, -0.33727136, 0.11167781, 0.009155, -0.022866763, 0.005533951, 0.12601624, -0.03468309, -0.09434828, -0.024818059, -0.07405298, -0.15768513, -0.16726346, -0.06595057, 0.19353347) * inp_3_0_1;
    result5 += M4(-0.08541471, -0.16924189, 0.041396763, 0.015475087, 0.17115368, 0.13935843, 0.20303997, 0.028013846, 0.03914277, 0.06865221, 0.01241453, 0.038714282, 0.10767945, 0.08260549, 0.1497872, -0.003076639) * inp_3_1_1;
    result5 += M4(0.27671388, 0.032656383, 0.099902384, -0.13109955, -0.08293625, -0.030300481, 0.038315933, 0.29419717, -0.015930442, 0.050407793, 0.020819083, -0.0017386589, 0.017431308, -0.29225692, -0.02989799, -0.18331347) * inp_3_2_1;
    result5 += M4(-0.06569525, 0.8629017, -0.08260625, 0.061461166, 0.0029196546, -0.019689057, -0.066908285, -0.020419415, 0.026359782, 0.028930267, -0.07995015, 0.05373659, 0.16285503, 0.27935442, -0.0014770825, -0.09198014) * inp_3_0_2;
    result5 += M4(-0.010030636, -0.26974282, 0.13482796, 0.067334205, 0.07143617, 0.031639494, -0.126793, -0.16655019, -0.053417113, -0.08822844, 0.06676803, -0.044502877, 0.026961608, -0.3208662, -0.04571035, -0.03760058) * inp_3_1_2;
    result5 += M4(0.1393679, 0.03626961, 0.14030764, 0.04892348, -0.08157433, 0.024898946, -0.059842184, -0.11266688, 0.025195621, 0.06267587, -0.007585398, 0.021174682, -0.18445438, 0.36225438, 0.084032886, 0.07873273) * inp_3_2_2;
    result6 += M4(-0.4641928, 0.21418814, 0.043069053, 0.1937276, 0.016511815, -0.0004795628, -0.003485375, 0.0072185104, 0.063207336, 0.017030608, 0.021120286, 0.18920435, -0.020409176, 0.06805725, -0.036584873, -0.34251234) * inp_3_0_0;
    result6 += M4(-0.10737162, -0.032349154, -0.04723789, -0.18976146, -0.03476587, 0.06819137, 0.005741066, -0.11304543, -0.16258207, 0.034100275, -0.014337058, -0.111493506, -0.2742143, -0.14103167, -0.047986146, 0.011797736) * inp_3_1_0;
    result6 += M4(0.046506744, 0.024793563, 0.024559924, -0.21751636, 0.060789924, 0.09669237, -0.0880334, -0.016028818, 0.110536255, -0.033822495, 0.02421046, -0.09629687, 0.49179104, 0.0030373465, 0.083174996, -0.09351132) * inp_3_2_0;
    result6 += M4(0.0979625, -0.20295683, 0.078715034, 0.32539374, 0.10915168, 0.06798971, -0.018339632, 0.112481266, 0.0010810237, -0.026096925, -0.013409975, -0.18919115, 0.15780258, -0.04375088, 0.075301506, 0.122687824) * inp_3_0_1;
    result6 += M4(0.021166287, -0.21687056, -0.088762246, -0.123087615, -0.008682862, 0.05577262, 0.15535028, 0.010314388, 0.11169758, -0.050572425, 0.02524952, 0.15223844, -0.08609121, 0.23826933, -0.041660126, 0.14628546) * inp_3_1_1;
    result6 += M4(0.12708136, 0.08975476, 0.023181958, -0.119556636, -0.042802684, -0.076235615, 0.030202346, 0.07447099, -0.16874473, 0.069822885, -0.011717638, 0.10667488, -0.21028344, -0.13343525, -0.05094395, 0.15868737) * inp_3_2_1;
    result6 += M4(0.3235911, 0.246408, 0.13083087, -0.18758868, -0.015358463, 0.048794594, 0.04826582, -0.017100973, -0.061680265, -0.020960664, -0.0163859, 0.09120823, -0.16231468, 0.039769717, -0.048372075, 0.08721501) * inp_3_0_2;
    result6 += M4(0.03889669, -0.053947832, -0.16974878, 0.07206486, -0.029255325, 0.03760333, 0.0043013594, -0.04055084, 0.013872555, 0.040686987, -0.013391235, -0.098517634, 0.15343857, -0.16313158, 0.070539795, 0.00021509123) * inp_3_1_2;
    result6 += M4(-0.07693275, -0.12588927, -0.055385772, 0.21134058, -0.05842295, 0.06727941, 0.00885826, 0.0709185, 0.08640614, -0.0061283214, 0.01574293, -0.059052538, -0.039693743, 0.14397699, -0.029542869, -0.082473956) * inp_3_2_2;
    result7 += M4(-0.05190244, -0.054420985, 0.041540634, 0.005712655, -0.013730356, -0.029829336, -0.13799478, -0.06419685, 0.057393927, -0.13765778, 0.0057806475, -0.04419063, -0.066855796, 0.41033912, -0.09601113, -0.16233273) * inp_3_0_0;
    result7 += M4(-0.16634467, -0.07896031, 0.08347476, -0.03600994, 0.100064285, 0.009680261, 0.011416894, -0.008643797, -0.06501771, 0.08498901, -0.16893989, -0.003847895, 0.026262391, -0.22370632, -0.11472851, 0.029483415) * inp_3_1_0;
    result7 += M4(0.004243239, -0.100785345, -0.27970678, -0.17671338, 0.049340017, 0.027377471, 0.03127678, -0.015358497, 0.019405983, 0.05634121, 0.14853626, 0.0154647855, 0.0076292306, -0.029460093, 0.21898134, 0.060022533) * inp_3_2_0;
    result7 += M4(0.04008658, 0.25174615, 0.03191642, 0.5974086, 0.13000312, -0.02125426, 0.0292592, 0.13164665, -0.024005063, -0.11994486, -0.03428617, 0.07585266, 0.015725562, -0.416273, 0.09843546, -0.0037847634) * inp_3_0_1;
    result7 += M4(0.094510645, 0.24825996, 0.01897398, -0.26158148, 0.21043968, 0.012483479, 0.04916805, 0.0630967, 0.07103491, 0.15561447, 0.054502275, -0.0003150257, 0.02044771, 0.16270803, -0.116279304, 0.24781822) * inp_3_1_1;
    result7 += M4(0.015495285, 0.2972982, 0.35345525, -0.18845339, 0.068883166, 0.0035287407, 0.044952992, 0.08539798, -0.07447563, -0.072417416, -0.09059275, -0.034165844, 0.048053198, 0.22143486, -0.12368038, -0.16225325) * inp_3_2_1;
    result7 += M4(0.011749403, -0.5525425, -0.30366263, 0.48875955, -0.23496984, 0.05263196, -0.010573139, -0.051322695, -0.011874343, 0.16181739, -0.003636144, 0.0017791846, 0.032289714, 0.24802302, 0.053176545, 0.0005880097) * inp_3_0_2;
    result7 += M4(0.08271831, 0.18437402, -0.15471019, 0.0621546, -0.18634082, 0.11720758, 0.087020144, -0.10232882, -0.012153655, -0.142367, 0.06270467, -0.0055558067, 0.017142883, 0.026320042, 0.09089323, -0.24522154) * inp_3_1_2;
    result7 += M4(-0.06548487, -0.04030401, 0.16714129, -0.289716, -0.13194431, -0.13892917, -0.11927318, -0.05711083, 0.051703632, 0.003986902, 0.007343517, 0.031077174, -0.1196945, -0.37468368, -0.0026876417, 0.19949237) * inp_3_2_2;
    const V4 inp_4_0_0 = inp[4][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_4_1_0 = inp[4][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_4_2_0 = inp[4][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_4_0_1 = inp[4][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_4_1_1 = inp[4][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_4_2_1 = inp[4][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_4_0_2 = inp[4][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_4_1_2 = inp[4][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_4_2_2 = inp[4][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.028501337, -0.032498002, 0.041742377, -0.036198482, -0.05022406, -0.21708047, 0.016502135, 0.10079413, 0.10264014, -0.021814974, -0.15598558, 0.048394345, 0.039621238, -0.06393471, 0.01058105, -0.043067075) * inp_4_0_0;
    result0 += M4(-0.033471435, -0.010740814, -0.017997019, 0.015881833, -0.04669797, 0.061318252, 0.023976145, -0.0950422, -0.10014168, 0.00413283, 0.00958143, 0.045276344, -0.041370768, -0.011398185, -0.010230595, 0.09412156) * inp_4_1_0;
    result0 += M4(-0.0035975135, 0.011823529, 0.003661108, 0.014134702, 0.19902286, 0.08849765, 0.0239018, -0.08510159, -0.078094855, 0.05413778, 0.09641204, -0.04019996, -0.040417716, -0.009934488, 0.06414003, -0.0909776) * inp_4_2_0;
    result0 += M4(-0.044227812, 0.13628614, -0.0589741, -0.0043714996, -0.017040793, 0.09993164, -0.013907497, 0.030320577, 0.11853374, -0.539124, 0.14740911, 0.08697109, -0.02025099, -0.2220861, -0.056291923, 0.03788586) * inp_4_0_1;
    result0 += M4(-0.024096059, -0.05107199, -0.0039150333, -0.021817658, 0.14587268, 0.055750243, 0.055479154, 0.1964869, -0.054344233, 0.30923018, -0.083993025, -0.0794277, -0.09379845, 0.17196093, 0.0070988815, -0.16563748) * inp_4_1_1;
    result0 += M4(0.06384143, -0.043809656, 0.030024678, 0.010375811, -0.20958515, -0.02664741, -0.10486594, -0.10701769, 0.0061279493, 0.0229904, 0.011038696, 0.021407409, 0.14567128, 0.04799895, 0.028875005, 0.07940654) * inp_4_2_1;
    result0 += M4(0.027309075, -0.06331916, 0.029172337, 0.013199073, 0.13754117, 0.09745345, -0.018351274, -0.11037507, -0.23839028, 0.5030434, -0.08619372, -0.12842594, -0.04900734, 0.1489212, 0.010313216, 0.0007514596) * inp_4_0_2;
    result0 += M4(0.048454635, 0.01413741, 0.020043809, 0.016886069, -0.07935776, -0.15454033, -0.08736256, 0.02693007, 0.18856694, -0.32453912, 0.1735864, -0.008622642, 0.19527365, -0.110464625, 0.024351463, 0.08189491) * inp_4_1_2;
    result0 += M4(-0.05539238, 0.043401293, -0.03801224, 0.016549982, -0.033573393, 0.0037911679, 0.12468378, 0.045741837, 0.0388641, 0.0137331225, -0.10848692, 0.034042563, -0.14517061, 0.022399072, -0.0608981, -0.004954646) * inp_4_2_2;
    result1 += M4(0.03439602, -0.027441492, 0.0037317562, -0.020587606, 0.033591516, 0.12633203, -0.04972526, -0.0033041246, -0.020159308, -0.16818526, 0.052164756, -0.0056580496, 0.06293669, -0.026287941, -0.030159524, 0.096257046) * inp_4_0_0;
    result1 += M4(-0.05528174, -0.013696878, -0.020885322, 0.0053830342, 0.24789472, 0.043464106, -0.0375021, -0.031131657, -0.08631768, 0.07987553, -0.07030414, 0.178824, -0.07892212, 0.06464853, -0.02345434, -0.014996862) * inp_4_1_0;
    result1 += M4(-0.00089357066, 0.011333243, 0.03905352, 0.006071301, -0.33272585, -0.030125683, 0.0184583, 0.13395424, 0.11698213, 0.027662419, 0.013353388, -0.21382071, -0.02220286, -0.031552397, 0.031481966, -0.048073407) * inp_4_2_0;
    result1 += M4(0.008122189, 0.012907259, -0.042602077, -0.0053025605, -0.11116049, 0.059802502, -0.021756109, 0.12212234, -0.023482472, 0.109654486, 0.17409563, -0.13464369, -0.057522535, -0.028213305, 0.04473061, -0.18901424) * inp_4_0_1;
    result1 += M4(-0.021746373, 0.06596691, 0.03078931, -0.019069215, -0.17057656, -0.26102573, -0.06350699, 0.1381066, 0.18579422, -0.042946827, -0.013941636, 0.041349195, 0.14742038, 0.019213999, 0.027135113, 0.093506776) * inp_4_1_1;
    result1 += M4(0.0040629036, -0.12332428, 0.01367097, 0.07044763, 0.3637614, 0.09224528, 0.12439672, -0.46854317, -0.09831975, -0.060871907, -0.14967218, 0.13511439, -0.03477788, 0.013423131, 0.021441273, 0.14672896) * inp_4_2_1;
    result1 += M4(-0.031475917, 0.023870956, 0.014098818, -0.010316382, 0.10557338, -0.16654107, 0.06084343, -0.09806287, 0.034022294, -0.076604985, -0.08999175, 0.04089936, -0.013663528, 0.067366436, 0.023334192, 0.027873524) * inp_4_0_2;
    result1 += M4(0.08901907, -0.0011845443, 0.027644297, -0.011896531, -0.049916893, 0.3115381, 0.037901178, -0.102237426, -0.08366911, -0.022650609, 0.025179619, -0.13449274, -0.057367995, 0.0010812387, -0.016577916, -0.08258796) * inp_4_1_2;
    result1 += M4(-0.042179022, 0.082840666, -0.0677767, -0.047849793, -0.055448264, -0.16343525, -0.08864688, 0.33295572, -0.06422149, 0.07727354, 0.07719131, 0.0744407, 0.01271183, -0.023921708, -0.06312893, -0.08229153) * inp_4_2_2;
    result2 += M4(0.043311987, 0.03920799, 0.08499997, -0.14009428, -0.124410324, -0.05866371, -0.16833332, 0.42460486, -0.08934771, -0.10326171, 0.056385074, -0.033315383, -0.0672601, -0.06200844, -0.011034799, -0.06748634) * inp_4_0_0;
    result2 += M4(-0.01372075, -0.01574042, -0.0602313, 0.06390709, 0.057960473, 0.079398654, 0.17745802, -0.5232174, -0.05400962, -0.033960935, 0.07325879, 0.07689693, 0.04283781, 0.056583025, 0.040818702, 0.21833026) * inp_4_1_0;
    result2 += M4(-0.016632374, 0.0019968187, -0.03129195, 0.088357925, -0.08280144, -0.09861661, -0.080277756, 0.18768576, 0.08052269, 0.16558811, 0.10677247, -0.1123574, 0.02062374, 0.0019740744, 0.060419496, -0.08113049) * inp_4_2_0;
    result2 += M4(0.01653658, 0.052331585, 0.0066918083, 0.05480998, 0.047534462, 0.0714538, 0.015856469, -0.34148243, 0.030921109, -0.24478127, 0.09638878, 0.4326831, 0.02105345, -0.075151965, 0.012738955, 0.1899191) * inp_4_0_1;
    result2 += M4(-0.0017488858, 0.015035796, 0.10791426, -0.0046132463, -0.16477664, -0.22743417, -0.19230528, 0.4484108, 0.34829137, 0.14530021, -0.13541548, -0.15577696, -0.024470124, 0.046432264, -0.05394563, -0.27798045) * inp_4_1_1;
    result2 += M4(-0.037074313, -0.0585019, -0.11909679, -0.0680991, 0.27241892, 0.20595247, 0.31890818, -0.1980977, -0.1923399, -0.012961347, -0.19870889, -0.06575645, -0.07692685, 0.0036747474, -0.01937965, 0.046767898) * inp_4_2_1;
    result2 += M4(-0.017400416, -0.078874245, -0.0664264, 0.043989085, 0.09510488, 0.077542804, 0.10613931, -0.042933367, -0.037055347, 0.3159191, -0.083585575, -0.4046708, -0.01789831, 0.11440186, -0.0075702416, -0.19867507) * inp_4_0_2;
    result2 += M4(0.0071849017, 0.006786318, -0.032416232, -0.07264857, 0.015009507, -0.07083537, 0.106787466, 0.05303945, -0.11464851, -0.18704842, 0.105538554, 0.07574464, 0.046960305, -0.08111558, -0.019508623, 0.13521874) * inp_4_1_2;
    result2 += M4(0.04671564, 0.02849086, 0.10468871, 0.059944756, -0.07282332, 0.030870767, -0.27236143, 0.03369138, 0.013875321, -0.065407895, 0.010434228, 0.1685898, 0.06672334, -0.016422099, 0.006902955, 0.031939656) * inp_4_2_2;
    result3 += M4(0.00689636, -0.06592742, -0.041052703, -0.009716337, -0.051975712, 0.20240934, 0.20009822, 0.05294845, 0.042626135, -0.16366443, 0.0021199046, -0.21061753, 0.018855028, 0.050442666, -0.11625233, 0.040373843) * inp_4_0_0;
    result3 += M4(-0.04472442, -0.004487701, 0.016733574, -0.12906088, 0.08611586, 0.07093734, -0.1900576, 0.19402565, 0.07777475, 0.21801785, -0.0126540065, -0.0036839242, 0.0052249013, -0.11809151, 0.089598805, -0.2237188) * inp_4_1_0;
    result3 += M4(0.03042609, 0.03774422, -0.00777721, 0.08703159, -0.03912593, -0.27345115, -0.28499025, -0.28595665, -0.15684505, 0.0153899025, 0.013607533, 0.124129385, 0.033163805, 0.018447246, 0.06706835, 0.09150493) * inp_4_2_0;
    result3 += M4(0.020877067, 0.090003, -0.025097083, 0.041065898, -0.08231007, -0.17585106, 0.3505885, -0.16120064, -0.27376097, 0.11192765, -0.37900883, 0.17624605, -0.09874509, -0.008290016, -0.17865613, -0.018174427) * inp_4_0_1;
    result3 += M4(-0.025545482, -0.0082803555, 0.0071168425, 0.02313114, -0.007641874, -0.023024334, 0.1303134, 0.19637902, 0.25162393, -0.17003688, 0.2716032, 0.021603633, 0.022172246, 0.02715026, 0.06967518, -0.005550302) * inp_4_1_1;
    result3 += M4(0.015960608, -0.08155264, -0.04603097, -0.14620313, 0.008179934, 0.21828046, 0.18969727, 0.4096675, 0.1239014, -0.012339784, -0.013975976, -0.09315124, -0.006627553, -0.040586747, -0.04617358, -0.11107131) * inp_4_2_1;
    result3 += M4(-0.04819641, -0.015906036, 0.13046873, -0.023875387, 0.12136973, -0.013941052, -0.5040011, 0.30526176, 0.2673789, 0.11529629, 0.255758, 0.06323085, 0.10831226, 0.022948686, 0.109913714, -0.021182634) * inp_4_0_2;
    result3 += M4(0.11876901, 0.0042022336, -0.064485125, 0.10173266, -0.121637724, -0.052259535, 0.12807633, -0.5382249, -0.47408524, -0.034083646, -0.19490685, 0.16368207, -0.05242081, 0.022125559, -0.022027154, 0.3128386) * inp_4_1_2;
    result3 += M4(-0.050161485, 0.04563198, 0.015278539, 0.058573812, 0.10111615, 0.058135573, -0.012207093, -0.13974948, 0.1367653, -0.065307036, 0.08792489, -0.22526331, -0.010383076, 0.04580226, -0.017262358, -0.0775268) * inp_4_2_2;
    result4 += M4(-0.06664678, -0.058673557, -0.0037921553, 0.0055883713, 0.07134976, -0.03193465, -0.13440442, 0.1817731, -0.01238214, 0.09936423, 0.05343181, -0.029669112, 0.017485436, -0.060649846, -0.038382187, -0.04068753) * inp_4_0_0;
    result4 += M4(-0.02353353, 0.15582371, 0.03902223, -0.049774386, 0.23614168, -0.19918306, 0.025636176, -0.20611614, 0.015420609, -0.03582778, -0.07945807, 0.1120217, -0.18546507, 0.11407637, 0.081859656, -0.018710202) * inp_4_1_0;
    result4 += M4(0.07254151, -0.083499506, -0.027918778, 0.0170676, -0.26859838, 0.2277576, 0.1400448, 0.058113236, 0.019912671, -0.019637544, -0.0037954713, -0.022017129, 0.134257, -0.03523558, -0.06391732, 0.022881858) * inp_4_2_0;
    result4 += M4(0.12080633, 0.04431205, -0.023627084, -0.028412005, -0.23556791, 0.033537492, 0.12908171, -0.10728827, -0.18836969, -0.14678976, 0.043183472, 0.2232377, -0.08500179, 0.05581575, 0.07220689, 0.11291759) * inp_4_0_1;
    result4 += M4(-0.059683695, -0.06499726, 0.024291296, 0.04810899, -0.04211044, 0.18283664, -0.1792283, 0.091497906, 0.15143704, -0.019825049, -0.079771556, -0.40267867, 0.14690915, -0.108874835, -0.12585607, -0.058232225) * inp_4_1_1;
    result4 += M4(-0.084626354, 0.01850995, 0.012066055, 0.014376771, 0.34916404, -0.15042691, -0.12229762, 0.053242616, -0.055003416, 0.058650292, 0.054950513, 0.10675843, -0.0952377, -0.010548456, 0.024346296, -0.048677683) * inp_4_2_1;
    result4 += M4(-0.048054297, -0.027967058, 0.029758312, -0.01020181, 0.2273826, 0.017365923, -0.12562408, -0.03584834, 0.3596261, -0.044629604, -0.12670773, -0.12582493, 0.11588963, 0.020746905, -0.022872882, -0.08127256) * inp_4_0_2;
    result4 += M4(0.10285533, -0.050449464, -0.08059687, -0.018262539, -0.37848926, 0.05011097, 0.17691205, 0.060306415, -0.19533573, 0.047506206, 0.102088764, 0.30813453, 0.026339674, -0.0016468533, 0.038869604, 0.09839562) * inp_4_1_2;
    result4 += M4(-0.0039321086, 0.07007668, 0.018508978, 0.017929057, 0.038867187, -0.13855365, 0.06996432, -0.0759373, -0.073284835, 0.0497792, 0.041705273, -0.09611099, -0.051891528, 0.03357219, 0.019999519, 0.051030405) * inp_4_2_2;
    result5 += M4(-0.032408573, 0.012954628, 0.02805172, 0.017654577, -0.077862166, 0.020497479, 0.08811784, 0.1547115, 0.08722207, 0.002758074, -0.15339355, -0.18418461, 0.046318557, -0.12194522, -0.017648976, -0.059472278) * inp_4_0_0;
    result5 += M4(0.012389813, -0.03284494, 0.033012494, 0.008930971, 0.0633571, 0.2931675, -0.13227788, -0.17137961, -0.046189994, -0.10552402, 0.08621566, 0.248859, -0.0035248278, 0.014285688, 0.06755755, 0.0412718) * inp_4_1_0;
    result5 += M4(0.022466315, -0.006947131, -0.04173053, 0.0058571477, 0.022195753, -0.42795184, 0.13301077, -0.00095974427, -0.031946164, 0.041609656, 0.012378063, -0.05885808, -0.013652105, 0.0705783, -0.0803422, 0.049014073) * inp_4_2_0;
    result5 += M4(0.008292182, -0.06593299, 0.0521454, -0.038637158, 0.057476018, 0.1555544, -0.037748806, -0.20558971, -0.0629633, 0.05627316, 0.043432165, 0.23756625, 0.017828269, 0.05402221, 0.17199306, 0.13124616) * inp_4_0_1;
    result5 += M4(-0.06247638, 0.031304322, 0.020051636, -0.041149903, 0.088406354, -0.17918494, -0.087703876, 0.35611904, 0.07737568, -0.2671464, 0.009129435, -0.28186294, -0.063662745, -0.19564901, 0.0019323882, -0.1755132) * inp_4_1_1;
    result5 += M4(0.026853036, -0.0017256155, -0.064108245, 0.043497093, -0.16186962, 0.40737376, 0.01619492, -0.0803005, -0.03951629, 0.15757416, -0.01173799, 0.07666027, -0.0033937374, 0.11413928, -0.07214158, 0.053094186) * inp_4_2_1;
    result5 += M4(-0.014845912, 0.09145128, -0.013189406, 0.016878424, -0.01889123, -0.30262142, -0.062099613, 0.1180657, -0.049786843, 0.2394621, 0.10408037, -0.11570408, -0.098328784, 0.17763728, -0.11770009, -0.07197523) * inp_4_0_2;
    result5 += M4(0.04356602, -0.06766165, -0.03404063, 0.015514155, -0.084599465, 0.2771538, 0.22861621, -0.1771653, -0.04931129, 0.12447532, 0.06430635, 0.11646892, 0.09989204, 0.14184777, -0.0032600653, 0.11451333) * inp_4_1_2;
    result5 += M4(-0.031333815, 0.02691153, 0.03179691, -0.03681594, 0.11876874, -0.2902933, -0.1539389, 0.003044856, 0.07224995, -0.24746148, -0.12381532, -0.023332639, -0.002834141, -0.23495841, 0.099186644, -0.07046896) * inp_4_2_2;
    result6 += M4(-0.017381975, -0.016643519, 0.029608402, 0.03528924, 0.05874351, 0.09002394, -0.035488088, 0.16451742, -0.18746689, 0.10299388, -0.011406651, -0.51226586, 0.04543686, 0.042822186, 0.000295842, -0.06843712) * inp_4_0_0;
    result6 += M4(0.009983424, -0.03527882, 0.024533, -0.06468151, -0.024047863, 0.19659728, -0.086634725, -0.031633385, 0.15322533, -0.16689438, -0.028453592, 0.113357306, 0.006152431, -0.14647532, 0.0073171197, 0.16634215) * inp_4_1_0;
    result6 += M4(-0.010775996, 0.028932394, -0.0025803843, 0.0689663, 0.03530679, -0.19789095, 0.011146269, -0.108453825, 0.034026437, 0.05127482, 0.041197874, -0.13216826, -0.089602455, 0.08921823, -0.015401981, 0.035453536) * inp_4_2_0;
    result6 += M4(0.014761601, 0.046144586, 0.013680943, 0.006399192, -0.10066242, -0.14818358, 0.07106009, -0.30221668, -0.24348715, -0.12907337, 0.07491813, 0.6244094, -0.18652225, -0.030943928, -0.016550684, 0.16219) * inp_4_0_1;
    result6 += M4(-0.06004034, -0.0060974485, -0.008659716, 0.099143595, 0.11651081, -0.30038348, 0.14252192, -0.020759894, -0.050597955, 0.058864526, -0.16222227, 0.013225708, -0.07031452, 0.14987804, -0.024798501, -0.110472925) * inp_4_1_1;
    result6 += M4(0.07951301, -0.002611832, -0.009486018, -0.2290084, -0.28376663, 0.35758618, -0.18736544, 0.32791102, 0.03870602, 0.050560582, 0.078329064, -0.2609141, 0.1356595, -0.12203838, 0.07038371, -0.19529137) * inp_4_2_1;
    result6 += M4(-0.03756107, 0.008046251, -0.04311789, -0.013210866, 0.13418002, 0.035431303, -0.055685025, -0.0102119045, 0.18969075, 0.04136428, 0.0422434, -0.4511317, 0.10800096, -0.034511868, 0.052010342, -0.24429856) * inp_4_0_2;
    result6 += M4(0.106922455, 0.010550341, -0.009889312, -0.04788766, -0.08397241, 0.15923834, -0.05933829, -0.052289102, 0.0038458912, 0.032122675, 0.04409494, 0.24473894, 0.10753126, -0.00064609095, -0.0006487492, 0.13516244) * inp_4_1_2;
    result6 += M4(-0.07121884, -0.010555482, 0.005400417, 0.12147718, 0.15748353, -0.18252921, 0.19373058, 0.06405511, 0.006395063, -0.02287764, -0.110980526, 0.30161607, -0.07047891, 0.06831382, -0.09526535, -0.0038954138) * inp_4_2_2;
    result7 += M4(0.0009032464, 0.0029969981, -0.05448892, -0.08096257, 0.08120788, -0.62858194, 0.3007573, 0.18367368, -0.027446346, 0.27442467, 0.05929318, 0.18540269, -0.042006385, 0.068010256, 0.059504505, 0.041720208) * inp_4_0_0;
    result7 += M4(-0.022383658, -0.03749509, 0.062281635, 0.12575771, -0.13315243, 0.36709714, -0.14643554, -0.2886493, -0.042639893, -0.16579126, 0.27630323, -0.024335427, 0.008491147, -0.10899782, 0.17456055, 0.16078854) * inp_4_1_0;
    result7 += M4(0.050348163, 0.023706246, -0.037683338, -0.020394275, 0.01459269, 0.14785977, -0.004192197, 0.062529646, 0.03441731, -0.12927999, -0.5366105, -0.08034771, 0.033657722, -0.09540858, -0.29651552, -0.08406721) * inp_4_2_0;
    result7 += M4(0.012283714, -0.008900109, 0.0023326124, 0.15582386, 0.09291412, 0.32990944, -0.056092158, -0.14680338, -0.14414513, 0.19548154, 0.1685804, -0.41662458, 0.00024180181, 0.23974599, 0.045214918, -0.24196567) * inp_4_0_1;
    result7 += M4(0.021945974, 0.024469053, 0.045654055, -0.013398447, 0.219957, 0.23570424, 0.040420733, -0.0023932762, 0.18203492, 0.010891317, -0.25348008, 0.41283295, -0.087481566, -0.13644253, -0.12811778, 0.048324667) * inp_4_1_1;
    result7 += M4(-0.058789812, 0.05965699, -0.03299138, -0.07993852, -0.28058347, -0.4435297, 0.003838362, 0.06755837, 0.02889362, 0.17195174, 0.06594751, 0.05009216, 0.047433488, 0.12504584, 0.16640961, 0.19565178) * inp_4_2_1;
    result7 += M4(-0.0372772, -0.08514108, 0.11124839, -0.0518807, -0.17101307, 0.19951442, -0.44460645, -0.1475136, 0.2771417, -0.5698664, -0.114025615, -0.2692116, 0.08948316, -0.3444112, 0.04047168, 0.03022153) * inp_4_0_2;
    result7 += M4(0.024777096, 0.018237282, -0.14429744, -0.09194565, -0.19629994, -0.41739547, 0.30329433, 0.3753069, -0.2631469, 0.2913003, -0.037803557, -0.022423023, 0.048471864, 0.15722021, -0.13102622, -0.040778607) * inp_4_1_2;
    result7 += M4(0.009145101, -0.014251083, 0.020218063, 0.07430466, 0.3554753, 0.1851563, 0.04149165, -0.1446305, -0.06289442, -0.10637244, 0.31844443, 0.1759244, -0.07932942, 0.06745871, -0.0066716247, -0.08626381) * inp_4_2_2;
    const V4 inp_5_0_0 = inp[5][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_5_1_0 = inp[5][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_5_2_0 = inp[5][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_5_0_1 = inp[5][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_5_1_1 = inp[5][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_5_2_1 = inp[5][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_5_0_2 = inp[5][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_5_1_2 = inp[5][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_5_2_2 = inp[5][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.14487545, -0.17160423, 0.048705466, 0.15721305, -0.05378785, 0.017684493, -0.02190003, 0.11726614, 0.004134306, 0.05513694, 0.07180701, -0.06336592, -0.040828884, -0.18765047, 0.15194973, -0.017511008) * inp_5_0_0;
    result0 += M4(0.067308344, 0.02003724, -0.06649579, 0.035189856, 0.005282479, -0.067472, -0.0126704285, -0.11287401, 0.004359737, -0.015151686, -0.030349016, 0.041064057, 0.16215737, -0.097196415, -0.03104119, -0.06606812) * inp_5_1_0;
    result0 += M4(-0.014337858, 0.008460683, 0.05668169, -0.13078801, 0.046509467, 0.03304736, 0.10695132, 0.033939004, 0.030890485, -0.030266667, -0.016283402, 0.057600953, 0.078502804, 0.013397677, 0.059509687, -0.09201839) * inp_5_2_0;
    result0 += M4(-0.017683703, 0.048911348, -0.023500051, -0.049236607, 0.1460796, -0.3526494, -0.0018817846, -0.0106862625, -0.06937126, 0.13826647, -0.09382714, -0.013432277, -0.23616734, 0.568435, -0.10191458, -0.11383223) * inp_5_0_1;
    result0 += M4(-0.032154623, 0.009587408, 0.14319599, -0.091824204, -0.07755498, 0.25645047, 0.010568613, 0.19811888, -0.0095190015, -0.15942399, -0.054214984, -0.07848526, 0.12143066, -0.17331904, 0.089831844, 0.29042396) * inp_5_1_1;
    result0 += M4(0.003561061, -0.07390303, -0.11331616, 0.12686175, -0.059063133, 0.0074740974, -0.056097236, -0.15084764, 0.011903642, 0.099676274, 0.097970955, -0.04355082, 0.009051899, 0.048929498, -0.009395171, -0.042226423) * inp_5_2_1;
    result0 += M4(0.020696942, 0.017849, -0.01040823, -0.06707944, -0.07046718, 0.24248765, 0.003361004, -0.059914164, 0.123420045, -0.19104978, 0.06464406, 0.10294659, 0.36242774, -0.32025006, 0.029345559, 0.09704417) * inp_5_0_2;
    result0 += M4(0.10302498, 0.018385662, -0.051784184, 0.086695544, 0.0988703, -0.14876729, 0.08207358, -0.1013353, -0.074303806, 0.17050105, 0.011589173, 0.0094479, -0.2684949, 0.11264198, -0.16982287, 0.059152782) * inp_5_1_2;
    result0 += M4(0.008785546, 0.09954225, 0.055380106, -0.055245094, -0.012930977, -0.016047405, -0.072647355, 0.06654659, -0.02382223, -0.057216864, -0.04572952, 0.0041757305, -0.12401904, 0.0048728585, 0.051346675, -0.07074739) * inp_5_2_2;
    result1 += M4(0.024792833, -0.14112833, 0.01973008, 0.05649872, -0.09660786, -0.09791372, 0.16524757, -0.07094173, -0.02482972, 0.055603668, 0.056332596, -0.031861443, 0.05179827, 0.10614248, -0.13116306, 0.1061169) * inp_5_0_0;
    result1 += M4(0.20434876, 0.09196387, -0.053179983, 0.08178851, 0.13500853, 0.099453546, -0.013747525, 0.07420215, -0.016611539, -0.1313599, -0.07534334, 0.032047402, 0.34721512, 0.18283838, 0.09662092, -0.00952582) * inp_5_1_0;
    result1 += M4(-0.19826926, 0.0014808781, 0.0865352, -0.11310373, 0.02889297, -0.030729467, -0.09680466, 0.031107124, 0.046216175, 0.07704591, 0.061058756, 0.008875887, -0.5202826, -0.0712744, -0.03841371, 0.08051542) * inp_5_2_0;
    result1 += M4(-0.1041009, 0.022262434, 0.0012561168, -0.1266561, -0.040280394, 0.08349965, -0.109505124, 0.07990964, 0.07589681, 0.01248841, -0.10717935, -0.07095789, -0.03584616, -0.065722555, -0.08894601, 0.044809826) * inp_5_0_1;
    result1 += M4(-0.04080169, 0.010225272, -0.11265153, 0.022797741, 0.12382152, -0.17424534, 0.059866156, -0.08384392, -0.11471599, 0.11438692, 0.014547949, -0.038466904, -0.16445488, -0.17643425, -0.07987047, -0.07944458) * inp_5_1_1;
    result1 += M4(0.025758298, 0.0022748024, -0.0794111, 0.114587724, -0.107433125, 0.048688333, -0.014495751, -0.07876867, -0.008468299, -0.12687962, 0.025861423, 0.106900424, 0.30469888, 0.120268725, 0.164976, -0.2532854) * inp_5_2_1;
    result1 += M4(0.014663514, -0.07491252, -0.017437093, -0.013965135, 0.040172577, -0.092287295, -0.010011614, 0.00018377477, -0.019634228, 0.035583768, -0.047923338, 0.1171189, 0.06922562, 0.043790404, 0.14833125, -0.1029593) * inp_5_0_2;
    result1 += M4(0.049796063, 0.12166938, 0.20878176, -0.012695056, -0.12240116, 0.19088581, -0.048469115, 0.030847434, 0.05961474, -0.04267812, 0.09861263, 0.04145584, -0.07089184, 0.17485015, -0.01803056, 0.06052556) * inp_5_1_2;
    result1 += M4(-0.02826202, 0.013290304, -0.070377104, -0.056082033, 0.043038618, -0.049613643, 0.03518067, 0.030806743, 0.007097693, 0.069611765, -0.032252822, -0.14488894, 0.08379807, -0.27310658, -0.07973521, 0.15545034) * inp_5_2_2;
    result2 += M4(0.004777358, -0.011930929, -0.021969482, 0.11056751, 0.052678745, -0.03322556, 0.025181968, -0.013370279, 0.14642441, 0.056453675, 0.09516531, -0.23694779, 0.002743051, -0.051188797, -0.23459984, 0.44212595) * inp_5_0_0;
    result2 += M4(-0.027820686, 0.028288847, 0.016262447, 0.041180767, -0.150635, -0.06323988, 0.11602517, -0.026515527, -0.060948994, -0.08299206, -0.17442437, 0.2397756, -0.1210503, 0.03277117, 0.06213776, -0.11659927) * inp_5_1_0;
    result2 += M4(-0.093777925, 0.021194287, 0.016661383, -0.11003224, 0.0056465184, 0.092856765, -0.012685888, -0.0020968856, -0.057644635, 0.014875682, -0.0077965073, -0.03592805, -0.11158727, -0.13625361, -0.11380065, 0.0017759353) * inp_5_2_0;
    result2 += M4(-0.025954146, 0.0013461969, -0.024648406, -0.09438298, -0.12145677, -0.12384006, -0.04772585, 0.07505571, -0.06890466, 0.067839205, -0.0678513, 0.014559, 0.17236426, 0.36397234, -0.09757843, -0.74017406) * inp_5_0_1;
    result2 += M4(0.03135827, -0.18783487, -0.0755456, 0.0657101, 0.34007713, 0.048000973, -0.15294373, 0.2177495, 0.0016295357, 0.08896897, 0.21745461, -0.21635062, -0.23017402, -0.31567594, -0.0044142194, 0.2403915) * inp_5_1_1;
    result2 += M4(0.15281585, 0.069029436, 0.15151843, -0.00618539, 0.006150375, 0.0014913725, 0.115502775, -0.1471148, 0.041680045, -0.09219337, -0.12446711, 0.23101422, 0.3149102, 0.1458362, 0.4698781, -0.062021475) * inp_5_2_1;
    result2 += M4(0.01975604, 0.01694945, 0.06833457, -0.005176524, 0.015928695, 0.11531035, 0.06344842, -0.015509292, -0.052742537, -0.13220845, 0.001933081, 0.14270711, 0.03078088, -0.24105065, 0.056160636, 0.5343719) * inp_5_0_2;
    result2 += M4(0.035794254, 0.08032635, -0.13032255, 0.0016536227, -0.075815074, -0.04796818, -0.1044732, -0.14409123, 0.10306038, 0.05937788, -0.15723316, -0.035225943, -0.0097134635, -0.023150293, -0.06304843, -0.019593531) * inp_5_1_2;
    result2 += M4(-0.05768308, -0.027078262, -0.0137278, 0.014670163, -0.0391974, 0.0013185189, -0.04720961, 0.08347625, -0.053424258, 0.01722233, 0.16490285, -0.08758822, 0.042712722, 0.27212936, -0.13239712, -0.22161157) * inp_5_2_2;
    result3 += M4(0.076304205, 0.0936, -0.04559145, -0.0024564764, -0.02671183, -0.16786881, -0.08044892, -0.19581306, 0.044225387, 0.0070009273, -0.10976952, 0.12924004, -0.037671037, 0.27487656, -0.0071782353, 0.1373205) * inp_5_0_0;
    result3 += M4(-0.020301407, 0.0003170418, 0.116233304, -0.042301804, 0.020694489, 0.20109649, -0.034801286, 0.15563256, -0.103959106, -0.056824304, 0.08460481, -0.10521214, 0.05246504, -0.17544676, -0.08866767, 0.20484477) * inp_5_1_0;
    result3 += M4(0.083369434, -0.056057252, -0.11516279, -0.11892053, -0.039910987, 0.0022059842, -0.05818419, 0.02507813, 0.056811996, 0.050061524, 0.06159384, -0.019531107, 0.06176774, -0.23392676, -0.15343434, -0.21678378) * inp_5_2_0;
    result3 += M4(-0.038426943, -0.029610377, -0.12385609, 0.12560731, 0.014580414, 0.09714395, -0.12906677, 0.11847, 0.102602564, 0.020921243, 0.03153712, 0.04516741, 0.13502969, -0.2280457, 0.44887325, -0.08459545) * inp_5_0_1;
    result3 += M4(-0.1161786, 0.0072595915, -0.039852273, 0.14671211, 0.17039524, -0.13803089, 0.26739827, 0.058654934, -0.13802612, 0.025848448, -0.19562443, -0.054355077, -0.33750972, 0.13451582, 0.19718222, 0.005318955) * inp_5_1_1;
    result3 += M4(-0.057749007, -0.017158277, -0.06759651, 0.00014074019, -0.099232174, 0.010897674, 0.12388254, 0.07017674, 0.05218598, -0.06912867, 0.035562, -0.014072939, -0.18798032, 0.19557178, 0.30724457, 0.40325192) * inp_5_2_1;
    result3 += M4(0.0626181, -0.027484976, 0.070837736, -0.07721638, -0.009539474, 0.08588088, 0.09169392, 0.16773517, -0.14678282, -0.060641993, 0.15514751, -0.21313979, -0.11056434, 0.06923274, -0.42457572, 0.020781307) * inp_5_0_2;
    result3 += M4(0.09048194, 0.024306875, 0.10712681, 0.16379644, -0.17848895, 0.021104747, -0.1544449, -0.27291206, 0.2996105, 0.015106289, 0.04193118, 0.13862494, 0.43881562, -0.04598897, 0.07423087, -0.35241327) * inp_5_1_2;
    result3 += M4(-0.038618684, 0.015168471, 0.038445685, -0.23600565, 0.14561793, -0.0998489, -0.047032118, -0.15962356, -0.19389665, 0.033053, -0.0862041, 0.10979863, 0.04790202, 0.02110294, -0.38260517, -0.11872257) * inp_5_2_2;
    result4 += M4(0.2038064, -0.22027019, 0.011519498, 0.0315649, -0.012706539, 0.12307168, 0.13686505, -0.055130284, 0.10214224, -0.02560099, -0.039740276, 0.024086332, 0.11414169, -0.108951814, -0.09055209, 0.13220549) * inp_5_0_0;
    result4 += M4(-0.22233202, 0.20623437, -0.097258955, -0.076147065, 0.26636714, -0.30390558, -0.24985388, 0.13784525, -0.11266426, 0.117381014, 0.0078284815, 0.025657138, 0.06821738, -0.15374, 0.023126934, -0.31432924) * inp_5_1_0;
    result4 += M4(0.11710884, 0.00017204539, -0.028754793, 0.0005588163, -0.18340276, 0.1650608, 0.10140369, -0.079275414, 0.085426815, -0.1422715, 0.018631997, 0.037399445, -0.09663513, 0.204864, 0.04344605, 0.07365369) * inp_5_2_0;
    result4 += M4(-0.027247507, 0.08690705, 0.017199215, 0.016632559, -0.084304094, -0.18416008, -0.028605336, 0.03685625, 0.09964805, -0.02680286, -0.0982869, -0.026104664, 0.08568218, 0.15346986, -0.031727802, -0.32782882) * inp_5_0_1;
    result4 += M4(-0.019776283, -0.093729414, 0.051771306, 0.09271549, -0.14045975, 0.28396788, 0.00499183, -0.20550175, -0.074020766, -0.05064825, 0.17142008, 0.12983744, -0.2570548, 0.20873146, -0.027318837, 0.5142755) * inp_5_1_1;
    result4 += M4(-0.0044934945, 0.076453224, -0.004864276, -0.03269062, 0.16559559, -0.10965849, -0.008354805, 0.108064145, -0.095865645, 0.096873745, 0.0042315656, -0.043925557, 0.35319585, -0.14348285, -0.20933945, 0.09885607) * inp_5_2_1;
    result4 += M4(0.052431464, -0.0028832727, 0.0003948411, -0.10554142, 0.14867292, 0.03184341, -0.07424044, -0.027859958, -0.29334727, 0.075082615, 0.1743756, 0.052498072, -0.32764167, 0.020022253, 0.06310184, 0.07649812) * inp_5_0_2;
    result4 += M4(-0.007696482, -0.056597758, -0.006384618, 0.096683405, -0.09336502, -0.01775242, 0.1420388, 0.067856826, 0.2811956, -0.018419646, -0.20785987, -0.117266014, 0.12885432, -0.12208167, -0.04485354, -0.26984853) * inp_5_1_2;
    result4 += M4(-0.10329611, 0.006612655, 0.04273599, 0.12975451, -0.08357311, -0.009403975, -0.03525734, 0.0012920485, 0.0014952259, -0.0018721912, -0.021300223, 0.019337686, -0.102081195, -0.044837065, 0.21767052, -0.12613463) * inp_5_2_2;
    result5 += M4(-0.0013765241, 0.055979736, 0.051297892, 0.034202304, 0.018769877, 0.14280763, -0.020301541, -0.16308391, -0.02004487, 0.045876544, 0.051553134, -0.009177164, -0.12243481, 0.12688288, 0.23991692, 0.2638696) * inp_5_0_0;
    result5 += M4(0.034714766, -0.100535035, -0.036328405, 0.060714815, -0.050259355, -0.13832022, -0.083592325, 0.18844905, -0.021835985, -0.09454935, -0.04479631, -0.03590102, 0.018836964, 0.21339712, -0.20532079, -0.12552153) * inp_5_1_0;
    result5 += M4(0.06389367, 0.007051279, -0.10132785, 0.009419095, 0.068815485, 0.034409296, -0.02378293, 0.0003968401, 0.0659766, 0.0983309, -0.0109843705, 0.02009493, 0.13747329, -0.30761394, 0.105394274, -0.0899214) * inp_5_2_0;
    result5 += M4(-0.012742045, -0.035827678, 0.18153493, -0.0077017094, -0.041391525, -0.1260696, 0.043319944, 0.15263295, -0.027294533, -0.09653262, -0.15680853, 0.02742822, 0.072620094, -0.1694349, 0.08835404, -0.20460767) * inp_5_0_1;
    result5 += M4(-0.010477131, -0.06913824, -0.05747798, -0.06790096, 0.14511408, -0.0839045, -0.0077786567, -0.058972, -0.059925873, 0.26318273, 0.16517878, -0.0020129182, -0.084907785, 0.29658177, -0.15582411, 0.30340704) * inp_5_1_1;
    result5 += M4(-0.12467282, 0.079009466, -0.04265613, -0.081022866, -0.10454997, -0.0037390082, 0.05824569, -0.04201197, 0.08005202, -0.2729313, -0.0365042, -0.020996293, -0.090580836, 0.17121324, 0.028330287, -0.12846133) * inp_5_2_1;
    result5 += M4(0.0035657387, -0.04349601, -0.08246734, -0.024217378, 0.027633833, 0.34061936, -0.012338658, 0.0057765166, 0.021727802, -0.090277836, 0.17104092, -0.010518078, 0.056020208, -0.5343775, -0.07831186, 0.24692102) * inp_5_0_2;
    result5 += M4(-0.0120178135, 0.13129312, 0.13369536, 0.024199694, -0.100464046, -0.17850943, 0.22656769, -0.035404127, 0.09965665, -0.16581202, -0.1870673, 0.012786002, 0.09907103, 0.051715292, 0.11104672, -0.24043939) * inp_5_1_2;
    result5 += M4(0.03364613, -0.01634712, -0.000108216234, 0.06563137, 0.06966388, -0.014031864, -0.15349235, -0.016074467, -0.11105, 0.28994662, 0.04000499, -0.016755728, -0.05231203, 0.116528414, -0.13706039, -0.025074242) * inp_5_2_2;
    result6 += M4(0.1221555, -0.035549484, -0.05539886, -0.14673644, 0.024920339, 0.0978014, -0.04907462, -0.15223008, -0.015605834, -0.00044811593, 0.13724218, -0.08030791, 0.41332978, -0.15260375, -0.076417334, 0.4542322) * inp_5_0_0;
    result6 += M4(-0.15790986, -0.048651226, -0.021013353, 0.16067572, -0.11097924, -0.042684693, -0.02185444, -0.10382293, -0.07571375, -0.09554541, -0.08703409, -0.027820094, -0.32593006, 0.30781785, -0.10018425, 0.05851567) * inp_5_1_0;
    result6 += M4(0.33601975, 0.07126457, 0.008180276, -0.12814368, 0.27809057, -0.05421029, 0.07694183, -0.08146193, 0.045660537, 0.069777265, -0.06771174, 0.16664776, 0.19526222, -0.16831273, 0.063676156, -0.057284635) * inp_5_2_0;
    result6 += M4(-0.13333909, 0.058477517, 0.05496978, 0.20944867, -0.12330191, -0.07275876, 0.1571845, 0.0895397, 0.05623866, 0.026890658, -0.06823, -0.05690843, 0.034209758, 0.18102919, -0.016447954, -0.66030276) * inp_5_0_1;
    result6 += M4(-0.08854254, -0.01793967, 0.038640924, 0.0064204605, 0.0015581348, -0.094273716, -0.08289648, 0.03392984, 0.08943608, 0.21944368, 0.03204067, 0.1104926, 0.22618754, -0.2594543, 0.28945115, -0.24935365) * inp_5_1_1;
    result6 += M4(-0.0844054, -0.051376175, -0.002673134, 0.020121487, -0.29825005, 0.13381165, -0.13974707, 0.095656045, 0.06043174, -0.16790132, 0.030074341, -0.06548197, -0.43295035, 0.2106437, -0.18050858, 0.6547746) * inp_5_2_1;
    result6 += M4(0.016956909, -0.011201033, -0.0046606213, -0.17229676, -0.015481639, 0.055481028, -0.0770852, -0.03155328, 0.01147108, 0.034666333, -0.02514347, 0.19892937, -0.04279175, 0.0073598363, -0.070535585, 0.4786945) * inp_5_0_2;
    result6 += M4(0.05174959, 0.04334956, 0.006304897, 0.059605777, 0.11830391, 0.041454658, 0.014777315, 0.03770415, -0.053028017, -0.17914602, 0.08211249, -0.19339016, -0.19213587, 0.020936515, -0.106671594, -0.3857754) * inp_5_1_2;
    result6 += M4(-0.025799569, 0.03235182, -0.0567284, -0.13193233, 0.14405708, -0.02438717, 0.07856137, 0.0430177, -0.11181154, 0.086171664, -0.009648999, 0.04567537, 0.21670514, -0.12696089, 0.2108189, -0.25610822) * inp_5_2_2;
    result7 += M4(0.010523993, -0.14366621, 0.085187376, -0.03796926, -0.03944769, -0.016998656, -0.02019955, 0.038467415, 0.00108887, 0.25764173, -0.18759021, -0.28159183, 0.054111756, -0.80722815, 0.24703035, 0.084264204) * inp_5_0_0;
    result7 += M4(-0.029389344, -0.08524073, 0.010677293, 0.16257846, -0.046593286, 0.03811754, -0.031969137, -0.029502267, -0.057995144, -0.034166075, 0.103917584, 0.13290235, -0.009255505, 0.28440636, -0.37132233, -0.065270156) * inp_5_1_0;
    result7 += M4(0.017886313, -0.13355653, -0.11892869, -0.05966338, 0.021549799, -0.008833239, 0.027631847, -0.016555903, 0.019678697, -0.04510477, 0.16712557, 0.026769245, 0.02833975, 0.18940929, 0.6111511, 0.0011692144) * inp_5_2_0;
    result7 += M4(0.07826066, 0.11714984, 0.007125341, -0.18402001, -0.07773169, 0.021707691, 0.08906625, -0.23501052, 0.081419215, -0.2707353, -0.11111728, 0.28695914, 0.2105243, 0.05975272, -0.13236572, 0.19848627) * inp_5_0_1;
    result7 += M4(0.041938618, 0.19984642, -0.12640493, 0.12735468, 0.17216486, 0.24686831, -0.09775958, 0.21539694, -0.1626948, -0.11084832, 0.091525055, -0.11382038, -0.023980213, 0.2902177, 0.18833382, -0.2651941) * inp_5_1_1;
    result7 += M4(0.014123405, 0.04643737, -0.02268234, 0.047757752, -0.013483261, -0.022759346, -0.058642365, 0.055256274, 0.08352807, 0.19713457, 0.04800165, -0.057878982, -0.2220578, -0.40779552, -0.18597387, -0.08666717) * inp_5_2_1;
    result7 += M4(-0.049710922, -0.0051879194, 0.06872514, -0.033438046, 0.13250633, -0.16988069, -0.04527019, -0.19362688, -0.116373196, 0.129323, 0.22683124, 0.13147761, -0.36144406, 0.63469833, -0.36487445, 0.06620963) * inp_5_0_2;
    result7 += M4(-0.009373005, 0.028044108, -0.0029026482, -0.08569764, -0.21816318, 0.06340744, 0.039358962, 0.06509732, 0.256029, 0.103783, -0.19389144, -0.20077503, -0.030296791, -0.34552154, 0.20733383, 0.24426457) * inp_5_1_2;
    result7 += M4(-0.07619316, -0.06628436, 0.013949042, 0.06730773, 0.038475007, -0.16576064, 0.07489411, 0.044181406, -0.119928606, -0.17178929, -0.07375216, 0.032672796, 0.3799344, 0.04634624, -0.14644834, -0.16204233) * inp_5_2_2;
    const V4 inp_6_0_0 = inp[6][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_6_1_0 = inp[6][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_6_2_0 = inp[6][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_6_0_1 = inp[6][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_6_1_1 = inp[6][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_6_2_1 = inp[6][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_6_0_2 = inp[6][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_6_1_2 = inp[6][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_6_2_2 = inp[6][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.036944494, -0.19967565, -0.12001727, 0.18755509, 0.012283716, 0.0026802325, 0.05197018, 0.097964145, -0.04225147, 0.18273093, -0.012834506, 0.014163178, 0.035176214, 0.11994631, 0.017261164, 0.014343045) * inp_6_0_0;
    result0 += M4(-0.03442685, -0.25631142, 0.03751219, -0.0007748609, -0.037585933, -0.06064166, 0.0067990026, -0.05947704, 0.009768935, 0.07521854, -0.038831633, 0.13656816, -0.056649376, -0.06887119, -0.19831248, 0.12040859) * inp_6_1_0;
    result0 += M4(-0.017041838, -0.029653858, 0.09310322, 0.08960219, 0.083052136, 0.033318985, 0.0064633414, -0.0264089, 0.0003509314, 0.0013177029, -0.03701681, 0.043567438, 0.08935713, 0.24886508, -0.025563736, -0.12677518) * inp_6_2_0;
    result0 += M4(0.14332269, -0.17627057, 0.041917488, -0.013317859, -0.02980862, 0.060990468, -0.015723059, -0.03127694, -0.053970125, 0.9797291, -0.17406666, -0.054347727, 0.05722515, 0.5294436, -0.19235669, -0.024199976) * inp_6_0_1;
    result0 += M4(-0.23099944, 0.041554462, 0.051770974, 0.05817821, 0.11932664, -0.029364482, -0.0172675, 0.05798414, 0.23458982, 0.067755066, 0.111495644, -0.31750745, 0.16773188, -0.26985756, 0.010626947, -0.13001058) * inp_6_1_1;
    result0 += M4(-0.0060689426, 0.07215701, -0.14998439, -0.18093142, -0.10592144, 0.015663575, -0.025597947, -0.022432998, 0.075124905, -0.102352716, 0.1059245, -0.18086852, 0.22230577, -0.094194956, -0.048197832, -0.01896015) * inp_6_2_1;
    result0 += M4(-0.23972215, 0.32593971, -0.08453366, -0.16481547, 0.044954065, -0.061207864, -0.054811016, -0.048041422, -0.111792535, 0.30740654, -0.03436402, -0.017449314, 0.08982878, 0.22826064, -0.0064858026, -0.11877089) * inp_6_0_2;
    result0 += M4(0.28884333, 0.07123941, 0.10732813, 0.07659588, -0.07099713, 0.008934912, -0.014415939, -0.0053134174, -0.6899811, 0.059372623, -0.15703402, -0.010475641, -0.111633904, -0.08963046, -0.13375798, -0.13950007) * inp_6_1_2;
    result0 += M4(0.03280046, 0.0691943, 0.045302697, -0.08485902, 0.017612794, 0.032371394, 0.074324004, 0.031483464, -0.352441, 0.039946683, 0.042882297, 0.07428785, 0.1600771, 0.080561765, -0.018895725, -0.09731285) * inp_6_2_2;
    result1 += M4(-0.06755739, -0.3000057, 0.1629116, 0.0868202, 0.04978717, 0.110192105, 0.033183195, 0.03208086, -0.017902263, -0.060255233, 0.007944979, 0.0066171545, -0.07905314, -0.21990968, 0.06927349, -0.03005389) * inp_6_0_0;
    result1 += M4(0.6698121, 0.58605736, 0.0816335, 0.26488054, 0.13990046, -0.09680338, 0.07285284, -0.07775494, -0.040934406, -0.094903655, -0.13958222, -0.007893427, -0.43528366, -0.01724251, -0.27025393, -0.025209103) * inp_6_1_0;
    result1 += M4(-0.10956677, -0.007972244, 0.057578195, -0.073627405, -0.16560787, 0.018366426, -0.12815717, 0.095519915, -0.36463243, -0.098777995, -0.1008887, -0.048599232, -0.32415262, 0.042110026, -0.0025722499, -0.12682225) * inp_6_2_0;
    result1 += M4(-0.078488834, -0.07636497, 0.0712175, 0.016768921, -0.11431465, -0.0059475293, -0.12105518, 0.0696205, -0.014103178, -0.21195593, 0.11872663, 0.12401621, 0.046319414, -0.2594622, 0.13530515, -0.05748265) * inp_6_0_1;
    result1 += M4(-0.1907787, -0.30604714, -0.17384696, -0.21829186, -0.111568846, -0.014824526, -0.079271294, 0.03555191, -0.30757508, -0.37242544, 0.30714467, 0.06004347, -0.21969593, -0.11729039, 0.061018914, 0.47859368) * inp_6_1_1;
    result1 += M4(-0.15224925, 0.04783557, -0.10310224, 0.06956383, 0.19912335, 0.012649738, 0.1930727, -0.13972847, -0.06042285, -0.116216145, -0.07322704, -0.31083706, 0.17845431, 0.030004304, -0.052400578, 0.14698932) * inp_6_2_1;
    result1 += M4(-0.124957085, -0.05666172, -0.09169483, -0.020408398, 0.09127924, -0.12869686, 0.059884805, -0.10026958, 0.028528625, 0.07582843, 0.071152896, 0.12621015, 0.031251617, -0.3362342, -0.16752551, -0.018972984) * inp_6_0_2;
    result1 += M4(-0.030894263, -0.028383462, 0.029151753, -0.08793264, 0.011751002, 0.08983085, 0.014432928, 0.0010462516, -0.06048285, 0.09799519, 0.370779, -0.5626547, -0.07311138, -0.07475939, 0.044188745, 0.21662761) * inp_6_1_2;
    result1 += M4(0.09939996, -0.06470144, 0.009426381, 0.008618198, -0.05574352, -0.005230792, -0.058350265, 0.10953379, 0.021337783, 0.06349101, 0.013979412, -0.48258153, 0.05597289, 0.14336504, -0.11833139, 0.051307883) * inp_6_2_2;
    result2 += M4(-0.2802379, 0.06575186, -0.20692001, 0.24305286, 0.007503178, -0.020474026, -0.023636993, 0.058526684, -0.04278353, 0.119098, -0.021353267, -0.016311938, 0.29508734, 0.099193, 0.0194962, -0.47862372) * inp_6_0_0;
    result2 += M4(-0.2453934, -0.124028884, 0.17021784, 0.0371019, -0.04139, -0.07275396, 0.0006170883, -0.19566078, -0.145592, 0.013031027, 0.083287045, 0.1581235, 0.012941595, 0.06748677, -0.20285995, 0.006192288) * inp_6_1_0;
    result2 += M4(-0.00022764594, 0.048846927, 0.016910965, -0.45185295, 0.05652763, 0.03185575, 0.037604194, 0.15050724, 0.0838502, -0.026923561, 0.12149843, -0.104825675, 0.0074304235, 0.07510158, 0.098061025, -0.107196376) * inp_6_2_0;
    result2 += M4(0.35337654, -0.059460297, -0.07764672, -0.12206768, 0.057254728, 0.03432741, 0.025120348, -0.122658744, 0.06155359, 0.30082572, 0.25814968, -0.15654345, 0.17023157, -0.03589007, 0.0082202675, -0.02978125) * inp_6_0_1;
    result2 += M4(0.2365305, -0.11887924, -0.14729121, 0.45517814, 0.034437858, 0.02805266, 0.0421054, 0.21288101, -0.44513264, -0.113560095, 0.19095585, -0.33065465, -0.18607673, -0.31838804, -0.04265418, -0.23199643) * inp_6_1_1;
    result2 += M4(0.31034935, 0.23054343, 0.22726725, -0.1347326, -0.04093553, -0.0124379955, -0.03298812, -0.1328269, 0.48419267, -0.1159716, 0.29693103, -0.21415308, 0.057307348, -0.061080914, -0.031392667, -0.074839234) * inp_6_2_1;
    result2 += M4(-0.12139281, 0.17418538, 0.035922267, -0.042205673, -0.06377194, -0.022132602, 0.048659638, 0.07855243, 0.31677297, 0.25795695, 0.098236375, -0.48101982, 0.10169297, 0.1455849, 0.042045534, -0.020934235) * inp_6_0_2;
    result2 += M4(0.024696104, -0.08019217, 0.0530051, 0.06849969, 0.013294041, 0.004367565, -0.018143194, -0.11156363, -0.17504904, 0.3786221, -0.019779807, 0.17071655, -0.08038029, -0.043809205, 0.055568274, -0.10549016) * inp_6_1_2;
    result2 += M4(-0.13214743, -0.14182377, -0.11136735, -0.007388966, 0.0031052623, 0.03164615, -0.064785235, 0.08883958, 0.42649943, 0.093914606, 0.048577573, -0.14112696, -0.21686853, -0.14776477, 0.08299661, 0.096879154) * inp_6_2_2;
    result3 += M4(-0.036539674, -0.038717035, -0.19442481, -0.2458306, -0.052757632, 0.029933438, 0.12897481, 0.023363441, 0.026583286, 0.04290543, -0.026008511, -0.014350349, 0.06427538, -0.36491454, 0.20100619, 0.01062764) * inp_6_0_0;
    result3 += M4(0.057018384, 0.57982105, -0.28756455, 0.39786503, -0.016774647, 0.079682164, -0.136208, 0.099223554, 0.054245424, -0.105121896, 0.14212392, 0.014582835, -0.090112105, -0.3957464, -5.992858e-05, 0.014945446) * inp_6_1_0;
    result3 += M4(0.0664164, -0.3584066, -0.13612805, -0.23609062, 0.09796867, -0.085284255, -0.07957001, -0.058667626, -0.08299696, 0.045576606, -0.009852749, -0.0032366537, 0.10066198, 0.05941768, -0.016941579, -0.011799797) * inp_6_2_0;
    result3 += M4(-0.047189776, 0.14789248, 0.38800052, 0.22758466, 0.058690216, -0.0052084997, 0.12656528, -0.01220262, -0.045506038, -0.08717602, 0.49184752, 0.08507806, -0.15326272, -0.028964303, 0.21192715, 0.14655071) * inp_6_0_1;
    result3 += M4(0.29073107, -0.28541234, 0.46065623, 0.15743847, 0.024536267, -0.024641298, 0.020493332, 0.104880296, -0.27750427, 0.069196306, -0.11157798, 0.023607522, -0.023508914, 0.040623043, 0.0068769474, -0.09910383) * inp_6_1_1;
    result3 += M4(-0.2921181, -0.061967805, 0.13703588, 0.09189116, -0.109660536, 0.024870774, 0.018300729, 0.062247325, -0.077498436, -0.0036966554, 0.027991986, 0.08287459, 0.3593638, 0.005810411, 0.13550405, -0.11900396) * inp_6_2_1;
    result3 += M4(0.15734886, 0.025986701, -0.20345068, -0.15718956, -0.001250648, -0.0149796065, -0.20427981, 0.04495984, 0.12364982, -0.007321561, -0.042715404, -0.055160973, -0.030419486, 0.040098224, 0.43406445, -0.0006441751) * inp_6_0_2;
    result3 += M4(-0.032131225, 0.05269363, -0.2122933, 0.05329741, -0.026311819, -0.04892374, 0.08393417, -0.2881799, 0.6886752, -0.11119675, 0.055636737, 0.5213091, -0.042950492, -0.024233202, -0.04472263, -0.20980778) * inp_6_1_2;
    result3 += M4(-0.10022518, -0.03187062, -0.09910099, -0.325532, 0.043152966, 0.053527296, 0.025874717, 0.04871363, -0.3305972, 0.0008651367, 0.033321608, 0.5348848, -0.09988578, -0.05755473, 0.019634284, -0.12395798) * inp_6_2_2;
    result4 += M4(0.047141854, -0.21286896, 0.1789418, 0.043325763, -0.013188623, -0.002344151, 0.012893131, 0.08757357, -0.08295396, -0.14011562, -0.005063648, -0.043271393, -0.0921394, 0.008021263, -0.05011517, -0.0070821713) * inp_6_0_0;
    result4 += M4(0.28400785, -0.2978412, -0.59148765, -0.14772741, 0.11951909, -0.03248427, -0.024468707, -0.037005227, 0.042500243, 0.10462589, -0.038290005, -0.04011123, -0.4332998, 0.47722405, 0.5235776, -0.088146746) * inp_6_1_0;
    result4 += M4(-0.005362321, 0.3154278, 0.22610484, -0.110531926, -0.0665615, 0.0041207396, 0.04045118, -0.06736566, 0.057012226, 0.06394188, -0.049293857, 0.014035235, 0.067586765, -0.021128563, 0.035518628, -0.24289767) * inp_6_2_0;
    result4 += M4(-0.037890963, -0.018358115, -0.031655088, -0.10525202, 0.07300559, -0.035530847, -0.060689475, -0.19171011, -0.22757868, -0.040337626, 0.24020848, 0.045211483, -0.059005555, 0.20406169, 0.1857301, 0.07951049) * inp_6_0_1;
    result4 += M4(-0.16790688, 0.18470974, 0.025034154, 0.044146076, -0.11170841, 0.098298, 0.045033902, 0.13582242, 0.062120218, -0.3176522, -0.17808782, -0.086211644, -0.23353624, 0.06171664, -0.09951519, -0.089094855) * inp_6_1_1;
    result4 += M4(0.10292055, -0.022489458, 0.011396742, 0.10808473, 0.067088395, -0.05858734, -0.0430135, 0.05561333, 0.15728465, -0.14185858, -0.10020357, -0.4487132, 0.09988148, 0.118363835, -0.0895973, -0.47334415) * inp_6_2_1;
    result4 += M4(0.14828493, -0.043387476, -0.075076394, -0.090575345, -0.04300751, 0.020152485, 0.049338613, 0.06599015, 0.16009802, 0.07705659, 0.083066896, 0.09650349, -0.07255332, -0.0027607111, -0.1301336, 0.13796131) * inp_6_0_2;
    result4 += M4(-0.056693174, 0.032343447, 0.18022269, 0.04166692, -0.055051945, -0.08896912, -0.038400758, -0.082301304, 0.6125771, 0.042286385, 0.088408515, 0.15559256, -0.09528197, -0.34153447, 0.198685, -0.11055426) * inp_6_1_2;
    result4 += M4(-0.27316737, 0.027304327, 0.08806091, -0.04104235, 0.025302505, 0.07713934, 0.021549663, 0.059713315, 0.15470283, -0.040890843, -0.19870003, -0.00888966, -0.17235947, -0.04011253, 0.047447607, -0.35290253) * inp_6_2_2;
    result5 += M4(0.030606838, 0.17542258, 0.045207813, 0.028775344, -0.01148848, 0.047917258, 0.05222929, 0.10838053, 0.022864604, -0.0267157, -0.098602355, 0.029074917, 0.041285552, -0.10180329, -0.27019313, -0.41111156) * inp_6_0_0;
    result5 += M4(0.008485686, -0.30881232, -0.29283515, 0.33189586, 0.026274122, 0.17194158, -0.033557456, -0.04760245, -0.111731425, 0.1921147, 0.0054538385, -0.08214871, -0.11257044, 0.08472914, 0.12532185, -0.44845772) * inp_6_1_0;
    result5 += M4(0.14357096, -0.16951063, -0.17094767, -0.07735461, -0.0075131785, -0.2706521, 0.01151896, -0.055010527, -0.019662498, -0.012713903, 0.13598862, 0.0590777, -0.18061914, -0.11988933, -0.10437295, 0.027259663) * inp_6_2_0;
    result5 += M4(0.1688571, -0.020760031, 0.09418021, -0.11976911, -0.03985959, -0.04767485, -0.039000727, -0.17164935, -0.009795336, -0.16490285, -0.19533731, 0.08067707, -0.008278056, -0.2686007, -0.25441724, 0.2992328) * inp_6_0_1;
    result5 += M4(0.056932304, -0.31134474, 0.07546048, -0.041124377, -0.025217872, -0.04460739, 0.04142374, 0.0519287, -0.42777023, 0.16439228, 0.048308138, 0.07034496, -0.20241363, -0.11741838, -0.116731495, 0.017949142) * inp_6_1_1;
    result5 += M4(-0.25999, 0.099378586, -0.023991227, -0.23691186, 0.03779193, 0.16640483, -0.048335593, 0.14289442, -0.037470922, 0.009426929, 0.17674498, -0.06854804, 0.11937484, 0.13856089, -0.102249965, 0.050649926) * inp_6_2_1;
    result5 += M4(-0.020126835, 0.5086378, -0.011824175, 0.023871979, 0.01450435, -0.025667263, 0.015386784, 0.06369808, 0.11420723, -0.061878428, -0.48056343, -0.21791884, -0.018738948, 0.49717495, -0.042604752, 0.03413257) * inp_6_0_2;
    result5 += M4(0.03968738, -0.1456935, 0.099275306, 0.10700328, 0.009802178, -0.060139444, 0.010024744, 0.011670227, -0.14327781, 0.17311084, -0.107571766, -0.52734965, -0.07404304, -0.24395253, 0.09032445, 0.053079385) * inp_6_1_2;
    result5 += M4(-0.13414863, 0.24953078, 0.20205657, 0.049447764, -0.0005640373, 0.045166947, -0.018732501, -0.0943955, -0.2385787, 0.09908038, -0.057395376, -0.16043495, 0.033228237, -0.22569309, 0.054085728, 0.11455014) * inp_6_2_2;
    result6 += M4(0.3918482, 0.17433432, 0.009146626, -0.96045876, 0.05615646, -0.0014473557, 0.038607895, 0.39416286, -0.07169875, 0.019225784, -0.04506775, 0.10079645, -0.17549907, 0.074796826, -0.00012869296, 0.4148142) * inp_6_0_0;
    result6 += M4(-0.18517727, -0.04248661, 0.049772043, 0.29656923, -0.020099174, 0.026085077, 0.008736178, -0.2278238, 0.15637682, -0.08224966, 0.031670827, 0.12534375, -0.017424822, 0.002134608, 0.14386216, 0.3036799) * inp_6_1_0;
    result6 += M4(0.6418229, -0.061928783, 0.2348707, -0.56112427, -0.05292602, -0.0013058305, -0.06993729, -0.077718616, -0.03465715, 0.005510801, -0.11701425, -0.17612962, -0.03227852, 0.053855177, -0.15561399, -0.12995449) * inp_6_2_0;
    result6 += M4(-0.48010778, -0.016896218, 0.09930131, 0.5012705, 0.09094784, 0.06190373, -0.032136753, -0.51364225, 0.098594815, 0.17036001, 0.068301044, 0.6142932, 0.089321494, -0.01605488, 0.049388185, 0.3411589) * inp_6_0_1;
    result6 += M4(-0.117335446, -0.04607408, -0.151066, 0.027214121, 0.0023934382, -0.11960942, 0.054180138, 0.1832932, -0.0045724427, -0.20163557, -0.08092108, -0.12673008, -0.011459009, -0.23815896, -0.12072423, -0.03680639) * inp_6_1_1;
    result6 += M4(-0.48661172, 0.22306374, -0.1520172, 0.6008698, -0.109077975, 0.049276777, -0.009849734, 0.21965948, -0.15171677, 0.09069706, -0.076033644, 0.108753085, 0.003964809, 0.002454488, -0.088214904, -0.04176991) * inp_6_2_1;
    result6 += M4(0.20387201, -0.021341335, -0.083330706, -0.61960894, -0.135989, -0.01612337, 0.00729583, 0.18229067, 0.02395173, 0.036806777, 0.034221902, -0.12119448, 0.181071, 0.20962963, 0.12587368, -0.006485741) * inp_6_0_2;
    result6 += M4(-0.16053502, -0.15183465, 0.013619628, 0.31886196, 0.016572911, 0.06287489, -0.059602436, -0.14231603, 0.027119568, -0.19202124, 0.12167672, -0.14593376, -0.07794655, -0.08068122, -0.0074994196, 0.029498434) * inp_6_1_2;
    result6 += M4(0.2038925, 0.0036630537, -0.081154905, 0.2126102, 0.16953398, -0.04525698, 0.053760458, -0.035672832, -0.117504485, 0.0370427, 0.1030739, 0.3319315, -0.055292223, 0.033748563, -0.11334572, 0.15699844) * inp_6_2_2;
    result7 += M4(0.029132446, -0.5769515, 0.15983392, 0.10779549, 0.010278157, -0.33410075, 0.0153546445, -0.058330994, 0.07115859, -0.09634331, -0.01681756, 0.041102674, 0.11119656, 0.15383838, 0.0238099, -0.016492322) * inp_6_0_0;
    result7 += M4(-0.11838046, 0.16002849, 0.21158144, -0.0051357634, -0.038316313, 0.10987621, -0.14123327, 0.011615726, -0.0629497, 0.23984233, 0.28618273, -0.14672318, 0.031176912, 0.055296402, -0.07858687, -0.10240062) * inp_6_1_0;
    result7 += M4(-0.057179935, -0.1683058, -0.104463644, 0.006176451, -0.0014645457, 0.17862664, 0.13675922, 0.025205523, 0.0073683676, -0.15001824, 0.012528705, -0.03354688, 0.19038428, -0.12732232, -0.51251364, 0.023216631) * inp_6_2_0;
    result7 += M4(-0.15798545, 0.26338452, -0.12153423, -0.37471431, 0.029715445, -0.03338121, 0.025348816, 0.20118253, 0.17186163, 0.29706174, -0.24028222, 0.603771, -0.21665105, 0.16796914, 0.008319218, 0.5816575) * inp_6_0_1;
    result7 += M4(0.4599811, 0.63645315, -0.06812711, 0.09734667, 0.08939335, 0.23816158, 0.12165169, -0.14684305, -0.35279658, 0.06312377, 0.16284677, -0.16988865, -0.29703006, -0.049696982, -0.019180346, 0.04116795) * inp_6_1_1;
    result7 += M4(0.027717974, 0.21661866, 0.02963244, -0.043779843, -0.1041128, -0.16780986, -0.12372318, -0.018145125, -0.106421284, -0.16903545, 0.22069709, 0.1918518, -0.2989716, 0.09631493, -0.07316544, 0.083262354) * inp_6_2_1;
    result7 += M4(0.19724238, -0.36173368, -0.9631451, -0.19386858, -0.05450094, 0.31982505, -0.055099245, -0.18987109, -0.09419254, -0.14873348, -0.60718155, 0.5322898, -0.097260356, -0.3941505, -0.10509361, 0.11016223) * inp_6_0_2;
    result7 += M4(-0.3281532, 0.3247075, 0.2917976, -0.033305753, -0.096880496, -0.30310374, 0.08208843, 0.15270272, 1.1714034, 0.057645824, -0.25037158, 0.05466978, -0.41555387, 0.29576534, 0.12832335, -0.05818495) * inp_6_1_2;
    result7 += M4(-0.036521304, -0.59588075, 0.4814691, 0.38674933, 0.15119122, -0.037582025, -0.071304366, 0.011837653, 0.31582767, -0.032424703, 0.09232408, 0.0030357488, -0.15668789, 0.16657645, 0.199947, 0.018929092) * inp_6_2_2;
    const V4 inp_7_0_0 = inp[7][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_7_1_0 = inp[7][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_7_2_0 = inp[7][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_7_0_1 = inp[7][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_7_1_1 = inp[7][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_7_2_1 = inp[7][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_7_0_2 = inp[7][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_7_1_2 = inp[7][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_7_2_2 = inp[7][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.0036442408, -0.14670588, 0.0060373186, 0.03394207, -0.042634364, -0.025424903, 0.03723891, -0.042159863, -0.0881734, -0.11579821, 0.15238097, -0.013456386, -0.0067135026, 0.044321146, 0.07675392, 0.0049952664) * inp_7_0_0;
    result0 += M4(0.061876275, 0.07526049, 0.012870677, -0.0040532146, -0.046122752, 0.11248861, 0.016022643, -0.017712228, 0.2117542, 0.13407232, 0.06021652, -0.20414445, -0.050865233, 0.010843999, -0.041802816, 0.007890666) * inp_7_1_0;
    result0 += M4(-0.055387974, -0.010310424, -0.041621435, -0.13018443, 0.011950597, 0.0021277035, -0.087642826, 0.08912526, 0.046133827, -0.062108263, -0.010490885, -0.30632812, 0.045433946, -0.004593913, -0.08538572, 0.03227908) * inp_7_2_0;
    result0 += M4(-0.07876785, 0.24361876, 0.02589284, -0.14525224, 0.08763353, -0.09755117, -0.0010613579, 0.07938792, -0.2102406, -0.18346778, -0.0019961637, -0.0017628827, 0.11917527, 0.27797765, -0.049132634, 0.020979177) * inp_7_0_1;
    result0 += M4(0.007144585, -0.14705089, -0.0400295, 0.06868954, -0.093900666, 0.64547527, -0.057200726, 0.40123692, -0.13926822, 0.32300523, -0.023980318, 0.2823626, -0.03189921, -0.2927669, 0.03131176, -0.15172978) * inp_7_1_1;
    result0 += M4(0.097690925, -0.0032369099, 0.032971963, 0.1269528, 0.014039831, 0.08855972, 0.17803223, 0.034531057, -0.12865347, -0.066063926, 0.015443418, 0.22356397, -0.05875199, 0.015472402, 0.058305524, 0.04754952) * inp_7_2_1;
    result0 += M4(0.07597306, -0.032901973, -0.04524315, 0.096660085, 0.30565777, 0.071324356, 0.0398423, 0.042098407, 0.11977153, 0.01568534, 0.11932585, -0.01746426, -0.11986398, -0.28249252, 0.020372858, 0.008804375) * inp_7_0_2;
    result0 += M4(-0.06510444, -0.0055899625, -0.049356453, 0.0187412, 0.50158584, 0.2681601, 0.117001034, -0.011641533, 0.10606254, -0.13848841, -0.07177858, -0.06365143, 0.0598863, 0.24719618, -0.011969535, 0.06585111) * inp_7_1_2;
    result0 += M4(-0.032349683, 0.03145508, 0.0537779, -0.063897684, 0.19792603, -0.034043744, 0.01922445, 0.009488463, 0.05094312, 0.08011092, -0.13742423, 0.06352516, 0.03806159, -0.027221879, 0.0054408517, -0.02509756) * inp_7_2_2;
    result1 += M4(0.07503969, 0.05098356, -0.027968705, 0.04324671, 0.092024915, -0.043228436, -0.060600016, -0.040897314, -0.08045672, 0.06587326, 0.04653294, 0.067915976, -0.11060248, -0.08781639, 0.09517214, -0.05865293) * inp_7_0_0;
    result1 += M4(0.058652796, -0.056069016, -0.02269759, 0.086421475, -0.04674432, -0.08723038, 0.109789446, -0.010967054, 0.3834439, -0.19912359, 0.12136836, -0.17640777, -0.090510465, -0.082314685, -0.006268565, 0.04729094) * inp_7_1_0;
    result1 += M4(-0.18331356, 0.03272054, -0.029204696, -0.100189626, -0.32993504, -0.0564822, -0.26883325, -0.12969957, -0.2508931, 0.09192104, -0.08068747, 0.24879047, 0.28902417, 0.07343988, -0.009295749, -0.038331136) * inp_7_2_0;
    result1 += M4(-0.059413746, -0.026269626, -0.0065648407, -0.09502397, 0.124097325, 0.12897652, 0.08461574, 0.02993698, 0.03355301, -0.06616287, -0.07158347, 0.06597898, 0.11037423, 0.05038417, -0.1377514, 0.08186609) * inp_7_0_1;
    result1 += M4(-0.09873389, -0.023260973, -0.0027020583, 0.024544489, -0.02883347, -0.1603975, -0.16539052, -0.096290655, -0.09646582, 0.16812909, -0.02219279, -0.093693875, -0.001594237, 0.043591585, 0.0010967962, -0.10691508) * inp_7_1_1;
    result1 += M4(0.20100015, 0.037026737, 0.14711216, 0.08131079, -0.5793635, -0.07098612, -0.3355013, -0.76627815, 0.069194704, -0.15190524, 0.110268615, -0.33443844, -0.23083447, -0.10344225, -0.010893658, 0.10966464) * inp_7_2_1;
    result1 += M4(0.015847739, -0.050612487, 0.021674136, -0.030361863, 0.011395899, -0.2153484, 0.05697008, 0.0065922257, 0.048055537, -0.15916058, 0.06915038, -0.20978335, -0.0012097551, 0.094550416, 0.03339496, -0.012672321) * inp_7_0_2;
    result1 += M4(0.03612482, 0.06127944, 0.037800625, -0.14033183, 0.048581526, -0.3008664, -0.29939064, 0.5514814, 0.072968766, 0.21155882, -0.06704078, 0.18266682, 0.070436135, -0.0894176, 0.024510123, 0.057923708) * inp_7_1_2;
    result1 += M4(-0.054855924, -0.029535025, -0.09083207, 0.08726941, 0.039269235, -0.0007863218, -0.04557254, -0.4694825, -0.19658886, 0.01885427, -0.09716895, 0.26497483, -0.029089592, 0.07612446, -0.016487515, -0.080591515) * inp_7_2_2;
    result2 += M4(-0.053229403, -0.062377576, -0.09849377, 0.17578836, 0.0026719917, -0.006830585, 0.056218743, -0.02153137, -0.010086018, -0.09728025, -0.05226268, 0.0041720294, 0.017230123, 0.038970836, 0.076554686, -0.2448952) * inp_7_0_0;
    result2 += M4(-0.034270357, 0.11236197, 0.09204133, -0.20235191, -0.3568749, -0.012110738, -0.068679936, 0.08955381, -0.043797925, 0.09507351, -0.040964745, -0.26474512, 0.031831954, -0.09176261, -0.18627751, 0.37436065) * inp_7_1_0;
    result2 += M4(0.008507656, -0.07809829, 0.0012296358, 0.14844179, 0.07257807, -0.043389224, 0.17160062, 0.05501582, 0.16285528, -0.017062329, 0.045154568, 0.152202, 0.010842558, 0.079697855, 0.013856618, -0.14852981) * inp_7_2_0;
    result2 += M4(0.1808408, 0.1666876, 0.13125914, -0.19935527, 0.12676585, 0.059419163, -0.08774952, 0.0523966, 0.018798608, -0.17687993, 0.025133839, -0.031289775, 0.0303249, 0.061341926, 0.06499304, 0.12550901) * inp_7_0_1;
    result2 += M4(-0.17555399, -0.16555674, -0.06076121, 0.10973854, -0.14257695, 0.27671555, 0.019388542, 0.44667628, 0.034132913, 0.010102924, -0.056429412, 0.32345036, -0.03543133, 0.009778563, 0.096347585, -0.21983227) * inp_7_1_1;
    result2 += M4(-0.06856662, 0.04764111, -0.047995087, -0.023001384, 0.15854159, -0.015145543, 0.5992324, -0.03347533, -0.097416915, 0.23013687, 0.12907566, -0.118328914, -0.08346258, -0.028713536, -0.092830196, 0.12057178) * inp_7_2_1;
    result2 += M4(-0.07385334, -0.049057137, -0.015345666, 0.05433581, 0.09762921, -0.08973504, 0.0067441086, -0.07880086, 0.15651026, 0.07660166, -0.021789186, -0.25391153, -0.022397975, -0.15266214, -0.112442315, 0.04381715) * inp_7_0_2;
    result2 += M4(0.14565952, -0.030339625, 0.03988828, -0.003410433, -0.18400182, -0.14522158, 0.098959014, 0.20270453, -0.061848145, -0.16001529, -0.16311395, 0.04102898, 0.020980125, 0.19236118, -0.056087974, -0.07122461) * inp_7_1_2;
    result2 += M4(0.057069443, 0.064195424, -0.02512969, -0.049953368, -0.14083764, -0.012009874, 0.3020715, 0.25047523, -0.08674021, 0.03289864, 0.093820184, 0.082784444, 0.0010650549, -0.10094933, 0.16113104, 0.012164657) * inp_7_2_2;
    result3 += M4(-0.025856342, 0.13079228, 0.08618113, 0.09018446, 0.011957774, 0.01558657, 0.23974837, 0.020313958, 0.021140616, 0.22350857, 0.42091158, 0.13621664, 0.031919684, -0.21892564, 0.04232442, 0.007004977) * inp_7_0_0;
    result3 += M4(-0.023658847, -0.23673977, -0.089851856, -0.017564598, 0.06135422, 0.083749965, -0.31049043, -0.029362967, 0.11678081, 0.055421017, -0.18577883, -0.2446101, -0.055230025, 0.10159207, -0.032379683, 0.0955802) * inp_7_1_0;
    result3 += M4(0.049933992, 0.013702904, -0.050213646, -0.1258348, -0.059889283, 0.075058684, -0.07826112, -0.112960786, 0.0040791156, -0.026270907, -0.21151836, 0.15925902, -0.0051617203, 0.117665894, 0.10880897, -0.06973216) * inp_7_2_0;
    result3 += M4(-0.031162802, -0.025551148, 0.23111911, -0.070619136, -0.047568373, 0.043086074, -0.23472102, -0.0218465, 0.07959249, -0.052954216, -0.13344175, -0.12169194, 0.03506201, 0.17531264, -0.01967535, -0.026097074) * inp_7_0_1;
    result3 += M4(-0.086934984, 0.09628961, 0.011462504, -0.07287086, 0.036667842, -0.32817438, 0.20883831, 0.17168154, -0.040815316, -0.028889213, 0.18540278, 0.11309168, -0.0061568995, -0.056632273, -0.2757164, -0.16861852) * inp_7_1_1;
    result3 += M4(0.0498194, 0.0001233524, -0.020805152, -0.0016146419, -0.31261382, -0.112071805, -0.035545394, -0.025336577, -0.18327142, 0.03855399, -0.07157443, 0.1887849, 0.04433564, -0.13562849, -0.036056798, -0.036998227) * inp_7_2_1;
    result3 += M4(0.08438835, -0.07086919, -0.22903387, 0.012830001, -0.018672585, -0.04658818, 0.29161862, -0.056236587, -0.037212007, 0.0027398022, -0.23530404, 0.2685287, -0.06578308, -0.010759878, 0.11499214, -0.114503466) * inp_7_0_2;
    result3 += M4(0.0813673, 0.06490341, 0.09098087, 0.005233356, -0.32707438, -0.07025287, -0.23037453, -0.5556168, -0.2980563, -0.091501504, 0.1989339, -0.33462068, 0.16198827, -0.015836729, 0.087699845, 0.15587159) * inp_7_1_2;
    result3 += M4(-0.06354313, 0.03618643, -0.0187467, 0.19293183, 0.7448956, -0.035743166, -0.092921406, -0.61832374, 0.33200088, -0.070930265, 0.04749195, -0.16444391, -0.17103323, 0.025791219, 0.0074768704, 0.15645593) * inp_7_2_2;
    result4 += M4(0.05437263, -0.04003176, -0.10683477, 0.06042497, 0.020114645, -0.10722655, -0.14862777, -0.019468976, 0.1303054, -0.07358252, 0.03345518, 0.18417852, -0.04273577, 0.064864375, 0.050297223, 0.0504767) * inp_7_0_0;
    result4 += M4(-0.13207294, 0.12690744, 0.10181047, -0.1743898, 0.094703175, -0.24912958, 0.07894256, -0.0005799152, 0.112046555, -0.24036738, 0.06880392, -0.0073224353, 0.11037301, 0.078139335, -0.12244816, 0.022380473) * inp_7_1_0;
    result4 += M4(-0.0070317653, -0.015765188, 0.01060603, 0.052184552, -0.057586037, 0.06544551, -0.20414752, 0.17318866, 0.031179454, -0.19085726, -0.06961953, 0.06612763, -0.04581052, -0.16009994, 0.06908934, -0.020400558) * inp_7_2_0;
    result4 += M4(0.09171846, 0.017358825, 0.005700263, -0.16853215, 0.15686621, 0.11183641, -0.013170923, 0.1677219, 0.10020577, 0.11719267, -0.12746456, -0.0025654745, 0.009026234, -0.014727843, 0.0025699341, -0.034113616) * inp_7_0_1;
    result4 += M4(-0.03825333, -0.013274811, -0.050379753, 0.18065588, -0.19414839, 0.31820002, 0.10230294, 0.24612242, -0.15326129, 0.31873095, -0.059226897, 0.021278521, -0.024131255, -0.12620166, 0.111928776, 0.07533664) * inp_7_1_1;
    result4 += M4(-0.018050402, -0.01658115, -0.031985983, -0.064446986, -0.13108797, 0.00072603754, -0.2724459, 0.8121937, 0.010791579, 0.1353151, -0.09497966, 0.26468667, -0.06933093, 0.10721632, -0.0587357, 0.013003995) * inp_7_2_1;
    result4 += M4(-0.08554044, -0.026467795, 0.01608519, -0.0070601474, -0.08209757, 0.039749227, 0.030943753, -0.024507716, 0.1676505, -0.038027298, 0.12542993, 0.087149836, -0.0780964, 0.0038648064, 0.017535258, 0.021429814) * inp_7_0_2;
    result4 += M4(0.112693675, -0.061938047, 0.0006560414, -0.05868785, -0.47106585, 0.07775038, 0.037286557, -0.05932837, -0.5388309, -0.064565204, -0.0034024431, 0.17437278, 0.10242731, 0.014758693, 0.024691008, -0.083288394) * inp_7_1_2;
    result4 += M4(0.04902074, 0.023247786, 0.050650448, -0.024688454, -0.24984325, 0.14256178, -0.14493294, -0.22340089, 0.19008614, -0.051822327, 0.12145915, -0.0018930776, 0.013943603, 0.0355288, -0.08697799, 0.060964204) * inp_7_2_2;
    result5 += M4(0.00020553917, -0.10061906, 0.08484927, 0.11521983, -0.030174702, 0.09445666, -0.03834772, 0.071895, -0.031671476, -0.16904919, 0.024292992, 0.14087951, 0.042974148, 0.061857823, -0.034443405, -0.10219108) * inp_7_0_0;
    result5 += M4(0.07948002, 0.24000065, -0.006065982, -0.081500955, 0.06277709, 0.031539433, 0.03251546, -0.01604461, 0.12478552, 0.10718378, 0.024151616, -0.035760287, -0.03499674, -0.25642496, -0.036765624, 0.17713948) * inp_7_1_0;
    result5 += M4(-0.085472055, -0.23345329, 0.061214764, -0.039946914, -0.056494184, -0.009982393, 0.14904405, 0.067551136, -0.157847, -0.054766107, 0.05777327, -0.0558625, -0.03153948, 0.26908046, 0.030948041, -0.060405105) * inp_7_2_0;
    result5 += M4(0.057546094, 0.16247673, 0.0258607, -0.1873618, -0.097382694, -0.087470934, 0.121518, -0.09825697, 0.02680612, 0.091855414, 0.044744655, -0.12517375, -0.0018338506, -0.13430773, -0.11997842, 0.055368513) * inp_7_0_1;
    result5 += M4(-0.12621236, -0.0512711, -0.030482791, 0.16059534, 0.15285975, 0.09402935, -0.08746276, -0.036353018, 0.1854088, 0.036302768, -0.036139514, -0.07086259, -0.087804146, 0.23447584, 0.13877739, -0.12293426) * inp_7_1_1;
    result5 += M4(0.09059555, 0.10278689, -0.066254295, 0.043405186, -0.6452943, -0.09849823, 0.34284416, 0.028468236, -0.10986583, 0.23232464, -0.111170776, 0.02072458, 0.1225423, -0.2577987, -0.049140982, 0.025229959) * inp_7_2_1;
    result5 += M4(-0.065766506, -0.22537358, -0.104178436, 0.0789597, 0.06886667, -0.2582994, 0.040388152, 0.20392878, -0.1325317, -0.17091429, -0.025321128, -0.03846908, 0.001345403, 0.08792447, 0.20960763, -0.029588128) * inp_7_0_2;
    result5 += M4(0.034698103, 0.16890207, 0.028553866, -0.120815694, 0.034419764, -0.03802536, -0.012798052, 0.3776902, -0.10802205, 0.25952816, 0.051863562, 0.13329516, 0.08949897, -0.17582385, -0.20624496, -0.0068664458) * inp_7_1_2;
    result5 += M4(-0.0036929054, -0.075137, 0.023706064, 0.04036705, -0.22011477, -0.12010217, 0.30943853, 0.1229249, 0.17553005, -0.32864487, -0.035663143, 0.04480098, -0.09330225, 0.20827624, 0.05076678, 0.040865038) * inp_7_2_2;
    result6 += M4(0.10160282, -0.018210733, 0.11960238, 0.22525762, 0.11117682, -0.077137575, -0.040505897, -0.47369707, -0.17267421, -0.094693065, -0.026843652, 0.21402717, -0.053942468, -0.051647536, 0.05917362, 0.08196573) * inp_7_0_0;
    result6 += M4(-0.0070946366, -0.001048489, -0.04395798, 0.047385514, -0.17359792, 0.071782224, 0.012977537, 0.15289062, 0.36385012, -0.08355867, 0.066009395, -0.40129784, 0.0038894862, -0.031020934, -0.028018793, -0.26828036) * inp_7_1_0;
    result6 += M4(-0.20096134, 0.06847282, -0.019137088, -0.14253293, 0.38593423, 0.0108374795, -0.19201893, -0.013232457, -0.20656346, 0.20854391, 0.007377611, 0.29297706, -0.004322105, 0.054973524, -0.0064922287, 0.15490766) * inp_7_2_0;
    result6 += M4(-0.028351197, 0.13654329, -0.087784864, -0.34244108, -0.08549062, -0.022519358, 0.045166582, 0.2874081, 0.12915353, 0.06444494, -0.20301273, -0.2697022, 0.13820696, -0.06394656, -0.06183011, 0.065495655) * inp_7_0_1;
    result6 += M4(0.053878203, -0.12534676, 0.11502889, -0.14114413, 0.1720413, -0.0640427, 0.15572014, 0.19419242, -0.10463681, -0.10591972, 0.053079914, 0.17672248, 0.03618163, 0.20323874, -0.049349543, 0.09785522) * inp_7_1_1;
    result6 += M4(-0.00811029, -0.02158674, -0.014832123, 0.19067298, -0.16853574, 0.09965899, -0.20408535, -0.06574402, 0.07465689, -0.046934202, 0.102180205, -0.1367306, 0.16719002, -0.1048629, 0.058244254, -0.19760638) * inp_7_2_1;
    result6 += M4(-0.014969022, -0.10568563, 0.005133914, 0.14000212, 0.022780273, 0.07151405, -0.033103086, 0.055076748, -0.042584285, 0.07026201, 0.091329716, 0.42965275, -0.121156685, 0.08935772, 0.03254957, 0.08257348) * inp_7_0_2;
    result6 += M4(-0.014114336, 0.104343824, -0.06931856, -0.056577235, -0.010596563, 0.06351124, -0.08960634, -0.03365306, 0.060028646, 0.0864245, -0.18660627, 0.13823965, -0.11220823, -0.14186066, 0.054975644, -0.042534065) * inp_7_1_2;
    result6 += M4(0.10685882, -0.045940585, 0.024158685, 0.02324897, 0.10659019, 0.23432274, -0.02124637, -0.70644546, -0.080191985, -0.021214869, 0.0922108, -0.39624286, -0.04814861, 0.017708082, -0.057600316, 0.07528421) * inp_7_2_2;
    result7 += M4(0.039666362, -0.4269836, 0.06070161, 0.028413888, 0.0025913678, -0.029585652, -0.119877785, -0.030185755, 0.020320455, -0.31869426, 0.03625279, -0.029997388, 0.02296383, 0.4362184, -0.19068524, -0.11691816) * inp_7_0_0;
    result7 += M4(0.007837039, 0.14507303, -0.10730836, -0.0016275003, 0.047276422, -0.073243216, 0.08194571, -0.0754958, 0.16505735, 0.09312824, -0.071741275, -0.023695335, -0.079611816, -0.08771879, 0.113777466, 0.01152029) * inp_7_1_0;
    result7 += M4(-0.025016088, 0.2017786, 0.011761619, 0.03399915, 0.022989403, 0.053096406, 0.3256524, -0.06636753, -0.21843442, 0.04122289, -0.020986786, -0.04491874, 0.04048062, -0.17276415, 0.06880957, 0.017317517) * inp_7_2_0;
    result7 += M4(0.08585497, 0.23837355, 0.024308184, 0.10632778, 0.14022683, -0.15036057, -0.11212348, -0.089054696, 0.044839095, -0.06613446, -0.00812582, -0.19733194, 0.053614777, -0.34399053, -0.09531722, 0.2372865) * inp_7_0_1;
    result7 += M4(-0.09875156, -0.0026923874, 0.048627354, -0.08264372, 0.21519653, 0.26710156, -0.09189746, 0.25634658, 0.107293695, 0.16347373, 0.0026017842, 0.15523301, -0.103245534, -0.1028121, 0.021487987, -0.0880444) * inp_7_1_1;
    result7 += M4(-0.07491263, -0.15506206, -0.12889482, -0.0693857, 0.03453258, -0.015016597, 0.2659475, 0.19250146, 0.14892294, -0.22486942, -0.08308755, -0.005629776, 0.056032006, 0.288737, 0.1442295, -0.028141294) * inp_7_2_1;
    result7 += M4(-0.07977681, 0.02595742, -0.0874773, -0.05161754, -0.23555714, 0.14743973, 0.34919596, -0.08454505, -0.0030413358, 0.31644538, 0.535135, -0.24707095, -0.09799671, 0.04375856, 0.32588616, 0.13514802) * inp_7_0_2;
    result7 += M4(0.03336791, -0.09290555, 0.14141479, 0.08801894, -1.2281753, -0.28514332, -0.02594772, -0.07835633, -0.39473787, -0.38881725, -0.05499787, 0.40771616, 0.26509523, 0.075894915, -0.27684388, -0.20175917) * inp_7_1_2;
    result7 += M4(0.14488767, 0.060125567, 0.013285371, 0.0035515772, -0.23833157, -0.04095364, 0.00050822133, 0.13909021, 0.12878568, 0.4207225, -0.36394024, -0.045843598, -0.14772438, -0.11483643, -0.0916808, 0.014762052) * inp_7_2_2;
    const ivec2 output_base = ivec2(gl_GlobalInvocationID) * ivec2(4, 2);
    imageStore(out_image, output_base + ivec2(0, 0), max(result0, V4(0.0)));
    imageStore(out_image, output_base + ivec2(1, 0), max(result1, V4(0.0)));
    imageStore(out_image, output_base + ivec2(2, 0), max(result2, V4(0.0)));
    imageStore(out_image, output_base + ivec2(3, 0), max(result3, V4(0.0)));
    imageStore(out_image, output_base + ivec2(0, 1), max(result4, V4(0.0)));
    imageStore(out_image, output_base + ivec2(1, 1), max(result5, V4(0.0)));
    imageStore(out_image, output_base + ivec2(2, 1), max(result6, V4(0.0)));
    imageStore(out_image, output_base + ivec2(3, 1), max(result7, V4(0.0)));
}

//!DESC ArtCNN C4F32 DS (Conv2D-2-ReLU)
//!COMPUTE 48 32 12 16
//!HOOK LUMA
//!BIND conv2d_1
//!SAVE conv2d_2
//!WIDTH LUMA.w 4.0 *
//!HEIGHT LUMA.h 2.0 *
//!COMPONENTS 4
//!WHEN OUTPUT.w LUMA.w / 1.3 > OUTPUT.h LUMA.h / 1.3 > *
#extension GL_EXT_shader_explicit_arithmetic_types_float16 : enable
#ifdef GL_EXT_shader_explicit_arithmetic_types_float16
#	define V4 f16vec4
#	define M4 f16mat4
#	define F float16_t
#else
#	define V4 vec4
#	define M4 mat4
#	define F float
#endif

const ivec2 ksize = ivec2(3, 3);
const ivec2 offset = ksize / 2;
const ivec2 wg_size = ivec2(gl_WorkGroupSize);
const ivec2 isize = wg_size + ksize - 1;
shared V4 inp[8][isize.y][isize.x];
void hook() {
    const uvec2 local_xy = gl_LocalInvocationID.xy;
    ivec2 base = ivec2(gl_WorkGroupID) * wg_size;
    for (uint y = local_xy.y; y < isize.y; y += wg_size.y) {
        for (uint x = local_xy.x; x < isize.x; x += wg_size.x) {
            const ivec2 input_base = (base + ivec2(x,y) - offset) * ivec2(4, 2);
            inp[0][y][x] = V4(conv2d_1_mul * texelFetch(conv2d_1_raw, input_base + ivec2(0, 0), 0));
            inp[1][y][x] = V4(conv2d_1_mul * texelFetch(conv2d_1_raw, input_base + ivec2(1, 0), 0));
            inp[2][y][x] = V4(conv2d_1_mul * texelFetch(conv2d_1_raw, input_base + ivec2(2, 0), 0));
            inp[3][y][x] = V4(conv2d_1_mul * texelFetch(conv2d_1_raw, input_base + ivec2(3, 0), 0));
            inp[4][y][x] = V4(conv2d_1_mul * texelFetch(conv2d_1_raw, input_base + ivec2(0, 1), 0));
            inp[5][y][x] = V4(conv2d_1_mul * texelFetch(conv2d_1_raw, input_base + ivec2(1, 1), 0));
            inp[6][y][x] = V4(conv2d_1_mul * texelFetch(conv2d_1_raw, input_base + ivec2(2, 1), 0));
            inp[7][y][x] = V4(conv2d_1_mul * texelFetch(conv2d_1_raw, input_base + ivec2(3, 1), 0));
        }
    }

    barrier();
    V4 result0 = V4(-0.06030993, -0.18949664, 0.015182512, 0.09619445);
    V4 result1 = V4(-0.03482897, -0.05249293, -0.009638269, -0.016478464);
    V4 result2 = V4(0.017338736, 0.035598457, 0.011148819, -0.044533733);
    V4 result3 = V4(0.03492964, -0.016571313, -0.062663615, -0.054031406);
    V4 result4 = V4(0.06710359, 0.10097546, 0.03665374, 0.019653283);
    V4 result5 = V4(-0.0017843647, -0.0044595515, -0.052310888, -0.016851803);
    V4 result6 = V4(-0.07017793, 0.047268998, 0.08072696, 0.0242515);
    V4 result7 = V4(-0.016865827, 0.039467253, 0.0019502136, 0.019199317);
    const V4 inp_0_0_0 = inp[0][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_0_1_0 = inp[0][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_0_2_0 = inp[0][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_0_0_1 = inp[0][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_0_1_1 = inp[0][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_0_2_1 = inp[0][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_0_0_2 = inp[0][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_0_1_2 = inp[0][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_0_2_2 = inp[0][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.020101279, -0.35410517, 0.28006744, 0.08663914, 0.023265176, -0.0025053364, 0.13159442, -0.12293535, 0.1265328, -0.061532903, -0.07122904, -0.293625, -0.00490153, -0.05276206, 0.16092497, -0.49980012) * inp_0_0_0;
    result0 += M4(0.19290231, 0.9025124, 0.8201907, -0.088382125, -0.08857249, 0.05774787, -0.5546504, -0.28218168, 0.4108857, 0.16212092, 0.14918874, -0.47784498, 0.015551204, 0.019166393, 0.6054657, 0.0032832772) * inp_0_1_0;
    result0 += M4(-0.029101541, 0.046435714, -0.16116987, -0.33825257, -0.15747255, -0.46088874, -0.5279546, -0.21988435, -0.0019320884, 0.098644234, -0.0001770148, -0.2633601, 0.10936358, 0.058972366, 0.4541514, 0.11590145) * inp_0_2_0;
    result0 += M4(-0.009056013, 0.030989535, 0.11899685, 0.040287763, -0.051347226, 0.12260273, -0.17672972, -0.103819266, 0.08521047, -0.19595285, 0.033385802, -0.07092241, 0.15700068, -0.42815876, -0.11174102, -0.07586752) * inp_0_0_1;
    result0 += M4(-0.20697692, -0.92671597, -0.1273465, -0.13974762, 0.020622801, -0.003832922, -0.070200875, -2.2233362, -0.0054845237, 1.3501109, 0.15469675, 1.4271778, 0.37425053, 0.20047313, -0.005327705, -1.6151639) * inp_0_1_1;
    result0 += M4(0.01604078, -0.20244288, -0.08741294, -0.096272476, -0.08363711, -1.3717843, 0.27644357, 0.60674447, 0.15627955, 0.20427643, -0.12867133, -0.27383792, 0.14485475, 0.13553958, 0.09082041, -0.61374146) * inp_0_2_1;
    result0 += M4(0.0257576, -0.16978376, -0.099578954, 0.05800108, -0.048227604, -0.010906622, 0.016439695, -0.12399109, -0.067040205, -0.12314603, -0.008967979, -0.113039166, 0.023438305, -0.0178847, -0.10625818, -0.08403077) * inp_0_0_2;
    result0 += M4(-0.027185246, -0.046917625, -0.09484453, -0.11550478, 0.044557866, 0.117379606, 0.013182721, -0.028285818, 0.07956808, 0.16380218, -0.28400218, -0.010999399, -0.09387765, -0.5850925, 0.11013864, -0.68357027) * inp_0_1_2;
    result0 += M4(0.07227808, -0.0029823892, -0.21979369, 0.048537537, -0.1027746, -0.0031889298, -0.075029194, 0.15139669, -0.0628973, 0.06391503, 0.06962481, 0.02200629, 0.017615594, 0.40636706, -0.05974789, -0.07580692) * inp_0_2_2;
    result1 += M4(0.2976057, -0.41753337, 0.43786776, -0.112171374, -0.23619935, -0.16614772, -0.17105938, -0.038833957, -0.008156212, -0.8555714, -0.042871837, -0.0006289102, 0.14266057, -0.24422118, 0.44640446, -0.41233355) * inp_0_0_0;
    result1 += M4(-0.12406317, 0.027730322, -0.19336882, -0.10621516, -0.21531068, 0.17680226, -0.08214146, -0.10790021, -0.07957512, -0.06261267, 0.18771619, 0.34875607, 0.20680073, -0.06255271, -0.35397652, -0.10719654) * inp_0_1_0;
    result1 += M4(0.13804756, -0.04804263, -0.11848491, 0.47130984, -0.32124972, -0.106216095, 0.09326995, 0.24440631, 0.035532456, 0.13430211, -0.1135181, -0.29534855, 0.13358009, -0.06911347, 0.01675705, 0.41002008) * inp_0_2_0;
    result1 += M4(0.13191958, -0.4555368, -0.0967613, 0.46895197, 0.1289692, -0.07572086, -0.41100016, 0.07257786, 0.16469955, -0.30381435, -0.20755768, -0.12424619, -0.41693398, 0.15606661, -0.008577172, -0.029855635) * inp_0_0_1;
    result1 += M4(-0.22019295, -0.1376703, 0.08650365, 0.10001901, 0.28173855, 0.11575108, 0.16874999, -0.0050227563, 0.3774207, 0.31600794, -0.10345772, -0.05984522, -0.14312312, 0.31494835, -0.10255951, 0.28471696) * inp_0_1_1;
    result1 += M4(0.09896433, -0.37460753, 0.1861945, 0.08628068, -0.06026498, -0.39374223, 0.30579656, -0.1185039, -0.035154324, 0.03527869, 0.091287754, 0.25008905, 0.19205098, 0.09888815, 0.15234135, -0.2986003) * inp_0_2_1;
    result1 += M4(0.118029624, -0.008516702, -0.10569862, -0.10191308, -0.1252029, -0.017103158, -0.2972995, -0.09201399, 0.025150301, 0.11352668, 0.31176552, 0.08695743, 0.2480199, 0.15765293, 0.21503279, 0.2556083) * inp_0_0_2;
    result1 += M4(0.064184256, 0.10683261, 0.08413316, -0.17104606, -0.17008576, 0.0555231, -0.28492394, 0.17664981, -0.08283345, 0.18675864, -0.022968179, -0.028522031, 0.032112967, 0.049122956, -0.17292173, -0.05253707) * inp_0_1_2;
    result1 += M4(0.017539948, -0.050528925, 0.010243035, -0.039314646, 0.020140631, 0.08307466, 0.07053625, -0.13795665, -0.13755982, 0.1377305, -0.15034027, 0.12501366, 0.106197685, -0.17295255, -0.12674235, -0.3507187) * inp_0_2_2;
    result2 += M4(-0.0676442, 0.2798219, -0.26334125, 0.10492754, 0.04529137, -0.0225712, -0.06427857, 0.026372617, -0.064207524, 0.053111006, 0.06633235, 0.10995372, -0.06165668, 0.17213918, 0.103595845, -0.11108953) * inp_0_0_0;
    result2 += M4(0.105092406, -0.5331738, 0.55125606, -0.004351551, -0.12674662, 0.0293073, 0.1314528, -0.1272611, -0.04503006, -0.30869746, -0.3254115, -0.06486336, 0.029685382, -0.22822879, 0.024976073, 0.08190164) * inp_0_1_0;
    result2 += M4(0.19026962, -0.22357056, -0.21473731, 0.15006398, -0.027784638, 0.107414156, -0.13945493, -0.2054147, -0.03675486, 0.14702858, 0.10494913, 0.10377914, -0.0545323, 0.08473948, -0.3328965, -0.21337947) * inp_0_2_0;
    result2 += M4(-0.40149856, 0.008957024, -0.26150694, 0.05144135, -0.009414171, -0.06399138, -0.16505636, 0.0075922473, -0.06568079, 0.03057978, 0.16879417, 0.06948154, 0.041279696, -0.1033363, -0.20721525, -0.1768217) * inp_0_0_1;
    result2 += M4(0.032140054, 0.570498, -0.84412235, 0.22313215, -0.17650433, 0.66884774, -1.2914184, -0.47706616, 0.008046656, -0.46702617, -0.028944127, 0.5051134, 0.27060038, -0.050160497, 0.21984252, 0.37883142) * inp_0_1_1;
    result2 += M4(0.34014916, 0.079698525, -0.26152173, -0.11468657, -0.44434783, 0.52344704, -2.0551214, -0.171717, 0.25237513, 0.10932493, -0.27692124, -0.02356951, 0.04865198, 0.017488793, -0.2525538, 0.14029862) * inp_0_2_1;
    result2 += M4(-0.050589353, 0.08361229, 0.08676714, -0.0489857, -0.03705499, -0.07856881, -0.033116512, -0.029437654, -0.032571174, 0.009350278, -0.26764446, -0.04995522, -0.119301595, 0.008302067, 0.012930774, -0.15806642) * inp_0_0_2;
    result2 += M4(0.09314861, 0.07667873, 0.03671034, -0.044690773, -0.2523008, -0.3085057, 0.085794516, -0.15571024, -0.02100341, 0.26839367, 0.012476824, 0.11249443, -0.17717105, 0.1755708, 0.097484104, 0.09175075) * inp_0_1_2;
    result2 += M4(0.13222618, -0.003792224, -0.0078079654, 0.09263818, -0.078186184, -0.14797707, -0.017316263, -0.384731, 0.033676244, -0.06708707, -0.13061938, -0.046156183, 0.309028, -0.30622035, -0.041106112, -0.32462353) * inp_0_2_2;
    result3 += M4(0.2933891, 0.10734633, 0.043465536, 0.020947497, -0.04793472, -0.016914561, 0.02061675, -0.06986499, -0.0601295, -0.045202967, -0.017066369, 0.090864204, 0.09238028, -0.06778325, -0.108855814, -0.13544546) * inp_0_0_0;
    result3 += M4(-0.32886055, 0.15152225, 0.15923952, 0.046245616, 0.36327928, -0.1649582, -0.19898632, -0.15458755, 0.007997473, 0.14422482, 0.24937567, -0.19290322, -0.20507923, -0.18587781, -0.0005173003, 0.09073358) * inp_0_1_0;
    result3 += M4(-0.13036956, -0.057497073, -0.12975752, -0.018053966, 0.35814252, -0.27156547, -0.1682681, 0.00698008, 0.055737752, 0.045305643, -0.095738426, -0.23580372, 0.17064132, 0.15605927, -0.18541338, -0.13374265) * inp_0_2_0;
    result3 += M4(0.40444484, 0.44520107, -0.16204871, -0.0220801, 0.004856236, 0.06172943, -0.1807548, -0.11374724, 0.061225798, -0.14886022, 0.057539165, -0.31978697, 0.010379271, 0.01505832, 0.03020854, 0.053527046) * inp_0_0_1;
    result3 += M4(0.24253744, 0.15951543, -0.0930377, -0.7276848, 0.34732184, -0.22167899, -0.117383346, -0.06598471, -0.1618058, 0.10988853, 0.031851564, -0.06256788, 0.22075847, 0.21905512, -0.47164378, -0.0966782) * inp_0_1_1;
    result3 += M4(0.009391212, 0.034285694, -0.10681356, -0.23971687, 0.32913107, -0.10473943, -0.50203234, 0.41944763, 0.024054235, -0.027000254, 0.586093, -0.0015864754, -0.324191, 0.06699464, -0.13469692, -0.006740816) * inp_0_2_1;
    result3 += M4(-0.02412411, 0.06506298, -0.09597504, 0.08727361, -0.07826563, -0.04138056, -0.0329931, 0.0049532577, 0.18710504, 0.13443553, -0.016784042, 0.3252642, 0.10901736, -0.062124077, -0.105086096, -0.11348829) * inp_0_0_2;
    result3 += M4(-0.11715005, 0.24422112, -0.010583985, -1.0774328, -0.1377914, -0.24615893, 0.10439729, -0.630682, -0.12502792, 0.07407715, -0.17429554, 0.4342548, -0.2744555, 0.37357023, 0.21370131, 0.21946116) * inp_0_1_2;
    result3 += M4(-0.14704126, -0.04841192, 0.0054409164, 0.42192286, -0.16239928, -0.12256808, -0.48227963, -0.56199294, -0.0077988375, -0.058350872, 0.059549462, 0.08112971, 0.016866077, -0.28199616, -0.045939445, -0.6000899) * inp_0_2_2;
    result4 += M4(0.051578436, 0.07574513, -0.08311514, -0.094506904, -0.019351086, -0.03994135, -0.2013505, -0.053737782, -0.10644847, -0.03716652, -0.006037517, -0.20759198, -0.065955274, -0.025639355, -0.03108868, -0.15957783) * inp_0_0_0;
    result4 += M4(-0.53227395, -0.13629952, 0.2736052, -0.25172985, 0.08843544, 0.083739504, -0.14896639, 0.032342453, 0.30325982, 0.22412667, 0.13294363, -0.09684498, 0.0091300635, 0.049917612, -0.27595112, -0.46310586) * inp_0_1_0;
    result4 += M4(-0.08246623, 0.22539097, -0.18912448, -0.1506009, 0.10359829, 0.12979895, -0.1193208, -0.11311012, -0.109849535, -0.10251482, -0.16502436, -0.048768505, -0.09813086, -0.15892366, 0.11454424, 0.038563825) * inp_0_2_0;
    result4 += M4(0.13934642, 0.18767363, -0.21737267, -0.14486428, 0.050037533, -0.11857233, -0.16436286, -0.03071755, 0.067969665, -0.25237224, -0.31437436, -0.09519086, 0.26249877, 0.26575777, 0.20926876, 0.15572454) * inp_0_0_1;
    result4 += M4(0.6181133, -0.29122418, -0.39185715, 0.4211404, 0.65378946, -0.45409536, 0.21033178, 0.6652133, -0.6224419, -0.44339785, -0.06966636, -0.27178282, -0.006403406, 0.18056141, -0.06178252, -0.020421157) * inp_0_1_1;
    result4 += M4(0.15071511, 0.24413487, 0.11548391, 0.19447832, 0.17463346, -0.06845565, 0.46572196, 0.27206087, -0.18558219, -0.031891245, 0.06227005, -0.25497243, 0.15413208, 0.1663793, 0.060110576, -0.12892863) * inp_0_2_1;
    result4 += M4(0.019566502, 0.13914707, -0.12418238, -0.032367185, 0.13262579, 0.027110627, -0.053045515, -0.12721047, 0.07364262, -0.1379143, -0.07026942, 0.20897914, 0.1040305, -0.24378213, -0.21751994, 0.030042721) * inp_0_0_2;
    result4 += M4(0.09825588, 0.050829872, 0.057765618, -0.1962459, 0.17047186, -0.02630842, -0.3170439, -0.7128352, -0.100038394, -0.009604496, -0.01683701, 0.61475205, 0.24394943, -0.17326237, -0.35041326, -0.0626887) * inp_0_1_2;
    result4 += M4(-0.0023033654, 0.15569572, 0.050712336, 0.22178504, 0.15502411, -0.12575664, -0.5353691, -0.55078924, 0.1646591, 0.085028306, -0.12881431, 0.11348627, -0.11900339, -0.23818937, -0.11942278, 0.105134726) * inp_0_2_2;
    result5 += M4(-0.14997327, -0.06980481, -0.028106444, -0.32166994, 0.0244939, 0.008450694, 0.046827514, -0.18954663, 0.01180598, 0.039024193, 0.14463262, -0.022342123, 0.16165036, -0.20086254, 0.06147782, 0.15501009) * inp_0_0_0;
    result5 += M4(-0.208427, -0.15796761, -0.12347807, -0.060295492, 0.0009418268, 0.053384002, 0.12555958, -0.056766313, 0.20688206, 0.07565946, 0.08528115, 0.1175976, 0.37646604, -0.025328865, 0.031827424, -0.0475272) * inp_0_1_0;
    result5 += M4(0.022018984, -0.34357664, -0.026507452, 0.094443515, 0.060284253, -0.09453106, 0.027817503, 0.13280866, 0.0362862, -0.19009824, -0.12423299, 0.15588343, 0.122105286, -0.15626147, -0.00889686, 0.20112608) * inp_0_2_0;
    result5 += M4(-0.046881065, 0.03865134, -0.018465962, 0.02575918, 0.22545804, 0.07873016, 0.19399649, 0.06123808, -0.25913975, -0.009877298, 0.03822979, -0.08589325, -0.06742487, -0.1008289, 0.23061478, 0.04478128) * inp_0_0_1;
    result5 += M4(0.07474331, 0.3117393, -0.1382288, -0.022682207, 0.14929745, 0.062428247, 0.4564845, 0.55250424, 0.06843344, -0.3696271, -0.17478529, -0.006444657, 0.1739998, -0.23692377, -0.37622988, -0.32394004) * inp_0_1_1;
    result5 += M4(0.0017754779, -0.22095567, -0.099588886, -0.30565014, 0.104977205, 0.44621128, 0.37087926, 0.2151806, 0.16770954, 0.15206501, 0.29215497, 0.078081794, -0.19135815, -0.057762515, 0.27080256, -0.03329196) * inp_0_2_1;
    result5 += M4(-0.10398913, -0.02665453, -0.09642801, -0.071431115, 0.11944199, -0.046118636, 0.08967419, 0.084092245, -0.09077823, 0.08581383, 0.06021696, 0.03785943, -0.34814262, 0.20153473, -0.11988814, -0.18357046) * inp_0_0_2;
    result5 += M4(0.120408304, -0.11875304, -0.10245665, 0.21781394, 0.025636679, -0.07166581, 0.081462376, 0.018147958, -0.036678083, 0.1287652, 0.06155769, 0.048486985, -0.046877004, -0.054091457, 0.34121743, 0.14181603) * inp_0_1_2;
    result5 += M4(0.0906816, 0.012704579, -0.06962936, 0.10743448, -0.14665279, -0.09961186, 0.023156922, 0.10670278, -0.12607108, 0.06588211, -0.16156262, -0.1110537, -0.22221115, 0.084400654, -0.05130934, -0.03003883) * inp_0_2_2;
    result6 += M4(-0.060134895, 0.24275818, 0.013915809, -0.08774872, -0.032742865, -0.03754062, -0.03666199, 0.018492388, -0.03083886, 0.0061663403, 0.12645458, -0.091533616, 0.049775604, 0.14465092, -0.069020346, 0.040911175) * inp_0_0_0;
    result6 += M4(-0.23872909, 0.19259226, -1.3096836, 0.08945853, 0.23159067, -0.10926726, -0.02921433, 0.1735388, -0.034358736, -0.07720809, -0.4821198, 0.12160775, -0.065654285, -0.3473123, -0.020171935, 0.009479732) * inp_0_1_0;
    result6 += M4(0.05415128, 0.060793795, 0.38044477, 0.035942573, 0.21186782, -0.18398024, 0.0009035082, -0.04827189, 0.03792917, 0.041796517, 0.034779433, -0.014232694, -0.07253886, -0.29967755, -0.110211864, 0.073690824) * inp_0_2_0;
    result6 += M4(0.0041582054, 0.1696217, 0.03872226, 0.061905973, 0.14727533, -0.22743338, -0.07855562, 0.16576177, 0.1674678, -0.11143166, -0.32170466, 0.15631011, -0.023789266, -0.41345334, 0.005014694, -0.20505501) * inp_0_0_1;
    result6 += M4(0.05722912, 0.009338215, 0.49071443, -0.020937884, -0.058695983, -1.0040601, -1.7555957, 0.49436823, 0.13498205, -0.2694163, 0.42576233, -0.49092004, -0.105117895, 0.13662043, 0.4321999, 0.038127214) * inp_0_1_1;
    result6 += M4(-0.39502206, -0.0745632, 0.23345551, -0.04999663, -0.42385614, -0.9459572, -1.3408185, 0.04012115, 0.16547497, -0.09851783, -0.21666308, 0.10097397, -0.1396194, -0.13890637, -0.0074272878, -0.06812618) * inp_0_2_1;
    result6 += M4(0.04049092, 0.32840717, -0.04483328, -0.011559661, 0.124419525, 0.038339965, 0.048310343, -0.024731645, -0.14790705, 0.024609316, 0.03954532, 0.019230729, 0.08201691, 0.14252406, -0.11814448, -0.0314584) * inp_0_0_2;
    result6 += M4(-0.03879589, 0.005139642, 0.2185021, -0.04354112, -0.0062207934, -0.08478937, -0.1776041, 0.12576947, 0.2770598, -0.012296981, -0.09585624, 0.09447126, -0.39242116, 0.08549089, -0.29511896, 0.11904524) * inp_0_1_2;
    result6 += M4(0.1498495, -0.14070356, 0.051491316, -0.006162745, -0.06612548, 0.28046063, 0.21322241, -0.15526669, -0.04203823, -0.2865232, 0.06960004, -0.093992226, 0.07003407, -0.25644127, 0.15850401, -0.05388009) * inp_0_2_2;
    result7 += M4(-0.121405534, 0.2916505, 0.1805825, -0.04437227, -0.026956832, -0.14585419, 0.121042766, -0.00012448317, -0.01800123, -0.13222776, -0.071801074, -0.009994229, -0.011191383, 0.039320875, 0.054214634, 0.0069511873) * inp_0_0_0;
    result7 += M4(0.13742776, -0.7063386, -0.24161735, 0.041927874, 0.07259971, 0.088132784, 0.14745742, -0.00024654204, 0.024967194, 0.38807395, 0.10283244, -0.002414409, -0.23190099, 0.3973448, -0.49274817, -0.023598276) * inp_0_1_0;
    result7 += M4(-0.09878754, -0.5183087, -0.262513, 0.01284452, 0.19958784, -0.121381834, 0.24008091, -0.05575447, 0.20475712, -0.17839934, -0.08760423, -0.009613468, -0.34880796, -0.01554077, 0.07163831, 0.065240376) * inp_0_2_0;
    result7 += M4(0.07693932, -0.35863015, 0.20696531, -0.027942404, 0.011441366, -0.10543338, -0.06389097, -0.030164393, 0.07324347, -0.012505472, -0.14143002, 0.049535528, -0.040912192, 0.045394152, -0.15441445, -0.09510011) * inp_0_0_1;
    result7 += M4(-0.11735015, -0.020255547, 0.024125246, -0.008255681, 0.04462236, 0.32395142, -0.015363923, 0.019466331, 0.06546691, -0.06459475, 0.050150372, -0.111243196, 0.31057653, -0.4080679, -0.22201952, 0.058596637) * inp_0_1_1;
    result7 += M4(0.028458685, -0.007626273, -0.21209307, 0.13224135, 0.041978784, 0.033270273, -0.06381724, 0.022018204, -0.20799974, -0.0058293967, 0.28134668, 0.019717593, 0.05060534, 0.20704953, 0.07280354, -0.065508075) * inp_0_2_1;
    result7 += M4(0.029504627, 0.2997532, -0.11265008, -0.02196281, 0.040289205, -0.07515934, -0.18935001, -0.014382201, 0.016446538, 0.09518028, 0.3552055, -0.03751666, 0.071736984, 0.6015264, 0.33192778, 0.015597527) * inp_0_0_2;
    result7 += M4(0.023311362, 0.31012943, -0.08344107, 0.046399213, 0.039307207, 0.08920801, 0.11723802, -0.014512544, 0.053820882, -0.22652088, -0.19759287, 0.023979917, -0.10786207, -0.36206076, 0.22450048, -0.16652621) * inp_0_1_2;
    result7 += M4(0.06935851, 0.23741809, -0.11355458, 0.00900168, -0.051952366, -0.13550861, 0.069240764, 0.09139269, 0.032432996, 0.058309004, -0.18527375, 0.031698212, -0.022370804, 0.00423138, -0.40373525, -0.027636005) * inp_0_2_2;
    const V4 inp_1_0_0 = inp[1][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_1_1_0 = inp[1][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_1_2_0 = inp[1][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_1_0_1 = inp[1][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_1_1_1 = inp[1][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_1_2_1 = inp[1][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_1_0_2 = inp[1][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_1_1_2 = inp[1][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_1_2_2 = inp[1][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.015705837, -0.08757259, -0.14574689, -0.09847139, -0.023988131, 0.029351102, -0.046173625, 0.19792497, 0.033339083, 0.06608545, -0.04684849, 0.18297032, 0.08758138, -0.038168628, -0.3811316, -0.15500279) * inp_1_0_0;
    result0 += M4(0.0327847, 0.08222872, -0.13625719, 0.07147411, -0.08297042, -0.34255463, -0.1262595, 0.25465968, -0.049948506, -0.11586507, -0.07628335, 0.011634714, 0.06761774, -1.0289401, -0.34447238, -1.9434855) * inp_1_1_0;
    result0 += M4(0.021834414, -0.013230594, -0.13641895, 0.01749121, -0.038680516, -0.11757919, 0.25313693, -0.31679016, -0.03152502, -0.13610989, -0.0187181, 0.25227174, -0.07321712, -0.056769315, 0.051314134, -0.060345) * inp_1_2_0;
    result0 += M4(0.1455386, -0.23165657, -0.027012661, 0.24051656, -0.0052298056, -0.084585905, -0.11842649, -0.11692568, 0.016843049, -0.041452315, -0.099465385, 0.22225513, -0.013818037, -3.4995494, 0.059732933, -0.15444201) * inp_1_0_1;
    result0 += M4(0.015176403, 0.2602909, -0.005789302, 0.21753718, -0.053968143, -0.2956733, -0.07757809, -0.32551628, -0.08824722, 0.482287, 0.24214083, -0.18114735, -0.109986715, -0.04563639, 0.1414004, 0.025047071) * inp_1_1_1;
    result0 += M4(-0.09138021, 0.05086565, -0.18625364, 0.15465932, 0.27898037, 0.50582874, -0.18266322, 0.7983667, 0.088135615, -0.00065260223, 0.031881798, 0.0038900354, -0.005494149, 0.13334584, 0.039374895, -0.1610047) * inp_1_2_1;
    result0 += M4(-0.028009122, 0.70917916, -0.20590565, -0.08250265, 0.034151994, -0.11686015, -0.088408396, -0.031532317, -0.07842866, -0.20779848, -0.12661883, 0.106488205, 0.024001857, 0.029276377, 0.04194033, -0.19492286) * inp_1_0_2;
    result0 += M4(-0.0807525, -0.41786858, 0.2772513, -1.9408312, -0.041749127, 0.07736463, 0.05840257, 0.018336438, -0.052833375, 0.29941913, 0.08520161, 0.10993205, -0.011482427, 0.043650422, -0.06380803, 0.25706142) * inp_1_1_2;
    result0 += M4(-0.050888583, 0.06494775, -0.11846093, -0.20360398, -0.009435878, -0.034057412, 0.10112421, -0.3560057, 0.10902857, 0.009935191, 0.04283132, 0.098031245, -0.035576344, -0.03662077, 0.10760328, -0.011724304) * inp_1_2_2;
    result1 += M4(0.17904142, 0.17865832, -0.033631448, -0.03777405, 0.07747217, -1.144423, -0.23656368, 0.077440985, -0.1752677, -0.11102885, -0.15037268, 0.36026397, 0.13219756, 0.23780695, -0.599006, 0.116624415) * inp_1_0_0;
    result1 += M4(0.017037537, -0.021105953, -0.00625365, -0.23848307, 0.26520988, -2.1479464, 0.061651263, -0.17636372, -0.0672601, 0.038131606, 0.110582694, 0.17954414, -0.21061431, -0.10809257, 0.2374416, 0.13028806) * inp_1_1_0;
    result1 += M4(-0.036073778, -0.05137396, -0.04880065, -0.07313776, -0.39852795, -0.0815146, -0.03949898, -0.15319917, -0.108371444, -0.07237653, 0.28858316, -0.35015506, 0.07656148, -0.01535037, 0.035620935, 0.17655964) * inp_1_2_0;
    result1 += M4(-0.54675287, 0.07529785, 0.074475795, -0.39869455, 0.20269322, 0.0719645, -0.15317307, -0.0800666, 0.22568841, -0.15209183, -0.022256179, 0.02257004, -0.3594915, 0.032393783, 0.26705188, 0.19439702) * inp_1_0_1;
    result1 += M4(-0.22772443, 0.022063656, 0.28183955, -0.01662453, 0.10686233, -0.32154772, 0.43590322, -0.049001478, 0.23697475, 0.15261252, -0.23175651, 0.1279099, -0.1764752, 0.0009334951, 0.24732122, 0.012949229) * inp_1_1_1;
    result1 += M4(-0.039610304, -0.010955573, -0.07794968, -0.171994, -0.10630965, 0.08293568, -0.11554044, -0.042115677, -0.112088956, -0.007499805, -0.016719189, -0.102588825, 0.0038386846, -0.026042204, 0.031807512, 0.25795093) * inp_1_2_1;
    result1 += M4(0.34312338, 0.1400166, -0.12588081, 0.099729784, 0.050401695, -0.020207305, -0.19152634, 0.015256083, -0.3766958, -0.20928502, -0.11923857, -0.16830494, 0.025611058, -0.0860877, 0.14053613, -0.03784331) * inp_1_0_2;
    result1 += M4(-0.10721079, -0.17891993, 0.31198898, -0.08903273, 0.2033635, 0.15018938, -0.23214741, -0.057415232, 0.16173063, 0.3114448, -0.38873303, -0.31833035, 0.030104188, -0.039745025, 0.048106734, -0.0072707455) * inp_1_1_2;
    result1 += M4(0.08475903, -0.0280167, 0.074594304, -0.14573091, -0.076486304, -0.088766046, 0.23225485, 0.018714916, -0.08139355, 0.056768987, -0.20290084, 0.22217944, -0.030439394, -0.0061971555, -0.014558601, 0.14819351) * inp_1_2_2;
    result2 += M4(-0.05678322, 0.038044047, -0.10878136, -0.022294987, -0.08334206, -0.03514968, -0.11100947, 0.018132476, -0.059759263, -0.016478887, 0.0044232304, 0.012782603, 0.20971744, 0.12951162, -0.2363734, 0.012111549) * inp_1_0_0;
    result2 += M4(-0.0047935, -0.16443412, 0.04636891, -0.08428936, -0.031667873, 0.10029496, -0.060145393, -0.009785381, 0.019986873, 0.13367203, -0.08843032, -0.08884685, 0.2048423, 0.2682516, -0.41760838, 0.007188837) * inp_1_1_0;
    result2 += M4(0.034791045, 0.06345544, -0.030539997, 0.012671537, 0.037223697, 0.033800922, 0.022833338, -0.01756363, -0.06537067, -0.011873557, -0.045356646, -0.09174092, 0.07182381, -0.013059471, -0.014961245, -0.18113816) * inp_1_2_0;
    result2 += M4(-0.051105045, 0.31895396, -3.1281343, 0.09133826, 0.18091266, -0.071542054, -0.034413975, -0.08605321, 0.09372991, -0.08368217, 0.03485669, -0.08150752, -0.080002636, 0.29741517, -0.8460002, -0.110172525) * inp_1_0_1;
    result2 += M4(0.15012856, 0.111757174, 0.11197605, -0.1841054, -0.114388764, 0.2670422, -0.06094665, -0.2639575, -0.18993965, -0.18473348, -0.05369458, -0.02939443, 0.09832141, 0.060799006, -0.06970623, -0.12318612) * inp_1_1_1;
    result2 += M4(0.10683464, 0.043307904, 0.06957554, 0.018783206, -0.12585194, -0.34254667, 0.31038046, 0.07785982, -0.101351, -0.032452278, -0.033618387, -0.027075376, 0.18947065, -0.14466545, -0.04470224, 0.0804046) * inp_1_2_1;
    result2 += M4(0.20479347, -0.07695956, -0.038807284, 0.14832774, 0.016658537, 0.016239636, -0.0152274035, 0.031426866, -0.004036695, 0.08464745, -0.071680516, -0.08907702, -0.037239, -0.083942674, -0.0656084, -0.03068332) * inp_1_0_2;
    result2 += M4(0.1817529, 0.5861253, -0.5573075, -0.49133024, -0.21304806, 0.14515616, -0.06643737, 0.04259107, 0.056184392, -0.7713701, 0.7665931, 0.11472885, -0.09939027, -0.07287062, 0.037568703, 0.010686666) * inp_1_1_2;
    result2 += M4(0.017431695, -0.08583485, -0.024563128, -0.08138473, 0.049856175, 0.081742816, 0.028083729, -0.065018386, 0.2870687, 0.038949415, 0.09087868, 0.35945094, 0.038861737, 0.019348554, -0.028804429, 0.070894755) * inp_1_2_2;
    result3 += M4(-0.26293215, 0.15456338, -0.09825861, -0.012118151, -0.14889264, -0.016729752, 0.081888154, -0.062640876, 0.34197384, 0.05361197, -0.055235732, -0.027815193, 0.35039487, -0.00048481033, 0.079942025, 0.23597965) * inp_1_0_0;
    result3 += M4(-0.16053268, 0.018971741, -0.20817347, -0.053127363, 0.10220925, -0.007842532, -0.1505869, 0.0914405, 0.21236458, 0.1309726, 0.079182535, 0.011915658, 0.61310154, -0.063002214, 0.16931431, 0.46861228) * inp_1_1_0;
    result3 += M4(0.12153794, -0.030432468, -0.006426738, 0.02428633, 0.24614468, -0.01828448, 0.018496696, 0.07758397, -0.14118606, 0.17314586, -0.04206767, 0.1250493, -0.058316987, -0.03486429, -0.028537309, 0.15987615) * inp_1_2_0;
    result3 += M4(-0.6072018, -0.03562055, 0.005235395, 0.34884167, -0.120981075, 0.04974372, -0.023701642, 0.09109101, 0.12505068, -0.021835998, -0.31228593, -0.21650532, 0.45668122, -0.2932674, 0.0016263924, -0.63253015) * inp_1_0_1;
    result3 += M4(-0.5531097, -0.0072509223, -0.17426589, 0.23104966, 0.21455364, 0.02814596, -0.03432074, -0.055913646, -0.061857205, 0.29164517, 0.2027527, -0.08697645, 0.07952781, 0.051526584, -0.13036363, -0.745042) * inp_1_1_1;
    result3 += M4(-0.2233578, -0.11440489, 0.029832406, -0.017601004, -0.16498294, 0.011793296, 0.082185395, -0.23161632, 0.121922575, 0.122292355, -0.19368723, -0.12142365, -0.03395147, -0.04186565, 0.0013574116, 0.16070162) * inp_1_2_1;
    result3 += M4(-0.05194804, -0.06577343, 0.052129745, 0.0023471273, 0.113537356, 0.02070727, -0.0069119525, 0.0054871617, 0.12847832, 0.07010345, -0.1892681, -0.14944568, 0.041468628, -0.045712143, 0.031387586, -0.5985787) * inp_1_0_2;
    result3 += M4(-0.20239231, -0.029962799, -0.05429951, 0.054766268, -0.037008047, 0.08904205, 0.0053576906, 0.034375798, -0.16764103, 0.45312268, -0.14036322, 0.63783115, -0.011320644, 0.015863223, -0.08146567, 0.2654658) * inp_1_1_2;
    result3 += M4(-0.1433075, -0.122361325, -0.0283212, 0.17461006, 0.18454675, -0.01767006, -0.09923615, 0.12920131, 0.34683147, 0.11826358, -0.027551806, -0.44317648, 0.12936379, 0.05039415, -0.027836207, -0.049108125) * inp_1_2_2;
    result4 += M4(0.082695924, 0.18121375, -0.07813922, 0.024336588, 0.06685882, 0.003593495, 0.114723034, 0.0025278965, -0.07381527, -0.036806427, 0.061976958, 0.14275831, 0.1412301, -0.020041253, 0.124992244, 0.30799243) * inp_1_0_0;
    result4 += M4(-0.027877063, 0.14446527, -0.018880904, -0.058292765, -0.05062022, -0.00018398653, -0.0470546, 0.09174476, -0.05090109, 0.056482, 0.024571292, -0.070263304, 0.29965645, -0.103370614, 0.32149836, 0.43295103) * inp_1_1_0;
    result4 += M4(0.0012528916, -0.049018558, 0.060594134, 0.012947512, 0.22603084, 0.07291147, 0.1159571, -0.19781348, -0.05958744, 0.379132, 0.12449491, 0.01270974, 0.021333795, -0.10528641, 0.011877262, 0.116228715) * inp_1_2_0;
    result4 += M4(0.34879473, 0.20936602, 0.24925159, 0.35849598, 0.122393675, -0.09732403, -0.15884364, 0.10804043, -0.020815928, -0.2427618, -0.21710189, -0.22250012, 0.174534, 0.0016760475, 0.12274344, 0.04098191) * inp_1_0_1;
    result4 += M4(0.13151297, 0.31066614, -0.2980211, 0.15315168, -0.074350506, 0.1459507, -0.042794634, 0.20930867, -0.17019382, 0.11552082, 0.20658825, -0.3305322, 0.15435448, -0.2749351, 0.07736303, 0.050429396) * inp_1_1_1;
    result4 += M4(-0.0472628, -0.031221708, -0.08600894, 0.023358526, -0.23963419, -0.20649116, -0.32379407, -0.21985549, -0.011182047, -0.098406725, -0.1557199, -0.11395769, 0.02815377, 0.055262674, 0.1328955, -0.019392012) * inp_1_2_1;
    result4 += M4(-0.056163255, -0.17809828, -0.2660559, -0.9267227, -0.058814917, 0.07896105, 0.18913375, 0.001325836, 0.2020648, -0.026079683, -0.21554257, -0.011550446, 0.024500657, -0.13475531, 0.0063152383, -0.29406548) * inp_1_0_2;
    result4 += M4(0.31022418, 0.23374051, 0.24914673, -0.099041626, 0.04631335, -0.24654107, -0.0042301645, -0.08926683, -0.72896755, -0.0058468543, 0.23755509, -0.47754082, -0.10225431, 0.15831073, -0.04574231, 0.007742179) * inp_1_1_2;
    result4 += M4(-0.038696084, 0.098414935, 0.091991685, -0.12147439, -0.044761106, 0.012964146, -0.14041997, -0.1488653, -0.097019486, -0.3028561, 0.0024245072, 0.0073835044, -0.029183332, 0.054050915, 0.018883185, -0.06996722) * inp_1_2_2;
    result5 += M4(0.012797014, -0.07771348, -0.020310335, -0.19170329, 0.18033248, -0.057783455, 0.039009202, 0.056983463, 0.13019794, 0.0768349, -0.013962999, -0.029045915, -0.027403252, -0.083993316, -0.07214096, 0.17893472) * inp_1_0_0;
    result5 += M4(-0.09019733, -0.09761272, -0.0018465327, 0.24140842, 0.0069835973, -0.2723055, -0.27888083, 0.18178602, -0.018096542, 0.14779197, -0.15614633, -0.1795719, -0.06060851, 0.53950465, 0.14872037, -0.2953805) * inp_1_1_0;
    result5 += M4(-0.0069469805, 0.00072726177, 0.030260839, 0.31199986, -0.077464394, 0.17491801, 0.084831245, -0.30777273, 0.13837875, 0.093439646, 0.0072422437, -0.028482957, -0.13717517, 0.07496953, -0.05841122, -0.38958743) * inp_1_2_0;
    result5 += M4(0.25842395, -0.0534104, 0.19725204, 0.5732922, -0.014739101, 0.011541628, -0.06762654, -0.07838946, -0.089392625, 0.078205906, -0.058084402, -0.31652772, 0.22751471, 0.1649566, -0.0034433668, -0.1108457) * inp_1_0_1;
    result5 += M4(-0.00040962943, 0.114395276, -0.08093822, 0.07925262, 0.06480871, 0.031564727, 0.09912229, 0.0033447568, -0.22824617, 0.19836572, -0.25685012, 0.014842591, 0.0791563, 0.13164817, 0.14517978, -0.5211333) * inp_1_1_1;
    result5 += M4(0.02455982, 0.012626938, -0.02414609, -0.25878477, -0.2985454, 0.111890055, 0.16922519, 0.16842273, -0.037140988, -0.12725596, -0.06281814, 0.15967487, 0.0057370323, 0.028131358, -0.09256715, -0.107456155) * inp_1_2_1;
    result5 += M4(-0.15736467, -0.0009949561, -0.15410614, 0.4454036, -0.02370601, 0.07257308, -0.03688127, -0.1203092, -0.119175, -0.1383757, 0.025429105, -0.073395364, 0.089515716, -0.047273073, 0.018875748, -0.043639444) * inp_1_0_2;
    result5 += M4(-0.27092332, 0.54201716, 0.13550659, 0.5140095, 0.034709737, -0.17743476, -0.09467708, 0.15228797, 0.043639418, -0.53414416, -0.37077555, 0.03201087, -0.051819697, -0.014487151, 0.0871122, -0.07787741) * inp_1_1_2;
    result5 += M4(0.09538161, -0.07099058, 0.043127216, -0.06860144, 0.15255186, 0.06702592, -0.09096986, 0.3046965, 0.0072517204, -0.0071811816, -0.07311316, 0.06473865, -0.05660396, -0.006702589, 0.0025521629, -0.0227959) * inp_1_2_2;
    result6 += M4(0.11393808, 0.2529656, 0.065638795, -0.0070618526, -0.069807224, 0.112068705, 0.06027538, 0.050825346, -0.043259904, -0.30898684, -0.023240106, 0.13450065, 0.2946264, -0.8799266, 0.28587252, -0.0019469464) * inp_1_0_0;
    result6 += M4(0.07097902, 0.0083183665, 0.073096626, 0.0039739283, 0.0036708557, -0.046524182, -0.074891865, 0.034804992, -0.16507661, 0.1771701, -0.055224244, 0.024664093, 0.123780064, -0.5328062, -1.1285102, 0.024417344) * inp_1_1_0;
    result6 += M4(0.02348762, 0.060864087, -0.014495935, -0.05054686, -0.088916406, -0.2207449, -0.2604791, 0.07249417, -0.045275997, 0.22939529, -0.047227457, 0.054593455, -0.019649668, -0.028459009, 0.1555842, -0.05769583) * inp_1_2_0;
    result6 += M4(0.07943063, -0.011858097, -0.58831656, 0.3276619, -0.07183317, -0.06892626, 0.054699942, 0.10584638, -0.16386367, 0.05900643, 0.08891144, -0.09862264, -0.25728536, -1.7810183, -0.3991901, 0.09335364) * inp_1_0_1;
    result6 += M4(0.04374285, 0.07729262, -0.042925317, -0.15216136, 0.08119837, 0.11471808, -0.4111256, -0.3011707, -0.05618382, 0.13557819, 0.07417684, 0.08916184, -0.08686528, 0.5100407, -0.26479897, -0.13543157) * inp_1_1_1;
    result6 += M4(0.064035796, 0.22294371, -0.026144844, -0.012370568, -0.16471474, 0.18938921, 0.5072837, 0.044579472, -0.14110321, -0.14616847, 0.18638737, 0.071257755, 0.011235274, -0.21993187, 0.009487769, 0.011316428) * inp_1_2_1;
    result6 += M4(-0.111767165, 0.35307768, 0.05786975, 0.030740857, -0.026383976, -0.19180562, 0.08850825, 0.018674716, -0.16321838, -0.48468983, -0.02754574, -0.030066885, 0.028040212, -0.12585014, 0.014031982, 0.045476582) * inp_1_0_2;
    result6 += M4(-0.05825943, 0.21480384, -1.5148319, 0.038128316, -0.12032085, 0.033705167, -0.025855822, -0.051541958, 0.12242965, -0.13969485, 1.5401154, -0.14226605, 0.046468828, 0.15892296, -0.013732848, -0.050481208) * inp_1_1_2;
    result6 += M4(-0.027205475, 0.25449505, 0.063216776, 0.014288834, -0.002127116, -0.13463655, -0.28212294, 0.104870185, -0.15161097, -0.19408913, 0.11953995, 0.086370304, -0.0336607, -0.062245227, -0.0045944657, 0.010266731) * inp_1_2_2;
    result7 += M4(0.0054832473, 0.12397913, 0.52314, -0.015598668, -0.091549, -0.025379725, -0.087624, 0.03574337, -0.13313192, -0.22816707, 0.056946564, 0.0093567185, 0.056934517, 0.16457592, 0.20037545, 0.01894461) * inp_1_0_0;
    result7 += M4(0.083990075, -0.022622146, 0.30039173, 0.088137805, 0.13066281, 0.29995215, 0.06275643, 0.02803964, -0.011623869, 0.058638208, -0.29823378, 0.012866384, -0.23301665, -0.17337266, 0.21031748, 0.02983413) * inp_1_1_0;
    result7 += M4(0.047045156, 0.036918767, -0.18782184, -0.047904037, -0.11728977, -0.26090726, -0.01803386, -0.05004084, -0.1196135, -0.006118971, -0.3201644, 0.06070806, 0.12671301, -0.08261868, 0.2887077, 0.0022361192) * inp_1_2_0;
    result7 += M4(-0.11459005, -0.22634235, -0.38108867, -0.06883171, 0.020694587, -0.052187312, -0.07569177, 0.010613253, -0.049231004, 0.03802635, -0.23017557, 0.028105024, -0.121288516, -0.069808796, 0.25114962, -0.012357365) * inp_1_0_1;
    result7 += M4(0.23108453, -0.2715041, -0.265973, 0.12563238, -0.08548222, 0.20210718, 0.07355029, -0.05776692, -0.03451845, 0.061854575, -0.3626237, -0.13277398, -0.050674524, -0.23682791, 0.4599022, 0.15432605) * inp_1_1_1;
    result7 += M4(0.015274087, -0.06916282, -0.16906759, -0.041118383, 0.113673046, -0.2715101, -0.18171956, 0.10902213, 0.021381473, -0.17755006, 0.048233666, 0.025476575, 0.063703835, -0.019278118, 0.23344864, 0.030485932) * inp_1_2_1;
    result7 += M4(0.13956727, -0.1200502, 0.28882644, 0.12324514, -0.09214347, 0.0661035, 0.12243416, 0.035143603, -0.014788869, 0.0050253645, -0.023668807, -0.060137566, -0.022511953, 0.08062004, 0.035053767, 0.012718093) * inp_1_0_2;
    result7 += M4(-0.022914244, 0.1214574, 0.27899665, 0.081318825, 0.06345362, -0.17690541, -0.087174356, -0.016199911, 0.21319334, -0.25621173, -0.25558436, -0.18953758, -0.020559471, 0.117694125, 0.2931878, 0.0015711633) * inp_1_1_2;
    result7 += M4(-0.008901676, 0.07831701, 0.04923326, 0.05121629, -0.046791695, -0.050387975, -0.092312746, 0.041667443, 0.06991114, -0.13188252, -0.27425835, -0.20410411, -0.020966193, 0.020182911, 0.105824396, 0.024914948) * inp_1_2_2;
    const V4 inp_2_0_0 = inp[2][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_2_1_0 = inp[2][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_2_2_0 = inp[2][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_2_0_1 = inp[2][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_2_1_1 = inp[2][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_2_2_1 = inp[2][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_2_0_2 = inp[2][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_2_1_2 = inp[2][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_2_2_2 = inp[2][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.092814386, 0.45775026, 0.5213297, 0.094253145, 0.02716614, -0.051142722, -0.08423291, 0.22713146, 0.18396759, 0.16938366, 0.46597824, 0.03608177, -0.009445381, 0.08850721, -0.049548317, 0.19509225) * inp_2_0_0;
    result0 += M4(-0.07967626, -0.27377096, 0.21975087, 0.4130142, -0.22819003, 0.0120467255, 0.20632908, 0.15918821, 0.037399527, 0.2779597, 0.26177385, 0.12683025, -0.04431283, 0.010559499, -0.17927204, -0.21211462) * inp_2_1_0;
    result0 += M4(-0.018077357, -0.06923307, 0.18740946, 0.4000061, 0.11990999, 0.45005998, 0.16962993, 0.3677591, 0.0094687315, -0.08173164, 0.029807393, 0.17136762, 0.21774039, -0.02698106, -0.3311936, 0.17532001) * inp_2_2_0;
    result0 += M4(0.13815403, -0.23232403, 0.25905025, 0.045491744, 0.07092845, -0.09583576, -0.0049267225, -0.15663426, -0.033748027, 0.12650584, -0.3109377, 0.28992295, -0.065639116, -0.012369777, 0.072588176, 0.08215768) * inp_2_0_1;
    result0 += M4(-0.007286646, -0.26170236, -0.30918992, 1.171271, 0.048696686, 0.29065692, 0.17045677, -1.2565033, 0.21383715, 0.30538064, -0.4050226, -0.47768462, 0.036891066, -0.13155185, 0.14892122, -0.94513774) * inp_2_1_1;
    result0 += M4(0.028870897, 0.08307629, -0.03316573, -0.10639322, -0.009287276, 0.3113228, 0.033080664, -0.037382085, 0.09654543, 0.06484134, -0.024523646, -0.07536409, 0.070641145, -0.19949584, 0.07394696, 0.35262474) * inp_2_2_1;
    result0 += M4(0.06408775, -0.11552507, -0.20478627, 0.1720749, 0.02228801, 0.06467018, -0.03362949, 0.04949373, 0.03943748, -0.08520571, -0.005543114, 0.14626193, 0.056281988, -0.03185487, -0.030019294, -0.085060365) * inp_2_0_2;
    result0 += M4(-0.0068926113, -0.31103078, -0.2491062, 0.45975727, -0.029700063, -0.037300974, 0.006816582, -0.22653654, -0.044787616, 0.13761318, 0.074645236, -0.23527503, -0.04096568, -0.25615218, 0.017205942, 0.4311577) * inp_2_1_2;
    result0 += M4(0.003547339, 0.08266298, 0.03986908, 0.3195109, 0.023006028, 0.010019493, 0.02645517, 0.03382074, -0.033641502, 0.021117087, 0.028541887, 0.017701743, -0.00074843015, -0.032236867, 0.50645727, 0.22653513) * inp_2_2_2;
    result1 += M4(0.01271737, 0.16673093, 0.1304197, 0.17008604, -0.0041544265, 0.3317953, 0.10138445, 0.13974383, -0.07921419, -0.7096312, 0.11065393, 0.16823693, 0.19588614, -0.08559395, 0.09482096, 0.34688038) * inp_2_0_0;
    result1 += M4(-0.044896383, 0.11181919, -0.22621563, 0.6336831, 0.4514736, 0.20700884, -0.072991915, 0.36533028, 0.3968442, -0.09231701, -0.018255945, 0.3543057, 0.044412542, 0.14601502, 0.032464173, -0.13750124) * inp_2_1_0;
    result1 += M4(-0.11890325, 0.0042684767, 0.10406433, -0.1436936, 0.061364863, 0.12423821, -0.0062511642, -0.25572217, -0.07105848, -0.09414918, 0.02933354, -0.18185432, -0.040355146, -0.002426237, -0.038215667, -0.06798831) * inp_2_2_0;
    result1 += M4(0.054971218, -0.17049567, -0.07605726, 0.23885807, -0.19514397, 0.06684454, 0.6325262, -0.06224581, 0.44604573, -0.23705693, -0.7774546, -0.24402986, 0.25855255, 0.14096765, -0.008919355, -0.1320874) * inp_2_0_1;
    result1 += M4(0.020357002, 0.51980746, -0.12875433, -0.37162942, 0.14296156, 0.04101057, -0.6130841, 0.04047241, 0.11556824, 0.283298, -0.2793631, -0.5364963, 0.09338444, 0.43943235, 0.1580923, -0.37505242) * inp_2_1_1;
    result1 += M4(-0.2266215, -0.047155976, -0.0028070945, 0.14938706, -0.08381178, -0.0013132134, -0.15564638, -0.2594628, -0.0439063, -0.065787844, 0.016042007, -0.13615529, -0.18690327, 0.20067558, -0.023083013, 0.40131986) * inp_2_2_1;
    result1 += M4(0.13605005, 0.007468202, -0.3510975, 0.110887066, -0.08789011, 0.0030543825, 0.37340862, 0.096323386, -0.023372052, -0.069242835, 0.026198447, -0.0037856314, -0.09124733, -0.20670955, 0.14138468, -0.21397921) * inp_2_0_2;
    result1 += M4(0.014320685, 0.1760587, 0.28390995, -0.1906815, -0.03362059, -0.10963648, -0.2570818, 0.14267191, 0.09673585, -0.12670103, -0.051737715, 0.085166626, -0.23872021, 0.0013363118, 0.21796082, 0.16754279) * inp_2_1_2;
    result1 += M4(-0.003217061, 0.05932071, -0.027195085, 0.27737984, -0.078786746, 0.041419193, -0.0041403277, 0.1154432, -0.005215704, -0.05328401, 0.0145055, -0.032274768, 0.12889975, 0.0013915228, 0.3268021, -0.29600385) * inp_2_2_2;
    result2 += M4(-0.16919145, -0.02339556, -0.11975917, -0.18796107, -0.042660635, -0.03570311, 0.047033995, -0.0028603463, 0.20921953, -0.15312079, -0.1888245, -0.0014985288, -0.018516323, 0.009912187, 0.19263527, 0.08873788) * inp_2_0_0;
    result2 += M4(0.00077348424, -0.028518833, -0.24635765, -0.03085097, -0.11051066, 0.03238698, 0.20448959, -0.018847598, 0.052270662, -0.19716991, 0.02137329, 0.24775666, 0.20831864, -0.06477407, 0.23972619, 0.14619708) * inp_2_1_0;
    result2 += M4(0.038661838, 0.19771567, -0.11687237, 0.07174482, -0.21497092, -0.06277456, 0.046138883, -0.06283013, 0.07124631, 0.007211457, -0.03506558, 0.0108648, -0.028258258, 0.11748308, 0.27080202, -0.08909715) * inp_2_2_0;
    result2 += M4(0.07731482, 0.20997816, -0.27249014, -0.20684892, 0.10844123, 0.055094372, 0.20387343, -0.0031151853, 0.11789715, -0.0686395, 0.046183497, 0.178479, 0.06625437, 0.20198733, -0.26136285, -0.08981151) * inp_2_0_1;
    result2 += M4(0.059791584, -0.55750525, 0.28059787, -0.3612343, 0.23472503, -0.65881574, 1.1438445, 0.35744208, 0.24223387, -0.32694224, 0.14648949, 0.8517398, 0.39240992, -0.33069992, 0.26866063, 0.15360549) * inp_2_1_1;
    result2 += M4(0.006905369, 0.28093493, -0.08213076, 0.062518954, -0.2132455, -0.12476432, 0.15646341, -0.071651794, -0.116510496, 0.16372761, -0.10705616, 0.10234325, 0.024531865, 0.3159172, 0.24662466, -0.03653588) * inp_2_2_1;
    result2 += M4(0.1221305, 0.13054802, -0.19909106, -0.12288414, -0.06698235, 0.10403767, -0.04831825, -0.018168485, 0.093052365, 0.14017363, -0.06681159, -0.06713354, 0.06921825, -0.035092324, 0.08895623, 0.13141425) * inp_2_0_2;
    result2 += M4(0.10899397, 0.21499716, -0.10872111, -0.36112684, 0.0403932, 0.11608565, 0.07820332, 0.1057553, 0.43729296, 0.0059949313, -0.07552206, 0.1835575, -0.07887075, 0.032997545, -0.019468486, 0.23971234) * inp_2_1_2;
    result2 += M4(-0.030460041, 0.07400979, 0.060889073, 0.22651044, 0.058038082, 0.03944214, -0.0650531, -0.081139244, 0.057853602, -0.0043854695, 0.040977117, 0.019137824, 0.29864132, 0.36628076, 0.13938288, 0.28221056) * inp_2_2_2;
    result3 += M4(0.35057044, 0.41704062, -0.18633854, -0.19228657, 0.04863304, 0.05364491, 0.008130434, -0.06508881, -0.5605144, 0.2295082, 0.02987997, 0.12013276, 0.08743647, 0.052817184, 0.009540287, 0.044861164) * inp_2_0_0;
    result3 += M4(0.6104237, -0.01806297, 0.025413873, 0.093747534, 0.2467213, 0.20814359, -0.26460925, 0.07862994, -0.25892353, 0.323068, 0.16535452, 0.010552754, -0.21430996, 0.24023825, -0.04990915, 0.06784939) * inp_2_1_0;
    result3 += M4(-0.0043212194, 0.07734947, 0.20869793, 0.14546277, -0.056904547, 0.07650816, -0.057542082, -0.23042575, -0.14991172, 0.019991813, 0.2516446, 0.12995991, 0.08643111, -0.00232024, 0.24138024, 0.27056637) * inp_2_2_0;
    result3 += M4(0.23917398, 0.31868932, -0.23665906, 0.13857982, -0.069166064, -0.009178056, 0.09243896, 0.10003533, -0.31654632, -0.019642843, 0.17102908, 0.30429733, 0.034729987, -0.073896304, 0.14556403, 0.15035088) * inp_2_0_1;
    result3 += M4(0.01909095, 0.05969945, -0.08706392, -0.19247584, -0.23393081, 0.34489277, 0.03882381, 0.46344456, -0.2809744, 0.41047862, 0.40789706, -0.27195525, -0.3669562, 0.09561591, 0.30979422, 0.23649983) * inp_2_1_1;
    result3 += M4(-0.028059548, -0.06396764, 0.10921153, -0.085578576, 0.1805504, -0.21622822, 0.24655335, -0.006480939, 0.14485586, 0.0013918349, 0.17845869, 0.008581246, 0.3857949, 0.0077028954, 0.28461692, 0.28207305) * inp_2_2_1;
    result3 += M4(-0.08781292, 0.08669868, -0.09298323, 0.4947495, 0.1610637, 0.118081465, -0.105517834, 0.0038016848, 0.0706152, 0.0028811384, -0.022725618, -0.009645629, -0.14953792, 0.13007891, 0.1075943, -0.07291444) * inp_2_0_2;
    result3 += M4(-0.14808182, 0.15069667, -0.24724561, 0.5288433, 0.11696382, 0.09816954, -0.19744284, 0.66427195, -0.1562091, 0.23490001, -0.13828155, 0.20825599, 0.09186699, 0.061937075, 0.29784667, 0.236802) * inp_2_1_2;
    result3 += M4(0.2078037, -0.026363002, 0.007866613, 0.020387873, 0.030396068, -0.012094757, 0.08869978, 0.13163315, -0.08094259, 0.08741374, 0.17529637, -0.02323783, -0.31478184, -0.04930671, 0.4544759, 0.37276232) * inp_2_2_2;
    result4 += M4(0.1237698, -0.06752338, 0.120449774, -0.14075252, 0.04056094, 0.15100405, 0.09272401, -0.1068255, -0.108206704, -0.12568088, 0.12092696, -0.06066922, 0.06880299, 0.03238648, 0.116945386, 0.06602621) * inp_2_0_0;
    result4 += M4(0.2737429, -0.083921, 0.0775947, 0.035990354, -0.24114487, -0.15216346, -0.20307475, 0.0010366262, -0.10667483, -0.19863427, 0.045222092, 0.0053321742, -0.21304773, -0.10564251, 0.2030825, 0.0005819532) * inp_2_1_0;
    result4 += M4(0.030088596, -0.10025536, -0.0068705264, 0.1704636, 0.0975064, 0.1264652, -0.042413156, 0.10908221, -0.0075633125, 0.08920921, -0.039690457, 0.07013045, -0.04766408, 0.267434, 0.3873668, 0.2456326) * inp_2_2_0;
    result4 += M4(0.32508588, 0.075257234, -0.11027838, 0.5938974, -0.07724042, 0.2360122, 0.07768924, -0.2092755, -0.033473365, -0.027028535, 0.07884026, 0.17027193, -0.09563506, -0.02201325, 0.0758091, -0.017488876) * inp_2_0_1;
    result4 += M4(-0.42168355, -0.6893434, -0.1228165, -0.37461355, -0.36831233, -0.14153562, 0.05584087, -0.85905, -0.15624541, 0.063371524, 0.12268652, 0.2825505, -0.43282175, 0.04444286, 0.06592473, -0.12757285) * inp_2_1_1;
    result4 += M4(-0.065865524, 0.113776326, -0.092310116, -0.08256259, 0.028396951, -0.14138938, 0.0017393365, -0.18951187, 0.016880896, 0.011594533, 0.14634833, -0.121401295, -0.3309328, 0.030478504, 0.16880141, 0.19349241) * inp_2_2_1;
    result4 += M4(0.052204404, -0.023084637, 0.13934132, 0.14047483, -0.07000722, 0.13290952, -0.0029259236, 0.17653532, -0.0352503, -0.04417376, -0.07767893, 0.18506913, -0.26599443, -0.043042358, 0.113055445, 0.19514775) * inp_2_0_2;
    result4 += M4(-0.007204896, 0.022588506, 0.092830144, 0.26664358, 0.04046923, 0.21121469, 0.11176891, 0.5670653, -0.18089983, 0.10270144, 0.1717538, 0.18855186, -0.12254393, 0.23727734, 0.1113986, 0.35825285) * inp_2_1_2;
    result4 += M4(-0.20539555, 0.27943522, 0.09060818, -0.13886109, 0.07748372, -0.24344629, -0.013751766, 0.15736414, -0.018225847, 0.14815313, -0.005227884, 0.14037892, -0.3474196, 0.17567065, -0.08067104, -0.124556355) * inp_2_2_2;
    result5 += M4(0.45351768, -0.16863489, -0.14741778, 0.120527886, -0.0917752, -0.11598884, 0.06770275, 0.35887426, -0.051668353, -0.20950675, -0.03567117, 0.18930845, -0.18976416, 0.116286345, -0.11599318, -0.11221706) * inp_2_0_0;
    result5 += M4(-0.19201484, 0.07397011, -0.43736386, -0.34597123, 0.030007025, 0.08945358, 0.013453104, -0.014290466, 0.115271434, 0.002712154, -0.037602823, -0.30058998, -0.3394213, -0.01966308, -0.34935477, -0.230221) * inp_2_1_0;
    result5 += M4(-0.025867218, 0.15124387, -0.010126741, -0.25509304, -0.086762905, 0.22258978, -0.00045751603, 0.1910163, 0.020999506, 0.021633577, -0.04909525, 0.33909813, -0.2835458, 0.20334764, -0.1678883, 0.6459441) * inp_2_2_0;
    result5 += M4(0.111018226, 0.015392786, -0.047559522, 0.30912855, -0.027720116, -0.18915927, -0.039460532, -0.15800977, -0.015962185, -0.18576878, -0.08464735, -0.101604626, -0.051691834, -0.030772895, -0.19688055, 0.04886778) * inp_2_0_1;
    result5 += M4(-0.12212136, -0.33172598, -0.32602623, -0.3520969, -0.31620753, -0.11119355, -0.21621007, -0.353382, 0.30344084, -0.1340065, -0.07095748, 0.095555715, -0.084244244, 0.04382839, -0.065049425, 0.07563029) * inp_2_1_1;
    result5 += M4(-0.039727923, 0.16741453, -0.12625971, 0.30847552, -0.17269325, -0.12493335, -0.06252848, 0.18203886, 0.13810119, -0.06299241, -0.0053993617, 0.20332372, -0.42707753, 0.1566236, -0.32355168, 0.1060833) * inp_2_2_1;
    result5 += M4(0.042013574, -0.019000195, -0.053222634, 0.4377623, 0.05402091, -0.04918156, -0.02074637, -0.12083328, 0.061658435, -0.00039105094, 0.073628746, -0.15111612, 0.04014055, -0.12034262, -0.15402688, 0.2452527) * inp_2_0_2;
    result5 += M4(-0.05115285, 0.026178783, 0.060348198, -0.048778225, 0.017718507, 0.19430067, -0.08348972, -0.008640437, 0.19800542, -0.045930844, -0.07758654, -0.14218116, 0.02492843, -0.06858557, -0.076303475, -0.21767354) * inp_2_1_2;
    result5 += M4(-0.051111378, 0.027283488, -0.008796158, 0.06262722, 0.05225935, 0.12727705, -0.11653825, -0.070035964, -0.002652343, 0.1356606, -0.0022275173, -0.059467852, -0.17236987, 0.26085, -0.08184413, 0.007661716) * inp_2_2_2;
    result6 += M4(-0.13758247, 0.47992253, 0.21775906, 0.2845674, 0.04453927, -0.406886, -0.06919492, -0.10307145, -0.04167412, -0.26078737, 0.051454995, -0.17165911, -0.013593679, -0.122028045, 0.06942926, -0.063283004) * inp_2_0_0;
    result6 += M4(-0.19820544, -0.23205929, 0.16273531, 0.023568176, -0.032042913, 0.04544865, 0.0976151, 0.13936931, 0.07475227, 0.048374742, 0.078204125, 0.08369175, -0.11861327, 0.42862648, 0.46874171, 0.0993222) * inp_2_1_0;
    result6 += M4(-0.047566347, -0.01097437, -0.14977643, 0.018071486, -0.04887133, -0.18932194, 0.023491217, -0.12996374, -0.09006416, 0.4799643, -0.07843592, -0.036851697, -0.032141473, 0.6271557, 0.061860934, -0.073943116) * inp_2_2_0;
    result6 += M4(-0.040768653, 0.41252497, 0.20944405, 0.28110722, -0.13969807, 0.042986885, 0.09534639, -0.23775226, 0.1902759, 0.07080608, 0.27593464, -0.31526846, -0.07154209, 0.03598083, -0.022488626, -0.076544754) * inp_2_0_1;
    result6 += M4(-0.2474895, -0.025719514, 1.181603, -0.25797772, 0.24323735, 0.66748005, 0.44224584, 0.106931366, 0.06370833, 0.6515982, 0.31633744, -0.016337378, 0.2386287, -0.10911091, 1.2094307, 0.14327483) * inp_2_1_1;
    result6 += M4(-0.08459774, -0.1542087, 0.029153034, 0.017847292, 0.011666947, -0.0017946442, 0.14261407, 0.0061641196, -0.03565322, -0.18183987, -0.050910074, 0.033391804, 0.060365707, 0.33646017, 0.20358579, 0.038558554) * inp_2_2_1;
    result6 += M4(-0.11412106, 0.26440597, 0.087573126, -0.03574152, 0.055675697, -0.3058767, 0.010506739, 0.047419723, -0.1812625, 0.3429038, -0.050177246, -0.08447909, -0.10878048, 0.3925083, -0.026502782, 0.08717694) * inp_2_0_2;
    result6 += M4(-0.14375906, 0.20155817, 0.17739007, 0.045522224, 0.13335922, -0.29594976, 0.02832019, -0.059141647, 0.21726283, 0.14950812, 0.04203529, 0.029713621, -0.08140007, 0.2286015, 0.048481353, 0.11652194) * inp_2_1_2;
    result6 += M4(-0.13737804, 0.078588456, -0.07307534, -0.04494173, -0.02751372, -0.08454657, -0.038484827, 0.04551547, 0.09687772, 0.27242988, -0.05658793, -0.0314594, -0.15531854, 0.25541696, -0.1370281, -0.12183571) * inp_2_2_2;
    result7 += M4(-0.28310254, 0.005997803, 0.31373084, 0.005446527, 0.0002452019, 0.17378171, 0.22201534, 0.016770395, -0.10792812, 0.06166719, 0.07546582, 0.013997493, 0.011557201, -0.08050668, 0.4103731, 0.01817979) * inp_2_0_0;
    result7 += M4(0.05124471, 0.0814469, 0.45216653, 0.08774796, -0.05700838, -0.05899795, -0.058164787, -0.01998645, -0.1975383, 0.06733052, 0.057164446, -0.019194119, -0.021868177, -0.33764395, 0.114389494, 0.04302247) * inp_2_1_0;
    result7 += M4(0.17362437, -0.15322286, 0.076523475, 0.05954363, 0.33265507, -0.057714056, 0.2833425, -0.09389441, -0.041759238, 0.082888, -0.07098715, 0.0021862816, 0.16215946, 0.28342775, 0.21340531, -0.119540654) * inp_2_2_0;
    result7 += M4(-0.21676382, -0.42677075, -0.29023856, -0.08343998, -0.08254946, 0.04697218, 0.24474649, 0.018500758, 0.0135736745, 0.038800906, -0.3417381, -0.008096089, 0.05308764, 0.16509211, -0.118634574, 0.05086184) * inp_2_0_1;
    result7 += M4(0.28108966, -0.35581377, 0.42670444, 0.17104603, -0.028277477, -0.23832886, -0.14341721, 0.12387, -0.29770765, 0.36060715, -0.2689208, -0.043599993, 0.10275973, -0.146927, -0.24191526, 0.14715308) * inp_2_1_1;
    result7 += M4(-0.12334428, -0.06976448, 0.04424896, 0.011060302, 0.032396942, 0.11583557, -0.3819374, 0.09333711, -0.03446021, -0.04592602, -0.1341562, -0.048440915, 0.04223094, 0.11316036, 0.05868731, -0.11699876) * inp_2_2_1;
    result7 += M4(-0.1401006, 0.20545499, 0.3380474, 0.116517164, -0.01822879, 0.041709322, 0.06731107, -0.060690224, -0.042079583, -0.07233919, 0.21130425, 0.0119945565, -0.007921584, -0.05680923, -0.31836784, 0.027063215) * inp_2_0_2;
    result7 += M4(0.12421945, -0.34222785, 0.21251677, 0.11377022, 0.0010091101, 0.06794327, -0.15728395, 0.005270039, -0.019705063, 0.13952792, 0.31342366, 0.042770274, 0.037433926, 0.2028092, 0.15786754, -0.05246563) * inp_2_1_2;
    result7 += M4(-0.042303283, 0.1502775, 0.056543365, -0.0403104, 0.058044646, -0.18642728, -0.14045279, -0.14531007, 0.09610679, -0.098002404, -0.050481316, -0.011583754, -0.118470564, 0.17346954, -0.33728594, 0.11069925) * inp_2_2_2;
    const V4 inp_3_0_0 = inp[3][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_3_1_0 = inp[3][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_3_2_0 = inp[3][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_3_0_1 = inp[3][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_3_1_1 = inp[3][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_3_2_1 = inp[3][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_3_0_2 = inp[3][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_3_1_2 = inp[3][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_3_2_2 = inp[3][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.14692838, -0.12111147, -0.15213746, 0.15581118, 0.0008130179, 0.10252497, 0.24653602, -0.11461402, -0.042584125, 0.14738551, -0.31342965, 0.205181, 0.1019357, -0.14641, 0.048974834, -0.19849941) * inp_3_0_0;
    result0 += M4(0.029081749, -0.1386969, 0.11758148, 0.31240052, -0.021337198, 0.15802175, 0.08650392, -0.13578159, -0.06477359, -0.055019844, -0.12777585, 0.36110687, 0.14576182, 0.14981514, 0.4671147, -0.35586426) * inp_3_1_0;
    result0 += M4(-0.058833443, -0.030612864, 0.021880029, 0.05918608, -0.03050231, 0.09368156, -0.024504276, -0.086931154, 0.009927722, -0.15519542, -0.17934167, 0.2578746, 0.16743796, -0.085789055, 0.103205636, -0.02516688) * inp_3_2_0;
    result0 += M4(-0.067665465, 0.26203918, 0.04896709, 0.06472998, -0.34898707, 0.039324947, 0.022273345, 0.076889046, 0.032210857, 0.022684183, 0.013276569, 0.45967886, 0.027373882, -0.12659383, -0.08333415, -0.16003403) * inp_3_0_1;
    result0 += M4(-0.09371908, -0.21643867, 0.29850742, 0.1274369, -0.23582973, 0.6290206, 0.23356012, 0.23926494, 0.13513495, -0.006488111, -0.097513035, 0.20548782, -0.074910685, 0.18578751, 0.25307056, -0.14770772) * inp_3_1_1;
    result0 += M4(-0.0058927545, 0.024122529, 0.09502321, 0.05032874, 0.038693883, -0.13404669, 0.14808683, 0.31328052, -0.027796593, 0.10508486, 0.36808738, 0.020085653, 0.047342256, -0.058597427, -0.020656912, 0.21525948) * inp_3_2_1;
    result0 += M4(0.07323677, 0.14344314, 0.016937835, 0.03026594, 0.002037428, -0.21817282, 0.10232084, 0.2326892, 0.04433352, 0.0675987, -0.01833885, 0.1534671, 0.013618856, -0.0067100115, 0.0248312, -0.1982332) * inp_3_0_2;
    result0 += M4(-0.0033629627, 0.079507634, 0.063632295, 0.16008516, 0.046326675, -0.12641722, -0.34852123, -0.92833996, 0.013299444, -0.07297162, 0.10774818, 0.3125456, -0.08381876, -0.010844426, 0.16710816, -0.14004934) * inp_3_1_2;
    result0 += M4(0.033117685, 0.01247883, -0.013392772, 0.05626357, 0.10466528, -0.0038299032, 0.08914373, 0.031289324, 0.07943641, 0.067179896, 0.23300803, 0.036950827, 0.047434747, 0.024523037, -0.017362904, 0.007762638) * inp_3_2_2;
    result1 += M4(-0.1309645, -0.10653213, 0.052347146, 0.041744888, -0.0662591, -0.12791239, 0.26025105, 0.05183515, 0.3792357, -0.040180232, -0.0745529, 0.020108158, 0.018741978, -0.01710846, 0.06550769, -0.11507443) * inp_3_0_0;
    result1 += M4(-0.34134194, 0.097534135, 0.11358802, -0.65246445, 0.04344613, -0.011788942, -0.26884, -0.20136128, -0.22968933, -0.2623836, -0.11312907, -0.13027467, 0.020443821, 0.022053195, -0.42191055, 0.16160718) * inp_3_1_0;
    result1 += M4(-0.011952782, -0.0053271777, -0.036026455, -0.11715564, 0.17817056, 0.030588025, -0.1552387, 0.1780288, -0.09234511, 0.009226989, 0.14017063, 0.037644308, -0.10160938, -0.1361626, -0.06172664, 0.014550386) * inp_3_2_0;
    result1 += M4(-0.02356529, -0.11562854, 0.12749253, -0.21684846, -0.051416285, 0.11025958, 0.50985086, -0.028296428, 0.24485442, 0.0957948, -0.044865955, 0.11211161, -0.14307234, -0.18415214, -0.009322644, -0.24593014) * inp_3_0_1;
    result1 += M4(-0.06912143, -0.086703986, 0.025523292, 0.022880264, 0.018128553, 0.1454287, -0.2810319, -0.9379868, -0.016444128, 0.050064564, -0.7873014, 0.26354653, 0.0040838704, 0.1437546, -0.4336249, -0.125176) * inp_3_1_1;
    result1 += M4(-0.023169328, -0.06813034, -0.008956409, 0.035607085, -0.06412795, -0.37431294, 0.057647247, 0.48934606, 0.08332386, 0.14018391, -0.16156039, 0.39155456, 0.0231464, 0.05508768, 0.15092519, -0.0146097755) * inp_3_2_1;
    result1 += M4(0.06343257, -0.052791547, -0.20371047, 0.0054229177, 0.3838054, 0.31273702, -0.02811591, -0.10253446, 0.260357, 0.07434509, 0.14753442, 0.14504059, -0.14874068, -0.1175117, 0.020237884, 0.014928708) * inp_3_0_2;
    result1 += M4(0.018792234, -0.02693521, 0.056046274, -0.092894614, -0.0485479, 0.27765942, 0.081814095, -0.22657141, -0.059473664, -0.1955013, 0.028227411, 0.10753706, -0.018640071, 0.031782478, 0.11056123, -0.39082986) * inp_3_1_2;
    result1 += M4(0.04198629, -0.029079577, -0.0018285969, 0.023332141, 0.05030945, -0.07162475, 0.0054953825, 0.56228954, -0.17978895, -0.03417979, -0.10149008, 0.093155116, 0.06442605, -0.043065675, 0.0016255368, -0.04989935) * inp_3_2_2;
    result2 += M4(-0.36030892, 0.078944355, -0.18857756, -0.05990952, 0.07455886, -0.05251713, -0.0051090056, -0.039101165, 0.034885667, 0.048004806, 0.11422568, -0.017952709, -0.035375144, -0.041697033, 0.41068405, 0.06644237) * inp_3_0_0;
    result2 += M4(-0.31519794, 0.27030393, 0.050814535, -0.20702355, 0.04396222, -0.09702622, 0.069906235, 0.054172944, 0.015441881, 0.07307045, 0.105866835, 0.03403838, 0.1650397, 0.0031161865, 0.2813577, 0.26431334) * inp_3_1_0;
    result2 += M4(0.0025720294, 0.026360529, 0.053500175, -0.062302288, 0.0038827884, -0.14104126, -0.030359466, -0.031585593, -0.005580377, 0.15418851, 0.12001884, 0.0032232562, -0.014808855, -0.0595, -0.10425655, 0.10842186) * inp_3_2_0;
    result2 += M4(-1.0159053, 0.2712083, -0.2274618, -0.19525728, 0.3286721, 0.024212062, 0.34237432, 0.0023633172, -0.08243866, 0.04327697, 0.17062522, 0.0812338, 0.058705285, 0.056776784, 0.19725974, 0.10706858) * inp_3_0_1;
    result2 += M4(-0.9754217, 0.39643088, 0.08384415, -0.019002125, 0.008034162, -0.03765589, 0.12708749, 0.1693009, 0.14002115, -0.2275052, 0.3161754, 0.2779292, 0.44566315, -0.36610427, 0.24121541, 0.16211781) * inp_3_1_1;
    result2 += M4(-0.09981038, -0.010519697, 0.043279484, -0.05397609, -0.083080314, 0.06102328, 0.060096193, 0.025661385, 0.028781693, -0.07872886, 0.15425219, -0.1664192, -0.09299514, 0.078064375, -0.0041114325, 0.15216744) * inp_3_2_1;
    result2 += M4(-0.3100619, -0.05217906, -0.012577075, -0.008498641, 0.16625594, -0.06268054, 0.13903798, -0.33413607, -0.1402356, -0.026237644, 0.07466762, 0.11587489, -0.045981288, 0.017125072, -0.0035456917, -0.023985622) * inp_3_0_2;
    result2 += M4(-0.3522556, -0.017941235, 0.028150987, -0.036215264, 0.1922464, 0.30259037, 0.3561168, -0.15671971, 0.13377321, 0.13726857, 0.11513545, 0.14193453, 0.26380983, 0.07033603, 0.12618902, 0.1151069) * inp_3_1_2;
    result2 += M4(0.0010988638, 0.009628073, 0.08850768, 0.07057755, -0.64645964, -0.03721896, 0.27304345, -0.00021796608, 0.08588452, -0.0030163964, 0.07845355, 0.14666754, -0.005301719, -0.07059081, 0.041224513, 0.04456309) * inp_3_2_2;
    result3 += M4(0.17098561, -0.21225302, 0.032212004, 0.07516023, 0.16741471, -0.08156003, 0.018459223, -0.019276025, -0.04266561, -0.07464242, 0.0048045553, 0.13691226, -0.1308774, -0.03653758, -0.17144959, -0.12490417) * inp_3_0_0;
    result3 += M4(0.22514841, 0.10876963, -0.051967237, 0.13466313, 0.1063379, 0.13982637, -0.0070827613, 0.0070814705, 0.15231112, 0.08656857, 0.24322504, 0.28306207, -0.08983324, -0.12905249, -0.24750054, 0.029669981) * inp_3_1_0;
    result3 += M4(-0.029331174, -0.05019918, -0.020387258, 0.098387115, -0.20570429, -0.07193065, 0.13149486, -0.117492296, 0.12462873, 0.04222443, 0.15905131, 0.08443716, 0.15845686, 0.07552116, 0.0400203, -0.07263659) * inp_3_2_0;
    result3 += M4(0.32865003, -0.27747312, -0.07385111, -0.8384036, -0.37514448, -0.07822764, -0.11525617, -0.16079687, 0.13380212, -0.22027981, 0.33290443, 0.2804797, -0.3754305, 0.16872446, -0.075374745, 0.17147844) * inp_3_0_1;
    result3 += M4(0.11068599, 0.063227676, 0.094167806, -0.9655466, -0.0224931, 0.22547068, -0.2301498, 0.1389436, 0.10304726, 0.21423401, 0.2510991, 0.19666527, 0.05246675, 0.031955507, 0.13364959, 0.04146474) * inp_3_1_1;
    result3 += M4(0.08330667, -0.08734077, -0.13055605, -0.039486594, 0.042882707, 0.018324453, 0.11918306, -0.18172112, 0.15577276, -0.0639524, 0.2297009, 0.16738959, 0.09518741, 0.046234433, 0.09261025, -0.30626756) * inp_3_2_1;
    result3 += M4(0.09374249, -0.14416511, 0.017816987, -0.7959249, -0.27933818, -0.055046517, 0.042783316, -0.20050222, 0.12691908, -0.08871593, 0.14767566, -0.10773411, -0.09422508, 0.1104429, -0.067667075, 0.12668075) * inp_3_0_2;
    result3 += M4(-0.021486295, -0.05175386, -0.020655328, -0.54880744, -0.21423298, 0.072388954, 0.0742033, 0.7301041, 0.12354646, 0.12900211, 0.14653043, 0.21005164, -0.13625099, -0.027309785, 0.16209084, 0.18638651) * inp_3_1_2;
    result3 += M4(0.034056365, -0.004330307, -0.027264228, -0.030456034, 0.45437467, 0.15048659, -0.13278432, 0.37020433, -0.042813547, -0.15272745, 0.1378869, 0.19980286, 0.060689304, -0.043238625, 0.021684958, -0.17881545) * inp_3_2_2;
    result4 += M4(0.31933388, -0.10545848, 0.18841591, 0.29884705, 0.036952343, 0.065300055, -0.013705677, -0.2674884, -0.07292307, 0.0050880243, 0.45208785, 0.2168285, -0.1603375, 0.16492891, -0.22554708, -0.110965505) * inp_3_0_0;
    result4 += M4(0.08088569, -0.25602707, 0.23939654, 0.12895928, -0.037271075, -0.45254627, 0.09330394, -0.031969294, -0.073135555, 0.12811437, 0.37826383, 0.32615075, -0.11773532, -0.35067004, -0.21373269, -0.0997702) * inp_3_1_0;
    result4 += M4(0.04187725, -0.084745735, 0.08368302, 0.16481975, -0.019760566, 0.17291155, -0.1713994, 0.034841243, 0.026426584, 0.30778816, 0.22466117, 0.096183494, 0.10563517, -0.08222008, 0.1728803, 0.086453974) * inp_3_2_0;
    result4 += M4(0.08865095, -0.3924146, -0.08606203, -0.1708729, -0.13543554, 0.4101897, 0.22456107, -0.01637794, -0.2770513, 0.03899345, 0.51959914, 0.20618556, -0.041548584, -0.01697581, -0.09472609, 0.01627263) * inp_3_0_1;
    result4 += M4(0.12914476, -0.22961643, 0.042244557, -0.24566534, -0.009235502, -0.145621, -0.37740305, 0.16654098, -0.21803446, 0.50521654, 0.017279249, 0.021602241, -0.124774456, -0.5261899, -0.104103506, -0.23090325) * inp_3_1_1;
    result4 += M4(-0.007798693, -0.025257608, 0.09680748, -0.06385327, -0.18843354, 0.5529512, 0.30071345, 0.22909062, 0.037995327, 0.02249011, -0.09654277, -0.11112084, 0.016105441, -0.14822678, 0.041262325, -0.09391565) * inp_3_2_1;
    result4 += M4(0.027880756, -0.22116439, -0.062917985, -0.57840115, -0.09565248, 0.3065776, 0.085095, -0.36107543, -0.18878067, 0.008106187, 0.06659535, -0.08394011, -0.013153196, 0.13580185, -0.08979597, 0.24454457) * inp_3_0_2;
    result4 += M4(0.036557134, -0.16733308, -0.038323637, -0.28148723, -0.14989273, -0.5937909, -0.27357727, -0.21386293, -0.20792626, 0.26913223, 0.24656378, 0.047377415, -0.02789813, 0.021264229, 0.05245229, 0.15610224) * inp_3_1_2;
    result4 += M4(-0.018323725, -0.011593895, 0.013239487, -0.028929444, -0.06449391, -0.02693889, 0.34843853, -0.1771688, -0.047025606, 0.12596382, 0.22223748, 0.15031444, 0.004877535, -0.12871747, -0.103676036, -0.16079369) * inp_3_2_2;
    result5 += M4(0.03933118, 0.11263694, 0.2566461, 0.31960988, -0.104549944, -0.10886361, 0.0020499679, 0.03503795, -0.0973337, 0.12653361, -0.09443361, 0.24211715, -0.17039008, -0.11127622, -0.094341464, -0.7840067) * inp_3_0_0;
    result5 += M4(0.107521504, 0.03470314, 0.0907192, -0.17806196, -0.049644656, -0.22911522, -0.12410826, -0.06832876, 0.005329371, 0.10653494, -0.23932095, 0.30597818, -0.0956151, -0.04683763, -0.025820239, -0.19081616) * inp_3_1_0;
    result5 += M4(0.11965377, -0.18808511, 0.07443349, -0.064586885, 0.073752016, -0.15454382, 0.035880484, 0.1369647, 0.05783673, 0.12680419, 0.0015620491, 0.2671599, 0.06611853, 0.2035373, 0.20511197, 0.2801575) * inp_3_2_0;
    result5 += M4(0.014333902, 0.24502926, 0.20572734, -0.02765964, -0.037774853, -0.029262828, -0.022296589, 0.007967717, -0.16905367, -0.03200147, -0.30716845, 0.062494166, -0.036488026, -0.13090745, 0.024982927, -0.4605031) * inp_3_0_1;
    result5 += M4(-0.06590879, 0.08648572, 0.37140003, -0.15140001, 0.10902391, -0.24238427, -0.12897381, -0.4221918, -0.12268316, 0.030996082, -0.39964014, 0.095584646, 0.068115026, 0.023058642, 0.050316732, -0.4047593) * inp_3_1_1;
    result5 += M4(0.07623236, -0.07598875, -0.031800512, -0.076989375, 0.24247114, 0.6699327, 0.20864943, 0.08332993, 0.08694623, 0.08753742, -0.27531746, -0.123265326, -0.043344706, 0.23931448, 0.0069674333, -0.09651304) * inp_3_2_1;
    result5 += M4(-0.11801844, -0.028813183, 0.07291747, -0.20734538, 0.15185261, 0.1397247, 0.24478458, 0.1116515, -0.21922448, 0.019204907, -0.2708815, -0.1836917, 0.027783178, 0.04978325, -0.030865327, -0.16090043) * inp_3_0_2;
    result5 += M4(-0.1021944, -0.039979376, 0.028811958, -0.13426091, 0.16178177, -0.349716, 0.25366402, -0.09178893, -0.42805418, 0.13548747, -0.3444287, -0.20403007, 0.2255964, -0.034190528, -0.07326117, 0.044638637) * inp_3_1_2;
    result5 += M4(-0.06769173, -0.002451504, -0.026483495, -0.072096, -0.11368758, 0.059324328, 0.13693771, -0.029595038, -0.12862687, 0.03851256, -0.20371336, 0.0012604872, -0.026968963, 0.012696852, -0.033182092, -0.01876786) * inp_3_2_2;
    result6 += M4(0.19025944, -0.2503727, -0.19488192, 0.11648709, -0.09257867, -0.016049217, -0.01280257, -0.019738179, 0.06907278, 0.05343072, 0.10035118, -0.003925116, -0.06144966, 0.14939643, 0.20140758, -0.08111011) * inp_3_0_0;
    result6 += M4(-0.089859106, 0.09683762, -0.2394, 0.047808904, -0.15331121, -0.028545244, 0.19536702, 0.011559323, -0.08677059, 0.15751365, 0.09658367, 0.09110439, -0.04503704, -0.13600786, 0.21760616, 0.020417092) * inp_3_1_0;
    result6 += M4(-0.010922761, 0.04984482, -0.008782288, 0.06676412, -0.13424495, 0.045589387, 0.094453886, 0.01103105, -0.065532565, 0.4459752, -0.087328844, 0.020820493, 0.07153857, -0.08724564, -0.24899699, 0.023893747) * inp_3_2_0;
    result6 += M4(-0.2618911, 0.29499227, -0.1494832, 0.043338615, 0.0023589716, -0.30825526, 0.37450582, -0.015342657, -0.14904764, 0.30613127, 0.109134056, -0.04737533, 0.063513495, 0.007839637, 0.12246456, -0.021875218) * inp_3_0_1;
    result6 += M4(-0.22358735, -0.040110994, -0.57551485, 0.102893785, -0.08840462, 0.62070763, 0.31295693, -0.09854806, -0.14711325, 0.69014096, -0.03695445, 0.14561456, 0.010714114, -0.5078608, 0.2532144, 0.08848725) * inp_3_1_1;
    result6 += M4(-0.051040318, 0.04251542, 0.0090760775, 0.034569014, 0.13511679, 0.23233825, -0.12911904, 0.05689379, 0.079725794, 0.67044234, 0.17402937, -0.04559655, 0.12345473, -0.16498809, 0.0059965975, -0.036050234) * inp_3_2_1;
    result6 += M4(-0.28070745, 0.24089248, -0.009172006, 0.01865086, 0.09950659, -0.60418046, -0.39854577, 0.26342916, -0.15124182, 0.15504995, -0.050657865, 0.023357024, 0.09803402, -0.26105535, 0.045072008, -0.0022394168) * inp_3_0_2;
    result6 += M4(-0.07634114, -0.08804105, -0.0040017925, -0.012841301, 0.41898152, -0.06813063, -0.101092875, -0.14367011, -0.07093414, -0.1408146, -0.0207761, 0.003560496, 0.10132433, -0.5293243, 0.1316919, -0.03322445) * inp_3_1_2;
    result6 += M4(-0.007472724, -0.061612986, 0.030996943, 0.033639368, 0.061629813, 0.05111713, -0.23219594, 0.1611767, -0.027369611, 0.17261893, 0.039863408, 0.04697717, 0.0043693883, -0.02019984, -0.03341138, 0.004032947) * inp_3_2_2;
    result7 += M4(0.096230276, -0.11974746, -0.18692406, -0.01999411, 0.008764586, 0.19160499, -0.026654202, -0.021959536, 0.0938068, 0.04426356, -0.20867357, -0.0061396607, 0.013659134, 0.05195137, -0.54068655, -0.020865422) * inp_3_0_0;
    result7 += M4(0.05642491, 0.14752163, -0.16090432, -0.03178654, -0.017223539, 0.1092576, -0.18431318, -0.030609151, -0.08711494, -0.40004727, -0.16715218, 0.023426006, -0.035785936, -0.023206068, -0.2244546, 0.030723559) * inp_3_1_0;
    result7 += M4(0.07086577, -0.010009297, -0.32247508, -0.005402118, 0.028425677, -0.04505394, 0.20340511, -0.006604768, 0.17007583, 0.11809417, -0.025299307, -0.0031100863, -0.0417665, 0.17957602, -0.2471179, 0.0417923) * inp_3_2_0;
    result7 += M4(0.026488151, 0.05878557, -0.17220183, -0.023860594, 0.28948328, -0.04589574, 0.08160615, 0.0857752, 0.011427143, 0.018803423, 0.4512373, 0.06310768, -0.016496232, -0.139361, -0.2093227, 0.006660535) * inp_3_0_1;
    result7 += M4(0.0061968053, 0.2414987, 0.0049527595, 0.14331153, -0.20958422, 0.475688, -0.49793914, 0.026697831, 0.033222105, -0.26458907, 0.6828433, 0.02060027, 0.02144156, -0.05688896, -0.71740615, 0.048326045) * inp_3_1_1;
    result7 += M4(0.020861913, 0.034993198, -0.014987927, 0.08356073, -0.22116904, 0.4180222, -0.03438566, 0.024797922, 0.013472563, 0.0071164034, 0.70240885, 0.052015685, 0.06509524, 0.0795667, -0.027538659, 0.030246137) * inp_3_2_1;
    result7 += M4(-0.024045218, -0.060232226, -0.047995728, 0.008134007, -0.16014932, -0.1346305, 0.025630724, -0.17199017, -0.104505315, -0.0016575222, 0.44111472, 0.071989484, 0.017525293, -0.15970801, 0.0047853542, -0.049545042) * inp_3_0_2;
    result7 += M4(-0.01306688, 0.010452928, -0.009586341, -0.017815435, -0.18339266, -0.8409496, -0.15123336, 0.05658916, -0.101352036, -0.18402839, 0.3871995, 0.11223929, 0.054033432, -0.1316974, -0.51740694, -0.001133326) * inp_3_1_2;
    result7 += M4(0.0027158735, 0.04344978, 0.06516804, 0.023118313, -0.047152735, -0.2521848, 0.089242205, -0.11225785, -0.041802198, -0.06949689, 0.0903216, 0.033560414, 0.034897, 0.10102754, 0.12383666, 0.008134967) * inp_3_2_2;
    const V4 inp_4_0_0 = inp[4][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_4_1_0 = inp[4][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_4_2_0 = inp[4][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_4_0_1 = inp[4][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_4_1_1 = inp[4][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_4_2_1 = inp[4][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_4_0_2 = inp[4][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_4_1_2 = inp[4][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_4_2_2 = inp[4][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.020287333, 0.053610664, 0.14926675, -0.009516379, -0.029203214, -0.058922075, -0.3365848, 0.0752115, 0.090057254, -0.34567925, -0.05017748, 0.061344113, 0.44497702, 0.5004309, 0.08985661, 1.8226533) * inp_4_0_0;
    result0 += M4(-0.012108014, -0.17541786, 0.13065836, -0.0019710287, -0.18110803, -0.045334827, -0.48128617, 0.055162527, 0.057965, 0.22237618, 0.6914222, -0.17413805, 0.12231888, 0.28022215, -0.4153985, -0.27701026) * inp_4_1_0;
    result0 += M4(0.044239607, -0.029280478, 0.2752431, 0.03267755, -0.051813964, -0.05087691, -0.27671435, -0.1582206, 0.080188885, 0.02556143, -0.017858343, -0.045163427, -0.099257, -0.6118307, -0.18147415, -0.351005) * inp_4_2_0;
    result0 += M4(0.030730113, 0.023686498, 0.083941095, -0.14568397, 0.029218689, 0.0889104, 0.020641958, 0.058169037, -0.048332643, 0.14354877, -0.03907499, 0.38841844, 0.84055626, 1.8850845, -1.2453778, 2.7371953) * inp_4_0_1;
    result0 += M4(0.39635518, 0.08215202, 0.05566093, 0.32524943, 0.4654052, 0.41175294, -1.091924, -1.113925, 0.6359636, -0.056450855, -0.47298285, -1.1475234, 0.33591986, 0.6711523, 0.69225246, 1.1854475) * inp_4_1_1;
    result0 += M4(-0.0126732085, 0.010640525, -0.23745272, 0.047286283, -0.023903212, 0.2563097, -0.27832636, 0.008832793, -0.040842842, 0.2916844, -0.37582394, 0.23858698, 0.41791472, -0.8590179, -0.5125591, -0.44006163) * inp_4_2_1;
    result0 += M4(0.011917992, 0.26589325, 0.046570133, 0.023570167, 0.065440156, -0.09931502, 0.009666248, 0.6360599, -0.091619164, 0.26704973, 0.011258808, 0.09631231, -0.06109953, 0.8432866, 0.4953753, 0.2275474) * inp_4_0_2;
    result0 += M4(0.08524301, -0.07141846, -0.029419906, -0.9523668, 0.07078967, -0.027285116, 0.31647584, 0.46473342, -0.05496218, -0.14523065, -0.054350942, 0.081890956, -0.31031162, -0.13088244, 0.58671767, -0.20541078) * inp_4_1_2;
    result0 += M4(-0.07050409, -0.0025643376, 0.16790056, 0.1252545, 0.022757081, 0.12634504, 0.15881333, 0.05963422, 0.022749541, -0.24265057, 0.2173597, 0.3327529, -0.0069369953, -0.103736684, -0.20616747, -0.2973642) * inp_4_2_2;
    result1 += M4(0.020577691, 0.36637092, -0.23185943, -0.1333979, -0.27426246, -0.092166185, -0.07104525, -0.060069025, 0.27472222, 0.35148636, -0.12404272, 0.21155919, -0.72543234, 0.1348985, -0.013220168, -1.9720031) * inp_4_0_0;
    result1 += M4(0.19299419, 0.20506084, 0.117750384, 0.06034354, -0.16045524, -0.13066988, 0.16717254, 0.19901848, 0.31921127, 0.19308908, -0.15590622, 0.25151283, -0.1882506, -0.16746841, 0.20261909, 0.53914183) * inp_4_1_0;
    result1 += M4(0.03432736, 0.255146, 0.05180808, 0.05607056, -0.04838594, -0.043487154, -0.0100043705, -0.226044, -0.05156384, 0.0046826038, -0.032585982, 0.24476862, 0.7735593, -0.13363421, 0.05759651, -0.5453639) * inp_4_2_0;
    result1 += M4(-0.029552083, 0.21041214, 0.19962597, -0.15891159, 0.27284637, 0.10981814, -0.048235647, 0.16104588, -0.8597703, 0.15663704, 0.42146704, -0.0380067, 1.2660574, -0.21908315, -1.2977241, -0.7281915) * inp_4_0_1;
    result1 += M4(-0.29611367, 0.15408964, 0.30107716, 0.2729277, 0.046294633, 0.2928169, 0.28813937, -0.045789972, -0.20924701, 0.02989142, -0.34315002, 0.11975432, -0.0077568325, -0.25467476, 0.5655991, -0.81387883) * inp_4_1_1;
    result1 += M4(0.06766633, -0.043444622, 0.088470146, -0.12812594, 0.10481173, 0.15296169, 0.07803992, -0.1656385, -0.28303823, -0.42627767, -0.09991758, -0.40632063, -0.056024894, -0.1242274, -0.28175756, -0.38509473) * inp_4_2_1;
    result1 += M4(0.27736384, 0.19568358, 0.035969913, 0.13001451, -0.07349784, 0.07765097, -0.324676, -0.33323967, -0.09476334, 0.20802896, 0.037700918, 0.051168174, 0.0004941261, -0.028062165, 1.742025, 0.53224957) * inp_4_0_2;
    result1 += M4(-0.093763016, -0.15776962, 0.13549064, 0.2813105, 0.1632138, 0.17306648, -0.7327331, -0.4743807, 0.38265812, 0.03326682, 0.12241913, -0.15596674, -0.019526662, -0.26473126, -0.16012321, -0.78244734) * inp_4_1_2;
    result1 += M4(0.026915012, -0.066122405, 0.15451594, 0.027668955, -0.1618922, 0.044082165, -0.186632, -0.05390392, 0.04317471, 0.14829557, 0.067602046, 0.037809614, 0.06603365, -0.1641115, 0.20955394, -0.20638396) * inp_4_2_2;
    result2 += M4(0.06679359, -0.11124601, -0.009358941, 0.023672273, -0.061759442, -0.054842666, -0.06702515, 0.0030806893, 0.0036293215, 0.005799973, 0.17731407, 0.028294524, -0.6064226, -0.07877881, -0.3036318, 0.26218235) * inp_4_0_0;
    result2 += M4(-0.045286927, 0.22449932, -0.26673308, -0.007539629, -0.09863171, 0.093674995, -0.08757615, -0.026695743, 0.1804895, 0.008625943, 0.27820137, 0.19407082, -0.20895226, 0.6792263, -0.52447623, -0.1517567) * inp_4_1_0;
    result2 += M4(-0.047778424, 0.15005352, 0.0402882, 0.056346335, 0.03651547, -0.11812056, 0.07327158, 0.07478515, -0.089489, 0.014372827, -0.012626943, -0.027242675, 0.51394945, -0.047782157, -0.34348637, -0.066120654) * inp_4_2_0;
    result2 += M4(0.14252222, -0.06316889, 0.050488316, -0.029355802, -0.2764165, -0.17102563, -0.1161484, -0.09446128, -0.060218383, 0.047973264, 0.16006726, -0.067334525, -1.4351808, -4.6861863, 1.6505264, 0.6618825) * inp_4_0_1;
    result2 += M4(0.11273395, -0.11790788, -0.07737059, -0.040311582, -0.15853636, 0.37009567, 0.1312517, 0.04951466, 0.120299935, -0.57455623, -0.22400689, -0.26112354, -0.38425532, 0.6649697, -0.5065427, -0.02642139) * inp_4_1_1;
    result2 += M4(0.061575845, 0.07809638, 0.01058312, 0.0330801, -0.42979312, -0.23867224, 0.25567994, -0.16295959, 0.08099989, 0.24675088, 0.27240232, 0.6238339, 0.42161918, 0.013450574, -0.25249785, -0.3108163) * inp_4_2_1;
    result2 += M4(0.10908912, 0.006094902, 0.0079133995, 0.05982163, -0.11657313, -0.2747902, 0.46447465, -0.117089584, 0.28367823, 0.09637765, 0.16543093, 0.2187194, -0.97769576, -0.52858, -0.04475847, -0.2669199) * inp_4_0_2;
    result2 += M4(0.026850736, 0.36099517, -0.43244603, -0.042811297, 0.08861416, -0.58879787, -0.5755591, 0.17855723, 0.18905419, 0.48886386, 0.17593615, -0.39443216, -0.14507191, 1.2131993, -0.25448674, 0.25756922) * inp_4_1_2;
    result2 += M4(-0.12831543, 0.074451245, -0.07210193, -0.19259973, -0.0030961137, -0.08996827, -0.24921557, 0.28061387, -0.3088467, 0.1531496, 0.15005964, 0.045763485, 0.4629898, 0.07249764, -0.032562673, 0.0015384565) * inp_4_2_2;
    result3 += M4(-0.3288111, -0.007495749, 0.10722167, 0.042221364, -0.05331955, 0.026029864, 0.15399824, 0.050896753, 0.031317577, -0.009296982, -0.17051314, 0.10326453, -1.0710366, 0.91493756, 0.8132579, -0.041516647) * inp_4_0_0;
    result3 += M4(-0.04815006, -0.005054582, 0.027232887, -0.0064906636, -0.038042095, -0.20482945, 0.05673717, 0.033147343, 0.0021209035, -0.11687591, -0.046407785, -0.07389452, 1.1888456, 0.09129514, -0.7853972, 0.29648152) * inp_4_1_0;
    result3 += M4(0.18631501, 0.099428564, -0.024164256, 0.089550875, -0.019352933, 0.0523732, 0.020274533, -0.053498946, 0.2794441, -0.048525106, -0.024982275, 0.058523506, -0.51229507, -0.19483118, 0.14496592, 0.015027526) * inp_4_2_0;
    result3 += M4(0.0082564745, -0.012639787, 0.2288111, 0.22561245, -0.28069744, -0.23852783, 0.31840742, 0.01270472, 0.32152748, 0.09592904, -0.025274167, -0.08557925, -1.1619632, 0.22977883, -0.04455376, 0.3504422) * inp_4_0_1;
    result3 += M4(-0.034751054, 0.04661549, -0.11049215, 0.49016598, 0.28804052, -0.017865011, -0.05604019, 0.21209769, -0.38354263, 0.05794265, -0.15168972, -0.13870881, 0.5158876, 0.14339483, -0.6567972, -0.38417956) * inp_4_1_1;
    result3 += M4(-0.26249155, -0.009550195, 0.1858691, 0.5062866, 0.03341032, 0.00092645484, 0.16227764, -0.2504169, -0.1385001, 0.17509316, 0.37594062, -0.29521757, -0.40195605, 0.23230074, -0.5474006, -0.29318193) * inp_4_2_1;
    result3 += M4(-0.02770629, -0.013192012, 0.047654323, 0.445726, -0.35443082, -0.26348528, 0.42758942, 0.06992084, 0.006314848, -0.022475703, 0.09078335, -0.0874246, 0.74039936, 0.57030904, 0.5026792, 1.3919446) * inp_4_0_2;
    result3 += M4(0.19149914, -0.0981959, -0.036375064, 0.1389969, -0.31041592, -0.6999299, -0.50164056, -1.2491343, -0.12947805, -0.21992537, 0.08482315, 0.29273057, -0.875883, -0.3364512, 0.040697224, -0.65264386) * inp_4_1_2;
    result3 += M4(-0.37488803, 0.07592216, -0.0033037765, 0.22130194, -0.015336412, 0.06837586, 0.21623932, 0.9328919, 0.17149787, 0.011620433, 0.09028444, 0.055979587, -0.65455663, 0.25479883, -0.10469035, 0.25030124) * inp_4_2_2;
    result4 += M4(-0.017154967, -0.32697964, -0.011899147, 0.15729, 0.030761376, -0.030807054, -0.08971294, 0.07029072, -0.14521652, -0.20387578, 0.057420485, 0.05726175, 0.03573584, -0.2796971, 0.70117396, 0.393056) * inp_4_0_0;
    result4 += M4(0.07017931, 0.3782423, 0.15321799, 0.17485046, 0.02431808, 0.24963878, -0.218507, 0.3068062, -0.21301761, -0.29000112, 0.19801378, -0.2925938, 0.32443488, 1.5208935, -0.7287117, 1.3004102) * inp_4_1_0;
    result4 += M4(-0.013020104, -0.23768216, 0.024373896, -0.037297174, 0.09707412, -0.13702261, -0.08513357, 0.0025344274, -0.0971744, 0.075026765, 0.104817204, 0.01779408, 0.18630752, 0.19520542, -0.49723738, -0.0017601977) * inp_4_2_0;
    result4 += M4(0.07249935, 0.20290823, 0.38616732, 0.4732055, 0.09379736, -0.24904642, -0.003215969, -0.042600386, 0.01239747, 0.17035659, -0.079079784, -0.2661824, -2.4108377, 0.16347861, 0.32413623, -0.7892506) * inp_4_0_1;
    result4 += M4(0.12051283, -0.1530206, 0.064496435, 0.49149778, 0.22879843, -0.21790986, 0.18491784, 0.24438283, -0.28560647, 0.16958053, 0.09074809, -0.41805962, 0.37967598, 0.5241693, -0.6618788, -0.3210555) * inp_4_1_1;
    result4 += M4(-0.05088464, 0.13983686, 0.17022671, 0.24948072, 0.13931932, 0.075167485, -0.1912262, -0.16615759, -0.048836842, 0.27023837, -0.13377684, -0.0826522, 0.07348809, -0.6161314, -0.40738332, 0.7591049) * inp_4_2_1;
    result4 += M4(-0.19975145, -0.13001637, 0.13077477, 0.26009327, 0.04375593, 0.10974376, 0.25374952, -0.2681393, -0.24440397, -0.3538296, -0.28902012, 0.21294597, -0.03486496, -1.4537909, 1.0559975, 0.111490645) * inp_4_0_2;
    result4 += M4(0.20293973, 0.09241519, -0.07425489, 0.14728193, -0.46791688, 0.46760857, 0.26782638, -0.30868068, -0.085070126, -0.41911426, -0.15385851, 0.020248957, 0.40597066, -1.5122589, -0.47539976, -0.47009158) * inp_4_1_2;
    result4 += M4(0.016388316, 0.1610947, 0.029536689, 0.32285964, -0.09857172, 0.1702048, 0.38871118, -0.06744162, -0.17072295, -0.3381607, -0.29825285, 0.112057894, -0.036540773, 0.14869073, 0.5429599, 0.63464576) * inp_4_2_2;
    result5 += M4(-0.04797261, 0.13317412, -0.05845373, 0.1098966, -0.08594293, 0.16909036, 0.1226807, -0.14812046, -0.20404151, -0.07973047, -0.16752014, -0.200571, 0.5525773, 1.055609, 0.5257726, -1.1277984) * inp_4_0_0;
    result5 += M4(0.2879278, 0.096781485, 0.031222994, 0.2297264, -0.064516544, 0.15577847, 0.17453672, 0.008791107, -0.1525947, -0.004548229, -0.08099757, -0.06116523, 0.6625299, 0.5467683, -0.43247318, -0.21441431) * inp_4_1_0;
    result5 += M4(-0.11613734, -0.02600888, -0.060826734, -0.4075648, -0.10494341, -0.05306265, 0.10333075, -0.36200717, -0.071304284, 0.08571422, -0.064038955, 0.15747675, 0.20355271, -0.14887162, -0.12111092, -0.8108239) * inp_4_2_0;
    result5 += M4(0.024553115, -0.090376325, -0.11113367, 0.031307805, 0.07776625, 0.33290556, 0.19793569, 0.3437654, 0.06514166, -0.09721329, 0.0027553807, 0.24652237, -2.7649221, -1.3052051, -1.1603937, 0.61206555) * inp_4_0_1;
    result5 += M4(0.099163435, -0.14836521, -0.06357624, 0.12854782, -0.049238645, 0.35963085, 0.047977414, -0.043707073, -0.07234704, -0.40041235, -0.21955235, 0.1092273, 0.08131592, 0.9361457, 0.13051586, 0.2551081) * inp_4_1_1;
    result5 += M4(0.08626778, 0.05940888, -1.7310005e-05, -0.040405165, -0.018668162, -0.08284393, -0.034134064, 0.030415945, 0.13689674, 0.3614816, -0.027488803, 0.35807976, 0.19819134, -0.32803145, 0.09963556, -0.6824439) * inp_4_2_1;
    result5 += M4(-0.005154029, 0.20938784, -0.08676629, -0.06399832, -0.28155306, -0.23153903, 0.089653574, 0.32611766, -0.14748463, 0.009487319, -0.015109804, -0.20863564, -0.7070977, 0.0953698, -0.8207213, -0.44115) * inp_4_0_2;
    result5 += M4(-0.25243685, 0.39788565, 0.12230283, -0.04356432, 0.009695129, -0.7766982, -0.41708744, 0.44880137, -0.31055272, 0.053521715, 0.41672093, 0.13375099, 0.74147004, -0.058142297, -0.33914724, 0.40424356) * inp_4_1_2;
    result5 += M4(-0.16356719, 0.02078599, 0.075744286, 0.033709988, 0.10797904, 0.17424043, -0.17569153, 0.38220808, -0.07451433, 0.07537722, 0.1112041, -0.18926243, 0.39836082, 0.39857042, -0.14904608, 0.5602208) * inp_4_2_2;
    result6 += M4(0.037616853, 0.14940578, -0.112031244, -0.005812842, 0.041354235, -0.01874736, -0.0328446, -0.06577084, 0.011824735, -0.24437952, -0.032081187, -0.028567417, 0.16432893, -2.143406, -2.8044536, 0.19316363) * inp_4_0_0;
    result6 += M4(0.24062365, -0.069415025, -0.065822706, -0.06812605, 0.16051742, 0.010979751, -0.14480555, -0.016701218, -0.08863246, -0.023575274, 0.21879672, -0.0755547, -0.02651813, 2.1830373, -0.1995549, -0.30611277) * inp_4_1_0;
    result6 += M4(0.014352779, -0.18229675, 0.026367933, -0.068035424, 0.095582, -0.03978999, -0.05732023, -0.0098278625, 0.012110811, 0.01466089, 0.024820993, 0.03805275, -0.120946944, -1.1922563, 0.21446012, -0.07293148) * inp_4_2_0;
    result6 += M4(0.03966792, 0.28114468, -0.030738996, 0.028663822, 0.028110784, 0.10431843, -0.6991808, -0.007680695, -0.18674727, 0.16743939, -0.23388691, 0.014202477, -0.38563544, -0.9044769, -4.0112824, 0.16374758) * inp_4_0_1;
    result6 += M4(-0.09145626, 0.23156527, -0.03120761, -0.118468076, -0.24730745, 0.35680923, -1.0813506, 0.051244725, -0.14565818, 0.40641186, 0.5991963, -0.07275347, -0.19954063, -2.0152276, -1.6681551, 0.19232686) * inp_4_1_1;
    result6 += M4(0.009112461, 0.045067325, -0.065153584, -0.05155993, 0.2762971, 0.13422392, 0.1894496, 0.008640615, 0.0015522937, -0.22187813, -0.07005267, 0.029923446, -0.4272002, 1.7979747, 0.2461607, 0.46994188) * inp_4_2_1;
    result6 += M4(-0.04740727, 0.22568972, -0.02440855, 0.057893407, -0.069835216, 0.8604293, 0.003987155, 0.28498352, -0.0699329, 0.059174128, -0.2533383, 0.114713326, 0.05737016, -4.3062716, -1.1953089, 0.0230405) * inp_4_0_2;
    result6 += M4(-0.28948444, 0.18473832, -0.812193, 0.11333851, 0.32255718, 0.015122611, 0.2571958, -0.16167155, -0.56022173, -0.005510526, 0.025901295, -0.081201434, -0.2963337, 0.95323884, -0.19547117, 0.040593047) * inp_4_1_2;
    result6 += M4(-0.052070957, 0.09820856, -0.04987117, -0.0984212, -0.03680913, -0.32809088, -0.3584713, -0.022080284, 0.09245819, 0.15622622, -0.15139492, 0.063234515, -0.17801675, -0.07562394, 0.5604041, -0.023371747) * inp_4_2_2;
    result7 += M4(0.06400284, 0.38174173, 0.34030324, 0.016381823, 0.095251314, -0.1844259, -0.016091917, -0.028287187, 0.018350204, 0.2896671, 0.600124, -7.2625124e-05, -0.14318332, 0.045889694, -1.8608142, 0.12717323) * inp_4_0_0;
    result7 += M4(0.008699108, 0.45377332, 0.5413454, 0.010386173, 0.13716385, 0.1079694, -0.17559488, 0.009336769, 0.095457345, -0.124020234, 0.5976416, 0.028621504, 0.7843453, -0.69772136, 0.4537442, 0.26865396) * inp_4_1_0;
    result7 += M4(-0.048679985, 0.5986147, 0.34311932, 0.0008041079, 0.13233669, -0.14200965, -0.054644685, -0.008342933, 0.05204441, 0.31756023, 0.13696736, 0.004974079, 0.51775765, -0.3954275, 0.182604, 0.053077947) * inp_4_2_0;
    result7 += M4(-0.026998978, 0.24933763, -0.11028407, -0.018752204, 0.07894924, -0.048799682, -0.08087473, -0.022835223, -0.005108459, -0.5659619, -0.10928539, 0.027225384, 0.35125583, -1.4858167, -1.0794938, 0.12513337) * inp_4_0_1;
    result7 += M4(0.06443046, 0.14695565, 0.4093462, -0.029218791, 0.23430419, -0.05307297, -0.19984478, 0.27032423, -0.05083672, -0.05021896, -0.13443273, -0.002421776, 0.0007494115, 0.31251636, -0.74186504, 0.41849443) * inp_4_1_1;
    result7 += M4(-0.015898896, -0.010984471, 0.016244827, -0.02657931, 0.30212414, -0.438848, -0.104609616, -0.09833874, -0.009387762, -0.18856923, -0.31227246, -0.032346908, -0.53600526, -0.54055995, 0.02229566, 0.1823245) * inp_4_2_1;
    result7 += M4(-0.008373414, 0.3089758, 0.026034066, 0.039627794, 0.036704026, 0.07897028, -0.0008268343, -0.041128602, 0.0052573513, -0.26072657, -0.20121084, -0.021725213, 0.10433701, 0.62718976, -2.0204313, 0.08489212) * inp_4_0_2;
    result7 += M4(-0.17084613, 0.7080406, 0.26738015, 0.1400083, 0.017941471, 0.012609908, -0.025322065, -0.099554226, -0.08841847, 0.17319725, -0.038703375, 0.00537254, -0.022780072, -0.08076143, 0.2740451, -0.04379438) * inp_4_1_2;
    result7 += M4(-0.119877174, 0.23259869, 0.19964506, 0.0668932, 0.112736285, 0.55118775, -0.2624508, -0.07739275, -0.08924623, -0.0062876544, 0.11041316, -0.14340575, 0.025909625, 0.60728186, -0.5088132, 0.115246005) * inp_4_2_2;
    const V4 inp_5_0_0 = inp[5][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_5_1_0 = inp[5][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_5_2_0 = inp[5][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_5_0_1 = inp[5][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_5_1_1 = inp[5][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_5_2_1 = inp[5][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_5_0_2 = inp[5][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_5_1_2 = inp[5][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_5_2_2 = inp[5][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.05294528, -0.18307234, 0.5263515, 0.23346347, -0.05710066, 0.009534292, 0.071182266, 0.11410225, 0.038570743, 0.0032792368, -0.057468753, 0.119669184, 0.007191335, -0.12148829, -0.1176742, 0.14160508) * inp_5_0_0;
    result0 += M4(-0.060176402, 0.44572288, 0.020161347, -0.043551624, 0.014474936, 0.2007174, 0.015212934, 0.13191828, 0.11193184, 0.5850305, -0.012595521, 0.25357607, 0.017501298, -0.10071429, -0.5393635, 0.3858128) * inp_5_1_0;
    result0 += M4(0.09697749, -0.034477223, 0.0110706305, -0.094843484, -0.026129901, -0.021815365, -0.3734249, 0.06430306, -0.17508733, -0.007282993, -0.26744422, 0.13984801, 0.03157513, -0.15225942, 0.04791176, 0.044797372) * inp_5_2_0;
    result0 += M4(-0.11863203, -0.745748, 0.15168819, 0.45442206, 0.007475112, 0.04067143, -0.043670733, 0.5905726, 0.020598756, 0.25046262, 0.1382867, 0.008655801, 0.056210797, 0.1523638, -0.1902057, -0.057532467) * inp_5_0_1;
    result0 += M4(0.07117034, 0.37129363, 0.38181657, -2.1060178, 0.09042727, -0.16742323, 0.1630706, 0.2692845, 0.1392026, -0.3153999, -0.059359293, -0.43335256, 0.009569061, -0.36971176, -0.023219382, -0.15554637) * inp_5_1_1;
    result0 += M4(-0.008297144, -0.10884628, 0.09529124, -0.023401558, -0.05705663, 0.07508644, 0.116256446, 0.4763155, 0.057474263, -0.1364104, 0.35191256, 0.121105745, 0.12210208, 0.07544635, -0.28825417, 0.19718738) * inp_5_2_1;
    result0 += M4(-0.052258376, -0.13234256, -0.0587839, -0.09523776, 0.03411131, 0.16366553, 0.06322529, 0.26779285, -0.005641396, 0.10205452, -0.032838352, 0.10319716, 0.027891552, 0.17702086, -0.055584095, 0.03062126) * inp_5_0_2;
    result0 += M4(0.11388065, 0.19551453, -0.07363707, -0.31004778, 0.06338082, 0.0631324, 0.14674689, 0.31963554, -0.02675442, 0.13770403, 0.012276204, -0.1698577, -0.02647276, 0.34197083, 0.18192168, 0.34846064) * inp_5_1_2;
    result0 += M4(-0.029306423, -0.07398371, -0.027487233, -0.01654775, 0.0068359417, -0.0027202894, 0.18202676, 0.20916857, -0.02511114, -0.04896642, -0.2649445, 0.07198052, -0.032795552, -0.014455678, 0.115065426, 0.1509686) * inp_5_2_2;
    result1 += M4(-0.011658827, -0.0030894002, 0.27093008, 0.008002963, 0.09807365, 0.003834553, 0.1528099, -0.014707249, -0.081750385, -0.20252383, 0.2522507, -0.079711854, 0.19421297, 0.3503982, -0.064064085, 0.22465876) * inp_5_0_0;
    result1 += M4(0.34698892, -0.113209575, -0.058655187, -0.7018395, 0.007484396, 0.16157863, 0.25216007, 0.19439735, -0.07598142, 0.28775075, 0.13955961, -0.06990275, -0.34171644, -0.107078835, 0.20607768, -0.19355142) * inp_5_1_0;
    result1 += M4(0.11493079, 0.13622482, -0.04526158, -0.032376193, -0.08743473, 0.07870098, 0.10391709, -0.10426203, 0.10307934, 0.08243338, 0.1770685, 0.108332135, -0.20488961, -0.019185765, 0.13580045, -0.0015605439) * inp_5_2_0;
    result1 += M4(-0.8890018, -0.26434952, 0.13694355, -0.057650696, -0.045958858, 0.12678865, 0.0002528417, 0.21796371, 0.2105693, -0.102606796, -0.013170327, -0.16921002, 0.22104609, 0.08960355, 0.2781314, 0.0194926) * inp_5_0_1;
    result1 += M4(0.013264986, 0.20613399, -0.19223587, 0.52435225, -0.238345, 0.1062619, 0.3274112, 0.06854202, -0.14392538, 0.11904017, -0.1853547, 0.18495406, 0.039432526, 0.3617771, 0.035260737, 0.003339628) * inp_5_1_1;
    result1 += M4(0.07274061, 0.022005016, 0.024919633, 0.11291902, -0.08719597, -0.0579031, 0.1092644, 0.23661101, 0.06054037, 0.036034215, -0.002817749, -0.09083961, -0.09646613, 0.116081856, 0.028860196, 0.05111923) * inp_5_2_1;
    result1 += M4(0.3144621, -0.061479032, 0.087000705, -0.45216402, 0.09072757, 0.2366523, 0.060998324, -0.016382826, 0.20591368, -0.15740515, -0.25227612, -0.14338791, 0.117179886, 0.31863442, 0.080788895, -0.009016209) * inp_5_0_2;
    result1 += M4(0.0011040879, -0.19779819, -0.17368403, 0.22551934, 0.011157102, 0.017654644, 0.06199417, 0.044823937, 0.1230141, 0.040211134, -0.19509986, -0.10522258, -0.051500756, -0.18678135, -0.2753393, 0.039058324) * inp_5_1_2;
    result1 += M4(0.036847394, -0.03434113, -0.096768305, -0.08457939, 0.057190236, -0.034320995, 0.045741804, -0.014321575, -0.026433941, 0.09505056, 0.10732429, 0.09628864, -0.10144094, 0.098385945, 0.047078796, -0.07304616) * inp_5_2_2;
    result2 += M4(0.17405705, 0.034117106, -0.07720319, 0.21498606, -0.1318747, -0.118554, 0.06932546, -0.09364076, -0.01637871, 0.031942714, -0.10794146, -0.031011363, 0.05994898, 0.01841737, -0.02371561, -0.009918559) * inp_5_0_0;
    result2 += M4(0.15909307, -0.030361924, 0.09405668, 0.16370843, -0.0004560411, 0.0097200265, 0.115846075, 0.081484504, -0.14118434, -0.48170546, 0.15010275, -0.2446016, -0.12760448, 0.09883311, 0.1669356, 0.14293028) * inp_5_1_0;
    result2 += M4(-0.024664827, -0.12175808, -0.029194975, 0.030848831, -0.12149323, 0.061483707, 0.19890273, -0.06851386, -0.12732759, 0.072234996, 0.007875875, -0.18996985, 0.04801749, 0.13612178, -0.0062240358, 0.0305737) * inp_5_2_0;
    result2 += M4(-0.124488294, -0.38828498, 0.112760484, 0.25056896, 0.09771242, -0.08779512, 0.20789135, 0.0121677825, 0.013509227, -0.0078048813, -0.041724492, -0.012637987, -0.04187158, -0.010474867, 0.024764357, 0.026976218) * inp_5_0_1;
    result2 += M4(0.119541205, -0.18051293, 0.31166855, -1.1100429, 0.18115988, 0.16439596, 0.0780618, 0.019121798, -0.2506426, 0.09652909, 0.13757223, -0.516754, -0.5186843, -0.16792412, 0.5347876, -0.18762504) * inp_5_1_1;
    result2 += M4(-0.12589115, -0.22877735, 0.055856664, -0.29919383, 0.02601319, 0.1752398, 0.15591799, -0.11975483, -0.12890962, 0.072078295, 0.08329335, -0.42788863, -0.13631657, 0.1307416, 0.19421503, 0.15023677) * inp_5_2_1;
    result2 += M4(0.1443288, 0.13598306, -0.14329457, 0.17906278, 0.04450698, -0.02580612, 0.1466984, -0.005383657, 0.033328727, 0.1115207, -0.12716782, -0.1038918, -0.12871316, -0.086235836, -0.10382993, 0.09322905) * inp_5_0_2;
    result2 += M4(-0.55250454, 0.10078672, -0.091853835, 0.13265179, -0.05417234, 0.09012699, 0.12566444, -0.04019788, -0.06238125, 0.048751794, 0.06807214, -0.2860939, -0.18533036, 0.20712073, -0.01237132, -0.020310046) * inp_5_1_2;
    result2 += M4(-0.09125113, -0.08516623, -0.043241814, -0.065130785, -0.052197322, 0.097700305, 0.12836906, 0.094231136, -0.054623503, 0.028790586, 0.036910255, -0.18334885, 0.017906044, 0.10772973, -0.00032363698, 0.013303766) * inp_5_2_2;
    result3 += M4(-0.13560797, -0.07841638, 0.42826945, -0.1716601, -0.0435315, -0.040396165, 0.024020685, 0.08065936, -0.26604444, 0.116708115, 0.09838544, -0.052335568, -0.13610294, -0.059471607, 0.058215033, 0.14577474) * inp_5_0_0;
    result3 += M4(-0.51812804, 0.14315106, 0.19098446, -0.108829774, -0.30853108, 0.19779545, -0.09062834, 0.24645011, -0.13444747, 0.3667498, -0.6872334, -0.1791194, -0.022060739, -0.32865077, 0.21213962, 0.009072129) * inp_5_1_0;
    result3 += M4(0.05883128, 0.020104704, -0.04268637, -0.19889423, -0.10217514, 0.0697082, 0.0027936215, 0.34032357, -0.29306346, 0.010152592, -0.4527446, 0.008984433, 0.11546807, -0.11335799, 0.037380844, 0.17645559) * inp_5_2_0;
    result3 += M4(0.008609728, 0.20839714, -0.2990293, 0.22748667, 0.069386154, -0.13653825, 0.084453, 0.31595924, -0.21734206, 0.18688107, -0.21291855, 0.018763259, -0.08722657, -0.1371592, 0.030940738, 0.030287424) * inp_5_0_1;
    result3 += M4(-0.015461433, 0.1586846, -0.334542, -0.37627426, 0.09780574, 0.017640553, 0.012023815, 0.20572712, -0.028285226, 0.24026543, -1.0300418, -0.159588, -0.24700956, -0.37189025, 0.12623835, -0.08099222) * inp_5_1_1;
    result3 += M4(0.011675215, 0.09213037, -0.2622001, -0.00611992, 0.09429593, 0.0557927, -0.06587054, 0.25377074, -0.14467514, -0.003941793, -0.6007078, 0.12651236, 0.018339956, -0.032474667, 0.26173615, 0.070787966) * inp_5_2_1;
    result3 += M4(-0.19463049, 0.23860689, -0.0260178, 0.35213864, 0.02728801, -0.06692749, 0.039767753, 0.05446293, -0.04196449, -0.052031565, 0.0802381, 0.047045764, -0.10951387, -0.0053655566, 0.060910046, -0.11632046) * inp_5_0_2;
    result3 += M4(0.45159486, 0.09351319, 0.56333923, -0.22606547, 0.058332644, -0.088995785, -0.011048084, 0.1089482, 0.011827133, 0.14013766, -0.37454194, 0.32726485, -0.019377846, -0.2483878, 0.17273155, 0.28772455) * inp_5_1_2;
    result3 += M4(-0.0023235793, 0.029033327, -0.030794881, 0.0053400206, -0.10984959, 0.079499096, 0.08073958, 0.28933495, -0.07134034, 0.08641224, -0.41792107, -0.08772838, -0.0771525, -0.017508283, 0.114593394, -0.088840775) * inp_5_2_2;
    result4 += M4(-0.14562556, 0.07588337, -0.17657855, -0.46944103, 0.096282355, 0.07519299, 0.19012797, -0.016876366, 0.04692916, -0.051957946, -0.041795827, 0.1544843, -0.014747569, -0.012679815, -0.025356526, 0.2805504) * inp_5_0_0;
    result4 += M4(-0.35372454, -0.20961124, -0.30210063, 0.00422873, -0.026797047, 0.41264004, 0.14716934, 0.17331752, 0.026292883, 0.004918332, -0.051546972, -0.016054139, 0.024168568, -0.047611885, -0.00600153, 0.010228018) * inp_5_1_0;
    result4 += M4(0.0317385, -0.0035658793, 0.009292502, -0.14632955, -0.0694484, 0.3153989, 0.16648768, 0.1386909, 0.0500721, 0.20590733, -0.073986135, -0.14541376, 0.198001, -0.10880139, 0.07506918, -0.083366804) * inp_5_2_0;
    result4 += M4(-0.27052194, 0.7283258, -0.008488026, -0.031861115, -0.06565781, 0.05854461, 0.174085, 0.20962426, 0.15503085, 0.33130068, 0.04655114, -0.08213671, -0.13026485, 0.1517414, 0.13126847, -0.028461356) * inp_5_0_1;
    result4 += M4(0.14332606, 0.18916373, -0.25035655, -0.044690076, -0.020169314, 0.23376167, 0.14303003, 0.14762177, -0.11196283, 0.34969014, -0.23020244, -0.113988034, -0.2166457, 0.30026403, -0.037950184, -0.269857) * inp_5_1_1;
    result4 += M4(0.20335521, -0.052311372, -0.100223385, 0.09924354, -0.16730756, 0.31251746, 0.16191798, 0.12054772, 0.030264327, 0.12931336, 0.07330821, 0.09944456, 0.04286417, 0.24256432, 0.19816239, 0.03033326) * inp_5_2_1;
    result4 += M4(-0.07357705, -0.4208868, 0.12074321, 0.36884278, -0.09554001, 0.015795717, 0.28783467, 0.04633824, 0.08806586, -0.044392228, -0.062060043, 0.03159248, -0.046541926, -0.11355886, 0.027589064, -0.24376585) * inp_5_0_2;
    result4 += M4(-0.015735624, 0.042314436, 0.42418265, 0.077094145, -0.19685428, 0.028288433, 0.22858512, -0.0067444523, 0.117741436, -0.108968906, -0.052300062, 0.0890944, -0.028744733, -0.24966753, -0.10020637, -0.3978485) * inp_5_1_2;
    result4 += M4(0.13346243, -0.102846, 0.046968807, 0.0854485, -0.13525823, 0.1674217, 0.12095065, -0.007321016, 0.02245272, -0.150626, -0.09621295, -0.17770272, 0.016198304, -0.3456139, -0.07487196, -0.5700557) * inp_5_2_2;
    result5 += M4(0.30594563, -0.018937707, -0.07306166, 0.019432912, 0.019736672, 0.052632995, -0.024670003, 0.13008162, 0.19831255, -0.113841176, 0.040722128, 0.18870874, -0.19422273, 0.052451786, 0.013929134, 0.0009200542) * inp_5_0_0;
    result5 += M4(0.13253415, -0.35493645, 0.27973637, 1.2351238e-05, 0.14368786, 0.25855276, -0.21331435, 0.110914685, -0.15960205, -0.18310966, 0.32096964, -0.006206369, -0.0034697158, 0.21208645, 0.10589063, 0.23245919) * inp_5_1_0;
    result5 += M4(0.065597676, -0.04536384, 0.077541076, -0.07732827, 0.00014954497, 0.08102704, -0.1258364, 0.15455875, -0.060550038, -0.13895264, 0.0914704, -0.4653797, -0.020356532, 0.039303184, 0.07868435, -0.27036962) * inp_5_2_0;
    result5 += M4(0.5559151, -0.4060976, -0.07886797, -0.2654331, -0.20162897, 0.15251076, -0.06483635, -0.12674637, 0.09628911, 0.033937715, 0.07028453, 0.098716, -0.003241189, -0.037591178, -0.021781636, -0.010433061) * inp_5_0_1;
    result5 += M4(0.40464467, -0.093215086, 0.3907174, 0.09822702, -0.3560824, 0.2625027, -0.034832958, 0.038390677, 0.52254015, 0.13850096, 0.11835889, -0.16749962, 0.07286739, 0.07584743, -0.33088276, -0.23803027) * inp_5_1_1;
    result5 += M4(0.10953877, -0.09293623, 0.0628522, -0.11450387, -0.1847622, 0.08576417, -0.17211074, 0.020992484, 0.06325261, 0.16472903, 0.25206658, -0.015899835, 0.19572306, 0.05843721, 0.074118376, 0.16290721) * inp_5_2_1;
    result5 += M4(-0.06206529, 0.032157753, -0.014956615, 0.18824854, -0.026266225, 0.023532553, -0.07477337, -0.035476685, -0.019576633, 0.044104323, 0.005521541, 0.064861335, 0.07662172, 0.002714053, 0.049170323, -0.047434907) * inp_5_0_2;
    result5 += M4(0.045274906, 0.18046752, -0.02809469, 0.076151505, -0.1982726, -0.01912711, -0.17288469, 0.042038497, -0.16467032, 0.07882285, 0.1405938, 0.014561342, -0.45040387, 0.3373341, -0.006642543, -0.3228949) * inp_5_1_2;
    result5 += M4(-0.093980186, -0.10993653, -0.037408657, 0.14909902, -0.1294475, -0.0062998054, -0.030386297, -0.071690254, -0.20219885, 0.026689725, -0.016860986, -0.03706212, 0.08151476, 0.032799076, 0.19684929, 0.13802202) * inp_5_2_2;
    result6 += M4(-0.33581957, 0.20260203, -0.22764531, -0.28632843, -0.146644, 0.26330504, -0.01022747, -0.0033477664, -0.014059887, -0.8603286, -0.06921629, 0.03228121, 0.014301179, -0.031542215, 0.063073084, -0.040472653) * inp_5_0_0;
    result6 += M4(-0.06717991, -0.009646381, 0.02732235, 0.1274669, 0.048896573, 0.07872911, 0.25053513, 0.08602215, -0.10266047, -1.0337867, 0.041411147, 0.02490564, 0.008699029, -0.12857491, -0.26008874, 0.019609367) * inp_5_1_0;
    result6 += M4(0.12152833, -0.35888916, -0.035550322, 0.058259003, 0.0036720948, 0.34338498, 0.115635894, 0.03151176, -0.020283883, 0.06137313, -0.14979821, -0.085091904, 0.02906739, 0.1484161, -0.104917124, -0.006774204) * inp_5_2_0;
    result6 += M4(-0.20112155, -0.033210244, 0.36217034, -0.05408529, -0.047538057, 0.28837854, 0.10213259, -0.028558053, 0.070677295, -0.055319227, 0.069504194, 0.20385735, 0.059155047, 0.07462485, -0.0312256, -0.09205638) * inp_5_0_1;
    result6 += M4(0.5170241, 0.29268783, 0.13486183, 0.061109923, 0.0011509192, 0.46286628, 0.15045835, 0.103644684, 0.31327948, -1.5328175, 0.5178739, 0.3828924, -0.096817285, -0.26823494, 0.4754328, -0.2945344) * inp_5_1_1;
    result6 += M4(0.06643228, 0.1700416, 0.022526404, -0.06576408, -0.10972851, 0.4683019, 0.05312041, 0.04330977, -0.019914908, -0.15495166, 0.012860715, 0.054094926, 0.021316653, 0.05540104, 0.008438292, 0.15602084) * inp_5_2_1;
    result6 += M4(0.14017381, -0.28933823, 0.12204811, -0.3860809, -0.013388125, 0.2306963, 0.10070297, 0.04147838, -0.03560395, -0.4624745, -0.12645622, 0.019949598, -0.017876433, 0.026715582, 0.019251976, -0.10064264) * inp_5_0_2;
    result6 += M4(0.021116575, -0.3101362, -0.30330476, 0.15934777, -0.045162372, 0.26735824, 0.13624084, 0.024864461, -0.06806867, -0.20988037, 0.15126626, -0.023588207, -0.385337, -0.097864285, -0.33538872, 0.034173843) * inp_5_1_2;
    result6 += M4(0.036115598, -0.123193525, -0.030735068, -0.008219425, -0.0036680587, -0.08108464, 0.029074775, 0.08041572, -0.12546709, 0.060379565, 0.09227953, -0.040782463, -0.22204459, -0.008417623, -0.0190456, -0.016591636) * inp_5_2_2;
    result7 += M4(0.22353604, 0.035015937, -0.2513624, -0.037259582, 0.026980583, -0.032921746, -0.19650617, -0.0012597531, -0.0064086383, -0.032325596, -0.45835844, -0.031131277, 0.042528197, -0.029440874, -0.23061493, 0.021863682) * inp_5_0_0;
    result7 += M4(0.071267545, 0.2732029, -0.70724857, 0.25894082, -0.12123143, 0.07303607, 0.027789185, 0.006554118, -0.28689393, 0.21228722, -0.23620757, -0.023469953, 0.040462397, -1.2697386, -1.2015591, 0.00028160572) * inp_5_1_0;
    result7 += M4(-0.09894673, 0.10864704, 0.123547286, -0.00057683926, 0.165099, 0.030067194, 0.29170674, -0.021896465, 0.1375905, 0.0798513, 0.07118693, -0.007561804, -0.041677304, -0.8681986, -0.51647735, 0.03090424) * inp_5_2_0;
    result7 += M4(0.017939357, -0.16266039, -0.18439099, -0.16169792, -0.15444373, 0.107698075, 0.1357885, 0.05495409, -0.118641555, 0.114218846, -0.2230028, 0.056490824, -0.06579039, -0.5914926, 0.008284703, 0.049900696) * inp_5_0_1;
    result7 += M4(0.18896924, 0.027918283, -0.36245006, 0.4226571, -0.097352326, -0.115854695, 0.11358072, -0.0067420183, -0.14796425, 0.23714444, -0.1115796, 0.081392325, 0.17170931, -2.135466, -0.5055936, -0.028148508) * inp_5_1_1;
    result7 += M4(0.11371933, 0.017042927, 0.015433228, -0.01115436, 0.039796922, -0.10439906, 0.35259372, 0.09214934, -0.1303379, 0.29055238, -0.20374791, 0.20005547, -0.24787268, -1.8632693, -0.79904664, -0.07036433) * inp_5_2_1;
    result7 += M4(-0.07742241, 0.16682592, -0.42569503, 0.0974911, -0.09371333, 0.13049638, 0.10437687, 0.01800017, -0.051312752, 0.22275555, -0.06473052, 0.015220334, -0.028365135, -0.035275374, -0.0070555825, 0.054430902) * inp_5_0_2;
    result7 += M4(0.10842117, 0.28937048, -0.38518763, 0.0964572, -0.1671835, 0.16297257, 0.14645459, 0.016164454, -0.010926163, 0.10627971, 0.11753186, 0.052070748, -0.038104355, -0.66763073, -0.10524279, -0.050321367) * inp_5_1_2;
    result7 += M4(0.08515938, -0.036547925, 0.033673424, 0.10610447, -0.06476227, 0.06993997, 0.28219965, 0.0076430384, -0.026306007, -0.032061033, 0.18640462, 0.019480411, -0.15643016, -0.5355557, -0.19308998, 0.042276785) * inp_5_2_2;
    const V4 inp_6_0_0 = inp[6][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_6_1_0 = inp[6][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_6_2_0 = inp[6][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_6_0_1 = inp[6][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_6_1_1 = inp[6][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_6_2_1 = inp[6][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_6_0_2 = inp[6][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_6_1_2 = inp[6][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_6_2_2 = inp[6][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.02554211, 0.07744702, -0.024418041, 0.15153724, -0.023430903, 0.014052021, -0.18667015, -0.7911414, 0.00072390155, 0.4163709, 0.09562851, -0.4966297, -0.03935244, 0.12351399, 0.030085197, 0.1174386) * inp_6_0_0;
    result0 += M4(-0.016999884, 0.040003862, -0.33244145, 0.4296528, -0.022767859, -0.04045051, -0.1983611, -0.630222, -0.077218525, 0.21529457, 0.11000972, -0.6185189, 0.014621757, 0.061595753, -0.076581, 0.038369507) * inp_6_1_0;
    result0 += M4(-0.0051569473, 0.04503566, -0.1361863, 0.16653568, -0.13875686, -0.01553151, -0.084816195, -0.6014816, -0.0021135712, 0.13927451, 0.25928786, -0.3285113, 0.030393971, -0.0405229, 0.18427138, 0.016744757) * inp_6_2_0;
    result0 += M4(0.052452713, 0.094814874, 0.13619639, 0.2747701, -0.121091425, 0.107315935, -0.07349878, -0.40692028, 0.047797453, -0.37850082, -0.059443105, 0.0014374686, 0.03776917, 0.049518023, -0.011358806, 0.02388875) * inp_6_0_1;
    result0 += M4(-0.1802712, 0.017876612, -0.14287536, -0.2259923, 0.2956988, 0.09131457, -0.13477683, -0.9235386, 0.3047582, 0.12562588, 0.18112747, 2.068224, 0.019819725, 0.061412837, -0.09091754, -0.14641896) * inp_6_1_1;
    result0 += M4(0.041672926, 0.103211045, 0.026871648, 0.3364003, -0.0113490615, 0.031248398, -0.064774245, -0.7095907, -0.0053242953, -0.06075507, -0.13786021, -0.5521421, 0.035773907, 0.06785926, 0.053457096, -0.045810062) * inp_6_2_1;
    result0 += M4(0.06709196, 0.23162779, 0.035915487, 0.09552267, -0.011442273, -0.0015742115, -0.23695719, -0.8701855, -0.023039319, -0.04126108, -0.21102381, -0.30478168, 0.0015156391, -0.0066749468, -0.033343595, 0.11977291) * inp_6_0_2;
    result0 += M4(0.09509097, -0.13093823, 0.2705805, -0.038353674, -0.103463516, 0.111047566, 0.37676468, -1.1096169, -0.1351548, 0.0036166483, -0.1597475, -0.23881646, 0.052819442, -0.011922481, 0.034513213, 0.052985396) * inp_6_1_2;
    result0 += M4(0.03605376, 0.037997834, 0.42870912, 0.17584464, 0.05017586, -0.08559289, 0.1508809, -0.44809866, 0.016347362, 0.012869627, 0.077784315, -0.18583691, -0.016675858, -0.038467005, -0.12804075, 0.016477637) * inp_6_2_2;
    result1 += M4(0.06301087, 0.1403381, -0.07963075, 0.048461, 0.07101672, -1.3372579, 0.1956093, -0.23565066, -0.114091694, -0.58893716, 0.1981051, 0.11341081, 0.06990829, 0.10959167, 0.01489626, -0.12549037) * inp_6_0_0;
    result1 += M4(-0.08689, 0.12563494, 0.03237184, -0.09320023, 0.07460693, -0.8753314, 0.48406044, -0.0739048, 0.020560192, 0.02096202, -0.18160294, -0.012432031, 0.08512327, 0.14253975, 0.07218307, -0.43830886) * inp_6_1_0;
    result1 += M4(-0.05988951, 0.0831657, 0.17328683, -0.12145579, 0.22108531, -0.16210976, 0.071001574, 0.3235073, 0.1282964, 0.00044646166, -0.013259236, -0.2316078, 0.039150063, 0.06747711, -0.048596304, -0.31477332) * inp_6_2_0;
    result1 += M4(-0.18243872, 0.22190087, -0.07605159, 0.043277636, -0.13077766, -0.5400914, -0.5497879, 0.11942623, -0.3590817, 0.1085857, 0.011134467, -0.084051944, 0.06848934, -0.0885015, 0.12264271, -0.17307977) * inp_6_0_1;
    result1 += M4(0.047512624, -0.07367485, 0.05208513, 0.18419676, -0.17072037, -0.21177939, 0.3431036, 0.3125232, 0.1867467, 0.25153622, -0.5964309, -0.41185933, -0.09405385, -0.27035588, -0.12137445, -0.34342775) * inp_6_1_1;
    result1 += M4(-0.09074356, 0.11293092, 0.020685464, -0.05971528, 0.17633794, -0.067124136, -0.00019469883, -0.15714592, 0.06713706, -0.04478597, 0.062013805, 0.64895636, -0.10277648, -0.053389225, -0.058587473, -0.4458292) * inp_6_2_1;
    result1 += M4(0.09062542, 0.116598636, -0.40916926, 0.02961168, -0.20095737, -0.16582748, -0.20305528, 0.15728304, 0.07220996, -0.07411098, -0.37069824, 0.12021228, 0.088758826, -0.016529305, 0.13680707, -0.18960212) * inp_6_0_2;
    result1 += M4(-0.054255735, -0.012894846, 0.18350577, -0.032945354, 0.008759321, -0.082796656, -0.14220168, 0.055522088, -0.11635498, 0.033086892, 0.27577728, -0.30648023, 0.009086531, 0.009218282, -0.043053407, -0.16683419) * inp_6_1_2;
    result1 += M4(-0.03380047, 0.053069945, 0.020929933, 0.07613244, -0.026669791, -0.011348423, 0.0057422076, -0.27110958, -0.19751008, 0.020866094, 0.045513257, 0.047638066, 0.057541214, 0.108123675, 0.054911226, -0.2682856) * inp_6_2_2;
    result2 += M4(0.030554844, 0.03572045, 0.08971279, 0.01652394, 0.046688832, -0.037887387, 0.02906391, -0.18157773, -0.10594964, -0.14757167, 0.005299762, -0.020030348, 0.07878509, -0.01722027, -0.033057913, 0.025998147) * inp_6_0_0;
    result2 += M4(-0.014847093, 0.089911476, 0.11082298, 0.0005709791, -0.18800282, 0.15928213, 0.22855939, 0.06054978, -0.0037365442, -0.07741353, -0.36414835, -0.062060993, 0.047810435, 0.022866638, 0.05022791, 0.0049745636) * inp_6_1_0;
    result2 += M4(-0.108438075, 0.07276686, 0.07861518, -0.0015500378, -0.09804497, -0.051180348, 0.2696045, -0.05341598, -0.008164914, 0.036335275, 0.0063460986, 0.31167513, 0.14278056, 0.05135019, 0.03639911, 0.022181788) * inp_6_2_0;
    result2 += M4(0.01515217, -0.05972563, 0.10860694, 0.06792553, -0.014763401, 0.0011145299, -0.29192817, -0.083999, 0.08010126, 0.0732326, -0.29788172, -0.018053373, 0.017531559, 0.0030025013, 0.04558925, -0.067517824) * inp_6_0_1;
    result2 += M4(-0.07215598, -0.09037577, 0.22486974, -0.10290897, 0.09132236, 0.15899985, 0.022946395, 0.13236414, -0.2972901, -0.68435293, 0.25743288, -0.020176867, 0.03282471, -0.036433842, 0.12010067, 0.017577305) * inp_6_1_1;
    result2 += M4(-0.0023169292, 0.15019262, 0.13848993, 0.061225537, 0.027837854, -0.08499973, 0.21663721, 0.24510784, -0.015838549, 0.26059788, -0.048127506, 0.2358253, 0.029443732, 0.006703943, 0.022144716, -0.039009023) * inp_6_2_1;
    result2 += M4(0.16876288, -0.024801118, 0.050072238, -0.008756673, -0.15700471, -0.16056415, -0.27113128, -0.025277304, -0.1986563, 0.04173933, -0.031018646, -0.07911903, 0.12940681, -0.04358332, 0.036251154, 0.017324904) * inp_6_0_2;
    result2 += M4(0.33520144, -0.10391006, 0.07291232, 0.09855798, 0.02480283, 0.057064813, -0.026664361, 0.09725241, -0.2588608, 0.20381095, -0.317244, -0.21990608, 0.043280087, -0.0023544354, 0.0038878003, -0.034675848) * inp_6_1_2;
    result2 += M4(0.015395662, 0.12929063, 0.06649255, -0.09612641, 0.57240075, -0.18645278, -0.0124980975, -0.13218981, 0.014395084, 0.48081312, -0.08888066, 0.35074776, 0.12919791, -0.0099899415, -0.06565642, -0.089743115) * inp_6_2_2;
    result3 += M4(0.06883064, -0.057717584, 0.01803959, 0.12844858, -0.10306227, 0.09794397, 0.0863854, 0.124853306, -0.07858926, -0.08402526, -0.04178928, 0.06160594, -0.2450585, 0.0628357, 0.0018402097, -0.01161053) * inp_6_0_0;
    result3 += M4(0.15120168, 0.108163886, 0.03036749, 0.15287119, -0.16090207, 0.018266557, 0.16903245, 0.37980324, 0.38741803, 0.0077719507, 0.31128472, 0.06017408, -0.1980548, -0.00090680603, 0.022282233, -0.0132925175) * inp_6_1_0;
    result3 += M4(-0.031443696, 0.08624399, 0.026635211, 0.1396155, 0.08009599, -0.110973425, 0.08318253, -0.03999903, -0.24286763, 0.10452462, 0.13269652, 0.05466685, -0.18813983, 0.0058034067, 0.06859745, 0.086272106) * inp_6_2_0;
    result3 += M4(-0.023468439, 0.043534853, -0.108139426, 0.008979633, 0.07027594, -0.14962007, -0.09339089, 0.11596425, 0.112790234, -0.06622885, -0.044322878, 0.054951098, -0.23457555, 0.005809642, -0.111951366, -0.0049428116) * inp_6_0_1;
    result3 += M4(0.09414019, 0.15507178, -0.06333664, 0.29515556, 0.14755109, 0.051581405, 0.0877036, 0.0739484, 0.060109966, 0.1631263, -0.05584321, -0.27533403, -0.2849042, 0.036321517, -0.11017913, 0.007943473) * inp_6_1_1;
    result3 += M4(0.09476126, 0.039871424, 0.019560594, 0.09989946, -0.10024173, 0.0778952, 0.1845869, 0.276957, 0.12282948, -0.17908837, -0.19138953, -0.087821946, -0.3460273, -0.010741593, -0.05045522, -0.025797265) * inp_6_2_1;
    result3 += M4(0.1481878, 0.06135316, -0.051243793, 0.059758868, 0.14224742, 0.04001503, -0.14079423, 0.070318624, 0.03562578, -0.07775703, 0.086967066, -0.27094206, -0.07871006, -0.023395196, -0.020041378, 0.03305436) * inp_6_0_2;
    result3 += M4(0.10090711, 0.11844167, -0.13368364, 0.044686973, -0.031314988, -0.05383369, 0.16003092, -0.21765998, 0.17847377, -0.17903939, -0.45932135, -0.47132406, -0.3520896, 0.050510325, -0.058249526, 0.08146978) * inp_6_1_2;
    result3 += M4(0.066563524, 0.017553398, -0.015322324, 0.2321099, -0.3125129, 0.13780308, -0.045165587, -0.22084086, 0.12487621, 0.05377831, 0.03221283, 0.07549885, -0.36101085, 0.02821294, -0.04445907, 0.12941076) * inp_6_2_2;
    result4 += M4(-0.13506386, 0.03842655, 0.16469559, 0.21084332, -0.14746653, 0.04172134, -0.025348667, 0.13090482, -0.029035471, 0.031605802, -0.036562927, -0.060143013, 0.085671194, 0.06915232, 0.06010845, 0.0018067133) * inp_6_0_0;
    result4 += M4(-0.10480867, 0.08242977, 0.4578821, 0.16401339, -0.15168807, 0.24365044, 0.16651563, 0.13792564, 0.121270835, -0.15398774, 0.0727199, 0.32841575, -0.014364466, 0.11983018, 0.05106531, -0.019346854) * inp_6_1_0;
    result4 += M4(-0.12958956, 0.30685604, 0.24078217, -0.015170546, -0.18511446, 0.020673124, -0.05799226, -0.045372695, 0.068803586, -0.28622958, -0.09357191, -0.16906284, 0.051287893, 3.9853425e-05, 0.08355953, 0.070748195) * inp_6_2_0;
    result4 += M4(-0.12920976, 0.11291544, 0.26080427, 0.16616657, 0.10032693, -0.055476926, 0.29485765, -0.042359486, 0.30542177, -0.074679226, 0.044018403, 0.33273107, 0.021678234, 0.060723893, 0.040537808, -0.056632828) * inp_6_0_1;
    result4 += M4(-0.35992992, 0.21483326, 0.043514322, 0.04897874, 0.3273311, 0.24129705, 0.097419254, -0.23559771, -0.5265218, -0.03682042, 0.20875251, -0.4795714, -0.10202399, 0.0160771, -0.064918295, 0.018511934) * inp_6_1_1;
    result4 += M4(-0.144579, 0.15784535, 0.19414255, -0.020178316, -0.11852958, 0.066231675, 0.14207882, -0.07034715, -0.34137923, 0.27115488, -0.048077103, -0.29734957, 0.0629785, 0.03671773, 0.0670557, 0.052941386) * inp_6_2_1;
    result4 += M4(-0.14744425, 0.25444528, 0.29128835, 0.045332868, 0.047209885, -0.17739187, -0.20945491, 0.05152537, 0.2425553, -0.32453316, -0.08712204, -0.091713466, -0.04763246, -0.0056311227, 0.07784398, 0.06603898) * inp_6_0_2;
    result4 += M4(-0.27279872, 0.12544765, 0.28977922, 0.015065573, -0.075183176, 0.10000506, -0.06249289, 0.082963064, 0.11124724, 0.16819118, -0.21639255, 0.5378149, 0.044404905, 0.013186591, 0.018554615, 0.06581168) * inp_6_1_2;
    result4 += M4(-0.09003893, -0.018327512, 0.15697041, 0.04077778, 0.00345966, -0.49304023, -0.15555747, 0.16794479, 0.024455167, 0.25910136, -0.16202061, -0.07934215, 0.04598001, 0.09871698, 0.10884198, -0.023563324) * inp_6_2_2;
    result5 += M4(0.13247137, 0.077544376, -0.053462382, 0.07273316, -0.047620438, 0.20425045, 0.12240195, 0.18982022, 0.0017753788, 0.13026522, 0.09163035, -0.026869291, 0.12643963, 0.023823466, 0.057187367, 0.16654919) * inp_6_0_0;
    result5 += M4(-0.14644991, -0.06896274, -0.038552176, 0.199033, -0.15528241, 0.19566211, -0.24062428, -0.195031, 0.025931241, 0.13968824, -0.056348644, 0.36377433, 0.08011223, -0.15994817, -0.03477257, 0.21986581) * inp_6_1_0;
    result5 += M4(-0.08705414, -0.052528586, -0.011306336, 0.14791024, -0.52213085, 0.1810334, -0.32274976, 0.11590132, -0.11789773, 0.09354141, -0.1150531, -0.07500506, 0.08184097, 0.015210023, -0.007633739, 0.10295149) * inp_6_2_0;
    result5 += M4(-0.05555538, 0.06273866, -0.09662574, -0.11352926, 0.10439168, -0.075580746, -0.16789676, -0.079062715, 0.010375286, -0.02276374, -0.031388823, 0.29523483, 0.05582425, -0.05562616, 0.009806833, 0.22876297) * inp_6_0_1;
    result5 += M4(-0.02875634, 0.15170816, -0.14159216, 0.040079072, -0.13054544, 0.07351159, -0.025207605, -0.08092293, 0.63648444, -0.92699355, 0.0028966058, -0.27761716, 0.1544509, -0.13193534, 0.04920723, -0.0205639) * inp_6_1_1;
    result5 += M4(-0.10689725, 0.19327272, -0.13059151, -0.14937985, -0.1549823, -0.24249773, 0.10282246, -0.26077056, 0.14244461, 0.014561346, -0.51212525, -0.06691555, 0.21119426, -0.13463072, 0.036399048, 0.2096091) * inp_6_2_1;
    result5 += M4(0.037318893, 0.0008983787, -0.030899402, -0.07357411, 0.018773714, -0.11582316, 0.033785757, 0.24119996, -0.33148748, 0.051354926, 0.14566797, -0.28620693, 0.08340365, -9.142858e-05, 0.027614512, 0.1816172) * inp_6_0_2;
    result5 += M4(-0.23726664, -0.047994766, -0.10958592, 0.13060708, 0.36556077, 0.13014118, 0.401155, 0.26454228, 0.065630995, -0.100662485, -0.18927349, 0.39813513, 0.11238146, 0.16372772, 0.06648947, 0.048237782) * inp_6_1_2;
    result5 += M4(-0.010550503, 0.18396983, 0.079867125, -0.33097604, 0.41419566, 0.004755278, 0.2547697, -0.1506039, -0.11233768, 0.16041337, 0.23667647, -0.22030793, 0.15458658, 0.032686476, 0.10122217, 0.25559014) * inp_6_2_2;
    result6 += M4(0.050327625, 0.10366902, 0.05545286, 0.09035585, 0.06409729, 0.098914936, 0.008696648, -0.18077758, 0.11236776, 0.19511458, 0.08712243, -0.026530936, 0.049533814, 0.1260343, 0.011647987, 0.0025547147) * inp_6_0_0;
    result6 += M4(-0.0155219445, -0.04321342, 0.030783555, 0.120680265, -0.038936187, 0.19250818, 0.017674165, 0.13425419, -0.15890658, -0.23392259, 0.26764384, 0.028105592, -0.028123982, 0.10901221, 0.062468912, 0.017694183) * inp_6_1_0;
    result6 += M4(-0.113430634, 0.40961385, -0.015149894, 0.034155093, -0.0353955, 0.09943453, 0.3004747, 0.11484952, -0.06272926, -0.14362554, 0.030788675, -0.07219544, -0.0534973, 0.16228275, 0.01032987, -0.004604275) * inp_6_2_0;
    result6 += M4(0.0152951935, -0.049755, 0.11579843, -0.014901592, 0.105060235, 0.11359176, -0.21377382, -0.07372498, -0.015089203, -0.087468624, -0.078505166, 0.16052014, -0.026044002, 0.3176443, 0.095587425, -0.03649125) * inp_6_0_1;
    result6 += M4(-0.13489315, -0.2901348, -0.0656313, 0.090838686, 0.08930315, 0.113264665, -0.26894227, -0.15436609, 0.18638995, -0.22352514, 0.21916735, -0.032502692, -0.027689235, 0.175667, 7.0512346e-05, -0.051296856) * inp_6_1_1;
    result6 += M4(-0.13678068, -0.03068411, 0.2103188, -0.020822931, -0.21989788, 0.13320516, 0.11124637, 0.05637984, -0.018840803, -0.49788606, -0.02272034, 0.047461532, 0.06121307, 0.07874275, 0.04007267, 0.051745653) * inp_6_2_1;
    result6 += M4(0.040244725, -0.067534395, 0.18680903, 0.16991082, 0.17893831, -0.09188219, -0.1712988, -0.0530733, -0.1492634, -0.48008597, 0.049112678, 0.028155874, 0.042270094, 0.3552218, 0.016090712, -0.020213433) * inp_6_0_2;
    result6 += M4(0.029844455, 0.04744869, 0.13974808, 0.0750974, 0.0754716, -0.18063356, -0.35210112, -0.010917656, -0.13829483, -0.25879243, -0.05621147, 0.0052711875, 0.10863528, 0.2153956, 0.019905303, -0.02896781) * inp_6_1_2;
    result6 += M4(-0.018907513, -0.14595622, 0.10142134, 0.010558421, 0.3481138, 0.26497486, 0.27373058, 0.08170226, -0.031480897, 0.1393268, -0.17492798, -0.012162793, 0.07182298, 0.27278912, 0.084131934, -0.017478343) * inp_6_2_2;
    result7 += M4(-0.022006035, 0.01569903, 0.2793054, 0.02701718, -0.047470417, -0.010653964, 0.047892574, 0.06327027, 0.1930436, -0.21843028, -0.4699941, 0.029902097, -0.015638493, 0.11430064, -0.35629588, 0.00019260842) * inp_6_0_0;
    result7 += M4(0.08829834, -0.19257887, -0.08679723, 0.00680864, 0.12093978, -0.2612015, 0.18951748, 0.05592177, 0.08267878, 0.22073723, 0.38105658, 0.014223674, -0.045152113, -0.037320945, -0.9021703, 0.022921514) * inp_6_1_0;
    result7 += M4(0.12318522, 0.22379102, -0.17142676, -0.010063686, -0.22774106, 0.11408792, -0.17825946, -0.038566418, 0.04140236, -0.32677004, -0.15207101, -0.0571964, -0.07458408, -0.055167787, -0.56430095, -0.018872658) * inp_6_2_0;
    result7 += M4(-0.07242291, 0.43656325, 0.26928622, -0.005875697, -0.12139801, 0.19551739, 0.23662119, -0.0012539359, 0.09125523, 0.06479916, -0.12607913, -0.050422214, -0.017271182, 0.1000989, -0.35842967, -0.005022966) * inp_6_0_1;
    result7 += M4(0.021229379, 0.14988922, 0.066033356, 0.060849797, 0.22452664, -0.1913882, 0.20445152, 0.24586418, -0.07552609, -0.16863655, -0.038956873, 0.106139824, -0.058656234, -0.027115209, -0.731348, 0.012820299) * inp_6_1_1;
    result7 += M4(0.03995205, 0.27957252, 0.24205667, 0.035212114, -0.16340932, -0.21233234, 0.21678792, -0.10831469, -0.16464627, 0.11238891, 0.07307667, -0.090126336, -0.12169019, -0.06645016, -0.53591573, 0.002631701) * inp_6_2_1;
    result7 += M4(-0.10880473, 0.19471544, 0.06856859, 0.08144502, 0.12706591, -0.033524234, -0.077424675, 0.083009556, 0.12465934, 0.04966622, -0.004327327, -0.024524631, 0.007115687, 0.091156214, -0.24433082, 0.048691254) * inp_6_0_2;
    result7 += M4(-0.018061558, 0.27482298, -0.12898123, 0.077373646, 0.11773608, 0.48891, 0.068160884, -0.0068038777, -0.18683214, 0.043283585, -0.15192051, -0.045558978, -0.028945293, 0.06737425, -0.2216428, -0.018925382) * inp_6_1_2;
    result7 += M4(-0.087976135, 0.3291667, 0.10394501, 0.051541634, 0.08101955, -0.11883137, -0.35696185, -0.14469975, 0.016686898, -0.013064125, 0.14121872, -0.08132158, -0.011372301, -0.0350572, -0.28620684, 0.01249251) * inp_6_2_2;
    const V4 inp_7_0_0 = inp[7][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_7_1_0 = inp[7][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_7_2_0 = inp[7][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_7_0_1 = inp[7][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_7_1_1 = inp[7][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_7_2_1 = inp[7][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_7_0_2 = inp[7][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_7_1_2 = inp[7][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_7_2_2 = inp[7][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.09257075, 0.1823096, -0.21634662, -0.25379652, -0.0025957804, 0.10681744, -0.08325174, 0.1743923, 0.057517294, -0.068730846, 0.12554735, 0.08650721, 0.029450264, -0.17472, 0.27317837, -0.15452634) * inp_7_0_0;
    result0 += M4(-0.036932047, -2.065985, -0.3719439, -0.115121156, 0.050020564, 0.006469929, 0.12021046, 0.1992383, -0.024084078, -0.06867952, -0.0028965974, 0.0056226137, -0.19865005, -0.16249627, 0.39291838, 0.2539792) * inp_7_1_0;
    result0 += M4(-0.23656343, -0.2575132, 0.083062775, -0.13419586, 0.02752362, 0.10736333, 0.16248056, 0.10934449, 0.013672868, 0.057374895, 0.04411359, 0.043841586, -0.11461909, 0.17587823, 0.27269801, -0.28666005) * inp_7_2_0;
    result0 += M4(-0.056932498, 0.12718183, 0.00048660688, -0.5197762, 0.01735784, 0.19958942, 0.08166421, 0.20704007, -0.008264947, -0.046440337, -0.024515186, -0.066474855, 0.033833954, -0.076467425, 0.12453373, -0.011585898) * inp_7_0_1;
    result0 += M4(-0.0475151, -0.28636134, -0.27979732, -0.9287847, -0.0045237644, 0.014252944, 0.02724634, -0.10431216, -0.054273926, -0.093869254, 0.091343254, 0.014670863, -0.00066608744, -0.073834136, 0.1901047, 0.07698162) * inp_7_1_1;
    result0 += M4(-0.111369684, 0.043285556, -0.038784184, -0.22606437, 0.08880649, 0.116440915, -0.13419522, 0.22330141, -0.013734063, -0.011867255, 0.05734393, -0.10033826, 0.062242247, 0.009681916, 0.0040939716, -0.091452144) * inp_7_2_1;
    result0 += M4(-0.03808723, -0.04799844, 0.12195753, -0.08831741, 0.050927192, 0.17643793, 0.14960499, 0.14824942, 0.0023632166, -0.06735791, 0.042163853, 0.038584117, 0.011221525, 0.027536586, 0.0055412683, 0.08810711) * inp_7_0_2;
    result0 += M4(0.03575236, 0.022775283, -0.11010542, 0.14720671, 0.043158624, 0.11889263, 0.060671538, 0.10300634, -0.02777418, -0.14403127, 0.0367001, 0.15907368, -0.019941635, -0.21546663, -0.08150749, 0.22259325) * inp_7_1_2;
    result0 += M4(-0.012518301, -9.111923e-05, -0.05663723, -0.13010322, -0.04857725, -0.009619181, -0.043953504, 0.019242108, -0.029413305, -0.058366008, 0.13193797, 0.062321052, 0.05284819, -0.12509969, 0.0054996116, -0.019377986) * inp_7_2_2;
    result1 += M4(0.31445038, -0.18468784, -0.92315316, 0.041040275, 0.14034447, 0.26258203, -0.0883373, -0.016717043, -0.062176432, 0.108352594, -0.053673018, 0.19547302, -0.17527695, -0.06344751, 0.5602042, -0.005027008) * inp_7_0_0;
    result1 += M4(-0.32873982, -0.49315986, 0.24323827, 0.3463301, 0.044669524, 0.14362168, 0.08570736, -0.22540037, 0.07945901, -0.123155355, -0.037267637, 0.30521023, 0.04953096, -0.029991433, -0.08064395, 0.2719282) * inp_7_1_0;
    result1 += M4(0.10787076, 0.12518436, -0.110058054, -0.1725516, 0.032350708, 0.19855124, 0.024285343, -0.5567124, 0.03301571, 0.02506064, -0.018912788, 0.30541286, 0.18252933, 0.1964207, -0.22832543, 0.14036581) * inp_7_2_0;
    result1 += M4(0.13301784, -0.1665132, -0.39003882, 0.09996974, 0.04471893, 0.19547157, 0.026940608, -0.17091234, -0.0481267, 0.10940903, -0.026770605, 0.24025734, -0.18289664, -0.029949246, 0.35663837, 0.061731175) * inp_7_0_1;
    result1 += M4(-0.19513749, -0.2976664, 0.53127754, 0.3007941, 0.050161276, 0.20516406, -0.11889578, -0.15312485, -0.07209259, 0.22662938, -0.08088177, 0.07483369, 0.03959702, -0.10487753, -0.022142384, -0.20863502) * inp_7_1_1;
    result1 += M4(0.024800219, 0.1539604, -0.07702112, -0.15008195, -0.028713726, 0.21390188, 0.08317736, -0.49165925, 0.040413246, 0.010288751, -0.015621482, 0.2224857, -0.010004072, 0.20140088, 0.0046554226, 0.10162428) * inp_7_2_1;
    result1 += M4(-0.023567665, 0.020326814, 0.0008811964, -0.10746292, 0.029010793, 0.081790365, -0.06417961, -0.12531413, -0.11656075, -0.010130611, -0.05483415, 0.202158, 0.05284007, 0.12873663, 0.012832633, -0.06707291) * inp_7_0_2;
    result1 += M4(0.037590124, 0.05392248, -0.045089837, 0.05184555, -0.018314935, 0.053204577, 0.009799874, -0.114775434, -0.03252722, -0.0054555805, -0.06575052, 0.29755193, 0.03286822, 0.10985806, 0.1881194, -0.017056609) * inp_7_1_2;
    result1 += M4(0.03762424, -0.025325758, -0.100530386, 0.060901776, 0.06320857, 0.110067084, 0.115468465, -0.022804378, -0.046564426, -0.09131337, -0.04104487, 0.16642635, -0.021010594, 0.08180086, -0.017347889, 0.051905546) * inp_7_2_2;
    result2 += M4(0.12385039, 0.098067254, 0.052663863, 0.08789354, -0.013140244, 0.10089518, 0.067286335, -0.011846045, 0.011184258, 0.015282317, 0.02738507, 0.010259672, -0.13370132, 0.059474178, 0.17470318, -0.012346385) * inp_7_0_0;
    result2 += M4(-0.17962502, 0.04916664, -2.6416447, 0.085862055, 0.08530624, 0.011363604, 0.015994117, 0.048806496, -0.011565776, 0.012859554, 0.043032866, 0.035567094, 0.046238706, -0.098858856, 0.26257798, 0.14982808) * inp_7_1_0;
    result2 += M4(0.20581011, -0.090510294, -0.2710506, -0.2513647, -0.07252622, 0.12322617, 0.11393274, 0.2046699, 0.07801539, -0.06411973, 0.0013864717, 0.09401051, 0.0048224875, -0.13186488, 0.15891427, 0.27556387) * inp_7_2_0;
    result2 += M4(0.13141695, -0.038767226, -0.2541279, 0.22480704, -0.027198035, -0.0027997948, 0.049900424, 0.07320725, 0.016822625, -0.1022203, 0.07815867, 0.045241322, 0.1270504, 0.035121992, 0.082096204, 0.024754338) * inp_7_0_1;
    result2 += M4(-0.73838395, 0.48079425, -0.8579414, 0.1074272, 0.38432166, -0.040729165, 0.27698967, -0.03885581, -0.010305045, -0.067346334, 0.07206246, 0.114156365, 0.08341371, -0.09932988, 0.32635415, -0.039687693) * inp_7_1_1;
    result2 += M4(0.22048613, -0.1159146, -0.046028875, -0.11373377, 0.10288477, 0.08288194, -0.032990947, 0.05251486, 0.0035005668, -0.10733768, 0.1015883, 0.03655177, 0.30376562, -0.19658901, 0.20173497, 0.22657612) * inp_7_2_1;
    result2 += M4(0.14172935, -0.10968994, 0.0067365402, 0.015081782, 0.14053759, -0.015793804, 0.11176598, 0.034945983, 0.046461914, -0.022059223, 0.081543334, 0.03742281, 0.08080048, -0.026054192, -0.0015458866, 0.064220324) * inp_7_0_2;
    result2 += M4(-0.17799474, -0.18405658, 0.040215287, 0.08674687, 0.23362926, -0.055936914, 0.006259053, -0.005249709, -0.0014769526, 0.04725907, 0.04718699, 0.06929424, 0.23510423, 0.07992328, 0.022283273, 0.12514678) * inp_7_1_2;
    result2 += M4(-0.05956566, -0.1097381, -0.05256863, -0.069504485, 0.25686988, 0.13506725, 0.09434084, 0.08119982, 0.09614062, -0.05260539, 0.039260685, 0.014794827, 0.18891509, -0.07068878, -0.054968596, 0.08830886) * inp_7_2_2;
    result3 += M4(0.32948974, 0.11892133, -0.015815133, 0.06723659, -0.16730273, 0.038505368, 0.053611126, 0.14957684, 0.028853664, -0.031587284, 0.011067537, 0.041573144, 0.08603443, 0.024222149, 0.1250051, -0.013171098) * inp_7_0_0;
    result3 += M4(0.58592504, -0.72488725, 0.2826553, -0.46705317, -0.38755426, 0.10509557, 0.006477338, 0.15284169, 0.08460809, -0.017496072, -0.100483865, -0.07442306, 0.13885997, 0.22010356, -0.051313743, -0.31218573) * inp_7_1_0;
    result3 += M4(0.027536187, -0.07243226, -0.09127496, 0.14158562, -0.16402611, 0.040412277, 0.0749298, 0.111247, 0.10243898, 0.0036254474, -0.012099834, -0.095406756, -0.2329115, -0.026692748, -0.08623917, -0.0011413824) * inp_7_2_0;
    result3 += M4(0.14665607, 0.100553975, -0.13486977, 0.17894213, -0.25710574, 0.048859235, -0.124630205, -0.017491361, 0.1972153, -0.03212562, 0.04436866, -0.033005826, -0.03418697, 0.0989917, 0.10131659, 0.017245898) * inp_7_0_1;
    result3 += M4(0.45015472, -0.22022322, 0.5508265, -1.4129444, -0.6948119, 0.12772906, -0.16938758, -0.055470668, 0.06278085, -0.005148243, 0.113595076, -0.111373305, 0.030957917, 0.1364927, -0.15051973, 0.11412223) * inp_7_1_1;
    result3 += M4(-0.3615535, -0.16168526, -0.15075718, -0.32166702, -0.5428949, 0.10618145, -0.0017500364, 0.23080012, 0.13775952, -0.031159773, -0.004048453, 0.0067615323, -0.083669774, -0.055527404, 0.059350673, -0.036503248) * inp_7_2_1;
    result3 += M4(-0.04678892, -0.07387867, 0.099080905, 0.11554786, -0.13216124, -0.011282474, -0.09489431, 0.022179415, 0.16701922, -0.0334658, 0.042804915, -0.13451414, -0.05508301, 0.077934094, 0.07881383, 0.052105464) * inp_7_0_2;
    result3 += M4(-0.011567561, -0.16438258, 0.1273543, -0.82896626, -0.1741835, -0.032530338, -0.07371471, 0.23394975, 0.2059264, -0.0046676546, 0.06238557, 0.01845773, -0.06685044, 0.18264367, 0.12635581, 0.28824037) * inp_7_1_2;
    result3 += M4(0.044634756, 0.019820673, -0.050058514, -0.11838367, -0.087716475, 0.09745679, 0.07907154, 0.12463668, 0.18470399, 0.028270064, -0.036570773, -0.07677773, 0.07904386, -0.035329394, 0.14564128, 0.33286792) * inp_7_2_2;
    result4 += M4(-0.16951898, -0.45461822, -0.3777659, -0.0068328143, 0.07642861, -0.04236647, 0.25282952, 0.21490464, -0.037035394, -0.07076986, -0.058417324, 0.09401361, 0.01436107, 0.11073637, -0.022391189, -0.0717221) * inp_7_0_0;
    result4 += M4(0.7755056, -0.14915438, 0.4054099, -0.2130427, -0.08057498, 0.08843309, 0.23816705, 0.048431188, -0.070702426, -0.14406799, -0.074575774, -0.013034677, -0.16990985, 0.089815885, -0.31112406, -0.3150487) * inp_7_1_0;
    result4 += M4(0.10254068, -0.37337875, -0.071062766, -0.10920985, 0.010573911, -0.0679011, 0.096667, 0.05770162, -0.015494273, -0.16443747, -0.045292743, 0.04818919, -0.16820647, 0.05802256, -0.18253657, -0.31437087) * inp_7_2_0;
    result4 += M4(-0.06975527, -0.5047302, -0.08155872, 0.19796138, 0.01893739, 0.010762465, 0.19794074, 0.012300038, -0.041130982, -0.08660414, -0.02053633, 0.11943582, -0.08128938, -0.02767936, 0.123259336, 0.13595067) * inp_7_0_1;
    result4 += M4(0.4307433, -1.540729, 0.0985482, -0.3302482, -0.093534224, 0.1800425, -0.1349392, -0.04024939, -0.041173484, -0.10759565, 0.07422125, -0.08695835, -0.056209896, 0.03994458, 0.04870259, -0.16061658) * inp_7_1_1;
    result4 += M4(0.12336959, -0.095608965, -0.37683403, 0.23644653, 0.019481767, 0.100876994, 0.0112018185, -0.031556204, 0.019517958, -0.13495052, -0.059812948, -0.011855447, -0.1806456, -0.25378397, -0.033591572, 0.08680823) * inp_7_2_1;
    result4 += M4(-0.010366952, -0.010218297, 0.02366543, -0.11070375, -0.092333056, 0.021522691, 0.2136385, -0.06535332, -0.0002846615, -0.101884656, -0.015534375, -0.018892014, -0.07717659, -0.04452094, 0.04036574, 0.18923932) * inp_7_0_2;
    result4 += M4(0.07733287, -0.09956635, -0.069819495, -0.49024153, -0.02064955, -0.021273822, 0.11240914, 0.16112603, 0.07049809, -0.029704412, 0.002096932, 0.04814634, -0.020329686, 0.088689625, 0.09295962, 0.34249434) * inp_7_1_2;
    result4 += M4(0.012854247, 0.021245686, 0.054059833, -0.1119854, -0.10715301, 0.0948816, -0.020418718, -0.021110307, 0.07621415, -0.12015224, 0.010143108, 0.17206231, 0.07081813, 0.01787992, 0.17576353, 0.31826866) * inp_7_2_2;
    result5 += M4(-0.09209378, 0.036302153, -0.089181654, -0.3037794, 0.15944195, 0.029055655, 0.031485718, 0.17087236, -0.09041098, 0.1218491, -0.06422955, -0.3321583, -0.13741407, 0.01003598, -0.17733777, -0.29776245) * inp_7_0_0;
    result5 += M4(0.059123643, 0.1528886, 0.55153006, 0.6113851, 0.1875372, -0.25683224, -0.010734723, 0.18341663, -0.02896469, 0.12045122, 0.02102857, -0.5280833, -0.07218491, 0.24040075, -0.21656618, -0.6535103) * inp_7_1_0;
    result5 += M4(-0.13255166, -0.69899255, -0.07704418, -0.24290663, 0.089602195, -0.08946614, 0.0372958, 0.16803958, -0.038444467, 0.11187477, -0.02934904, -0.46748722, -0.22143742, -0.32200933, -0.41928336, -0.7596293) * inp_7_2_0;
    result5 += M4(0.12676817, 0.06857442, -0.022434434, -0.06583523, 0.155758, 0.020884592, -0.0033867226, 0.06384078, -0.08360943, 0.047078576, -0.03894262, -0.2683771, -0.15388885, 0.05576919, -0.10507181, -0.35077456) * inp_7_0_1;
    result5 += M4(-0.032265693, 0.41475025, 0.51574695, 0.48461667, 0.11525045, 0.032542262, -0.11913611, -0.04161357, -0.17507613, 0.03172466, -0.0919083, -0.48202175, -0.016603319, -0.1814033, -0.0084224455, -0.40434504) * inp_7_1_1;
    result5 += M4(0.087911986, -0.3204088, -0.04078081, -0.111445986, 0.13860992, -0.02096413, -0.06660991, 0.29854894, -0.007243396, -0.04756991, -0.08573929, -0.457365, -0.17065442, -0.30737776, -0.1273082, -0.42537898) * inp_7_2_1;
    result5 += M4(0.02869501, -0.05173706, 0.053873833, 0.10931508, 0.1688689, -0.06488933, -0.04092949, 0.15278383, 0.023351632, 0.030324584, -0.014810881, -0.24021189, -0.005375224, 0.09504683, -0.035216182, 0.22338305) * inp_7_0_2;
    result5 += M4(-0.01234053, -0.13413978, 0.124272436, -0.05903395, 0.09385258, -0.01308072, 0.023532402, 0.35167715, -0.030866338, -0.058234002, 0.017115206, -0.34579366, -0.0634183, 0.02548044, -0.11234873, 0.1683788) * inp_7_1_2;
    result5 += M4(-0.023091305, -0.043829437, -0.025253436, 0.026751475, 0.077774175, -0.14556573, 0.10866342, 0.04775005, 0.0062078894, -0.05834163, -0.04838948, -0.41028214, -0.011431193, -0.014920897, -0.082430825, 0.14114983) * inp_7_2_2;
    result6 += M4(-0.023988044, 0.10887966, 0.32101345, 0.054344803, 0.014951986, 0.0988241, 0.11104643, 0.06821826, 0.016528731, -0.4618753, 0.009097619, -0.00063811353, -0.118487716, -0.40569106, 0.025157297, -0.086738974) * inp_7_0_0;
    result6 += M4(0.3232896, -0.52883613, -0.74659497, 0.043277845, -0.10874067, 0.22208337, 0.20571963, 0.07118125, 0.10849081, -0.3429098, 0.007220143, -0.0023241045, -0.17740256, -0.26300833, 0.28011033, 0.028230794) * inp_7_1_0;
    result6 += M4(-0.30248666, 0.18460858, -0.017572064, -0.027710875, -0.064185895, 0.04065146, 0.1710959, 0.046098966, 0.03584504, -0.28151676, 0.051117126, -0.006066532, -0.014350776, -0.03398035, 0.6452413, -0.0023246398) * inp_7_2_0;
    result6 += M4(0.19178744, -0.1661538, 0.23750949, -0.38347486, 0.07588044, 0.12599078, 0.13216814, -0.05027521, 0.017124766, -0.93582094, -0.0043101264, -0.027899059, -0.14753398, -0.7299096, -0.011643046, 0.028248545) * inp_7_0_1;
    result6 += M4(-0.090660445, -0.027681096, -1.9365726, 0.24689785, -0.07816528, 0.14774697, 0.19470988, -0.061978143, 0.040296584, -0.62360555, 0.11818987, 0.008942323, 1.6091565e-06, -0.41237465, 0.3682221, 0.14384195) * inp_7_1_1;
    result6 += M4(0.07081382, 0.20237054, 0.46001932, -0.09583214, 0.084871, 0.014530999, 0.13605662, 0.007981865, 0.06227881, -0.17546709, 0.099631205, 0.04055302, 0.09144396, -0.45246652, 0.61086553, 0.02222795) * inp_7_2_1;
    result6 += M4(0.11355308, 0.0038564485, 0.015479103, -0.043335218, 0.034059085, 0.38513327, 0.15273865, 0.023949001, -0.027136061, -0.37513795, -0.0048039705, 0.009610154, 0.012627281, -0.43115616, 0.035060734, 0.0139967045) * inp_7_0_2;
    result6 += M4(-0.024619397, 0.23577401, -0.01048606, 0.0018951625, 0.16376902, 0.089292504, 0.121285126, 0.0027648655, -0.06557997, -0.25106815, 0.109150074, 0.053799078, 0.17022693, -0.29741582, -0.033159036, -0.023037853) * inp_7_1_2;
    result6 += M4(0.029117163, 0.21340366, -0.021992162, 0.032903284, 0.049556304, 0.00014673216, 0.108583346, 0.018263195, -0.04045054, -0.24324888, 0.007507853, 0.021741783, 0.09200081, -0.008631634, -0.006281902, 0.04291702) * inp_7_2_2;
    result7 += M4(-0.10547108, -0.19692925, -0.15132923, 0.030036984, -0.019735003, 0.078142956, 0.23228022, 0.0012654542, 0.051623825, -0.07914944, 0.36866793, -0.006842854, 0.1214896, 0.07313845, 0.033532064, -0.03636748) * inp_7_0_0;
    result7 += M4(0.02170761, -0.3919867, 0.18488608, -0.26405764, -0.02982302, -0.017649243, 0.14210938, 0.024582567, 0.14683515, -0.10783779, 0.2920112, -0.021575587, 0.012491898, -0.07059595, 0.07191099, 0.005114636) * inp_7_1_0;
    result7 += M4(0.007297819, -0.04229493, -0.23701493, 0.18058556, 0.026938053, 0.19000478, -0.4040429, 0.06015802, 0.053328242, -0.06629156, 0.27804816, 0.018265119, 0.19437718, 0.055896066, -0.0005288111, 0.0073626717) * inp_7_2_0;
    result7 += M4(0.13355558, -0.06260675, -0.009489939, 0.17439817, -0.024337705, 0.35357842, -0.13592887, 0.0065834466, 0.06612364, -0.11898795, 0.31480864, 0.038434792, -0.0473003, 0.19679555, -0.16607133, 0.019419415) * inp_7_0_1;
    result7 += M4(0.081085585, -0.09657316, 0.2199482, 0.36843804, -0.19322807, 0.055356927, -0.04192797, -0.020679774, 0.026782334, -0.08007975, 0.12121738, 0.01976963, 0.0029618784, 0.21333721, -0.45414394, 0.019717894) * inp_7_1_1;
    result7 += M4(0.022468409, -0.3293806, 0.32995376, 0.29518098, -0.0149108935, 0.11616899, -0.11220225, 0.0126380995, 0.0551814, -0.14161077, 0.17651957, -0.006950067, 0.03518372, 0.11540033, -0.008582479, -0.09216686) * inp_7_2_1;
    result7 += M4(0.04801482, -0.07751103, -0.062896214, 0.015426616, 0.00941837, 0.18375099, -0.0059094145, 0.052523013, 0.013963883, -0.12170719, 0.20278795, 0.0084748445, 0.015889602, -0.041927684, 0.045518626, 0.035598315) * inp_7_0_2;
    result7 += M4(0.018428024, -0.114171535, 0.131589, -0.032886814, 0.06650087, 0.051136594, 0.1259049, -0.019208485, -0.032159798, -0.15183116, 0.23116794, 0.0028419327, 0.04104496, -0.101696424, -0.006673692, 0.007396843) * inp_7_1_2;
    result7 += M4(-0.03819762, -0.00028586597, 0.17010991, 0.087190695, 0.078293175, 0.00039077032, 0.11366802, 0.020394213, 0.0032507165, -0.16428013, 0.052588116, 0.06536043, 0.0121761225, -0.10455477, -0.1311449, -0.022974387) * inp_7_2_2;
    const ivec2 output_base = ivec2(gl_GlobalInvocationID) * ivec2(4, 2);
    imageStore(out_image, output_base + ivec2(0, 0), max(result0, V4(0.0)));
    imageStore(out_image, output_base + ivec2(1, 0), max(result1, V4(0.0)));
    imageStore(out_image, output_base + ivec2(2, 0), max(result2, V4(0.0)));
    imageStore(out_image, output_base + ivec2(3, 0), max(result3, V4(0.0)));
    imageStore(out_image, output_base + ivec2(0, 1), max(result4, V4(0.0)));
    imageStore(out_image, output_base + ivec2(1, 1), max(result5, V4(0.0)));
    imageStore(out_image, output_base + ivec2(2, 1), max(result6, V4(0.0)));
    imageStore(out_image, output_base + ivec2(3, 1), max(result7, V4(0.0)));
}

//!DESC ArtCNN C4F32 DS (Conv2D-3-ReLU)
//!COMPUTE 48 32 12 16
//!HOOK LUMA
//!BIND conv2d_2
//!SAVE conv2d_3
//!WIDTH LUMA.w 4.0 *
//!HEIGHT LUMA.h 2.0 *
//!COMPONENTS 4
//!WHEN OUTPUT.w LUMA.w / 1.3 > OUTPUT.h LUMA.h / 1.3 > *
#extension GL_EXT_shader_explicit_arithmetic_types_float16 : enable
#ifdef GL_EXT_shader_explicit_arithmetic_types_float16
#	define V4 f16vec4
#	define M4 f16mat4
#	define F float16_t
#else
#	define V4 vec4
#	define M4 mat4
#	define F float
#endif

const ivec2 ksize = ivec2(3, 3);
const ivec2 offset = ksize / 2;
const ivec2 wg_size = ivec2(gl_WorkGroupSize);
const ivec2 isize = wg_size + ksize - 1;
shared V4 inp[8][isize.y][isize.x];
void hook() {
    const uvec2 local_xy = gl_LocalInvocationID.xy;
    ivec2 base = ivec2(gl_WorkGroupID) * wg_size;
    for (uint y = local_xy.y; y < isize.y; y += wg_size.y) {
        for (uint x = local_xy.x; x < isize.x; x += wg_size.x) {
            const ivec2 input_base = (base + ivec2(x,y) - offset) * ivec2(4, 2);
            inp[0][y][x] = V4(conv2d_2_mul * texelFetch(conv2d_2_raw, input_base + ivec2(0, 0), 0));
            inp[1][y][x] = V4(conv2d_2_mul * texelFetch(conv2d_2_raw, input_base + ivec2(1, 0), 0));
            inp[2][y][x] = V4(conv2d_2_mul * texelFetch(conv2d_2_raw, input_base + ivec2(2, 0), 0));
            inp[3][y][x] = V4(conv2d_2_mul * texelFetch(conv2d_2_raw, input_base + ivec2(3, 0), 0));
            inp[4][y][x] = V4(conv2d_2_mul * texelFetch(conv2d_2_raw, input_base + ivec2(0, 1), 0));
            inp[5][y][x] = V4(conv2d_2_mul * texelFetch(conv2d_2_raw, input_base + ivec2(1, 1), 0));
            inp[6][y][x] = V4(conv2d_2_mul * texelFetch(conv2d_2_raw, input_base + ivec2(2, 1), 0));
            inp[7][y][x] = V4(conv2d_2_mul * texelFetch(conv2d_2_raw, input_base + ivec2(3, 1), 0));
        }
    }

    barrier();
    V4 result0 = V4(0.046154436, 0.013960375, 0.02907767, 0.063306764);
    V4 result1 = V4(0.026477156, 0.007070577, 0.03543984, -0.030506935);
    V4 result2 = V4(-0.02337673, 0.011788571, -0.003582679, -0.009954224);
    V4 result3 = V4(-0.017296355, -0.008060533, 0.014464033, 0.017362526);
    V4 result4 = V4(0.024555132, -0.0063927863, -0.01741822, -0.033612486);
    V4 result5 = V4(0.015374098, -0.020512251, 0.0059450916, 0.00397639);
    V4 result6 = V4(0.023082007, 0.021058742, 0.034707475, 0.008326397);
    V4 result7 = V4(0.028589653, -0.012625527, 0.0026645318, -0.012785573);
    const V4 inp_0_0_0 = inp[0][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_0_1_0 = inp[0][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_0_2_0 = inp[0][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_0_0_1 = inp[0][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_0_1_1 = inp[0][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_0_2_1 = inp[0][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_0_0_2 = inp[0][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_0_1_2 = inp[0][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_0_2_2 = inp[0][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.06951286, 0.019779397, 0.05162767, -0.007805347, 0.18946666, 0.0073214, 0.01389422, -0.19228676, 0.25113657, 0.0025343478, 0.0021805044, 0.19116592, 0.35033876, 0.12718064, -0.030805973, 0.26810288) * inp_0_0_0;
    result0 += M4(0.089663476, -0.02614248, -0.14061718, -0.079488896, -0.52839464, 0.04447717, 0.083985575, -0.28446254, -0.25524792, 0.017291088, 0.08776683, -0.055389214, -4.2576356, -0.030488413, -0.02277366, 0.45490122) * inp_0_1_0;
    result0 += M4(0.059977524, -0.010849769, -0.037362248, 0.055843595, -0.39063445, -0.034424357, -0.059815977, -0.1420803, 0.4274667, -0.0054432, -0.0046827, -0.07502252, 0.011168261, 0.07765353, -0.078019254, 0.30540282) * inp_0_2_0;
    result0 += M4(0.0039019517, 0.11190907, 0.22155654, 0.13197538, -1.1244345, -0.094368845, 0.016058695, -0.12140493, -0.106561296, 0.0393857, -0.04248957, -0.10475548, 0.48158348, 0.015445582, -0.0037666864, 0.08359278) * inp_0_0_1;
    result0 += M4(-0.09696443, 0.17889632, 0.40978572, 0.36920655, -0.36403087, -0.14123814, 0.2604093, -0.10010644, -0.3402141, -0.03542405, 0.0075807734, -0.4194306, -0.2130585, 0.019733038, -0.0673705, -0.6076458) * inp_0_1_1;
    result0 += M4(0.1274611, 0.061678134, -0.069059744, 0.033886757, -1.6520323, 0.02812608, -0.12852877, -0.4201193, 0.5368584, -0.007811236, 0.09607723, 0.0042300355, -0.16226718, -0.04687882, 0.08172674, 0.14690922) * inp_0_2_1;
    result0 += M4(0.15043285, 0.07854987, -0.024468308, 0.2460604, 0.14841013, 0.016798751, -0.02980514, -0.029779613, -0.3099253, -0.054637667, 0.09408032, 0.037301183, 0.24593765, 0.029723465, -0.025993727, -0.033617485) * inp_0_0_2;
    result0 += M4(-0.016269643, 0.03252946, 0.008818762, -0.15300214, -0.4196648, 0.054719042, 0.0002479208, 0.09499802, 0.11044075, -0.0041010077, 0.054913163, 0.07903136, 0.040677443, -0.067261785, 0.087890424, -0.007961537) * inp_0_1_2;
    result0 += M4(0.12747237, 0.021960186, 0.01727953, 0.097056784, -0.22911908, -0.0081683565, 0.045712788, -0.007436028, -0.23822656, -0.054023407, 0.04628943, 0.1331919, -0.56053007, -0.10552344, -0.03628019, 0.088960245) * inp_0_2_2;
    result1 += M4(-0.009355234, -0.054310992, -0.01676259, 0.06918052, -0.048390746, -0.07571518, -0.0783705, -0.0072379853, 0.0803878, -0.020112036, -0.06709986, 0.10181752, -0.26159564, -0.20895459, -0.0756235, 0.15628247) * inp_0_0_0;
    result1 += M4(-0.12715279, 0.14425679, -0.04419111, -0.16368814, -0.16560617, 0.019744117, -0.41781843, 0.008178086, -0.14156614, -0.19039653, -0.05249283, 0.20565422, -0.1018442, -0.11360472, 0.16251156, 0.22831994) * inp_0_1_0;
    result1 += M4(0.003407602, -0.031437296, 0.08021519, 0.040362235, -0.06146887, 0.08931746, -0.09103681, 0.14686425, 0.042816907, 0.05900675, -0.0939002, 0.014792454, -0.25697967, 0.122564875, 0.13414654, -0.00432971) * inp_0_2_0;
    result1 += M4(0.059145182, -0.105585374, -0.0981208, -0.11108025, -0.08063444, 0.038937733, -0.075436786, 0.02212984, 0.024616616, -0.0067723054, -0.081692405, 0.09112649, 0.021156, -0.17274748, 0.16337588, 0.25740525) * inp_0_0_1;
    result1 += M4(-0.13351557, -0.12172447, -0.026794976, 0.22534037, -0.04661557, -0.69538355, -0.256572, -0.25486463, -0.04953525, 0.039785832, -0.20549889, -0.13302028, 0.2821104, -0.7232469, 0.45133018, -0.5428798) * inp_0_1_1;
    result1 += M4(-0.053439815, 0.13302663, 0.11153759, -0.039570738, -0.11528997, 0.012406186, 0.15133482, 0.3668328, 0.057648595, 0.059205584, -0.14385206, -0.025513636, 0.05122079, -0.34025502, 0.2688848, -0.26443735) * inp_0_2_1;
    result1 += M4(0.019102529, 0.060775835, 0.2050022, 0.086999156, -0.031155745, -0.06255218, -0.26443434, -0.13143678, -0.06407257, 0.1232485, -0.05090738, 0.1042784, -0.18905765, -0.030108878, -0.026812484, -0.27497754) * inp_0_0_2;
    result1 += M4(-0.09954791, -0.27384278, -0.006458452, 0.22542115, -0.026197175, -0.011017327, -0.22959547, 0.041788694, -0.06241052, 0.07896614, -0.11692802, -0.1491219, -0.16258177, 0.13394296, 0.0097803855, -0.30948) * inp_0_1_2;
    result1 += M4(-0.10591266, -0.12722185, -0.024432734, -0.22068729, -0.039148256, 0.035194356, -0.17772457, 0.013642409, -0.05494006, 0.17082155, -0.08294896, -0.13544771, -0.03261689, -0.12737052, -0.011576879, 0.16390118) * inp_0_2_2;
    result2 += M4(0.0074353535, 0.04702865, -0.0031961326, 0.011375865, 0.011514046, -0.061751816, -0.039716423, 0.08489028, 0.05988055, 0.013603665, -0.048606094, 0.04658515, -0.2166475, -0.14956984, 0.22029866, 0.007986804) * inp_0_0_0;
    result2 += M4(-0.0269187, -0.09044432, 0.092312984, -0.06273729, 0.07833744, -0.117373444, 0.043080077, 0.02637554, 0.13269916, 0.20717905, -0.09301423, 0.054897215, 0.01302825, 0.1711419, 0.07762405, -0.029467301) * inp_0_1_0;
    result2 += M4(-0.000654707, 0.12697789, -0.033046607, 0.011304374, 0.14342467, -0.009110472, 0.014146314, 0.094060466, 0.06998, -0.032539386, -0.08160519, 0.070437215, -0.009227484, 0.11769687, 0.25654718, -0.22105458) * inp_0_2_0;
    result2 += M4(-0.036879383, 0.11629693, -0.03750081, -0.037961986, 0.10591177, 0.07176364, 0.009715821, -0.03615839, 0.10513894, -0.10294592, -0.07307428, 0.0077805677, 0.01842724, -0.20829228, 0.19468795, 0.15298711) * inp_0_0_1;
    result2 += M4(0.19131754, 0.18734093, -0.34323147, 0.15321547, 0.45373732, -0.26493222, -0.6343523, 0.16481699, 0.08872385, -0.103227995, -0.031077398, -0.10002662, 0.1763534, -0.21961604, 0.6496552, 0.14035755) * inp_0_1_1;
    result2 += M4(0.008969931, -0.10771505, 0.03995839, 0.07400047, 0.074937634, -0.34494287, -0.0715015, 0.23010136, 0.10906552, 0.0329542, -0.19077265, 0.061473094, 0.19321914, 0.3019528, -0.080549136, 0.07150262) * inp_0_2_1;
    result2 += M4(-0.16007602, -0.21483383, 0.022110963, -0.0070522074, 0.115002975, 0.04309177, -0.052068517, -0.02141873, 0.17558105, -0.14927943, -0.12900731, 0.04352755, -0.05196793, 0.16934992, 0.28545588, -0.005935601) * inp_0_0_2;
    result2 += M4(-0.60319704, 0.19608255, 0.32121763, -0.10151406, 0.32423753, -0.06920974, -0.054091156, 0.09517776, 0.108965136, 0.04534261, -0.040984172, -0.054704092, -0.15215333, -0.26527768, 0.11807376, -0.05611492) * inp_0_1_2;
    result2 += M4(0.039111942, 0.18138316, 0.05387097, -0.19146545, -0.0136820795, -0.079328425, -0.0015939092, -0.013573375, 0.053685155, 0.11398319, -0.09183746, -0.031363364, -0.0634117, -0.2444917, 0.08538163, 0.1135766) * inp_0_2_2;
    result3 += M4(-0.12709291, 0.118508175, 0.008290286, 0.033283137, 0.010582378, -0.070530474, -0.08129531, -0.09936263, -0.017091228, 0.067465246, -0.0016500353, 0.06852973, -0.14947744, 0.11587489, -0.059253678, -0.14461514) * inp_0_0_0;
    result3 += M4(-0.050592266, -0.08950886, -0.051200397, -0.09188532, 0.2879743, 0.038398422, 0.00055905356, -0.6466699, -0.16846447, 0.005641283, 0.03167606, 0.017843358, -0.21233244, 0.020948892, 0.028407158, 0.009063505) * inp_0_1_0;
    result3 += M4(-0.04694507, 0.14670978, 0.026435815, -0.043347217, 0.44452995, 0.076857395, 0.059190586, -0.06647892, 0.02920276, -0.13288346, 0.03628442, 0.08186566, -0.17845686, -0.124023385, 0.1447684, 0.006062412) * inp_0_2_0;
    result3 += M4(0.03250506, 0.074914314, 0.098527394, 0.053629205, -0.08524476, -0.17174995, -0.21931873, 0.010506056, -0.06440805, -0.03192244, 0.008509686, -0.059190992, -0.15862316, 0.122784585, -0.18293831, 0.27971342) * inp_0_0_1;
    result3 += M4(-0.44060856, 0.08715103, -0.17462991, 0.36551175, -0.7369859, -0.1543245, -0.015327001, 0.12638216, -0.026045812, 0.02113387, 0.030493762, -0.061263382, 0.19448447, -0.26664352, 0.5360077, -0.09276251) * inp_0_1_1;
    result3 += M4(0.18006678, -0.17003132, 0.028212285, 0.04300493, 0.20922612, -0.18510817, 0.17563888, -0.12895238, 0.028320517, -0.06901847, -0.086758636, -0.03583499, -0.088927135, 0.40625876, 0.3309749, 0.18706091) * inp_0_2_1;
    result3 += M4(0.22414808, 0.1867832, -0.098648824, 0.0035406689, -0.2366818, 0.00806658, 0.0092632845, -0.075685434, 0.029615523, -0.03416249, -0.0074919094, -0.029550744, -0.24968643, 0.078453675, -0.069186136, 0.122736424) * inp_0_0_2;
    result3 += M4(0.1588641, 0.06725589, 0.31129888, -0.1619866, -0.33958918, -0.14921355, -0.086959265, 0.07574261, 0.037583735, 0.090815224, -0.1027763, 0.00975459, -0.15489174, 0.25596768, -0.01784332, 0.21752575) * inp_0_1_2;
    result3 += M4(0.025124595, 0.18744153, -0.13601331, -0.09171938, 0.0075462805, 0.20700577, -0.113255754, 0.12744299, -0.009444729, -0.031318452, -0.11913072, 0.04535059, -0.13910776, 0.23973095, 0.09042233, 0.15458821) * inp_0_2_2;
    result4 += M4(0.06391737, 0.0039220243, 0.017529316, 0.00287849, -0.0052214935, 0.057980727, 0.07378534, -0.35778683, -0.040142354, 0.015200987, -0.04450938, -0.019640585, 0.04303496, 0.07987745, 0.17358467, -0.10930088) * inp_0_0_0;
    result4 += M4(-0.0058635394, 0.009862279, 0.045282014, 0.025810286, -0.009102118, 0.12049984, 0.18821695, -0.2374358, 0.1370771, 0.059573915, 0.027144713, -0.35953414, -0.082624, 0.0600253, 0.10479479, -0.047153443) * inp_0_1_0;
    result4 += M4(0.0199518, -0.0037571078, -0.07502356, -0.09520845, -0.10296961, 0.12885527, -0.10569706, 0.180632, -0.05157356, 0.07463032, 0.06718413, -0.031594682, 0.026291654, 0.06644091, -0.08222021, 0.015301056) * inp_0_2_0;
    result4 += M4(-0.06309402, -0.078927994, -0.022995444, 0.13014755, -0.13916638, 0.09814245, -0.029797567, -0.06942973, 0.06312316, 0.07350114, 0.10144518, -0.060916036, -0.030156652, -0.13975164, 0.14235376, 0.12537953) * inp_0_0_1;
    result4 += M4(-0.20079662, 0.0026046946, -0.18276273, -0.053840313, -0.21683742, -0.12586766, -0.10279752, -0.18240562, -0.090444416, 0.16264774, 0.07925139, -0.038913753, 0.24431558, -0.70967263, 0.27185658, -0.5093012) * inp_0_1_1;
    result4 += M4(0.042150352, 0.040512618, 0.06915201, -0.36175588, -0.0011023969, 0.0542361, 0.06295472, 0.43106422, -0.086685896, 0.068677485, 0.09304363, 0.1437832, 0.00033836684, -0.03182288, -0.22362971, -0.21767725) * inp_0_2_1;
    result4 += M4(-0.122242756, 0.025394334, -0.056329146, -0.17080274, 0.03889781, 0.06294749, -0.0024496627, 0.014174894, -0.034535293, -0.00013896982, 0.008258299, -0.060596097, -0.002026312, -0.04951147, 0.04166718, -0.02197295) * inp_0_0_2;
    result4 += M4(0.028672196, -0.13221942, 0.07398909, -0.07689547, -0.08423586, 0.092441894, -0.12112076, -0.52085, -0.026170688, 0.06907739, 0.007625206, -0.2014018, -0.04945731, 0.019251, 0.010531185, -0.485913) * inp_0_1_2;
    result4 += M4(-0.00536146, 0.03504485, -0.0551228, 0.06560338, -0.085766956, 0.14303471, -0.09732228, 0.09361178, -0.0974903, 0.075091735, 0.030290022, 0.0766803, 0.055410717, -0.01942199, -0.08831329, -0.29051366) * inp_0_2_2;
    result5 += M4(0.065498404, 0.06799123, 0.0038216496, -0.13175164, -0.019074218, 0.029940324, -0.028449511, 0.14847827, -0.0442699, 0.11816907, 0.016081903, -0.14795092, -0.21241282, -0.044859227, 0.051758606, -0.29648665) * inp_0_0_0;
    result5 += M4(-0.03333697, -0.091150925, 0.010649767, 0.037177794, 0.05726686, -0.26647726, -0.015872795, 0.069332495, -0.13598487, -0.22327957, 0.0004573983, 0.048578814, 0.03626917, 0.05001469, 0.1628874, -0.09296081) * inp_0_1_0;
    result5 += M4(-0.027738761, -0.10104413, -0.0025373322, 0.028623072, -0.2662653, 0.42913428, 0.027043337, -0.048535068, -0.07000842, -0.105914295, 0.040752683, 0.05236868, -0.059331566, -0.17652994, 0.13531627, -0.27310503) * inp_0_2_0;
    result5 += M4(-0.032473445, 0.19250882, -0.01916763, -0.098379426, -0.06733589, 0.045296147, -0.0043751164, 0.10489684, 0.08368834, 0.23663212, 0.016421093, -0.053648174, 0.18304358, 0.22611846, -0.07416644, -0.04230073) * inp_0_0_1;
    result5 += M4(-0.5571463, -0.173976, 0.08824351, 0.1376526, -0.12893817, -0.1915168, 0.035938945, 0.22752409, -0.09946212, -0.20644575, -0.0359291, -0.02433645, 0.44610798, -0.099541634, -0.38869512, -0.03557864) * inp_0_1_1;
    result5 += M4(-0.17069197, -0.6691052, -0.030939467, -0.08198899, 0.0032001014, 0.010540122, -0.029664654, -0.35324612, 0.029806554, -0.05479539, 0.010388811, 0.16611464, 0.22220397, 0.019885696, 0.01377033, -0.14952241) * inp_0_2_1;
    result5 += M4(0.0713334, -0.042850714, -0.019191282, -0.046318464, -0.068064354, -0.10664882, -0.04099451, -0.017295552, 0.07995307, -0.007521558, 0.0037181096, -0.03663565, -0.0020996297, -0.0754646, 0.039608598, -0.17752826) * inp_0_0_2;
    result5 += M4(-0.05363486, 0.043373995, 0.061318822, -0.14207011, 0.03231885, -0.17016377, 0.03762814, 0.16577685, 0.034440175, -0.05938638, 0.019922705, 0.01205502, 0.03302831, -0.03250401, -0.029477565, 0.07571145) * inp_0_1_2;
    result5 += M4(0.00052854983, -0.083144315, 0.006499128, -0.07946999, 0.031727962, -0.104348525, 0.02361953, 0.26339206, -0.004641003, -0.14507292, 0.0005490908, 0.10686075, 0.026757918, -0.24957469, -0.021605743, -0.3248276) * inp_0_2_2;
    result6 += M4(0.015744274, -0.043061573, -0.0312825, -0.0058785966, -0.046074055, 0.193629, 0.006300591, 0.057149183, -0.0020167937, -0.043268286, 0.025052601, -0.00053101376, 0.23942477, 0.19674274, -0.20498045, 0.06432375) * inp_0_0_0;
    result6 += M4(0.055747617, 0.07634596, 0.07574122, 0.013270684, 0.07038633, -0.22928134, 0.22058588, 0.17474517, -0.10469031, -0.16915365, -0.1751536, 0.0040804646, -0.4115549, -0.19087152, -0.63980484, 0.060467947) * inp_0_1_0;
    result6 += M4(-0.03609388, -0.11590779, -0.09112573, -0.018190295, -0.025280077, -0.23811592, 0.1231628, -0.06609508, -0.068308204, 0.052280277, 0.13197652, -0.0013807392, -0.013416703, 0.1427796, -0.21534298, -0.084038846) * inp_0_2_0;
    result6 += M4(0.43082967, -0.22863498, 0.012581507, -0.011800958, -0.04766379, 0.2165139, 0.0020707452, 0.12477741, 0.0113508385, -0.06978504, 0.07089052, 0.013786427, -0.16114081, 0.6802867, -0.14240481, 0.09443741) * inp_0_0_1;
    result6 += M4(-0.3145177, -0.22360235, -0.15000963, 0.32505974, -0.46336025, -0.13142928, -0.53259915, 0.17859054, 0.17722073, -0.2775171, 0.29252565, 0.059703153, -0.24386066, -0.600127, 0.3036014, -0.05721123) * inp_0_1_1;
    result6 += M4(-0.09809595, 0.14152771, 0.059068788, -0.009704045, -0.09548149, -0.21053326, 0.1670906, -0.08785032, -0.065750726, -0.020166706, -0.031841137, 0.04425501, 0.328203, -0.0313398, -0.0048878053, -0.031269133) * inp_0_2_1;
    result6 += M4(0.16075963, 0.013924303, -0.07819492, 0.044102218, -0.027565632, -0.30025443, 0.14669763, -0.011541694, 0.050232485, -0.058624398, -0.12951834, 0.061752826, -0.12908117, 0.43016666, -0.087729506, -0.041889656) * inp_0_0_2;
    result6 += M4(-0.27084997, 0.07816237, 0.16300759, -0.07116006, -0.039572787, -0.013424275, -0.0038486766, 0.028613612, 0.1181487, -0.059968587, 0.0012739233, 0.041935373, 0.2927168, 0.32638806, 0.0029953632, 0.07401439) * inp_0_1_2;
    result6 += M4(0.18833265, -0.070133545, -0.14156228, -0.009270609, 0.090669766, 0.11136016, -0.16294423, 0.08520936, 0.081601195, 0.0031610779, -0.09895746, 0.052994043, 0.018211903, 0.23446038, -0.021667562, 0.0815705) * inp_0_2_2;
    result7 += M4(-0.026291095, -0.044596564, 0.014026045, -0.0029043888, -0.026248705, -0.04078686, 0.025817452, -0.009138958, -0.025835961, 0.03465427, -0.008712299, -0.02223619, -0.044627503, -0.23760766, -0.087124474, 0.025098477) * inp_0_0_0;
    result7 += M4(0.01076579, 0.03526675, 0.001001067, 0.10176191, -0.02913142, 0.10236277, -0.008336372, 0.06897406, -0.039721265, -0.011846915, -0.012438076, -0.014024144, 0.0052101403, -0.05815607, -0.083810404, 0.013636348) * inp_0_1_0;
    result7 += M4(-0.11555424, 0.028244024, -0.025945172, 0.05474715, -0.17647214, 0.42859077, 0.061517507, 0.09276712, -0.011963445, 0.08712734, 0.013084217, -0.112095214, -0.11376928, -0.108376145, -0.044165447, 0.15286112) * inp_0_2_0;
    result7 += M4(0.11566234, -0.027429538, 0.029210106, -0.0041026464, -0.06804465, -0.02359381, 0.010030821, 0.004475981, -0.100717016, -0.0076603517, 0.012419813, -0.025153922, 0.027631542, -0.10216466, -0.04145729, -0.12355544) * inp_0_0_1;
    result7 += M4(-0.16366248, -0.31307897, 0.03377862, -0.07458446, 0.11321224, -0.09729926, -0.05003341, -0.12769245, 0.019223114, 0.10633228, 0.06346305, -0.14142166, 0.3862735, 0.57672745, -0.2284711, -0.13648148) * inp_0_1_1;
    result7 += M4(-0.02695027, -0.07196557, 0.08373126, 0.10369958, -0.10540224, 0.15373726, -0.08759694, -0.3027006, 0.06252949, 0.09842554, -0.038096163, 0.015382639, -0.08629417, -0.17272823, 0.003413359, 0.19946119) * inp_0_2_1;
    result7 += M4(0.066617884, 0.026183004, -0.053016208, 0.045350943, -0.073132835, 0.018806472, 0.0020908678, -0.073350854, 0.013564616, 0.030399155, 0.02418392, 0.055707034, -0.056426052, -0.13056414, -0.00607677, -0.06910916) * inp_0_0_2;
    result7 += M4(0.12692305, 0.0064408923, -0.02371121, 0.15204558, -0.08238291, 0.055263326, -0.04158586, 0.042566523, 0.06832422, -0.01007787, 0.003926186, 0.011600929, 0.09846351, 0.017045321, -0.050043005, -0.07953928) * inp_0_1_2;
    result7 += M4(-0.10514263, -0.06962278, 0.06728271, -0.41784155, -0.14084525, 0.07394816, -0.02640144, 0.004722843, -0.016705008, 0.060907282, -0.009551773, 0.043610457, -0.06254988, -0.124559954, 0.019431448, 0.19765809) * inp_0_2_2;
    const V4 inp_1_0_0 = inp[1][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_1_1_0 = inp[1][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_1_2_0 = inp[1][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_1_0_1 = inp[1][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_1_1_1 = inp[1][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_1_2_1 = inp[1][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_1_0_2 = inp[1][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_1_1_2 = inp[1][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_1_2_2 = inp[1][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.1988386, 0.0054602595, 0.044146564, 0.033618074, 0.1545056, 0.009526259, 0.01530943, 0.04196746, -0.20425576, -0.09166625, 0.028138833, 0.20444298, -0.18592341, -0.089466475, -0.074909605, -0.16133162) * inp_1_0_0;
    result0 += M4(0.18341497, 0.025033206, 0.004421821, -0.017691065, -0.10357461, -0.04684573, 0.035004616, 0.07980741, -0.71804386, -0.101172745, 0.10158384, -0.12059224, -0.61895704, -0.01108333, 0.023425296, -0.02327748) * inp_1_1_0;
    result0 += M4(0.15306853, -0.014107632, 0.05198656, 0.07076368, -0.024932275, 0.03388053, 0.0010104345, -6.795602e-06, 0.18340181, -0.08256223, -0.064376175, 0.014236829, -0.038116008, 0.008724553, 0.069428466, 0.03972708) * inp_1_2_0;
    result0 += M4(0.4136795, 0.00094722427, 0.018745895, -0.13758115, 0.12539515, 0.06402323, -0.0813841, -0.22332259, 0.71669316, 0.068657294, -0.1827111, -0.015590233, 0.6961219, 0.07249007, 0.03322086, 0.064379506) * inp_1_0_1;
    result0 += M4(0.5860039, 0.067840956, 0.13436522, -0.20117065, 0.07014513, 0.0224524, -0.14670733, -0.26290333, 0.37097755, 0.111030616, 0.038408183, -0.07457638, -0.13219611, -0.046493605, 0.14705165, 0.1875801) * inp_1_1_1;
    result0 += M4(-0.2436192, -0.060247954, 0.012287919, -0.29952982, 0.078972235, 0.13031784, -0.27129295, -1.3936594, -0.21194653, 0.06187735, 0.028659942, -0.009948176, -0.20334905, -0.008048573, -0.0073528714, 0.028988348) * inp_1_2_1;
    result0 += M4(-0.5416483, -0.010898732, 0.11635703, 0.13324015, -0.3268421, -0.06145291, 0.07124351, 0.114581764, -0.28470373, 0.032700278, 0.017482832, 0.09095668, 0.14511557, 0.012329486, -0.06299881, 0.28753978) * inp_1_0_2;
    result0 += M4(0.47364962, 0.026741227, 0.0016806653, 0.19634284, -0.89420503, -0.06651919, 0.15281938, 0.0053669293, -0.31073382, -0.13646017, 0.09533345, 0.17672504, 0.060543273, 0.023109574, 0.08727492, 0.27828777) * inp_1_1_2;
    result0 += M4(0.23297647, -0.050732765, 0.106334805, 0.040042497, -0.82604027, 0.6648149, -0.80524427, 1.0345731, -0.16003288, -0.026702318, 0.012794376, -0.29546583, -0.3701623, 0.009455742, -0.11385843, -0.2488562) * inp_1_2_2;
    result1 += M4(-0.013468228, -0.00605431, -0.050266, -0.024740783, 0.0013397869, -0.073070735, -0.017071767, -0.0055473875, -0.08594464, 0.006907247, -0.030394504, 0.0071812263, -0.03294932, 0.06809796, -0.08889214, -0.2489744) * inp_1_0_0;
    result1 += M4(0.011257864, -0.037420735, 0.1846257, -0.027069518, -0.07086176, 0.06663869, -0.17507577, 0.015006922, -0.15072216, 0.026576174, -0.20259778, 0.16202913, 0.087356314, 0.09318416, 0.13450952, -0.13195482) * inp_1_1_0;
    result1 += M4(0.020634504, -0.14583038, 0.1555732, 0.117478915, -0.03250777, -0.015990082, 0.17581469, -0.04263481, -0.07302162, 0.2709583, -0.09875189, -0.18981415, -0.01603242, -0.12643099, -0.0021963413, 0.11268879) * inp_1_2_0;
    result1 += M4(-0.051426765, 0.08491675, 0.13036425, 0.028445214, 0.043981187, 0.004802587, -0.0924079, 0.07163765, -0.100459754, 0.079173766, 0.38623792, 0.4294303, 0.060494218, -0.023229998, 0.15775448, 0.061150193) * inp_1_0_1;
    result1 += M4(0.079449885, 0.1508595, 0.07451425, 0.10729157, -0.06223816, -0.06678264, 0.07042088, -0.07268964, 0.15775207, -0.18857141, -0.48987243, 0.11517693, -0.06346297, -0.26216108, 0.08528108, 0.07961056) * inp_1_1_1;
    result1 += M4(-0.025748074, 0.20450148, -0.29961511, -0.32225955, 0.13951488, 0.12079824, 0.18117754, 0.008646176, 0.21103425, 0.04926343, 0.20143406, -0.23135875, 0.020978356, 0.10084631, 0.14529675, 0.1600824) * inp_1_2_1;
    result1 += M4(0.13553809, -0.035804465, -0.026511915, 0.061363947, 0.025140991, -0.0932729, 0.0628409, -0.18237504, -0.10289887, -0.14886063, 0.054720838, 0.065087594, -0.010040872, 0.16282758, 0.10699438, 0.14463736) * inp_1_0_2;
    result1 += M4(-0.16729112, 0.18906567, -0.17308511, -0.046663538, -0.057411417, 0.056463998, 0.07236616, -0.1824285, 0.071636215, 0.08651772, -0.44952774, -0.25238505, 0.20608024, 0.05530626, -0.0889456, 0.057681296) * inp_1_1_2;
    result1 += M4(0.0794609, 0.19360596, -0.13606828, -0.24181016, 0.018126065, -0.07581905, -0.5192884, -0.26033124, -0.15402009, 0.14226022, -0.11499438, 0.17103867, -0.11211229, -0.11834047, 0.1919996, 0.21465893) * inp_1_2_2;
    result2 += M4(0.07443171, -0.004302491, -0.15786174, -0.0041806465, -0.004869633, -0.053016268, 0.08553767, 0.0039982675, 0.015301001, 0.05015959, -0.19957857, 0.04532905, -0.14043358, -0.0664566, -0.0010657143, -0.123311706) * inp_1_0_0;
    result2 += M4(0.009174458, 0.04797834, -0.20989418, 0.09738944, 0.02556251, -0.08525523, -0.14629544, -0.08627995, 0.1116863, -0.005364923, -0.22766754, 0.10786525, -0.06789072, -0.025025975, 0.00096961495, 0.010605945) * inp_1_1_0;
    result2 += M4(0.15354702, 0.25029287, -0.07175941, 0.03899957, -0.018429827, 0.107892305, 0.10225895, 0.03815734, -0.09650221, -0.0149292955, 0.060334273, 0.08909602, 0.028316326, 0.094733596, -0.033883594, -0.07391652) * inp_1_2_0;
    result2 += M4(0.071588315, 0.041667145, -0.35250843, 0.037703056, -0.01765784, 0.106260575, -0.08404227, 0.06802342, 0.059100084, -0.22227077, -0.06750176, 0.16371903, -0.031558923, -0.08377988, 0.035299413, -0.06737014) * inp_1_0_1;
    result2 += M4(0.09479712, 0.0672367, -0.28696075, 0.11044988, -0.039442327, -0.32817516, 0.37441412, 0.1829936, 0.19538046, 0.32622343, -0.27586183, -0.336252, 0.040817566, 0.09240249, -0.069095664, -0.0719477) * inp_1_1_1;
    result2 += M4(-0.054305878, 0.19243714, -0.1090408, -0.07379078, -0.054838527, 0.17802192, 0.10111971, 0.25983968, -0.007975545, 0.12277373, 0.0066030035, 0.24093133, -0.06735211, 0.04703338, -0.0075104986, 0.10596841) * inp_1_2_1;
    result2 += M4(-0.020940458, 0.04945806, -0.15142535, 0.008471336, -0.021987656, 0.16407233, 0.102908514, -0.15698285, 0.154674, -0.15145978, -0.2632699, 0.0997458, -0.0407333, -0.28803664, -0.032603186, 0.027328748) * inp_1_0_2;
    result2 += M4(0.10525753, -0.19018254, -0.17567617, 0.02816753, -0.04386773, 0.073393, 0.36713642, 0.09583013, 0.31529123, -0.20426941, -0.27392888, 0.055675104, 0.030369572, 0.08561224, -0.109823145, -0.056964528) * inp_1_1_2;
    result2 += M4(0.02849281, -0.008196293, -0.09180209, 0.040277064, -0.914123, 1.1896113, 0.5529874, 0.03810221, 0.13893296, -0.13962202, 0.004034014, -0.00425748, -0.061123103, -0.19768463, 0.07596587, 0.034090977) * inp_1_2_2;
    result3 += M4(-0.05084657, -0.037694067, 0.021976534, 0.12028895, -0.011937477, -0.06250223, -0.08684158, 0.02069446, 0.25840005, -0.14897932, 0.028764993, 0.07816405, 0.104842655, -0.009322023, 0.009734571, -0.15578736) * inp_1_0_0;
    result3 += M4(-0.18632862, -0.019758819, 0.008738619, 0.054951087, 0.1447272, 0.0056772702, 0.14040671, -0.09621906, 0.2622453, -0.14036676, -0.108419806, 0.17602322, 0.056464463, -0.04809765, -0.0828882, -0.050471425) * inp_1_1_0;
    result3 += M4(-0.1827896, -0.04045556, -0.011954553, 0.10487714, 0.0067251627, 0.10695996, -0.12262115, 0.13044713, 0.033910777, -0.13509575, 0.11078159, -0.1324995, -0.14390506, -0.06465215, -0.016134713, 0.058515754) * inp_1_2_0;
    result3 += M4(-0.12393574, -0.0023154393, 0.058694623, -0.0043622516, -0.015261968, -0.06794944, -0.012004768, 0.015080888, 0.00612622, -0.19876474, -0.2688852, -0.08570719, -0.005123294, 0.05694314, -0.093938395, -0.032611877) * inp_1_0_1;
    result3 += M4(0.19936475, -0.009214494, -0.032111008, -0.05931715, 0.1256363, 0.018208306, 0.29542863, 0.23783931, -0.06463215, 0.07454446, 0.06312943, -0.28787467, -0.09876626, 0.08685396, -0.13046333, 0.11141386) * inp_1_1_1;
    result3 += M4(0.11856318, 0.07068122, -0.0947468, -0.3112945, 0.09458136, 0.11495281, -0.1844599, 0.0071078553, 0.17772205, 0.27993143, 0.14018115, 0.12405257, 0.17493303, 0.12525019, -0.035818633, 0.15784396) * inp_1_2_1;
    result3 += M4(0.09975333, 0.079248115, 0.016093766, 0.040281102, 0.03687483, 0.14988105, 0.11638173, -0.09193853, 0.04296923, -0.2154733, -0.16664104, 0.060822185, -0.17350562, -0.04250068, -0.1950187, 0.16804828) * inp_1_0_2;
    result3 += M4(0.023817843, 0.013596218, -0.1040873, 0.14572097, -0.22744028, -0.16858302, 0.078116305, 0.15878549, 0.30565697, 0.21048915, -0.12768489, 0.22356766, 0.021026678, 0.26705682, -0.16016133, 0.19624977) * inp_1_1_2;
    result3 += M4(0.07620569, 0.095018625, -0.23784581, 0.034266345, -2.4080093, 1.7103156, 0.74577045, 0.40749606, -0.20708168, 0.042217415, 0.14176482, 0.13776752, 0.12569146, -0.3151861, 0.09590746, -0.17874143) * inp_1_2_2;
    result4 += M4(-0.015089179, 0.0056826835, 0.030577863, 0.21024765, 0.0101314895, -0.029051658, 0.07635113, -0.09838754, -0.1508642, 0.006835429, -0.045607433, 0.16739163, -0.031902373, -0.029702665, 0.056577273, -0.10761323) * inp_1_0_0;
    result4 += M4(-0.060006496, 0.06567641, -0.007609162, 0.07544043, -0.027126556, 0.041597616, 0.074051544, 0.087917544, -0.20514806, 0.07287019, 0.1564442, -0.117985435, 0.035672374, -0.027869413, -0.011098094, 0.10853351) * inp_1_1_0;
    result4 += M4(-0.10479131, 0.110336445, -0.08095285, 0.093493044, -0.07627579, -0.041296326, -0.15793015, 0.086294785, -0.38751748, 0.13563944, 0.04143413, 0.27791274, 0.010915388, 0.017457793, -0.02235303, 0.06721836) * inp_1_2_0;
    result4 += M4(-0.1171415, -0.02827208, -0.057732217, -0.13303365, -0.13631646, -0.046137802, 0.04441444, 0.066795155, -0.18905498, 0.1628085, -0.25704452, 0.24789312, -0.0065180077, 0.01675097, -0.19470605, 0.36057654) * inp_1_0_1;
    result4 += M4(-0.1488759, 0.122772, -0.08061381, -0.0066811396, 0.33471885, -0.09514152, 0.2051452, 0.2981599, 0.065409325, 0.31429365, 0.291167, 0.3103965, 0.15582749, -0.09146738, -0.059332572, -0.17762168) * inp_1_1_1;
    result4 += M4(-0.14745681, 0.080794565, 0.12525876, -0.059929285, 0.19408524, 0.04298378, -0.28258625, -0.18556942, -0.041854184, -0.054993566, 0.0482539, -0.30629757, 0.005640858, -0.04117302, -0.017296933, 0.04509341) * inp_1_2_1;
    result4 += M4(-0.056516666, 0.019854462, 0.01067251, 0.01822582, 0.040110666, -0.0063530025, 0.0985185, -0.1654031, -0.24023455, 0.011521494, -0.10003365, 0.007842321, -0.12012137, 0.050861508, -0.12460202, -0.033481967) * inp_1_0_2;
    result4 += M4(-0.25147864, 0.089730985, 0.03828968, -0.24360693, -0.11741755, -0.26275438, -0.24923034, -0.26511776, -0.21157736, 0.120853566, 0.14877181, -0.0883431, 0.061165, 0.10643368, -0.10429463, -0.41859993) * inp_1_1_2;
    result4 += M4(-0.16595788, 0.1234203, 0.0115244035, 0.19778073, 0.373275, -0.35362032, -0.009689092, -0.20552453, 0.04729637, 0.13081923, -0.09040052, -0.32088268, 0.1018149, -0.06313277, 0.105468504, 0.09496667) * inp_1_2_2;
    result5 += M4(0.0019035229, -0.071477465, 0.009671416, 0.0052092588, 0.00024119286, 0.04141217, -0.013661523, -0.08321293, -0.027303694, 0.024433073, 0.009261303, 0.021681836, 0.013826896, -0.25003514, -0.03261602, 0.0043513617) * inp_1_0_0;
    result5 += M4(-0.080566764, -0.18636137, 0.019104587, 0.10441823, 0.09290826, -0.09301372, 0.035330895, 0.048017632, -0.30087003, -0.05304361, -0.04379451, 0.29104722, 0.035096064, 0.076351814, -0.06600876, -0.04488788) * inp_1_1_0;
    result5 += M4(-0.15909089, -0.097124726, 0.036972117, 0.061198134, -0.048426706, 0.16738251, 0.009145416, 0.05250918, -0.017221222, -0.06564861, -0.054317497, 0.2518092, 0.016360352, -0.034429155, 0.023273366, -0.00231188) * inp_1_2_0;
    result5 += M4(-0.0110062435, -0.28327307, 0.024746144, 0.0492639, -0.075262554, -0.15270227, -0.07594634, -0.10502504, -0.047499064, 0.07545557, -0.0013967687, -0.0050612157, 0.03781431, -0.041764826, -0.011379291, -0.1302261) * inp_1_0_1;
    result5 += M4(-0.046460934, -0.13792439, 0.08064569, 0.25487408, 0.15470919, 0.19944076, 0.35803726, 0.20630333, -0.05630134, -0.3636206, -0.010092173, 0.15224041, -0.10201083, -0.11044216, 0.043847702, -0.094454125) * inp_1_1_1;
    result5 += M4(-0.04835151, -0.33414757, 0.03231619, 0.016448898, -0.21605173, 0.21913482, -0.12778884, -0.21947403, 0.09418751, -0.2353646, -0.014887638, 0.1592975, -0.0041528787, 0.20040655, -0.05485375, -0.12263311) * inp_1_2_1;
    result5 += M4(0.023210736, 0.11674416, 0.036771227, -0.027086785, 0.027207619, -0.13979153, -0.023325834, -0.067774095, -0.06831749, 0.16719241, 0.03779252, -0.08727839, 0.06390591, -0.089551516, 0.019361896, -0.027656512) * inp_1_0_2;
    result5 += M4(-0.0031572836, 0.09927632, 0.036728904, 0.082714945, -0.19862805, -0.28823018, 0.14279631, 0.05639014, 0.11936915, 0.07213571, -0.051020242, 0.2541152, -0.020925274, -0.28099373, -0.023148, -0.1587543) * inp_1_1_2;
    result5 += M4(0.07308756, -0.113399506, 0.03426936, 0.22672129, 0.4133208, 0.3272743, 0.4667867, -1.0969431, -0.01570524, 0.10293636, 0.023587424, -0.017539144, -0.067393854, -0.18799719, -0.06611216, -0.15474099) * inp_1_2_2;
    result6 += M4(0.061731443, -0.14641246, 0.10432394, -0.010476296, -0.1345073, 0.030341882, -0.15210897, 0.06448005, 0.04660598, 0.030254623, 0.13167591, -0.031156138, 0.14782324, 0.095686756, 0.14250791, -0.019283922) * inp_1_0_0;
    result6 += M4(-0.10073969, 0.08130911, -0.0024082966, 0.033272892, 0.13794683, -0.120153785, 0.23196384, 0.026273668, 0.027267272, -0.0786504, 0.026935982, -0.016063496, 0.16309357, -0.117476724, 0.05370403, 0.024303215) * inp_1_1_0;
    result6 += M4(-0.09481845, 0.04277484, -0.025776485, 0.036424406, -0.07726157, 0.02180079, -0.21876535, -0.09221934, 0.041562878, -0.020350395, 0.048983485, 0.006393474, 0.047255788, 0.0066625387, -0.08254016, 0.00061995286) * inp_1_2_0;
    result6 += M4(0.09359129, 0.023760084, 0.052929126, 0.01988713, 0.012054968, 0.11514955, 0.258355, -0.03534341, -0.2590762, 0.13443209, -0.37673292, 0.014098378, 0.15675181, 0.0256226, -0.033415403, 0.03061808) * inp_1_0_1;
    result6 += M4(-0.073194176, -0.117618255, 0.057549078, 0.08969582, -0.22529817, -0.174495, -0.39325768, -0.065112196, 0.26246196, -0.35277364, -0.011657871, 0.056430794, 0.078078866, 0.29031903, -0.32168782, -0.04047242) * inp_1_1_1;
    result6 += M4(0.097254835, 0.017891426, 0.05585119, 0.07537519, 0.13445309, 0.29767215, 0.3889892, -0.086722374, -0.2483614, -0.29257658, 0.04396928, 0.017816054, -0.14810836, -0.1184399, 0.11955252, -0.0071332804) * inp_1_2_1;
    result6 += M4(0.13290083, -0.16055934, -0.053899232, 0.046307266, 0.070560716, -0.03869243, 0.051583152, 0.033817653, -0.06482883, 0.014785309, 0.08072744, -0.1010747, 0.13985972, 0.32470042, -0.29911965, -0.018657131) * inp_1_0_2;
    result6 += M4(0.08997885, 0.15485896, -0.1852022, -0.013753136, -0.021133147, 0.28237924, 0.0035267146, -0.08212916, 0.3151404, 0.16142271, -0.13146627, 0.07536393, 0.0021757842, -0.054012157, 0.073558316, 0.019880014) * inp_1_1_2;
    result6 += M4(0.20922427, 0.0068745348, -0.26352835, 0.11810913, 1.6064372, 0.8762702, 0.21500152, -0.23321883, 0.10260027, -0.12076341, 0.33259374, -0.04720933, -0.23571554, 0.10347611, 0.091237545, -0.048179705) * inp_1_2_2;
    result7 += M4(-0.013978424, 0.038009338, -0.0154894125, 0.049301393, -0.02129118, 0.004365517, 0.034024492, 0.003996351, -0.050696317, 0.14225736, 0.037290443, -0.015402016, -0.03807536, 0.0946111, 0.01729495, -0.08138631) * inp_1_0_0;
    result7 += M4(-0.099587604, 0.055065222, 0.0013320724, -0.09271055, 0.048116405, 0.025215222, 0.020425797, 0.064928904, -0.33174706, -0.018609263, -0.010463793, -0.17651476, 0.045007546, -0.00618884, -0.016288703, 0.094571754) * inp_1_1_0;
    result7 += M4(-0.08119314, -0.20295034, 0.03812513, -0.12471932, -0.08692052, -0.20638876, -0.016946945, -0.099246755, -0.044468872, 0.26204035, -0.049589872, 0.10632394, 0.056319617, -0.07815546, -0.02255231, 0.14602336) * inp_1_2_0;
    result7 += M4(0.00690414, 0.0054765814, -0.012552877, -0.029233845, 0.01730028, -0.12704244, -0.03927571, -0.008608475, -0.009434496, 0.115999535, 0.017657476, 0.15912308, 0.010883523, -0.07266793, 0.007427479, 0.14964388) * inp_1_0_1;
    result7 += M4(-0.19519293, 0.046775248, -0.036788672, -0.104886904, 0.24256492, 0.14321353, 0.31142688, -0.004275195, 0.08905463, -0.4159571, 0.034270864, 0.019216431, -0.048632465, -0.24716666, 0.045400526, 0.13001376) * inp_1_1_1;
    result7 += M4(-0.12341046, -0.03863148, 0.07538117, -0.3810411, -0.070835836, -0.06857866, -0.17591429, 0.054462973, 0.3243944, 0.09873025, 0.09400855, -0.42945477, -0.0666365, 0.12297559, -0.033229128, -0.105945714) * inp_1_2_1;
    result7 += M4(-0.0802008, -0.008279531, 0.015548534, 0.014206742, -0.0069315075, 0.008640391, -0.005935365, -0.044781107, -0.011145791, 0.05160035, -0.030685462, 0.026562523, -0.12151202, 0.028168878, -2.1302458e-05, 0.012255182) * inp_1_0_2;
    result7 += M4(-0.101320565, 0.037738778, -0.016568374, -0.093081415, 0.110122606, -0.015213654, -0.008094084, -0.25273746, -0.2893255, 0.11333214, 0.079169795, -0.28929794, 0.0011590365, 0.0028108486, -0.019269653, 0.17198059) * inp_1_1_2;
    result7 += M4(-0.050644036, 0.0022344887, 0.02202982, -0.38405797, 0.48037487, -2.054239, -0.14562164, 0.9631989, 0.03150362, 0.136017, 0.05482855, 0.05872506, -0.07164614, -0.06738375, -0.028010989, -0.20735262) * inp_1_2_2;
    const V4 inp_2_0_0 = inp[2][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_2_1_0 = inp[2][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_2_2_0 = inp[2][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_2_0_1 = inp[2][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_2_1_1 = inp[2][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_2_2_1 = inp[2][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_2_0_2 = inp[2][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_2_1_2 = inp[2][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_2_2_2 = inp[2][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.16796866, 0.004353339, 0.0054184506, 0.5926852, -0.017795797, -0.038110424, -0.19759876, -0.12338793, -0.68333757, -0.15810074, 0.095472045, 0.099043615, 0.110018075, 0.04480834, 0.042056497, 0.22246031) * inp_2_0_0;
    result0 += M4(-0.2924386, 0.17138077, 0.050570775, -0.09663689, 0.092456914, -0.032277208, -0.11295001, -0.38028744, -0.4136303, -0.047848817, 0.09876979, 0.13101973, 0.090683624, 0.057226922, 0.025636151, -0.052697778) * inp_2_1_0;
    result0 += M4(-0.2287109, 0.07749369, 0.034368277, -0.024881592, -0.01416739, -0.042068936, -0.023845177, -0.058171753, -0.4804336, -0.06840375, -0.11785441, -0.036870398, -0.32341754, 0.0013700515, -0.018601298, -0.04752564) * inp_2_2_0;
    result0 += M4(0.008424971, 0.025559125, 0.15846184, 0.053550195, -0.2516168, -0.005160706, 0.06388292, -0.04711938, -0.7705386, -0.04302366, -0.070726104, -0.04938115, 0.24688135, 0.116303146, -0.051677547, -0.013354144) * inp_2_0_1;
    result0 += M4(-0.28746966, -0.0084152585, 0.18237536, 0.059398487, -0.022249395, 0.11651045, 0.06842811, 0.26222837, -0.06444814, -0.25349498, 0.51050156, 0.26867172, -0.3840454, 0.06673555, -0.14962022, -0.2681442) * inp_2_1_1;
    result0 += M4(0.527523, 0.037615675, -0.055061735, -0.106846355, 0.047272615, -0.040085375, -0.16217825, -0.10879695, -0.5541798, -0.02671524, 0.05088079, -0.30550584, -0.40198305, -0.0013729908, 0.08796638, 0.05865763) * inp_2_2_1;
    result0 += M4(0.2099581, -0.019793453, 0.013541644, -0.097204275, -0.10257805, 0.0067334357, 0.0073221, -0.07154536, -0.11064953, -0.076970935, -0.027543537, -0.21572599, -0.10167316, 0.023516823, -0.002333305, 0.04975315) * inp_2_0_2;
    result0 += M4(-0.18100037, 0.028864423, 0.011441089, -0.015455287, 0.240073, 0.08128771, 0.058474585, -0.2112817, 0.00037095963, 0.023627348, -0.16182774, -0.68081844, -0.029083552, 0.042480927, -0.03035021, -0.06861952) * inp_2_1_2;
    result0 += M4(-0.14396313, -0.0023734001, -0.010389819, 0.12403447, -0.052763294, 0.008478368, -0.1219792, -0.04940808, 0.2237656, -0.074575365, -0.08708139, -0.44339558, 0.44223574, 0.0045847003, 0.054876704, -0.152421) * inp_2_2_2;
    result1 += M4(-0.011707395, -0.09962801, 0.008612756, 0.09604416, 0.055201158, 0.0381196, 0.0026006172, 0.00022704952, -0.17164141, -0.02230813, -0.26402, 0.043514755, 0.022584904, 0.06575987, -0.044399966, 0.07669866) * inp_2_0_0;
    result1 += M4(0.07548363, 0.10927711, 0.011474695, -0.093051255, -0.060811434, -0.0017685575, -0.0060103987, -0.21835616, -0.075002186, 0.18541583, -0.19243209, -0.25457636, 0.06280915, 0.12926906, -0.021202765, -0.05278301) * inp_2_1_0;
    result1 += M4(0.03980123, -0.12385114, -0.029518493, -0.045866728, 0.06646418, -0.016651003, -0.04675601, 0.100417644, -0.13829665, 0.28475478, -0.069307305, 0.060925666, -0.11641976, 0.069572, -0.028824179, -0.15625282) * inp_2_2_0;
    result1 += M4(0.2056749, -0.16667442, -0.02229444, -0.31709072, -0.05514729, -0.067053765, -0.22359464, -0.09847569, -0.1892892, -0.09552566, -0.86858314, -0.56844693, -0.03160675, 0.23806843, -0.009148416, 0.23079793) * inp_2_0_1;
    result1 += M4(0.19613485, -0.038145162, -0.059052717, -0.08339325, -0.12251883, 0.31992054, 0.022160832, 0.041065324, 0.31243813, -1.3044038, 0.45996866, -0.8326036, 0.19670276, -0.1523588, 0.10555632, 0.4291141) * inp_2_1_1;
    result1 += M4(0.12878488, -0.02590294, 0.06485415, -0.16799995, 0.038311012, 0.12576225, -0.057857033, 0.1136882, -0.17871478, 0.2271076, -0.010293557, 0.03673423, 0.08163675, -0.14451587, -0.16550776, -0.11743061) * inp_2_2_1;
    result1 += M4(0.038797043, 0.16966794, 0.036277324, -0.04819128, -0.07024815, -0.08535062, 0.03917454, 0.08635323, -0.090231635, 0.23768136, -0.2019117, 0.1847939, -0.13217543, -0.051281527, -0.024165733, 0.030874956) * inp_2_0_2;
    result1 += M4(-0.023358261, -0.096292265, -0.11218812, 0.16391852, 0.033278838, -0.17058586, 0.008301309, 0.02024151, -0.22358918, -0.06740451, -0.062633835, 0.30467743, -0.028081866, 0.031460434, -0.07451596, -0.014324713) * inp_2_1_2;
    result1 += M4(0.026020331, 0.088890836, -0.029147588, -0.059216164, -0.106523044, 0.015882654, -0.10239954, -0.08906523, -0.25990754, -0.02373817, -0.22597113, 0.24217868, 0.052078042, 0.031313553, -0.048289686, 0.024771824) * inp_2_2_2;
    result2 += M4(-0.1510171, -0.06750697, 0.19829446, 0.06582949, -0.0416908, -0.072204836, 0.1521069, 0.06753058, 0.2165122, 0.120841235, -0.24101985, 0.028412458, 0.102186, -0.16616674, -0.04985226, 0.05470794) * inp_2_0_0;
    result2 += M4(-0.011526009, -0.36327633, -0.059588715, -0.07470683, 0.0677345, -0.07161266, 0.065055445, -0.018050112, 0.094672725, -0.27729115, -0.17406212, 0.10977267, -0.13978001, -0.08675093, 0.04516709, -0.024959126) * inp_2_1_0;
    result2 += M4(-0.069633216, -0.09431509, 0.05612249, 0.06026514, 0.022623118, 0.0038358322, -0.07830149, 0.120782115, 0.10927947, 0.1268199, -0.031243501, -0.025118968, -0.066690095, 0.043115586, 0.061689384, -0.029984808) * inp_2_2_0;
    result2 += M4(0.024914926, 0.039008044, -0.01832356, -0.004632074, -0.010852857, 0.041286986, 0.07850815, -0.015828231, 0.30448204, 0.2979819, 0.032937523, 0.04017069, 0.09107081, -0.4106371, -0.09947203, 0.033914864) * inp_2_0_1;
    result2 += M4(-0.12743509, -0.11811281, -0.23554735, 0.108667396, -0.08782739, 0.05478144, -0.11083635, 0.017756969, 0.21912862, -0.92735595, -1.109251, 0.5745608, -0.08557118, -0.28914216, 0.070147716, -0.13181236) * inp_2_1_1;
    result2 += M4(0.15195248, -0.14227135, -0.07971597, 0.0065304674, 0.061417136, -0.20862703, 0.059257638, 0.0882106, 0.19109264, -0.123845465, -0.27176642, 0.34075546, 0.120893665, 0.23161809, 0.04707949, -0.035445336) * inp_2_2_1;
    result2 += M4(-0.013529831, -0.022940312, -0.009601299, -0.0536066, 0.00025246694, -0.15078953, 0.075209744, -0.0049211164, 0.23991391, 0.127908, -0.21497871, 0.11350103, -0.0543662, 0.04977138, 0.0109277675, -0.0016229078) * inp_2_0_2;
    result2 += M4(0.12954764, -0.052079067, -0.14278384, 0.035688117, -0.2520885, 0.22592853, 0.16211717, -0.14190033, 0.6270509, 0.15725836, -0.31629115, -0.032813966, -0.06411179, -0.10535897, 0.06361146, -0.01691294) * inp_2_1_2;
    result2 += M4(-0.018468335, 0.027626904, -0.05055208, 0.061254077, 0.05917375, -0.0060797743, -0.017242532, -0.02689842, 0.23692109, 0.15172073, -0.31722766, 0.11301315, 0.029994426, -0.023863927, -0.008489874, 0.019727888) * inp_2_2_2;
    result3 += M4(0.16527294, -0.09380304, -0.08672205, -0.1701417, -0.03010085, 0.008312595, 0.010580416, 0.08833726, 0.35778663, -0.10493297, -0.14320737, -0.27177605, -0.0024336665, 0.016569298, -0.1710352, 0.06302167) * inp_2_0_0;
    result3 += M4(-0.15828262, 0.033815574, -0.009354891, 0.029020471, -0.09180932, -0.015920298, 0.048797745, -0.00478824, 0.67244595, -0.045032296, -0.0050689024, -0.5656135, 0.15368229, -0.027694702, 0.031192418, -0.24503872) * inp_2_1_0;
    result3 += M4(0.1763768, 0.19239193, 0.27304986, -0.04775033, 0.11524437, 0.0715663, 0.05248719, 0.021061992, -0.09079018, 0.094271995, 0.05968597, -0.19545074, 0.007746074, -0.04959414, 0.05019197, -0.09173144) * inp_2_2_0;
    result3 += M4(-0.32136166, -0.027326232, 0.018649917, -0.08533539, -0.20027304, -0.048908316, 0.17787115, 0.006199194, -0.20934959, 0.06497865, 0.11384246, -0.03768858, -0.2985639, -0.08771137, -0.05826763, 0.12412598) * inp_2_0_1;
    result3 += M4(0.03745142, -0.05709661, 0.050765503, 0.17343873, -0.19251391, 0.3096964, -0.033066515, 0.40886095, -2.1284034, -0.52457255, -0.38623872, 0.10975521, 0.22697149, -0.33716655, 0.10795724, -0.011751002) * inp_2_1_1;
    result3 += M4(0.032941777, -0.003896465, 0.108371936, -0.1913977, 0.162677, -0.27645046, -0.12759498, -0.06327332, 0.37016657, -0.5781647, 0.40498784, 0.1226416, -0.2103161, -0.0019373508, -0.069461346, 0.032594997) * inp_2_2_1;
    result3 += M4(-0.022256022, -0.085107006, 0.010827431, 0.007973852, -0.035806693, 0.017222114, -0.0015001994, -0.06374226, -0.093380235, 0.02402912, -0.041378655, 0.02036917, -0.01369511, -0.09597312, -0.10405183, -0.11907352) * inp_2_0_2;
    result3 += M4(-0.03701094, -0.07193948, 0.046038844, -0.11959648, -0.03339999, 0.13885306, 0.13666733, -0.15682282, -0.48363134, -0.25885093, 0.04900421, -0.061521597, 0.13577989, -0.009364361, -0.06320263, -0.009002746) * inp_2_1_2;
    result3 += M4(-0.039080236, 0.2029637, -0.04133338, 0.12597074, 0.06997844, -0.10113617, -0.0372917, -0.03908648, 0.21855399, -0.001245031, -0.06034478, -0.114035875, -0.01501142, 0.09432319, 0.04559112, -0.010965633) * inp_2_2_2;
    result4 += M4(0.08019493, -0.18203428, -0.0174601, 0.03522669, 0.074471466, 0.06413948, -0.0035519807, 0.10811151, 0.072337925, 0.19095275, 0.20878573, -1.4606493, -0.05781689, 0.021275587, -0.113412395, 0.2889052) * inp_2_0_0;
    result4 += M4(-0.00577664, 0.032095887, 0.17807475, -0.016819643, 0.17904407, -0.0045221928, 0.057097282, -0.09209523, -0.30450496, 0.2351884, 0.13900481, -0.30083084, 0.032386202, 0.048129104, 0.15909198, 0.32841262) * inp_2_1_0;
    result4 += M4(0.004348034, -0.09251223, -0.0028018085, -0.13166447, -0.09683142, 0.08172835, 0.026222926, 0.1684497, -0.2959546, 0.121384114, -0.032636054, 0.46111268, 0.021532305, 0.047813334, 0.030068835, -0.037517443) * inp_2_2_0;
    result4 += M4(0.04408362, -0.114188686, 0.16408928, -0.19901325, 0.09525925, -0.025316456, -0.070804365, -0.061345477, -0.009410928, 0.3274071, 0.39892676, 0.10814234, 0.15117131, -0.16041869, -0.23787072, -0.4551405) * inp_2_0_1;
    result4 += M4(0.04635987, -0.13223577, 0.060699593, 0.4696408, 0.031376954, -0.053886306, -0.09591562, 0.1717115, -0.71818626, -0.6676892, -0.8581496, -1.5223356, 0.26816422, -0.17383412, 0.15207003, -0.3758446) * inp_2_1_1;
    result4 += M4(0.050181318, -0.004893062, 0.03600665, 0.20216684, 0.08990556, 0.006988433, 0.18184568, 0.1790516, -0.4243239, 0.2706828, -0.06361402, 0.16749592, 0.068572484, 0.08081094, -0.06419751, -0.10163991) * inp_2_2_1;
    result4 += M4(0.030835606, -0.06252368, 0.06532133, -0.04678959, 0.0062267026, 0.024171947, -0.038829356, -0.14625618, -0.22890343, 0.20153014, 0.15591131, 0.059738986, -0.020165863, 0.061465915, 0.014388985, 0.1495432) * inp_2_0_2;
    result4 += M4(-0.008607725, 0.063461944, -0.019335281, 0.01865479, 0.048551023, -0.004334002, 0.04912913, 0.032267388, -0.06138668, 0.18500297, -0.0017047456, -0.0069184704, 0.028998177, 0.04138341, 0.08159398, 0.008308213) * inp_2_1_2;
    result4 += M4(-0.03855741, 0.049833965, -0.06031273, -0.20642084, -0.07387112, 0.090438366, 0.043682877, 0.32818744, -0.008642997, 0.104170606, 0.044468034, 0.3931014, -0.0002922155, 0.023405617, 0.0097786095, 0.039519962) * inp_2_2_2;
    result5 += M4(0.12810303, 0.17655611, -0.046538915, -0.21372198, 0.009625252, -0.05685811, -0.006559598, 0.0795813, -0.17378592, -0.1370766, -0.055107374, 0.34244063, 0.05707402, 0.13340618, 0.038962696, 0.0445159) * inp_2_0_0;
    result5 += M4(0.3071591, 0.056825906, 0.005517737, 0.14848237, 0.035278298, -0.26050407, -0.04963923, 0.13656141, 0.025800725, -0.31046396, 0.030708024, 0.17016786, 0.14560728, 0.027799262, -0.0014385383, 0.008868027) * inp_2_1_0;
    result5 += M4(0.044622775, -0.04162196, 0.021666346, -0.1803509, -0.010025705, 0.35433498, -0.012662009, 0.115696386, -0.14623412, 0.2737645, 0.046969265, 0.085649304, -0.018981256, 0.11766893, -0.013543384, 0.061610006) * inp_2_2_0;
    result5 += M4(0.0052194754, -0.13060573, 0.025073694, 0.07769542, 0.05931427, -0.016702438, 0.0023234368, -0.028033968, -0.13776028, -0.23236708, -0.16725518, 0.36489788, -0.010634823, -0.29450068, 0.11114288, 0.0043143076) * inp_2_0_1;
    result5 += M4(-0.005723787, 0.068859994, 0.047666047, -0.0059309993, -0.048599377, -0.113715634, -0.020121198, -0.074759595, 0.117261246, -0.9114916, -0.07047434, 0.07158698, 0.16302304, -0.12831523, 0.027848292, -0.01448749) * inp_2_1_1;
    result5 += M4(0.035372607, 0.07112473, -0.016986491, -0.07425614, -0.05761779, 0.026602386, -0.10421071, 0.006965065, 0.04243633, 0.08367724, 0.07300679, 0.08389271, -0.01874789, -0.2563437, 0.011427355, 0.048494995) * inp_2_2_1;
    result5 += M4(-0.06573621, 0.017322853, -0.042379923, 0.060226128, -0.005272417, -0.056462623, 0.02882004, 0.060677223, 0.017804349, -0.97113043, -0.19488193, 0.17044844, -0.047851074, 0.22890219, -0.029319014, -0.04529051) * inp_2_0_2;
    result5 += M4(-0.03287656, 0.06024143, 0.051939633, 0.0034924448, 0.0038361019, 0.097174376, 0.0005346883, -0.018866481, -0.30447084, -0.2067209, -0.08289015, 0.085753806, -0.021413978, -0.038522992, -0.048554122, -0.089125976) * inp_2_1_2;
    result5 += M4(0.08438765, -0.060417388, 0.022673566, -0.06911181, 0.027046897, 0.04602456, 0.0072395, 0.014039794, -0.17134981, 0.122971244, -0.017253583, 0.24171485, -0.016309865, -0.036305483, -0.017202199, -0.06544705) * inp_2_2_2;
    result6 += M4(-0.22809944, 0.07572306, -0.19351678, 0.10094158, -0.1750803, 0.1633931, -0.039775122, 0.012645724, 0.037789807, -0.2504574, 0.20357004, 0.07267781, -0.031012652, 0.05082989, -0.0008729396, 0.057357498) * inp_2_0_0;
    result6 += M4(-0.16650203, -0.105523326, 0.0021629855, 0.023432333, 0.065588675, -0.0793943, 0.08057709, -0.0053267735, -0.0889325, -0.23483774, 0.13484743, 0.18589917, 0.012452465, -0.041628983, -0.06493211, 0.069988385) * inp_2_1_0;
    result6 += M4(-0.025921144, 0.15959778, 0.042844653, -0.059959523, -0.011213253, -0.26907903, 0.055348296, -0.0036136904, 0.02211449, -0.13891132, -0.036239292, 0.056355882, 0.005172481, -0.008842067, -0.025342768, 0.040138166) * inp_2_2_0;
    result6 += M4(-0.16748437, -0.53618705, 0.13618712, 0.051987007, 0.08028417, -0.11762554, 0.05081888, -0.05624705, 0.17612323, -0.9184302, 0.2849475, 0.28720886, -0.2277009, 0.42329293, -0.3867523, 0.038146764) * inp_2_0_1;
    result6 += M4(0.12017571, -0.13289358, -0.51440966, -0.07518758, 0.14947386, 0.28554904, -0.45497152, -0.058517512, -0.86072814, -0.05625304, -2.2138224, -0.009704204, -0.35664886, 0.036921833, 0.6403266, -0.099394605) * inp_2_1_1;
    result6 += M4(-0.1290003, 0.023656359, 0.11360294, -0.05214469, -0.28949347, 0.13094838, 0.2023811, 0.0007190709, -0.4460275, -0.5266424, 0.37530646, 0.08891323, 0.19240528, -0.21808137, -0.10012372, -0.033260472) * inp_2_2_1;
    result6 += M4(0.00887158, 0.28551942, 0.15366898, 0.015796378, -0.060136136, -0.102984235, 0.0926162, 0.026535168, 0.12030074, 0.23569553, -0.037733365, 0.09656184, -0.0071299565, 0.023384415, -0.08452442, 0.060417574) * inp_2_0_2;
    result6 += M4(-0.111943595, 0.07268265, 0.1550202, -0.03653165, -0.21024497, -0.087805286, 0.0024718961, -0.10281578, -0.42199996, -0.38730827, 0.29294598, 0.12217214, 0.026600325, -0.14209731, 0.037665628, -0.0018771559) * inp_2_1_2;
    result6 += M4(0.15925549, -0.05468627, 0.00071524375, 0.014477783, 0.037161678, 0.013484354, 0.03049996, -0.012676532, -0.25934377, -0.14581586, 0.073178835, 0.123532586, 0.03298123, -0.1027334, 0.048217572, -0.0059902254) * inp_2_2_2;
    result7 += M4(-0.046778295, -0.005545317, -0.040465306, 0.07253376, -0.12552501, 0.0015791538, -0.013256937, 0.0067904084, -0.18373571, 0.11969182, 0.020376699, -0.17464474, 0.12676778, -0.023827348, -0.021288421, 0.17490381) * inp_2_0_0;
    result7 += M4(0.3479687, 0.031724643, 0.07465258, 0.16308545, 0.03034252, 0.09186223, 0.01808355, 0.13867073, -0.044945482, 0.4699583, -0.07210499, 0.026075764, 0.06283818, -0.01244497, -0.0036815198, 0.038032725) * inp_2_1_0;
    result7 += M4(0.07360185, 0.06624041, 0.039544832, 0.12964095, -0.03707324, 0.0928131, -0.029927868, -0.06175031, -0.062067125, 0.18168972, 0.04335705, 0.2381523, -0.05908417, 0.11422746, 0.00018822517, 0.1933427) * inp_2_2_0;
    result7 += M4(-0.0020739296, -0.032068167, 0.05521645, -0.039494757, 0.08073398, -0.09082879, 0.02440044, 0.016168825, -0.18024048, 0.14202207, 0.070379585, -0.23549758, -0.1830946, -0.0050973087, -0.051101834, -0.07642531) * inp_2_0_1;
    result7 += M4(0.16357069, -0.064594306, 0.03382304, -0.08504021, -0.081077896, -0.1896443, -0.0378617, 0.098414876, 0.38469622, 0.46970525, -0.06905155, -0.04466567, 0.42416945, -0.045380082, -0.0344935, -0.39740098) * inp_2_1_1;
    result7 += M4(0.037837427, -0.03339726, 0.077025026, 0.08919732, 0.16282257, 0.039311767, 0.030161705, 0.33949625, -0.05790563, 0.4626024, -0.08155964, 0.24858893, -0.017794758, 0.052794237, 0.030947287, 0.1306416) * inp_2_2_1;
    result7 += M4(0.093028426, -0.04485251, -0.0065687452, -0.022669178, 0.092557356, -0.009156115, 0.013476216, 0.13787052, -0.16739094, 0.10352508, -0.0026280815, -0.036649242, 0.08971629, -0.010507359, -0.012309125, 0.01727639) * inp_2_0_2;
    result7 += M4(0.101282656, 0.066215664, -0.011806568, 0.0019568952, -0.05561024, 0.022303393, 0.03628424, 0.1838337, 0.02127215, -0.08331451, -0.07088556, 0.0660645, 0.023168756, 0.0281265, -0.082708254, -0.23837453) * inp_2_1_2;
    result7 += M4(-0.05874723, -0.0016885428, -0.019070646, 0.15765944, -0.084656194, 0.003052134, 0.031910352, -0.1661668, -0.24457407, 0.02329477, -0.019773278, 0.079128474, -0.034763083, -0.007851738, -0.018310755, 0.038955424) * inp_2_2_2;
    const V4 inp_3_0_0 = inp[3][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_3_1_0 = inp[3][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_3_2_0 = inp[3][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_3_0_1 = inp[3][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_3_1_1 = inp[3][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_3_2_1 = inp[3][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_3_0_2 = inp[3][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_3_1_2 = inp[3][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_3_2_2 = inp[3][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.117943265, 0.030242674, 0.0066244327, 0.12307061, 0.09951394, -0.049280062, -0.10198084, -0.15013093, 0.48763958, -0.031933986, -0.0018029237, -0.24164319, 0.13119523, 0.030557377, -0.25053725, 0.33753252) * inp_3_0_0;
    result0 += M4(-0.2806651, -0.012407193, 0.061249185, 0.13409552, 0.43321362, -0.0271102, -0.0147737255, -0.23964354, 0.061952084, -0.029688891, -0.007434801, 0.08353617, 0.62339216, 0.022157747, 0.10338537, -0.010586449) * inp_3_1_0;
    result0 += M4(-0.014553197, -0.014585895, -0.0493509, 0.11442637, -0.3122322, -0.023907155, -0.10620721, -0.11382642, -0.021076595, -0.061864983, -0.20232406, -0.19817202, -0.14532414, -0.010764761, 0.041872047, 0.071135886) * inp_3_2_0;
    result0 += M4(-0.05721981, -0.03517094, -0.08602985, 0.3123854, -0.68246114, 0.07391283, -0.005373693, 0.15104006, 0.31387818, 0.022970848, 0.041870896, -0.19231373, -0.045976028, -0.03276211, -0.0064220843, 0.20556296) * inp_3_0_1;
    result0 += M4(-0.18437162, -0.0120033715, -0.076240845, 0.1714936, 0.1830893, -0.08724528, 0.006251373, 0.29496512, -0.059142977, -0.09672378, 0.27977705, -0.123267695, -0.16666938, -0.084873535, 0.0512147, 0.3853948) * inp_3_1_1;
    result0 += M4(-0.22512485, 0.009440466, -0.052976727, -0.1294349, -0.30615756, -0.022958573, 0.06923309, -0.02805679, 0.1849886, 0.029722288, -0.07675002, -0.12940137, 0.054595813, 0.009858051, 0.0014578854, -0.019668926) * inp_3_2_1;
    result0 += M4(0.05876996, -0.045140248, 0.028776621, -0.1521194, 0.26934338, -0.047615223, -0.091312736, 0.00724592, -0.35550302, -0.051563587, 0.030727737, -0.064567566, 0.349946, 0.014762025, -0.12822548, -0.11476887) * inp_3_0_2;
    result0 += M4(-0.29297447, -0.019226111, -0.01829224, -0.3138021, 0.2682934, -0.042525284, 0.0038056169, -0.23144512, 0.0063251867, 0.041718945, 0.05512112, 0.23257072, -0.2925445, 0.010767053, 0.033088733, -0.16484067) * inp_3_1_2;
    result0 += M4(0.24785039, -0.065993786, -0.039987262, -0.040816702, -0.02847814, -0.015416403, 0.006775572, 0.07118908, -0.31564602, -0.038210306, -0.08073443, -0.18977761, 0.20947127, -0.007332585, -0.101321004, -0.015586215) * inp_3_2_2;
    result1 += M4(0.034167994, 0.050123733, 0.09130873, 0.03313912, 0.027507758, -0.11015259, 0.030766364, 0.12983528, -0.14864378, 0.0804325, 0.0821116, -0.071610264, 0.047134478, -0.04474604, 0.024716446, -0.04032019) * inp_3_0_0;
    result1 += M4(-0.10612616, 0.018933224, -0.18036017, -0.025089514, 0.034617, 0.113168046, 0.0031302865, -0.30266428, -0.08767085, -0.04635213, -0.01318085, 0.048276134, 0.16642769, -0.07267565, 0.057723284, -0.08578205) * inp_3_1_0;
    result1 += M4(0.00029099596, 0.07355365, 0.0047610477, -0.03709351, -0.043883227, 0.107200265, -0.08982231, -0.04125965, -0.019804595, 0.016250342, 0.06255791, 0.0132085, 0.00592906, -0.23852889, 0.13151091, 0.046296995) * inp_3_2_0;
    result1 += M4(-0.12813964, 0.07636109, -0.11067749, -0.03383863, 0.1473779, -0.0597428, -0.14730112, -0.0809187, -0.1160865, -0.0980092, 0.115838535, -0.15801485, 0.1225264, 0.037505936, -0.16050863, 0.1035946) * inp_3_0_1;
    result1 += M4(0.056410417, 0.05495604, 0.22824116, -0.10542161, 0.010598626, -0.2106343, 0.32698283, -0.042331606, -0.1024758, -0.058449827, 0.25630215, -0.17422964, 0.27677402, -0.119833745, -0.31934595, -0.2433981) * inp_3_1_1;
    result1 += M4(0.07848299, -0.10041186, -0.05895874, -0.12603143, 0.039928775, 0.018645274, 0.006488768, 0.1750105, -0.15522794, 0.1575458, 0.14383726, 0.17355868, 0.076337904, 0.101048686, 0.14404535, 0.1545375) * inp_3_2_1;
    result1 += M4(0.008405277, -0.046287183, -0.054561637, -0.113851264, -0.09734935, 0.028903361, -0.02282995, -0.032638144, 0.02963919, 0.10257557, -0.0062811207, -0.02344965, -0.32974696, 0.020465057, 0.13463567, 0.08450471) * inp_3_0_2;
    result1 += M4(-0.119065404, -0.08588291, -0.008666604, -0.16394714, 0.020989249, -0.28507915, 0.037578307, 0.09975051, -0.034432247, 0.22106786, 0.06563788, -0.040766194, -0.04598791, -0.0778917, 0.046585098, 0.09649589) * inp_3_1_2;
    result1 += M4(-0.07398968, 0.19227566, -0.21858484, -0.13184364, -0.066153444, 0.06392486, -0.030075282, 0.040993337, -0.14012116, -0.20804532, -0.025624042, 0.055506, -0.22860344, -0.09467458, -0.1773037, 0.031699188) * inp_3_2_2;
    result2 += M4(-0.029672943, 0.03899605, 0.014023156, 0.028770426, 0.06531842, -0.08099475, -0.011576551, -0.065331385, -0.059659347, -0.08843622, 0.09826435, 0.032416444, -0.022181205, -0.13819273, 0.2112614, 0.00032855157) * inp_3_0_0;
    result2 += M4(-0.058700334, -0.03971319, 0.08716128, -0.013234415, -0.049148582, -0.33375573, 0.21362884, -0.0511314, 0.0011929974, 0.0073851594, -0.013027553, 0.020206874, -0.02184481, -0.032237113, -0.15774077, -0.24651007) * inp_3_1_0;
    result2 += M4(-0.050972223, -0.048732467, 0.013469613, 0.03685233, -0.037338186, -0.03184109, 0.06941311, 0.018538902, 0.0103570195, 0.091191575, -0.01820941, 0.056396794, 0.02760936, -0.055013113, 0.118001536, 0.015685184) * inp_3_2_0;
    result2 += M4(-0.04171701, 0.074672155, 0.119122475, -0.045094386, -0.08397962, 0.18009044, -0.08141905, 0.0042956197, -0.013705214, 0.2092069, 0.087900795, -0.10748836, -0.072533615, -0.1704241, 0.2645297, -0.012341328) * inp_3_0_1;
    result2 += M4(-0.09920988, 0.040954053, 0.15168852, -0.029219976, 0.12856974, -0.15204884, 0.08307584, 0.13763139, 0.082631834, -0.22025527, 0.083894014, 0.061971825, 0.11052342, -0.2080028, 0.11401492, 0.09847924) * inp_3_1_1;
    result2 += M4(0.03946296, -0.13404107, -0.14866906, 0.062466476, -0.014510915, -0.061380874, -0.003248108, -0.021780277, -0.11494043, -0.35637483, 0.024151322, 0.029227773, -0.06418614, -0.014918828, 0.09934295, 0.016994324) * inp_3_2_1;
    result2 += M4(0.052874044, 0.057769496, -0.035954475, -0.0014359555, -0.03507657, 0.14088187, 0.063025504, 0.05170231, -0.112413436, -0.0015512846, 0.017801058, -0.032475404, 0.06466106, 0.19101529, 0.15323308, -0.044230886) * inp_3_0_2;
    result2 += M4(-0.08217195, 0.023599856, 0.0884032, 0.20355633, 0.07694633, 0.19173737, -0.034940526, -0.02986161, 0.069877185, 0.054893803, 0.06831158, -0.05927064, 0.030667242, 0.11700297, 0.08731461, 0.037239715) * inp_3_1_2;
    result2 += M4(3.9355305e-05, 0.24337418, -0.1020636, -0.02390529, 0.00094652816, -0.07636492, -0.0038254182, 0.07894799, -0.020011885, -0.05804185, -0.061257537, -0.0028940702, -0.06405834, 0.020952096, 0.048428692, -0.020107226) * inp_3_2_2;
    result3 += M4(-0.08611522, -0.0030633835, 0.02189528, 0.14000839, 0.14932804, -0.047372933, -0.09368367, -0.011557483, -0.115251936, -0.14001617, 0.065752424, 0.15318215, -0.18349184, 0.20443131, -0.02295523, -0.21454376) * inp_3_0_0;
    result3 += M4(-0.013499657, -0.11522196, -0.04971766, -0.11239269, 0.31965235, 0.039509565, -0.015082557, -0.19332309, 0.0011510348, -0.0645072, -0.06401536, 0.117475, -0.081555, -0.00951397, -0.058909968, -0.19359668) * inp_3_1_0;
    result3 += M4(0.11442435, 0.057763647, 0.06611125, -0.07191904, 0.17325407, -0.12421027, 0.05197403, -0.15029322, 0.17038576, -0.005146689, 0.1006128, 0.048644926, 0.20567545, 0.05145151, 0.11137819, 0.08528231) * inp_3_2_0;
    result3 += M4(0.16003942, -3.3150907e-05, 0.0070066247, -0.019554853, 0.041475564, 0.1436076, 0.12044624, 0.13803759, -0.18920083, -0.15232064, 0.04921814, 0.022810167, -0.08622365, -0.10994686, -0.16303337, 0.27949423) * inp_3_0_1;
    result3 += M4(-0.13519229, -0.00046537616, -0.00803051, -0.002670266, -0.12520467, 0.24414322, -0.31013006, 0.46830323, -0.18853736, -0.21156433, -0.064110674, -0.06583749, 0.14203164, -0.24647444, 0.03886973, 0.16679661) * inp_3_1_1;
    result3 += M4(-0.052417547, -0.1262613, 0.30006817, -0.027322678, -0.24786156, 0.14183944, -0.11108187, 0.08446815, 0.15358454, -0.27940327, 0.3865485, 0.0698262, 0.052902114, -0.10276484, 0.2164768, 0.060318075) * inp_3_2_1;
    result3 += M4(-0.152687, -0.06125825, 0.0853731, -0.11389394, 0.19383189, 0.058418185, 0.08441318, -0.022815466, -0.20685303, 0.01683592, -0.017315945, -0.08860894, -0.20900452, -0.047437556, -0.03901929, -0.025181215) * inp_3_0_2;
    result3 += M4(-0.13552234, -0.28931892, 0.27641112, 0.029471377, 0.09424902, 0.034360107, 0.072275706, -0.23128235, -0.099077426, -0.0025129898, -0.05876969, 0.31904563, -0.1493638, -0.008458294, 0.026285687, -0.052293908) * inp_3_1_2;
    result3 += M4(-0.18742041, 0.20366986, 0.09079042, 0.006426303, -0.0062572835, 0.018803107, 0.07560453, 0.065252095, 0.108800955, -0.034670737, 0.060591985, -0.17718829, -0.019702809, 0.23826146, -0.06167693, 0.039978813) * inp_3_2_2;
    result4 += M4(-0.015386157, 0.05103523, -0.04939271, -0.028285494, -0.10908649, 0.052585904, -0.08230308, -0.06607971, 0.020193009, 0.031122614, 0.109361984, 0.34524497, 0.06601478, 0.02469218, -0.04926184, -0.12838101) * inp_3_0_0;
    result4 += M4(0.09900421, -0.030329136, 0.1317596, -0.11485956, 0.077560045, -0.063222356, 0.11935639, 0.009036372, -0.012955523, -0.054438338, 0.0783902, 0.2810597, 0.03879771, -0.105843924, 0.035478305, -0.17986257) * inp_3_1_0;
    result4 += M4(-0.16013484, -0.0069578043, 0.015016985, 0.23568287, -0.019975025, -0.02698052, 0.08709504, 0.35014558, -0.18128225, 0.062407367, -0.06553249, 0.4476042, -0.006489513, -0.10934068, 0.006000093, 0.06968978) * inp_3_2_0;
    result4 += M4(-0.00949283, -0.029555524, 0.1476233, -0.075271405, -0.009419856, 0.0006498467, -0.023004157, -0.048048493, -0.019771306, 0.054787535, 0.18355633, 0.26324442, 0.08955021, -0.08167543, 0.13725458, 0.3256893) * inp_3_0_1;
    result4 += M4(-0.015070554, 0.09056343, -0.07537809, 0.02878329, -0.19720118, 0.062774114, -0.2598924, -0.11100944, -0.11561329, -0.09053698, -0.17336324, 0.04989278, -0.05602445, -0.1857058, -0.044799477, -0.4904713) * inp_3_1_1;
    result4 += M4(-0.025314027, -0.027335994, -0.018225256, 0.08461425, -0.05194805, 0.036333036, -0.042440824, 0.1344151, -0.23054053, 0.017436175, 0.030418402, -0.0026702052, 0.026996722, -0.0026418008, -0.028363517, -0.27824703) * inp_3_2_1;
    result4 += M4(0.08045865, -0.06808081, 0.17505842, 0.19585909, -0.034328762, 0.011651474, 8.585873e-05, 0.20200796, -0.098371595, -0.014589147, -0.006992848, 0.085458964, -0.12194295, 0.07183732, 0.011256393, -0.23082475) * inp_3_0_2;
    result4 += M4(-0.004237925, -0.110647194, -0.035186645, 0.114165716, 0.13241611, -0.07975399, 0.13083139, 0.15853931, -0.17520268, -0.0064742626, -0.1829679, 0.028576856, 0.06842715, -0.07971203, -0.013566683, -0.05044988) * inp_3_1_2;
    result4 += M4(-0.111759104, -0.0061194194, -0.12973712, 0.16469255, 0.04026171, -0.0042172684, -0.050689608, -0.09867114, -0.07597501, 0.026703093, 0.052539367, 0.32755303, 0.022298982, 0.07262004, -0.029951995, 0.22317965) * inp_3_2_2;
    result5 += M4(-0.04589889, -0.1700961, 0.041727692, 0.057572555, -0.11573646, 0.057667863, 0.014733862, -0.16571233, -0.030412704, 0.02232687, 0.0047421586, 0.10976462, 0.10103363, -0.001501548, -0.054797508, -0.09722908) * inp_3_0_0;
    result5 += M4(0.063752145, -0.12318388, -0.038131345, 0.096433565, 0.21837004, -0.30460277, -0.05619734, -0.09607765, 0.07292359, 0.0056649055, -0.004409458, 0.059781693, 0.28648508, -0.37015936, 0.051962342, 0.090850726) * inp_3_1_0;
    result5 += M4(0.08248033, -0.095022075, 0.023815673, 0.07698641, 0.009463538, 0.22625594, -0.049227472, 0.031202137, -0.059456125, 0.2570124, -0.0019178537, 0.058436733, 0.036837075, 0.2259562, 0.003514414, 0.054817382) * inp_3_2_0;
    result5 += M4(0.043171752, -0.14216763, 0.052822225, 0.14686976, 0.0037456136, 0.024276383, -0.07217371, -0.08771341, 0.020989383, 0.07552622, -0.06849164, 0.07359534, -0.06196992, 0.29374972, 0.0013454268, -0.018778538) * inp_3_0_1;
    result5 += M4(0.12448144, -0.25932413, -0.05669318, 0.3865967, -0.08423301, -0.08631723, 0.036437146, 0.13520303, -0.10872071, 0.37122872, 0.089274436, 0.17916323, 0.23856466, -0.31223458, 0.09331747, 0.08494256) * inp_3_1_1;
    result5 += M4(0.07935825, -0.015793413, -0.004430869, 0.13564052, -0.0035263547, 0.008020884, 0.015443144, -0.0027888524, -0.013840567, 0.29553613, 0.018471401, 0.045685817, 0.015729582, -0.23279236, -0.029914219, -0.15790932) * inp_3_2_1;
    result5 += M4(-0.05844352, -0.21084571, -0.06762929, 0.086550415, 0.06307971, -0.0773085, -0.014781467, -0.042379368, -0.014764717, 0.19694203, 0.0093195215, 0.048619684, -0.034869105, -0.2788434, -0.12442911, -0.11472378) * inp_3_0_2;
    result5 += M4(-0.06938975, -0.15931551, 0.052231982, 0.31147853, -0.057599988, 0.07596822, -0.072734766, -0.10787622, -0.015077388, 0.08641691, -0.01303272, 0.21553394, -0.13437454, 0.17654926, 0.008164055, -0.0104285) * inp_3_1_2;
    result5 += M4(-0.027702188, -0.37073874, 0.057528455, 0.26037458, -0.015939297, 0.2860098, -0.0421271, -0.041493136, -0.019593235, 0.040143214, -0.0074477373, -0.1480908, 0.013700538, 0.18530339, -0.024919817, -0.015354406) * inp_3_2_2;
    result6 += M4(-0.081318356, 0.01634046, -0.13166702, 0.009953035, -0.03138005, 0.29554525, -0.018485973, -0.053790417, -0.15551287, -0.046681345, -0.11619412, 0.038581625, -0.03599094, 0.1093203, -0.12555301, 0.072015405) * inp_3_0_0;
    result6 += M4(-0.004183471, -0.064997636, 0.0032962663, 0.029089654, 0.19479582, -0.07437283, 0.21372472, -0.060901716, -0.06984817, -0.01293764, -0.07764473, -0.005844471, 0.0056209234, -0.0091442475, -0.073516116, 0.045606066) * inp_3_1_0;
    result6 += M4(-0.074166045, -0.061076898, -0.1695986, -0.0072265435, -0.06087117, -0.014023703, 0.018294103, -0.030809075, 0.0011470735, 0.067205496, -0.10846549, -0.024832236, -0.16580811, -0.027764784, -0.017928822, -0.0490393) * inp_3_2_0;
    result6 += M4(-0.10295331, 0.06856639, 0.0021618812, -0.0028160391, 0.15130311, 0.22185224, 0.19721402, -0.09036344, 0.040194497, -0.3659347, 0.13353132, 0.00923402, -0.060178194, 0.33426598, 0.04790711, 0.04346229) * inp_3_0_1;
    result6 += M4(-0.12697273, -0.17730021, 0.0021863652, 0.020253835, -0.07999572, -0.035700776, -0.5168328, 0.2262964, -0.08820926, 0.0129926875, -0.22008327, 0.10282794, -0.086927466, 0.17854582, -0.25409985, -0.045835186) * inp_3_1_1;
    result6 += M4(-0.07165077, -0.025258053, 0.12348888, -0.0438888, -0.005321219, -0.15621033, -0.049013276, -0.0048679034, -0.3208475, -0.01902369, 0.34629962, -0.05080503, 0.035657786, 0.33729032, 0.23773365, 0.004248891) * inp_3_2_1;
    result6 += M4(-0.06870152, -0.15034552, 0.1639509, 0.003026908, -0.09996184, 0.19899416, -0.090840474, 0.026737008, -0.10395767, -0.071628794, 0.052483723, 0.024933705, -0.16986863, 0.24636017, 0.05348849, -0.0141907735) * inp_3_0_2;
    result6 += M4(-0.20073575, -0.12717491, 0.041953895, -0.07818724, -0.35924944, -0.18338394, 0.0733834, -0.010143243, 0.11103868, 0.31483614, -0.16782834, 0.0068257055, -0.093583465, 0.015305496, 0.026163545, 0.0042250087) * inp_3_1_2;
    result6 += M4(0.0827573, -0.04452964, -0.08241607, -0.036053054, 0.06489467, -0.043965686, 0.06030696, -0.027328858, -0.042328767, 0.034892738, 0.007921372, -0.020471582, 0.08873748, -0.076041035, -0.0696009, -0.041911494) * inp_3_2_2;
    result7 += M4(-0.010384198, -0.056118034, -0.0014014033, -0.04569562, 0.06313678, -0.053701293, -0.0064332807, -0.10776547, -0.055194344, 0.036582645, -0.011364639, 0.09185881, -0.005194107, -0.051549233, 0.05031885, 0.1739917) * inp_3_0_0;
    result7 += M4(-0.05685323, 0.01873954, -0.0044867974, 0.04213519, -0.020686083, 0.1038758, -0.01133838, -0.018902147, 0.07717898, 0.020795405, 0.034213755, 0.0047642025, 0.08732408, -0.34294954, 0.029455861, -0.14591552) * inp_3_1_0;
    result7 += M4(-0.031627923, 0.06140832, -0.00692754, 0.0086924415, 0.0056778644, 0.018787358, -0.00479346, 0.0560067, -0.19669545, 0.005638528, -0.017794074, 0.17055427, -0.058229238, 0.03870251, -0.004461976, -0.21675524) * inp_3_2_0;
    result7 += M4(-0.103117116, 0.031758744, -0.00951164, 0.019784642, -0.15630934, 0.07465282, -0.0150638055, -0.0458857, 0.11034316, -0.19571476, 0.03043192, 0.19449665, 0.08818271, 0.0056288405, 0.023989394, -0.046629187) * inp_3_0_1;
    result7 += M4(-0.012339205, 0.11250362, -0.03297283, 0.060399964, -0.052679885, 0.19599773, 0.049505733, 0.24083252, 0.20645379, 0.18157232, 0.02727392, 0.22397864, 0.3932469, 0.18893664, 0.013014186, 0.1304911) * inp_3_1_1;
    result7 += M4(0.08576363, -0.07272341, 0.0014826074, 0.20178097, -0.19288853, 0.0071982457, -0.09225006, -0.4876177, 0.09378215, 0.06365435, -0.12414833, 0.27061886, -0.014055709, -0.09657173, -0.0397299, 0.05931617) * inp_3_2_1;
    result7 += M4(0.037333068, -0.07665561, 0.008698786, -0.0801694, -0.030897528, -0.0062119514, 0.015960004, 0.07595162, 0.041096877, 0.0068219877, -0.015415586, -0.020281585, -0.06760726, -0.061771307, 0.0029141472, 0.00024270397) * inp_3_0_2;
    result7 += M4(0.03530384, 0.0006628986, -0.028563274, 0.052601553, 0.007857865, -0.120918185, 0.001320021, -0.10128034, 0.036988247, -0.011005162, 0.019336326, 0.22823699, 0.24445827, -0.0014291245, -0.04093586, -0.052956853) * inp_3_1_2;
    result7 += M4(-0.10189602, -0.05097317, -0.00661715, 0.34995323, -0.082311265, 0.062357884, -0.09115671, 0.118986554, -0.04045772, 0.058584344, 0.008978515, 0.12891509, -0.1671297, -0.04173863, -0.01928386, 0.036647405) * inp_3_2_2;
    const V4 inp_4_0_0 = inp[4][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_4_1_0 = inp[4][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_4_2_0 = inp[4][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_4_0_1 = inp[4][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_4_1_1 = inp[4][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_4_2_1 = inp[4][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_4_0_2 = inp[4][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_4_1_2 = inp[4][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_4_2_2 = inp[4][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.3018137, -0.013020348, -0.049549904, -0.28026092, 0.23046254, 0.0073603066, 0.07876367, 0.11498545, -0.09971798, 0.0171323, -0.08660343, -0.102540925, -0.049266655, -0.021785554, 0.06578679, 0.012559194) * inp_4_0_0;
    result0 += M4(0.09360284, 0.041297667, -0.011407872, -0.22327155, -0.058291133, 0.049998548, -0.0016157784, 0.32392028, -0.1554103, 0.08582959, 0.00045326023, -0.30957094, 0.1727904, 0.03028479, -0.032546252, 0.18225563) * inp_4_1_0;
    result0 += M4(-0.15269941, -0.0007726305, -0.019145932, -0.023588426, 0.4908673, 0.01632372, -0.018428618, 0.036203094, -0.105519205, 0.053077597, -0.03129017, -0.0905639, -0.08410142, -0.013844415, 0.02462343, -0.10925957) * inp_4_2_0;
    result0 += M4(-0.043896165, 0.009609951, 0.060030267, -0.31710848, -0.30268708, -0.0055676526, 0.003751909, -0.16113909, 0.6572956, 0.02484029, -0.09845621, -0.06603069, 0.39214316, 0.009188212, -0.057801366, -0.1008217) * inp_4_0_1;
    result0 += M4(-0.32501817, 0.17007567, 0.0023070236, 0.061213236, 0.47512704, -0.03765019, -0.016657507, 0.08354025, -0.18601365, 0.062339336, -0.024578474, 0.0036705313, 0.009339171, 0.022463446, -0.017062802, -0.46684402) * inp_4_1_1;
    result0 += M4(0.4459742, -0.030933673, 0.07043147, 0.032040488, 0.02642156, -0.051713355, 0.0056665447, 0.109782174, 0.6417111, 0.008452648, -0.019417236, 0.17113593, 0.14958274, -0.009424756, 0.047757134, -0.21612719) * inp_4_2_1;
    result0 += M4(0.18609999, -0.039677013, 0.05630982, 0.07515954, 0.7838686, -0.055841103, -0.047054734, -0.07511009, -0.2653964, 0.046451315, 0.042494748, 0.017005099, -0.37005898, -0.024555542, 0.033625413, -0.05479377) * inp_4_0_2;
    result0 += M4(-0.2004114, -0.0534004, 0.011974997, 0.12889145, 0.6502674, -0.007364523, -0.09515183, -0.06600448, -0.015182954, 0.02587331, -0.011109028, 0.16523658, -0.3700366, 0.017806876, -0.03955656, 0.08430159) * inp_4_1_2;
    result0 += M4(0.06464741, -0.032058395, 0.1063676, 0.035703897, 0.18288997, -0.04292027, 0.013897333, -0.063932896, -0.171026, 0.015645461, 0.017234506, 0.18442217, -0.62919784, -0.020096572, 0.0182239, -0.08293234) * inp_4_2_2;
    result1 += M4(-0.102511235, -0.07265324, -0.14810495, 0.07598158, 0.056533657, -0.11597489, 0.16719119, 0.07353281, 0.11707296, 0.03156239, -0.02504804, -0.07039278, -0.053926397, 0.030805368, -0.07913526, 0.031133305) * inp_4_0_0;
    result1 += M4(0.14906424, 0.13490197, 0.043523293, -0.21805875, 0.14061184, -0.1283112, 0.10604765, 0.13399039, 0.09116153, -0.11604729, 0.03155367, 0.044148177, 0.027032405, -0.039540134, -0.07004252, 0.015027843) * inp_4_1_0;
    result1 += M4(-0.005321433, 0.022156551, 0.13702907, -0.05038215, -0.074196614, 0.01177882, -0.12157675, 0.095419966, 0.095140144, -0.06975208, -0.09753053, 0.0189856, -0.0023372706, -0.014808191, -0.08641319, 0.028876146) * inp_4_2_0;
    result1 += M4(0.06470664, -0.117487796, 0.0737644, -0.2593704, 0.017199075, -0.05189011, -0.13021274, -0.07510754, -0.13526417, 0.13220762, 0.038943026, -0.010082275, -0.012134603, 0.05732854, 0.12488482, 0.047654998) * inp_4_0_1;
    result1 += M4(-0.0132052675, -0.058064867, 0.6246033, 0.30574813, 0.08275789, 0.028124081, 0.087949574, 0.041601907, -0.07685102, 0.1927248, 0.19874029, -0.023784792, 0.16383569, -0.29347178, -0.03506492, 0.14331093) * inp_4_1_1;
    result1 += M4(0.013810233, -0.20379466, -0.0029574377, -0.05429025, 0.0061847204, -0.1287188, 0.056821436, -0.19968429, 0.033311073, 0.047808524, -0.12539726, 0.04197058, 0.027535906, -0.15413971, -0.09273763, 0.0714396) * inp_4_2_1;
    result1 += M4(0.0575832, 0.09988246, 0.017086057, -0.14190671, 0.02969055, 0.05706945, 0.008795121, 0.028488481, 0.07869117, -0.12035158, -0.098097935, 0.08089159, -0.039384056, -0.017109664, -0.21429273, -0.05338497) * inp_4_0_2;
    result1 += M4(-0.04461614, 0.13620065, -0.05700041, 0.012478189, 0.02777042, 0.0639019, 0.16530146, 0.07414691, 0.043548822, 0.07562601, 0.050822187, -0.023906594, -0.023614276, 0.20055115, -0.034724034, -0.08018155) * inp_4_1_2;
    result1 += M4(0.13410544, -0.032066744, 0.017151728, 0.09274798, -0.04446435, -0.096632, 0.017900407, -0.007629201, 0.11410772, -0.013138801, -0.039242238, -0.08851863, -0.07235172, 0.08360792, -0.13465318, 0.0148715135) * inp_4_2_2;
    result2 += M4(-0.03242549, 0.004419844, 0.03611307, 0.006906492, 0.032073546, 0.116895914, -0.08528831, 0.033674378, 0.08197862, 0.155986, -0.01880573, -0.023366774, 0.022254653, 0.016554639, -0.124440424, 0.07043768) * inp_4_0_0;
    result2 += M4(-0.07359907, -0.2923346, 0.027871694, 0.043719213, 0.030615322, 0.17880802, -0.16001432, 0.13926299, 0.0441133, 0.079303585, -0.054398555, -0.08275422, -0.19819435, 0.030343017, -0.1052359, 0.1543761) * inp_4_1_0;
    result2 += M4(-0.04237484, 0.16342366, 0.02005737, -0.06121913, 0.04584682, 0.046291165, 0.0035695513, -0.03507176, -0.004354566, -0.10556133, 0.019531237, -0.023535118, 0.08837553, 0.16393757, -0.13937518, 0.04758497) * inp_4_2_0;
    result2 += M4(0.052561242, 0.28211197, -0.036559436, -0.05935242, -0.053437144, 0.034789626, -0.0020002313, -0.028297711, -0.0033113817, -0.05700376, -0.014717272, 0.067595705, 0.2550543, -0.040316723, -0.16562659, 0.038181175) * inp_4_0_1;
    result2 += M4(-0.27514645, -0.0349075, -0.051908866, -0.05192868, -0.11496198, 0.12404065, -0.01078087, -0.02280336, -0.08217899, 0.08553893, 0.12134196, -0.13929915, 0.34713256, 0.13653333, -0.09395911, -0.0011488404) * inp_4_1_1;
    result2 += M4(-0.053193584, -0.045043357, 0.059109066, -0.01817236, -0.050541423, 0.06252233, 0.04097031, 0.025567947, -0.021865303, 0.06030683, 0.083756246, -0.024146432, 0.24712478, -0.10600784, -0.21732959, 0.12873065) * inp_4_2_1;
    result2 += M4(-0.14941038, 0.22233866, -0.049015146, -0.0096986275, -0.053760286, 0.11942078, 0.028837195, 0.025498915, -0.07593266, 0.060939454, -0.020343922, 0.054884106, 0.1514911, 0.030558199, -0.082864664, 0.025275368) * inp_4_0_2;
    result2 += M4(-0.046890736, -0.16881321, -0.07375432, 0.085812606, -0.035885133, 0.13108827, -0.014424787, 0.015986072, -0.092035905, -0.15928257, 0.025071139, -0.033235837, 0.067316666, -0.13274764, -0.11927083, 0.045213807) * inp_4_1_2;
    result2 += M4(-0.15622507, 0.010858122, -0.08711453, 0.06825224, 0.005581884, 0.114213176, -0.061111126, 0.012844964, -0.045832682, 0.06767977, 0.017168824, -0.052418064, 0.07182294, -0.08665962, -0.10286861, -0.030191699) * inp_4_2_2;
    result3 += M4(0.18116899, -0.12024191, -0.020289863, -0.032588612, 0.050053496, -0.053340368, 0.005563338, 0.17409396, 0.17397444, -0.00076671556, 0.0702323, 0.17487217, 6.3295345e-05, -0.057362445, 0.01953525, 0.028768746) * inp_4_0_0;
    result3 += M4(0.2771388, -0.0501204, 0.029183485, 0.014728141, -0.03528127, -0.112404175, 0.018219598, 0.15193923, -0.13361287, 0.04453793, -0.040170576, 0.1015498, 0.23394102, -0.08231008, 0.07266424, 0.0048812907) * inp_4_1_0;
    result3 += M4(-0.12989089, -0.012848164, 0.011233716, -0.13329805, -0.38493955, -0.06285304, -0.30414128, 0.046480022, -0.12380106, -0.13137563, -0.06053729, 0.059972897, 0.018283306, 0.23430018, 0.034507837, -0.07846396) * inp_4_2_0;
    result3 += M4(-0.13225663, 0.031584825, 0.14657196, -0.03157697, 0.13204928, 0.034892954, 0.08914248, 0.03534104, 0.12665145, -0.0048880544, 0.04371077, 0.055747267, 0.10043238, -0.08453545, -0.018510431, -0.020916501) * inp_4_0_1;
    result3 += M4(-0.059966773, -0.227455, -0.0067515178, -0.1509037, -0.004857745, 0.048709746, -0.015395928, 0.14943959, 0.1520141, 0.2411044, 0.103362985, -0.19666965, -0.13677107, -0.18017587, -0.008109747, -0.1718905) * inp_4_1_1;
    result3 += M4(-0.19178551, -0.003407673, 0.061623983, 0.028470395, -0.057583366, 0.002965698, -0.061391983, 0.04080423, -0.08365545, -0.094243266, -0.12791899, -0.0071939714, 0.058824167, -0.02254071, 0.14933398, 0.01385749) * inp_4_2_1;
    result3 += M4(0.032817446, -0.0075291716, 0.020894537, 0.029269142, 0.115077965, 0.10567794, 0.059080318, -0.035731893, 0.14758964, -0.08354966, 0.080107294, 0.08751252, 0.041561387, 0.037754018, 0.01589498, -0.033128597) * inp_4_0_2;
    result3 += M4(-0.009653976, -0.23833406, -0.06733429, 0.061069015, 0.04092955, -0.19128436, 0.09763306, -0.09594019, 0.1255435, 0.09470599, -0.05103819, -0.050317887, -0.042732842, -0.03576084, -0.08459222, 0.1450791) * inp_4_1_2;
    result3 += M4(-0.058467787, -0.04665239, 0.03950108, 0.06538467, 0.17838772, 0.06649395, 0.006720808, -0.02996861, 0.032910023, -0.16033942, -0.043170772, -0.007888994, 0.13054013, 0.0015997355, -0.09093533, 0.019502891) * inp_4_2_2;
    result4 += M4(0.05621141, -0.0070204176, 0.1360875, -0.4978411, -0.06944513, 0.06870188, -0.13048346, 0.07466566, -0.11524261, 0.054494422, -0.122359, 0.28658253, -0.094356954, 0.03857421, 0.049333915, 0.18112426) * inp_4_0_0;
    result4 += M4(-0.15828863, -0.043078594, 0.013517729, 0.34176984, -0.07920714, 0.0064644627, -0.1663052, 0.2033255, 0.14449607, 0.00909411, -0.17875399, -0.013445555, 0.08276654, 0.070221156, 0.107563525, -0.3718084) * inp_4_1_0;
    result4 += M4(-0.010050604, -0.07914685, -0.066875175, 0.15993077, 0.023663511, 0.062218074, -0.10984393, -0.25780618, 0.07957397, 0.026925031, 0.0047597196, -0.17008698, -0.034396913, 0.036735974, 0.0071018604, -0.053559918) * inp_4_2_0;
    result4 += M4(-0.0072490214, -0.11809649, 0.070471585, 0.12101479, 0.0173843, -0.0629167, 0.055351395, 0.25662354, -0.06421128, 0.04280437, -0.1780826, -0.059715718, -0.10793822, 0.07774815, 0.07210582, -0.013328952) * inp_4_0_1;
    result4 += M4(-0.035426382, -0.280493, 0.08077352, 0.11304653, -0.1723971, 0.093129374, 0.10798222, -0.06681247, 0.15449065, 0.110271096, 0.02327949, 0.0047790296, -0.0006787552, 0.21360156, 0.05260607, -0.12151236) * inp_4_1_1;
    result4 += M4(-0.05155416, -0.10719303, 0.079216234, -0.09337489, -0.11862967, 0.04589038, -0.045843262, 0.16303205, 0.12735629, -0.030692367, -0.0019168695, -0.035095155, 0.015140135, 0.05351701, 0.14804676, -0.31221804) * inp_4_2_1;
    result4 += M4(-0.018582089, -0.02294081, 0.012760249, 0.062923506, -0.0077005676, 0.025273662, 0.10746959, 0.08425032, -0.004409264, 0.029865256, 0.0022876514, -0.09756803, -0.12040683, 0.09576443, 0.04933016, 0.22419393) * inp_4_0_2;
    result4 += M4(-0.05918672, -0.05983035, 0.008804105, -0.09146206, -0.0095958365, 0.012148726, 0.065903775, 0.08164011, -0.018798828, -0.011830717, 0.03490683, -0.13279028, -0.09900676, 0.070885174, -0.009258596, 0.059239604) * inp_4_1_2;
    result4 += M4(-0.023585327, -0.13291384, -0.027211834, -0.039077748, -0.01639261, -0.020878738, 0.034964733, 0.11542578, 0.027260667, -0.018557478, 0.026066512, 0.018621871, -0.11280682, 0.10234741, -0.013041469, -0.2966604) * inp_4_2_2;
    result5 += M4(-0.0136086, -0.13967893, -0.05672726, -0.017335646, 0.0002134992, 0.09448045, 0.06270034, -0.078941286, -0.07205996, 0.043352064, 0.01783664, 0.1078018, -0.040103342, -0.024944196, -0.0041525005, 0.06408441) * inp_4_0_0;
    result5 += M4(0.26046932, 0.08081783, -0.0026105838, -0.16491257, -0.2090159, 0.4857235, 0.020958273, 0.022247694, -0.14756675, -0.07696551, -0.015256944, 0.033914264, -0.093787044, -0.26747847, -0.029387733, -0.018129993) * inp_4_1_0;
    result5 += M4(0.0081372205, 0.17594741, -0.01835372, -0.082027026, -0.049048062, -0.43753862, 0.05662334, -0.09679483, 0.008639389, 0.13172048, -0.020759149, -0.0068657957, -0.055586882, 0.058574375, -0.048314776, -0.03031471) * inp_4_2_0;
    result5 += M4(-0.029041655, -0.039066724, 0.0009940702, -0.03593689, -0.0015097432, -0.08908485, 0.05059, -0.016737664, 0.04419851, -0.1625572, 0.0031798813, -0.016945878, 0.0012924188, -0.20417224, -0.04237855, 0.13504201) * inp_4_0_1;
    result5 += M4(0.105942436, -0.0060203006, -0.117634654, -0.28217366, -0.071329586, 0.15485147, 0.03289657, -0.04104036, -0.014807765, -0.1803419, -0.050553773, -0.07889858, -0.13749526, -0.12923276, -0.067825206, -0.050327845) * inp_4_1_1;
    result5 += M4(0.051187716, 0.18204735, 0.108026184, -0.08945399, 0.03727977, -0.16325471, -0.034230474, -0.003462421, -0.038614407, 0.078651845, 0.014615852, -0.018881401, -0.014327837, 0.13446791, -0.06634488, 0.11709771) * inp_4_2_1;
    result5 += M4(-0.038507503, -0.07080439, -0.015010201, -0.046422526, 0.10531295, -0.2468859, -0.061738282, -0.017277166, -0.008462534, 0.05361002, 0.03797328, 0.11470449, -0.025898127, 0.092822306, 0.037952233, 0.056602817) * inp_4_0_2;
    result5 += M4(-0.023347149, -0.19628732, -0.03872699, -0.11229214, -0.13448998, 0.10533867, 0.036562324, 0.014279157, 0.12513895, -0.19101495, -0.053062905, -0.04848787, 0.017107269, -0.1821743, -0.0069183214, 0.09404242) * inp_4_1_2;
    result5 += M4(-0.03227529, 0.12732248, -0.021304062, -0.07905249, -0.021448711, 0.11905183, 0.0032582947, 0.04585242, 0.05207712, 0.0023941072, 0.014149266, -0.034820117, 0.0021628106, -0.11335137, 0.0017905742, -0.031041035) * inp_4_2_2;
    result6 += M4(-0.04058973, -0.07187266, -0.048245005, -0.058409475, 0.0025057867, 0.01161525, -0.064393, 0.038169686, 0.040845774, -0.023999969, 0.11442557, -0.11690591, -0.08499407, -0.012445591, 0.07114449, 0.02432088) * inp_4_0_0;
    result6 += M4(0.11987514, 0.13733396, 0.074726395, 0.017786475, 0.006030609, 0.28876612, -0.119516104, -0.041554976, 0.21656185, 0.16279544, 0.016650187, -0.0756368, -0.20061487, 0.05403968, -0.079880014, 0.03902825) * inp_4_1_0;
    result6 += M4(0.02995434, 0.07341209, -0.05555031, 0.007695395, 0.14666489, 0.035554152, -0.09962714, 0.023405029, 0.10243957, 0.08107151, 0.18483637, 0.013110199, -0.04393688, -0.19006371, -0.11488489, -0.01185219) * inp_4_2_0;
    result6 += M4(0.080807015, -0.27972746, 0.18659483, -0.052981157, 0.13467878, 0.057468936, 0.039438944, -0.03500904, 0.035672143, 0.03718163, 0.06469253, -0.13655765, -0.042455085, -0.014213549, 0.032412726, 0.02429518) * inp_4_0_1;
    result6 += M4(0.029401718, 0.5607173, -0.13189836, -0.20543773, -0.13149017, -0.1157152, -0.045588057, 0.09082627, 0.059664458, -0.03953999, 0.10082501, -0.025385829, -0.11394407, -0.33968523, 0.22842035, 0.11088961) * inp_4_1_1;
    result6 += M4(0.0032660908, -0.09505528, 0.15133452, 0.010049385, 0.13449298, 0.16609296, -0.09271402, 0.0065965946, 0.04271979, 0.15847062, 0.01665258, 0.0037471196, -0.035698283, -0.22211246, 0.14823902, 0.010999567) * inp_4_2_1;
    result6 += M4(-0.009943366, 0.1606514, -0.08436454, 0.047936417, 0.012122863, -0.0029268272, 0.022986077, 0.023396913, 0.033284854, -0.15824519, 0.012160615, 0.019969994, 0.017868774, 0.013249255, 0.018331576, -0.018886345) * inp_4_0_2;
    result6 += M4(-0.0010369304, -0.15849379, 0.10686487, 0.09754523, -0.15479888, -0.052375086, 0.20981438, 0.009045211, 0.009542479, -0.062365968, 0.037965465, 0.019717189, 0.14714737, 0.15809226, -0.24163301, -0.00027660147) * inp_4_1_2;
    result6 += M4(0.034207944, -0.027098095, -0.12269275, 0.018180076, -0.07763203, -0.005401103, 0.03841211, 0.003807574, 0.049063385, 0.0042332793, -0.034070786, 0.060251683, 0.08278491, -0.047795538, -0.034730222, 0.04080575) * inp_4_2_2;
    result7 += M4(0.008598643, -0.01025056, 0.0045395726, -0.096066974, 0.07140563, -0.015885344, -0.00956278, 0.005679324, 0.13667487, 0.13057019, -0.017723098, -0.0055476734, -0.050012007, 0.074083365, -0.029034605, -0.07625551) * inp_4_0_0;
    result7 += M4(0.01800466, 0.015401482, -0.00824349, 0.04876423, -0.09541705, -0.05596238, -0.07870143, 0.0097069, 0.029075416, -0.10356644, 0.020305566, -0.08926225, -0.013300934, 0.15569907, 0.007299236, 0.03364238) * inp_4_1_0;
    result7 += M4(0.029649973, -0.23253144, 0.008012129, 0.0293041, -0.011772736, -0.09664959, 0.046410628, 0.091916546, 0.18355888, 0.035857975, 0.029123047, 0.057809796, -0.08223166, 0.16766658, -0.02238666, 0.2290184) * inp_4_2_0;
    result7 += M4(-0.02459202, -0.064524606, 0.0041800616, -0.08289541, -0.039344985, -0.0053382074, 0.02327857, -0.0024502662, 0.109673, 0.12996426, -0.0037074056, 0.08952443, -0.09802353, 0.057846185, -0.020785473, -0.033096336) * inp_4_0_1;
    result7 += M4(0.1309247, -0.068670444, 0.056578074, 0.21144387, -0.08541983, 0.15116541, 0.015910532, 0.0010102838, -0.07968999, -0.16495757, 0.025902065, 0.10995739, 0.027150916, -0.024738224, -0.0028570362, 0.03322488) * inp_4_1_1;
    result7 += M4(0.015034497, -0.007966447, -0.021764537, -0.1271603, 0.0018018684, -0.006687505, -0.03810887, 0.12558247, 0.039489444, -0.005153149, 0.085822605, 0.042885393, 0.057968013, 0.09277222, -0.031174585, -0.16044112) * inp_4_2_1;
    result7 += M4(0.071814775, 0.019486276, -0.030888414, -0.06513687, -0.060411055, 0.02697889, -0.004368822, -0.07968975, 0.06810273, 0.066823676, -2.2222277e-05, 0.12901045, -0.04993632, 0.012455367, -0.006846335, -0.08985208) * inp_4_0_2;
    result7 += M4(0.047369767, -0.07567291, -0.07577727, 0.0013337944, 0.10370303, -0.017996993, 0.0045560016, -0.0064403173, 0.032737333, 0.07194323, 0.046736404, -0.10356291, -0.07516085, 0.08213864, -0.014916402, -0.16007093) * inp_4_1_2;
    result7 += M4(0.12204659, -0.0750289, -0.11127567, 0.15250552, -0.0722747, -0.0057040807, 0.044014167, -0.12777184, 0.10511566, 0.09367614, 0.023128327, -0.02566416, -0.0076413206, 0.083310634, -0.0058278544, 0.23494488) * inp_4_2_2;
    const V4 inp_5_0_0 = inp[5][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_5_1_0 = inp[5][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_5_2_0 = inp[5][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_5_0_1 = inp[5][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_5_1_1 = inp[5][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_5_2_1 = inp[5][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_5_0_2 = inp[5][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_5_1_2 = inp[5][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_5_2_2 = inp[5][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.011721162, 0.037893116, 0.037961323, -0.021029618, -0.2675032, 0.008968081, 0.04289497, -0.062190104, 0.2527399, -0.026867647, 0.059709974, 0.16515842, -0.14266753, -0.053697567, -0.078994036, 0.07856596) * inp_5_0_0;
    result0 += M4(-0.6820754, 0.016760645, 0.1331083, 0.09619399, 0.2505178, -0.036970217, -0.027043987, -0.033308513, 0.098804824, -0.045849636, 0.16648555, 0.29743576, -0.03851231, 0.031232486, 0.045882825, -0.09577001) * inp_5_1_0;
    result0 += M4(0.24689634, -0.03623781, -0.004356022, 0.033638116, 0.42862874, 0.024743767, -0.07738755, -0.16451004, -0.03778639, -0.04020436, 0.016666628, 0.087962635, -0.49754864, -0.040173866, -0.035523754, -0.08025394) * inp_5_2_0;
    result0 += M4(0.04018679, -0.050866, 0.14708944, -0.075042434, -0.6824839, -0.016237877, 0.03546044, -0.115550056, -0.4093353, 0.040067866, -0.05937008, 0.22491606, 0.043442104, -0.025330726, -0.08845115, 0.029490972) * inp_5_0_1;
    result0 += M4(-0.33390588, -0.0043647974, -0.11386547, 0.26776576, 0.3982712, 0.11293044, -0.08897535, -0.2762458, -0.14043559, 0.09604754, 0.101611726, 0.088368036, 0.23426753, -0.004100531, -0.038618118, -0.049380895) * inp_5_1_1;
    result0 += M4(-0.35210901, 0.032589503, 0.04127396, 0.08811926, 0.34959012, -0.012691546, 0.01530366, -0.074639, -0.7249213, 0.045440674, -0.04721674, 0.014715046, -0.09325223, -0.008113824, 0.080229506, -0.06433467) * inp_5_2_1;
    result0 += M4(0.06936163, 0.04956252, 0.023253657, 0.033603113, 0.02851412, 0.015586781, 0.047240246, 0.0034469317, 0.16390446, -0.005879735, -0.044507023, 0.10764289, -0.2657611, 0.017317483, -0.027648233, -0.053186733) * inp_5_0_2;
    result0 += M4(-0.66831166, -0.06398386, 0.051580183, -0.039921228, -0.109676614, 0.056308176, -0.01216149, 0.07253502, 0.38800812, -0.0046815337, 0.071977265, -0.11073208, 0.09114479, -0.023365064, -0.045707423, 0.08103386) * inp_5_1_2;
    result0 += M4(-0.058781754, -0.02478885, -0.014608687, -0.10874189, -0.2278535, 0.016666925, 0.033388015, 0.08891696, 0.26244208, -0.021977616, -0.01277096, -0.075115934, 0.03498475, 0.025368053, 0.04548589, -0.017030057) * inp_5_2_2;
    result1 += M4(-0.09549132, 0.03081619, -0.09044785, 0.014256907, -0.021531858, -0.09713204, 0.03591245, -0.0035455339, 0.18419833, 0.10825727, 0.15057975, -0.16548185, -0.042864207, -0.053235877, -0.14212896, 0.009753234) * inp_5_0_0;
    result1 += M4(-0.07772967, -0.079382576, 0.18724231, -0.017428977, 0.045996785, -0.11890164, 0.0859048, 0.06009567, 0.013348985, -0.22457969, 0.14836961, 0.11804216, 0.056101236, -0.030903568, 0.032602653, -0.0048336335) * inp_5_1_0;
    result1 += M4(-0.029903853, 0.12598144, -0.08853185, 0.013201634, -0.08388492, -0.017915348, -0.062871315, -0.026848132, 0.073303945, -0.023195881, 0.12301724, 0.03710043, -0.16950871, -0.07780667, -0.115344726, 0.10491709) * inp_5_2_0;
    result1 += M4(0.02147613, -0.041079924, 0.06244598, -0.034756336, 0.051101938, -0.11578834, 0.13018285, -0.19475932, 0.046852846, 0.079804465, 0.2622825, 0.23894155, 0.009555988, -0.020025386, -0.07291939, -0.037409134) * inp_5_0_1;
    result1 += M4(0.043373205, -0.031030627, -0.020678198, -0.20832287, 0.024430968, 0.3525706, 0.14054616, 0.10715913, 0.009966185, 0.06921046, -0.30603698, -0.21747349, 0.013927606, -0.11654058, 0.19235551, 0.036126222) * inp_5_1_1;
    result1 += M4(-0.083719805, 0.035442732, 0.2220659, -0.055704895, -0.092189796, -0.027315594, -0.08466276, -0.066525675, 0.00788856, 0.07373484, 0.06072475, 0.12241572, -0.05917617, -0.044625483, -0.022594133, 0.07020917) * inp_5_2_1;
    result1 += M4(0.014820169, 0.07566522, -0.0036050195, -0.001830757, 0.13157102, -0.090900086, -0.07868407, -0.035633087, 0.11755223, -0.014938838, 0.05154112, -0.016292926, -0.09019971, -0.05277714, 0.044650853, 0.16798666) * inp_5_0_2;
    result1 += M4(-0.006422794, -0.11358153, 0.052220766, -0.11469787, -0.016895516, -0.1354395, 0.09364336, -0.10643564, -0.0071886904, -0.08660507, 0.06918152, 0.12114062, 0.002495695, 0.02108617, 0.053645693, -0.022034928) * inp_5_1_2;
    result1 += M4(0.009712308, -0.14748222, -0.01162147, 0.15091604, -0.046011917, -0.016533302, -0.030364316, -0.029280609, -0.0017716546, 0.006174706, -0.050855566, -0.020467972, 0.026991017, -0.008605737, 0.018835438, -0.008565582) * inp_5_2_2;
    result2 += M4(0.029471558, 0.031943727, -0.0784393, -0.046158485, 0.07246974, 0.09442726, 0.07416822, 0.031421084, -0.08808091, -0.051613268, 0.031446304, -0.08413232, -0.05934732, -0.09923784, -0.02298034, 0.02042894) * inp_5_0_0;
    result2 += M4(-0.014088945, 0.30382258, -0.0024223924, -0.019754415, 0.07506246, 0.038779646, 0.016009431, 0.04795287, 0.12704217, 0.20257737, 0.11944925, 0.0763808, -0.0049288296, -0.19501044, 0.1031508, 0.026875393) * inp_5_1_0;
    result2 += M4(-0.02778397, -0.0476556, -0.09866514, -0.01817695, 0.11191423, -0.02095964, 0.12546583, 0.0066911546, -0.04374633, -0.035135437, 0.021489853, -0.041104678, 0.04504083, -0.08852117, -0.07422349, -0.11090967) * inp_5_2_0;
    result2 += M4(0.023475228, -0.0628877, 0.02209939, 0.035580292, 0.019368393, 0.0056697507, 0.061445683, -0.027358206, -0.14387932, -0.045619976, -0.16289587, 0.09081755, 0.09962136, -0.0483214, 0.16546626, -0.07938211) * inp_5_0_1;
    result2 += M4(-0.25694802, 0.028857624, 0.12654129, -0.08231107, 0.08247106, 0.11996857, 0.07987115, -0.030400101, 0.0785678, -0.080137424, -0.11477524, -0.036016468, -0.06968586, -0.19178984, -0.04059003, 0.11404235) * inp_5_1_1;
    result2 += M4(-0.09050018, -0.003931425, -0.021381164, -0.035229374, 0.019139513, 0.10308035, 0.08388677, -0.099238425, 0.031126646, -0.15838753, 0.08852368, 0.13509995, 0.037508976, 0.047188178, -0.07786567, -0.017333182) * inp_5_2_1;
    result2 += M4(0.12948568, -0.10788771, 0.008514704, -0.010528516, -0.17677073, 0.0431416, 0.055351004, -0.08385578, -0.07389325, 0.19039464, -0.067013085, -0.008601595, -0.027597845, -0.06702431, -0.04131158, 0.01905796) * inp_5_0_2;
    result2 += M4(-0.16597399, 0.018187724, 0.078663535, -0.04408701, 0.022586292, -0.05259922, 0.11070529, -0.016909948, 0.17419352, 0.07875131, 0.02580433, 0.019245574, 0.071642846, 0.010659092, 0.032837003, -0.031048631) * inp_5_1_2;
    result2 += M4(0.064914726, -0.020250438, -0.02670876, 0.08200261, -0.043452036, -0.064315565, 0.15611549, -0.0033010785, 0.031512666, 0.04575384, -0.005696005, -0.056569405, 0.04930197, -0.06608894, -0.004199125, 0.061653487) * inp_5_2_2;
    result3 += M4(-0.019622495, 0.057539873, 0.022142664, -0.034139063, 0.10444221, 0.016971897, 0.057910774, 0.028622936, -0.1479045, -0.00011735312, -0.10847691, -0.06502588, -0.0826891, -0.017014958, -0.081570104, -0.03642136) * inp_5_0_0;
    result3 += M4(-0.0382132, 0.10249073, 0.05384503, 0.14880507, 0.0052842135, 0.13426338, 0.030213261, 0.10195696, -0.43574005, 0.0761363, 0.034288604, 0.09499084, 0.09582373, -0.16329944, 0.05994237, -0.18967184) * inp_5_1_0;
    result3 += M4(-0.055853445, -0.13856839, -0.14751707, -0.114789024, 0.017997606, -0.011560973, -0.08181387, 0.066008985, -0.004585338, -0.008327356, 0.024961667, 0.08633505, 0.21173483, 0.1559318, 0.043383025, -0.062694244) * inp_5_2_0;
    result3 += M4(0.03341657, -0.01836578, 0.09372257, -0.12695159, 0.20400779, 0.011588761, 0.16305582, 0.042204775, 0.13987704, 0.015291795, -0.19032024, 0.043227304, -0.16239165, 0.0035410204, -0.05875671, -0.037307862) * inp_5_0_1;
    result3 += M4(0.1061466, -0.14879288, 0.1717854, 0.09753445, 0.0169785, 0.09563755, 0.18283965, -0.118901946, -0.00228193, 0.29119498, -0.2151758, 0.11920872, -0.019218478, -0.19105037, -0.034887284, -0.06698726) * inp_5_1_1;
    result3 += M4(0.0127204405, 0.28161058, -0.056217026, -0.015628234, -0.1422318, 0.05431422, -0.31500682, 0.07526634, 0.1449324, -0.32750857, 0.17043921, 0.021725053, -0.01738095, 0.076675184, 0.109186396, -0.019547395) * inp_5_2_1;
    result3 += M4(-0.22648525, -0.03914505, -0.13362065, 0.1249676, 0.099280775, 0.073033854, 0.052310735, -0.0508873, 0.054763474, -0.015103868, 0.04981763, 0.012730538, -0.13653103, 0.010727579, -0.043875925, 0.00085129566) * inp_5_0_2;
    result3 += M4(-0.048126213, 0.098441556, 0.102313675, -0.02563681, 0.15265478, -0.034660272, 0.12051945, 0.09107134, -0.0864309, 0.06625779, -0.11600276, -0.151097, -0.14686368, 0.030519392, -0.022978632, 0.060010992) * inp_5_1_2;
    result3 += M4(-0.07095806, 0.11953449, 0.1674687, -0.03240722, -0.051910978, 0.1952078, -0.17774998, 0.036619235, 0.07932835, -0.13305546, 0.00832669, -0.014674792, -0.032386098, -0.14897849, 0.0763327, 0.047493376) * inp_5_2_2;
    result4 += M4(0.07651004, -0.04780724, 0.023373108, 0.009000228, -0.06329396, -0.052488048, -0.042411905, 0.17377797, -0.115078636, -0.0927667, -0.36530435, 0.2957649, 0.009829621, -0.020662341, 0.16109765, -0.18863773) * inp_5_0_0;
    result4 += M4(-0.17527246, 0.04539302, -0.1084098, 0.32355526, -0.19003625, -0.04407469, -0.11089954, -0.15803543, 0.1780638, -0.051346306, -0.119961284, -0.044894014, -0.020442719, -0.0067573343, -0.013216026, -0.2740464) * inp_5_1_0;
    result4 += M4(0.0074573373, -0.046949472, 0.090904705, 0.18628238, 0.01751359, 0.060978606, 0.03433963, 0.002664773, 0.07892481, -0.013664677, 0.025283167, -0.07665809, 0.032150317, 0.08063547, -0.062231865, -0.61813724) * inp_5_2_0;
    result4 += M4(0.037206046, 0.020059125, -0.033928804, 0.0862657, -0.06701828, -0.09571023, 0.031865038, -0.14888412, -0.031130435, -0.06855253, -0.17311196, 0.004140546, 0.15482366, -0.07104268, 0.08841237, 0.15739973) * inp_5_0_1;
    result4 += M4(0.07857262, -0.098748334, 0.11115123, -0.30924296, -0.10911173, 0.07475086, -0.059862155, 0.046999816, -0.011055513, 0.36036563, -0.0051163067, -0.45503417, 0.08875827, -0.10895697, 0.03800326, -0.32582393) * inp_5_1_1;
    result4 += M4(-0.09232994, 0.05578486, -0.04025543, -0.47446552, 0.05557701, 0.118986525, -0.057958562, -0.055745587, -0.027169429, -0.019093966, 0.024411863, 0.15306266, -0.06443085, 0.05920936, -0.075786196, -0.0989736) * inp_5_2_1;
    result4 += M4(0.022604186, -0.010792333, -0.12706356, -0.047728106, 0.066253334, -0.049603783, -0.014229159, -0.10699834, 0.060725305, -0.03031649, 0.022361265, -0.14319228, 0.028668681, 0.026904287, -0.15305296, -0.44111165) * inp_5_0_2;
    result4 += M4(0.14800301, -0.060419247, -0.0011125851, -0.18412638, -0.11491754, -0.013583526, -0.027342347, 0.22770956, 0.1549803, 0.0033710352, 0.04563561, -0.16395132, 0.05190715, 0.0042746537, -0.004632277, -0.16931736) * inp_5_1_2;
    result4 += M4(0.035118237, -0.019827496, -0.051558193, -0.08930315, -0.021453165, 0.025032924, -0.017469162, -0.27270868, 0.0070189, 0.0071863434, 0.07106144, 0.07895499, 0.02105514, -0.02583536, 0.017511014, -0.5598377) * inp_5_2_2;
    result5 += M4(0.077227205, -0.08836899, 0.019323207, 0.030815665, -0.00944659, 0.010823681, 0.066734746, -0.0023522687, -0.020959266, 0.13549338, 0.016007943, -0.12903526, 0.013311547, 0.07456845, -0.045074284, -0.015414884) * inp_5_0_0;
    result5 += M4(-0.11371994, 0.08310612, -0.018728746, -0.1580665, -0.11740435, 0.30073237, 0.08504343, -0.038677517, -0.10454964, 0.077919416, -0.014378775, 0.10767681, 0.24930267, 0.060736142, 0.031407394, -0.0065933727) * inp_5_1_0;
    result5 += M4(-0.048882253, -0.15258676, 0.01536242, 0.013373502, -0.07711078, -0.2787352, 0.021204766, -0.044815265, 0.030563032, 0.0013279741, 0.0059247604, -0.06921739, -0.078431144, 0.05205062, -0.01360213, -0.22575042) * inp_5_2_0;
    result5 += M4(0.0068459027, 0.06527184, -0.012772497, 0.11385593, 0.02210313, 0.10926471, 0.11120923, -0.06821426, 0.006362939, -0.21250993, 0.06569661, -0.062118452, -0.020597184, 0.082511514, -0.12413743, -0.09487294) * inp_5_0_1;
    result5 += M4(0.12200882, -0.24574187, -0.080343075, -0.07370511, 0.07225613, 0.090789184, -0.03565121, -0.23790377, -0.0016171224, 0.024101358, 0.081586406, 0.26333913, 0.11634868, 0.216618, 0.0038331607, -0.1456091) * inp_5_1_1;
    result5 += M4(0.03789772, -0.119113386, 0.0057833726, 0.01820191, -0.06559009, -0.0034727657, 0.028064014, 0.034134135, 0.049094353, 0.043499276, -0.053544752, -0.040222183, 0.028502434, 0.26777795, 0.07733642, 0.13420485) * inp_5_2_1;
    result5 += M4(-0.080254644, 0.116081856, 0.037343424, 0.03396638, -0.0049748076, -0.085696, -0.0048974613, -0.026827617, 0.014609966, -0.23313248, -0.019679822, -0.06587613, -0.032526772, 0.12048548, -0.005382651, -0.18288279) * inp_5_0_2;
    result5 += M4(-0.0022187144, -0.15783168, 0.041558743, -0.05361857, 0.1574054, 0.20794438, 0.06051477, -0.07748536, -0.081595786, -0.0047044433, -0.05692804, 0.0036695348, 0.0018137372, 0.03189351, -0.060693014, -0.19346946) * inp_5_1_2;
    result5 += M4(-0.03222261, 0.019704737, -0.024438951, -0.020567207, -0.013221745, -0.01722513, -0.026230602, -0.0693451, -0.05179016, -0.042875435, -0.00920541, -0.014096494, 0.003981904, 0.04870083, -0.026147056, 0.06663038) * inp_5_2_2;
    result6 += M4(0.07787148, -0.26384467, 0.011676844, -0.02207523, 0.032181203, -0.036070604, 0.012219352, 0.0035797453, 0.27868605, 0.30027768, -0.0082854675, 0.078140594, -0.22096178, 0.08304536, 0.02280953, 0.031540133) * inp_5_0_0;
    result6 += M4(0.08977667, -0.020501947, -0.079327084, 0.026937539, -0.22049923, 0.15196484, -0.056442417, 0.0037787603, -0.052464418, -0.1266233, 0.024953026, 0.06405727, -0.06927179, -0.1426037, -0.124035664, 0.02071273) * inp_5_1_0;
    result6 += M4(-0.15415257, 0.00984475, -0.022525402, 0.0333031, 0.008785093, 0.12324702, 0.22380774, 0.005448452, 0.039889734, 0.07223483, 0.047106378, 0.01803392, 0.15585807, -0.042027473, -0.03884451, -0.033903874) * inp_5_2_0;
    result6 += M4(-0.017510561, -0.243866, 0.1274941, 0.07615289, -0.06913453, 0.06877117, -0.069497816, -0.056763362, -0.22494292, 0.3152982, -0.24556457, -0.053800654, -0.00065411045, 0.04979994, 0.056617808, -0.042906404) * inp_5_0_1;
    result6 += M4(-0.18930498, -0.20302846, 0.016491262, -0.045709252, 0.05475915, -0.21390517, 0.09095977, -0.0783542, 0.2454851, -0.14994183, 0.039400306, -0.02026505, 0.025605332, -0.01510014, 0.03912481, -0.05040129) * inp_5_1_1;
    result6 += M4(0.08427402, 0.046083182, -0.15796402, 0.08737499, 0.18236597, 0.187027, -0.2346845, 0.04855453, -0.006506632, 0.16391608, -0.17802262, -0.011089989, -0.00043126184, -0.2826288, -0.04379184, 0.04542469) * inp_5_2_1;
    result6 += M4(0.005365856, 0.15875809, -0.1774488, -0.007934175, 0.11262232, -0.09951517, -0.066277325, -0.0018027063, 0.11495508, 0.061509926, -0.0037259187, 0.033062886, 0.10815757, -0.2585787, -0.11372748, -0.0373796) * inp_5_0_2;
    result6 += M4(0.12366854, 0.100223675, 0.17615482, -0.034040328, 0.01790381, -0.055687055, -0.057332296, -0.010895123, -0.075547695, -0.41722578, 0.20263882, 0.066596866, 0.032483425, 0.2084058, 0.008139326, -0.00045217568) * inp_5_1_2;
    result6 += M4(-0.08017867, -0.11695695, 0.05743284, -0.008279936, 0.047469452, -0.026131393, -0.109735854, 0.08195898, -0.031221077, 0.13522838, -0.033978086, 0.0064558447, -0.002375607, -0.05149196, 0.14960049, -0.06624722) * inp_5_2_2;
    result7 += M4(0.081209324, -0.028714128, 0.0021929748, 0.058186285, -0.0950291, -0.0913209, -0.022429103, -0.08616681, -0.026134642, -0.1173318, -0.008964317, 0.08102435, -0.047208596, 0.026899552, 0.024035495, 0.06669985) * inp_5_0_0;
    result7 += M4(-0.16618584, -0.090714104, 0.011466817, -0.08024786, 0.03317937, 0.09215869, -0.029218724, -0.09907535, 0.038347412, -0.22595273, -0.026188083, 0.08608112, 0.0702895, 0.24769723, -0.043554742, 0.036762547) * inp_5_1_0;
    result7 += M4(-0.02057048, -0.04270443, 0.02104316, 0.012558346, -0.042396948, -0.30786493, -0.013110814, -0.20534857, -0.003529861, 0.0070374333, 0.01203986, -0.25062162, -0.10011119, -0.004675176, -0.007653467, 0.071387716) * inp_5_2_0;
    result7 += M4(0.14369315, 0.0013947814, 0.012088887, 0.12475135, 0.13386431, -0.06119068, -0.033876635, -0.08276338, -0.15755667, 0.10685149, -0.08195113, 0.03904365, 0.07992427, 0.002878067, 0.0198095, -0.030077608) * inp_5_0_1;
    result7 += M4(-0.108935684, -0.08334782, -0.025399676, 0.31102282, -0.20988548, -0.031502366, -0.021197326, -0.15839596, -0.34462228, 0.027497541, 0.0734338, -0.18645774, 0.14525814, -0.02783693, -0.0144077735, 0.13518248) * inp_5_1_1;
    result7 += M4(-0.1224809, 0.13160174, 0.03992526, 0.14536801, -0.128994, -0.15359859, 0.055042196, -0.17033787, -0.078164, 0.09054559, -0.026914524, -0.07891849, -0.081034996, 0.07418141, -0.053278297, 0.23650119) * inp_5_2_1;
    result7 += M4(0.020418456, 0.0017180349, -0.0024439113, 0.010073244, 0.16791458, 0.008547276, 0.0122704115, 0.062033657, -0.101842724, -0.10953232, 0.010202412, -0.08762975, -0.016289804, -0.020739308, -0.013219377, 0.031240124) * inp_5_0_2;
    result7 += M4(-0.10924049, -0.07867336, 0.02539937, 0.15251854, -0.19085774, 0.08920638, 0.041758053, 0.026767002, 0.16656391, 0.01634522, 0.0255307, -0.19063987, -0.06409963, -0.039194122, -0.051366374, 0.04086669) * inp_5_1_2;
    result7 += M4(-0.09019067, -0.08373332, 0.026086537, 0.035140105, -0.055622805, -0.015215585, 0.0070737256, -0.16421403, 0.010024095, 0.043256454, 0.02205574, -0.09592782, 0.060438868, 0.013132059, -0.03560052, 0.1781079) * inp_5_2_2;
    const V4 inp_6_0_0 = inp[6][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_6_1_0 = inp[6][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_6_2_0 = inp[6][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_6_0_1 = inp[6][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_6_1_1 = inp[6][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_6_2_1 = inp[6][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_6_0_2 = inp[6][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_6_1_2 = inp[6][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_6_2_2 = inp[6][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.25549865, 0.01866839, 0.039431408, 0.0753304, 0.1725007, -0.061865397, 0.07727514, 0.16810784, -0.059729807, -0.04274904, -0.08062974, 0.3478331, -0.026573451, -0.006545144, 0.023644714, -0.033793736) * inp_6_0_0;
    result0 += M4(0.47809342, 0.09237844, -0.026488371, -0.13278842, 0.009860102, 0.031124985, 0.1301854, 0.016672969, 0.023711758, -0.017662121, 0.0017375834, 0.6921392, 0.039318793, 0.024982283, -0.007810607, 0.061958924) * inp_6_1_0;
    result0 += M4(-0.15623093, 0.014311557, 0.14955118, 0.014277792, -0.34723818, 0.008921972, -0.05297361, -0.07265093, 0.03600852, 0.054790206, -0.2037285, 0.32811937, -0.57240623, 0.039201368, -0.049047258, 0.082307) * inp_6_2_0;
    result0 += M4(0.06928064, -0.006959872, 0.0088244155, 0.053622093, -0.81694806, 0.042343218, 0.0052555366, 0.006321203, -0.16311483, -0.046471424, -0.17837903, 0.3065725, -0.38574913, 0.0114259, 0.09595935, 0.12092536) * inp_6_0_1;
    result0 += M4(-0.01886233, -0.016139468, 0.061330173, 0.20047931, -0.10324704, -0.038250603, -0.07436317, -0.089537635, -0.4855959, 0.062155254, -0.17602552, -0.29303935, -0.14101923, 0.354044, 0.13469261, 0.072011925) * inp_6_1_1;
    result0 += M4(-0.47206083, -0.040447254, -0.0018539244, -0.18087026, 0.37101793, 0.057238765, 0.13909726, 0.07571212, 0.0256801, 0.0322376, -0.04864739, 0.34420902, -1.063352, -0.001727623, 0.1266632, 0.17253888) * inp_6_2_1;
    result0 += M4(0.1578929, -0.03390434, 0.0806618, 0.036293495, 0.27410576, -0.044947542, 0.042924695, -0.37348467, 0.3635727, 0.015724663, -0.11577997, 0.10109946, 0.3974768, 0.017818283, -0.07406808, -0.12674846) * inp_6_0_2;
    result0 += M4(-0.37514296, 0.012327078, 0.08459914, -0.040639833, -0.070884064, 0.04308976, 0.06383135, -0.028534614, -0.8085789, 0.015912509, -0.13277994, 0.12843055, 0.2227934, 0.14561489, -0.10201895, -0.03196952) * inp_6_1_2;
    result0 += M4(0.071313255, -0.03557252, -0.030578012, -0.06832, -0.06431258, 0.0023394506, 0.06066815, -0.035635736, -0.10628849, 0.0053122654, -0.11690702, 0.0974707, -0.35626644, -0.007770228, -0.13165861, 0.15097927) * inp_6_2_2;
    result1 += M4(0.010722532, -0.060413886, 0.0025617443, -0.05683347, -0.0060687508, -0.120515764, 0.119657815, 0.118034236, -0.03994501, 0.06560819, -0.03154207, -0.011890977, -0.014872438, 0.069835655, 0.068981834, 0.02688391) * inp_6_0_0;
    result1 += M4(0.24419926, 0.08883889, -0.10131201, -0.008026025, -0.035980567, -0.024986153, -0.18385056, 0.27137303, -0.12669353, -0.17854938, 0.004269868, 0.08400904, -0.059264977, 0.009212962, 0.20261711, 0.07123043) * inp_6_1_0;
    result1 += M4(0.07798851, -0.17539379, -0.077602245, -0.08202841, -0.07434742, 0.013545524, -0.14933003, -0.20787378, -0.041921273, 0.037946254, 0.1274501, -0.039973296, -0.03796856, 0.13724388, 0.08839161, 0.124869615) * inp_6_2_0;
    result1 += M4(0.08170982, -0.048988625, 0.03693926, -0.10507414, -0.011644511, 0.03556909, -0.16627087, -0.28572088, 0.030055389, -0.27458343, 0.17420484, 0.034359317, 0.117676854, -0.0221129, 0.004023882, -0.029756268) * inp_6_0_1;
    result1 += M4(-0.05770874, -0.05147325, 0.16323897, -0.28102764, -0.07634861, 0.042023398, 0.049035445, 0.16080263, 0.25359872, -0.31389818, -0.16069245, -0.4295387, 0.16972777, -0.07821529, -0.2811962, 0.18837163) * inp_6_1_1;
    result1 += M4(-0.013196783, 0.05365806, -0.03590078, 0.19361295, -0.043539464, -0.112989284, 0.09403909, 0.12190717, 0.0069803502, -0.3190949, -0.06670349, -0.37155473, -0.017948, -0.2000558, 0.13491306, -0.056607045) * inp_6_2_1;
    result1 += M4(0.1284963, 0.06658746, -0.05565732, -0.08531909, -0.081944264, -0.23453861, -0.02944824, 0.1105278, -0.052361686, 0.051452957, 0.020084629, -0.16310951, -0.11119623, 0.09712203, -0.03247958, 0.08194657) * inp_6_0_2;
    result1 += M4(-0.17648262, -0.04194244, 0.019492624, 0.07680583, -0.055062804, 0.0644879, -0.0028255635, -0.03934646, -0.06274635, 0.12614647, -0.043755352, -0.4082762, 0.110561386, 0.063503906, 0.03613171, -0.03619323) * inp_6_1_2;
    result1 += M4(0.03832698, 0.12350637, -0.0036962922, -0.07070246, 0.028923258, 0.05070263, -0.06726418, 0.013338185, -0.047030736, -0.040520255, -0.12090491, 0.030764194, -0.061692808, 0.049384993, 0.023283351, 0.096599735) * inp_6_2_2;
    result2 += M4(-0.06490009, 0.029529115, -0.13896349, 0.054773726, 0.08093386, 0.081606455, -0.11549969, 0.039770808, -0.14457898, -0.12251246, 0.33668455, -0.026537383, 0.00017089196, -0.07004364, -0.066840746, 0.026754238) * inp_6_0_0;
    result2 += M4(-0.104944244, -0.063328594, -0.11092855, -0.28835315, 0.07227332, -0.069939286, -0.03327757, 0.0072622574, -0.10432136, -0.014355709, 0.3767047, -0.0011466517, 0.034036037, 0.0520003, -0.04560535, -0.089053154) * inp_6_1_0;
    result2 += M4(-0.012875275, 0.07240265, -0.08844178, -0.0016045844, -0.06358234, -0.24024178, 0.02832516, -0.02751322, -0.04479485, 0.04643321, 0.21285772, -0.17683576, -0.015262856, -0.121714145, 0.101196475, 0.07444145) * inp_6_2_0;
    result2 += M4(0.068888314, 0.16422369, 0.14311512, 0.009709359, -0.053808656, -0.068812825, 0.035032794, 0.01729212, 0.02736326, -0.31465796, 0.37375423, 0.04317057, -0.011579161, -0.0055909026, -0.019905778, 0.0117315445) * inp_6_0_1;
    result2 += M4(-0.01383526, -0.19636992, 0.08177079, -0.10478928, 0.040655535, -0.1872379, -0.07483336, 0.09180969, -0.5919428, -0.64135414, 0.33191758, -0.34774902, -0.003577091, 0.2438514, -0.45238173, 0.02329428) * inp_6_1_1;
    result2 += M4(0.08742648, -0.10930704, -0.10967379, 0.1598697, 0.09839403, 0.0044719228, -0.1447543, -0.043181848, 0.03095599, 0.11189882, 0.34316647, -0.23582105, -0.07503516, 0.107340045, 0.28095832, -0.19461145) * inp_6_2_1;
    result2 += M4(-0.0773454, -0.049098548, 0.024317497, -0.035777126, 0.17988834, 0.24042441, 0.0072232634, 0.044626728, -0.0077642705, 0.012671561, 0.22687876, -0.08253654, 0.051944323, -0.074755825, -0.008454468, 0.0121917315) * inp_6_0_2;
    result2 += M4(0.017509246, 0.16466491, -0.033762597, 0.07442264, -0.0007044734, -0.06585714, -0.022964668, 0.048137713, -0.10244437, -0.43407726, 0.31047267, -0.12457055, 0.012120161, -0.029711563, -0.08088706, -0.065171115) * inp_6_1_2;
    result2 += M4(0.025078382, 0.07302156, -0.19376186, -0.013463125, -0.09276473, -0.14218143, -0.012104147, -0.0036720722, -0.04313563, -0.086019024, 0.27816364, -0.06808137, 0.0123799965, -0.23956847, 0.1473852, 0.044234656) * inp_6_2_2;
    result3 += M4(-0.060124185, 0.02964481, 0.08073284, -0.004008714, 0.058839694, -0.0019996124, -0.052490678, 0.0013800273, -0.2902847, 0.08928566, 0.0627425, -0.025271183, -0.051869914, 0.02183179, 0.052776176, -0.074258514) * inp_6_0_0;
    result3 += M4(0.20515329, -0.035329938, 0.07085165, -0.23556328, -0.019601462, -0.009886647, -0.0017190491, 0.012053612, -0.19232869, -0.014435839, 0.12781787, -0.090060465, 0.012441126, 0.03893103, 0.050168425, -0.101745434) * inp_6_1_0;
    result3 += M4(-0.3040352, -0.039447855, -0.06745311, 0.0027688064, 0.32785347, -0.24568759, -0.12546945, -0.12869407, -0.0010663611, 0.14885174, 0.20825167, 0.030246252, 0.059467927, -0.01858566, -0.03548577, 0.11417122) * inp_6_2_0;
    result3 += M4(-0.108961545, 0.01895359, -0.09948317, 0.07393426, 0.025064157, -0.020117035, -0.1335698, 0.114958376, -0.09186519, -0.025641374, -0.07754947, -0.062238764, 0.050348278, 0.025851944, 0.01312635, 0.081415914) * inp_6_0_1;
    result3 += M4(-0.077808745, -0.034618214, 0.11309309, 0.02343958, -0.5843565, 0.06767406, -0.01981566, 0.035289142, 0.016124979, -0.38559628, -0.04768318, -0.44093588, 0.088390596, 0.07934073, 0.17547393, -0.17262931) * inp_6_1_1;
    result3 += M4(0.12207544, 0.09287224, 0.17857175, -0.023664603, -0.010184892, -0.54612154, 0.062446453, -0.058599617, -0.13269894, 0.08949014, 0.20284721, -0.21253866, -0.020540196, 0.30230653, -0.20761885, -0.04011421) * inp_6_2_1;
    result3 += M4(-0.1677915, -0.06910467, -0.08355984, -0.017220227, -0.09741055, 0.042653956, 0.010194234, -0.23767547, -0.4827399, 0.023105418, -0.020651368, -0.050830055, 0.011677661, -0.0730411, -0.055030324, 0.05537162) * inp_6_0_2;
    result3 += M4(-0.30972093, 0.07018591, 0.068023615, 0.07269477, -0.2212539, -0.43075815, -0.0553753, 0.14001456, -0.15684688, -0.034323182, 0.018167213, 0.06104422, -0.079221055, -0.04678329, -0.047663692, 0.08379899) * inp_6_1_2;
    result3 += M4(-0.026087854, -0.032500334, 0.0033097228, 0.012425568, 0.05607942, 0.114676826, -0.011855493, 0.06767334, -0.20579301, 0.31347534, 0.13655138, 1.9253137e-05, -0.029825034, -0.11276443, 0.087472655, 0.10441599) * inp_6_2_2;
    result4 += M4(-0.006078451, -0.008773633, 0.10098322, 0.2002498, -0.082468845, -0.039756265, -0.1474487, -0.6115549, 0.12704664, 0.06878666, -0.029457128, 0.04267096, 0.0048497044, -0.039484803, -0.07040233, 0.06407345) * inp_6_0_0;
    result4 += M4(-0.008150626, 0.032703687, -0.023249546, -0.16531357, 0.17779018, 0.0983855, 0.010703559, -0.0038155895, 0.036134955, -0.20824419, -0.16178057, -0.27969116, 0.08986767, -0.02695196, -0.008107025, -0.24657308) * inp_6_1_0;
    result4 += M4(-0.008391275, -0.016550655, 0.056095034, -0.24330078, 0.026730182, -0.019198906, 0.04042745, -0.27376536, 0.095858276, 0.011933748, 0.023762833, 0.0036537824, 0.030621376, -0.0073574428, -0.043834385, -0.5579115) * inp_6_2_0;
    result4 += M4(0.1182304, -0.20825738, 0.024918564, 0.035473596, 0.10321347, 0.08121824, 0.19633111, -0.18626368, 0.09444193, -0.16716143, -0.047001842, -0.014835151, 0.05562857, 0.07494259, -0.053967353, 0.17265205) * inp_6_0_1;
    result4 += M4(0.13157704, -0.23872721, -0.06415162, 0.20018277, -0.11895376, -0.045944173, -0.12815389, 0.056551974, 0.078141086, -0.46908808, -0.052165095, 0.02573726, -0.1272355, 0.15258187, -0.015615808, 0.25665927) * inp_6_1_1;
    result4 += M4(0.0022445316, 0.044898253, 0.037637554, -0.13287598, 0.17976245, -0.005953429, 0.066081084, -0.108803086, 0.16451728, -0.20745552, -0.13972534, -0.5015971, 0.33940026, -0.29965377, 0.039690398, 0.32395595) * inp_6_2_1;
    result4 += M4(0.0028797772, -0.119873255, 0.054569308, -0.20836231, 0.12402392, -0.01058834, 0.07498839, -0.13982067, 0.16676766, -0.047854334, -0.114750065, 0.16688111, -0.015720403, 0.019169101, -0.08022852, -0.038580406) * inp_6_0_2;
    result4 += M4(0.03203938, 0.01563017, -0.08276119, 0.20426126, 0.1653681, -0.038361255, 0.061459854, 0.026679076, -0.067444205, -0.19745728, -0.12873738, -0.09771651, -0.00662605, 0.11057593, -0.045540214, -0.0010764198) * inp_6_1_2;
    result4 += M4(-0.077313624, 0.10608052, -0.029530916, -0.026247151, -0.006116678, 0.06610318, -0.07694282, -0.20848513, 0.026706625, -0.06596403, -0.03489263, -0.60301745, 0.099831104, -0.07211317, 0.02185356, -0.23710665) * inp_6_2_2;
    result5 += M4(0.051036842, -0.070101045, 0.0076258653, -0.25687984, -0.15503407, 0.15259914, 0.02484544, -0.12776048, 0.13563901, 0.026677137, -0.105840765, -0.11673823, 0.08311901, 0.0043343147, 0.04496344, -0.011476563) * inp_6_0_0;
    result5 += M4(0.16740581, -0.10520855, 0.050404746, -0.2833493, -0.016757231, -0.14620103, 0.021391403, 0.023473393, 0.21174249, -0.009807144, -0.012756758, -0.12120394, -0.11895197, -0.15645972, 0.08674257, -0.055513643) * inp_6_1_0;
    result5 += M4(-0.0032651674, 0.028415114, -0.01085212, 0.076489635, -0.067214355, 0.053001076, 0.009928771, -0.11443755, 0.09485472, -0.2986348, 0.018733673, 0.027140986, -0.04402707, -0.3694604, 0.017697671, -0.056551408) * inp_6_2_0;
    result5 += M4(0.017705293, -0.108980246, -0.005268396, 0.03667514, 0.06415095, 0.18305895, -0.030992107, -0.123827815, 0.004990119, -0.05596839, -0.064798206, 0.009950763, -0.019921888, 0.1894277, 0.04557261, 0.07858067) * inp_6_0_1;
    result5 += M4(0.16307506, -0.03297335, 0.08366294, 0.07378231, 0.05629082, 0.055768274, 0.0013053485, 0.06787066, 0.15019238, -0.06970416, -0.05096177, -0.28106695, 0.008839735, 0.36189583, 0.25212407, -0.16219346) * inp_6_1_1;
    result5 += M4(-0.06884668, 0.10646922, -0.04470124, 0.113750465, -0.0023711799, -0.010810032, 0.036641017, -0.07756196, 0.089190625, -0.2557141, 0.0066953446, -0.0114227105, 0.1923027, 0.08385698, -0.07933594, -0.19107307) * inp_6_2_1;
    result5 += M4(-0.055827245, 0.07985487, -0.0065163695, -0.071106, -0.007960518, 0.03009167, -0.155677, -0.058543287, -0.009229207, 0.00030598027, -0.06297964, 0.1121032, -0.008305476, -0.031655904, -0.001069855, 0.118007824) * inp_6_0_2;
    result5 += M4(0.006271654, 0.020482942, 0.0580864, 0.16882296, -0.086223945, 0.37361482, -0.038729917, -0.035595994, 0.017166276, 0.034542464, -0.0015000039, -0.024286738, 0.065703884, 0.008056918, 0.14082085, 0.0023469627) * inp_6_1_2;
    result5 += M4(-0.11100939, 0.0038935402, 0.039870683, 0.041402586, 0.03540563, 0.14193092, -0.021172678, -0.01802144, 0.051030092, -0.0092426175, 0.05517654, -0.04659906, 0.03650614, -0.058513884, 0.021432618, -0.07831038) * inp_6_2_2;
    result6 += M4(-0.1747323, -0.21779425, -0.19843435, 0.06751147, -0.025943011, -0.16784571, -0.051159214, 0.017094033, -0.15737604, 0.29452977, -0.16706952, -0.0017267764, 0.022958133, -0.064478494, -0.010968503, -0.004184686) * inp_6_0_0;
    result6 += M4(0.30146813, 0.34986874, 0.03457329, 0.0023804966, -0.05284655, 0.027543742, 0.022582548, 0.017253684, -0.33407766, -0.02581084, -0.16100311, -0.14577448, -0.1315029, 0.0641243, 0.009503475, -0.0018062114) * inp_6_1_0;
    result6 += M4(-0.11238792, -0.3118194, -0.20792252, 0.03821905, 0.07153985, -0.11460494, 0.22284661, -0.042073302, -0.025173608, 0.29784298, -0.09008297, -0.000690546, -0.06963081, 0.21155137, 0.11864023, 0.012520024) * inp_6_2_0;
    result6 += M4(-0.093623936, 0.04364195, -0.14851455, 0.0043330044, 0.07184012, -0.29248852, 0.13209835, -0.03895646, -0.4199203, 0.36946222, -0.024672195, -0.06576357, 0.045985006, -0.038515203, -0.12745512, 0.013827264) * inp_6_0_1;
    result6 += M4(0.12542972, 0.08338523, 0.16755381, 0.0015490071, -0.055396736, -0.112609394, -0.43484095, 0.112511285, -0.3406657, -1.6889157, 0.25336587, 0.07728422, 0.30068925, 0.012618532, 0.090825036, 0.027451979) * inp_6_1_1;
    result6 += M4(-0.11768097, -0.13016264, 0.23070724, -0.020566653, -0.13671917, -0.13845585, 0.083283044, 0.017803995, 0.041880526, 0.033216015, -0.071154036, -0.055030577, 0.030110914, 0.076147206, -0.17985517, -0.09704828) * inp_6_2_1;
    result6 += M4(0.06504614, -0.053005297, 0.09580988, 0.011805402, -0.14352565, -0.2898266, 0.35243306, 0.019256571, -0.044895515, 0.21492226, -0.13387816, -0.098959744, -0.05058982, 0.008044507, 0.0421887, -0.058626264) * inp_6_0_2;
    result6 += M4(0.027136788, 0.07399385, -0.01848117, 0.07319213, 0.08301548, 0.04245848, -0.07067542, -0.06680896, 0.098050855, 0.5405217, -0.18807316, -0.11377978, -0.070072696, -0.14026546, -0.019714894, -0.008139303) * inp_6_1_2;
    result6 += M4(-0.023696572, -0.113541186, 0.048270397, 0.046302304, 0.074642, -0.06430253, -0.03721746, 0.029897857, -0.1350287, 0.48020157, -0.22814687, -0.010631202, -0.18137679, 0.10290172, 0.03861555, 0.005425785) * inp_6_2_2;
    result7 += M4(-0.043678343, 0.07878771, -0.0026410879, 0.10027678, -0.042669285, -0.20669271, -0.02107486, -0.023597993, -0.034341488, 0.073938675, 0.013565109, 0.031561196, 0.03253068, 0.0015547898, -0.001736512, -0.007849782) * inp_6_0_0;
    result7 += M4(0.17422676, -0.25808188, 0.026429845, 0.23748839, 0.0018216933, -0.09649758, 0.010997649, -0.12775292, 0.03209481, 0.22237456, -0.033558823, 0.08718474, 0.034938045, -0.15168412, -0.01732834, -0.09994423) * inp_6_1_0;
    result7 += M4(0.0715702, 0.065067284, -0.005710641, -0.21383144, 0.12620367, 0.1561253, -0.03215091, 0.09256261, 0.0027961663, 0.040739745, 0.014528441, 0.11030755, 0.028778994, -0.46644512, -0.046322778, -0.19935724) * inp_6_2_0;
    result7 += M4(0.13022985, 0.021983033, -0.00611117, 0.047153316, -0.1404043, 0.21325867, 0.044710014, -0.23655568, 0.12556912, 0.08647302, -0.023609513, 0.10599391, -0.0053708525, 0.06860996, 0.015292603, 0.0034106297) * inp_6_0_1;
    result7 += M4(-0.05710224, -0.20527647, 0.030291786, 0.18743707, -0.02753568, 0.09346952, -0.031154072, 0.25468433, 0.4219303, 0.10251947, 0.012956088, -0.05990817, -0.28417805, 0.034892008, -0.002166548, 0.0518759) * inp_6_1_1;
    result7 += M4(0.05634898, -0.10008858, -0.013386941, -0.21846488, 0.110872746, -0.17806438, -0.018531784, -0.111652076, 0.15504411, -0.011016819, -0.023749785, 0.40551686, -0.23797524, -0.34949446, -0.063826986, -0.6289416) * inp_6_2_1;
    result7 += M4(0.033402734, 0.03279184, 0.021279575, -0.055837028, 0.107040554, -0.102352515, -0.01056952, 0.003426736, 0.17881922, -0.039088637, 0.009109081, -0.03351382, 0.032603364, 0.016052075, -0.014520571, 0.08379942) * inp_6_0_2;
    result7 += M4(-0.10806929, -0.027973205, -0.0063217166, 0.1537553, 0.083728485, 0.027788192, -0.0058636917, 0.010633223, 0.19679567, 0.119612984, -0.026499374, 0.11809744, -0.02244865, 0.056009702, 0.038470633, 0.16641618) * inp_6_1_2;
    result7 += M4(0.009059396, -0.0039634905, 0.0040612933, -0.0305225, -0.0996885, 0.062311884, 0.00021816736, 0.052810926, 0.04272384, -0.035895634, 0.033798188, 0.4901415, 0.01757463, 0.013709559, 0.063173756, -0.37690845) * inp_6_2_2;
    const V4 inp_7_0_0 = inp[7][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_7_1_0 = inp[7][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_7_2_0 = inp[7][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_7_0_1 = inp[7][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_7_1_1 = inp[7][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_7_2_1 = inp[7][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_7_0_2 = inp[7][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_7_1_2 = inp[7][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_7_2_2 = inp[7][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.18032736, 0.050479252, 0.005876508, 0.06858246, -0.4785183, 0.00014059662, -0.09229342, 0.33764073, -0.15640056, -0.016646868, -0.06139659, 0.4200014, -0.1535797, 0.020241763, -0.09050193, -0.2683885) * inp_7_0_0;
    result0 += M4(-0.20224199, 0.049171705, 0.16441678, -0.101897836, 0.17011185, 0.025424536, -0.06579401, 0.20290676, -0.21370354, -0.013846008, -0.007633334, -0.043663375, -0.35537693, -0.06907114, -0.15016839, -0.39507294) * inp_7_1_0;
    result0 += M4(0.052384224, 0.024622671, 0.008592438, -0.036565542, -0.59922826, 0.030931639, -0.07663378, 0.12371977, -0.060436424, -0.011845648, -0.07395168, 0.07634689, 0.37125593, -0.053063698, 0.05506938, 0.18361165) * inp_7_2_0;
    result0 += M4(-0.31955332, 0.14502892, 0.1541057, -0.0905574, -0.3184567, -0.034196157, 0.010021351, 0.18502766, -0.20708933, 0.014940901, -0.08106392, -0.022636933, -0.063852206, -0.10520436, 0.06511961, -0.18697466) * inp_7_0_1;
    result0 += M4(-0.48950142, 0.20653297, -0.06559811, 0.09515106, 0.64888424, 0.045211665, -0.044382475, 0.0801683, -0.24973856, 0.032321654, 0.07277969, -0.18748628, 0.39062756, 0.16686733, 0.0508868, 0.31631133) * inp_7_1_1;
    result0 += M4(0.18605852, 0.044731054, 0.009679378, 0.13006915, -0.14221776, 0.015834799, 0.031175695, 0.09585184, -0.11521312, -0.020347241, -0.04888903, 0.067057624, -0.2607118, -0.018796489, -0.07759444, -0.083259575) * inp_7_2_1;
    result0 += M4(-0.4214358, -0.07993732, -0.0810058, -0.14225806, -0.70254534, -0.020780366, -0.17984764, -0.016969034, 0.20089582, -0.036504794, -0.005692155, -0.00876083, 0.0065827863, 0.027459724, 0.048963793, 0.0007158412) * inp_7_0_2;
    result0 += M4(0.49230707, -0.061756205, 0.055137776, -0.14117642, 0.08052035, 0.06304406, -0.06774376, 0.033046175, -0.06373938, 0.061640225, -0.041062515, -0.16795193, -0.20042707, -0.01414873, -0.06974255, 0.13710639) * inp_7_1_2;
    result0 += M4(-0.15319839, 0.024190333, 0.0033777477, -0.13636555, -0.3842527, 0.012898345, -0.08426037, -0.22921689, -0.055722, -0.026843602, -0.033507857, 0.12778291, 0.11333519, 0.02305126, 0.025138648, -0.051793348) * inp_7_2_2;
    result1 += M4(0.15167187, 0.04052181, 0.051499814, -0.051355515, -0.09730643, -0.046150703, -0.018547693, 0.19201955, 0.014888786, 0.09916987, 0.024740681, 0.113984555, -0.109757975, -0.02257992, -0.012654931, 0.17300725) * inp_7_0_0;
    result1 += M4(0.056027174, -0.14329425, 0.012754826, 0.20878546, -0.013215755, -0.45915154, -0.114562355, -0.11534854, -0.07824376, 0.02796894, 0.04891409, -0.09926257, -0.11632747, 0.10281914, -0.10120723, -0.15251838) * inp_7_1_0;
    result1 += M4(0.07482496, 0.07586083, -0.018717982, 0.02950684, -0.00048356235, -0.089203, 0.00037414892, -0.09704038, -0.02137893, 0.12625127, 0.042644955, 0.06536979, -0.08493772, -0.083800964, -0.09819245, -0.09968831) * inp_7_2_0;
    result1 += M4(-0.032102194, -0.04286297, -0.2188296, -0.033568747, 0.12474417, -0.009217146, -0.051939543, -0.29312596, 0.058097277, -0.22929974, 0.02688358, -0.05266634, -0.0035563286, -0.0803656, 0.050397053, 0.004289798) * inp_7_0_1;
    result1 += M4(0.041554455, 0.2137591, -0.3172376, 0.0063375575, 0.31678882, 0.1306767, 0.13500659, 0.13709976, 0.10134335, -0.0820301, 0.03757666, -0.07091453, 0.06782225, 0.137762, 0.024666248, 0.1926446) * inp_7_1_1;
    result1 += M4(0.032918397, -0.0751776, 0.018586734, -0.014197054, 0.14500639, 0.028323973, -0.04415429, -0.08964801, -0.050650004, 0.053558264, 0.007284771, -0.06289769, -0.0024701077, -0.016055245, -0.011765297, -0.14654371) * inp_7_2_1;
    result1 += M4(0.079403445, 0.1614709, 0.054709833, -0.023636337, -0.0008569465, -0.090762004, -0.024871018, 0.076839834, 0.1390312, -0.111886166, -0.07020376, -0.10865693, -0.023332248, 0.018594544, -0.020588495, 0.0699533) * inp_7_0_2;
    result1 += M4(0.13043465, -0.16906843, 0.17410235, -0.016587721, 0.09529156, -0.1412246, -0.1015444, -0.2619501, 0.010556095, -0.06689264, 0.032463726, -0.11440151, 0.06459736, 0.022662738, -0.060449153, -0.1251332) * inp_7_1_2;
    result1 += M4(0.057430122, 0.13455506, 0.06741186, 0.06733027, 0.0033541906, -0.10104781, -0.0792447, 0.04259137, 0.043193515, -0.09730177, -0.09888537, -0.30431592, -0.0073371036, -0.061093885, 0.067217804, 0.06844643) * inp_7_2_2;
    result2 += M4(0.043650765, 0.17472705, -0.075174294, 0.09738522, 0.0587685, -0.110935934, 0.00911566, 0.0749145, -0.08007071, -0.03505842, -0.015978215, 0.1453788, 0.019691944, -0.061872683, -0.038818017, -0.088857695) * inp_7_0_0;
    result2 += M4(0.0706152, 0.11909051, -0.13561052, 0.06428141, 0.04069775, 0.026937399, 0.061149474, 0.039920103, 0.01026088, 0.29938108, -0.03457314, 0.013883477, -0.16379185, -0.22469775, 0.13129596, -0.13386832) * inp_7_1_0;
    result2 += M4(-0.0010245107, -0.13177843, -0.06437259, 0.057430938, 0.07705311, -0.1375089, -0.06265362, 0.05156296, -0.07609393, -0.06419758, 0.015797935, 0.046950605, 0.022693451, 0.21934384, 0.059046015, -0.08674352) * inp_7_2_0;
    result2 += M4(-0.100005575, 0.030961672, -0.21404554, 0.07136767, 0.040291462, 0.12218968, -0.004619797, -0.02126642, 0.08576357, 0.16930866, 0.0026514558, 0.10614519, -0.029110657, 0.021427771, 0.04340369, -0.028939063) * inp_7_0_1;
    result2 += M4(0.009746474, -0.27443102, 0.0045091505, 0.27205056, -0.029201437, -0.22663358, -0.004254249, 0.09219433, 0.03522666, 0.0008660928, -0.018817615, 0.058708098, 0.07897582, 0.20996122, -0.1062763, -0.12250883) * inp_7_1_1;
    result2 += M4(0.03840751, 0.019301904, -0.06901862, -0.033423014, -0.010593006, 0.10942218, -0.05112877, -0.02277775, -0.028062483, 0.03636537, 0.059804805, 0.041310992, -0.08274394, -0.14351466, 0.021221854, 0.058782294) * inp_7_2_1;
    result2 += M4(0.096656546, -0.11612794, -0.04023125, -0.031309377, 0.07459929, -0.31584713, -0.031235885, 0.05126064, 0.038398597, 0.1624781, -0.06415882, 0.026896754, 0.017566567, -0.1334861, -0.041360937, 0.0025727842) * inp_7_0_2;
    result2 += M4(0.049767476, 0.16513607, 0.018216977, -0.04333753, 0.109964155, -0.110190265, 0.018696489, -0.045746658, -0.009702958, -0.017226124, -0.015863918, 0.07367491, -0.06213333, 0.03520165, 0.022900669, 0.013943887) * inp_7_1_2;
    result2 += M4(0.006474783, 0.08566169, -0.076600455, -0.032858126, 0.08858227, -0.10096379, -0.05481828, 0.061686356, 0.03466854, 0.1333723, -0.07597452, 0.00031332974, 0.04497317, 0.03829336, 0.040395677, -0.0029758695) * inp_7_2_2;
    result3 += M4(-0.051315155, -0.020224273, 0.06539861, 0.076010786, 0.07267098, 0.037251048, -0.06839353, -0.11282046, 0.06347203, -0.0668686, -0.05516247, -0.10578729, 0.19663435, 0.061168138, 0.050980493, -0.18209188) * inp_7_0_0;
    result3 += M4(0.017460514, 0.016782261, 0.051801343, -0.11736255, 0.04418943, -0.054203, 0.06761381, -0.24979825, -0.06565813, 0.015005665, 0.11721789, -0.025497334, 0.0064750304, -0.012675121, -0.030789608, -0.2403155) * inp_7_1_0;
    result3 += M4(-0.032085083, -0.20711522, 0.030110424, 0.12039292, 0.1506484, -0.054581784, 0.07141841, -0.24734105, 0.029699259, -0.039148327, 0.01691902, 0.006748872, -0.113343045, 0.1508421, -0.15029453, -0.0033176013) * inp_7_2_0;
    result3 += M4(-0.27385038, -0.0114126075, -0.06532417, -0.033342987, -0.20137684, 0.03496213, 0.050336882, -0.31805322, -0.24365687, -0.041502293, 0.14109702, 0.0023533402, 0.27013627, -0.017014926, 0.3143423, -0.289955) * inp_7_0_1;
    result3 += M4(0.45353395, -0.1335793, 0.09707096, 0.15229295, 0.06301656, -0.035181914, -0.03322299, -0.24046217, 0.07310247, 0.058447987, 0.092483684, -0.11137178, -0.30406585, 0.039050546, -0.4479199, 0.17189887) * inp_7_1_1;
    result3 += M4(-0.0356764, 0.029237332, -0.21204613, -0.03606512, -0.06195998, 0.04618513, 0.07466742, -0.10806956, -0.04543264, -0.07947648, 0.028966783, -0.1925889, 0.06641371, 0.0096426895, 0.087748714, 0.015357943) * inp_7_2_1;
    result3 += M4(-0.20363016, -0.06907092, -0.21628092, 0.14888011, -0.045266088, 0.058579672, -0.11850184, -0.028375583, 0.042659663, 0.026968464, 0.0976844, -0.033970937, 0.15262458, -0.09443096, -0.010306658, 0.033706617) * inp_7_0_2;
    result3 += M4(-0.15655135, -0.18420532, 0.037195552, -0.05652797, -0.10763971, -0.076349884, -0.06209913, -0.03685003, 0.20381412, 0.029165488, 0.08071291, -0.044974547, 0.01735027, 0.102438696, -0.09231395, 0.021403462) * inp_7_1_2;
    result3 += M4(0.12492211, -0.012640553, 0.09594369, -0.027329003, 0.035905793, 0.10350715, -0.05709271, -0.13211082, -0.12845235, 0.12005577, 0.02015525, -0.08766518, -0.0043450333, -0.014607149, 0.08112402, -0.039718714) * inp_7_2_2;
    result4 += M4(-0.08335508, 0.011942219, -0.0742204, 0.3430409, -0.011064466, 0.11332454, 0.07984786, -0.13927656, -0.04168866, 0.019086378, 0.024102522, 0.47008246, 0.1422555, -0.08029718, 0.30483496, -0.6829666) * inp_7_0_0;
    result4 += M4(0.09970081, 0.02512142, -0.008085956, -0.21880539, 0.08472903, -0.031116063, 0.0046569235, -0.07413766, -0.035722908, 0.017001446, -0.018825684, -0.090840675, -0.04963943, -0.2350272, 0.11215908, 0.24380179) * inp_7_1_0;
    result4 += M4(0.07266065, 0.009547508, 0.09374605, -0.08564594, 0.015904212, -0.027063204, -0.031113354, 0.03225022, -0.04442178, 0.015072106, 0.066691756, 0.29809752, -0.053652037, -0.03813257, -0.07685945, 0.0030184765) * inp_7_2_0;
    result4 += M4(0.012042214, 0.0071395026, 0.1291913, 0.071588404, 0.11776088, 0.016206311, 0.062744945, 0.04816015, 0.15116332, -0.092037134, 0.037725806, -0.08825357, 0.000703825, -0.10383406, 0.2501493, 0.21531095) * inp_7_0_1;
    result4 += M4(-0.12443392, 0.1454785, 0.18579781, 0.06893543, 0.12399287, -0.08617317, 0.028573606, 0.49850252, 0.11406292, 0.037035365, -0.07780465, -0.15599465, -0.08598744, 0.27194202, -0.17271425, -0.050922357) * inp_7_1_1;
    result4 += M4(0.016695796, 0.052880466, -0.016640693, 0.030424925, -0.005675486, 0.05423147, -0.054998163, 0.14953025, -0.0036806962, -0.0052578244, 0.060790632, 0.16633503, -0.016062815, 0.02446363, 0.01448428, -0.1469078) * inp_7_2_1;
    result4 += M4(0.122555286, -0.051665723, -0.008493639, -0.00026637036, -0.07202258, 0.052863587, -0.09800246, -0.15710363, 0.01785941, -0.018438658, 0.070645496, -0.049943905, -0.06198696, 0.03342617, 0.053972606, 0.10212321) * inp_7_0_2;
    result4 += M4(0.24016288, -0.07905725, 0.1202052, 0.15428461, 0.07350792, -0.01662723, -0.12749398, 0.14278422, -0.00574111, 0.04019782, 0.08178794, 0.3138293, 0.047661424, -0.0005042935, -0.035862844, -0.15156364) * inp_7_1_2;
    result4 += M4(-0.041996203, 0.019070735, -0.00035805913, 0.009599833, 0.101184554, -0.017838927, -0.008331747, 0.02124096, 0.09349601, -0.019191898, -0.035568327, -0.15018336, 0.022469915, -0.017476669, 0.021401804, 0.058833446) * inp_7_2_2;
    result5 += M4(-0.058798548, -0.011790879, -0.04051363, 0.08265998, 0.06924325, 0.24650285, -0.02521507, -0.0478403, 0.11422108, -0.04154651, -0.043445934, -0.0052977554, 0.10469157, -0.079459704, 0.031535905, -0.217932) * inp_7_0_0;
    result5 += M4(-0.16076672, 0.00868976, 0.029634034, 0.029546356, 0.037487168, 0.2049415, -0.036556043, 0.00019055109, -0.07992575, -0.43318, 0.051107336, 0.04452307, 0.1716046, -0.060919773, -0.026201574, -0.25146294) * inp_7_1_0;
    result5 += M4(0.03614023, -0.10366395, -0.0031975603, -0.04241305, 0.11177064, -0.067343846, -0.0005508218, -0.012410488, 0.0014072052, -0.116734244, 0.0051585073, 0.013292399, -0.053088274, 0.08742793, -0.029136127, 0.0826905) * inp_7_2_0;
    result5 += M4(0.02852107, 0.123815075, -0.13125327, 0.05570643, -0.010573843, -0.0064200037, -0.01891146, 0.108631216, -0.004488471, -0.19399065, 0.01045258, 0.1305044, 0.06276305, 0.33171433, 0.124607906, -0.33796513) * inp_7_0_1;
    result5 += M4(0.0036519002, 0.25214872, -0.06846677, 0.11648258, 0.03655316, 0.56597066, -0.021027872, 0.121814296, 0.053644564, -0.2746118, 0.005907392, -0.12748119, -0.038639374, -0.15619795, 0.24656172, 0.26426375) * inp_7_1_1;
    result5 += M4(0.024242124, 0.002055055, 0.011273863, 0.00947118, 0.09554435, 0.33549458, 0.03236135, 0.12490797, -0.001216489, -0.31781363, -0.010423083, -0.023314286, 0.029352296, -0.14510688, 0.056184955, -0.012959017) * inp_7_2_1;
    result5 += M4(-0.0046131597, -0.18937649, -0.119857974, 0.027449576, 0.044008188, 0.062358934, -0.10818125, 0.07070842, -0.092962064, -0.06259546, -0.010402574, -0.0035647117, 0.013334596, 0.12680759, 0.021323135, -0.006236905) * inp_7_0_2;
    result5 += M4(-0.07067456, 0.27566347, -0.047290694, -0.054389216, 0.056548923, 0.23689167, -0.020057451, -0.012384099, 0.15046217, -0.056955423, 0.030537404, 0.054886054, 0.0502582, -0.10422401, -0.03758954, -0.002990442) * inp_7_1_2;
    result5 += M4(-0.008282139, -0.04705908, 0.014731588, -0.0030775156, 0.082554534, -0.05054812, -0.070862, -0.011318489, -0.02704458, -0.63179624, -0.010016325, 0.07513592, -0.007434886, -0.045841817, -0.015490263, -0.020398403) * inp_7_2_2;
    result6 += M4(0.07534006, 0.0041038007, 0.05491227, 0.020379135, -0.13889816, -0.020223796, 0.028185872, 0.013526368, -0.38151032, -0.07624867, -0.05724299, 0.05364103, -0.143156, -0.1350139, 0.054598477, -0.10810209) * inp_7_0_0;
    result6 += M4(-0.061927646, -0.015490008, -0.005776677, -0.0013243305, -0.0014050329, -0.072949946, -0.25296813, 0.019975279, -0.0608969, 0.0057331803, -0.1406989, 0.0022485498, 0.15226193, 0.10241778, 0.19799414, -0.10131972) * inp_7_1_0;
    result6 += M4(-0.003994286, 0.0727863, 0.1708452, 0.030679997, 0.0093101235, -0.110233724, -0.06061389, -0.03563321, -0.13335977, -0.061316244, 0.06965086, 0.036469683, 0.03068355, -0.15174496, -0.0930373, 0.0054836) * inp_7_2_0;
    result6 += M4(-0.084836975, -0.11941689, -0.11577216, 0.054915566, 0.22212887, -0.12751697, 0.15490238, 0.00045272135, -0.30083495, -0.06863955, -0.1493726, -0.02186336, 0.16335514, -0.25551578, 0.27240834, -0.07039017) * inp_7_0_1;
    result6 += M4(0.06439895, -0.10309105, 0.25980318, 0.02358214, -0.22516735, 0.06245773, -0.1084356, 0.0016729629, 0.0017303487, 0.020015992, 0.34333593, -0.03748244, -0.029250884, 0.22112697, -0.3632283, 0.22646648) * inp_7_1_1;
    result6 += M4(0.11782064, 0.07759329, -0.19164701, -2.091163e-06, 0.1460974, -0.11237607, 0.037869696, 0.03687861, -0.03444956, 0.07499665, -0.044257984, 0.001312126, -0.07766496, -0.0025277461, -0.013074869, 0.016024107) * inp_7_2_1;
    result6 += M4(-0.25031295, 0.045460634, -0.16240096, 0.16595274, -0.09420293, 0.034714352, 0.014386672, -0.013163524, -0.083211906, -0.17903885, 0.29961032, -5.6946545e-05, -0.032549936, 0.084362835, -0.04483449, -0.037351184) * inp_7_0_2;
    result6 += M4(-0.04490095, -0.20344874, 0.07404613, -0.10610628, 0.23947807, -0.05113229, 0.02636826, -0.07599486, -0.19290766, 0.17722195, -0.09912739, -0.0025382559, 0.06087254, -0.023672266, -0.044593938, 0.0602632) * inp_7_1_2;
    result6 += M4(0.009675719, 0.08937356, 0.10039644, 0.025037015, -0.08011474, -0.16355367, 0.066157766, -0.08180008, -0.027176777, -0.14826848, -0.070916, 0.008472469, -0.026422173, 0.0056279097, 0.055154692, -0.037830047) * inp_7_2_2;
    result7 += M4(0.02177959, 0.010971479, 0.007863472, -0.027009632, -0.10860227, 0.04839371, 0.016960477, -0.036508035, -0.04487451, 0.102468334, 0.0020849349, -0.07615669, 0.23670946, -0.004045775, 0.010164423, -0.014477323) * inp_7_0_0;
    result7 += M4(0.105766304, 0.24630561, 0.034509607, -0.13582262, 0.03360633, 0.08575465, 0.00013718088, 0.07679556, -0.1056348, -0.1114228, -0.042993356, 0.057569247, 0.06926414, -0.39212027, 0.05494213, -0.09163916) * inp_7_1_0;
    result7 += M4(0.04966397, 0.0014135579, 0.013015152, -0.051635526, -0.14061703, 0.07572927, 0.012975059, 0.027552597, 0.059380826, 0.0017688319, 0.0111296065, -0.031469274, -0.025760664, -0.07389278, -0.030793322, 0.14149317) * inp_7_2_0;
    result7 += M4(-0.14805509, 0.0611934, 0.04565332, -0.17063169, -0.026927467, -0.038892645, 0.039382763, -0.09836769, 0.09700756, -0.004192293, 0.010153645, 0.09639069, 0.08445913, -0.055114236, 0.06937406, -0.0902275) * inp_7_0_1;
    result7 += M4(0.1642096, 0.22263113, 0.25606045, -0.05316456, 0.03876973, 0.051160507, 0.029817099, -0.0077071087, -0.046673886, 0.015261828, 0.028718753, -0.17920472, -0.1390941, 0.08279905, 0.45463878, 0.011310871) * inp_7_1_1;
    result7 += M4(0.048892364, 0.047284402, 0.047763724, 0.09375965, 0.03104397, -0.07231438, 0.06777406, 0.3835322, -0.039784126, 0.076029345, 0.021309532, 0.1520005, -0.036932033, 0.0366679, 0.07721581, -0.031781025) * inp_7_2_1;
    result7 += M4(-0.10723723, 0.07789635, -0.026342547, -0.22010042, -0.09865962, 0.017184082, -0.04114808, -0.115952246, 0.10482514, -0.116197854, -0.00600914, 0.03125787, 0.047293596, 0.081628725, -0.025930265, -0.028738428) * inp_7_0_2;
    result7 += M4(-0.07871179, -0.12755631, 0.102019414, -0.019434277, -0.08410707, 0.038190063, 0.005206577, -0.045085955, -0.08041662, 0.07639658, 0.011015143, -0.13226035, -0.05045824, -0.00382491, 0.08625636, 0.016623862) * inp_7_1_2;
    result7 += M4(0.07867102, 0.04837481, 0.063453466, 0.0077841226, -0.074005894, 0.048833337, 0.051147934, -0.103827566, -0.031380456, -0.025954286, 0.073298894, -0.021739235, 0.0400684, -0.060599495, -0.06368975, -0.023849227) * inp_7_2_2;
    const ivec2 output_base = ivec2(gl_GlobalInvocationID) * ivec2(4, 2);
    imageStore(out_image, output_base + ivec2(0, 0), max(result0, V4(0.0)));
    imageStore(out_image, output_base + ivec2(1, 0), max(result1, V4(0.0)));
    imageStore(out_image, output_base + ivec2(2, 0), max(result2, V4(0.0)));
    imageStore(out_image, output_base + ivec2(3, 0), max(result3, V4(0.0)));
    imageStore(out_image, output_base + ivec2(0, 1), max(result4, V4(0.0)));
    imageStore(out_image, output_base + ivec2(1, 1), max(result5, V4(0.0)));
    imageStore(out_image, output_base + ivec2(2, 1), max(result6, V4(0.0)));
    imageStore(out_image, output_base + ivec2(3, 1), max(result7, V4(0.0)));
}

//!DESC ArtCNN C4F32 DS (Conv2D-4-ReLU)
//!COMPUTE 48 32 12 16
//!HOOK LUMA
//!BIND conv2d_3
//!SAVE conv2d_4
//!WIDTH LUMA.w 4.0 *
//!HEIGHT LUMA.h 2.0 *
//!COMPONENTS 4
//!WHEN OUTPUT.w LUMA.w / 1.3 > OUTPUT.h LUMA.h / 1.3 > *
#extension GL_EXT_shader_explicit_arithmetic_types_float16 : enable
#ifdef GL_EXT_shader_explicit_arithmetic_types_float16
#	define V4 f16vec4
#	define M4 f16mat4
#	define F float16_t
#else
#	define V4 vec4
#	define M4 mat4
#	define F float
#endif

const ivec2 ksize = ivec2(3, 3);
const ivec2 offset = ksize / 2;
const ivec2 wg_size = ivec2(gl_WorkGroupSize);
const ivec2 isize = wg_size + ksize - 1;
shared V4 inp[8][isize.y][isize.x];
void hook() {
    const uvec2 local_xy = gl_LocalInvocationID.xy;
    ivec2 base = ivec2(gl_WorkGroupID) * wg_size;
    for (uint y = local_xy.y; y < isize.y; y += wg_size.y) {
        for (uint x = local_xy.x; x < isize.x; x += wg_size.x) {
            const ivec2 input_base = (base + ivec2(x,y) - offset) * ivec2(4, 2);
            inp[0][y][x] = V4(conv2d_3_mul * texelFetch(conv2d_3_raw, input_base + ivec2(0, 0), 0));
            inp[1][y][x] = V4(conv2d_3_mul * texelFetch(conv2d_3_raw, input_base + ivec2(1, 0), 0));
            inp[2][y][x] = V4(conv2d_3_mul * texelFetch(conv2d_3_raw, input_base + ivec2(2, 0), 0));
            inp[3][y][x] = V4(conv2d_3_mul * texelFetch(conv2d_3_raw, input_base + ivec2(3, 0), 0));
            inp[4][y][x] = V4(conv2d_3_mul * texelFetch(conv2d_3_raw, input_base + ivec2(0, 1), 0));
            inp[5][y][x] = V4(conv2d_3_mul * texelFetch(conv2d_3_raw, input_base + ivec2(1, 1), 0));
            inp[6][y][x] = V4(conv2d_3_mul * texelFetch(conv2d_3_raw, input_base + ivec2(2, 1), 0));
            inp[7][y][x] = V4(conv2d_3_mul * texelFetch(conv2d_3_raw, input_base + ivec2(3, 1), 0));
        }
    }

    barrier();
    V4 result0 = V4(-0.019630853, -0.020203933, -0.016947085, 0.0055588647);
    V4 result1 = V4(0.0005831133, -0.0007624754, -0.057713095, -0.0013824301);
    V4 result2 = V4(0.000582304, -0.0035592646, -0.0030043782, -0.0017010076);
    V4 result3 = V4(-0.007895178, -0.0049307807, 0.03371632, -0.0019162835);
    V4 result4 = V4(-0.002510284, -0.021630984, -0.0020967158, 0.000118758144);
    V4 result5 = V4(-0.03566632, 0.0017878207, -0.019784033, -0.02032119);
    V4 result6 = V4(0.014409585, -0.015226171, -0.01803513, -0.013970582);
    V4 result7 = V4(0.0026150357, -0.018559238, -0.05595339, -0.013973795);
    const V4 inp_0_0_0 = inp[0][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_0_1_0 = inp[0][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_0_2_0 = inp[0][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_0_0_1 = inp[0][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_0_1_1 = inp[0][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_0_2_1 = inp[0][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_0_0_2 = inp[0][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_0_1_2 = inp[0][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_0_2_2 = inp[0][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.029014226, 0.02286098, 0.0081961695, 0.18927036, 0.021511538, 0.006577356, 0.14287521, -0.0472957, 0.030186946, -0.0056116544, -0.12529857, 0.16250785, 0.05940398, -0.00018061756, 0.013013353, 0.121426724) * inp_0_0_0;
    result0 += M4(0.0016672837, 0.02836233, -0.13011564, 0.32340467, -0.15651502, -0.048530936, -0.18542454, 0.6418662, -0.033237074, 0.0025541573, 0.018127903, -0.23362169, -0.0498743, 0.0028444855, 0.068563856, -0.38719884) * inp_0_1_0;
    result0 += M4(-0.006043839, 0.0014504881, 0.023476617, 0.08702376, -0.05205502, 0.0719835, 0.21044049, -0.36214885, -0.20561059, -0.05082029, -0.08126836, -0.9515646, 0.08384253, 0.016390778, -0.03647875, 0.16314434) * inp_0_2_0;
    result0 += M4(0.03853323, 0.03348712, -0.07415701, -0.20441748, 0.062092114, -0.033913635, 0.16614418, 0.16653623, 0.060301516, -0.006468069, -0.22351803, -0.057053983, 0.0546671, 0.0058100885, -0.051612545, 0.055597324) * inp_0_0_1;
    result0 += M4(0.056310363, 0.040550705, 0.04714214, -0.1269283, 0.25436494, -0.06238797, 0.43541268, 0.05074541, 0.010693781, 0.01575818, -0.31833112, 0.13186376, -0.12276363, -0.021268085, 0.07319676, 0.036531564) * inp_0_1_1;
    result0 += M4(0.031084059, 0.038761348, -0.10399106, -0.15012205, 0.14398682, 0.2157425, -0.02963219, -0.10139355, -0.2559023, -0.15922908, -0.16959515, -0.72604275, 0.06673284, 0.058750052, -0.040536027, -0.022704523) * inp_0_2_1;
    result0 += M4(0.0127577055, 0.025290428, -0.15938003, 0.060165573, 0.16222054, 0.042568143, -0.24509065, 0.29887938, -0.034171894, -0.010991284, -0.019111086, -0.12077677, 0.01179633, -0.01677325, -0.103763774, 0.07203628) * inp_0_0_2;
    result0 += M4(-0.015917573, 0.00629798, 0.11867525, 0.15612048, -0.123835675, -0.07461596, 0.20048828, 0.39307493, -0.19838874, 0.038067248, -0.016625904, -0.045404784, 0.035524856, 0.015619378, 0.040166475, -0.017756773) * inp_0_1_2;
    result0 += M4(-0.052201565, -0.0010495051, -0.09704569, -0.0125026945, 0.09934836, 0.06249272, 0.04696253, 0.06718747, 0.061186686, 0.02372086, -0.1190902, -0.2189934, -0.001933659, 0.012925932, 0.02648657, 0.05132481) * inp_0_2_2;
    result1 += M4(-0.00036498695, -0.04452495, 0.29249427, -0.05172869, 0.09003552, 0.026997186, 0.11141529, 0.08583193, 0.10439039, -0.046539534, 0.12898177, -0.07808358, -0.019148855, -0.026603134, 0.21846323, -0.012237659) * inp_0_0_0;
    result1 += M4(-0.0055218115, -0.035623163, -0.4953928, -0.034527436, -0.20458177, -0.053086787, -0.019447295, -0.02387746, 0.057497162, -0.058329705, -0.0464759, -0.08567999, 0.0031367177, 0.076419696, -0.02823875, -0.049390435) * inp_0_1_0;
    result1 += M4(-0.018414484, 0.005558188, 0.4976289, -0.020347513, -0.015447653, 0.034862943, 0.15256602, -0.0069722803, 0.0055704205, 0.0053789937, -0.038530752, -0.042676605, -0.0076524243, 0.0041864603, 0.33455196, 0.021020979) * inp_0_2_0;
    result1 += M4(0.028603641, -0.043814722, 0.044359192, -0.044417094, -0.11979691, -0.0059503103, -0.2848493, 0.029121201, 0.026647048, -0.029983778, 0.12048326, -0.09479337, 0.016006177, 0.008213738, -0.07959892, 0.024229923) * inp_0_0_1;
    result1 += M4(0.007870896, -0.016312473, -0.006975012, -0.04605848, -0.07427102, 0.19281355, -0.00682843, 0.12676834, 0.030929262, -0.051497154, 0.44266582, -0.05822477, -0.013055593, -0.022421382, 0.14065067, -0.019508194) * inp_0_1_1;
    result1 += M4(0.02420337, -0.007711442, -0.04936852, 0.012465729, -0.0072742226, -0.006717581, 0.34797803, 0.026124336, -0.0011535122, -0.03982118, -0.15071878, 0.0015570943, -0.0071034497, -0.008120205, -0.18773887, -0.019774416) * inp_0_2_1;
    result1 += M4(-0.018047445, 0.012255613, 0.16907851, -0.20813647, 0.015623354, 0.028289808, 0.03042336, 0.06800969, 0.011430498, -0.0023240258, -0.08012987, -0.051207505, -0.006501869, -0.0045704925, -0.010869555, -0.027900323) * inp_0_0_2;
    result1 += M4(-0.010329303, -0.002614601, -0.12939474, -0.08537139, 0.008637331, -0.04339309, -0.014794819, -0.07985395, 0.009845105, 0.011468925, 0.047703113, 0.04539838, -0.016324125, 0.02028817, 0.020103468, 0.010832398) * inp_0_1_2;
    result1 += M4(-0.017507602, 0.009991947, 0.18268687, -0.01931067, -0.0039618867, 0.01322992, -0.13871619, 0.035292637, 0.0071499078, 0.0016662447, -0.071454816, 0.010700005, -0.009343838, -0.003103896, -0.058992397, 0.008827326) * inp_0_2_2;
    result2 += M4(-0.022360092, -0.024585938, 0.012468244, -0.023268726, 0.057562694, 0.024998406, -0.003654725, 0.014073296, -0.02933028, -0.0005155262, 0.008373309, -0.0009797658, -0.017680697, 0.012224118, -0.0077857478, 0.0030357346) * inp_0_0_0;
    result2 += M4(-0.041468244, -0.042466484, 0.009938595, -0.034723274, -0.044817507, 0.015789622, 0.0238177, -0.02964391, -0.053147726, -0.060535315, 0.0017136515, -0.02217204, -0.0320946, -0.01339253, -0.00029197114, -0.0048734825) * inp_0_1_0;
    result2 += M4(0.0018553702, -0.0018916146, -0.024649672, -0.009348391, 0.031440075, 0.051722493, -0.025512626, 0.02460595, -0.046916652, -0.03979553, -0.0076102396, -0.02465471, 0.019709874, -0.030787937, -0.00662363, -0.0031850743) * inp_0_2_0;
    result2 += M4(-0.07614971, -0.015964098, -0.008010357, 0.0193449, -0.09889024, -0.008806954, -0.051586486, 0.062619805, -0.05139002, -0.08964276, -0.008525172, 0.02558982, -0.02468337, -0.021545256, -0.010298514, 0.0047768774) * inp_0_0_1;
    result2 += M4(0.0031604904, -0.015616078, -0.004574913, 0.005983716, 0.22566447, 0.16052958, 0.24768417, 0.106585234, -0.05666145, 0.020500515, 0.057517227, 0.08823466, 0.10623635, 0.065674156, 0.004574517, 0.044363253) * inp_0_1_1;
    result2 += M4(-0.02919068, -0.044087663, -0.04182008, 0.007634517, 0.071114026, -0.041319273, -0.029796546, -0.07033041, -0.075307, -0.0071974928, -0.047769476, 0.05643925, -0.0631704, 0.032489892, -0.011689293, 0.024004612) * inp_0_2_1;
    result2 += M4(-0.060891133, -0.015097426, -0.00602813, -0.0036396077, 0.06637916, 0.040520873, -0.013916534, -0.003781549, 0.012504943, -0.005011449, 0.006455793, -0.014676174, 0.009511567, 0.002131537, -0.014233269, -0.00082147674) * inp_0_0_2;
    result2 += M4(-0.05749497, 0.009882231, 0.02075907, 0.025210945, -0.023709182, 0.007413075, -0.029025603, 0.080308825, -0.04012819, -0.002242185, 0.018235713, 0.0026939006, -0.045790248, -0.008002251, 0.011350207, -0.0020454957) * inp_0_1_2;
    result2 += M4(0.0042867004, -0.001520298, -0.008601928, -0.004694065, -0.03402932, 0.023709262, -0.0060744174, 0.017052386, 0.021042774, -0.013173614, 0.0012947391, 0.02885447, 0.03809055, -0.0067739836, -0.017886298, -0.010642107) * inp_0_2_2;
    result3 += M4(-0.11896284, -0.0040692682, -0.0016504464, 0.0004920083, 0.012623236, 0.008226253, 0.014901253, -0.0043690368, -0.082686685, 0.022600291, -0.014997319, 0.0042307554, 0.0108349165, -0.0134602925, 0.0072312374, -0.000689318) * inp_0_0_0;
    result3 += M4(-0.21077223, 0.013969263, -0.013139927, -0.00075837, -0.02271607, 0.107767135, -0.02892698, 0.010119396, -0.014522214, -0.10768658, 0.020370826, 0.0030224086, -0.23178218, 0.04090447, 0.0074397856, -0.0075724022) * inp_0_1_0;
    result3 += M4(-0.08027118, 0.024853531, -0.004054488, 0.008719902, 0.048856925, -0.05724147, -0.0019149181, 0.032542847, -0.12848514, -0.011497779, -0.013102208, -0.020779578, -0.04764413, -0.054644037, 0.010570201, -0.0013388946) * inp_0_2_0;
    result3 += M4(0.05712379, -0.013769518, 0.015971625, 0.0075891623, 0.0757308, 0.05473208, -0.019739043, -0.0026143373, 0.10433699, -0.013385879, -0.06385475, 0.006962632, 0.028972514, 0.047322992, 0.009722473, 0.0057626464) * inp_0_0_1;
    result3 += M4(-0.0016253823, 0.0017754447, 0.009702391, 0.004490395, 0.29265055, 0.093463585, 0.09480777, -0.01757753, 0.063353114, -0.042550355, 0.0414134, -0.0047213505, 0.16713844, -0.07402214, 0.04793968, -0.001277557) * inp_0_1_1;
    result3 += M4(0.0416618, -0.06685264, 0.004990251, 0.0016105425, 0.019349145, 0.011370598, -0.066365674, -0.14382748, 0.010875058, -0.13790098, 0.02992195, -0.035382234, 0.12013545, 0.026033413, 0.025101531, -0.011434843) * inp_0_2_1;
    result3 += M4(-0.012702466, -0.022263238, 0.0109757595, -0.003967512, 0.036699843, 0.026117634, -0.014674625, -0.005841459, -0.021983346, -0.013719651, 0.016085519, 0.005359452, 0.042051155, -0.0007351093, -5.2675492e-05, -0.0038532952) * inp_0_0_2;
    result3 += M4(-0.059835084, -0.0026594873, 0.007124497, 0.0075223697, 0.12245092, 0.01882776, -0.064726464, -0.009025023, 0.06716304, -0.08127598, 0.0044084215, -0.019494103, 0.039593652, 0.02973433, -0.012165752, -0.007955989) * inp_0_1_2;
    result3 += M4(-0.0054553235, -0.06640824, 0.007293754, 0.073941045, -0.044550393, 0.06297899, -0.013565356, -2.637694, -0.123844065, 0.0019184167, 0.0074141305, 0.038503673, -0.06065938, -0.061184783, -0.0013925295, 0.054132912) * inp_0_2_2;
    result4 += M4(-0.026127562, -0.0041976483, -0.022390334, -0.0105110975, 0.024650306, 0.021222431, -0.032034237, 0.022851942, -0.0019197012, 0.0156006515, 0.022551665, 0.02503144, -0.014082687, -0.0012198872, -0.027811456, 0.026952988) * inp_0_0_0;
    result4 += M4(-0.012254751, -0.0077135363, 0.013710671, 0.053174593, -0.0037334692, -0.02571984, 0.09320505, -0.03457815, -0.0021306134, -0.018536951, -0.06540211, -0.045206018, 0.0012736021, 0.02287754, 0.027595008, -0.000421966) * inp_0_1_0;
    result4 += M4(-0.022553097, -0.024589464, 0.010795575, 0.006501352, 0.06412809, -0.028648729, -0.096422, -0.050105393, -0.013353669, -0.035211112, 0.02724979, -0.04957072, -0.012292776, 0.011521384, -0.008958791, -0.011490093) * inp_0_2_0;
    result4 += M4(0.002019394, 0.030969458, -0.05549346, -0.017136952, -0.06850184, 0.0044981455, 0.18289922, 0.03160261, 0.041382864, 0.067405365, -0.07764348, 0.026261086, 0.008031152, 0.018218277, 0.08281696, 0.0124400165) * inp_0_0_1;
    result4 += M4(0.004617735, 0.023163835, -0.08605309, -0.014701205, 0.28924486, 0.082107976, 0.072019525, 0.14618741, 0.09891649, 0.069795795, -0.17603545, -0.18307668, 0.038023796, -0.021143164, 0.049117997, -0.055362076) * inp_0_1_1;
    result4 += M4(0.004877653, -0.0048084, -0.057186626, -0.012841204, 0.03266396, 0.026480515, 0.027754197, 0.020090263, 0.011224292, -0.0076010097, -0.03780078, -0.05892749, -0.0001761632, 0.008298759, -0.0627332, -0.0053169993) * inp_0_2_1;
    result4 += M4(-0.011136487, 0.04123378, -0.052292895, -0.019789081, 0.014489758, 0.026405992, 0.05589674, 0.020034615, 0.005518296, 0.044025082, 0.009326618, -0.02592584, -0.010741502, -0.07799121, -0.03366292, 0.019791855) * inp_0_0_2;
    result4 += M4(0.020833978, 0.02329031, -0.12778509, -0.033678327, 0.122767806, -0.1028542, -0.065426886, -0.021382326, -0.017003525, 0.0076207435, 0.07062517, -0.038685832, -0.010205284, -0.17194957, 0.043617837, -0.0030683735) * inp_0_1_2;
    result4 += M4(-0.025998538, 0.042133212, -0.13010345, -0.0133836595, -0.0035832853, 0.0663176, 0.15308344, 0.047842987, -0.040174585, -0.022907376, -0.13112526, -0.018549638, -0.009079984, -0.017022168, 0.048700243, -0.048816714) * inp_0_2_2;
    result5 += M4(-0.08422751, -0.061414275, 0.021921255, 0.019953791, -0.155246, 0.03102979, 0.11863413, -0.0050528785, 0.032032315, -0.009350831, -0.12857702, -0.061223995, -0.008112628, -0.008381336, 0.040425498, 0.012387881) * inp_0_0_0;
    result5 += M4(0.045930717, -0.0019981393, -0.025224224, 0.013685926, 0.16378328, 0.05068127, -0.20324448, 0.13128464, -0.4402548, -0.12201885, -0.12289866, -0.1182842, 0.09679726, -0.004218763, 0.013038136, -0.009886519) * inp_0_1_0;
    result5 += M4(-0.04940058, -0.012019502, -0.046503708, 0.00046980745, -0.093096055, -0.06857195, 0.0673977, -0.085657746, -0.16239756, 0.033885464, 0.002751634, -0.034666903, 0.0020922343, -0.010313496, 0.023731267, 0.023138069) * inp_0_2_0;
    result5 += M4(-0.008875217, -0.026397893, -0.07529155, 0.005844306, 0.43230805, 0.08028124, -0.0020285328, 0.013712952, -0.13052371, -0.031796113, -0.017119048, -0.07523789, 0.005714663, 0.020409834, 0.027895339, 0.017706744) * inp_0_0_1;
    result5 += M4(0.107625775, -0.05159834, -0.056567498, 0.013878047, 0.3607516, -0.13157165, 0.24996325, 0.25271222, -0.9908888, -0.017925864, 0.02844476, -0.074534446, 0.18332194, 0.08598201, -0.0053493404, -0.019792704) * inp_0_1_1;
    result5 += M4(-0.034790784, -0.054137055, -0.07362781, -0.05030805, 0.057649516, 0.23303865, 0.048352018, -0.13492043, 0.13066287, -0.17399506, -0.47024837, 0.0060545476, -0.076648526, -0.0324354, 0.01494537, 0.03447306) * inp_0_2_1;
    result5 += M4(-0.040216707, -0.0076181795, -0.10403451, -0.0008722097, 0.07219848, 0.013103693, -0.09207495, -0.028758006, -0.20868886, -0.0032260853, 0.01612088, -0.037258353, 0.07180076, 0.012670302, -0.04394917, -0.040524766) * inp_0_0_2;
    result5 += M4(-0.07210772, -0.0978529, -0.031958137, 0.0013781558, -0.3536042, 0.08252107, 0.19297393, -0.060011446, -0.036941912, -0.04320907, -0.16498645, 0.039187606, -0.08434386, -0.055350408, 0.013405099, 0.07059518) * inp_0_1_2;
    result5 += M4(-0.085791804, -0.0025719828, -0.07942844, 0.01634747, 0.35047504, -0.043747373, -0.13976678, 0.01381238, -0.18693866, 0.008215981, -0.09271342, 0.07897367, 0.00096213474, 0.004560909, 0.021399712, -0.016688786) * inp_0_2_2;
    result6 += M4(0.3223437, -0.018705817, 0.06751043, -0.0109014865, -0.10212151, -0.0037406085, 0.066958375, 0.0049891314, -0.39397705, 0.08194618, -0.056244407, 0.0040602107, 0.08120611, 0.012320487, -0.003206553, 0.0075087645) * inp_0_0_0;
    result6 += M4(-0.054189127, -0.0028977247, -0.012377708, 0.031863876, 0.2894241, 0.032615557, 0.035830118, -0.00535661, 0.00462917, -0.05204217, -0.11705109, -0.062104456, -0.1951999, 0.059019603, 0.03314441, 0.040476114) * inp_0_1_0;
    result6 += M4(0.082985446, -0.018295703, 0.03723544, 0.02039639, -0.1810117, 0.030380568, 0.048163444, 0.06526705, -0.12062065, 0.025709538, 0.04809297, -0.13523605, 0.057400957, -0.023971481, 0.026222024, 0.025785083) * inp_0_2_0;
    result6 += M4(0.23005432, -0.086861625, -0.08698359, -0.05102407, 0.14953217, -0.028862594, -0.037677787, -0.013135612, -0.22857912, 0.03793947, -0.018877022, -0.034815054, 0.008169726, -0.008387647, -0.026381133, 0.011607434) * inp_0_0_1;
    result6 += M4(-0.05569171, -0.17986742, -0.044953182, 0.06194192, 0.09930409, 0.15347257, 0.08099259, 0.11737241, 0.09663801, -0.019254424, 0.08801743, 0.006841677, -0.06647213, -0.1020976, -0.0428524, -0.021629326) * inp_0_1_1;
    result6 += M4(-0.13889302, -0.055922303, -0.013920943, -0.019078638, 0.02862868, 0.032787018, 0.024457015, -0.072279535, -0.06962793, -0.13755675, -0.06916918, -0.11167986, 0.0067497985, 0.007652537, -0.0863669, 0.04223071) * inp_0_2_1;
    result6 += M4(0.34172896, -0.04689342, 0.05320212, -0.021878943, 0.13657944, 0.07766678, -0.011383482, 0.0109207155, -0.37339437, 0.006298722, 0.019683305, 0.022954052, 0.12135468, -0.0058612274, -0.002438863, -0.030645557) * inp_0_0_2;
    result6 += M4(-0.14507887, -0.036314886, 0.0034808067, 0.017740933, 0.09654371, -0.005357622, -0.089366354, 0.023192521, -0.012849051, -0.024793372, -0.027878536, 0.0386016, -0.12511098, 0.0031512538, 0.021270586, 0.060388796) * inp_0_1_2;
    result6 += M4(-0.03241134, -0.009064323, -0.00350595, -0.029303275, -0.089444615, -0.058749825, 0.0052322163, 0.005338132, -0.083218835, 0.04321367, 0.06598143, -0.006712348, 0.023389319, -0.0081983805, -0.014149985, -0.0053997408) * inp_0_2_2;
    result7 += M4(0.0064165816, 0.023772758, 0.086096935, -0.0419835, -0.026597466, 0.02639482, 0.05991931, 0.02503154, 0.036016885, 0.046884898, 0.0047327075, -0.038178045, 0.0033412774, 0.0048297886, 0.14081632, 0.010048216) * inp_0_0_0;
    result7 += M4(-0.004972997, -0.008644248, -0.05860016, -0.038632367, -0.019423716, 0.013841948, 0.023917865, -0.02891661, 0.022171177, -0.013412425, -0.057241354, -0.04816984, 0.0023165236, 0.08313761, 0.023467792, 0.01381914) * inp_0_1_0;
    result7 += M4(0.0034928194, 0.022333624, 0.101272546, -0.034427397, -0.0023897928, 0.036586553, 0.36943752, 0.10322516, 0.003057702, 0.01892867, -0.03277542, -0.13049407, 0.004935227, 0.013546172, 0.15273683, 0.011910718) * inp_0_2_0;
    result7 += M4(0.010330783, -0.058083672, -0.14421818, 0.03815183, -0.076944225, -0.051426183, -0.27902767, -0.043273088, 0.021580737, -0.047322042, 0.08699017, 0.054481838, 0.027461756, -0.01565848, -0.019957056, -0.032303173) * inp_0_0_1;
    result7 += M4(-0.0040990915, -0.09441461, 0.022021735, 0.014058743, 0.17577283, 0.07905663, 0.274869, 0.15590426, -0.06257914, -0.03169794, 0.2290933, 0.12794632, 7.5104836e-06, -0.09693202, -0.029876424, -0.008335588) * inp_0_1_1;
    result7 += M4(-0.01216488, -0.017005894, -0.07825837, 0.06475841, 0.004018844, -0.029422164, -0.05830795, -0.055286463, -0.0062307264, -0.20999432, -0.48252273, 0.11072392, 0.0011911088, -0.073066555, -0.0017202836, 0.011448987) * inp_0_2_1;
    result7 += M4(-0.012969522, 0.028715475, 0.2714674, 0.039124627, 0.00291461, -0.049124815, 0.049753662, 0.039011918, 0.010069735, 0.004955023, 0.1608026, -0.02332728, -0.014405712, -0.029833458, -0.00845848, -0.0059459484) * inp_0_0_2;
    result7 += M4(-0.023417247, -0.019720431, -0.28668615, 0.028356358, -0.028887328, -0.0729845, 0.09207323, -0.11115837, 0.024810126, 0.07568514, -0.009047153, 0.008955133, 0.018734671, 0.03185333, -0.04836737, 0.035784885) * inp_0_1_2;
    result7 += M4(-0.008643981, 0.012448063, 0.35191035, 0.012885441, 0.025677921, 0.04071838, -0.15476061, 0.014943695, 0.0040757344, 0.048151486, -0.040000495, -0.002305258, 0.0015839412, -0.012352128, 0.11379945, 0.013315272) * inp_0_2_2;
    const V4 inp_1_0_0 = inp[1][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_1_1_0 = inp[1][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_1_2_0 = inp[1][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_1_0_1 = inp[1][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_1_1_1 = inp[1][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_1_2_1 = inp[1][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_1_0_2 = inp[1][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_1_1_2 = inp[1][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_1_2_2 = inp[1][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.10564019, 0.0009871034, -0.12116235, 0.32802612, -0.09213678, -0.016930902, 0.051417485, -0.062487297, -0.101270474, -0.03158069, -0.067239426, -0.28947434, 0.02506021, 0.025509289, 0.08423762, 0.07842099) * inp_1_0_0;
    result0 += M4(-0.10425539, -0.008326125, 0.111927345, -0.13338113, -0.04713821, 0.011306315, 0.029043349, -0.8414187, 0.20375124, 0.022708708, -0.04757196, 0.48072198, -0.067657754, 0.02330587, 0.02401908, -0.43521905) * inp_1_1_0;
    result0 += M4(0.07858136, -0.021606168, 0.0077208164, 0.74871874, 0.030818643, -0.045482773, -0.25177312, -0.20828071, 0.013966357, -0.008500098, 0.02039617, 0.18799765, -0.015039352, -0.026127663, -0.06517165, -0.2311267) * inp_1_2_0;
    result0 += M4(0.014315629, 0.016890429, -0.020689115, -0.012798945, 0.02254788, -0.0043233726, -0.20252874, 0.15531352, -0.20411412, -0.044801183, -0.034396082, -0.21203661, 0.039384857, 0.056365322, -0.039837394, -0.08068615) * inp_1_0_1;
    result0 += M4(0.08297107, 0.016703697, 0.14841911, 0.06592308, -0.26658666, -0.01807931, -0.026633313, -0.51989126, 0.033557344, 0.05997684, -0.12977509, -0.26879063, -0.1780621, -0.077619635, -0.06124032, -0.0089097535) * inp_1_1_1;
    result0 += M4(0.04503286, -0.0052308957, 0.012190699, 0.21704927, 0.065742716, 0.06765363, -0.13569011, -0.004085615, -0.078904085, -0.07511553, 0.100375704, -0.051172227, 0.040330682, 0.044720694, -0.15970008, 0.13857487) * inp_1_2_1;
    result0 += M4(-0.12408519, -0.022959512, -0.020957824, -0.2104158, 0.11115637, -0.0047819996, -0.45600432, 0.48425263, -0.10535357, -0.043936994, 0.011736408, -0.23284417, -0.0038364066, 0.0077831256, -0.011003734, -0.051441323) * inp_1_0_2;
    result0 += M4(0.13418292, 0.019407382, -0.08678784, 0.15254323, -0.11067747, 6.2348976e-05, -0.02868386, -0.9265139, -0.010668649, 0.036019105, -0.017969547, -0.3172346, 0.014602508, -0.06897064, 0.02937137, -0.98752344) * inp_1_1_2;
    result0 += M4(-0.11254668, -0.02977125, 0.015502765, -0.53555274, -0.048482466, 0.013014542, 0.088991776, -0.28638998, -0.011431273, -0.031799152, -0.029584879, -0.36541834, -0.061192404, 0.031801604, 0.06912044, -0.1711109) * inp_1_2_2;
    result1 += M4(0.036788475, -0.024526373, -0.6516839, -0.114943884, 0.08729029, 0.019594409, -0.04815362, -0.06253301, 0.035611235, -0.028184127, 0.181006, -0.023496365, 0.012979062, 0.008367028, 0.26580444, 0.07834232) * inp_1_0_0;
    result1 += M4(-0.0017992669, -0.081134364, -0.30674392, -0.028183026, 0.043881692, 0.021846997, -0.16429113, 0.06883632, 0.032522794, -0.023744315, -0.29403198, -0.06164467, 0.065965205, 0.03797162, -0.25638998, 0.03211797) * inp_1_1_0;
    result1 += M4(0.005406836, -0.0023838226, -0.54298896, -0.0011952851, 0.008673561, -0.04833578, -0.22858115, -0.029089939, 0.0026423628, -0.011947744, -0.8271357, 0.0055868584, -0.002335089, 0.014559685, 0.65556645, 0.023631036) * inp_1_2_0;
    result1 += M4(0.0057950364, -0.0028225773, 0.17997375, -0.10089108, -0.06399851, -0.04083491, -0.2769135, -0.15603465, 0.014321025, -0.002614829, -0.22837797, -0.042070158, 0.08913583, 0.02853458, -0.6531322, 0.052335713) * inp_1_0_1;
    result1 += M4(-0.010592094, -0.042031135, -0.53494, -0.011655881, -0.0023926012, 0.015333817, -0.21902274, -0.04240381, 0.032275237, -0.041106526, 0.10210777, -0.08320675, 0.020675581, -0.038554907, -0.4581991, -0.038672093) * inp_1_1_1;
    result1 += M4(0.0044920654, 0.0045692464, 0.00090195925, -0.035185367, 0.0015931199, -0.011826896, 0.07412675, 0.0030263194, -0.010371283, 0.0011932265, -0.114924744, -0.011444286, -0.012450722, -0.012948182, -0.19795877, -0.018955104) * inp_1_2_1;
    result1 += M4(0.04728661, -0.035827335, -0.04946188, -0.10324664, 0.027213868, -0.0069419392, -0.05179075, 0.04254913, 0.0038773401, -0.011933547, 0.15198012, 0.003966487, -0.01176764, 0.007661447, -0.10880418, 0.023878902) * inp_1_0_2;
    result1 += M4(0.004264003, 0.0026208924, 0.049869258, -0.028042417, 0.011503123, 0.0018902518, -0.023232024, -0.019563181, -0.0025172047, -0.008057449, -0.09679248, -0.06520401, -0.0016907458, 0.020145494, 0.039808873, 0.0714251) * inp_1_1_2;
    result1 += M4(0.010206913, -0.0012352889, 0.08781393, 0.0008406442, 0.007822184, -0.008304924, -0.03537706, -0.0032865962, 0.0020027542, -0.0027273255, -0.15653197, 0.0028748908, -0.008910178, -0.019475082, 0.15478474, -0.024207039) * inp_1_2_2;
    result2 += M4(-0.007974266, -0.07511412, 0.01331159, 0.0018125548, 0.018510763, 0.029278515, 0.014852634, 0.028470527, -0.03525058, -0.009520992, -0.0037032785, 0.0012591606, 0.13107724, -0.0076376605, -0.0050187786, -0.021123527) * inp_1_0_0;
    result2 += M4(-0.121487595, -0.047232125, 0.021946186, -0.026206082, 0.024193281, 0.06301761, -0.039558094, 0.035069708, 0.002650147, -0.03986637, 0.006318133, 0.035465963, -0.086715594, -0.0817644, 0.0017224872, -0.056187835) * inp_1_1_0;
    result2 += M4(0.006475299, 0.018153133, -0.00863129, 0.0095861, -0.023927487, -0.050041266, 0.0050529586, -0.0028280925, -0.06130866, -0.020197442, 0.019199343, 0.0074563855, 0.06571905, 0.030047469, -0.0012489391, -0.019109381) * inp_1_2_0;
    result2 += M4(-0.058493968, 0.016616693, 0.010488884, 0.0063233757, -0.11529822, 0.002079019, 0.012040106, 0.006380833, 0.035635438, -0.013378892, 0.022759544, 0.0035850354, -0.025836097, -0.013258461, -0.06790903, -0.01859846) * inp_1_0_1;
    result2 += M4(-0.12920573, -0.14313377, -0.017513728, 0.036742184, -0.04497386, 0.11606931, 0.15286815, 0.02963015, -0.16326885, -0.0736226, 0.0015204649, 0.040470857, 0.16410014, -0.089971825, -0.009220336, 0.02190897) * inp_1_1_1;
    result2 += M4(-0.019661693, 0.007848105, 0.020523924, -0.010055541, -0.09519459, 0.010009867, 0.027387196, 0.052971937, -0.00074890454, -0.073003404, 0.02795627, 0.008711411, -0.0678765, 0.08048196, -0.03384864, 0.05285065) * inp_1_2_1;
    result2 += M4(-0.07076568, 0.0021829738, 0.0033287331, 0.022028739, -0.03917492, -0.031058284, 0.0131844515, 0.0018446267, -0.047293853, -0.009462318, -0.00093317096, -0.013098985, 0.07184707, -0.016932108, -0.025307218, -0.037247244) * inp_1_0_2;
    result2 += M4(-0.039302178, -0.0939744, -0.00035090165, 0.034448225, 0.03175705, -0.028638596, -0.05838136, 0.045880176, 0.045557085, -0.05433778, 0.010939126, -0.010817058, -0.05643269, -0.014178044, 0.02756147, -0.03972019) * inp_1_1_2;
    result2 += M4(-0.01057952, -0.06087359, -0.011186025, 0.011804587, -0.017458282, 0.0164759, 0.000643385, 0.03430929, -0.0741279, -0.06092643, -0.0009261822, 0.023434984, 0.05975373, -0.08686781, -0.022238605, -0.02783096) * inp_1_2_2;
    result3 += M4(-0.21668924, -0.0018456831, -0.015881535, 0.0025236132, 0.020698212, -0.008043453, 0.01530254, 0.001561705, 0.009237476, 0.013618812, 0.007864316, 0.0012479818, -0.0055469135, -0.023037616, 0.005127602, -0.0010967504) * inp_1_0_0;
    result3 += M4(-0.34475142, -0.1139067, -0.0031779855, 0.0031889058, 0.052832097, -0.08852734, 0.037328858, -0.002523469, 0.06502466, -0.07621191, -0.0033256887, -0.0013271789, -0.16083215, -0.016656205, -0.022743348, -0.0070232656) * inp_1_1_0;
    result3 += M4(-0.107955754, 0.0037210134, -0.021177102, 0.0141778765, -0.014323636, -0.05090193, -0.012349464, 0.0055510327, 0.0288632, 0.015572724, 0.0069105364, 0.0025495498, -0.09478825, 0.0060060616, -0.009166795, 0.007911287) * inp_1_2_0;
    result3 += M4(-0.118987404, -0.0048904107, -0.0001670392, -0.0033507263, -0.085788794, -0.04884034, -0.01908255, -0.008862365, -0.004044651, -0.051348597, 0.014379893, 0.0028131409, 0.014599414, -0.0070618703, -0.023657532, -0.0005558133) * inp_1_0_1;
    result3 += M4(-0.23639749, -0.18969756, -0.051911425, -0.00027612536, -0.1469669, -0.020090185, -0.028428596, -0.026700597, -0.20728114, 0.112096466, 0.008268507, -0.0031451392, 0.11812107, -0.099277325, -0.041207004, 0.02887686) * inp_1_1_1;
    result3 += M4(-0.03144088, 0.08155473, 0.0010938682, -0.0024200168, 0.034090806, 0.011459201, -0.003164837, -0.00759923, -0.1410256, -0.105183795, -0.0048975428, 0.009601632, 0.12163046, 0.086094886, 0.05815644, 0.01558153) * inp_1_2_1;
    result3 += M4(-0.044572305, -0.028940426, 0.0128844725, -0.0012975035, -0.049914967, 0.016282542, 0.002327248, 0.0020151201, -0.025021676, 0.025435338, -0.00958043, -0.0057402886, -0.12721567, 0.0068981885, -0.0011482995, 0.00040864968) * inp_1_0_2;
    result3 += M4(-0.10851981, -0.11716497, -0.0028677483, 0.008277846, 0.031903747, -0.13438885, -0.044756144, 0.0386938, -0.0019153835, -0.09885151, -0.023784906, 0.02711763, -0.098005794, 0.1134859, 0.044021208, -0.012123633) * inp_1_1_2;
    result3 += M4(-0.010081716, -0.048188884, -0.0144090215, -0.7849137, 0.07649272, 0.02387461, -0.010298057, -1.8996816, -0.019490851, 0.028405992, 0.009493138, -0.5216453, -0.0066947658, -0.15144758, 0.002107536, -0.8440536) * inp_1_2_2;
    result4 += M4(0.0041373926, -0.031676404, -0.07344062, -0.024613284, -0.005659224, -0.019528538, 0.03916561, -0.02652958, 0.005908541, -0.0751768, -0.029393097, -0.0046547004, 0.00032582093, 0.023829535, -0.041727073, -0.06716299) * inp_1_0_0;
    result4 += M4(-0.019929102, -0.07482185, -0.1337666, -0.05998741, 0.019123657, 0.007977295, -0.07598002, -0.1098417, 0.030904502, -0.054647066, -0.049063966, 0.08450172, -0.07078054, -0.053575158, 0.03195287, 0.0170796) * inp_1_1_0;
    result4 += M4(-0.020884069, -0.015331419, -0.031278346, 0.022443643, 0.005919153, -0.039955568, 0.05766617, 0.005538881, 0.017394764, -0.02439917, -0.020323474, -0.022216892, -0.0058453106, 0.0068845693, -0.0027451732, 0.015330908) * inp_1_2_0;
    result4 += M4(0.018747551, -0.07477498, 0.07328981, -0.010140858, 0.0051164404, 5.9453643e-05, -0.032282695, 0.008762313, 0.04823261, 0.02650726, -0.0057209823, -0.036476843, -0.05119907, 0.048798714, -0.10922935, 0.0413809) * inp_1_0_1;
    result4 += M4(0.0024015487, -0.035491273, -0.057986613, -0.08858876, 0.013404442, -0.028687216, 0.12608816, -0.12677118, 0.08299463, 0.033300314, -0.08823404, -0.09989066, 0.13545477, 0.14588962, 0.02350962, 0.019470949) * inp_1_1_1;
    result4 += M4(0.006355667, -0.019985197, 0.01735482, 0.038518872, 0.011628806, 0.00353034, -0.13648103, 0.053808495, 0.015433845, 0.011491723, -0.076385275, -0.02952067, -0.009119481, 0.025154224, -0.052297704, -0.0141719775) * inp_1_2_1;
    result4 += M4(0.018554643, -0.09325859, -0.07255845, -0.062368333, 0.017205639, -0.1872029, 0.07133896, -0.1051178, 0.0050375643, -0.018869733, -0.07257633, 0.0019997992, -0.009564156, 0.08737795, 0.028308073, 0.032500405) * inp_1_0_2;
    result4 += M4(0.02472202, -0.11595651, -0.118725576, -0.07820457, -0.03081071, -0.048043665, -0.1784976, -0.064481504, -0.009228719, -0.07230997, 0.019451298, -0.062154338, -0.014775567, 0.071928576, -0.008541906, -0.0056853653) * inp_1_1_2;
    result4 += M4(0.019554771, -0.041783683, -0.2048251, -0.013899762, 0.0030491564, 0.02964482, 0.119087726, 0.0027610825, 0.022493904, -0.06482813, -0.1802083, 0.0018826112, -0.015794976, -0.053776175, -0.088921025, -0.07277107) * inp_1_2_2;
    result5 += M4(0.10212752, -0.06158509, -0.013099818, 0.004439775, 0.1653267, 0.0729414, -0.0520197, 0.038833942, 0.05501931, -0.030644529, -0.05117315, 0.042658757, -0.37974694, -0.12085927, 0.094700344, -0.11828593) * inp_1_0_0;
    result5 += M4(-0.016211286, -0.030393012, 0.038791526, 0.027253574, -0.15506344, -0.13464929, 0.09767936, -0.12906228, -0.070268154, -0.034337457, -0.015251173, -0.044899747, 0.09403228, 0.076571144, -0.13684484, 0.017976819) * inp_1_1_0;
    result5 += M4(-0.055664077, -0.02595298, -0.018015595, -0.035426784, 0.12503995, 0.018510127, -0.023496257, 0.034832276, -0.06412033, -0.022447987, -0.012775494, 0.015910631, 0.17283283, 0.009489777, -0.020970944, 0.05619129) * inp_1_2_0;
    result5 += M4(0.025043027, -0.033545773, 0.027682658, 0.001807103, -0.08915673, -0.016941214, 0.010497499, -0.11441297, -0.013735894, -0.011003915, -0.10752142, 0.007602714, -0.45816112, 0.025335327, 0.29057547, -0.1146705) * inp_1_0_1;
    result5 += M4(-0.025527827, -0.05719856, 0.037479647, 0.031216253, -0.11414065, 0.15228136, 0.028756686, -0.05535962, -0.14358433, -0.10588122, 0.043749172, -0.11600279, 0.18555386, -0.18816279, -0.37286878, -0.036026258) * inp_1_1_1;
    result5 += M4(0.09790024, 0.026784526, 0.09396815, -0.036566455, -0.1848125, -0.021412935, -0.08275013, 0.029398764, 0.21138544, -0.011075172, -0.10437544, 0.039135743, -0.37219775, -0.06130057, 0.1017845, 0.13139236) * inp_1_2_1;
    result5 += M4(-0.074961, -0.0008856749, -0.0048028766, -0.025535125, 0.11347011, -0.057601273, 0.026919857, -0.010748921, -0.13829665, -0.017013084, 0.0009415979, -0.06665906, 0.048011724, -0.015380583, -0.012741363, 0.036383122) * inp_1_0_2;
    result5 += M4(-0.30884805, -0.046225503, 0.034406472, -0.05064812, -0.03939748, 0.059534766, 0.0013943191, -0.03271031, 0.01579074, -0.0072057396, 0.00036167345, -0.037148103, 0.2505463, -0.110507295, -0.24168527, -0.0017526996) * inp_1_1_2;
    result5 += M4(0.24251416, -0.09137612, -0.081253305, -0.030102711, -0.22542667, 0.0010232467, -0.048650727, -0.023068296, -0.037886474, -0.09738003, -0.051062103, 0.022420654, -0.48296562, 0.005969272, 0.10309865, -0.051050644) * inp_1_2_2;
    result6 += M4(0.5372062, -0.16106609, -0.04149448, 0.041472502, -0.01749466, -0.06428957, -0.049371015, 0.01859799, -0.054764625, -0.04691539, -0.022502897, -0.03537589, 0.12195047, 0.020899633, 0.04923248, -0.00029914296) * inp_1_0_0;
    result6 += M4(-0.051922254, -0.16971667, -0.10884476, -0.058757637, -0.021530448, -0.075081855, -0.110569865, -0.041633, 0.1934725, -0.10908616, 0.01622351, 0.020740192, -0.17505358, 0.07253326, 0.004558935, -0.032147415) * inp_1_1_0;
    result6 += M4(-0.03277899, -0.053984832, -0.06443889, 0.08375847, 0.0994722, 0.006644484, 0.06896404, -0.0011605065, 0.05780748, -0.050081287, -0.010715517, -0.05907234, 0.03446792, 0.017113209, 0.115609095, 0.012099106) * inp_1_2_0;
    result6 += M4(0.18454687, -0.0031910907, 0.01387486, 0.051649995, -0.08683453, 0.014209436, -0.029626219, 0.017437108, -0.088259235, 0.017888414, -0.011841363, -0.00934567, -0.11641673, -0.024534848, 0.053118058, -0.07896368) * inp_1_0_1;
    result6 += M4(0.10851011, -0.13876878, -0.07454382, -0.12032542, -0.19298133, 0.19720665, 0.018810134, -0.050703805, 0.062382832, -0.059305806, -0.026566492, -0.018668585, -0.34138036, -0.24775398, 0.051397804, -0.0011364371) * inp_1_1_1;
    result6 += M4(-0.03667229, 0.0026650503, -0.049831904, 0.016634941, 0.009555454, 0.055380695, -0.008289743, -0.00358401, -0.05957922, 0.03174134, 0.04522486, -0.046526268, 0.055495445, -0.03551521, -0.025630176, 0.081085905) * inp_1_2_1;
    result6 += M4(-0.69086117, -0.049464934, -0.04811634, -0.016634312, -0.3549309, 0.0646538, -0.033196762, 0.027491497, -0.27360734, 0.0015055344, 0.0156024285, -0.042446084, 0.017941222, 0.0162519, 0.022245537, 0.022293659) * inp_1_0_2;
    result6 += M4(0.40971538, -0.17927046, 0.020264598, -0.08730099, -0.13249333, 0.059938595, -0.052376024, -0.0787026, 0.09049811, -0.049459867, 0.024893174, -0.04821187, -0.33588773, 0.057962198, 0.046425126, 0.0029218057) * inp_1_1_2;
    result6 += M4(-0.21319477, -0.049604084, -0.015987242, -0.044675022, 0.031381965, -0.046972707, -0.052350983, -0.008150279, -0.047885746, -0.04321621, -0.011105961, -0.03372666, 0.10683438, -0.011646137, -0.012465078, -0.0034802137) * inp_1_2_2;
    result7 += M4(0.015621053, -0.031790942, -0.07677681, -0.058822174, -0.00063354627, -0.0768685, -0.36099964, 0.01805292, 0.0062560607, -0.024716243, -0.041198432, -0.0074002133, 0.03339227, -0.011244067, 0.15665485, 0.015834711) * inp_1_0_0;
    result7 += M4(-0.0023703424, -0.14517693, -0.14158814, -0.013083191, -0.014407214, -0.068160966, -0.19246788, 0.08724711, 0.01284312, -0.03811861, -0.10875274, 0.019034864, -0.009427322, 0.028130127, 0.023039078, -0.09475219) * inp_1_1_0;
    result7 += M4(-0.0154258385, -0.04839113, 0.030025655, -0.084296465, 0.00014344443, -0.040306654, -0.18481742, 0.0053816065, -0.0032385632, -0.041210465, -0.28662044, -0.02798222, -0.011000202, 0.08470507, 0.05868549, 0.009511077) * inp_1_2_0;
    result7 += M4(0.019055532, -0.0044698995, -0.19585451, -0.025215251, 0.020125717, 0.020899389, 0.02705329, -0.078781515, 0.022865314, 0.05907692, 0.15164897, -0.028814226, -0.10603417, 0.037819006, -0.1780903, 0.03064547) * inp_1_0_1;
    result7 += M4(0.027429081, -0.11999357, -0.6162681, -0.11776403, 0.017813709, 0.020937547, -0.24109325, -0.02840179, -0.034118906, 0.108237356, 0.026714433, -0.051721197, -0.008281638, -0.04505502, -0.18829937, 0.08958135) * inp_1_1_1;
    result7 += M4(0.023737492, -0.030369999, -0.23213406, -0.08568402, 0.002146959, 0.059657242, 0.25986722, 0.05114112, 0.0052577197, -0.04842675, -0.04327735, -0.10425699, -0.0012165165, 0.12730141, -0.2505404, -0.012244716) * inp_1_2_1;
    result7 += M4(0.022858046, -0.052733544, -0.2724457, -0.028704168, -0.03711218, -0.04269634, -0.12762643, -0.052078452, -0.025268279, 0.004388526, 0.06639666, -0.0071201907, -0.051490918, 0.08538524, -0.027352115, -0.03820489) * inp_1_0_2;
    result7 += M4(-0.013099071, -0.17201267, -0.3055251, -0.034689076, 0.01446058, -0.012838688, -0.18217815, -0.03572515, -0.020696506, -0.014170187, -0.20371588, -0.0047768406, 0.0054908437, 0.051226217, 0.014367226, 0.0539353) * inp_1_1_2;
    result7 += M4(0.0018133505, -0.049717527, -0.21436131, 0.03560726, 0.021964261, -0.022915874, -0.2130354, -0.02230845, 0.011661267, -0.032011364, -0.30568662, -0.009268537, -0.018580599, -0.06072703, 0.12954317, -0.0105308555) * inp_1_2_2;
    const V4 inp_2_0_0 = inp[2][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_2_1_0 = inp[2][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_2_2_0 = inp[2][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_2_0_1 = inp[2][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_2_1_1 = inp[2][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_2_2_1 = inp[2][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_2_0_2 = inp[2][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_2_1_2 = inp[2][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_2_2_2 = inp[2][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.02949464, -0.02356985, 0.032172278, 0.043848157, -0.14357899, -0.023314655, -0.06341459, -0.55366105, -0.009629065, 0.01687559, -0.020027634, 0.02693547, -0.014682138, -0.009004141, -0.1879035, -0.3012157) * inp_2_0_0;
    result0 += M4(0.0412154, 0.054317806, -0.047446992, 0.017945472, 0.063780636, -0.02248521, -0.30467847, 0.22875768, -0.0010157306, 0.0037280566, -0.1307477, 0.17373756, -0.019100914, -0.026129737, 0.082013875, -0.2764095) * inp_2_1_0;
    result0 += M4(0.06075337, -0.013232479, 0.18890007, -0.095520295, -0.09148896, -0.0022510088, 0.043159336, 0.15157928, 0.03252904, -0.01468051, -0.02605866, 0.053885538, -0.083202064, 0.008962122, -0.016032347, -0.053589802) * inp_2_2_0;
    result0 += M4(0.041937552, 0.023623368, 0.069814496, 0.1273915, -0.12229944, -0.037536364, 0.14406304, -0.109094806, -0.0050321096, -0.0102569675, 0.054721393, -0.2196644, -0.16508111, -0.044940084, -0.1742843, 0.045224085) * inp_2_0_1;
    result0 += M4(-0.42586023, -0.0010559262, -0.24764259, -0.0942418, 0.08910765, 0.007466642, -0.32232785, -0.29508692, -0.3078156, 0.09070344, -0.37599328, -0.86200905, -0.01907109, -0.056224506, -0.2995207, -0.11489803) * inp_2_1_1;
    result0 += M4(0.1588742, -0.123077355, 0.026685163, 0.2859154, -0.18465415, -0.06393044, 0.15188298, -0.21405996, 0.115594916, -0.22523737, -0.08735009, 0.1984797, -0.06358481, -0.041796107, -0.008815579, -0.26249075) * inp_2_2_1;
    result0 += M4(0.04482481, -0.0116510475, -0.057131525, 0.1814658, -0.18096238, -0.01214541, 0.0804038, 0.22453931, 0.0046470896, -0.011856588, 0.064444266, -0.11210435, 0.053828172, -0.023894459, -0.18481912, -0.28954685) * inp_2_0_2;
    result0 += M4(-0.15944853, 0.023152718, 0.025517419, -0.93156207, 0.09392915, -0.017814713, -0.32125312, -0.22752365, 0.021918816, 0.05328419, 0.054821707, 0.117180504, -0.1173496, -0.044847786, 0.051322717, 0.54038405) * inp_2_1_2;
    result0 += M4(0.086439624, -0.020307312, -0.08316797, 0.6322039, -0.05394367, 0.0040287576, -0.10940878, -0.36724934, -0.0034562685, -0.008336475, 0.01860736, 0.08380833, -0.10037761, 0.028244877, 0.03503143, -0.37087402) * inp_2_2_2;
    result1 += M4(-0.053751286, 0.014390391, -0.9158875, 0.044073164, 0.06357939, -0.017064448, 0.0035360425, 0.036730543, -0.057384007, 0.068006314, -0.52228975, 0.050486427, -0.035812348, -0.029167471, -0.049512945, -0.09070449) * inp_2_0_0;
    result1 += M4(0.032960627, -0.124916025, -0.08292904, -0.030631246, -0.02242392, -0.13404274, -0.24944508, -0.17220043, -0.022385735, -0.17297384, -0.060927, -0.07183669, 0.015446384, -0.06427171, -0.38601103, -0.026711056) * inp_2_1_0;
    result1 += M4(0.0018064982, -0.020125015, -0.8266053, 0.012886235, 0.028517677, 0.022567913, 0.13987564, 0.018321484, 0.023683392, -0.029732374, -0.2015438, 0.0067892177, 0.006656577, 0.04710627, -0.44580504, -0.0059177363) * inp_2_2_0;
    result1 += M4(0.014413041, -0.044464964, 0.18808262, -0.5380751, 0.055882145, 0.029777233, -0.0002438424, -0.07118383, -0.03762712, 0.03204967, -0.13353603, -0.37225226, -0.009751197, 0.01911453, -0.19437753, -0.3164939) * inp_2_0_1;
    result1 += M4(0.0123919565, 0.011953078, -0.24194354, 0.07185415, 0.0006968793, -0.064041235, -0.10071755, -0.038334418, -0.002658149, 0.005593597, -0.28099483, 0.011412254, 0.015102343, 0.13475555, -0.046345327, 0.022389282) * inp_2_1_1;
    result1 += M4(-0.02504501, -0.038566455, 0.14227201, -0.041204974, -0.0036371762, -0.021842973, -0.098501846, -0.016957434, 0.010278224, 0.016272843, 0.18863592, -0.009678392, 0.009037947, 0.008596352, 0.27319637, -0.0031779096) * inp_2_2_1;
    result1 += M4(-0.01438294, 0.009463615, 0.046743885, 0.10147421, -0.0067832144, -0.010674907, -0.01454492, -0.1015601, 0.025184143, 0.0024183074, -0.09338495, -0.04954126, 0.0013701606, 0.003991467, -0.06200298, 0.07727345) * inp_2_0_2;
    result1 += M4(0.01043237, 0.011537056, -0.14804915, -0.1048008, -0.0014263225, -0.0035356276, -0.0057846415, -0.023671687, 0.014703273, 0.0153625775, 0.048723765, 0.042805083, -0.02450708, -0.018580599, -0.27853864, -0.120987706) * inp_2_1_2;
    result1 += M4(-0.0041671186, -0.007998098, 0.0076339487, 0.025865654, -0.0013985473, -0.0025089022, -0.12417633, -0.0027084188, 0.026599893, 0.0061252234, 0.046620928, -0.04220568, 0.0045302208, -0.014585104, 0.039205838, -0.005503192) * inp_2_2_2;
    result2 += M4(0.05469642, 0.00880122, -0.019153029, 0.015226725, 0.03583239, 0.035023905, 0.00992571, -0.0043312674, 0.055041716, -0.02204679, 0.020149091, 0.020197013, -0.09083792, 0.02364468, -0.0016606109, 0.0061462414) * inp_2_0_0;
    result2 += M4(-0.061547946, -0.0018271441, 0.0088682985, 0.040277943, -0.14790225, -0.06317741, 0.029175717, -0.043704834, -0.0018453582, 0.08118154, -0.05384852, 0.012119945, -0.03433264, -0.016911367, 0.0028185789, -0.042835437) * inp_2_1_0;
    result2 += M4(0.0054593366, 0.021853486, 0.04659759, 0.029852001, -0.01293814, 0.0619008, -0.0019879139, 0.017971735, -0.069691315, 0.030055402, -0.0016301244, 0.02215551, 0.014197163, -0.030741313, 0.01558277, -0.025564628) * inp_2_2_0;
    result2 += M4(0.04392027, -0.003015399, 0.018362205, 0.00871312, 0.065119736, 0.036311697, 0.03780289, -0.020801047, 0.0322611, 0.07270227, 0.023561196, -0.050607797, -0.1979295, -0.06451647, 0.007017661, -0.041320626) * inp_2_0_1;
    result2 += M4(-0.32901508, -0.13026823, -0.051715016, -0.1375664, -0.08602337, 0.24440564, 0.070425205, 0.19089387, -0.20469517, -0.24090773, -0.20666267, -0.18494904, -0.044046536, -0.09996543, -0.058940813, -0.039476983) * inp_2_1_1;
    result2 += M4(0.027192784, 0.019866485, 0.04592014, -0.00095344684, -0.0018714699, -0.053862017, 0.031182764, -0.033335127, 0.040235125, 0.079145186, -0.0066408487, -0.0035072442, -0.0031215341, -0.0043301205, 0.002010794, 0.012240556) * inp_2_2_1;
    result2 += M4(-0.010838469, 0.003973489, -0.008007419, 0.005636849, -0.05296129, 0.0033810083, -0.008491928, -0.018088007, 0.012011103, 0.022602243, 0.0018436884, 0.007800881, 0.02962768, -0.05088667, -0.009430806, 0.060763363) * inp_2_0_2;
    result2 += M4(0.032553524, 0.016853621, -0.0065063294, 0.0061273403, -0.03521912, 0.011301963, 0.04109519, 0.03674073, 0.01972461, 0.007992671, 0.016564455, 0.048319828, 0.12877032, 0.035887253, 0.019275358, -0.0006587917) * inp_2_1_2;
    result2 += M4(-0.04961192, -0.038295984, -0.0019344696, 0.01921268, -0.042087883, 0.054504704, 0.006324845, 0.013472749, 0.06010804, -0.038210396, 0.0037293867, -0.0012134209, -0.08395253, 0.015729297, 0.007872595, 0.038557947) * inp_2_2_2;
    result3 += M4(0.0682392, -0.01748233, -0.0025476834, 0.00022846021, -0.12002295, -0.031518534, 0.014348131, -0.0013067108, -0.014461619, -0.046497814, 0.004438042, -0.0017682567, -0.1351002, 0.04503463, 0.017927628, -0.0015654366) * inp_2_0_0;
    result3 += M4(-0.42533174, 0.053671844, -0.0052764644, 0.0024646216, -0.14282458, -0.0029951879, -0.022127312, 0.0011293038, 0.14441976, 0.031062428, -0.009593276, -0.0069406694, -0.14666834, -0.05477885, 0.041265152, -0.010142168) * inp_2_1_0;
    result3 += M4(-0.05935503, 0.0066683567, -0.003438396, -0.0073810522, -0.13152257, -0.0058360184, -0.00055884593, 0.0021168806, 0.04417406, -0.08899797, -0.0034196305, 0.0086868005, -0.014169692, 0.0013284974, 0.0044233636, -0.00239655) * inp_2_2_0;
    result3 += M4(-0.19460349, 0.03835693, 0.022882506, -0.0071562035, -0.011227329, -0.038072616, 0.036838446, -0.012046283, -0.079764806, 0.014744349, -0.0041977875, -0.004493605, -0.10678511, -0.046111245, 0.06074222, -0.0127014415) * inp_2_0_1;
    result3 += M4(-0.6618206, -0.271684, 0.059243232, -0.010867134, 0.06333855, -0.00892665, -0.06081472, -0.00085095206, -0.38751158, -0.16827194, -0.006961252, 0.02012217, -0.25355256, -0.098920874, 0.14217387, -0.0006212) * inp_2_1_1;
    result3 += M4(-0.0925359, -0.016826864, 0.0011438022, -0.017197141, 0.017672235, -0.13393383, -0.05751393, 0.0003054376, -0.00395151, 0.0640665, -0.030452793, 0.001139068, -0.06925891, -0.045174018, 0.024533877, -0.028412389) * inp_2_2_1;
    result3 += M4(0.0568973, 0.02727236, 0.012254896, -0.0010458834, 0.03114695, -0.11237583, -0.015673188, 0.024437401, 0.014852821, -0.043585304, -0.020317275, 0.0057403, 0.038072452, -0.048989598, -0.034877747, 0.008052977) * inp_2_0_2;
    result3 += M4(-0.08299502, -0.16301166, 0.023015415, -0.009295113, -0.07966392, 0.027263688, 0.046476077, -0.044820357, -0.054188374, 0.0017814604, -0.014217653, -0.023214472, 0.16599613, -0.16101956, -0.06787493, 0.039176553) * inp_2_1_2;
    result3 += M4(0.011562686, -0.007154924, 0.015487015, -1.249324, -0.068126306, -0.013976628, 0.008679863, -0.27954614, -0.027012719, -0.010006287, -0.013441648, -1.2898427, 0.06538114, 0.022704804, -0.0017430465, -1.6017876) * inp_2_2_2;
    result4 += M4(-1.6767905e-05, 0.054012895, 0.05395114, 0.025822282, 8.373989e-06, -0.004030579, 0.010605015, -0.006342089, 0.017250761, 0.04053831, -0.0012769063, -0.000120780765, -0.0021776715, 0.018011857, -0.07837788, 0.037070435) * inp_2_0_0;
    result4 += M4(0.002993812, 0.011511728, 0.039229706, -0.03720411, -0.049716037, -0.02377402, 0.053049948, 0.06520724, -0.013229867, 0.008027749, 0.04185892, -0.10819173, 0.013655095, -0.0023796023, 0.050672296, -0.038204633) * inp_2_1_0;
    result4 += M4(0.045987323, 0.055015836, -0.087926336, 0.08060905, 0.0019731976, 0.016741956, -0.11273221, -0.1017184, 0.019596394, -0.0014549987, -0.046335258, -0.010973263, 0.006860217, -0.010650581, -0.03986876, -0.012183278) * inp_2_2_0;
    result4 += M4(-0.0056915237, -0.14458571, -0.05675197, 0.027984906, 0.03632888, 0.0042609335, 0.034971327, -0.18149531, 0.010510183, -0.14883968, 0.03500206, -0.12933722, -0.016236927, -0.010194545, 0.0062120752, -0.12531868) * inp_2_0_1;
    result4 += M4(-0.119703576, -0.078816004, -0.19367667, -0.0388088, 0.071721986, 0.03552824, -0.029865729, -0.055697083, -0.22531575, -0.13237704, -0.21622662, -0.046340745, -0.11498288, 0.1068085, -0.27314258, -0.17678747) * inp_2_1_1;
    result4 += M4(0.034425646, -0.003032999, 0.2652338, 0.008746375, 0.030992877, -0.05970579, -0.013371291, -0.085708655, -0.005362292, -0.0013392637, -0.050137974, 0.06467549, -0.0052085123, -0.077882536, 0.055377983, 0.0019261253) * inp_2_2_1;
    result4 += M4(0.0063337353, -0.0548766, 0.09298851, -0.003673725, 0.006425546, 0.024745962, -0.10903742, -0.010701397, 0.015213928, 0.024895772, -0.0048282547, 0.017960574, 0.004345396, -0.5222029, -0.12920025, -0.075153746) * inp_2_0_2;
    result4 += M4(-0.00925755, -0.0808667, -0.43999755, -0.063682474, 0.00047808915, 0.05355608, 0.18102412, 0.04184798, -0.03046436, 0.018914824, 0.029403169, 0.01175165, 0.085052945, -0.36315736, -0.3834316, -0.012637597) * inp_2_1_2;
    result4 += M4(0.009134696, 0.026736109, -0.124241546, 0.002771604, 0.005974923, 0.002184874, 0.020605639, -0.0220492, 0.004900664, 0.044306125, -0.044663448, 0.023604482, 0.027447697, -0.010122655, -0.0449803, 0.010615064) * inp_2_2_2;
    result5 += M4(0.19852966, -0.020750927, 0.1009414, -0.00064920576, 0.16173467, 0.039475527, 0.03594438, 0.073212296, 0.18383077, 0.02193832, -0.056991376, 0.14154854, 0.27796927, -0.045176808, -0.07516182, -0.056829788) * inp_2_0_0;
    result5 += M4(-0.08755043, -0.05543641, -0.05351581, -0.002128459, 0.110897236, 0.055706143, 0.06669801, 0.0640928, -0.0029081604, 0.033184998, -0.043074943, -0.15503189, -0.14285378, 0.0066907518, 0.08276026, 0.063158445) * inp_2_1_0;
    result5 += M4(0.1255744, 0.0013409907, 0.022942856, -0.063292235, -0.12895972, -0.08206139, 0.10281916, -0.053325243, 0.014380615, 0.001914901, -0.036685426, 0.021416144, -0.056183733, -0.0069214012, 0.0046769013, -0.000763444) * inp_2_2_0;
    result5 += M4(0.028492054, 0.009656496, 0.08011515, -0.012448892, -0.028989911, -0.024642931, -0.06551496, 0.07061513, 0.16786176, 0.041954942, -0.100194275, 0.11098428, 0.3751338, -0.23052543, -0.2795384, -0.09157766) * inp_2_0_1;
    result5 += M4(-0.15152113, -0.24092163, -0.34685025, 0.011411849, -0.004497781, 0.14216553, -0.0014919408, 0.07369032, -0.5091528, -0.2567205, -0.17475855, -0.18816756, -0.8630488, -0.100157514, 0.093546525, 0.08602679) * inp_2_1_1;
    result5 += M4(0.03280494, 0.09269089, -0.0051226118, 0.020073058, -0.076450646, 0.062494732, 0.05080121, -0.104285486, -0.25753534, 0.079407856, 0.07939573, 0.020195654, 0.09275388, -0.011420669, -0.083275124, -0.024738723) * inp_2_2_1;
    result5 += M4(0.11159275, -0.013040227, 0.021128668, -0.0112917945, -0.06622273, 0.021610757, 0.1263201, -0.016419679, -0.08711444, 0.04884584, 0.017160228, 0.06205989, -0.29715282, 0.03623376, 0.1307768, -0.010562509) * inp_2_0_2;
    result5 += M4(-0.13296938, 0.07763097, 0.04882142, -0.015518627, 0.07212257, 0.0005022548, -0.031401016, 0.030251468, -0.20142704, 0.044103432, 0.013669651, 0.03545684, -0.22302571, -0.029193953, -0.044090565, 0.0029605723) * inp_2_1_2;
    result5 += M4(0.033154186, -0.064496465, 0.020766297, -0.006475509, 0.036452472, 0.018332554, -0.029324416, 0.016537188, -0.13196795, -0.045240168, -0.033111095, -0.05673973, 0.4177384, -0.06414283, -0.028649967, 0.0066752294) * inp_2_2_2;
    result6 += M4(0.219756, -0.041017868, 0.0056035034, -0.0023022508, -0.012698562, 0.04770604, -0.010528559, -0.0072988328, -0.22237776, 0.004266959, 0.020746693, 0.09496946, -0.10932979, 0.09357496, -0.0175649, 0.09548029) * inp_2_0_0;
    result6 += M4(0.06190553, 0.0832598, -0.07343053, -0.03331077, -0.03223299, -0.014383905, 0.089496255, 0.06884077, -0.27129576, 0.018229282, -0.1013769, -0.1258466, -0.17042087, 0.043919038, -0.17308068, -0.12756926) * inp_2_1_0;
    result6 += M4(-0.0019934545, -0.02596509, -0.35205722, 0.03576349, -0.0905059, -0.03782968, 0.12409483, 0.009205904, 0.07962228, -0.03384441, -0.2118896, 0.010711773, 0.047737014, -0.018883089, -0.095552385, -0.023482164) * inp_2_2_0;
    result6 += M4(0.3619714, -0.12037536, 0.06980492, -0.0038480277, -0.13599412, -0.045780998, -0.020638432, 0.09540492, 0.05226319, -0.025044406, 0.024637537, 0.061739076, -0.31584847, -0.25328162, -0.067744434, 0.11376082) * inp_2_0_1;
    result6 += M4(-0.19830525, -0.21468021, 0.09263018, 0.09266251, 0.12444597, -0.09227372, -0.22695811, -0.07547027, -0.0727088, -0.22940782, 0.06838378, -0.15768127, 0.13925691, -0.18157776, 0.030119682, -0.19574264) * inp_2_1_1;
    result6 += M4(-0.05944051, -0.0012434443, 0.1762373, 0.0006792802, -0.109633476, 0.024355927, -0.13502625, -0.058956724, 0.017240528, 0.022909405, -0.06632788, -0.03402968, -0.087934464, -0.027659507, -0.0009917293, 0.01566622) * inp_2_2_1;
    result6 += M4(0.04458839, 0.038288187, -0.015756315, 0.009438311, -0.08370704, -0.04700996, -0.016691001, -0.004300983, -0.36216176, 0.036551442, -0.022637002, 0.038327064, -0.00678544, 0.028308153, -0.026902799, 0.013125011) * inp_2_0_2;
    result6 += M4(-0.27208656, 0.015620107, -0.022632696, -0.019374894, -0.06151895, -0.0657133, 0.05166568, -0.030753745, -0.11960672, 0.018760882, 0.0045448993, 0.017949808, 0.29144016, -0.06008608, -0.15503906, 0.013849654) * inp_2_1_2;
    result6 += M4(0.01564954, 0.055554185, 0.0035712742, -0.03207525, 0.026650697, 0.008084935, 0.03679324, 0.00605627, 0.05822736, 0.019464124, -0.020481063, 0.02136087, -0.08392423, -0.071137525, -0.09649514, 0.0028041643) * inp_2_2_2;
    result7 += M4(0.015169834, -0.041150194, -0.017009806, 0.0012689976, -0.015867354, 0.0043616393, -0.00081089925, 0.0096781785, -0.0003568763, 0.028840352, -0.06420557, 0.034825213, 0.00823519, 0.032143362, 0.20170394, -0.015382856) * inp_2_0_0;
    result7 += M4(0.037529998, 0.07387278, 0.038433976, -0.07730492, 0.020754652, 0.06207833, 0.13563812, -0.038597647, 0.009823582, -0.023707598, -0.16410941, 0.09153002, 0.0229071, 0.09851835, -0.06231312, -0.05842138) * inp_2_1_0;
    result7 += M4(0.0060254824, 0.1149415, 0.27617148, -0.08997059, -0.0068580993, 0.02710906, -0.090495795, -0.069184296, -0.0026935367, 0.006274825, 0.024280496, 0.088774465, 0.0038366765, 0.0075402874, -0.11683698, -0.048249345) * inp_2_2_0;
    result7 += M4(0.028746137, -0.0012390536, -0.15352866, 0.036746267, 0.057018925, -0.012095724, -0.43297306, -0.05296296, -0.027034031, -0.06205044, -0.4788296, 0.0071023176, -0.006986947, -0.060595367, -0.5484854, -0.099282764) * inp_2_0_1;
    result7 += M4(0.0017670684, -0.23480757, -0.5568705, 0.024553766, 0.03787743, -0.07319837, -0.08334985, 0.00047246588, -0.007588862, -0.30313137, -0.48883456, -0.21123178, -0.011918053, -0.18551442, -0.23255639, -0.21069267) * inp_2_1_1;
    result7 += M4(0.012516819, -0.05016788, 0.04585929, -0.037996676, 0.011828998, -0.02394974, -0.030527798, 0.0288461, 0.0027477394, 0.0016802227, 0.02716893, -0.14519374, 0.007239097, -0.08933499, -0.16258042, 0.023957748) * inp_2_2_1;
    result7 += M4(0.008158955, -0.00818096, -0.107622735, -0.0014487421, 0.02508072, -0.008796246, -0.07216556, -0.020194868, 0.025864953, -0.03991024, -0.019141449, 0.0028773923, -0.10667418, -0.11250257, 0.05944879, -0.041024342) * inp_2_0_2;
    result7 += M4(0.015652463, -0.0006525648, -0.15620656, -0.016115017, 0.055091377, 0.05294303, 0.05863178, 0.01614067, 0.0073467665, 0.08539402, 0.013220218, -0.009380887, 0.016233368, -0.091030106, -0.3899186, -0.06322934) * inp_2_1_2;
    result7 += M4(-0.0011162924, -0.0026909818, -0.15085468, -0.0016250635, 0.009944611, -0.0323558, -0.057525035, -0.036159366, 0.0013127221, -0.048909828, -0.055695895, 0.00026873243, 0.016639616, 0.0026476237, 0.41914657, 0.025210846) * inp_2_2_2;
    const V4 inp_3_0_0 = inp[3][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_3_1_0 = inp[3][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_3_2_0 = inp[3][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_3_0_1 = inp[3][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_3_1_1 = inp[3][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_3_2_1 = inp[3][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_3_0_2 = inp[3][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_3_1_2 = inp[3][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_3_2_2 = inp[3][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.054315854, 0.030307936, 0.03766012, 0.51454693, -0.08649307, -0.0133611765, -0.10191315, -0.007581892, 0.010203365, -0.02686331, 0.0811901, -0.38367695, 0.09899108, 0.011714739, 0.021331301, 0.09277202) * inp_3_0_0;
    result0 += M4(0.051038675, -0.060006436, 0.09902917, 0.3980208, 0.08050372, -0.017675012, -0.07142045, -0.07443943, 0.00087232096, -0.015523441, -0.17490548, -0.095428295, 0.05254522, 0.0259969, 0.03082155, 0.33479217) * inp_3_1_0;
    result0 += M4(0.037476953, 0.050700758, 0.018484779, 0.26009429, -0.009441369, 0.051093034, 0.021881728, 0.15057726, 0.05127647, -0.032157443, -0.17988147, 0.021294158, 0.00917461, 0.027953213, -0.051379945, -0.015706338) * inp_3_2_0;
    result0 += M4(0.19851623, 0.039369546, -0.104972675, 0.13152802, -0.2700963, 0.05708977, -0.23159616, 0.03609799, 0.029479995, 0.0028905305, 0.028750751, 0.09786328, 0.12667042, 0.04096755, -0.0819568, 0.3491587) * inp_3_0_1;
    result0 += M4(-0.0075886445, -0.05317235, 0.17702451, -0.20325261, 0.10281223, -0.04463063, 0.015691927, -0.21990807, -0.43298295, -0.034710333, 0.035785303, 0.11157636, 0.072399065, -0.09207286, -0.064204514, -0.41392314) * inp_3_1_1;
    result0 += M4(-0.024569225, 0.17419423, 0.09997392, -0.24918704, 0.029509276, 0.046693504, 0.014606605, 0.08950168, 0.106913224, 0.13517809, -0.10032791, 0.36182383, -0.19582082, 0.05015067, 0.25517172, 0.2020313) * inp_3_2_1;
    result0 += M4(0.057505127, 0.04165924, -0.10751758, 0.4718396, -0.01684778, -0.036223996, -0.08392044, -0.5912091, 0.08977338, -0.0114714615, -0.05578842, -0.419702, -0.043833826, 0.0028689273, -0.057310544, 0.27692243) * inp_3_0_2;
    result0 += M4(0.097841024, -0.05784601, -0.009333032, 0.2722964, 0.13566212, 0.033511322, 0.1456301, 0.3773936, -0.26736465, 0.011105461, 0.16115591, -0.3290671, 0.016062884, -0.09520534, -0.09942017, 0.04089697) * inp_3_1_2;
    result0 += M4(-0.016774932, 0.055390358, 0.0013968677, 0.2768353, 0.039554663, 0.044335272, -0.063459545, -0.056899894, 0.00022852067, 0.059426866, -0.024805697, -0.0086093135, 0.004982413, 0.045921158, -0.096116975, 0.15277113) * inp_3_2_2;
    result1 += M4(-0.03234731, -0.0074114725, -0.0496608, 0.009806503, 0.07025092, -0.041442838, -0.37877434, 0.007710327, -0.12112772, 0.031340484, 0.12555282, 0.0082793245, 0.024661297, -0.0050467323, 0.17927137, -0.09630689) * inp_3_0_0;
    result1 += M4(-0.009705246, 0.008794614, -0.31637263, 0.038539067, -0.03618988, 0.004235446, -0.14130706, 0.004674193, 0.0040041795, 0.06392189, 0.08569125, -0.003123273, -0.053594768, -0.012327473, 0.41141784, 0.1269668) * inp_3_1_0;
    result1 += M4(-0.025786942, -0.029483365, -0.04689398, 0.012546755, 0.008805503, 0.015299673, 0.3252905, -0.0031295768, -0.018097406, -0.027879817, -0.31472853, -0.044150505, 0.028019449, -0.00023530817, 0.40416777, 0.022850811) * inp_3_2_0;
    result1 += M4(-0.008530995, -0.013274461, -0.017435908, 0.007650722, -0.0006207342, -0.053161036, 0.018013459, 0.016698232, -0.036166456, 0.0033362254, -0.3330711, -0.16480063, 0.013148754, 0.030061444, 0.19018202, 0.17065929) * inp_3_0_1;
    result1 += M4(-0.019577766, -0.017811859, 0.0015911807, -0.120968774, -0.022033596, 0.00647512, -0.27560246, 0.03656895, 0.030322183, 0.02558911, -0.110455796, -0.0072826934, 0.033348612, 0.027479649, -0.30512637, -0.09283864) * inp_3_1_1;
    result1 += M4(-0.019895613, 0.02708775, 0.017599838, -0.003571469, 0.0036991437, -0.009063081, 0.10931214, -0.022982173, 0.012488726, 0.0023532922, -0.121767156, 0.0028383743, 0.003121918, 0.0053249765, 0.36604604, 0.0003092426) * inp_3_2_1;
    result1 += M4(-0.0050940947, 0.0034671803, 0.035918936, -0.019917712, 0.020069117, 0.00855055, 0.071556814, 0.049229134, -0.01714347, -0.0048057013, -0.075563, 0.08770633, 0.011548991, 0.017354421, 0.03787056, 0.06321525) * inp_3_0_2;
    result1 += M4(0.004519281, -0.0073385634, -0.19771545, 0.025713885, -0.010973366, -0.0032758317, -0.015541131, -0.021452626, 0.0040585594, 0.0016111486, -0.07596171, 0.0014919937, 0.0014955263, -0.03752618, 0.1511083, -0.0013675957) * inp_3_1_2;
    result1 += M4(-0.0049793012, 0.004734952, 0.094875, 0.014785426, -0.00046361776, -0.0054128524, 0.12993924, 0.019667976, 0.011546603, -0.007246613, -0.13443129, -0.007861491, -0.011181492, -0.011804816, -0.02216903, -0.03176577) * inp_3_2_2;
    result2 += M4(-0.001579794, 0.0148131875, -0.03363462, 0.0027528529, -0.09915684, -0.08173125, -0.026745528, -0.012489439, 0.01299641, 0.025019204, -0.016889583, 0.014439402, -0.032332364, -0.047523145, -0.002975039, -0.010298583) * inp_3_0_0;
    result2 += M4(0.08648498, 0.0024772326, -0.002961114, 0.0120992325, -0.008175749, 0.010958028, -0.014249327, 0.01806545, -0.019276321, 0.043118022, 0.0018195432, -0.008640524, 0.09850765, 0.014287916, -0.008431001, 0.034403816) * inp_3_1_0;
    result2 += M4(-0.049981605, -0.06926945, -0.029227283, -0.026176844, 0.025717866, 0.021689538, -0.002244128, -0.0009383842, 0.0010884677, 0.018683989, -0.025289144, 0.018044556, 0.009815244, -0.0036331504, -0.014195599, 0.034956943) * inp_3_2_0;
    result2 += M4(-0.018169554, 0.012349744, -0.0017938939, 0.00028543288, -0.114774615, -0.080030575, -0.029269699, -0.084765516, -0.016737377, 0.0065952875, -0.011336187, 0.029181767, 0.011179501, 0.05008848, -0.028088627, -0.030179983) * inp_3_0_1;
    result2 += M4(-0.07364941, 0.050037563, 0.011461789, -0.052426573, 0.036444653, -0.0025574386, -0.03719705, 0.076142184, 0.1680529, -0.17064746, -0.015315398, -0.087448955, -0.038886376, -0.06692121, 0.02810194, -0.056523446) * inp_3_1_1;
    result2 += M4(0.01318835, 0.017288297, -0.030590994, -0.025117375, -0.051502027, -0.030646387, 0.037961893, -0.021839336, -0.030621324, 0.045812845, -0.008018343, 0.0252791, 0.036246136, -0.033890255, 0.014186754, -0.031204628) * inp_3_2_1;
    result2 += M4(0.021648671, 0.0077299224, -0.013025323, 0.007022625, 0.018361371, 0.036468726, -0.013509966, 0.015116171, 0.015725018, -0.0033855287, -0.0012584104, 0.035749402, 0.10297531, 0.005748159, 0.03227324, -0.0049986923) * inp_3_0_2;
    result2 += M4(-0.029663159, 0.008199839, 0.015688749, -0.0040096035, 0.01609513, 0.030630475, -0.0046121133, 0.030365026, -0.058981128, 0.041700244, 0.0023504125, -0.011300085, -0.09523927, -0.03847807, 0.008625168, 0.09906475) * inp_3_1_2;
    result2 += M4(0.023615377, 0.00095897884, -0.023295823, -0.0013845534, 0.017109618, 0.029298158, -0.006595015, -0.0036005354, -0.007291923, -0.018977348, 0.0046239016, 0.032799263, -0.05469631, 0.07596233, -0.0030966934, 0.07182282) * inp_3_2_2;
    result3 += M4(0.07946048, 0.017386623, -0.020704307, 0.0037951414, 0.06821082, 0.037825353, 0.002753876, -0.0027508584, 0.05489286, 0.03170449, 0.0014311017, 0.0022382613, 0.050645843, -0.006323991, -0.024367373, 0.00041726787) * inp_3_0_0;
    result3 += M4(0.0734235, -0.005793319, 0.03263896, 0.0006059661, -0.037162457, -0.002659424, -0.0011559058, -0.011139047, 0.03689328, 0.0041313674, 0.0020861567, 0.013504437, 0.05844062, -0.091387674, -0.01798884, 0.0033039649) * inp_3_1_0;
    result3 += M4(0.06952204, 0.020087602, -0.011791293, -0.007694682, -0.032339573, 0.0072449315, -0.0033902097, -0.0006066774, -0.0413044, -0.012710899, -0.003941739, -0.018049527, 0.04620126, 0.042682502, -0.010857479, -0.0016386216) * inp_3_2_0;
    result3 += M4(0.048938636, 0.021476137, -0.015230304, -0.008077273, -0.32206842, -0.21390258, 0.012124646, 0.011837484, -0.068903245, 0.016672751, -0.0026926138, -0.000522277, -0.019129515, -0.031214822, -0.042710815, 0.00047146628) * inp_3_0_1;
    result3 += M4(-0.08961494, -0.0116382055, 0.039486438, 0.0053669713, -0.010748929, 0.04789789, 0.022652034, -0.03297543, -0.18662672, -0.20604417, -0.0816969, 0.008012697, 0.051118117, 0.13223939, -0.08597153, 0.011194528) * inp_3_1_1;
    result3 += M4(0.024311319, -0.047058586, -0.023909165, 0.003065705, 0.06971258, -0.020436425, 0.0010601573, 0.009534241, -0.034757, 0.101227574, 0.009904219, 0.005138809, 0.012194527, -0.07921533, -0.056618437, -0.030282509) * inp_3_2_1;
    result3 += M4(0.0038384022, -0.015566147, -0.011294777, 0.00041754008, 0.03288913, 0.045512468, 0.02107508, 0.00015203943, -0.03082915, 0.017209167, -0.0016917093, 0.000977724, 0.03792062, -0.03170428, -0.0028735406, 0.011575303) * inp_3_0_2;
    result3 += M4(0.015297238, 0.0030779873, 0.003189614, 0.014125273, 0.025152566, -0.017973874, -0.012230016, -0.03560448, 0.0010257654, 0.009524603, -0.02741566, -0.011250525, 0.060501035, -0.034352403, 0.057744414, -0.0072469017) * inp_3_1_2;
    result3 += M4(0.027940886, -0.05296162, -0.009103395, -0.2727923, 0.060901757, -0.015219565, -0.006186121, 0.20531692, -0.07757659, -0.00011265743, 0.009102073, -0.7411601, 0.088261716, 0.07043158, 0.0229641, -1.4192594) * inp_3_2_2;
    result4 += M4(-0.025768073, 0.0109291915, 0.03518494, 0.017271267, -0.011540112, 0.002283048, 0.08092032, -0.09106908, 0.00018566349, 0.05574331, -0.023829287, 0.017426496, 0.0066620535, -0.011347661, 0.05158637, -0.0030605462) * inp_3_0_0;
    result4 += M4(0.027601559, 0.004448448, -0.060763646, -0.06059555, 0.0069441516, -0.0023665573, -0.03037829, 0.021699078, 0.0014957136, 0.030520998, 0.038930397, 0.0035747658, -0.0034995403, 0.01806036, -0.103505425, 0.0050509027) * inp_3_1_0;
    result4 += M4(-0.019419288, 0.02813772, 0.019166514, -0.0046836464, 0.0036404429, 0.026126128, -0.04730791, -0.017405445, -0.018117914, 0.013010833, -0.0056623765, 0.034219213, -0.012634234, -0.016654896, 0.0034115964, 0.0031636925) * inp_3_2_0;
    result4 += M4(-0.02304091, 0.040708303, -0.06001599, 0.067807876, -0.03517108, -0.14128582, -0.22254771, -0.066788696, 0.008410535, -0.041966, 0.011189199, -0.016583089, -0.034395956, -0.04347719, -0.13640112, -0.005699866) * inp_3_0_1;
    result4 += M4(-0.0015244156, 0.06342359, 0.069375955, -0.057013243, -0.0071155955, 0.072153166, -0.059943154, 0.076180585, 0.0050630225, -0.09646081, -0.033390466, -0.18673055, -0.035155285, -0.04400597, 0.12394051, 0.18558574) * inp_3_1_1;
    result4 += M4(-0.05826955, 0.013413446, -0.09373873, -0.04154796, 0.01844847, -0.019731466, 0.088771924, 0.029803028, -0.004551195, -0.047063332, -0.11085688, 0.12320931, -0.0064699724, 0.028511629, -0.07508922, -0.07683056) * inp_3_2_1;
    result4 += M4(-0.019322561, -0.06102507, 0.080346875, -0.017442439, -0.016763337, 0.10916651, 0.06135421, -0.006815756, 0.0007763458, -0.06708806, 0.14765729, 0.017404292, 0.02365549, 0.17264743, 0.17900267, -0.013020303) * inp_3_0_2;
    result4 += M4(0.005540262, -0.06816612, -0.14272328, -0.009777824, 0.008010294, 0.062541366, -0.009663029, -0.08483947, 0.026989376, -0.023899697, -0.23492461, 0.036529724, 0.08942257, 0.14719237, -0.13781811, 0.11972269) * inp_3_1_2;
    result4 += M4(-0.0048642172, -0.066799, 0.021364866, -0.06530534, -0.0017660721, 0.039018564, -0.009317903, 0.014913278, 0.007312249, -0.03729055, 0.055915724, 0.004504095, 0.009534455, 0.06341952, 1.5528512e-05, 0.008691904) * inp_3_2_2;
    result5 += M4(0.020605307, -0.011089028, 0.13256526, -0.052032683, -0.20384853, 0.007103205, -0.19743298, 0.03285812, 0.021069141, 0.014604715, -0.059879296, -0.012495916, -0.19991899, 0.011463516, -0.056968123, -0.0077559184) * inp_3_0_0;
    result5 += M4(0.16785757, 0.0030894137, 0.1340232, 0.13355374, 0.07952184, -0.03680629, 0.037545398, 0.08182301, -0.034028277, -0.013541879, 0.10699047, -0.071001574, -0.06399732, -0.048156444, 0.047964208, -0.012171985) * inp_3_1_0;
    result5 += M4(-0.011384558, -0.03488634, -0.056153126, -0.061684642, 0.06806214, 0.03161181, 0.03928494, -0.04330207, 0.1319544, -0.015388915, -0.07995667, 0.037057813, 0.09587005, 0.009406612, -0.031831875, 0.014962792) * inp_3_2_0;
    result5 += M4(-0.065601856, 0.010691301, 0.0532336, -0.051634245, -0.37824416, -0.12982585, -0.12397296, 0.034838766, 0.054911185, 0.017282408, 0.051395886, -0.030601839, -0.5659513, -0.0062909275, 0.17853878, -0.066808075) * inp_3_0_1;
    result5 += M4(0.15219666, -0.073666014, 0.04965218, 0.085613124, -0.22529568, 0.044459186, 0.007949877, 0.042892106, -0.21022089, -0.036888264, -0.15409057, -0.086420596, 0.23080005, 0.026083576, -0.11394975, 0.07606442) * inp_3_1_1;
    result5 += M4(0.16530591, 0.023790706, -0.011582767, -0.024630476, -0.050696746, 0.03303397, -0.014536218, -0.056243945, -0.018177863, -0.03479113, -0.004147157, 0.120171614, 0.059061665, 0.0038592604, -0.042892337, 0.009600585) * inp_3_2_1;
    result5 += M4(0.055515096, 0.050536852, -0.022230797, 0.0044950726, -0.024846574, 0.041235607, -0.08575755, 0.038505502, 0.17682916, 0.035644937, 0.01752273, 0.03058078, 0.086961895, 0.0018021693, -0.07853007, 0.079354525) * inp_3_0_2;
    result5 += M4(0.020755734, -0.022331567, -0.061544698, -0.025830325, 0.07214659, 0.034052297, 0.049643386, -0.005222395, -0.15937093, -0.07222568, -0.107371576, -0.024741232, 0.04150369, 0.067277394, -0.09839273, -0.15134043) * inp_3_1_2;
    result5 += M4(0.04807306, -0.019063137, 0.061128274, 0.0092886975, 0.18806039, -0.015561076, -0.0051985863, -0.0087440405, -0.25799108, 0.039577354, 0.079648204, 0.024675252, -0.19985919, -0.006755626, -0.026215399, 0.0520218) * inp_3_2_2;
    result6 += M4(0.30390823, 0.020501921, 0.04805823, -0.06621922, -0.015249634, -0.12887597, -0.021235434, 0.050926697, -0.16575232, -0.014748938, 0.013278694, 0.04460146, 0.0027680306, -0.08148816, 0.022364669, 0.010934176) * inp_3_0_0;
    result6 += M4(0.20406486, -0.12566334, -0.01432943, -0.05309906, -0.006593016, 0.059147976, -0.0035915107, 0.04206289, -0.015505332, 0.021929445, -0.024150278, -0.14964159, 0.26356724, -0.06025386, 0.08871716, 0.013733802) * inp_3_1_0;
    result6 += M4(-0.050205473, 0.006415771, -0.10399218, 0.10162174, 0.035028942, 0.05498369, -0.024469702, 0.023352457, 0.0009323095, 0.01259265, -0.07402342, 0.08315865, -0.1421752, -0.02268483, 0.07750779, 0.018454274) * inp_3_2_0;
    result6 += M4(0.079207346, 0.008300207, -0.013014116, -0.03851194, -0.21213347, -0.0073747407, 0.01696302, -0.09700006, 0.04467683, -0.033140313, -0.04783255, 0.028253164, 0.012350103, 0.07740636, -0.014444798, 0.0043487195) * inp_3_0_1;
    result6 += M4(0.01807809, 0.13786663, 0.052179072, 0.022258712, -0.09707588, 0.014028653, 0.0041554207, 0.15385358, -0.12721254, -0.12141812, 0.012792989, -0.13782926, 0.19964015, 0.27261713, -0.04730914, 0.05520133) * inp_3_1_1;
    result6 += M4(-0.031636246, 0.051052686, 0.013822689, 0.037879936, -0.04641027, -0.027305258, 0.05961217, 0.031242002, 0.0073914956, -0.14799239, -0.04670323, 0.15345202, 0.007073851, 0.05124267, 0.0523822, 0.014483815) * inp_3_2_1;
    result6 += M4(0.23372713, -0.00989149, -0.009435774, -0.008812121, 0.07481902, 0.017672583, 0.02246085, -0.024916472, -0.105452605, 0.03715951, -0.015104086, 0.028118443, -0.29089496, 0.06444709, -0.020945504, 0.054961424) * inp_3_0_2;
    result6 += M4(0.1442698, -0.068494104, -0.015088225, -0.009216835, 0.09896045, 0.01942005, 0.0062666344, 0.06547772, -0.21730103, -0.04185359, 0.02823365, 0.035135448, 0.08446392, 0.046115883, -0.028875928, -0.115661606) * inp_3_1_2;
    result6 += M4(0.049911387, -0.04402344, -0.0029052433, 0.047872927, 0.013824683, -0.002336361, -0.025060587, -0.009402808, 0.077805385, -0.02100996, -0.010050796, -0.0326005, 0.12395024, 0.0351807, 0.010602509, 0.057904065) * inp_3_2_2;
    result7 += M4(-0.009417451, 0.021671994, -0.09801634, 0.0454227, -0.011052633, -0.007723918, -0.13394125, 0.011875143, 0.0069561778, -0.04588778, -0.019454243, 0.0329755, 0.0027912313, -0.10344088, -0.19804947, -0.014701913) * inp_3_0_0;
    result7 += M4(-0.0127249295, -0.07682817, -0.23921761, 0.08281285, -0.008463702, -0.0384081, -0.19701773, 0.040480655, -0.019214626, -0.06561749, -0.04758473, -0.0020861803, -0.02113224, -0.053451315, 0.0028164687, 0.046243627) * inp_3_1_0;
    result7 += M4(-0.0016619975, -0.03895433, 0.05996535, 0.057698138, -0.0075781713, 0.06790655, 0.11090226, 0.019577725, -0.0072361543, -0.0033260917, -0.23619588, 0.02017983, -0.009485154, -0.124381244, -0.08479964, 0.0725922) * inp_3_2_0;
    result7 += M4(0.021064265, 0.02486322, -0.026176335, -0.024411667, 0.032535475, -0.11799067, 0.0001677916, -0.1290912, 0.01118008, -0.021246672, -0.19046235, -0.012899484, -0.021600328, 0.023459645, 0.49558398, 0.05738286) * inp_3_0_1;
    result7 += M4(0.04116767, 0.06727334, -0.13413928, -0.038683444, 0.002386932, 0.03668072, 0.103069745, 0.07041929, 0.022825709, -0.1341301, -0.30208012, -0.029685494, 0.04924293, 0.22446446, 0.3556645, 0.1369396) * inp_3_1_1;
    result7 += M4(0.010494575, 0.09701747, 0.060196687, -0.065667145, -0.001651031, -0.021386893, 0.09381029, 0.0119486535, 0.0015527179, -0.08416465, -0.32502642, -0.06578076, -0.003299509, 0.1752645, 0.446606, 0.0799174) * inp_3_2_1;
    result7 += M4(0.009749115, -0.037539054, -0.07856848, -0.013709623, 0.026956793, 0.0791147, 0.35218245, 0.10874431, -0.031560194, 0.020191723, 0.122894645, -0.03297223, 0.052877117, 0.0998178, 0.3810457, 0.015579134) * inp_3_0_2;
    result7 += M4(0.0058319815, -0.014323539, -0.22228782, -0.011999186, 0.0027822794, -0.028720282, 0.11303368, 0.018080695, -0.0032609988, 0.04609295, 0.076308176, 0.034869004, -0.0050333026, 0.004994906, -0.09829331, -0.061349858) * inp_3_1_2;
    result7 += M4(-0.0013132154, -0.009380491, 0.1529272, -0.0010732947, 0.003404691, 0.04588256, 0.11748522, 0.05178305, -0.015224876, 0.047998644, -0.21839756, 0.051946025, 0.0029602963, 0.06764577, 0.35156026, -0.004381589) * inp_3_2_2;
    const V4 inp_4_0_0 = inp[4][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_4_1_0 = inp[4][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_4_2_0 = inp[4][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_4_0_1 = inp[4][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_4_1_1 = inp[4][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_4_2_1 = inp[4][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_4_0_2 = inp[4][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_4_1_2 = inp[4][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_4_2_2 = inp[4][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.036380295, -0.0034899914, -0.08783686, -0.1418619, 0.0047959606, -0.029464077, -0.21090825, 0.17575453, -0.003872291, -0.009554285, -0.07212815, -0.4766315, 0.052686576, 0.0038521395, -0.14210181, 0.21342552) * inp_4_0_0;
    result0 += M4(0.05423329, 0.021760859, 0.20986934, -0.09753783, 0.14492704, -0.006632365, 0.15831396, -0.587669, 0.23957707, 0.012349426, 0.028890606, 0.31633446, -0.060501643, -0.035392787, 0.012135381, -0.7305586) * inp_4_1_0;
    result0 += M4(-0.020588122, 0.010246647, -0.05666967, -0.26177633, 0.036033466, 0.051386308, -0.050308403, 0.27404383, -0.033459894, -0.010099392, -0.039405886, 0.24626529, -0.031674724, 0.01674998, -0.20484154, -0.13994777) * inp_4_2_0;
    result0 += M4(0.13704468, 0.017250488, -0.16773464, 0.2650823, -0.09485869, -0.015353574, 0.12993343, -0.34779328, -0.17394379, -0.046820402, 0.113700025, -0.12736009, 0.070801504, -0.020245776, -0.24483787, -0.057481434) * inp_4_0_1;
    result0 += M4(-0.31520092, -0.01001837, -0.3534247, 0.0053825574, -0.13529882, 0.08884386, -0.098817736, -0.35936865, 0.20105608, 0.06124993, -0.17391597, 0.25924078, -0.1773179, -0.10844964, -0.15071106, -0.029405816) * inp_4_1_1;
    result0 += M4(-0.064969845, -0.10883879, 0.07054897, -0.44149357, 0.21642914, 0.023797732, -0.030483752, 0.32969272, 0.074324846, -0.07123357, -0.1703094, 0.20394105, -0.047158547, 0.05149358, -0.34160092, -0.18457998) * inp_4_2_1;
    result0 += M4(-0.06450896, -0.019507032, -0.18723527, 0.20223755, -0.036437996, -0.041134693, 0.122630335, -0.029747333, -0.023719335, -0.03066493, 0.06142014, -0.13640186, 0.019143494, -0.011022388, -0.25649232, 0.2742824) * inp_4_0_2;
    result0 += M4(-0.088832475, 0.019458568, -0.29711705, -0.6919155, 0.0382599, 0.058981758, 0.140561, -0.24696992, -0.030074544, 0.0006669616, -0.06430839, 0.2873934, -0.18383065, 0.013073896, 0.018682862, -0.75154424) * inp_4_1_2;
    result0 += M4(-0.045830917, -0.028971015, 0.007392609, -0.3767271, -0.0019745969, 0.03832943, 0.11034421, 0.17107904, 0.033657115, -0.0006176393, 0.034761317, 0.11794339, 0.053871017, -0.00049891206, -0.25796595, 0.32575172) * inp_4_2_2;
    result1 += M4(-0.079997934, -0.041962232, -0.6178472, 0.040897116, -0.102274895, 0.05048019, 0.47783756, 0.07829887, 0.20411032, -0.019218592, 0.15957001, -0.12055146, -0.056412466, -0.023705479, 0.24970028, 0.011522375) * inp_4_0_0;
    result1 += M4(0.01164706, 0.09915082, -0.62197185, 0.026376652, 0.021334108, -0.03816917, -0.024031848, 0.015756765, 0.018469589, -0.018596342, -0.10139853, -0.022334058, -0.028180564, -0.0894664, 0.08719273, -0.062018465) * inp_4_1_0;
    result1 += M4(-0.003640731, 0.009667221, -0.4617525, -0.00072852423, -0.039242484, -0.019235488, -0.43389535, -0.032469116, 0.0041179326, 0.006157765, -0.07833568, -0.011815631, -0.021701932, -0.09797507, 0.5109372, 0.025736123) * inp_4_2_0;
    result1 += M4(0.050691936, 0.013189936, -0.78655976, 0.056514192, -0.001169361, -0.004534778, 0.007551947, -0.4933174, 0.011223421, 0.023503872, -0.16939536, 0.19286062, -0.006769029, -0.06962875, -0.19993277, -0.29189786) * inp_4_0_1;
    result1 += M4(0.010967908, -0.04121953, -0.72942454, -0.029976387, 0.0617382, 0.03050032, 0.050672896, 0.026078302, -0.0150793055, -0.00622913, -0.070017405, -0.044685874, 0.014144849, -0.04538613, 0.20526175, -0.066887155) * inp_4_1_1;
    result1 += M4(0.0036923348, 0.046220236, -0.0719901, 0.018438688, -0.004686628, -0.013794622, -0.21791345, 0.02296315, -0.0069882157, -0.025698029, -0.1717542, -0.003972355, 0.001316215, -0.04785788, -0.5879735, -0.02433646) * inp_4_2_1;
    result1 += M4(-0.027738651, 0.005873504, 0.09753426, -0.020808343, -0.0042312946, -0.028074563, 0.18245865, 0.10053258, 0.012274734, 0.002811127, 0.10424476, -0.067287795, -0.004674516, 0.011887984, -0.024741331, -0.1715041) * inp_4_0_2;
    result1 += M4(0.008161968, 0.004016125, -0.03478462, -0.0028829754, -0.0079947915, 0.013601364, 0.22718312, 0.06975822, -0.0071728355, 0.0019671163, -0.117276095, 0.03191032, -0.024408855, -0.014883164, -0.015909586, -0.091252096) * inp_4_1_2;
    result1 += M4(-0.0066564996, -0.012371795, 0.022292925, 0.0054139346, -0.010402067, 0.006611986, 0.005566904, 0.008166976, -0.016824462, 0.009165492, 0.12503548, 7.873607e-05, -0.017900331, 0.035930842, 0.26372743, 0.050605286) * inp_4_2_2;
    result2 += M4(0.061530586, 0.006215644, 0.01760483, -0.008382748, -0.11067897, 0.016527515, 0.005712524, -0.013279194, -0.04375924, -0.032088146, 0.0052938983, -0.006084397, -0.0058650137, 0.024149662, -0.0013330342, -0.03899846) * inp_4_0_0;
    result2 += M4(0.08288145, -0.016888432, -0.0066983914, -0.008992347, 0.1325464, -0.015978495, 0.025865, 0.029661374, -0.005870199, 0.028620312, -0.009073212, 0.03694535, -0.076142706, -0.01939176, 0.0147289345, 0.0015503741) * inp_4_1_0;
    result2 += M4(0.011555271, -0.037613265, -0.007213205, 0.0057167457, -0.028772166, -0.06368081, -0.0063705677, -0.021366661, 0.0297308, -0.016066711, -0.00075569685, -0.006092977, -0.020086007, 0.014356459, -0.009013368, -0.0026702115) * inp_4_2_0;
    result2 += M4(-0.0073330235, -0.018543018, 0.004102061, -0.02518465, 0.10907779, 0.018959653, 0.039944872, 0.0053424877, 0.071293265, -0.014934514, 0.042280175, -0.025728572, -0.09168962, -0.016956018, -0.020566572, -0.013077225) * inp_4_0_1;
    result2 += M4(-0.1268247, -0.023160262, -0.0994719, -0.026749771, 0.01181737, 0.074071914, -0.13385102, -0.108061664, -0.01004953, -0.027268277, 0.056937773, 0.09080762, 0.10715432, 0.011247546, 0.01427642, 0.034878027) * inp_4_1_1;
    result2 += M4(0.059429042, -0.03792002, 0.025036033, -0.02201924, -0.024667246, 0.017447127, -0.0362713, 0.056404535, -0.038722705, -0.045914847, 0.002775175, 0.048381418, -0.12658995, -0.032172047, -0.015955163, 0.0119411545) * inp_4_2_1;
    result2 += M4(0.026508208, 0.001993063, 0.029188927, -0.04191957, -0.046275817, 0.0024459453, -0.004736741, -0.03473948, -0.030642346, -0.037960086, -0.011557733, -0.036196366, 0.021020604, -0.00042413248, -0.002251711, 0.0012698546) * inp_4_0_2;
    result2 += M4(0.050738182, -0.028680146, 0.013808977, -0.042407542, 0.068234555, 0.04367899, 0.013855036, -0.03867529, 0.042987697, -0.06921649, -0.0054449253, 0.03196437, -0.11016987, 0.014720304, -0.0035736728, -0.014884298) * inp_4_1_2;
    result2 += M4(-0.05553661, 0.0013156066, 0.01658008, 3.8219164e-06, 0.061579835, -0.05478697, 0.010209602, -0.0068524345, 0.022187822, 0.007326568, -0.0016615703, 0.0029105565, 0.047776476, 0.023593139, 0.009077922, -0.0068046055) * inp_4_2_2;
    result3 += M4(-0.021636033, -0.009681403, -0.006674432, -0.0016117063, -0.053953107, 0.0426434, -0.005730214, 0.002795855, -0.009980966, 0.030116973, 0.007441833, 0.0012294573, -0.073019, -0.026495036, 0.00020881441, -0.009062286) * inp_4_0_0;
    result3 += M4(-0.09005293, -0.007905638, 0.013396831, 0.00023445922, 0.30235958, -0.014852752, 0.028534386, -0.00651374, 0.010369283, -0.106462255, 3.8504742e-05, -0.0019981728, -0.04336175, 0.06313804, -0.00021365794, -0.006549755) * inp_4_1_0;
    result3 += M4(-0.00035998155, 0.050012913, 0.000106709, 0.007499425, 0.089299664, -0.04745237, 0.00938978, 0.004079116, -0.018496398, -0.039705347, -0.0084362235, 0.010732207, 0.05222794, 0.0015888687, 0.0044623283, 0.003847604) * inp_4_2_0;
    result3 += M4(0.025514586, 0.085099496, -0.005522202, 0.015304287, 0.11402071, 0.022388445, 0.014685565, -0.0009200893, -0.09551002, -0.122550406, 0.01136277, -0.007015497, 0.16417067, 0.013585167, -0.0054550814, -0.003173147) * inp_4_0_1;
    result3 += M4(-0.27088094, 0.019462781, 0.01949606, -0.009879548, -0.21403842, 0.0983023, -0.10664192, 0.006108758, -0.12319542, 0.20075582, -0.12505932, -0.0061204676, 0.17988583, -0.08949372, 0.0019347003, 0.0041134497) * inp_4_1_1;
    result3 += M4(-0.103450194, -0.036010057, -0.011630207, -0.016884698, -0.02788478, 0.08499072, 0.043950897, 0.011412022, 0.12391076, -0.013858037, 0.01584802, -0.01933635, 0.07312944, -0.0612639, 0.009622239, -0.004510867) * inp_4_2_1;
    result3 += M4(-0.09374226, -0.037200183, 0.014928372, 0.0010188735, 0.069776006, -0.020685896, 0.008251117, 0.014829769, 0.055178724, 0.026227608, -0.0094766505, -0.008085552, 0.066762924, 0.07407996, -0.00021714815, -0.005302891) * inp_4_0_2;
    result3 += M4(-0.0075875856, 0.019485703, -0.0041497704, -0.009982057, -0.018280974, 0.018580865, 0.030623518, -0.009212654, 0.08892845, -0.035797704, -0.02074808, -0.025541985, -0.019148272, -0.25871935, 0.0029911404, 0.002325437) * inp_4_1_2;
    result3 += M4(0.06384416, 0.040572796, -0.0022575783, -1.1420674, -0.046429962, 0.019336062, 0.019280067, -1.6516366, -0.014767885, 0.005557631, 0.011545762, -0.7297257, -0.07804735, 0.00851267, 0.012658107, -0.52300566) * inp_4_2_2;
    result4 += M4(0.0032693911, 0.03274794, 0.043356106, 0.03913015, 0.009155537, 0.021447245, -0.02437202, 0.01881453, 0.013823758, 0.004457205, -0.08053447, -0.027254682, -0.034055322, -0.015656909, 0.057489578, -0.08153641) * inp_4_0_0;
    result4 += M4(-0.0013997308, 0.03155598, -0.083776474, 0.030604392, 0.032877967, 0.054687187, -0.039342374, 0.12196676, 0.029505735, 0.014930254, -0.018423868, -0.007125555, -0.008708337, 0.0123238815, -0.026203495, 0.011568379) * inp_4_1_0;
    result4 += M4(0.004438667, -0.0273087, 0.06523531, 0.049134377, -0.0053809723, -0.014375149, 0.077725634, -0.01903995, 0.033829797, -0.021593707, -0.024035936, 0.002139325, 0.000434621, 0.018608324, 0.024417559, -0.03956627) * inp_4_2_0;
    result4 += M4(0.02982234, -0.047758352, -0.10001253, 0.017921215, 0.10331017, 0.15461753, 0.09976858, 0.09928215, 0.033321377, -0.079562284, 0.04957612, -0.10442731, -0.015094084, 0.03332427, 0.0917733, 0.029832482) * inp_4_0_1;
    result4 += M4(-0.08964243, -0.13682616, -0.058703046, -0.031467825, -0.10960406, 0.04929474, -0.18759963, -0.3016533, 0.046261407, 0.027672693, -0.1313958, 0.13757555, 0.019622695, 0.05510076, -0.029191654, -0.09680346) * inp_4_1_1;
    result4 += M4(0.003161483, 0.022965934, 0.026541866, -0.05923413, -0.030192977, -0.00088497053, -0.011561964, 0.044605374, 0.012787321, -0.039626013, -0.18094091, 0.010091419, -0.023626665, 0.008861475, 0.02381804, 0.0061293743) * inp_4_2_1;
    result4 += M4(0.0004986802, -0.07351269, 0.07791931, -0.032903977, -0.014621468, -0.100439645, -0.05798121, 0.02950674, -0.0017096291, 0.08361609, -0.10732986, -0.021404203, -0.016725698, 0.01684794, -0.010681881, -0.0039078337) * inp_4_0_2;
    result4 += M4(-0.038165677, -0.0695999, -0.025167728, 0.04300625, -0.037260555, -0.007561456, 0.2085303, 0.021647256, 0.00999071, 0.07502709, 0.046473406, -0.043189447, -0.022536399, 0.08448336, -0.030263713, -0.13834356) * inp_4_1_2;
    result4 += M4(0.011547609, -0.028540198, -0.049025487, 0.052106164, -0.020389317, 0.0059719807, -0.06330791, -0.044758774, -0.013596575, 0.055375148, -0.05921033, -0.005469655, -0.018135134, 0.05128665, -0.02556082, -0.06292623) * inp_4_2_2;
    result5 += M4(0.23616323, -0.05031422, -0.0034876599, 0.031145947, 0.24818936, 0.0302281, -0.0247939, -0.004192674, 0.20751902, 0.024411973, -0.14260906, 0.101380914, -0.050158843, 0.022123333, 0.03663624, -0.034214657) * inp_4_0_0;
    result5 += M4(-0.23831972, -0.0064858613, -0.028504893, -0.051656518, 0.19924329, 0.061141122, -0.14646927, 0.08286938, -0.15846002, -0.08491522, -0.035381425, -0.075538106, 0.19719632, 0.034685064, 0.058576964, -0.025867514) * inp_4_1_0;
    result5 += M4(-0.17995597, -0.029906508, 0.08917914, -0.034881044, 0.13453874, 0.045598377, 0.00850196, 0.119027644, -0.0012034088, -0.059920996, -0.00320406, 0.033835083, 0.036897834, -0.0118491575, 0.020927414, -0.011648264) * inp_4_2_0;
    result5 += M4(0.030492883, 0.07476317, 0.16089576, -0.038885623, 0.33707803, 0.044633422, -0.09156085, 0.06213759, -0.012303359, -0.09512382, -0.087484285, 0.09594416, 0.124775276, 0.027646946, 0.021840394, -0.04654952) * inp_4_0_1;
    result5 += M4(-0.59143, -0.093542755, -0.15266271, -0.06279261, -0.2434347, -0.544943, -0.1684478, 0.06422059, -0.11909034, 0.07991743, 0.31965742, -0.07738673, -0.038964704, 0.019431371, 0.0070903976, 0.019722188) * inp_4_1_1;
    result5 += M4(0.0759864, 0.0008528529, -0.10885294, -0.010254866, -0.06343555, -0.09002659, 0.084906675, 0.11957859, -0.08601708, -0.052642867, -0.04988037, -0.007970746, 0.09135083, -0.03651564, -0.034575164, 0.09430225) * inp_4_2_1;
    result5 += M4(-0.066693366, -0.031476438, -0.0046523237, -0.052905124, -0.18217368, 0.010578686, 0.17473231, -0.059775762, -0.1822715, -0.013425451, 0.03636841, -0.053300973, 0.22889522, 0.026266804, -0.055748712, 0.023054829) * inp_4_0_2;
    result5 += M4(-0.09023598, 0.0231071, -0.0005561984, -0.017104875, -0.26399654, 0.03757102, 0.015643217, 0.06204881, 0.1805975, -0.0586794, 0.0025237498, -0.000594014, 0.13225378, -0.09107814, -0.009286263, 0.049993202) * inp_4_1_2;
    result5 += M4(0.22398123, 0.009719407, -0.029209206, 0.004648987, 0.0046009403, -0.024154155, 0.15385835, 0.014440248, 0.030251589, -0.016884329, -0.07979834, -0.030168258, -0.23176348, 0.026309827, 0.028870646, 0.043285176) * inp_4_2_2;
    result6 += M4(-0.43379298, 0.030038001, -0.07767635, 0.0074480423, -0.27195957, 0.06984191, -0.0063045095, 0.037740696, -0.26387292, -0.026242903, 0.003637742, 0.098332785, 0.016995898, 0.080835186, -0.04031453, -0.017095767) * inp_4_0_0;
    result6 += M4(-0.004246825, 0.0028224557, -0.116326354, 0.016331246, -0.05774728, 0.03237647, -0.0001721832, 0.033778936, -0.23125762, -0.0050692298, 0.09953401, -0.0044605536, -0.14916427, 0.019176641, -0.064965084, -0.027513957) * inp_4_1_0;
    result6 += M4(-0.049572367, 0.012860502, -0.010751411, -0.01415806, 0.20739669, 0.086128786, -0.1923856, -0.051838044, 0.08634109, 0.00907178, 0.11935713, 0.073800944, 0.040289775, -0.028278619, -0.076562904, -0.022440363) * inp_4_2_0;
    result6 += M4(-0.24107365, -0.07937353, 0.020785958, 0.0031801465, -0.1357309, 0.08627046, -0.034196924, 0.05194556, -0.44237745, 0.102913156, 0.027449023, 0.0757706, -0.1565952, 0.0513024, -0.052920267, -0.014027862) * inp_4_0_1;
    result6 += M4(0.076569565, -0.20088851, -0.13823849, -0.07676857, 0.16245067, -0.2523675, -0.0007205186, -0.0054110554, -0.31927586, 0.13458678, 0.023989735, -0.022193564, 0.14599766, -0.049036127, -0.072871715, 0.04174214) * inp_4_1_1;
    result6 += M4(0.054940574, 0.019696543, 0.055747025, -0.07536452, 0.050803326, -0.10412333, 0.016412135, 0.019654129, -0.03952146, -0.003151653, -0.048484992, 0.035699043, -0.030760778, 0.056488615, -0.2607573, 0.034216944) * inp_4_2_1;
    result6 += M4(-0.5171205, -0.025004026, 0.014893224, 0.008683979, -0.211134, -0.02383933, 0.019062772, -0.036130987, 0.065986134, -0.067757845, 0.01164191, -0.011873978, -0.04913593, -0.002083849, -0.011018765, 0.03226251) * inp_4_0_2;
    result6 += M4(0.1668885, -0.033413794, 0.039875533, -0.02514423, 0.16652322, 0.070412405, 0.053880546, -0.017836487, -0.13058256, -0.019620018, -0.024715304, -0.022343278, -0.998977, -0.00815804, -0.010345262, 0.009525035) * inp_4_1_2;
    result6 += M4(-0.08499069, 0.01141396, -0.016826574, -0.02168003, 0.123839505, 0.049729776, 0.073240355, 0.010634464, 0.0058316356, 0.03231472, 0.011353218, -0.014293186, 0.092393614, 0.086187586, 0.06466312, 0.06291995) * inp_4_2_2;
    result7 += M4(0.0011556491, 0.037384823, -0.22883376, -0.0013126024, -0.04062926, 0.032604747, 0.24972658, 0.02756332, 0.010626175, -0.00051818526, 0.10343982, 0.017419543, 0.0050601745, 0.05675354, 0.29973325, 0.04578761) * inp_4_0_0;
    result7 += M4(-0.009191253, 0.01175041, -0.026460709, -0.043895178, -0.036810428, -0.06187302, -0.05270285, 0.03862491, -0.012923763, -0.0869517, -0.067232326, 0.093995005, 0.0034143087, 0.046554122, 0.0053896885, 0.008173666) * inp_4_1_0;
    result7 += M4(-0.011060513, -0.06870076, -0.098920345, -0.004346952, -0.0034685412, -0.043995377, -0.25803337, -0.0776707, -0.011671287, -0.025558814, -0.13480005, 0.017878532, -0.001327167, 0.011246587, 0.06386739, -0.032328114) * inp_4_2_0;
    result7 += M4(0.011483899, -0.025194615, -0.45176688, -0.08233033, -0.032523073, 0.088833064, -0.096951544, 0.0051010125, 0.0329795, 0.0050644428, 0.13591573, 0.06330439, -0.005869377, -0.024528576, -0.4965906, -0.035148658) * inp_4_0_1;
    result7 += M4(-0.019560773, -0.19995795, -0.6187494, -0.20867999, 0.010528096, -0.13933884, -0.13964583, -0.042185064, -0.03924202, 0.22711454, 0.10020683, 0.012650224, -0.00037646928, -0.08956661, 0.2612014, 0.0072628283) * inp_4_1_1;
    result7 += M4(0.0041069337, 0.06255956, -0.2616958, -0.018234765, -0.022010244, -0.11517897, -0.36681372, -0.031433675, -0.028744362, 0.058785617, 0.2263271, 0.05259863, 0.002351623, -0.09980612, -0.16653296, 0.057118) * inp_4_2_1;
    result7 += M4(-0.013570208, 0.10269598, -0.15851836, 0.036451314, 0.012149419, -0.0026949772, 0.1023906, 0.0062117297, 0.027653148, -0.031886455, -0.05417527, 0.008629089, -0.015849534, 0.011522947, 0.09813515, 0.004191201) * inp_4_0_2;
    result7 += M4(-0.0016982512, -0.04525864, -0.42326707, 0.051875032, -0.0072344705, 0.11756441, 0.4213877, 0.06814944, 0.006257594, -0.01687695, 0.1580408, -0.035481073, 0.008777881, 0.004880249, -0.13231875, 0.06587732) * inp_4_1_2;
    result7 += M4(0.007775645, 0.049827214, 0.10467628, 0.014413747, -0.02502015, 0.09493599, 0.14816707, 0.03106107, -0.00011648729, 0.00594749, 0.06330615, -0.03440764, -0.011787949, 0.112478316, 0.34382775, 0.08111796) * inp_4_2_2;
    const V4 inp_5_0_0 = inp[5][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_5_1_0 = inp[5][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_5_2_0 = inp[5][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_5_0_1 = inp[5][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_5_1_1 = inp[5][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_5_2_1 = inp[5][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_5_0_2 = inp[5][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_5_1_2 = inp[5][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_5_2_2 = inp[5][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.08333967, 0.011421889, 0.11664162, 0.39396796, 0.021902101, 0.0003499147, 0.07612623, 0.12175804, -0.058718447, 0.012608792, -0.060260076, 0.016490625, 0.16412248, 0.050016712, 0.025333768, 0.018155728) * inp_5_0_0;
    result0 += M4(-0.29617354, 0.007589052, -0.27730608, -1.4229045, -0.1160415, 0.0021424845, 0.08839397, -0.7607373, 0.1249401, 0.060877696, 0.074407615, -0.33305424, -0.2863919, -0.06357322, -0.03547036, -0.6676305) * inp_5_1_0;
    result0 += M4(-0.04900496, 0.037379008, 0.15788396, -0.33005843, 0.091360494, -0.04061889, -0.076422624, 0.28810555, 0.054824818, 0.083238, 0.1918283, 0.38165346, 0.05597679, 0.049651083, 0.044038914, -0.056280203) * inp_5_2_0;
    result0 += M4(-0.06427907, 0.0070991376, 0.088748135, -0.2157318, -0.014503172, -0.027313257, 0.12798136, -0.22733903, -0.07273969, -0.010391546, -0.055078164, -0.14918047, 0.1478902, 0.01873115, -0.05077475, 0.008662993) * inp_5_0_1;
    result0 += M4(-0.03695306, 0.03239904, -0.30017543, -0.065224655, -0.12987286, -0.04310372, -0.17464949, 0.11849645, 0.17904112, -0.011687923, -0.30345693, 0.13255681, -0.2909277, 0.008057539, -0.41463858, -0.7889213) * inp_5_1_1;
    result0 += M4(0.023014775, -0.050038394, 0.014925209, 0.04931278, 0.021920985, 0.06511956, 0.0002137051, -0.109317794, 0.092462674, -0.15559328, 0.13214372, -0.25005814, -0.036398772, 0.0039769504, 0.04089996, -0.638778) * inp_5_2_1;
    result0 += M4(-0.16340446, -0.057307, 0.48035783, -0.44656652, 0.146567, 0.034004353, -0.046336066, 0.23182853, -0.18350513, -0.02504686, 0.061231308, 0.037447155, 0.0938486, -0.0031760277, 0.019381015, -0.34471622) * inp_5_0_2;
    result0 += M4(0.26452816, 0.08390242, -0.028910087, 0.80734545, 0.017617049, -0.14048198, 0.006482139, -0.5437967, 0.04382175, 0.035703473, -0.013352236, -0.024485594, -0.1863618, 0.026656361, -0.03918774, -0.937619) * inp_5_1_2;
    result0 += M4(-0.020624971, 0.05528504, 0.029032538, 0.5145849, -0.017594978, 0.07719026, 0.08997274, 0.06560444, 0.000285615, -0.08993183, -0.13740857, 0.6568468, -0.07033215, -0.1524613, 0.1623048, -0.30009347) * inp_5_2_2;
    result1 += M4(-0.024649087, 0.027187567, -0.12507068, 0.067237385, -0.012682525, 0.037245147, -0.89697677, -0.017100656, 0.03322117, -0.03888771, 0.47391358, -0.030272357, -0.120315865, -0.028607864, -0.75294685, -0.07918916) * inp_5_0_0;
    result1 += M4(0.05536591, -0.111655064, 0.01577062, -0.035698608, 0.0044509233, -0.03347181, 0.4066368, 0.025608776, 0.029298296, -0.07960766, 0.42216155, -0.07395237, 0.018788522, -0.15885046, -1.3155875, -0.0021334256) * inp_5_1_0;
    result1 += M4(-0.006209056, -0.030117406, -0.40922484, 0.0044417926, -0.014199001, 0.022765342, -0.6484923, -0.014981119, 0.030789856, 0.08541737, 0.61759603, 0.019239806, 0.004689295, 0.009242462, -0.8025019, 0.013805084) * inp_5_2_0;
    result1 += M4(0.10752561, 0.0051500406, 0.29258895, -0.08505117, -0.007469432, 0.02999793, 0.61382407, 0.018623577, -0.0062840465, 0.01889839, 0.1362163, 0.067568235, -0.042511, -0.03480789, -0.04170731, -0.3034407) * inp_5_0_1;
    result1 += M4(0.024882527, -0.07457014, -0.08897756, 0.08734714, -0.0023208018, -0.013685176, 0.049045235, 0.0010875851, 0.0417957, -0.033301357, 0.12046458, 0.064124145, -0.059232328, 0.032327723, -0.08452184, -0.015425462) * inp_5_1_1;
    result1 += M4(-0.007871425, -0.04764698, -0.34593108, -0.011978634, 0.01949231, 0.0026820523, -0.0033999975, 0.036353253, 0.02958873, -0.007205353, -0.228622, -0.075859584, -0.0045728525, 0.027070034, 0.07013869, 0.007040719) * inp_5_2_1;
    result1 += M4(-0.022180416, 0.0016016312, 0.042087387, -0.0023142442, 0.015311656, -0.00065957714, -0.19917949, 0.026721543, 0.0063898503, -0.0024339044, -0.082008325, -0.08832473, 0.0069390205, 0.013572554, 0.046471115, -0.0014028423) * inp_5_0_2;
    result1 += M4(-0.01453005, -0.005529587, 0.04961345, -0.046984795, -0.016217869, -0.022817105, -0.056734905, -0.008015891, 0.020701988, 0.026441883, -0.25874138, -0.050856728, 0.019932656, -0.0118779745, 0.08876789, 0.05818641) * inp_5_1_2;
    result1 += M4(-0.017473832, 0.0017917561, -0.0050135315, 0.01028943, -0.006864128, 0.01445976, -0.0691626, -0.017201284, 0.0007423925, -0.0090062395, -0.1951909, 0.021151524, -0.0017929715, -0.003341484, 0.12193786, 0.027570732) * inp_5_2_2;
    result2 += M4(-0.01337703, -0.040191807, 0.016983964, 0.00097402657, -0.010873824, -0.009673816, -0.0065111453, -0.015262973, -0.01949314, -0.024734965, 0.032043748, -0.035334155, 0.027301786, 0.012465609, -0.006810791, -0.036217056) * inp_5_0_0;
    result2 += M4(0.005665324, 0.02489004, 0.0008669482, -0.01521067, -0.0023465417, 0.013876046, -0.018561749, -0.022934288, 0.03513589, -0.09592462, 0.02967652, -0.026250472, -0.076485455, 0.030804513, 0.021152128, 0.019765839) * inp_5_1_0;
    result2 += M4(-0.07205131, -0.0045729135, 0.011745689, 0.0018625689, -0.02274289, 0.05560007, -0.011854123, 0.02380985, 0.011130511, -0.021083219, 0.03290216, -0.015718175, 0.0142448135, -0.011268402, -0.01885753, 0.009982545) * inp_5_2_0;
    result2 += M4(0.06497665, -0.03245727, 0.06291978, 0.027389538, 0.08801292, 0.025413414, -0.0049673314, 0.0031450852, 0.038713794, -0.03702989, 0.023669632, -0.09296012, -0.05495926, 0.02417087, 0.010663983, 0.07307905) * inp_5_0_1;
    result2 += M4(-0.0857886, 0.015923984, 0.01212979, -0.12200662, -0.056006826, -0.07609907, 0.029490868, -0.06399761, -0.18119709, 0.13169484, -0.11460586, 0.023339845, -0.021311989, -0.1252562, -0.03875252, -0.07764663) * inp_5_1_1;
    result2 += M4(0.0309262, 0.003504655, 0.007847503, -0.005167781, 0.05068589, 0.010624247, -0.018557666, 0.0008189457, -0.015387881, -0.14612661, 0.26160657, -0.06558364, 0.0025307238, 0.024071123, 0.016721481, 0.015320905) * inp_5_2_1;
    result2 += M4(0.009053564, 0.055623777, 0.027467286, -0.09038976, -0.0071879257, -0.09548455, -0.052858785, 0.013600631, -0.025725564, -0.0010903189, 0.025795193, -0.011152754, -0.030557098, 0.033029728, 0.0148722045, -0.017060269) * inp_5_0_2;
    result2 += M4(-0.00971069, 0.023631368, 0.09173257, -0.008764473, -0.06693117, 0.044271823, 0.0031808384, -0.007179988, 0.065571815, -0.0777225, -0.053624026, -0.036817104, 0.019093744, 0.008928722, -0.011828543, 0.014236826) * inp_5_1_2;
    result2 += M4(-0.015805334, 0.0050145057, 0.013463592, -0.006999735, 0.031993672, 0.028100364, -0.01396432, -0.022730565, -0.027783317, 0.025410727, 0.09667642, -0.045832474, 0.01703691, -0.0017765082, -0.001803319, -0.005822149) * inp_5_2_2;
    result3 += M4(0.054247994, -0.014589579, -0.017078482, 0.003753213, -0.06160483, -0.01619493, 0.00439086, 0.0012733352, -0.14091335, -0.034655374, 0.0044431565, -0.002013041, -0.13968565, -0.008917815, -0.010853735, 0.0015223346) * inp_5_0_0;
    result3 += M4(0.04705881, 0.08811856, -0.014349928, 0.00587463, 0.09772441, 0.073334225, 0.009045951, 0.003626761, -0.22171451, -0.14361921, -0.045902, -0.004412931, 0.01698341, 0.019291017, -0.0021865598, -0.0034016862) * inp_5_1_0;
    result3 += M4(0.024689116, -0.011561938, -0.0057464736, -0.005806331, 0.114804104, -0.0012921327, 0.0078091356, 0.014535022, -0.09828248, 0.007561789, 0.026903786, -0.001342464, 0.00020966076, 0.015795738, -0.0012305793, 0.0027894592) * inp_5_2_0;
    result3 += M4(0.055408955, 0.015059497, 0.01217424, 0.00042246946, -0.022732092, -0.08195749, -0.02029763, -0.00087688636, 0.09105202, -0.04055965, 0.0038086378, 0.0064821625, 0.16855904, 0.007609558, 0.018436445, 0.00036847353) * inp_5_0_1;
    result3 += M4(-0.19119112, -0.038557135, -0.0515908, 0.010205609, -0.21675286, 0.014943928, -0.013617628, 0.0049909865, 0.26118392, 0.20322427, -0.045269843, 0.06906791, -0.26446897, -0.08771047, 0.046184067, 0.010451827) * inp_5_1_1;
    result3 += M4(-0.043820024, -0.017206183, 0.01545375, 0.011508815, -0.04314388, 0.012077893, 0.014275914, -0.025720278, 0.18515207, -0.090922624, -0.058769234, 0.20169196, -0.09246609, -0.046891548, 0.012638604, -0.015772158) * inp_5_2_1;
    result3 += M4(-0.031128516, -0.006867225, 0.031071624, -0.009757052, 0.14832574, 0.03748604, 0.0022000212, -0.0041525587, -0.0068835923, -0.03746232, 0.012551968, -0.01065121, -0.011746205, 0.027581658, 0.017809473, -0.010158842) * inp_5_0_2;
    result3 += M4(-0.1437907, 0.072315365, 0.09837226, -0.059529234, -0.01982794, 0.02473044, -0.011935614, 0.01583299, -0.130663, -0.050234724, -0.008006365, -0.05318172, -0.040717535, -0.04363175, 0.020041062, -0.020595768) * inp_5_1_2;
    result3 += M4(-0.10698102, 0.024461597, 0.020094154, -0.2514701, 0.013265725, -0.048385404, -0.00027174546, -1.09914, 0.0217536, -0.054307077, -0.005172583, 0.24954069, 0.14232014, 0.025980933, -0.010516244, -1.3011694) * inp_5_2_2;
    result4 += M4(0.012730154, -0.0003357301, -0.026912017, 0.017250964, -0.0014002038, -0.006169688, -0.01846868, -0.0073398002, 0.003476205, -0.106256284, -0.003861976, -0.053039715, -0.044129584, -0.025571141, -0.0332234, 0.042310335) * inp_5_0_0;
    result4 += M4(-0.0040537724, 0.049232136, 0.036732223, 0.004669483, 0.0021518688, 0.03276125, 0.06394655, 0.02553165, -0.09810836, -0.09036995, -0.14247596, 0.05236686, -0.008278465, 0.004748878, 0.08115042, 0.014921526) * inp_5_1_0;
    result4 += M4(-0.011942977, 0.0046943054, -0.016755896, -0.019923983, 0.027646529, -0.0408333, 0.020189421, 0.026804632, 0.017283073, 0.00815693, 0.06387331, -0.011298458, -0.0045564277, 0.0060155895, -0.0049308427, 0.0021229195) * inp_5_2_0;
    result4 += M4(0.030122787, 0.032693494, -0.064075135, 0.004927064, 0.008486181, 0.007928127, -0.21293813, -0.03652193, 0.020441491, 0.06951669, -0.074447684, -0.018143654, 0.009762328, 0.038104143, 0.016769404, 0.018711274) * inp_5_0_1;
    result4 += M4(-0.065759145, -0.08258699, -0.008577256, -0.023982953, 0.016749257, 0.04824361, -0.158637, 0.009589743, -0.107465856, 0.23988362, 0.10912098, 0.048319593, -0.09828676, -0.0564648, 0.037657563, -0.16690893) * inp_5_1_1;
    result4 += M4(-0.011191023, -0.025442515, 0.07944294, -0.018285302, -0.025035288, 0.023802126, -0.13170846, -0.021484297, 0.11610502, -0.01796121, -0.06980714, -0.0023755245, -0.0066997884, -0.0020131783, -0.07263523, -0.01383307) * inp_5_2_1;
    result4 += M4(-0.008652577, 0.10857369, 0.0030869197, 0.0085628955, -0.022499975, 0.0019411223, 0.018054, 0.023826241, 0.019503841, 0.062896974, -0.12375739, -0.027733227, -0.016661579, 0.049854673, -0.0057509616, 0.025719872) * inp_5_0_2;
    result4 += M4(0.013599855, 0.087083854, -0.10856698, 0.0787104, 0.041659255, -0.042988345, -0.15415856, 0.053397465, -0.032750424, -0.05883762, 0.13574311, -0.018403415, -0.024814192, -0.18713449, 0.011633942, -0.02408194) * inp_5_1_2;
    result4 += M4(-0.032173064, 0.0005632311, 0.17521708, -0.03251726, -0.007237736, -0.024949763, -0.01269889, -0.03256258, 0.06604197, -0.054761205, 0.018827915, -0.010153702, 0.028491873, 0.026990002, 0.13417743, 0.05306979) * inp_5_2_2;
    result5 += M4(-0.46076715, -0.03835556, 0.092119806, -0.13683666, -0.40153405, -0.038313076, -0.07510527, -0.0028708715, -0.058508728, -0.06280881, -0.101366, 0.040996056, -0.5135632, -0.05494163, 0.08932372, -0.118799016) * inp_5_0_0;
    result5 += M4(0.077851556, 0.092568666, -0.08975199, -0.082138225, -0.12140621, -0.049036402, -0.30763695, 0.05392082, 0.15310591, 0.024238605, 0.059515905, 0.04958534, 0.16253914, 0.13489988, -0.0018254855, -0.0056434507) * inp_5_1_0;
    result5 += M4(-0.017140172, 0.018558126, 0.044835724, 0.03758419, -0.3335715, -0.044065926, -0.15945636, 0.13207881, -0.13799009, -0.035582036, 0.08203489, -0.029903859, 0.04280802, -0.04908084, -0.055111255, 0.061538808) * inp_5_2_0;
    result5 += M4(0.08512337, 0.057432022, -0.05793682, 0.113070086, -0.65645665, -0.018844545, 0.08007824, -0.08319205, -0.099719435, -0.053093348, -0.04732173, 0.035040293, -0.17369995, 0.081802, 0.0069152503, 0.013818663) * inp_5_0_1;
    result5 += M4(0.05226874, -0.16881038, -0.429247, 0.051154457, -0.4041682, -0.03826176, -0.27028728, 0.013309577, 0.036888722, 0.0066179987, -0.096840434, -0.0153465895, -0.40367657, -0.08863324, -0.26089334, -0.12346498) * inp_5_1_1;
    result5 += M4(0.080630876, 0.007403074, 0.018696288, 0.03486996, -0.53844726, -0.048058756, -0.024670463, 0.08707717, 0.12349679, 0.07828903, 0.11004499, -0.07305551, -0.038752798, 0.085918464, 0.025746627, -0.0017505673) * inp_5_2_1;
    result5 += M4(-0.002579193, -0.07923969, 0.0055346903, 0.086216204, -0.7677999, 0.0060311714, -0.05743355, 0.030149117, -0.11719923, 0.0021021657, 0.028428057, -0.010132948, 0.010012154, -0.03209031, -0.034197282, 0.005405887) * inp_5_0_2;
    result5 += M4(0.30126354, 0.004900834, 0.019359022, 0.10080989, -0.012376663, -0.012842482, -0.48181167, -0.0852535, 0.045094017, 0.032322653, 0.049204852, -0.045708198, -0.2625209, 0.028029114, 0.07192668, -0.09521221) * inp_5_1_2;
    result5 += M4(-0.15229025, 0.032461878, 0.28814632, 0.027907552, -0.42140406, -0.00843124, -0.011013631, -0.0397028, -0.026725382, -0.046011698, 0.04764103, 0.0020315147, 0.26580846, -0.038391996, -0.20709522, 0.036907017) * inp_5_2_2;
    result6 += M4(-0.49603897, 0.032991644, -0.012306862, -0.008858483, -0.061432686, 0.004047213, 0.010161226, -0.017595386, 0.1636038, 0.02442932, 0.027665978, -0.022927545, -0.28083742, 0.084384166, 0.019335292, -0.051675323) * inp_5_0_0;
    result6 += M4(0.010468519, 0.019988356, -0.07527766, -0.1362536, -0.82755256, -0.097585596, 0.06963214, 0.04354973, -0.063526936, -0.014189566, 0.2076637, -0.01883498, 0.04533331, 0.007842347, -0.087354325, 0.013850365) * inp_5_1_0;
    result6 += M4(-0.12308421, 0.03509043, -0.13615099, -0.07474519, 0.22753562, -0.0640788, -0.05955798, -0.021276833, 0.063510254, -0.07990957, 0.16032316, 0.07256544, -0.0043086205, 0.013238542, 0.027316153, 0.0757648) * inp_5_2_0;
    result6 += M4(0.064420484, -0.05915879, -0.008973796, -0.019384092, -0.29594302, 0.032972347, 0.04846462, -0.044614945, -0.07077872, 0.012311379, 0.0132917855, -0.00066736597, -0.93404675, -0.11638888, -0.06304583, 0.008686778) * inp_5_0_1;
    result6 += M4(0.033095017, -0.026073724, -0.13196215, 0.033665888, 0.024250347, 0.13020274, 0.08528711, -0.0038196975, 0.042086553, 0.031558212, -0.2758761, -0.04419443, 0.021498779, -0.08228368, 0.0041186158, 0.0074639814) * inp_5_1_1;
    result6 += M4(0.10637943, -0.05584717, -0.103738055, 0.08264814, 0.053603753, -0.0023136064, -0.004030416, 0.035335377, 0.012709672, 0.094020896, -0.04149824, -0.055575907, -0.05248619, 0.045057416, 0.04306777, -0.18885593) * inp_5_2_1;
    result6 += M4(0.41841748, -0.02085122, 0.052419163, -0.019220613, 0.13695946, -0.121948294, -0.10177785, 0.027817046, 0.0631623, -0.07775465, -0.02085053, -0.004174766, -0.79901224, 0.038760357, 0.009698287, 0.009137501) * inp_5_0_2;
    result6 += M4(-0.010784973, 0.043316945, 0.07441658, 0.04078524, -0.2905492, -0.038453043, -0.047238715, -0.04154774, -0.017227666, 0.008357197, 0.04402219, -0.003216264, 0.17577714, 0.018764058, 0.090054475, 0.0058264155) * inp_5_1_2;
    result6 += M4(0.049587935, -0.042862646, 0.032054413, 0.10379431, -0.00926074, 0.06216661, 0.008191211, 0.015544379, -0.040396497, -0.06490637, -0.059074316, 0.04729326, -0.07256928, -0.116144985, 0.046790965, -0.10062332) * inp_5_2_2;
    result7 += M4(-0.026746806, -0.0063732825, 0.05959826, -0.0046909433, -0.01493843, 0.00091434905, -0.18743551, 0.042012163, 0.05049809, -0.011447756, 0.031230304, 0.07076543, 0.023222156, 0.04555705, 0.22618751, -0.09086003) * inp_5_0_0;
    result7 += M4(0.010794275, 0.020761797, -0.047566608, -0.10262253, 0.0073432922, -0.09196361, 0.056593876, 0.08068368, 0.064352036, 0.1264159, 0.16359529, 0.031426273, -0.013067484, 0.038718514, 0.13587828, -0.09500652) * inp_5_1_0;
    result7 += M4(0.0062483763, -0.023964386, -0.003666416, -0.07511666, -0.020724932, -0.047577046, -0.7031702, 0.020938085, 0.0029568274, 0.05904167, 0.32249817, 0.087013856, 0.013166503, -0.0063485606, 0.16617064, 0.004658752) * inp_5_2_0;
    result7 += M4(-0.035494223, -0.033370055, 0.003249448, 0.019358322, 0.006183159, 0.09134789, 0.24519368, -0.020232636, 0.0016878666, 0.08260238, 0.41712976, 0.009454556, 0.0033954813, -0.04785539, -0.49010435, 0.039766386) * inp_5_0_1;
    result7 += M4(-0.015572773, -0.06296638, -0.005425812, -0.02225108, -0.0035907472, 0.17883499, -0.0443186, 0.056811422, 0.23244329, 0.0111541925, -0.05173598, -0.09468949, -0.00042827058, -0.13293467, -0.74386233, -0.029772324) * inp_5_1_1;
    result7 += M4(-0.006373085, -0.045244444, -0.36012122, -0.09992298, -0.028348578, 0.03312543, 0.3028363, 0.031892773, 0.004392838, -0.03490129, -0.004295334, -0.060854845, -0.0030662187, -0.08384605, -0.5475483, 0.016388731) * inp_5_2_1;
    result7 += M4(0.018044852, 0.024126219, 0.03939256, 0.03579257, 0.001946877, -0.085655205, -0.61338675, 0.002266555, -0.013386549, 0.019497227, -0.10257079, -0.05379031, 0.038396224, 0.01709771, 0.10570205, 0.040749304) * inp_5_0_2;
    result7 += M4(-0.0038927256, 0.017349208, 0.13160044, 0.03181566, 0.02285283, -0.10267082, -0.037421342, -0.037303418, 0.010198675, -0.045056447, -0.10474039, 0.054058995, -0.039446745, 0.010803979, -0.07517606, 0.020779043) * inp_5_1_2;
    result7 += M4(-0.017049355, -0.011825863, 0.048654985, 0.032639876, -0.016166212, -0.020618454, -0.14206491, 0.019776486, 0.0010591405, 0.0032294807, 0.21098244, -0.013465747, 0.013606508, -0.10624778, -0.39349365, -0.0015858234) * inp_5_2_2;
    const V4 inp_6_0_0 = inp[6][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_6_1_0 = inp[6][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_6_2_0 = inp[6][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_6_0_1 = inp[6][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_6_1_1 = inp[6][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_6_2_1 = inp[6][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_6_0_2 = inp[6][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_6_1_2 = inp[6][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_6_2_2 = inp[6][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.043332297, -0.014377192, -0.080488846, -0.06631219, 0.00903335, 0.0383047, 0.01676503, 0.49419522, -0.04207586, -0.0010973342, 0.0017579851, 0.2508717, -0.07846938, 0.029180245, 0.091409884, -0.124136895) * inp_6_0_0;
    result0 += M4(0.12559173, 0.011290455, 0.17368375, 0.28245246, -0.09166459, 0.010246669, -0.08672176, -0.50188375, -0.0015455661, 0.03474074, 0.035682194, 0.2226478, -0.12436347, 0.01796184, -0.17148945, -0.33008215) * inp_6_1_0;
    result0 += M4(-0.06679831, 0.011730193, 0.1987763, -0.2737282, 0.003927589, 0.033655684, -0.10046258, 0.17817019, 0.016783055, 0.012914701, 0.016040456, 0.20325226, -0.020918561, -0.0135937035, -0.073106594, 0.012576402) * inp_6_2_0;
    result0 += M4(-0.08129438, -0.04273667, -0.06791754, -0.07889697, 0.16080794, 0.08177421, -0.035484154, 0.08670589, -0.076130755, 0.012518887, 0.034122802, -0.20771162, -0.020850848, -0.007269756, -0.1077691, -0.16133615) * inp_6_0_1;
    result0 += M4(0.035797328, 0.04962516, -0.06422945, -0.16986492, -0.06582517, -0.082394615, 0.023071205, 0.056809388, 0.107274845, 0.03904543, 0.11872531, 0.040202506, -0.32486072, -0.028858397, -0.14329316, -0.5060849) * inp_6_1_1;
    result0 += M4(-0.04949637, -0.07817578, -0.009223264, -0.032794278, 0.011168818, 0.17180462, -0.045416962, 0.1531539, 0.038168203, -0.0019058319, 0.004420817, 0.023303475, -0.19856735, -0.031255912, 0.09192223, -0.43757716) * inp_6_2_1;
    result0 += M4(-0.08994586, -0.03731863, 0.19433053, -0.30883062, 0.10239321, 0.013170655, -0.14223666, -0.03580948, -0.020386713, 0.012460651, 0.03233486, 0.038706016, -0.04519236, -0.0108860275, -0.046011094, -0.20857796) * inp_6_0_2;
    result0 += M4(0.092503205, 0.0027167718, -0.0057640616, 0.22906594, -0.061495975, -0.040565554, -0.1303442, -0.20432097, 0.08981018, -0.002331685, 0.15038045, -0.016637767, 0.16734882, -0.079746865, -0.025504183, -0.21633725) * inp_6_1_2;
    result0 += M4(0.032639775, -0.0016152647, -0.030264027, 0.06232754, -0.03252976, 0.08887677, -0.029743828, -0.061830655, 0.032990485, 0.02069313, -0.008767754, 0.32882684, 0.0037135843, 0.04818343, -0.05781765, -0.30432776) * inp_6_2_2;
    result1 += M4(0.18254404, 0.007023544, 0.19997646, -0.1294229, 0.020645225, -0.041317534, -0.25974247, -0.08465931, -0.022262888, 0.010912279, 0.1410021, 0.04637032, -0.32318723, -0.0036948267, -0.10547101, -0.019716267) * inp_6_0_0;
    result1 += M4(-0.010818667, -0.07089831, 0.24304372, 0.020169597, -0.028867992, 0.038999893, -0.007096987, -0.028133156, -0.020012012, 0.034865037, 0.044881698, 0.054903813, -0.055157755, 0.008320665, -0.39614087, -0.041533917) * inp_6_1_0;
    result1 += M4(0.01705527, 0.041950878, -0.21108535, 0.0011876612, -0.008091079, -0.0577998, -0.43838036, -0.031092338, 0.0032684817, 0.014348297, 0.12793125, 0.023079015, 0.007440506, 0.008587015, -0.19309007, -0.009189978) * inp_6_2_0;
    result1 += M4(-0.0024729036, 0.0380431, 0.07555356, 0.2447987, 0.0017309173, -0.010395209, 0.31079626, -0.037594575, -0.011810419, 0.022023687, 0.26038897, 0.052912857, -0.008570085, -0.011552544, -0.23152494, -0.019091753) * inp_6_0_1;
    result1 += M4(-0.04270095, 0.07598949, -0.10431557, 0.08375825, -0.01813992, 0.065414675, 0.014647973, -0.1138702, -0.006944555, 0.0030062946, 0.049630083, 0.09395628, -0.016886886, 0.08216145, -0.024746148, 0.093466856) * inp_6_1_1;
    result1 += M4(0.01818649, -0.010126846, 0.047577467, -0.039601665, -0.0010942457, -0.022518469, 0.091618925, -0.023581319, 0.005756635, -0.01435444, 0.21682355, -0.020879155, 0.0075598485, -0.02828723, -0.020965496, -0.001949865) * inp_6_2_1;
    result1 += M4(0.014376053, 0.024825223, -0.08176737, 0.040240217, 0.013297813, 0.013316002, 0.031194491, 0.057364177, 0.0015454098, 0.007446174, -0.03505964, 0.026251644, 0.0012399943, 0.014698601, 0.07014454, 0.04881295) * inp_6_0_2;
    result1 += M4(0.013962704, -0.008336851, -0.08824748, -0.12179033, 0.004927966, -0.0037224595, -0.107665025, -0.016337175, -0.006318701, 0.0110256625, 0.071001686, -0.021002062, 0.00042326876, -0.02397745, -0.15467094, -0.116949685) * inp_6_1_2;
    result1 += M4(0.020354219, -0.011617306, -0.06619461, -0.0074034133, 0.0057157134, 0.008752395, -0.21601935, -0.0053082393, -0.000392168, 0.011512692, -0.07419855, -0.00063944026, 0.00096870464, 0.004081132, 0.08922514, -0.03804124) * inp_6_2_2;
    result2 += M4(-0.1231277, -0.04509508, -0.0034936653, -0.015059272, 0.020379696, 0.029995108, -0.014218472, -0.00021767359, -0.016820427, -0.0022709859, 0.007729199, -0.010889928, 0.052829344, 0.02161878, 0.016301781, 0.012735931) * inp_6_0_0;
    result2 += M4(0.0744256, 0.076082274, -0.0011494496, 0.023065703, 0.0581367, 0.012680858, 0.012694621, 0.015195861, 0.01636022, 0.02157405, -0.008098103, -0.027824396, -0.12690464, -0.012606015, 0.045917593, -0.07370335) * inp_6_1_0;
    result2 += M4(0.0041577346, -0.009108256, 0.007063968, -0.0059181806, -0.117997564, -0.026005944, -0.009289647, -0.012073323, 0.060582392, 0.013918362, -0.0028407415, 0.005471382, 0.017061684, 0.06754428, 0.0061389436, 0.006295731) * inp_6_2_0;
    result2 += M4(0.13576834, 0.0035801742, 0.038021564, -0.027635518, -0.06319789, 0.013916106, 0.0087700095, 0.032328546, 0.055108465, 0.02848068, 0.007795516, -0.008245575, -0.10850881, -0.0005480408, -0.052535098, 0.07553291) * inp_6_0_1;
    result2 += M4(-0.057622965, -0.038355753, 0.039395235, 0.040872194, 0.041987643, 0.033162583, -0.00033942747, -0.051906917, 0.09183527, 0.13337317, -0.017249499, -0.0044697546, 0.050102055, -0.12880197, -0.15792972, -0.3138467) * inp_6_1_1;
    result2 += M4(0.06461565, -0.0452197, 0.047619864, -0.0020515309, -0.08666278, 0.023060897, -0.013710903, -0.01716436, 0.021559432, 0.039545093, 0.0014229503, -0.015903031, -0.026729183, -0.034547083, 0.03905944, -0.077720165) * inp_6_2_1;
    result2 += M4(0.022253152, 0.007640731, 0.005909585, -0.011254731, 0.03183623, -0.04995794, -0.0018443519, 0.011464202, 0.023527715, 0.024825389, 0.004405872, -0.019099561, 0.024715433, -0.02878368, 0.011970843, 0.019611832) * inp_6_0_2;
    result2 += M4(0.03831637, 0.026373709, -0.01111479, 0.03128865, 0.017608816, 0.023089528, 0.009005184, 0.026298616, 0.03246168, 0.032839302, 0.014053395, -0.008041265, -0.00983807, -0.061087653, 0.0015417669, 0.014035069) * inp_6_1_2;
    result2 += M4(-0.10121087, 0.011310987, 0.0047188117, -0.0083891675, -0.003480097, 0.024381327, -0.010190609, -0.0055995425, 0.0141550265, 0.024149809, -0.0007066083, -0.010508926, -0.089584045, 0.072138116, 0.0058450694, 0.004410908) * inp_6_2_2;
    result3 += M4(-0.035038523, 0.040623028, -0.007545786, 0.000211672, 0.019939806, -0.07294372, 0.015763957, -0.0042124526, -0.060138952, -0.028309146, -0.004303688, 0.0025914304, 0.12399618, -0.03877816, 0.0249435, -0.0056933137) * inp_6_0_0;
    result3 += M4(-0.0420027, -0.07854148, 0.017072048, -0.0008687768, -0.03291392, -0.014374888, 0.014246792, 0.00020560694, -0.030989356, 0.021775544, -0.019060997, 0.0010221389, -0.029605122, 0.012150253, 0.019139498, -0.0015450785) * inp_6_1_0;
    result3 += M4(0.022324303, 0.058415607, -0.013112765, 0.006204944, 0.027393328, -0.060674854, 0.009099869, -0.012623702, -0.025956722, 0.046260364, -0.010029415, 0.007711598, 0.024963208, 0.08430007, -0.013572675, 0.013662044) * inp_6_2_0;
    result3 += M4(-0.0036949848, -0.078845054, 0.026675308, 0.012533102, 0.019003965, 0.016140178, 0.010315194, -0.006051955, 0.122890845, 0.018336557, -0.0009780278, 0.0038419657, -0.08010542, -0.05692978, 0.011654074, 0.002498952) * inp_6_0_1;
    result3 += M4(-0.14509092, 0.1608519, -0.04387457, -0.06754008, -0.121985056, -0.1718665, 0.10394824, -0.0034890547, 0.17192592, 0.03866943, -0.044627022, -0.0032386556, -0.17857142, -0.19382107, 0.0678997, 0.029718429) * inp_6_1_1;
    result3 += M4(-0.006995825, -0.04293646, -0.061527487, 0.05182663, -0.043008693, -0.0043861833, 0.025782768, 0.006373135, 0.10551491, 0.10384799, -0.015285439, 0.014919352, -0.13469398, -0.06547428, -0.0805178, 0.010914549) * inp_6_2_1;
    result3 += M4(0.02208204, 0.0650555, -0.025874268, -0.009691239, -0.019785617, 0.050954223, 0.004638607, -0.016083382, -0.013072881, -0.0016749277, 0.00683565, 0.0017152556, -0.21259254, 0.08317361, 0.025581174, -0.0055785044) * inp_6_0_2;
    result3 += M4(0.09466161, -0.028358992, -0.040004253, 0.034183316, 0.07536053, 0.023133976, 0.01757506, -0.013094293, -0.014288379, 0.056445025, 0.009219702, -0.0036399171, -0.046435297, 0.081140496, -0.04162579, -0.002235806) * inp_6_1_2;
    result3 += M4(0.055553105, 0.08088184, -0.008009948, -0.22753847, 0.03553615, -0.065425366, 0.0019478707, -0.49297482, 0.039439525, 0.01557839, 0.0026492237, -0.056701604, 0.011669341, 0.0076518008, -0.028221775, -1.6017205) * inp_6_2_2;
    result4 += M4(0.011367794, 0.027336624, -0.02349224, 0.04495588, -0.007413163, 0.044424493, 0.003020182, 0.034318395, -0.00698715, -0.015981171, 0.10228053, 0.0063005225, 0.011335029, -0.021104407, 0.054949548, -0.05159743) * inp_6_0_0;
    result4 += M4(0.074798554, -0.04490351, -0.088873334, -0.018236373, -0.010978549, 0.078075156, -0.058612216, -0.07251333, -0.029559532, -0.038611945, 0.001923, 0.00027036946, -0.05482899, -0.03277773, -0.021309424, 0.006844568) * inp_6_1_0;
    result4 += M4(0.014248899, -0.03851628, -0.052872907, 0.068987966, -0.018304093, 0.035079487, -0.0017512444, -0.023423437, -0.0077106664, 0.00383543, -0.018890155, 0.020115424, 0.007213108, 0.016664598, -0.07925334, -0.030902792) * inp_6_2_0;
    result4 += M4(0.04664659, -0.06619677, 0.030556843, -0.03203254, -0.031023974, 0.06702593, -0.02244827, 0.15989687, -0.0035659363, -0.0007910926, 0.089576244, -0.012593203, -0.08245995, -0.011583045, -0.0742212, 0.051530134) * inp_6_0_1;
    result4 += M4(0.028665613, -0.012577556, -0.1087432, 0.114664935, 0.026843734, -0.11754216, 0.06698387, 0.008731316, -0.0052947276, 0.022968149, 0.08450233, 0.06086195, -0.12335593, -0.010759306, 0.00398709, 0.064146504) * inp_6_1_1;
    result4 += M4(0.070074365, -0.0032613366, -0.0029638119, -0.04035658, -0.02352318, -0.01243501, -0.029568462, -0.019636033, 0.004134089, 0.01436446, 0.054191332, 0.025113603, 0.09276342, -0.0070793917, -0.053715225, -0.045503087) * inp_6_2_1;
    result4 += M4(0.017474182, -0.020552034, -0.10180618, 0.051640533, -0.009749781, 0.041000813, -0.01713524, 0.02209197, 0.0014285474, 0.10782737, -0.025552854, 0.042746074, 0.007077808, -0.078457445, -0.009340124, 0.029864077) * inp_6_0_2;
    result4 += M4(0.039870344, -0.0774889, -0.13539994, 0.011194178, 0.037408654, 0.07373557, 0.004986887, 0.021657431, -0.016565625, 0.11861111, 0.16784264, 0.013123299, 0.082150236, -0.097027525, -0.2318027, -0.011818387) * inp_6_1_2;
    result4 += M4(0.007518363, 0.03141021, -0.019461932, 0.09085669, -0.013174869, -0.011560554, 0.044810187, -0.00036722346, -0.0289138, 0.07761437, 0.09045137, 0.0011832833, 0.0060305726, 0.017883053, -0.026976049, 0.09035868) * inp_6_2_2;
    result5 += M4(-0.04468038, -0.022385178, -0.15726711, 0.02123018, -0.16632213, 0.0034201723, 0.19013795, 0.031959645, 0.12506889, 0.051641382, 0.024639266, 0.06360613, -0.19092005, -0.01942603, 0.049447987, -0.07994745) * inp_6_0_0;
    result5 += M4(-0.1005116, -0.04007247, -0.025413608, 0.046106424, -0.104182504, -0.0027621645, 0.033041146, -0.11546465, 0.15051484, 0.049479358, 0.06710434, 0.038667876, -0.08013104, 0.021098467, -0.035272878, 0.05686449) * inp_6_1_0;
    result5 += M4(-0.17923245, -0.043831356, 0.0018323553, -0.06093899, 0.013493708, -0.024358341, -0.07723428, 0.069753274, -0.008933989, -0.008455388, -0.032212343, -0.04924502, -0.3295535, -0.07092386, -0.032771323, -0.06537877) * inp_6_2_0;
    result5 += M4(0.23601371, -0.04872319, -0.1513476, 0.13861717, -0.21339995, 0.015496643, -0.009045353, 0.04291372, 0.25116348, 0.050497506, -0.036239468, 0.073429435, -0.1507921, 0.032965396, -0.097130015, -0.1596891) * inp_6_0_1;
    result5 += M4(-0.024422867, -0.07476173, 0.056936577, 0.04048466, -0.15879175, 0.10721412, -0.110990584, 0.016676635, 0.035436947, 0.09462268, 0.10078811, 0.022442436, -0.25823027, -0.051827442, -0.1698131, 0.15227672) * inp_6_1_1;
    result5 += M4(0.28979465, 0.05724979, -0.05521714, -0.042524338, -0.047154605, -0.05123926, -0.090806834, 0.043589815, 0.055851683, 0.044585366, 0.06107546, -0.030272827, 0.12539274, 0.07261656, 0.013705203, -0.14315435) * inp_6_2_1;
    result5 += M4(-0.053861752, -0.007488851, -0.010934571, 0.011328316, -0.04846107, -0.03488473, -0.059064552, 0.012903349, 0.030700052, -0.019999228, -0.011935754, 0.011462976, 0.07391946, -0.06847386, -0.14440969, 0.039374985) * inp_6_0_2;
    result5 += M4(-0.20014568, 0.06512544, -0.022995988, -0.03356457, 0.25916198, 0.019705886, 0.0152826635, -0.023507657, -0.007586443, 0.061641917, 0.10341651, 0.034882184, 0.13838406, 0.018763196, 0.052666623, 0.05918333) * inp_6_1_2;
    result5 += M4(0.136243, -0.053442836, -0.13511246, 0.0025580633, 0.1138576, 0.0071883085, -0.035530653, 0.03810132, 0.05762128, 0.04968306, 0.026131865, 0.013911896, 0.06961758, 0.042012323, -0.037109982, -0.045700237) * inp_6_2_2;
    result6 += M4(0.10297815, -0.09981974, -0.01488965, 0.0179118, 0.27892834, 0.059375215, 0.0064688628, -0.058811698, 0.27585065, -0.021414923, 0.056856003, -0.03020367, -0.035481803, -0.12275734, -0.034952916, -0.06408202) * inp_6_0_0;
    result6 += M4(0.10436862, -0.1147277, 0.12999411, -0.019599251, -0.1536522, -0.036289223, 0.030268716, -0.022310035, -0.049499333, 0.008321259, 0.09678967, 0.06342723, -0.0007026786, -0.105600014, -0.011613988, 0.023196394) * inp_6_1_0;
    result6 += M4(-0.1171643, -0.062064853, -0.038041152, -0.005928313, 0.09718247, -0.050367363, -0.0047570923, -0.05697843, 0.005175337, -0.018172426, 0.19555148, 0.02320028, -0.0829016, -0.083134085, -0.015067847, -0.08734985) * inp_6_2_0;
    result6 += M4(0.16986749, 0.05223131, 0.020024903, 0.030544803, -0.12923816, 0.039957423, 0.030129485, -0.06372535, -0.11830069, 0.055305004, 0.011592451, -0.022941533, -0.4231864, -0.1534134, -0.04454991, -0.091279164) * inp_6_0_1;
    result6 += M4(-0.032288596, 0.11129392, 0.009423402, 0.06851816, 0.043272506, 0.0008306889, 0.12099234, 0.14365031, 0.047576573, 0.09701245, -0.012605625, 0.07375083, 0.26906615, -0.27436495, 0.050849006, -5.1567768e-06) * inp_6_1_1;
    result6 += M4(-0.043197103, 0.0130282035, 0.054667767, -0.024831485, 0.07697463, -0.1530302, 0.08547747, 0.026715731, -0.03250728, 0.101979405, -0.008973825, 0.060965456, -0.11122125, 0.04020316, 0.060815427, -0.03218629) * inp_6_2_1;
    result6 += M4(-0.16931139, -0.038455084, -0.02244914, -0.013750047, 0.1791467, -0.053868316, 0.005281963, -0.001778717, -0.10472638, 0.038182605, -0.00085563253, 0.0029218458, -0.15345363, -0.02560928, -0.027652983, -0.024375292) * inp_6_0_2;
    result6 += M4(0.17765707, -0.09735637, -0.07428319, 0.01233911, -0.028753452, -0.081867345, -0.072785564, 0.0028685643, 0.015957179, 0.11356993, 0.0012284879, 0.039963633, 0.23338291, -0.064257815, -0.093671046, -0.018596549) * inp_6_1_2;
    result6 += M4(-0.10880716, -0.06470289, -0.062493168, 0.00545643, -0.021667898, -0.014744693, -0.07544274, 0.0368637, -0.07024395, 0.03494621, -0.011488307, 0.024664382, -0.06384221, -0.022640646, -0.0835214, 0.027719498) * inp_6_2_2;
    result7 += M4(-0.009228005, 0.014123517, 0.10596036, -0.007396036, 0.01300227, 0.030818967, 0.022506984, 0.0034125017, -0.006575523, 0.004100767, 0.06264965, 0.001892128, -0.027297098, -0.031819984, -0.102415524, -0.0069772196) * inp_6_0_0;
    result7 += M4(-0.007389542, -0.019223057, 0.02061048, 0.06780323, 0.000875358, -0.06896766, -0.04079004, -0.051525336, 6.749343e-05, 0.0029261813, 0.10223511, -0.0075562326, -0.0051420415, 0.054267626, 0.16781387, -0.10338951) * inp_6_1_0;
    result7 += M4(0.0025697201, -0.050384857, -0.120460294, 0.022646759, 0.0074091125, -0.0287789, -0.012341631, -0.05051047, 0.0006096386, 0.024696717, 0.10078787, -0.0156161385, -0.020851033, 0.03852885, -0.33858654, -0.022351947) * inp_6_2_0;
    result7 += M4(0.066401854, -0.012245457, 0.24627563, 0.059977997, 0.015233223, 0.10566705, 0.2311431, 0.03708189, -0.0040017436, 0.032816254, 0.22700575, 0.022072181, -0.14335777, 0.02478264, -0.35654092, -0.08231101) * inp_6_0_1;
    result7 += M4(0.03341173, 0.065120466, -0.026360381, 0.016492775, 0.005999619, 0.17095757, -0.055731185, 0.13711114, -0.002136683, -0.05489179, 0.09192377, 0.1364922, -0.0026806595, -0.15378393, -0.63558525, 0.115244746) * inp_6_1_1;
    result7 += M4(0.011990092, 0.08521507, 0.113053605, 0.012105212, 0.007856928, 0.045388944, 0.032750156, 0.0033127363, 0.0031788624, 0.0020994016, 0.046746198, 0.062163968, 0.036256086, 0.18488203, 0.24416682, 0.06923279) * inp_6_2_1;
    result7 += M4(0.012042225, 0.0046930388, -0.079075955, -0.01187981, -0.009747119, 0.00027468588, -0.160026, -0.00027201205, 0.016349299, 0.022668315, 0.117218636, 0.010106807, -0.06478647, 0.033604674, -0.066599935, -0.018665517) * inp_6_0_2;
    result7 += M4(-0.00024250716, -0.068138495, -0.20656036, -0.07606658, -0.0024029315, -0.106036134, 0.029002447, -0.005625167, 0.0017133693, 0.06501927, 0.08614489, 0.015360999, -0.008059606, -0.15363906, -0.2708422, -0.09379124) * inp_6_1_2;
    result7 += M4(0.0119940825, -0.00862066, 0.043323446, -0.0397725, -0.0046180156, 0.043170933, -0.16421588, -0.009885347, 0.00063538324, 0.011789609, 0.14348866, -0.004623234, 0.011499534, -0.025823915, 0.08056418, -0.077109) * inp_6_2_2;
    const V4 inp_7_0_0 = inp[7][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_7_1_0 = inp[7][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_7_2_0 = inp[7][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_7_0_1 = inp[7][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_7_1_1 = inp[7][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_7_2_1 = inp[7][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_7_0_2 = inp[7][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_7_1_2 = inp[7][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_7_2_2 = inp[7][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.040170055, 0.00879078, 0.0012368743, -0.31770676, -0.0027730165, 0.002083311, 0.09678669, -0.13613385, -0.030688368, 0.025266718, 0.23029685, -0.046581134, -0.05481202, -0.0036689707, 0.03295225, -0.010785736) * inp_7_0_0;
    result0 += M4(-0.12984921, -0.04836447, -0.03235185, -0.30996758, -0.16969255, -0.023023738, -0.23616482, -1.0872197, 0.25116944, -0.09566722, -0.24564943, 0.1125025, 0.20271294, -0.016967857, -0.12860954, 0.3084914) * inp_7_1_0;
    result0 += M4(-0.03286122, -0.03881154, -0.1983098, -0.26250648, -0.023873536, 0.028781937, 0.086933404, 0.13543805, -0.021412801, -0.03457631, -0.01005973, -0.22055659, -0.1625026, -0.15724608, 0.11830158, -0.30057508) * inp_7_2_0;
    result0 += M4(-0.23399554, -0.0064418553, -0.1947766, -0.31333092, 0.04586455, -0.016594814, -0.09457843, -0.3284784, 0.027750041, 0.07406504, 0.14213674, 0.31870946, 0.072512716, 0.03229966, 0.07185042, 0.3838668) * inp_7_0_1;
    result0 += M4(-0.35478106, -0.11056482, -0.26676628, -0.726233, -0.36631864, -0.13309199, -0.34414285, -1.0571644, -0.12371925, 0.06641371, 0.21894406, -0.026601257, 0.101361014, -0.038793247, 0.14480884, -0.29117197) * inp_7_1_1;
    result0 += M4(-0.07974877, -0.20646305, -0.30896568, -0.2483887, 0.057003386, -0.16073, -0.041072752, -0.009927738, -0.0026659023, 0.026298808, -0.19030665, 0.20252825, -0.10058842, -0.19898476, 0.031478595, -0.5169593) * inp_7_2_1;
    result0 += M4(0.033951744, 0.025361683, -0.13624711, -0.17803304, 0.057052936, 0.047850117, -0.44744214, 0.08604044, 0.19573985, 0.024481036, -0.05230609, -0.06104629, -0.06694523, 0.007114736, -0.0363824, -0.23867817) * inp_7_0_2;
    result0 += M4(-0.12942965, -0.04201597, -0.2091742, -0.8084698, -0.29348415, -0.29627064, -0.34415838, -1.2164366, -0.12021396, 0.021354495, 0.03898087, -0.16717887, 0.1481454, -0.01851237, 0.04686286, 0.28784633) * inp_7_1_2;
    result0 += M4(-0.05676525, -0.06584945, -0.0034480311, -0.37265328, -0.0071569704, -0.1472044, -0.09926492, -0.19328405, 0.02138315, 0.014337378, 0.10167605, 0.025296994, 0.029125908, -0.0934067, -0.035944477, -0.061428618) * inp_7_2_2;
    result1 += M4(-0.035262026, -0.012866059, 0.58150923, -0.076198354, -0.064291, -0.008916476, -1.3872691, 0.04675476, 0.31490034, 0.05691547, -0.56729853, 0.1632358, 8.0856975e-05, 0.006256946, -0.13564943, 0.010256218) * inp_7_0_0;
    result1 += M4(0.0074381237, -0.03426422, 0.31923148, -0.075658314, 0.0011978394, -0.19310358, -0.6504667, -0.09864096, 0.024382742, -0.053964846, -0.07132453, -0.14546697, 0.02669548, -0.030046916, -0.12733, -0.028166117) * inp_7_1_0;
    result1 += M4(0.015077627, -0.086718544, 0.36181802, -0.0027520682, -0.022381209, 0.018237533, -1.2222703, 0.014109561, 0.001017459, -0.0316191, 0.14567511, -0.008393273, 0.009843432, -0.0015355378, -0.13125846, -0.0012177961) * inp_7_2_0;
    result1 += M4(0.015445047, -0.029548708, 0.015421104, -0.12709233, 0.009143391, -0.07267672, -0.07502246, -0.19516027, -0.078236915, 0.007614515, 0.13758248, -0.117916964, 0.005538698, 0.0057892976, -0.5115815, -0.00039974842) * inp_7_0_1;
    result1 += M4(0.024745079, 0.023062162, 0.061139904, -0.06906079, -0.010915971, -0.1025094, -0.4293453, -0.032248545, 0.027109362, 0.025129123, 0.017920086, 0.12872833, -0.0017372041, -0.037135, 0.10388494, -0.00062965706) * inp_7_1_1;
    result1 += M4(0.007662136, 0.020297322, -0.18906718, 0.010941685, -0.011295618, 0.04597594, -0.28803274, -0.00955054, -0.017083652, 0.013755821, -0.08150079, 0.01767514, -0.010126711, -0.005957836, 0.15280046, 0.010836384) * inp_7_2_1;
    result1 += M4(-0.010667588, -0.005557579, -0.066218756, -0.005657301, -0.032649036, 0.015651727, -0.030584276, -0.33647418, -0.016504014, -0.01900862, 0.19532813, 0.08798786, 0.010086759, 0.005227926, 0.06368677, -0.038496993) * inp_7_0_2;
    result1 += M4(-0.01852501, -0.0084255515, 0.04034876, 0.040511366, -0.0007811298, 0.016281666, -0.116627894, -0.018926168, -0.0060920096, 0.009587319, 0.22868216, 0.017168185, -0.008137484, -0.0031746775, 0.08053533, -0.01176141) * inp_7_1_2;
    result1 += M4(-0.00068617973, 0.007825774, -0.0699372, 0.0112982225, -0.00517248, -0.00669365, 0.064659506, -0.001904597, 0.010313116, 0.004072099, 0.04457813, 0.0017720166, -0.0024410298, 0.00553868, 0.041898593, 0.0016244368) * inp_7_2_2;
    result2 += M4(-0.0030892247, -0.013152086, -0.023460997, -0.022195542, 0.06960121, -0.03378513, -0.012734452, -0.044608217, 0.14570518, 0.035890512, -0.04974639, 0.038624484, -0.05051223, 0.011312129, 0.020314407, 0.009202232) * inp_7_0_0;
    result2 += M4(-0.045445934, -0.06749423, 0.010304166, 0.0026410883, -0.029163234, -0.022783319, 0.006227208, -0.009592117, -0.09471648, -0.116224356, 0.063726574, -0.030770019, 0.03562266, 0.052972082, -0.022885015, 0.018556012) * inp_7_1_0;
    result2 += M4(-0.06057579, 0.004292042, -0.0059224167, 0.01595622, 0.007080103, 0.012628858, 0.0012358013, -0.013857562, -0.0679045, 0.053988606, -0.007451439, 0.0017449697, -0.0033686925, -0.004443708, -0.00054356, -0.012488927) * inp_7_2_0;
    result2 += M4(-0.06890312, -0.057443157, -0.007880981, -0.024848366, -0.32874304, -0.11758152, -0.012939342, -0.015246167, -0.05681874, 0.0036625464, -0.07783227, 0.16233294, 0.024473041, -0.037588045, -0.024286084, 0.00042091263) * inp_7_0_1;
    result2 += M4(-0.18700713, -0.171565, -0.019573348, -0.047552686, -0.25887063, -0.13596152, -0.09883276, -0.054886375, 0.1310805, 0.17696138, -0.04335991, 0.0062475414, -0.1255835, -0.021597255, 0.00418413, 0.029230425) * inp_7_1_1;
    result2 += M4(-0.08730935, -0.07382161, -0.011119422, -0.010805046, -0.02990161, -0.0039636292, 0.019130617, -0.02865904, 0.05073468, -0.079868674, -0.0044641183, 0.010501907, 0.04346923, -0.0073870504, 0.0044405293, -0.013394242) * inp_7_2_1;
    result2 += M4(-0.0425775, 0.007910439, -0.00451353, -0.0110107735, -0.037672054, 0.045811098, -0.021798734, 0.031967014, 0.02210099, 0.031199394, -0.033206787, 0.0043991506, -0.014342013, -0.0018564997, 8.796192e-05, 0.0029727365) * inp_7_0_2;
    result2 += M4(0.066219054, 0.007408612, 0.0048309267, -0.020938314, -0.014186328, -0.10169508, -0.037767228, -0.027179705, -0.020161252, -0.013316539, -0.025956776, 0.070725575, 0.0065208958, 0.0007321184, -0.013148873, 0.004365841) * inp_7_1_2;
    result2 += M4(0.017838147, -0.0599798, -0.0063994387, -0.012765161, 0.0016088508, -0.027735902, -0.006756153, -0.01852038, 0.030647118, 0.023446847, -0.023460357, 0.0060053444, -0.028863132, -0.015597247, -0.00185031, 0.009397509) * inp_7_2_2;
    result3 += M4(-0.03201502, -0.0010246583, 0.010989766, -0.0047446936, 0.0061979843, -0.008312802, -0.017878242, 0.005390319, 0.12812085, 0.019803267, -0.017565656, -0.025300695, -0.1915557, 0.019446215, -0.020936776, 0.014262476) * inp_7_0_0;
    result3 += M4(-0.044430923, 0.011942378, 0.012007951, -0.004036818, 0.09318656, 0.065292984, -0.011544049, 0.002966337, 0.17581789, 0.07717444, -0.011008261, -0.019071843, -0.027180845, -0.03418957, 0.051458538, -0.0017600029) * inp_7_1_0;
    result3 += M4(0.005984382, -0.09225365, 0.00584854, -0.00597495, 0.024184918, 0.0013790216, -0.0033698229, -0.0022800565, -0.006031373, -0.029797323, 0.005940698, -0.0481922, 0.028752485, 0.042500462, 0.0066547124, -0.012201175) * inp_7_2_0;
    result3 += M4(-0.07417269, -0.07959531, 0.037612814, -0.00014259825, -0.21161473, -0.016628623, -0.06292587, -0.0009790997, -0.059170805, 0.012380262, -0.012455642, 0.0026435868, 0.11860272, -0.045396388, -0.03282649, -0.008758662) * inp_7_0_1;
    result3 += M4(0.050734576, -0.14357178, -0.016442062, 0.020068625, -0.31421384, -0.18012916, -0.022807991, 0.011238566, -0.09350336, -0.022817556, 0.051955026, 0.25679734, 0.035627697, 0.079022005, 0.040292684, 0.0035482477) * inp_7_1_1;
    result3 += M4(-0.07836848, -0.04716197, 0.021246552, 0.01598618, -0.1328892, -0.07019074, -0.0056017055, 0.02103905, -0.0115075335, 0.012342157, 0.012847684, 0.17228512, 0.010082911, -0.08910863, 0.009279556, -0.016739776) * inp_7_2_1;
    result3 += M4(0.08649918, -0.047305316, -0.0032224304, -0.003618699, 0.217499, 0.0014365435, -0.057874106, 0.008726625, 0.06814135, 0.107716545, -0.02757312, -0.024661582, -0.07406181, 0.005893448, -0.010811424, 0.00017096454) * inp_7_0_2;
    result3 += M4(-0.011903334, -0.023815079, 0.044893086, 0.007732493, -0.038073614, -0.2754917, -0.05237828, 0.008801407, 0.10103767, -0.01804861, -0.02441906, 0.2654082, -0.026461288, -0.038591847, 0.014705571, -0.04049266) * inp_7_1_2;
    result3 += M4(0.043069012, -0.025669351, -0.0033935586, -2.2353628, 0.03882596, -0.014284271, -0.008517233, -2.3202024, 0.041138355, 0.020434074, 0.028381271, -0.22164856, 0.019828506, 0.0037719132, 0.007986745, -1.163318) * inp_7_2_2;
    result4 += M4(0.0002759149, -0.014297367, -0.060594898, 0.0011020658, -0.0037627462, -0.009287328, 0.06840296, -0.030913215, -0.042986907, -0.003753549, -0.07948919, -0.062851205, -0.02184866, 0.023386208, 0.026136829, 0.013427554) * inp_7_0_0;
    result4 += M4(0.019702563, -0.009202947, 0.049250197, -0.058384757, -0.007893416, -0.04115985, 0.014424874, 0.067425355, -0.086646475, 0.05979366, 0.25502187, 0.2199998, -0.003894179, 0.02103982, -0.08872753, 0.062736094) * inp_7_1_0;
    result4 += M4(0.0072805765, 0.016012758, 0.002770096, -0.0882601, -0.0013448952, -0.02205779, -0.0070295865, -0.039200023, -0.03162062, 0.09314597, 0.06560124, -0.033720408, -0.007319617, -0.02370985, 0.06322515, -0.022014001) * inp_7_2_0;
    result4 += M4(-0.0064613847, -0.0120754335, -0.052823476, -0.09010493, -0.0154256085, -0.08766769, -0.004660266, 0.0020030353, -0.118510425, 0.03479459, 0.08218821, 0.251885, 0.027644197, -0.032074682, -0.1915171, -0.06989497) * inp_7_0_1;
    result4 += M4(-0.04328099, -0.0038845097, -0.07026959, -0.11553873, -0.06946208, 0.0055034687, -0.3379573, -0.059294883, 0.06170084, -0.031481843, 0.07800894, -0.3192906, -0.011575509, 0.058792926, 0.07218566, 0.023024771) * inp_7_1_1;
    result4 += M4(-0.03240775, -0.075432256, -0.15528429, 0.026118899, -0.025469221, -0.0056431633, 0.16879301, -0.04541229, 0.0061127217, -0.16090788, -0.1217467, 0.065204844, -0.014038512, -0.0021795367, -0.034178946, -0.07117251) * inp_7_2_1;
    result4 += M4(-0.02350012, 0.018019803, -0.08145114, -0.0037304433, 0.009198829, -0.2621893, -0.12554725, 0.017643845, -0.029905628, -0.044784375, 0.15356708, 0.06203971, 0.0026143093, -0.0027342928, 0.1265926, -0.032561176) * inp_7_0_2;
    result4 += M4(-0.005483659, 0.022613153, -0.05692663, 0.022970015, -0.04623334, -0.20955627, -0.27429685, -0.12931368, -0.0017410197, 0.05144904, -0.1855843, 0.05127522, -0.022047061, 0.024974583, -0.08394408, -0.018942911) * inp_7_1_2;
    result4 += M4(0.003697093, 0.00924427, -0.115256645, -0.030475203, 0.023104329, -0.055441324, -0.28992662, 0.032772895, -0.041231953, 0.0679779, 0.10657169, -0.047586758, 0.012098259, 0.022591356, -0.037535552, 0.005478513) * inp_7_2_2;
    result5 += M4(-0.23588176, -0.024660893, -0.10735465, -0.11005753, -0.19915296, -0.06150915, 0.030206146, -0.18097535, -0.16100816, -0.107516736, 0.25560904, -0.065518, 0.051735435, 0.005487137, 0.016982034, -0.028786631) * inp_7_0_0;
    result5 += M4(-0.3020115, -0.04583721, -0.06096122, -0.18708955, 0.06259903, 0.05000909, -0.25071955, -0.026292704, 0.26089892, 0.1895104, -0.16219428, 0.030648354, -0.035002287, -0.026570026, 0.016462337, 0.009223368) * inp_7_1_0;
    result5 += M4(0.035113703, 0.0007197818, 0.0050879144, 0.06786782, -0.028672962, -0.032618422, 0.02478821, -0.017744321, 0.12171519, 0.0019136637, 0.02896062, 0.0018193993, -0.080604725, 0.0059052585, -0.025928361, -0.070181005) * inp_7_2_0;
    result5 += M4(-0.27176052, -0.051526636, -0.2550345, -0.12122562, -1.3687475, -0.16636789, 0.098430686, -0.41978508, 0.09851637, 0.25347197, -0.014472201, -0.038936555, 0.07995173, -0.071483, 0.05234322, -0.039819974) * inp_7_0_1;
    result5 += M4(-0.26387942, -0.141253, -0.16026425, -0.20974897, -0.8063747, -0.40512562, -0.47479665, -0.13446213, -0.026785344, -0.18278493, 0.102402255, -0.0677977, 0.016939525, 0.0101193, 0.05034911, -0.012257474) * inp_7_1_1;
    result5 += M4(-0.58033735, -0.03901744, -0.17041922, 0.052547816, -0.12124777, 0.091615446, 0.060746245, -0.04700261, -0.15232754, -0.030884821, 0.020195061, 0.105344385, 0.18593702, -0.047375068, -0.16525519, 0.008977754) * inp_7_2_1;
    result5 += M4(-0.09666921, 0.002328804, 0.0538851, -0.023760736, -0.2853305, 0.07848313, 0.020272091, -0.2598813, 0.2615085, -0.011578752, -0.008579952, 0.09905439, -0.14233845, 0.0029739216, 0.03213612, -0.019288804) * inp_7_0_2;
    result5 += M4(-0.050264858, -0.009159237, -0.13446109, -0.018111296, -0.41309023, -0.28588086, -0.14788406, -0.061952088, -0.03200147, 0.04391269, 0.023240728, -0.10571647, -0.11079374, 0.0081549585, 0.083741836, 0.031377733) * inp_7_1_2;
    result5 += M4(-0.24040218, -0.036332857, -0.16806267, -0.019890696, -0.103977, -0.08309535, 0.025292035, -0.046988808, -0.10393251, 0.02831244, -0.038792275, 0.02315635, 0.1380606, 0.009215899, -0.09240578, 0.0103388345) * inp_7_2_2;
    result6 += M4(-0.21957657, 0.02260168, 0.04084859, -0.007397026, -0.85985386, 0.011286408, 0.006953699, -0.027205557, -0.055374842, 0.08058203, 0.0057887104, -0.08548193, 0.10049568, 0.09112614, 0.054497544, 0.022654442) * inp_7_0_0;
    result6 += M4(-0.2490752, 0.036372196, 0.097053766, -0.040957425, 0.064196184, -0.022562513, -0.23970383, -0.15074764, 0.23395152, -0.030192599, -0.21903421, 0.06494475, 0.16699745, -0.04300912, 0.084191814, 0.07870796) * inp_7_1_0;
    result6 += M4(0.11111937, -0.00081526436, -0.027196024, -0.14221947, 0.073211536, 0.009635636, -0.21276033, -0.09407007, -0.06657562, 0.019889034, 0.10021066, -0.012405741, -0.17190264, 0.0067330934, 0.05091085, 0.048014894) * inp_7_2_0;
    result6 += M4(-0.4845586, -0.044372994, -0.0426583, 0.00026433764, 0.16342631, -0.059990246, 0.043995123, -0.009990055, 0.13572362, -0.034926336, 0.029572064, -0.013405462, -0.09710495, -0.033668283, -0.026853854, 0.001509953) * inp_7_0_1;
    result6 += M4(-0.3232986, -0.1375822, -0.019090483, -0.14520475, -0.28257218, -0.37013343, -0.05959359, -0.28005248, -0.26794755, 0.028461326, 0.27544314, 0.08487218, -0.02341637, 0.06801576, 0.04092385, -0.029959915) * inp_7_1_1;
    result6 += M4(-0.092469215, -0.05295939, -3.449105e-05, -0.12201405, 0.0061697494, 0.051163614, 0.023172926, -0.38215867, 0.15915576, 0.03521512, -0.1610684, -0.0043512085, -0.008754576, 0.027427554, -0.0032736147, -0.024468465) * inp_7_2_1;
    result6 += M4(-0.2249278, 0.025219278, -0.005792902, 0.027398532, -0.49709424, -0.16311894, -0.009293571, 0.009286756, 0.19434364, 0.0973109, -0.04395024, 0.002858404, -0.17783603, 0.028645404, 0.009057016, -0.018712334) * inp_7_0_2;
    result6 += M4(-0.3624977, -0.001685465, 0.023018925, -0.00034201, -0.15600505, -0.10098932, -0.060331207, -0.17791216, -0.29622853, 0.10954148, 0.016200326, -0.06623833, 0.3089193, 0.06020788, 0.01725866, -0.016012425) * inp_7_1_2;
    result6 += M4(0.15409826, -0.0017898841, -0.03496938, -0.0055928137, 0.07152, 0.0630523, -0.06365794, -0.04207143, 0.10070232, -0.004046987, 0.07017069, 0.046724077, -0.06491649, 0.0070210346, 0.0026242593, 0.0043371688) * inp_7_2_2;
    result7 += M4(0.0063494714, 0.030893061, 0.20738971, 0.009725964, -0.006238466, -0.018991247, -0.11535743, 0.03105094, 0.027448805, 0.005034717, -0.14260997, -0.02166966, -0.012111499, 0.07162522, 0.30808243, -0.023271156) * inp_7_0_0;
    result7 += M4(0.006528013, 0.045818917, 0.04327694, -0.022046652, -0.013184086, -0.004212293, -0.29531595, -0.21273133, 0.027405335, 0.022755422, -0.09129892, -0.076302685, -0.028868543, -0.019366952, -0.025072182, 0.054803476) * inp_7_1_0;
    result7 += M4(0.005486784, 0.028803518, -0.067935765, 0.0171914, -0.0037943108, 0.057264592, 0.09885854, -0.074120276, 0.0061029717, 0.03860528, -0.0439841, -0.03013773, 0.0048840283, -0.0070463405, -0.1019312, 0.01672855) * inp_7_2_0;
    result7 += M4(-0.048293386, -0.0059310794, 0.058479495, -0.016074115, -0.08597155, 0.015242765, -0.7294107, -0.08372895, -0.107319936, 0.053833477, -0.1877522, -0.07787382, 0.011401124, -0.07948725, -0.69820046, 0.015217523) * inp_7_0_1;
    result7 += M4(-0.037518654, -0.05710835, 0.2737387, 0.001820521, 0.0029282658, -0.374839, -0.27562177, -0.15981282, -0.058898415, 0.03226202, 0.15775973, 0.13906306, -0.007783065, 0.06655205, 0.14744824, 0.010021627) * inp_7_1_1;
    result7 += M4(-0.029149791, 0.028445054, -0.034745507, -0.022145817, -0.009788568, -0.19211297, -0.89718074, -0.19969675, -0.0004889386, -0.08408756, 0.09870253, 0.019107496, 0.0023063784, 0.048243333, 0.23284648, 0.01982245) * inp_7_2_1;
    result7 += M4(-0.01971595, 0.03605075, -0.009993605, 0.0004088743, -0.055799834, -0.02577352, -0.6042398, 0.011487422, 0.045998637, -0.05060031, -0.066881076, 0.024459615, -0.01766897, -0.0017198857, 0.09532179, -0.0066043725) * inp_7_0_2;
    result7 += M4(-0.009768666, 0.062479354, 0.13706839, -0.0032427788, 0.006405431, -0.030994287, -0.07858133, 0.05442072, -0.003907805, 0.121671036, 0.2213819, -0.00042118045, 0.010267511, -0.0025276972, 0.00702972, -0.032701667) * inp_7_1_2;
    result7 += M4(-0.015125362, 0.015370666, -0.034531247, -0.0141335605, -0.00044835484, -0.05034096, -0.11207093, 0.06368235, -0.017524939, 0.010628195, -0.0049342047, 0.019093288, 0.01404857, -0.013035478, -0.0022595918, -0.0154873775) * inp_7_2_2;
    const ivec2 output_base = ivec2(gl_GlobalInvocationID) * ivec2(4, 2);
    imageStore(out_image, output_base + ivec2(0, 0), max(result0, V4(0.0)));
    imageStore(out_image, output_base + ivec2(1, 0), max(result1, V4(0.0)));
    imageStore(out_image, output_base + ivec2(2, 0), max(result2, V4(0.0)));
    imageStore(out_image, output_base + ivec2(3, 0), max(result3, V4(0.0)));
    imageStore(out_image, output_base + ivec2(0, 1), max(result4, V4(0.0)));
    imageStore(out_image, output_base + ivec2(1, 1), max(result5, V4(0.0)));
    imageStore(out_image, output_base + ivec2(2, 1), max(result6, V4(0.0)));
    imageStore(out_image, output_base + ivec2(3, 1), max(result7, V4(0.0)));
}

//!DESC ArtCNN C4F32 DS (Conv2D-5)
//!COMPUTE 48 32 12 16
//!HOOK LUMA
//!BIND conv2d_4
//!SAVE conv2d_5
//!WIDTH LUMA.w 4.0 *
//!HEIGHT LUMA.h 2.0 *
//!COMPONENTS 4
//!WHEN OUTPUT.w LUMA.w / 1.3 > OUTPUT.h LUMA.h / 1.3 > *
#extension GL_EXT_shader_explicit_arithmetic_types_float16 : enable
#ifdef GL_EXT_shader_explicit_arithmetic_types_float16
#	define V4 f16vec4
#	define M4 f16mat4
#	define F float16_t
#else
#	define V4 vec4
#	define M4 mat4
#	define F float
#endif

const ivec2 ksize = ivec2(3, 3);
const ivec2 offset = ksize / 2;
const ivec2 wg_size = ivec2(gl_WorkGroupSize);
const ivec2 isize = wg_size + ksize - 1;
shared V4 inp[8][isize.y][isize.x];
void hook() {
    const uvec2 local_xy = gl_LocalInvocationID.xy;
    ivec2 base = ivec2(gl_WorkGroupID) * wg_size;
    for (uint y = local_xy.y; y < isize.y; y += wg_size.y) {
        for (uint x = local_xy.x; x < isize.x; x += wg_size.x) {
            const ivec2 input_base = (base + ivec2(x,y) - offset) * ivec2(4, 2);
            inp[0][y][x] = V4(conv2d_4_mul * texelFetch(conv2d_4_raw, input_base + ivec2(0, 0), 0));
            inp[1][y][x] = V4(conv2d_4_mul * texelFetch(conv2d_4_raw, input_base + ivec2(1, 0), 0));
            inp[2][y][x] = V4(conv2d_4_mul * texelFetch(conv2d_4_raw, input_base + ivec2(2, 0), 0));
            inp[3][y][x] = V4(conv2d_4_mul * texelFetch(conv2d_4_raw, input_base + ivec2(3, 0), 0));
            inp[4][y][x] = V4(conv2d_4_mul * texelFetch(conv2d_4_raw, input_base + ivec2(0, 1), 0));
            inp[5][y][x] = V4(conv2d_4_mul * texelFetch(conv2d_4_raw, input_base + ivec2(1, 1), 0));
            inp[6][y][x] = V4(conv2d_4_mul * texelFetch(conv2d_4_raw, input_base + ivec2(2, 1), 0));
            inp[7][y][x] = V4(conv2d_4_mul * texelFetch(conv2d_4_raw, input_base + ivec2(3, 1), 0));
        }
    }

    barrier();
    V4 result0 = V4(-0.00033857618, -5.211393e-05, -0.00046920136, -0.04654421);
    V4 result1 = V4(0.0027993727, -0.0014499743, 0.003381108, 0.002033252);
    V4 result2 = V4(-0.0063091437, -0.003208225, 0.0006721176, 0.0056411973);
    V4 result3 = V4(-0.0011719827, -0.033943694, -0.0014030745, -0.0015691749);
    V4 result4 = V4(0.0014802504, 0.0014747928, -0.0015503808, -0.000237806);
    V4 result5 = V4(-0.000551342, -0.00029516843, 0.0015785063, -0.0012769451);
    V4 result6 = V4(2.7722475e-05, 0.0014956064, 0.035652187, 0.0064491234);
    V4 result7 = V4(-0.00026030163, -0.03161675, -0.0033621937, 0.0010696175);
    const V4 inp_0_0_0 = inp[0][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_0_1_0 = inp[0][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_0_2_0 = inp[0][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_0_0_1 = inp[0][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_0_1_1 = inp[0][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_0_2_1 = inp[0][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_0_0_2 = inp[0][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_0_1_2 = inp[0][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_0_2_2 = inp[0][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.0055008903, 0.040040284, 0.03821417, -0.025478395, -0.09336844, -0.19005331, 0.0041426015, -0.06207188, -0.03533431, 0.09100611, -0.21562052, 0.05521167, 0.0148525, -0.01880247, -0.07190061, -0.070490435) * inp_0_0_0;
    result0 += M4(-0.06648988, 0.13202338, 0.014137371, 0.002976689, -0.15417841, -0.036936734, 0.051903993, 0.013108721, 0.12092494, 0.022438575, -0.03815395, 0.023259934, -0.072726, 0.24906468, -0.18974805, 0.16460207) * inp_0_1_0;
    result0 += M4(0.0125353895, -0.10785856, 0.050304852, 0.038287044, 0.0061141234, 0.0048369113, -0.036233388, -0.0509886, 0.026460862, 0.21335657, 0.010817808, -0.058360185, -0.109920286, 0.084420666, -0.045916367, 0.068335384) * inp_0_2_0;
    result0 += M4(-0.1607079, -0.05391135, 0.17744143, 0.022901597, -0.13761657, 0.07367111, -0.0014986272, -0.12867892, 0.01668362, -0.09370987, 0.08125384, 0.019830888, 0.054841943, 0.024319157, 0.09737227, 0.05723623) * inp_0_0_1;
    result0 += M4(0.09528175, -0.16803679, 0.026742999, 0.0103471745, -0.2244919, 0.111276686, -0.2019129, 0.3596232, -0.005050413, 0.09969625, 0.06993515, -0.23216967, -0.2364243, 0.075453386, 0.1808636, -0.16794966) * inp_0_1_1;
    result0 += M4(0.04436569, 0.04602664, -0.07125175, 0.023568979, -0.014757421, 0.024136547, -0.016083388, 0.06678989, -0.03417033, 0.07393436, -0.08875034, 0.0038435166, -0.092170835, 0.08144975, -0.0996393, 0.0838242) * inp_0_2_1;
    result0 += M4(0.1152554, 0.06888826, -0.08245708, 0.038977798, 0.13445279, 0.19078267, -0.08225348, 0.0017612935, 0.008786101, -0.09720064, -0.08100206, 0.028934252, -0.024967859, 0.027158182, -0.03962079, -0.0025172734) * inp_0_0_2;
    result0 += M4(0.004256607, 0.10033306, 0.106189825, -0.056177154, -0.09165286, 0.08794716, -0.13838723, -0.036856197, 0.16221692, -0.16571492, 0.05173908, -0.040524326, 0.112713166, -0.16705117, 0.010272158, -0.13570237) * inp_0_1_2;
    result0 += M4(0.0009024955, 0.037982672, 0.0150674, 0.039002396, 0.027338922, 0.02292527, 0.0099601075, 0.012461556, 0.13556844, -0.12963817, 0.20211858, -0.097479045, 0.07684909, -0.15500484, 0.08432899, 0.10545347) * inp_0_2_2;
    result1 += M4(-0.13945793, -0.031421147, 0.058424983, 0.13356897, -0.14354034, -0.022662189, 0.13210016, -0.046321996, 0.22991648, -0.014257525, -0.07331968, -0.04960742, 0.17871836, -0.012267222, 0.020015914, -0.03466255) * inp_0_0_0;
    result1 += M4(0.14785966, -0.11845738, -0.012382634, -0.06398047, -0.21350554, 0.011480587, 0.018944198, -0.047925662, 0.015893184, 0.046179336, 0.043034818, -0.013914095, 0.35578358, 0.030437624, -0.06385517, 0.10684798) * inp_0_1_0;
    result1 += M4(-0.15164743, 0.010013076, -0.12866935, 0.017407395, -0.019674592, -0.03330712, 0.017320154, -0.15847497, 0.3212293, -0.06625159, 0.09453292, 0.04359272, -0.04829651, -0.058096375, -0.11817275, -0.0015015651) * inp_0_2_0;
    result1 += M4(0.035237744, -0.0061727352, -0.016305124, 0.006250792, -0.007786182, -0.16131032, -0.08144286, -0.04523907, 0.08686253, 0.0656864, 0.016857903, -0.082258135, -0.045420714, 0.054513894, 0.028904388, 0.033936556) * inp_0_0_1;
    result1 += M4(-0.020308923, 0.15573776, 0.15191197, -0.011976431, -0.2090145, -0.24527687, 0.4135083, 0.04819112, -0.0651052, -0.08085929, 0.08943662, 0.10672608, -0.09993008, 0.06420894, 0.0024723897, -0.038494125) * inp_0_1_1;
    result1 += M4(0.26566312, -0.041133165, -0.08233638, -0.044740815, -0.08911803, 0.0552542, 0.06473897, 0.05723115, -0.05815753, 0.16166504, -0.10840947, -0.019410508, -0.048973553, -0.014839897, 0.08466536, 0.15185966) * inp_0_2_1;
    result1 += M4(0.054321043, 0.067332834, 0.0077196895, -0.05037081, -0.19387913, 0.039284933, 0.04562723, -0.04286526, -0.042993493, -0.0471203, -0.020111596, 0.28844228, -0.12654372, -0.028620189, 0.14294736, -0.056952067) * inp_0_0_2;
    result1 += M4(-0.08932884, -0.036871273, 0.020192133, 0.00128216, -0.10099248, 0.11201541, 0.13712461, -0.014323802, 0.117175356, 0.13990857, -0.061360285, -0.027479066, -0.22863752, -0.1223995, 0.12916198, -0.0034384197) * inp_0_1_2;
    result1 += M4(-0.061732933, 0.024225527, 0.23375747, -0.06904772, 0.029857783, 0.0036627979, 0.024528049, -0.026710648, 0.16729724, -0.0062084515, -0.21606815, 0.02139915, 0.0011679479, 0.021964999, 0.08592029, -0.087391384) * inp_0_2_2;
    result2 += M4(0.0001689916, 0.13425444, -0.14913946, 0.10494884, 0.05037515, 0.029287072, -0.01576243, 0.029871952, -0.113786794, -0.18922783, 0.07768059, -0.09693995, 0.17987432, 0.044651266, -0.041353982, 0.26805663) * inp_0_0_0;
    result2 += M4(0.16857997, 0.084800825, 0.0915486, 0.14004445, -0.01864318, -0.03173774, 0.03218675, -0.03742809, -0.052868664, 0.0027378227, 0.011767111, -0.056010798, -0.18017766, 0.22932, -0.1959998, 0.40383193) * inp_0_1_0;
    result2 += M4(0.09521529, -0.081043795, 0.06612548, 0.13360025, 0.0791742, 0.045078903, 0.010410423, -0.08329159, -0.18936586, 0.04514708, -0.027012305, -0.15414499, 0.19884205, -0.028052315, -0.052206345, 0.2673888) * inp_0_2_0;
    result2 += M4(0.020038571, 0.08377264, -0.0059735286, 0.03005524, 0.08429772, 0.10145283, 0.12034709, -0.33877838, -0.112059645, -0.1543233, -0.021589277, -0.14347444, 0.07406417, 0.07946413, -0.07384155, 0.05627196) * inp_0_0_1;
    result2 += M4(-0.037296973, 0.18684307, -0.22782172, 0.59133446, -0.18724981, -0.25357243, -0.14497101, -0.025236132, -0.09228171, -0.40975732, -0.02948288, -0.8100769, -0.05143526, 0.16157964, -0.054445364, 0.41785368) * inp_0_1_1;
    result2 += M4(0.02490756, 0.028316107, -0.0718174, 0.034487803, -0.07977564, -0.09640628, 0.019067757, -0.08654339, -0.073026955, -0.0013935091, -0.014127905, -0.16971977, -0.046018537, -0.07550468, -0.007683933, 0.043455623) * inp_0_2_1;
    result2 += M4(-0.050713804, 0.05200905, -0.013416636, 0.15574546, -0.09022452, 0.063800655, -0.04092265, -0.081436396, 0.054089632, -0.1394468, 0.00083400717, -0.19350049, -0.049108233, 0.07916948, -0.017489465, 0.07420918) * inp_0_0_2;
    result2 += M4(0.12828913, 0.08871344, 0.046650425, 0.23765475, -0.05984087, -0.046504088, -0.104585245, -0.07054799, -0.028161684, 0.084795274, 0.021476893, -0.13975838, 0.45168048, -0.016797554, 0.31043333, 0.45872617) * inp_0_1_2;
    result2 += M4(0.018843658, -0.031844802, 0.053112507, 0.22836947, 0.0009473444, 0.060992558, -0.007834259, -0.083464734, 0.123688996, 0.020567061, 0.07470798, -0.07484856, -0.036755264, 0.06131316, 0.12439284, 0.2406411) * inp_0_2_2;
    result3 += M4(0.010637446, 0.08122302, -0.019096095, 0.047429033, 0.10295111, -0.057108458, 0.051800206, -0.09622697, -0.110481866, -0.02986421, 0.15930514, -0.053985056, -0.014512489, 0.060827628, -0.16927683, -0.0016388934) * inp_0_0_0;
    result3 += M4(0.043081682, 0.019654011, -0.07826177, 0.11452588, 0.01971762, -0.0071579376, 0.23394085, -0.11074407, -0.117683664, -0.027853398, -0.04529608, 0.030902887, 0.10984874, 0.02655838, -0.14568935, 0.1176308) * inp_0_1_0;
    result3 += M4(-0.032719, -0.003899051, 0.12977327, 0.005213001, 0.19031972, 0.016623251, -0.043424, 0.018945998, 0.034317106, -0.018435407, -0.14291012, 0.08580988, -0.058565613, -0.013042563, -0.03564731, -0.014366938) * inp_0_2_0;
    result3 += M4(0.067188665, -0.037597932, 0.118612364, -0.14210613, 0.30456132, -0.006149907, 0.14681987, -0.095257685, 0.09503866, -0.036499668, -0.039651684, -0.060061008, -0.13800874, 0.04697595, 0.14861009, -0.032573536) * inp_0_0_1;
    result3 += M4(-0.13532096, 0.019680874, 0.065936804, 0.2148044, 0.4126885, 0.10177626, -0.15395397, 0.28993788, -0.022751521, -0.04908474, 0.049986634, 0.07207853, -0.05573347, -0.006349617, 0.013991933, -0.046055276) * inp_0_1_1;
    result3 += M4(0.06691425, -0.017459981, -0.20049527, 0.12047687, -0.030848352, 0.03305165, 0.056996267, -0.014725983, 0.10794889, 0.053853672, -0.063334696, -0.108595416, -0.11766006, -0.025964156, 0.06843098, 0.0075015035) * inp_0_2_1;
    result3 += M4(-0.13300757, 0.058681577, -0.16092683, 0.07958526, 0.03625983, -0.06859724, -0.07384222, 0.1839895, -0.14609885, -0.017700635, 0.1626891, -0.08824729, -0.07492295, 0.026738742, 0.095774725, -0.08439779) * inp_0_0_2;
    result3 += M4(-0.06047365, -0.029309483, 0.027905019, -0.12892249, -0.047943015, -0.062774755, 0.027742205, -0.056321442, -0.12687564, 0.096873604, 0.051679395, -0.06945621, 0.067893945, -0.012911296, 0.17691594, -0.42516413) * inp_0_1_2;
    result3 += M4(-0.142951, -0.0789655, 0.11082419, -0.115432784, 0.09941218, 0.05737683, -0.041983347, 0.0026551129, 0.13242412, 0.03465745, 0.056379084, -0.014236554, -0.13590129, -0.10400298, 0.08489601, -0.16156541) * inp_0_2_2;
    result4 += M4(0.07946611, -0.08810808, -0.037494518, -0.018036272, -0.10239115, -0.03422281, 0.2976986, -0.0063930065, 0.008926732, 0.062025674, -0.1328528, 0.11211853, 0.05645577, 0.22442925, -0.21878129, -0.028210795) * inp_0_0_0;
    result4 += M4(-0.31619015, 0.015856063, 0.047678404, 0.12965232, 0.14411479, -0.012104081, -0.01205837, -0.0064168507, 0.07872266, 0.1232382, -0.14530195, -0.02493601, -0.24723399, -0.14511658, -0.007034702, 0.52052313) * inp_0_1_0;
    result4 += M4(0.013744939, 0.046401337, 0.027801223, -0.048134726, -0.022361498, -0.10134498, -0.048769273, -0.07630715, 0.054323196, 0.027330521, -0.12419663, 0.14270769, -0.13269983, 0.09647564, -0.007998369, 0.13477814) * inp_0_2_0;
    result4 += M4(0.041459005, 0.09478397, 0.050986633, 0.058322255, -0.008559293, 0.09354447, 0.098972045, -0.07751526, -0.07286118, -0.058386564, -0.04602933, -0.034306467, -0.16708785, -0.041280538, 0.11783937, 0.104105264) * inp_0_0_1;
    result4 += M4(0.55840594, 0.07197267, -0.015520731, -0.19044724, -0.2854422, 0.026131295, -0.12815309, -0.15732098, -0.2506088, -0.13136332, -0.00794052, 0.22846168, 0.33262902, -0.012272023, 0.10581234, -0.012546424) * inp_0_1_1;
    result4 += M4(0.073102735, -0.07661012, -0.023832794, 0.11653179, 0.035942573, 0.07983359, -0.016712526, 0.035339754, -0.21198012, 0.02860666, 0.07182768, 0.06967221, 0.02876816, -0.13938835, -0.026907066, 0.027632328) * inp_0_2_1;
    result4 += M4(-0.011455213, -0.005727743, 0.039152816, -0.061022323, -0.03742619, -0.22659154, -0.041192085, 0.05435202, -0.13963936, 0.04238641, 0.20046064, 0.15223168, -0.032706287, -0.120154746, 0.040191863, -0.0822429) * inp_0_0_2;
    result4 += M4(-0.22490734, -0.091488376, -0.049039673, 0.15828055, 0.2138342, -0.065546446, 0.0027719352, -0.019293124, 0.12412302, 0.062152885, 0.21304351, -0.10996941, -0.3631115, 0.22981869, -0.07905039, -0.20793441) * inp_0_1_2;
    result4 += M4(-0.12442689, 0.09817936, 0.08632807, -0.12684083, 0.014537491, -0.012443994, 0.026914144, 0.005441795, 0.17579606, 0.047339164, -0.0064497422, 0.009215208, -0.12870379, 0.012891812, 0.06326978, -0.009157409) * inp_0_2_2;
    result5 += M4(-0.08083854, -0.039649226, 0.025587806, -0.0022639206, 0.037355367, 0.12955017, -0.099845916, 0.026468385, 0.22832048, 0.025300706, -0.041735224, -0.015993282, 0.05743137, -0.12377402, 0.0028058626, -0.14376421) * inp_0_0_0;
    result5 += M4(-0.068195954, 0.032814953, -0.05617883, -0.039529122, 0.04395742, -0.1654701, -0.14468645, 0.060611114, 0.14325377, -0.004798819, -0.011921389, -0.1081723, -0.27493536, -0.071100324, 0.023415534, 0.06090124) * inp_0_1_0;
    result5 += M4(-0.03914808, -0.070434295, 0.06770984, -0.047501974, 0.0569247, 0.07515299, 0.0086178975, 0.078467295, -0.09694434, 0.02423476, 0.017578192, 0.14268064, 0.020323545, -0.12080692, 0.115722865, 0.1331762) * inp_0_2_0;
    result5 += M4(-0.023336824, -0.011723584, 0.092374586, 0.0024106086, 0.01960992, -0.04250571, -0.084592536, 0.0005539204, -0.05781805, -0.02344869, 0.063145205, -0.03281951, 0.07265283, -0.07864308, -0.07356117, 0.04270495) * inp_0_0_1;
    result5 += M4(0.019285986, -0.0905782, 0.14888233, 0.18408167, 0.04944697, 0.2121639, 0.16386084, 0.022062365, -0.068983756, 0.12300373, 0.1493408, 0.07770018, -0.049842708, -0.014165909, -0.007032601, 0.20252082) * inp_0_1_1;
    result5 += M4(0.01912835, 0.042419817, -0.07206427, 0.059513763, -0.024394296, 0.056551762, -0.034608487, -0.00612085, 0.026618112, 0.1266841, 0.22224109, -0.033696514, 0.054842282, 0.055901617, -0.20016406, -0.058923457) * inp_0_2_1;
    result5 += M4(-0.03408175, 0.112561494, -0.061991837, -0.073797986, 0.007059498, 0.022556465, -0.050500143, 0.006241442, -0.0333419, 0.0286376, -0.046588276, 0.06361652, -0.08222098, 0.15489368, -0.14433159, -0.0044032945) * inp_0_0_2;
    result5 += M4(-0.023731587, -0.050678678, -0.24115007, -0.057736788, 0.14450802, -0.14987263, 0.14091256, 0.10036237, -0.056601472, -0.15323421, -0.06795621, -0.07708453, 0.37814432, -0.0151232565, -0.122136876, -0.30736414) * inp_0_1_2;
    result5 += M4(-0.06704461, -0.0063380045, 0.12926394, 0.046971995, 0.06588133, -0.0016408568, -0.018893411, -0.026099065, 0.055257995, -0.13353862, -0.08175561, -0.059331574, -0.054767583, -0.021483652, -0.027032321, -0.028508449) * inp_0_2_2;
    result6 += M4(0.01425053, -0.066083536, -0.036516074, -0.09471972, -0.01917465, 0.08822432, 0.0054744007, 0.00077392196, 0.09598642, 0.07268489, -0.009358444, 0.13574734, 0.26164466, 0.2056832, -0.02143377, -0.22798204) * inp_0_0_0;
    result6 += M4(0.020088753, -0.0032444014, -0.10021951, -0.028943067, -0.015441297, -0.04786329, 0.13857308, 0.0153693035, -0.09176709, -0.10377119, -0.004214386, 0.08277664, 0.15718272, -0.029264322, -0.23311013, 0.32333535) * inp_0_1_0;
    result6 += M4(-0.014242281, -0.006784125, 0.02495672, -0.08600399, 0.05721316, 0.013770397, -0.009889364, -0.017172905, -0.06616058, 0.004448295, 0.0057181804, 0.055790193, -0.033669945, -0.08074925, 0.0009131132, -0.083347626) * inp_0_2_0;
    result6 += M4(-0.18327117, 0.048898723, 0.056775946, -0.16974813, 0.09243583, 0.20578541, 0.12283505, 0.0916725, 0.00037371792, -0.026782166, -0.017420102, 0.07591624, 0.060373317, -0.040889155, -0.009145755, -0.17541145) * inp_0_0_1;
    result6 += M4(0.05597793, 0.12249189, -0.15861003, -0.22713469, -0.15099624, 0.24619852, 0.26635623, 0.340753, -0.093362994, -0.09242913, 0.13126941, 0.051121436, 0.0940384, 0.047092125, -0.16073695, -0.30952117) * inp_0_1_1;
    result6 += M4(-0.08482115, 0.1552472, 0.06704411, -0.05003729, 0.031578347, -0.027218739, 0.009955349, 0.04267379, 0.12975028, -0.08699395, -0.031255, 0.10593689, 0.116384186, 0.16735807, 0.07591615, 0.029843358) * inp_0_2_1;
    result6 += M4(0.08991798, -0.026941206, -0.10918902, 0.09491319, 0.047999892, -0.18327224, -0.02882669, 0.25335747, -0.096923605, -0.18606159, 0.08949498, -0.09901268, 0.03860873, -0.052575983, -0.06533276, 0.112260096) * inp_0_0_2;
    result6 += M4(0.05932772, 0.013044016, -0.078759156, -0.10447518, 0.10649924, -0.06282437, 0.09376782, 0.17171042, 0.087686285, -0.13624698, 0.036514968, -0.12604444, -0.01590929, 0.0981113, -0.009037743, -0.35171157) * inp_0_1_2;
    result6 += M4(0.06050632, -0.10242034, -0.017536448, 0.16742243, -0.0074237105, 0.046862736, -0.05776824, -0.022105247, 0.045347773, -0.092322394, 0.014679722, -0.19808187, -0.011551481, -0.19418558, -0.012217746, 0.15039246) * inp_0_2_2;
    result7 += M4(0.14684616, -0.029255051, -0.15527017, 0.00861559, 0.029546928, 0.003194165, -0.16437432, -0.019029837, -0.117811695, 0.0380705, 0.13244219, -0.13763173, 0.16435699, 0.006976756, -0.13059309, 0.2650596) * inp_0_0_0;
    result7 += M4(0.117154755, -0.061386127, -0.015056915, -0.08387719, -0.093112126, 0.0495995, 0.1122886, 0.14690968, -0.074060336, -0.044724382, 0.115512475, -0.06485729, 0.23345496, -0.17779376, -0.28721744, 0.2944409) * inp_0_1_0;
    result7 += M4(0.009200014, -0.0016826433, -0.029547809, -0.044900782, -0.009306141, 0.009256042, -0.05660038, -0.003180737, 0.022846641, 0.038223352, -0.044420887, 0.032508902, 0.19245386, -0.078919314, -0.11645058, 0.04067542) * inp_0_2_0;
    result7 += M4(-0.19690105, 0.028848086, 0.047259156, -0.024896339, -0.07390086, -0.017633632, 0.09685313, -0.013801847, -0.027607383, -0.019397747, 0.09385051, -0.0610609, 0.031614963, -0.027679043, -0.06561788, -0.032778267) * inp_0_0_1;
    result7 += M4(-0.23555553, -0.27432325, -0.48214766, -0.095824264, -0.46281072, 0.103836864, -0.08512496, 0.049324796, -0.034777597, 0.4015953, 0.6456607, 0.09709889, -0.19845897, -0.24093357, -0.40266168, -0.20628609) * inp_0_1_1;
    result7 += M4(-0.05924828, -0.0688816, 0.01347782, -0.017761623, 0.009259878, 0.09404022, -0.09576252, 0.023180272, 0.10217725, 0.053873513, 0.015351408, 0.022545667, 0.02794716, -0.08300157, -0.061708894, 0.060976714) * inp_0_2_1;
    result7 += M4(0.044441983, -0.040408324, -0.014086003, 0.071962275, 0.2136606, -0.051369544, 0.00027544732, 0.18606934, -0.08679569, 0.077435136, 0.040705185, -0.013840602, 0.13151778, -0.033223905, 0.031148056, 0.06811249) * inp_0_0_2;
    result7 += M4(0.1567668, -0.11909305, -0.14607765, 0.09982093, 0.017231664, 0.038438715, 0.047035974, -0.015216987, 0.02002179, 0.08256783, 0.24045995, -0.073004335, 0.18223229, -0.039312623, -0.15983072, 0.19523779) * inp_0_1_2;
    result7 += M4(-0.13495843, -0.08332125, -0.13689105, -0.0068533407, 0.03927797, -0.0035840173, -0.016581707, -0.010625582, 0.11280562, 0.069906205, 0.1431308, -0.122072205, -0.100774094, -0.12048784, -0.17714357, 0.17013277) * inp_0_2_2;
    const V4 inp_1_0_0 = inp[1][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_1_1_0 = inp[1][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_1_2_0 = inp[1][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_1_0_1 = inp[1][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_1_1_1 = inp[1][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_1_2_1 = inp[1][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_1_0_2 = inp[1][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_1_1_2 = inp[1][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_1_2_2 = inp[1][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.012059773, 0.05716743, -0.078864895, -0.0016451637, -0.038298003, 0.07146095, -0.084323205, -0.005114981, 0.05743578, 0.03972149, -0.0011730526, -0.008434305, 0.012674199, 0.023245703, 0.07544545, 0.040041696) * inp_1_0_0;
    result0 += M4(0.017043397, -0.023217566, -0.029793337, 0.006432389, -0.010647316, 0.0236554, -0.018099383, 0.01977082, -0.044221293, -0.078467175, -0.05519627, -0.0330085, -0.08948057, -0.11937098, 0.0741295, 0.008016856) * inp_1_1_0;
    result0 += M4(-0.006243105, -0.059244655, 0.058311544, -0.024302134, 0.0769363, -0.1034658, 0.09595889, 0.028867826, -0.0003642194, 0.10800453, 0.08599967, -0.019489119, -0.029602684, 0.29575205, -0.1472287, 0.07809142) * inp_1_2_0;
    result0 += M4(0.004803237, -0.08810946, 0.044457607, 0.06871357, 0.119693935, 0.1363349, -0.028868204, -0.07188803, 0.12151866, -0.03556942, 0.21142213, 0.160273, 0.007004508, 0.10160139, 0.07276651, 0.0051063257) * inp_1_0_1;
    result0 += M4(-0.043380808, 0.0426722, 0.05261468, 0.041584764, -0.095519066, -0.14382896, 0.1594156, -0.17915004, -0.05451941, 0.043272365, -0.0440294, -0.008667617, 0.3775092, -0.044392616, 0.47569, 0.15976448) * inp_1_1_1;
    result0 += M4(0.096929766, 0.07771875, 0.05050513, 0.016342288, -0.05379303, 0.03163601, -0.0477634, -0.0680153, -0.010155337, 0.008810118, -0.03868616, 0.09654413, 0.061329838, -0.11847393, -0.03285462, -0.05487639) * inp_1_2_1;
    result0 += M4(0.02376415, -0.019311389, 0.10640152, 0.018369133, -0.16605675, 0.025131362, -0.74221766, 0.046217572, 0.53277826, 0.25047415, -0.5549857, 0.04798833, -0.119201764, -0.024885759, -0.07373259, -0.015902294) * inp_1_0_2;
    result0 += M4(-0.0048952466, -0.09510981, 0.12714678, -0.07892796, -0.37628353, -0.061140966, -1.0103354, 0.25518143, -0.3751466, -0.030401053, 0.08248349, 0.20273587, -0.116148844, -0.129168, -0.109951116, -0.09977741) * inp_1_1_2;
    result0 += M4(0.0122273015, -0.03819175, -0.3058438, -0.16344221, -0.06238269, 0.120535605, -0.18198031, 0.113099374, -0.09720687, 0.20368741, 0.2635339, 0.1326661, -0.064674914, -0.10952229, -0.055259336, 0.022077031) * inp_1_2_2;
    result1 += M4(-0.027812919, -0.03727152, -0.03346944, 0.025632704, 0.046239678, 0.03292401, -0.06687773, 0.0468592, 0.087313496, 0.0047171004, -0.066393405, 0.033749964, -0.117906116, 0.06676515, 0.048526052, -0.15590584) * inp_1_0_0;
    result1 += M4(-0.015439315, -0.0022972918, -0.12710708, 0.009706818, -0.06270437, 0.089781016, 0.12650855, -0.023953138, 0.029180981, 0.07955586, 0.17985816, -0.2085678, 0.38085765, -0.032514732, 0.28935075, 0.032957193) * inp_1_1_0;
    result1 += M4(-0.015678287, -0.051428735, -0.05039185, 0.102829255, -0.025314827, -0.13608277, -0.04243956, -0.040526673, 0.064878695, 0.026294062, -0.03453546, 0.14471084, -0.44960737, -0.64543504, -0.009323038, -0.20842369) * inp_1_2_0;
    result1 += M4(0.032294754, 0.040051874, -0.021913871, -0.007082222, -0.40120402, -0.05179589, -0.22602852, 0.038738474, 0.29776508, 0.059017017, 0.013607145, 0.040163193, 0.005523097, 0.16581048, -0.03953907, -0.13949932) * inp_1_0_1;
    result1 += M4(0.39112958, 0.019914271, 0.10255176, -0.1411907, 0.5202728, 0.43640533, 0.21748663, 0.12450153, 0.02010695, 0.09049246, 0.033095337, -0.10409151, 0.33730525, 0.43532255, -0.075323984, -0.06729257) * inp_1_1_1;
    result1 += M4(-0.26738295, 0.13976943, -0.01965471, -0.028478468, -0.12834638, -0.3320292, 0.062244814, -0.16792396, -0.0739774, 0.04903803, -0.12660007, 0.08824605, -0.10712735, -0.3738959, -0.17587818, 0.14833337) * inp_1_2_1;
    result1 += M4(0.00795786, -0.003796244, 0.02142866, 0.0724671, -0.124062456, -0.006484297, -0.20458584, -0.28118905, -0.25011638, -0.0805813, -0.13472721, 0.068577215, -0.005556602, -0.11961111, 0.019058442, -0.003008943) * inp_1_0_2;
    result1 += M4(-0.17573124, -0.25926527, 0.046443988, -0.0046678646, -0.2540151, 0.4008228, -0.463303, -0.003869083, 0.1671688, 0.2169858, 0.07567604, -0.19003591, 0.44273102, -0.15517478, 0.06416943, -0.16173203) * inp_1_1_2;
    result1 += M4(0.034276437, -0.08049083, 0.1472962, 0.1863214, -0.008537238, 0.095983036, -0.69005644, -0.024002606, 0.07883799, 0.031977177, -0.11961545, 0.21520156, -0.10713608, 0.0064501525, 0.24662067, -0.045330927) * inp_1_2_2;
    result2 += M4(0.09535293, -0.06034477, -0.017754931, 0.031144576, -0.0075969785, -0.0006555495, -0.068856634, 0.016557455, -0.083619155, -0.009452033, 0.043700986, -0.025881292, -0.035429925, 0.022691123, 0.0032745698, -0.07626702) * inp_1_0_0;
    result2 += M4(0.13971524, 0.08654631, -0.039306957, -0.03312064, 0.027857214, -0.04806061, 0.06707722, -0.04708635, -0.047795128, 0.07269561, 0.017797261, -0.068356276, 0.11346859, -0.13044348, -0.031279538, -0.053305686) * inp_1_1_0;
    result2 += M4(0.072185494, 0.005629806, 0.022027154, 0.012991186, 0.021257956, 0.021199994, 0.05987223, 0.01315667, -0.17931311, -0.050805423, 0.009831274, -0.04743878, -0.051851884, 0.04131325, -0.094160475, 0.020642813) * inp_1_2_0;
    result2 += M4(-0.010202901, 0.02530993, -0.04145763, 0.11436319, 0.034525365, -0.039091866, -0.20642157, -0.16300228, -0.09449638, 0.069656156, 0.09505493, 0.02288177, -0.085687555, -0.021164909, 0.03344298, -0.07494722) * inp_1_0_1;
    result2 += M4(0.05166634, -0.20340566, 0.26609978, -0.02019657, -0.039226346, -0.03337542, -0.07358468, -0.111025564, -0.23232327, -0.11503773, -0.02512665, -0.03902946, 0.02380491, 0.3275542, 0.062712684, 0.09950738) * inp_1_1_1;
    result2 += M4(0.035684336, -0.081037834, -0.22590415, 0.030550614, 0.059104055, 0.09326073, 0.24844447, 0.082507744, 0.031739395, 0.16247411, -0.14110155, 0.0807007, -0.07282667, 0.15846302, -0.20329396, 0.18012315) * inp_1_2_1;
    result2 += M4(-0.04701629, 0.03250838, 0.061830036, 0.13005663, -0.041730877, -0.046456207, -0.13225852, -0.049698602, -0.054359984, -0.05841956, -0.14052893, 0.11531643, 0.05387512, -0.05276956, -0.056805097, -0.088512324) * inp_1_0_2;
    result2 += M4(0.123154834, -0.09064338, 0.1852385, 0.05733561, -0.031412385, -0.12941234, -0.34790686, -0.42584348, -0.2828954, 0.22206312, -0.05454853, 0.13965127, -0.06472564, 0.072554484, 0.13653709, 0.086538516) * inp_1_1_2;
    result2 += M4(0.12583914, -0.23166081, -0.0839096, 0.1464046, -0.28002983, 0.2213012, -0.37287596, -0.37040117, -0.060711976, 0.03199641, 0.07316787, 0.043666027, 0.0323333, -0.05158499, 0.037219763, 0.07891403) * inp_1_2_2;
    result3 += M4(-0.0010114687, 0.090104654, -0.048400793, -0.013697193, 0.020841962, -0.008555631, 0.07079522, -0.23207042, -0.07165372, -0.036108255, 0.029477378, -0.06382695, 0.097774945, 0.01673049, -0.009196186, 0.11895675) * inp_1_0_0;
    result3 += M4(0.14247727, 0.064516276, -0.054380056, 0.009918445, -0.08682447, 0.08164449, -0.15639955, 0.17410539, 0.09282515, -0.065459475, -0.08557291, 0.12347635, 0.06673487, -0.16030073, 0.26376262, -0.26081628) * inp_1_1_0;
    result3 += M4(0.037656598, 0.051937614, -0.06318192, 0.011859172, 0.1108276, -0.029432949, 0.13574977, 0.030473797, -0.2074061, -0.028325573, 0.13581394, -0.096615456, -0.021653038, 0.110804126, 0.14690435, 0.13597941) * inp_1_2_0;
    result3 += M4(-0.16340753, -0.07175192, 0.046707522, -0.103000484, -0.105200455, 0.045546915, 0.026525773, 0.021252692, -0.25527197, 0.0056057787, -0.10250607, 0.11214476, 0.055933524, 0.031233963, -0.13449547, 0.14835888) * inp_1_0_1;
    result3 += M4(0.075966045, -0.28426722, -0.06569272, -0.012672187, 0.048041716, 0.08698268, 0.0025115819, 0.0014672922, -0.008680538, -0.079891734, 0.11429251, -0.16004582, 0.38773283, 0.17664795, -0.15115115, -0.28403497) * inp_1_1_1;
    result3 += M4(-0.2557369, 0.053671375, 0.13331819, -0.029444179, 0.06165634, 0.059978563, -0.08554058, -0.02212329, -0.34705564, -0.008839001, -0.0029214607, -0.014748565, 0.4742814, -0.21338736, 0.043938294, 0.39484996) * inp_1_2_1;
    result3 += M4(0.026144825, 0.08404073, 0.10983975, 0.086493276, -0.64324796, -0.07772704, -0.17528819, 0.394255, -0.1329625, 0.10091636, -0.30293345, -0.042958472, 0.034031715, -0.017776554, 0.11535831, 0.122516826) * inp_1_0_2;
    result3 += M4(-0.010937271, -0.08652104, 0.21669084, -0.08526272, -0.6829697, -0.22820544, 0.048099555, 0.007229182, -0.15370141, 0.009962227, 0.20458816, -0.10500798, 0.20670193, 0.06527794, 0.08205848, -0.20301081) * inp_1_1_2;
    result3 += M4(-0.031794775, 0.09259995, -0.21529353, 0.09867467, -0.57176954, 0.053420357, 0.2345717, 0.20783865, -0.51649654, 0.092682816, 0.14416458, -0.040351205, 0.39748433, 0.028762514, 0.1401173, -0.20980519) * inp_1_2_2;
    result4 += M4(0.059017073, 0.061972745, -0.01839773, -0.031122537, 0.10679584, 0.024163269, 0.12119215, 0.17703925, 0.06590449, -0.0563125, -0.06998603, -0.012719758, -0.09352851, -0.15253572, -0.077775106, 0.04044123) * inp_1_0_0;
    result4 += M4(-0.06973423, -0.08242985, -0.022739973, 0.04381929, -0.024059743, 0.061810873, -0.07965713, 0.046277802, -0.084137924, 0.09339245, 0.015797589, -0.073477864, 0.2525248, -0.009107878, 0.13558583, 0.27625665) * inp_1_1_0;
    result4 += M4(0.029347757, 0.0036350635, 0.03840104, -0.0035045343, -0.054932863, -0.07692286, -0.0012521505, -0.15993053, 0.096534505, -0.039552514, -0.005358732, -0.055923406, 0.0073634814, 0.5167613, 0.013705958, -0.26164812) * inp_1_2_0;
    result4 += M4(0.0007580405, -0.037925523, -0.0011118382, 0.032609347, -0.042403497, 0.10385308, -0.15674022, -0.251819, -0.13433194, -0.107644334, 0.18344425, -0.095280096, -0.05091677, -0.10755968, -0.20214444, 0.10812456) * inp_1_0_1;
    result4 += M4(-0.36975265, 0.10834255, -0.08003177, 0.05101498, -0.06384099, -0.3914846, 0.00041025475, 0.36220244, 0.3686714, -0.08634353, -0.15973052, -0.019790925, -0.5464487, 0.12254814, -0.3883457, 0.36614713) * inp_1_1_1;
    result4 += M4(-0.16934372, -0.15841691, -0.044950306, -0.18618119, 0.15516856, 0.24614608, 0.11173066, -0.03309139, 0.1006585, 0.15881272, 0.12866277, 0.08817601, -0.15096901, -0.21967463, 0.23691766, -0.028090958) * inp_1_2_1;
    result4 += M4(0.05310273, 0.043171402, 0.086372286, 0.009141314, 0.042094085, -0.3440094, 0.39085177, -0.4831404, 0.18241194, 0.0013146275, -0.25416282, 0.11490463, 0.14287579, 0.004375975, 0.0825925, 0.12463079) * inp_1_0_2;
    result4 += M4(0.4814734, -0.148054, 0.013752365, 0.18009695, -0.3585172, -0.146239, 1.0368989, -1.0555606, -0.06215093, -0.12760828, -0.058454614, -0.03442635, -0.059379973, 0.02456827, -0.1493426, 0.12463049) * inp_1_1_2;
    result4 += M4(-0.1138796, 0.31484425, -0.08298672, 0.15247667, 0.3329841, -0.23497023, 0.20661953, -0.2322985, -0.013297015, 0.086119704, 0.15395626, -0.105612405, 0.021572778, 0.10009072, 0.20034966, -0.08513417) * inp_1_2_2;
    result5 += M4(-0.00038169743, -0.06450465, 0.013206497, -0.088094406, 0.027484467, 0.07201945, 0.015074406, -0.09933047, -0.073428705, 0.025415074, -0.052788083, -0.006651866, -0.059244893, -0.25892556, 0.020114213, 0.060483787) * inp_1_0_0;
    result5 += M4(0.00689536, 0.069345675, -0.025189467, -0.06741825, -0.039434053, -0.13454212, 0.021830155, 0.03483645, 0.047613323, 0.045149036, -0.022667488, 0.08573697, 0.19832925, -0.40519664, 0.0743087, 0.10722445) * inp_1_1_0;
    result5 += M4(0.009492822, -0.0037784355, 0.06248918, 0.013292263, 0.00069497636, 0.0366736, -0.055103716, 0.059165534, 0.004672803, -0.07055219, 0.0029433626, -0.024844151, 0.12493549, -0.7867982, -0.58025944, -0.21975455) * inp_1_2_0;
    result5 += M4(0.001050904, 0.0146837635, -0.04574529, 0.10690885, -0.01361782, 0.105780356, 0.04041239, -0.031947754, -0.07373307, 0.035656404, 0.05155069, 0.24783309, -0.054360986, -0.30858085, -0.10588055, -0.059958484) * inp_1_0_1;
    result5 += M4(0.20496897, 0.12870112, -0.058519438, 0.08376191, -0.022784948, 0.13326985, 0.26592183, 0.08329928, -0.079983704, 0.027403956, -0.27069014, 0.0028907384, -0.16614927, -0.7703318, 0.012079895, 0.26153743) * inp_1_1_1;
    result5 += M4(-0.07272137, 0.14642929, 0.16084719, -0.20865643, -0.0067136167, -0.2842206, -0.27806094, -0.09338169, -0.035001185, -0.0353403, 0.09251099, -0.14541832, 0.067289695, -1.071835, -0.27161756, -0.53292406) * inp_1_2_1;
    result5 += M4(-0.09050276, 0.016388595, 0.040668942, 0.025914704, 0.15949382, -0.01824759, -0.08531987, -0.27030364, 0.0025822215, 0.17334636, -0.18343244, -0.080979496, 0.09677813, -0.1596562, 0.109882966, 0.038775574) * inp_1_0_2;
    result5 += M4(-0.1413254, -0.24608794, -0.080066256, -0.03915451, -0.10647768, -1.137157, -0.34617174, -0.08819499, 0.03500336, -0.11630728, -0.085407816, 0.25469247, -0.23205402, -0.35125443, 0.007356423, -0.0201355) * inp_1_1_2;
    result5 += M4(0.09948331, -0.05031442, -0.08301437, 0.13498205, -0.24063364, -1.3358799, 0.050099503, -0.22905706, -0.18052778, -0.13673082, 0.13169907, 0.051259268, -0.048390318, -0.5443882, -0.08985491, 0.11005883) * inp_1_2_2;
    result6 += M4(0.056949317, 0.007159933, -0.0035620686, -0.12794094, -0.08963886, -0.028114695, -0.010528492, 0.041745804, -0.003821106, -0.090141766, 0.017976705, 0.056568038, 0.009532607, 0.04504768, 0.012435493, 0.1034698) * inp_1_0_0;
    result6 += M4(0.0009396516, -0.034477852, -0.08153047, -0.15583807, 0.08876918, 0.24374215, 0.033245787, -0.05888273, 0.016068857, 0.023455286, 0.017904082, 0.120244466, -0.019959971, -0.043497667, -0.008910644, 0.10155337) * inp_1_1_0;
    result6 += M4(-0.0658824, 0.18872403, 0.005241687, -0.13141958, 0.024456188, -0.21457995, -0.038373172, 0.019170497, -0.030900665, -0.001437599, 0.0086882105, 0.0769212, -0.20137195, 0.12229854, 0.03878628, -0.014378287) * inp_1_2_0;
    result6 += M4(-0.21344481, -0.031521376, -0.0043078694, -0.2200148, -0.1622734, 0.059458546, 0.08725464, -0.12933102, 0.06563178, -0.18012014, -0.13919914, 0.44187993, 0.05851809, 0.0064023533, -0.03864558, 0.10596844) * inp_1_0_1;
    result6 += M4(0.262959, -0.1290537, 0.0739153, -0.24555607, 0.2111988, -0.13396165, 0.009519021, -0.1570787, 0.07156181, 0.07634185, -0.20605129, 0.46941367, 0.4456404, 0.2773888, -0.18212692, 0.30846265) * inp_1_1_1;
    result6 += M4(-0.008059678, -0.15947364, 0.33264115, -0.21736442, -0.090223506, 0.015212134, 0.040714577, -0.27530262, -0.04741964, 0.0887245, -0.1470857, 0.34935862, 0.17896633, -0.18377076, -0.104007766, -0.06419573) * inp_1_2_1;
    result6 += M4(0.09364479, 0.06939338, -0.068596296, 0.085539944, -0.61186796, 0.10078235, -0.06970609, 0.056787305, 0.05809103, -0.11052339, -0.07408968, 0.032724462, -0.057123672, 0.039711647, 0.024160072, -0.087207116) * inp_1_0_2;
    result6 += M4(-0.1261126, 0.056299333, -0.01903221, 0.17823109, -0.44715047, -0.2591218, 0.35320738, -0.007762339, -0.25087973, 0.03213844, -0.09740567, 0.03280368, 0.07008119, 0.078857444, -0.037831113, -0.007181009) * inp_1_1_2;
    result6 += M4(0.10676735, -0.007814142, 0.1381455, 0.14295544, 0.3191652, -0.19476484, -0.102793, 0.104208805, 0.07466653, 0.08374993, -0.122043036, 0.12390872, -0.024429934, -0.080078505, -0.05302589, -0.26027018) * inp_1_2_2;
    result7 += M4(0.028224524, 0.04321128, -0.043853678, -0.04303479, -0.18218105, -0.0069272653, -0.04428781, 0.013981883, -0.034301504, 0.0203772, -0.04756501, 0.16002321, 0.091034904, -0.01155648, 0.03430416, 0.019668229) * inp_1_0_0;
    result7 += M4(0.07753012, -0.0076613734, -0.033725873, -0.12396285, 0.14253268, -0.09061047, 0.03273882, -0.0020961629, -0.04706008, -0.05898291, 0.11304634, 0.030304018, -0.13242471, 0.038218826, 0.17317449, 0.2796462) * inp_1_1_0;
    result7 += M4(-0.0662321, 0.029532053, -0.085095055, 0.02976887, -0.013448455, -0.042173877, 0.058400523, 0.018880654, 0.03952247, 0.024860185, -0.12010396, 0.027629087, -0.17307387, -0.0075285924, 0.11312583, -0.033258826) * inp_1_2_0;
    result7 += M4(-0.047715314, -0.04729886, 0.0721176, 0.08059095, 0.05286146, 0.049846813, -0.12751165, 0.015601787, -0.055032045, 0.024094833, 0.074012466, 0.05256108, 0.035800874, -0.02041223, -0.050325383, -0.048539884) * inp_1_0_1;
    result7 += M4(0.03346662, -0.16009906, 0.26031104, -0.1698477, 0.19746922, 0.2702088, -0.26734167, 0.12499584, -0.017590558, 0.19037649, -0.21142039, 0.020295484, -0.46252644, -0.033626284, 0.07853712, -0.15674591) * inp_1_1_1;
    result7 += M4(0.01175841, -0.06744432, 0.2910994, -0.10239086, -0.2814292, -0.040220745, -0.13346551, -0.110715374, -0.028909195, -0.07466383, 0.04509794, 0.19257963, -0.19082698, -0.1060442, 0.25096893, 0.043528516) * inp_1_2_1;
    result7 += M4(-0.13684401, 0.008113096, 0.06749155, 0.062847525, 0.9023796, -0.030626552, -0.17223376, 0.42837083, -0.097155645, 0.029769838, -0.09845425, 0.40341792, 0.119613454, 0.010234417, 0.0067745107, 0.049939554) * inp_1_0_2;
    result7 += M4(-0.30711752, -0.03456402, 0.04554331, 0.14442766, 1.1494461, 0.16982731, 0.22479841, 0.14150201, -0.15936835, -0.101201765, 0.15719861, -0.11471562, 0.1521283, -0.015476089, 0.14456816, -0.07768541) * inp_1_1_2;
    result7 += M4(-0.031777374, 0.12877725, 0.4301505, 0.0064106956, 0.69235635, 0.0779587, 0.038384173, 0.00026669962, 0.16801775, 0.0012411203, -0.022870371, -0.023279088, -0.105702356, -0.023973687, 0.05170024, 0.10075755) * inp_1_2_2;
    const V4 inp_2_0_0 = inp[2][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_2_1_0 = inp[2][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_2_2_0 = inp[2][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_2_0_1 = inp[2][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_2_1_1 = inp[2][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_2_2_1 = inp[2][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_2_0_2 = inp[2][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_2_1_2 = inp[2][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_2_2_2 = inp[2][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.119210534, -0.046689074, -0.03883785, 0.010501195, 0.05540147, 0.10575708, 0.16727477, 0.081714146, 0.010441477, -0.03224425, -0.116736904, -0.042683095, 0.013460011, -0.029328471, -0.010791575, -0.009815912) * inp_2_0_0;
    result0 += M4(0.119025335, 0.0127056725, -0.12874013, -0.0041830232, 0.06444307, 0.014304839, 0.09158716, 0.030007245, -0.09602631, -0.08846306, -0.08941599, -0.024421308, -0.010736074, -0.13130991, -0.018695902, 0.022015352) * inp_2_1_0;
    result0 += M4(-0.24075505, -0.14834902, -0.12751892, -0.0049896743, 0.07017397, -0.05847175, -0.07186146, -0.08396846, 0.059897743, 0.13341166, 0.0024268162, -0.08865947, -0.044016242, 0.035191566, -0.024878157, 0.09843897) * inp_2_2_0;
    result0 += M4(0.01336664, -0.060980618, 0.14528637, -0.05022282, -0.2332288, -0.09033641, 0.18158118, 0.033419766, -0.03480591, 0.07227199, -0.02247423, -0.11430169, 0.056421638, -0.012982187, -0.048045576, 0.015259326) * inp_2_0_1;
    result0 += M4(0.099937804, 0.46402177, 0.38013545, 0.22067249, 0.108865805, -0.11073912, -0.26095396, -0.09191549, -0.033087928, -0.14467408, -0.09141014, -0.09456989, -0.06636351, -0.080263115, 0.119624525, 0.1479756) * inp_2_1_1;
    result0 += M4(0.06363207, 0.013973629, 0.15927719, -0.1571763, -0.061212625, 0.06714816, -0.065712206, 0.046172846, 0.007737184, 0.051504444, 0.033230484, -0.03751037, -0.0089869285, 0.015556724, 0.050579157, 0.054090478) * inp_2_2_1;
    result0 += M4(0.04976652, -0.0038331822, 0.15507294, 0.028176868, -0.118018396, 0.028880667, 0.06080324, -0.021181168, 0.030497607, 0.04475299, 0.043411434, -0.007668049, 0.036267776, 0.040245842, -0.04567875, 0.01401697) * inp_2_0_2;
    result0 += M4(0.187161, -0.054413315, 0.1639266, -0.07749777, -0.040453095, 0.0002078059, 0.0519475, 0.054854978, -0.06829962, -0.009735663, 0.029353762, 0.00512157, 0.04470108, 0.06061558, -0.06545799, 0.034634676) * inp_2_1_2;
    result0 += M4(0.05163237, -0.08886301, 0.21910627, -0.13732614, 0.093931556, 0.038826235, -0.22503886, 0.05196871, 0.00411725, 0.019613136, 0.09779307, 0.006333425, -0.03704866, -0.0045951097, -0.14204095, 0.0055744667) * inp_2_2_2;
    result1 += M4(0.15765049, -0.072629556, 0.030457556, 0.17097142, 0.033673886, 0.06881499, 0.03592561, -0.24762292, 0.083667, 0.027738174, -0.00074092514, -0.07722585, -0.032601457, 0.10171067, 0.0027536459, 0.08110402) * inp_2_0_0;
    result1 += M4(0.060685314, -0.015674824, -0.099995986, -0.0399487, 0.06270354, 0.055436768, 0.021705898, -0.05981645, -0.031943254, -0.1263745, 0.16570279, 0.06629958, 0.033690464, 0.022661095, -0.023116559, -0.02853095) * inp_2_1_0;
    result1 += M4(0.08719316, 0.15867421, -0.25915948, 0.07749243, 0.006065532, 0.23656072, 0.20127049, 0.0918699, 0.11314135, -0.12968966, 0.177022, -0.11428088, -0.067376405, -0.05030923, -0.16454549, 0.014189506) * inp_2_2_0;
    result1 += M4(0.0032644207, -0.010584407, 0.06701175, 0.11920757, 0.18052922, 0.10745175, 0.1909139, 0.02665002, -0.1081425, 0.072243944, 0.051244937, -0.09621919, 0.07049503, 0.14322501, -0.02595246, 0.09260165) * inp_2_0_1;
    result1 += M4(-0.23046367, 0.14439599, 0.1470515, -0.13889977, -0.25033507, -0.5309813, 0.10235818, 0.19882376, 0.032619663, 0.26320174, 0.020204127, -0.014219849, 0.22321391, -0.049060144, -0.1281255, 0.06529282) * inp_2_1_1;
    result1 += M4(0.101803325, 0.22486669, 0.062095363, 0.09893067, -0.15983236, -0.12773745, 0.05394249, 0.023641324, -0.095761545, 0.23589288, -0.045721352, -0.17557822, -0.022813149, -0.026281847, 0.009904645, 0.110548645) * inp_2_2_1;
    result1 += M4(0.07235501, -0.06893859, 0.0931121, 0.28031272, 0.16706629, 0.16109051, -0.12219461, -0.056650087, -0.13316126, -0.03550483, 0.053971894, 0.124244235, 0.048599917, 0.11145328, -0.0073408755, -0.045119572) * inp_2_0_2;
    result1 += M4(-0.027577026, -0.047935158, 0.12033387, 0.031460434, -0.085737936, -0.077876404, -0.08904048, 0.10613565, -0.0021520574, 0.04473005, -0.01619206, 0.16994226, -0.21744606, -0.032282855, 0.12310161, -0.15471318) * inp_2_1_2;
    result1 += M4(-0.03450798, -0.014878816, -0.005244815, 0.050909184, 0.0633156, 0.19341716, 0.189101, -0.13597865, 0.060511, -0.15702993, -0.11548058, 0.090386756, -0.08967349, 0.0529664, 0.09390205, -0.120987825) * inp_2_2_2;
    result2 += M4(-0.099716336, 0.0206073, 0.038134918, 0.025791958, 0.048816465, -0.09037814, 0.088722035, -0.06132513, -0.08560063, -0.0019220945, -0.08408985, -0.006943003, -0.075006105, 0.11844617, -0.103573, -0.01826205) * inp_2_0_0;
    result2 += M4(0.060195934, 0.17774852, -0.021004975, 0.12235989, 0.0096782455, 0.004615004, -0.030170303, -0.11994218, -0.15664683, 0.050146993, 0.094307914, -0.17819157, 0.053562418, -0.011582047, 0.14070433, 0.1304386) * inp_2_1_0;
    result2 += M4(0.11520325, 0.013279046, -0.007471889, 0.09948853, -0.050775364, 0.002050347, 0.12776206, -0.09170206, -0.004861289, 0.008987602, -0.043881692, -0.13894787, 0.003307056, -0.008775438, -0.055927895, 0.068667606) * inp_2_2_0;
    result2 += M4(-0.02958709, -0.14366132, 0.06452568, 0.023178902, -0.12661244, 0.07423362, 0.03329557, -0.05073679, 0.05844408, -0.10768905, -0.19035172, -0.25100878, -0.16196139, 0.11642275, -0.1460564, 0.13767032) * inp_2_0_1;
    result2 += M4(-0.21421829, -0.27727458, 0.37527063, -0.38310337, 0.22485845, -0.0038291842, -0.38720486, 0.5349276, 0.18370171, -0.32447988, -0.17590958, -0.8582276, 0.046016753, 0.18476714, 0.036804903, 0.6549796) * inp_2_1_1;
    result2 += M4(0.028359544, -0.064651184, 0.124199435, -0.22734062, -0.09183432, -0.037896167, 0.055285007, 0.003247758, 0.06264667, 0.09656024, 0.023982335, -0.061420474, -0.10126678, -0.06177706, -0.057019945, 0.0642879) * inp_2_2_1;
    result2 += M4(0.004467001, 0.014075538, -0.05422827, 0.05398185, -0.16666986, 0.121347636, -0.018773494, 0.15815446, 0.12217999, -0.0015489972, -0.070373766, -0.08823121, -0.105006814, 0.049145352, 0.020339742, 0.10880402) * inp_2_0_2;
    result2 += M4(-0.007867756, 0.14892356, 0.069037154, 0.12898186, -0.046252396, 0.00013000076, 0.1364165, 0.017750157, 0.008767697, -0.04052973, -0.16781019, -0.08728476, 0.038207114, 0.02412487, 0.06241999, 0.021956038) * inp_2_1_2;
    result2 += M4(0.046081003, -0.05321007, -0.0256513, 0.080286935, -0.075417705, 0.020875918, 0.034252618, -0.012061754, 0.11414756, -0.029432736, 0.087685816, -0.053597003, -0.04845102, -0.039175928, -0.09469389, -0.015734492) * inp_2_2_2;
    result3 += M4(-0.033805713, -0.0008133791, 0.011845789, 0.016739262, 0.10888628, 0.013922021, 0.06401611, -0.02587727, 0.063002035, -0.06793361, 0.045622077, -0.068882465, -0.057501353, 0.043849345, -0.0146066435, 0.05579271) * inp_2_0_0;
    result3 += M4(0.038356986, 0.019356972, -0.09146968, -0.06572172, 0.122912735, 0.021600973, -0.052424956, 0.054065045, 0.04254331, -0.039959647, 0.04532304, 0.17331614, 0.023271222, 0.005905924, -0.018259542, -0.05871839) * inp_2_1_0;
    result3 += M4(0.20507015, -0.06299371, -0.16708127, 0.029433161, -0.02310035, -0.026174359, -0.16481657, -0.16995455, -0.027030943, 0.055499285, 0.0029948042, 0.09389146, -0.036678895, 0.01474875, 0.07563015, 0.03938696) * inp_2_2_0;
    result3 += M4(0.019661581, -0.033956382, 0.15673678, -0.005756747, 0.09469912, 0.011890621, -0.10370978, 0.19550614, -0.10132591, -0.013564558, -0.19996768, -0.043927606, -0.16053824, -0.023350272, 0.042631377, -0.09326185) * inp_2_0_1;
    result3 += M4(-0.05544698, 0.024809234, 0.035667326, -0.21380088, 0.051770266, 0.008997337, -0.2303206, 0.20642708, -0.3474238, 0.025504956, 0.0026472148, -0.008161099, 0.019385235, -0.02843729, 0.25050315, 0.028845923) * inp_2_1_1;
    result3 += M4(-0.09380775, -0.028316729, -0.1427356, -0.10714575, -0.07571349, 0.023767909, -0.098260336, 0.06843741, -0.16487911, 0.022219839, 0.06085165, -0.03339353, 0.05178141, 0.029247597, -0.021404527, -0.023897873) * inp_2_2_1;
    result3 += M4(-0.035048343, 0.059053447, 0.20411915, -0.1338426, 0.0468856, -0.026280029, -0.049956605, 0.0007338997, 0.09681243, -0.025564875, -0.008708173, 0.020936925, 0.06477414, 0.009833558, -0.0625285, 0.11324239) * inp_2_0_2;
    result3 += M4(0.102137074, 0.026838463, -0.00027916464, -0.120751634, 0.080129124, -0.03787573, -0.07255862, 0.110684514, 0.02130058, -0.032067016, 0.13008538, 0.036075383, -0.14456409, -0.08446966, -0.13433668, -0.05169812) * inp_2_1_2;
    result3 += M4(0.0801684, -0.0136966435, -0.10360681, -0.1273021, -0.11129879, 0.0015963251, 0.032624982, 0.057238415, 0.007387065, 0.08452757, -0.08163987, 0.011496506, -0.009233649, 0.027931726, -0.010127449, 0.002231746) * inp_2_2_2;
    result4 += M4(0.026107367, 0.07048594, 0.1103495, -0.11693625, -0.24929495, -0.05210659, 0.009830281, 0.10020228, 0.016992232, -0.07785775, 0.022631932, 0.22334224, -0.02525314, 0.0074370084, 0.04859325, -0.07926462) * inp_2_0_0;
    result4 += M4(-0.20209627, -0.069525994, -0.08001047, -0.09476812, 0.11634032, -0.03689052, 0.024179373, 0.072857514, 0.21087903, 0.097456135, -0.026172949, -0.09156805, -0.04686917, -0.07941167, 0.03395063, -0.03348874) * inp_2_1_0;
    result4 += M4(0.124355435, 0.15236968, 0.051674288, 0.2469954, 0.03340045, -0.06584444, -0.055979695, 0.055910714, -0.11421858, 0.06875074, -0.044408195, -0.11495282, 0.031149806, -0.0116400365, 0.0035837784, -0.011389078) * inp_2_2_0;
    result4 += M4(0.06458687, 0.100493714, 0.045393474, 0.023623029, -0.26960823, -0.12111673, -0.0851588, -0.14006545, -0.08702851, 0.09015783, -0.12584293, -0.117269106, -0.07299301, 0.06575899, -0.0045303116, -0.048412755) * inp_2_0_1;
    result4 += M4(0.23797287, 0.17636879, -0.6685462, -0.20598316, -0.12906513, -0.23714544, 0.33349743, 0.16308485, -0.0678274, -0.04737519, 0.07760143, 0.18696977, 0.15999004, -0.08573363, -0.10769633, -0.12392106) * inp_2_1_1;
    result4 += M4(0.2270809, -0.14684159, -0.13385369, 0.21505344, -0.13453518, 0.08525869, -0.009907513, 0.083485186, -0.11266089, -0.10121205, 0.14130206, -0.001235002, 0.037974562, 0.035038374, -0.035425458, 0.052744918) * inp_2_2_1;
    result4 += M4(0.04467926, 0.022081103, 0.10770057, 0.18265326, -0.14365911, 0.06216253, -0.19980668, -0.021949833, 0.1140759, 0.014602233, -0.059602376, -0.15263943, -0.030107617, 0.0436545, -0.11660001, -0.09897569) * inp_2_0_2;
    result4 += M4(-0.1261603, -0.02653629, -0.17183591, 0.1663143, -0.04480713, 0.09043595, 0.044781327, 0.18586089, -0.10122428, 0.003802116, -0.006479857, -0.017904226, 0.048779413, -0.05163214, 0.09599822, 0.09380484) * inp_2_1_2;
    result4 += M4(0.06018047, 0.08108601, 0.044398572, 0.27287248, 0.06400987, 0.35155657, 0.0076385224, -0.035552066, -0.0054244953, 0.008057301, 0.10722474, 0.0007148956, 0.0037589062, 0.035683397, 0.029441932, 0.021545308) * inp_2_2_2;
    result5 += M4(0.053778283, 0.28528768, -0.06123459, 0.083771124, 0.13950457, -0.23683834, 0.08356634, -0.06773585, 0.089083105, 0.029674152, -0.031206857, -0.06825872, -0.09786522, 0.08393015, -0.009954267, 0.11243962) * inp_2_0_0;
    result5 += M4(-0.03887216, 0.22968875, -0.09594185, -0.13610193, -0.03198368, -0.25976458, 0.19351098, 0.17617372, 0.093272924, 0.032826148, -0.18444568, 0.14346643, -0.04185061, 0.023573816, 0.016124219, -0.09798897) * inp_2_1_0;
    result5 += M4(-0.25245994, 0.40774533, -0.04139962, 0.29051283, 0.10006931, -0.12614349, 0.1747573, -0.113059394, 0.054762956, -0.043345746, -0.0028853847, 0.14132431, -0.037471816, -0.016215984, -0.08130566, 0.013099781) * inp_2_2_0;
    result5 += M4(-0.0047396007, 0.30617055, 0.1541037, -0.006686757, 0.00046419926, -0.45809048, 0.0035648677, -0.089505784, -0.07390483, -0.07419051, -0.09723532, 0.0339748, 0.05140011, 0.02371354, 0.027925728, 0.02532192) * inp_2_0_1;
    result5 += M4(0.32169822, 0.64894867, 0.58899575, -0.17935637, 0.06925575, -0.7185153, -0.392406, 0.17175616, -0.164859, -0.029976781, -0.07163208, -0.1613776, -0.10847889, 0.015687933, 0.06256832, -0.07519869) * inp_2_1_1;
    result5 += M4(0.23041332, 0.44610524, 0.1712342, 0.06365912, 0.19116223, -0.25325367, 0.08545049, -0.17623323, -0.21280253, 0.0063762832, 0.034383874, -0.13819128, 0.059927512, -0.034792062, -0.016822018, 0.051395297) * inp_2_2_1;
    result5 += M4(-0.12344676, 0.26158774, 0.019484498, 0.12136747, 0.09424177, -0.23410615, 0.0017049996, 0.004514343, 0.07242814, 0.052045498, -0.007657273, 0.04518586, -0.0008106348, -0.034441512, 0.03617946, -0.049525738) * inp_2_0_2;
    result5 += M4(-0.034927204, 0.567088, -0.1006309, 0.12283057, -0.018474106, -0.32160872, -0.11217372, 0.06253433, 0.02562512, 0.09061914, 0.11133961, -0.015944455, -0.04359815, -0.07859582, -0.15252326, -0.014587648) * inp_2_1_2;
    result5 += M4(0.1493147, 0.3399222, -0.23303299, 0.17266762, -0.06825782, -0.17149262, 0.17459655, -0.09216597, 0.013724986, -0.14088652, 0.04443413, -0.008396707, 0.009406686, -0.06734309, 0.0057248087, 0.00945308) * inp_2_2_2;
    result6 += M4(0.062185038, -0.09844748, -0.039499164, 0.033928365, 0.057741944, 0.09226052, 0.026414061, 0.012146365, -0.106181055, 0.032188617, 0.006419159, 0.034248658, -0.025182337, 0.04092088, -0.03384848, -0.02080292) * inp_2_0_0;
    result6 += M4(-0.0625775, -0.23504727, 0.039612163, 0.0042083883, 0.03353332, -0.1412001, 0.0068477574, 0.040970482, 0.19454166, -0.033027735, 0.07263514, 0.05208678, 0.0150084235, -0.05150253, -0.014129573, -0.0562562) * inp_2_1_0;
    result6 += M4(0.044503383, 0.33163717, -0.037447687, 0.00087746425, 0.041847285, 0.039558418, 0.026371462, 0.006108909, -0.062391814, 0.06447978, 0.008039021, -0.11596189, 0.0151021695, 0.0066883075, 0.004421142, 0.067051314) * inp_2_2_0;
    result6 += M4(0.04522606, 0.13230674, -0.0042952057, -0.03961297, -0.047645397, 0.018772665, 0.016704211, -0.057010274, 0.21092102, -0.05317713, -0.0007738601, 0.1980903, -0.13373375, 0.061212935, 0.023165604, -0.055052612) * inp_2_0_1;
    result6 += M4(-0.17397553, -0.23179212, -0.16286978, 0.27229297, -0.15792727, 0.12804514, -0.109132834, -0.19456947, -0.126262, 0.053079765, 0.51736873, -0.09552904, -0.0076753455, 0.055315133, -0.15363446, 0.061240092) * inp_2_1_1;
    result6 += M4(0.19073883, -0.071776964, 0.025742993, 0.08457235, -0.053318154, 0.12322886, 0.04867062, 0.07378872, 0.054430027, -0.034911703, -0.028161261, -0.0843376, 0.0011639538, 0.034361765, -0.00034657586, 0.055823937) * inp_2_2_1;
    result6 += M4(0.072973035, -0.055157382, -0.0013093787, -0.10063247, 0.06597927, -0.07514755, -0.051521443, 0.06195094, 0.057622723, 0.1345359, 0.06640946, -0.010474883, -0.02039321, -0.01159695, -0.054511182, 0.031650163) * inp_2_0_2;
    result6 += M4(0.19437926, -0.04444946, -0.094295226, 0.021084396, 0.025142659, 0.051373795, -0.002011888, 0.095847055, -0.09343858, -0.11345659, 0.034876283, 0.077077135, -0.036032714, -0.1306647, 0.023067795, 0.02403538) * inp_2_1_2;
    result6 += M4(-0.0037745624, 0.013178992, 0.039044656, -0.15839806, 0.025399173, -0.12232109, -0.002097052, 0.032139894, 0.03893354, 0.18052283, -0.014334295, -0.078582354, 0.060208183, 3.145041e-05, 0.036595874, -0.003993465) * inp_2_2_2;
    result7 += M4(-0.012351323, -0.007020195, -0.039159235, -0.07356754, 0.13625385, 0.0048539434, 0.12400752, -0.17231067, -0.12407627, 0.030319499, -0.07181447, -0.010404383, 0.049781837, -0.016370079, -0.08235963, 0.11224101) * inp_2_0_0;
    result7 += M4(-0.15590602, -0.054281965, 0.06616242, 0.051906615, -0.054861777, -0.0624152, 0.22834131, 0.055869542, -0.0048256353, -0.021311583, -0.2208189, -0.0033533135, 0.043215655, 0.036784783, -0.036917403, 0.1208041) * inp_2_1_0;
    result7 += M4(-0.12158632, -0.024639651, -0.04164941, -0.05928095, 0.08036424, 0.025908608, 0.0019728532, 0.11430116, -0.12880184, 0.08554722, -0.01838312, 0.11582244, -0.01035332, -0.055028908, 0.021672376, -0.03281161) * inp_2_2_0;
    result7 += M4(-0.058760773, 0.007513693, 0.0729139, -0.09194471, -0.04526353, -0.084393024, 0.1903075, -0.039424203, -0.09869729, 0.00810536, -0.07227934, 0.038465504, 0.07528734, 0.0021674812, -0.011742288, -0.040679332) * inp_2_0_1;
    result7 += M4(-0.25331065, -0.10846307, 0.708073, -0.047661927, 0.055392772, -0.057975415, 0.18108125, 0.039338198, 0.19540733, 0.5183824, 0.05190783, -0.14563239, -0.23541285, -0.24106182, -0.03925942, 0.014404427) * inp_2_1_1;
    result7 += M4(0.10321028, 0.10302252, 0.07065324, 0.13749246, 0.019748881, -0.021082152, -0.16698997, 0.0038232056, 0.076405525, 0.0525409, 0.101853035, -0.038061306, 0.050866842, 0.012845092, -0.10580409, 0.0078802155) * inp_2_2_1;
    result7 += M4(-0.07031003, 0.0010839779, -0.01935958, -0.078741804, 0.044557344, -0.021654638, -0.013927434, -0.098349966, 0.022826858, -0.0052575907, -0.08374733, 0.07951779, -0.040346056, -0.0229279, -0.023640346, 0.056133028) * inp_2_0_2;
    result7 += M4(-0.33382887, -0.08234722, -0.075560436, -0.10808978, 0.08086833, -0.023644771, -0.017951885, 0.09221891, -0.027481312, 0.041308258, -0.013457815, -0.0007248234, 0.08055086, -0.0071629086, -0.05203989, -0.03185341) * inp_2_1_2;
    result7 += M4(-0.12033959, 0.02597854, 0.2018133, 0.04918074, 0.06689774, -0.022369476, -0.11845302, -0.14206433, -0.012550103, 0.038854536, 0.054565016, -0.013141956, 0.006666628, -0.0006866235, -0.015923845, -0.081939355) * inp_2_2_2;
    const V4 inp_3_0_0 = inp[3][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_3_1_0 = inp[3][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_3_2_0 = inp[3][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_3_0_1 = inp[3][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_3_1_1 = inp[3][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_3_2_1 = inp[3][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_3_0_2 = inp[3][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_3_1_2 = inp[3][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_3_2_2 = inp[3][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.13743378, -0.0775773, 0.15459493, 0.07604143, -0.22990678, -0.046382975, -0.27278385, -0.1365688, -0.029326493, 0.07362296, -0.028695205, 0.06955529, -0.69356185, -0.25919393, 2.1670847, 0.6991939) * inp_3_0_0;
    result0 += M4(0.09522774, 0.23248398, 0.0055527566, 0.047710624, -0.0729103, -0.11976985, -0.21565966, -0.050842695, 0.009044374, -0.16572237, -0.06049678, -0.01061676, -0.0025671439, 0.17088592, 0.29526833, 0.049957544) * inp_3_1_0;
    result0 += M4(-0.07860414, -0.30148745, -0.12514053, 0.12905963, 0.090269975, -0.274147, -0.07931763, 0.077214085, -0.035016395, -0.0014335355, 0.06257938, 0.03393078, 0.040165942, -0.116686516, -0.1658397, 0.009683523) * inp_3_2_0;
    result0 += M4(-0.24417679, -0.042052064, 0.053023446, 0.12314307, 0.013046413, 0.08133184, -0.06832346, -0.12063268, -0.0029631855, -0.010620728, 0.11395925, -0.010603545, 0.0830874, 0.22241993, 0.27043942, 0.2059458) * inp_3_0_1;
    result0 += M4(0.14929171, 0.046473745, -0.09046149, 0.03250168, -0.1392021, 0.2524155, 0.49457714, 0.2251422, 0.09060516, 0.06653155, -0.024097526, -0.06094995, -0.13515803, -0.0771835, -0.37827495, 0.18615669) * inp_3_1_1;
    result0 += M4(-0.017955368, -0.018858327, 0.012479345, -0.0621993, 0.009720244, 0.1270974, -0.0007394759, 0.019080468, -0.026002254, -0.068076864, -0.019422987, 0.04416094, 0.038842212, -0.09452968, 0.08588119, 0.11409659) * inp_3_2_1;
    result0 += M4(-0.054629534, -0.014727507, -0.022140747, 0.0141780265, 0.13056152, 0.04931344, -0.14006776, 0.039502975, -0.11876983, -0.057961106, -0.08551188, 0.014486342, -0.07716371, -0.050354365, -0.14067294, 0.025632137) * inp_3_0_2;
    result0 += M4(0.08004688, 0.19486257, -0.055659227, 0.107346214, 0.11219107, -0.04161709, 0.0026936866, -0.07733443, 0.033238508, 0.12938885, -0.01596445, 0.030354882, 0.1847576, -0.12912424, 0.085141405, 0.08215033) * inp_3_1_2;
    result0 += M4(-0.0064310203, -0.11593306, -0.011221703, -0.061599, 0.052642193, -0.03865307, -0.03267413, -0.062858626, 0.0763478, 0.0039602835, 0.082744285, 0.012498492, -0.10740914, 0.18308781, 0.06125812, 0.034608785) * inp_3_2_2;
    result1 += M4(0.20018756, 0.047869485, 0.24296093, 0.009797422, -0.36069274, -0.1704151, -0.12735647, 0.35979876, 0.009694131, -0.03128229, -0.042556368, -0.016654184, 0.87093717, 0.6739249, 1.0951092, 2.5869234) * inp_3_0_0;
    result1 += M4(-0.19121139, -0.12914562, -0.091771185, -0.1516909, -0.09670318, -0.054752942, -0.018303687, 0.21020322, -0.10093475, -0.027356213, 0.05108199, 0.055973712, 0.37985185, -0.19673002, 0.3394248, -0.06763138) * inp_3_1_0;
    result1 += M4(-0.2467268, 0.12735787, -0.060134172, 0.070573226, -0.0873626, 0.06157731, -0.11546102, 0.058753584, -0.043428354, -0.009762072, -0.04406596, -0.058824144, -0.009725757, 0.24598587, 0.03194342, 0.11888991) * inp_3_2_0;
    result1 += M4(0.09144271, 0.019837514, 0.14291927, 0.11708461, -0.24301651, -0.03945649, -0.12689735, -0.0037089733, 0.02378123, 0.15684874, -0.053608157, 0.08638607, 0.3851368, 0.19824961, 0.071584396, -0.052914586) * inp_3_0_1;
    result1 += M4(-0.15127222, -0.011190187, -0.013503422, -0.1526213, 0.36236733, -0.066156454, -0.10312205, -0.16931024, 0.009652182, -0.11068442, 0.22919166, -0.09263494, -0.36097232, 0.4241134, -0.08886202, 0.15937205) * inp_3_1_1;
    result1 += M4(0.066700354, 0.053011674, 0.006187166, 0.009763046, 0.025374074, 0.05124184, -0.010189415, 0.050247684, -0.0408296, -0.06370937, -0.03167791, -0.1635215, -0.056518868, -0.5772532, 0.004505324, -0.13619693) * inp_3_2_1;
    result1 += M4(0.08990585, 0.0621896, 0.023452861, -0.13756433, -0.14771667, -0.09099252, 0.04970861, 0.052747276, -0.0649559, -0.058580328, -0.015459464, -0.0060880254, -0.13172357, -0.10850414, 0.3173808, -0.049025998) * inp_3_0_2;
    result1 += M4(-0.08289032, -0.11066249, -0.05703214, 0.27027026, 0.0136759, -0.02246509, -0.01136761, -0.017577583, -0.042513408, 0.105250694, -0.112345815, 0.07376739, -0.0011483385, -0.26890665, -0.24043705, -0.014064036) * inp_3_1_2;
    result1 += M4(-0.04517703, -0.027818736, 0.065366924, -0.18123713, 0.060223307, 0.17496808, 0.023027522, -0.0934669, 0.11555, 0.1242177, 0.0025294772, 0.0670779, 0.08019211, 0.3682534, -0.05907095, 0.0682388) * inp_3_2_2;
    result2 += M4(-0.013985352, 0.005617688, 0.14419533, 0.011967171, -0.0022866803, 0.13279134, -0.11119055, 0.12602626, -0.099940136, 0.032322228, -0.10960388, 0.0021809023, -0.19047146, -0.15918149, -2.340845, -1.1799726) * inp_3_0_0;
    result2 += M4(0.03527892, 0.09124828, -0.16763456, -0.0008539915, 0.083691575, 0.0856758, -0.049770854, 0.049129028, 0.06793196, -0.013256047, 0.16776608, 0.13372692, -0.064959966, -0.14367302, 0.010778536, -0.07717949) * inp_3_1_0;
    result2 += M4(0.050565373, 0.12506044, -0.05821966, 0.17991857, 0.10836748, 0.0010156183, -0.042304866, 0.12643808, 0.018870495, -0.0061986735, -0.041639898, 0.040789694, -0.07612905, -0.08212254, -0.047350444, 0.09552464) * inp_3_2_0;
    result2 += M4(-0.21995945, 0.04218771, 0.06722781, 0.11076082, 0.023122702, -0.05977072, 0.09413939, 0.019498838, -0.025831325, -0.009146579, -0.013833045, -0.051093504, 0.3996612, -0.28821686, -0.051475614, 0.003721678) * inp_3_0_1;
    result2 += M4(-0.049731288, -0.0025708885, -0.0053061154, 0.47251529, 0.007205664, 0.09114548, 0.42525703, 0.17769948, 0.041465487, 0.020584196, -0.010052914, -0.17641376, 0.13301538, -0.020802481, 0.07820173, -0.103667915) * inp_3_1_1;
    result2 += M4(-0.14421101, 0.016246788, -0.043745432, -0.03559261, -0.006091402, -0.09956937, 0.05945553, 0.068716116, -0.03164876, 0.0859001, 0.051884383, 0.033996377, 0.09215956, 0.08000334, -0.039702453, 0.032560855) * inp_3_2_1;
    result2 += M4(-0.22247267, 0.058697604, -0.022572814, 0.15056711, 0.041310843, -0.049758866, -0.08713887, -0.053382862, 0.06515659, -0.008848865, -0.020783149, -0.105593845, 0.14879169, 0.049630046, 0.0044312016, 0.005168508) * inp_3_0_2;
    result2 += M4(-0.14990003, 0.012436959, 0.07425869, 0.077199504, 0.1512946, -0.060135867, -0.015879882, -0.025705753, 0.017019983, -0.017781582, 0.032662027, 0.0029181927, -0.09369264, -0.11556107, -0.11015105, -0.060980286) * inp_3_1_2;
    result2 += M4(-0.15717454, 0.033924058, -0.23681171, -0.027798487, 0.010795009, 0.019131351, 0.02392358, 0.011456643, 0.021594932, 0.039922684, -0.014195072, 0.053828575, -0.07143607, 0.038203824, 0.08080364, -0.050651602) * inp_3_2_2;
    result3 += M4(0.14709549, -0.038960278, -0.05146012, 0.051677316, -0.09873203, 0.03697199, -0.26135147, 0.07909529, -0.008876248, 0.03478469, -0.08694066, 0.037252173, -0.67826945, -0.6997805, -0.54191256, 0.32937688) * inp_3_0_0;
    result3 += M4(0.045945723, -0.042127613, 0.15358572, 0.071041, 0.080468796, -0.027574968, 0.10278041, -0.10515952, 0.1150073, -0.06955663, 0.037964962, -0.024689069, 0.2845443, 0.08512697, -0.148276, 0.13342027) * inp_3_1_0;
    result3 += M4(0.039191306, -0.0543566, -0.13979529, -0.052024636, -0.1039837, 0.013920217, -0.03190331, -0.00980454, 0.10108319, 0.00019699484, 0.05198158, -0.06816528, -0.13689181, -0.041516773, -0.06226383, -0.12635574) * inp_3_2_0;
    result3 += M4(-0.09795378, 0.05234306, 0.14030433, 0.16223449, 0.012504339, -0.030090153, -0.18578024, 0.29850656, 0.10250324, -0.041253135, 0.023546701, -0.071078084, -0.06452443, 0.21764398, -0.30965805, 0.27687293) * inp_3_0_1;
    result3 += M4(0.26022616, 0.119523026, -0.12516199, -0.24437857, 0.15796712, 0.04759658, 0.17135735, -0.11564203, -0.06326903, -0.01215352, 0.13099143, 0.17910825, -0.017280636, 0.3921076, 0.36526677, -0.41410106) * inp_3_1_1;
    result3 += M4(0.04715267, -0.07045792, 0.12650323, 0.09220016, 0.09766924, 0.031261705, 0.059097026, 0.044854555, -0.1270552, 0.011052187, 0.049074985, 0.03223367, 0.0777757, 0.098906234, -0.05155498, 0.14048177) * inp_3_2_1;
    result3 += M4(-0.05699991, 0.05446854, 0.09012197, -0.041093938, -0.20007184, 0.012315592, -0.05860979, -0.041183356, 0.0009808992, -0.01105508, 0.01780747, 0.016802052, -0.0031982667, -0.0004696341, 0.07593974, -0.16397713) * inp_3_0_2;
    result3 += M4(-0.012193457, -0.032062944, 0.12472864, 0.028595569, -0.22230746, -0.0038936404, 0.010451105, -0.11922933, 0.072720245, 0.07976997, -0.0797132, 0.028267184, -0.13464998, 0.06560637, -0.21103032, 0.16999657) * inp_3_1_2;
    result3 += M4(-0.01809862, -0.0044874507, -0.08478642, -0.19345094, -0.033588946, -0.050840005, -0.079772405, -0.029397968, 0.0019217303, -0.020709718, -0.048139308, -0.10960752, 0.10694534, -0.03005909, 0.12134001, -0.039176997) * inp_3_2_2;
    result4 += M4(-0.22681917, 0.048896093, 0.05858589, 0.07275371, 0.31903198, 0.20294596, 0.28141803, -0.23471862, 0.10478505, -0.18525617, 0.009647084, 0.04328329, 0.009500673, 0.248304, 1.1928518, 2.8094654) * inp_3_0_0;
    result4 += M4(0.079156935, -0.23026602, -0.20330574, -0.13539071, -0.28505364, 0.1882178, -0.02135058, -0.09897718, -0.04360079, 0.1833842, 0.004596309, -0.07140752, 0.04903525, -0.1291695, -8.6287146e-05, 0.12687106) * inp_3_1_0;
    result4 += M4(0.09309504, 0.15388936, 0.16333283, 0.1950748, -0.20155287, 0.16679457, 0.15538387, -0.005755977, -0.06685386, 0.022120768, 0.03324897, -0.027611794, -0.050734546, 0.12757985, -0.0564725, 0.1809795) * inp_3_2_0;
    result4 += M4(0.024820983, -0.13189544, 0.110988215, -0.0224392, 0.08793148, 0.0992465, 0.09922301, -0.06401229, -0.25817153, -0.09340855, -0.015671303, 0.044794675, 0.10302639, 0.028476393, 0.16613643, 0.19118007) * inp_3_0_1;
    result4 += M4(-0.069128, 0.108724065, -0.28168046, -0.25890198, 0.3451382, 0.1531076, -0.59845614, 0.06701137, 0.16036798, -0.097862974, -0.021863006, -0.08076757, 0.024759483, -0.14002825, -0.38646042, -0.18105546) * inp_3_1_1;
    result4 += M4(0.09187516, -0.09900953, 0.12167535, 0.15696175, 0.14512329, -0.20516208, -0.07163517, 0.048659265, 0.052961394, -0.007677359, 0.060897604, 0.068089284, -0.064505406, 0.13736242, 0.2232474, -0.03505823) * inp_3_2_1;
    result4 += M4(-0.07801495, -0.077614814, 0.087086394, -0.037031747, 0.045535475, -0.13166164, 0.13849434, 0.017728256, -0.045413993, -0.0287876, -0.057747576, -0.0163802, 0.036483962, -0.052191902, -0.23864695, 0.09214348) * inp_3_0_2;
    result4 += M4(0.18900591, -0.09690658, -0.23193038, -0.1860512, 0.054377846, 0.02889169, 0.124596655, 0.07696116, 0.0076383087, 0.110024534, 0.060194857, 0.050250124, -0.05717693, 0.31348768, 0.39123145, 0.06438538) * inp_3_1_2;
    result4 += M4(0.06623723, 0.015325458, 0.123916484, 0.20632483, 0.09890677, -0.05502257, 0.017570946, 0.07055507, 0.055907592, -0.022501519, 0.0053108004, -0.093870506, 0.089598134, -0.24578793, -0.16441973, -0.10990344) * inp_3_2_2;
    result5 += M4(0.05241836, 0.05818583, 0.03728926, 0.054598987, -0.10718471, 0.45330194, -0.09602169, -0.09518871, -0.044176217, 0.018283427, -0.06288891, 0.0627268, -0.16080052, 1.37205, -1.5665834, 0.53008) * inp_3_0_0;
    result5 += M4(-0.0036514292, -0.21400285, 0.119203664, 0.030715868, 0.09889202, 0.5381365, -0.12782423, -0.06765696, 0.08405428, 0.15223254, -0.12417311, 0.04515441, 0.022893367, -0.20520988, -0.0638635, -0.17096448) * inp_3_1_0;
    result5 += M4(-0.020046506, 0.21904597, -0.08019009, -0.15496424, -0.027290065, 0.20619743, -0.06530918, -0.06443043, -0.03887176, -0.030556997, -0.007254748, 0.027741164, -0.024711467, 0.04160179, -0.1412107, -0.28570858) * inp_3_2_0;
    result5 += M4(0.049606677, 0.045444287, -0.00043521542, -0.029492633, -0.0415081, 0.73443377, 0.26997942, -0.014692581, 0.06220905, 0.040376946, 0.10145144, 0.062324002, 0.16336842, -0.052223958, 0.09970001, -0.5028943) * inp_3_0_1;
    result5 += M4(0.06523728, -0.085263774, 0.0011326718, 0.053761654, -0.0035965028, 0.95743746, 0.7477119, -0.035765063, -0.08751125, -0.03753457, 0.155604, 0.07577924, -0.07161507, 0.16847187, -0.5257671, 0.24313071) * inp_3_1_1;
    result5 += M4(0.1119954, 0.0005643336, 0.048740946, -0.19827154, -0.06813625, 0.30657402, -0.13912651, 0.11201025, -0.019914135, -0.0711925, -0.00522174, -0.06515151, -0.028999826, -0.045516882, 0.43353838, 0.2962083) * inp_3_2_1;
    result5 += M4(-0.003700008, 0.14813228, -0.053706866, -0.06243477, -0.058179285, 0.20797543, -0.02549225, 0.022856962, 0.0577882, -0.07467874, -0.012531976, 0.034662683, 0.0042932522, 0.06695607, -0.30564892, 0.12487173) * inp_3_0_2;
    result5 += M4(-0.03325752, -0.07819879, 0.019601706, 0.054629367, -0.023959804, 0.12855506, 0.05682647, 0.04618856, 0.015098086, 0.056614436, -0.05824183, 0.02851005, -0.017276246, -0.024486104, 0.6316854, -0.062992185) * inp_3_1_2;
    result5 += M4(-0.024946105, 0.063696474, -0.16072622, 0.14022864, -0.09328802, 0.06931399, -0.14064582, 0.10136449, 0.006875657, -0.059277408, -0.08602614, -0.023080131, 0.013205498, -0.032794517, -0.29733187, -0.06958811) * inp_3_2_2;
    result6 += M4(0.12854388, -0.04456958, -0.0072816922, 0.12903635, -0.21612078, -0.3322425, -0.0050730444, -0.09321873, 0.010334773, 0.09166321, -0.021678204, 0.081564285, -0.90639555, -1.3642967, 0.118520565, 1.3951122) * inp_3_0_0;
    result6 += M4(-0.22710419, 0.0004937966, 0.049306877, 0.13005526, -0.1881056, 0.098229334, 0.096403666, -0.07955791, 0.032411866, -0.026701204, -0.11125668, 0.056110315, 0.054371286, 0.049510937, 0.0761009, 0.1341935) * inp_3_1_0;
    result6 += M4(0.028192602, 0.10552379, -0.07272914, 0.19746882, -0.033354077, 0.15418604, -0.028725775, -0.015583128, -0.070572436, -0.082333155, -0.06963208, -0.026125325, -6.780798e-05, 0.05299896, -0.02537679, -0.03267686) * inp_3_2_0;
    result6 += M4(0.04744776, 0.015132809, -0.095310256, 0.24545729, -0.10989459, -0.016783996, 0.026525462, -0.14574361, -0.082028136, -0.16129042, 0.060700297, -0.09062467, -0.15206432, 0.26323137, -0.10646283, -0.07691558) * inp_3_0_1;
    result6 += M4(-0.011437579, -0.021990674, -0.5266631, 0.26648933, -0.17621346, 0.12025797, -0.06729357, 0.17881857, 0.06876631, 0.0035882443, 0.14502385, 0.027698435, -0.15092932, -0.23471901, -0.17506269, -0.14121222) * inp_3_1_1;
    result6 += M4(0.2037862, -0.19341804, -0.021347212, 0.20856364, -0.043693602, 0.02009561, 0.04480623, 0.037669368, 0.045374718, 0.029160835, -0.09947019, -0.08149922, 0.24401972, 0.02319867, -0.10327185, 0.17808697) * inp_3_2_1;
    result6 += M4(-0.06832205, 0.03894866, -0.12460161, 0.04387876, 0.011844074, 0.048379514, 0.030239442, 0.013385601, -0.004455774, 0.12155058, 0.072129086, -0.020920634, 0.2028045, -0.06923312, 0.015208354, 0.023130765) * inp_3_0_2;
    result6 += M4(-0.05488779, -0.04377326, 0.03005286, -0.10001797, 0.1299602, -0.06783736, -0.005428693, 0.0853495, 0.1304117, -0.02187177, -0.062692806, -0.067774706, 0.100029826, 0.18460256, 0.025708377, 0.13719384) * inp_3_1_2;
    result6 += M4(0.18560651, -0.06309304, -0.013492998, 0.057805818, 0.07458863, -0.11406334, 0.025546934, 0.023308642, -0.06348876, -0.039802924, -0.06916206, -0.044550523, -0.27049333, -0.11332728, 0.015863286, -0.068568125) * inp_3_2_2;
    result7 += M4(0.06262835, -0.023336228, 0.31419337, -0.039741818, 0.048795898, 0.03667468, -0.3905964, -0.085389815, -0.027950369, 0.048500452, -0.08914542, -0.15533838, -2.778815, 0.93203396, 1.3111796, 0.7031096) * inp_3_0_0;
    result7 += M4(-0.19161712, -0.059829056, 0.23010226, -0.08236014, 0.063287124, 0.02500517, 0.023533095, 0.099372156, 0.08781328, 0.009950076, -0.0054959105, 0.11845002, -0.12743093, 0.19588943, 0.047859106, 0.110294074) * inp_3_1_0;
    result7 += M4(-0.073567696, -0.14995852, 0.26951218, -0.056943815, -0.08648442, -0.014115944, 0.12211304, 0.058491867, 0.06854767, 0.002547312, -0.022824084, -0.07398149, 0.1208006, 0.016173385, 0.057769127, -0.08002236) * inp_3_2_0;
    result7 += M4(-0.06789037, -0.050543457, 0.22281863, 0.09938124, 0.23900862, -0.0073395176, -0.06551443, -0.2546602, 0.1577725, 0.048800405, 0.047557388, 0.029820088, 0.17885308, 0.058454048, 0.09116399, 0.028243342) * inp_3_0_1;
    result7 += M4(0.2652585, -0.31618658, 0.9376004, -0.32623643, -0.15105191, -0.18480718, -0.6792083, -0.25867605, -0.0003257738, 0.18517329, -0.013060582, -0.09979974, 0.14907216, 0.2147212, 0.14274046, -0.15623626) * inp_3_1_1;
    result7 += M4(-0.10080939, -0.015748903, 0.13926174, 0.005308318, -0.045732334, -0.034259308, 0.020246282, 0.09639418, -0.039349627, -0.025374329, 0.0076283948, 0.04559424, -0.23836333, -0.048923437, -0.04928491, 0.11696404) * inp_3_2_1;
    result7 += M4(-0.028694207, -0.040640015, 0.10578893, 0.021926876, 0.07852944, 0.017887186, -0.0039413185, 0.09334048, -0.03976701, 0.061952375, -0.058903467, -0.063233234, -0.14408205, 0.028767597, 0.009666105, -0.06596929) * inp_3_0_2;
    result7 += M4(0.08289807, -0.005074379, 0.046801075, -0.14128394, 0.041591868, -0.010919925, 0.028951606, -0.169852, 0.03666246, -0.058141258, -0.05563189, 0.16049115, 0.040358376, 0.05350756, -0.021063035, 0.048122473) * inp_3_1_2;
    result7 += M4(0.09126893, 0.006521611, 0.06726036, 0.12351346, -0.089014284, -0.0009596446, 0.082918495, 0.039711352, 0.007959468, -0.036518544, -0.016661452, -0.021520426, 0.13794026, -0.009214462, -0.05247854, -0.023244072) * inp_3_2_2;
    const V4 inp_4_0_0 = inp[4][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_4_1_0 = inp[4][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_4_2_0 = inp[4][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_4_0_1 = inp[4][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_4_1_1 = inp[4][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_4_2_1 = inp[4][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_4_0_2 = inp[4][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_4_1_2 = inp[4][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_4_2_2 = inp[4][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.050314486, 0.01234508, 0.0457722, 0.065427944, 0.10599044, 0.13737154, -0.15358216, 0.014507715, 0.22273019, -0.0745818, -0.36552805, -0.052129276, 0.07893248, 0.06993732, 0.09661815, -0.030138606) * inp_4_0_0;
    result0 += M4(0.039770063, 0.19628088, -0.17120944, -0.0022827087, -0.15662043, -0.03817782, -0.04320678, -0.039857432, 0.35841236, -0.21556313, 0.027892392, -0.00020257189, -0.14289343, -0.073521286, 0.14398922, -0.06262079) * inp_4_1_0;
    result0 += M4(0.077135935, -0.1077025, -0.086328216, 0.020270165, -0.003280647, 0.14353968, 0.0051606074, 0.08842516, -0.10588579, 0.09384506, -0.057823297, -0.09360177, 0.08638457, 0.017500201, 0.18482074, -0.01315717) * inp_4_2_0;
    result0 += M4(-0.042716302, -0.04656139, 0.14364421, 0.07412452, 0.020199765, -0.08736418, -0.16710861, 0.04464258, 0.23737006, -0.16228607, -0.32016775, 0.17725262, 0.16141345, 0.10004551, -0.21341273, 0.09857487) * inp_4_0_1;
    result0 += M4(0.10219227, -0.028318807, 0.21816072, -0.100884624, 0.064651325, 0.20567678, -0.06154518, 0.25284714, 0.16365264, -0.13636969, -0.29824463, 0.008617155, -0.050910693, -0.27114284, -0.03548552, 0.0459725) * inp_4_1_1;
    result0 += M4(0.024413101, -0.05155971, -0.08833155, -0.031573948, -0.0511423, 0.10961698, 0.21711741, 0.10507809, -0.12837403, 0.058955915, 0.049740165, 0.029437091, -0.11294196, 0.076249614, 0.14179721, 0.023166858) * inp_4_2_1;
    result0 += M4(-0.0025099055, 0.003633794, 0.04386379, -0.01063574, 0.03688277, 0.0062004193, -0.044679273, 0.02352445, -0.17047045, -0.11754515, 0.06412983, 0.016478866, -0.078005716, 0.010679334, 0.09917406, -0.033320814) * inp_4_0_2;
    result0 += M4(-0.034873217, 0.042440142, 0.11589704, 0.047333453, 0.056741662, -0.002068451, 0.053238243, 0.081415616, -0.0008803987, 0.118196316, 0.066697374, 0.025312088, -0.027313478, 0.053740446, -0.25856307, 0.0005163699) * inp_4_1_2;
    result0 += M4(-0.018613294, 0.0022109363, -0.046476234, 0.010934204, 0.030433407, -0.010128401, 0.0087163625, 0.061458547, -0.28266367, 0.17486572, 0.104012705, -0.032123875, -0.02318217, 0.025184346, -0.18358694, 0.07033524) * inp_4_2_2;
    result1 += M4(0.10876419, -0.018713888, 0.05525598, -0.07905964, -0.016760094, -0.055106446, -0.18350153, 0.03858175, -0.3686307, -0.21639915, -0.3062961, 0.29383525, -0.2934152, 0.10350974, -0.09274966, -0.10570905) * inp_4_0_0;
    result1 += M4(-0.038883902, -0.12332278, -0.054555245, -0.14097747, -0.09927568, -0.067464046, 0.04371749, 0.028573874, 0.14472227, -0.235556, -0.005695101, 0.2746705, -0.036943946, 0.13304406, 0.0674111, 0.046691988) * inp_4_1_0;
    result1 += M4(-0.08413653, 0.038313553, -0.016500756, 0.015080631, -0.0053587304, 0.15481783, 0.07242327, 0.23102944, 0.21619076, 0.058674272, 0.08042733, -0.013933975, -0.06107515, 0.0075748055, 0.023280878, -0.0031307358) * inp_4_2_0;
    result1 += M4(-0.05762286, 0.0107485205, 0.09466851, 0.014127581, -0.029796483, -0.036114477, -0.022773765, 0.01710817, -0.35827044, -0.23759025, -0.028421752, -0.0570124, 0.32149196, 0.084289856, 0.02584352, -0.12548861) * inp_4_0_1;
    result1 += M4(0.009412653, -0.0213026, 0.09737437, -0.122524485, -0.19554894, 0.11490311, -0.006996998, -0.18885149, -0.22277355, -0.0033560055, -0.16668917, -0.06646127, 0.18238421, 0.025499698, -0.19925705, -0.10191142) * inp_4_1_1;
    result1 += M4(-0.13254017, -0.094938055, 0.21372685, -0.114378326, 0.22877538, -0.16579336, 0.026951328, 0.29152313, 7.889076e-06, -0.07967906, 0.02032147, -0.050151624, 0.07939679, -0.1382793, 0.0023208978, 0.093699455) * inp_4_2_1;
    result1 += M4(0.09454636, -0.027046358, -0.052100107, 0.08319277, -0.019166987, -0.008003821, 0.0057145595, 0.058287248, 0.10011434, -0.10978893, 0.10332575, -0.027472321, 0.052350108, 0.15722539, 0.0037066296, 0.037944067) * inp_4_0_2;
    result1 += M4(0.18466635, -0.07065492, -0.059519242, 0.12892711, -0.0138833765, -0.0436406, 0.008561208, -0.019860951, 0.15702333, 0.15033188, -0.15561579, 0.023609601, -0.028739689, 0.071547024, 0.033674624, -0.079861484) * inp_4_1_2;
    result1 += M4(-0.05239561, -0.0087667145, -0.031508468, 0.09287855, 0.0010393894, -0.009542036, 0.03492512, -0.029760301, 0.0680089, 0.07456965, -0.14068927, 0.06097413, 0.10328973, -0.10449501, 0.11792154, -0.03488784) * inp_4_2_2;
    result2 += M4(-0.04241004, -0.051970974, 0.1252291, -0.07854183, -0.06994528, 0.02533896, 0.03725961, -0.036436375, -0.009249693, 0.06887217, 0.11542745, -0.19510259, 0.030087115, 0.05601046, -0.15403998, -0.06312867) * inp_4_0_0;
    result2 += M4(-0.060738724, 0.04702828, -0.05639101, -0.018317988, -0.22372647, 0.06637358, -0.024865482, 0.05798952, 0.36988547, 0.1544741, -0.08685895, 0.27795777, -0.0018320075, -0.1224612, 0.22758566, -0.18821333) * inp_4_1_0;
    result2 += M4(-0.03070947, 0.10711961, -0.06170394, 0.007274309, -0.12198121, 0.044851538, -0.06892911, 0.0027152998, 0.12944102, 0.021727921, 0.06680537, -0.090738356, 0.0019482749, -0.07405838, 0.035215627, -0.015741834) * inp_4_2_0;
    result2 += M4(-0.1511379, -0.021001915, 0.15617271, -0.09707135, -0.07296969, -0.07858663, 0.09227006, -0.036523245, -0.12203116, -0.096438, 0.37528032, 0.031529967, 0.070239566, 0.20241193, -0.08763888, 0.07876687) * inp_4_0_1;
    result2 += M4(-0.12654379, -0.074759364, -0.06434909, -0.2770474, 0.1931326, 0.18807587, -0.17441466, 0.13158914, -0.2596258, -0.076099105, -0.26584417, -0.2392154, 0.21227357, 0.30166617, 0.03491981, 0.43113837) * inp_4_1_1;
    result2 += M4(-0.1318002, 0.02610113, -0.013975038, -0.14477696, -0.13165857, 0.15372238, -0.029648107, -0.03282238, -0.05596789, -0.078234896, -0.09678245, -0.00857379, 0.03578323, -0.104422234, 0.08834135, 0.13134867) * inp_4_2_1;
    result2 += M4(-0.04936694, -0.008886069, 0.070290014, 0.025139544, -0.041393716, -0.034299247, 0.026748262, 0.10463046, -0.063401416, -0.013477538, 0.08126181, -0.05263043, 0.03287632, 0.011458612, -0.11308612, 0.12452631) * inp_4_0_2;
    result2 += M4(-0.13533235, 0.06055579, 0.1504198, 0.015584058, -0.016025532, 0.060449418, 0.03684478, -0.011277718, 0.037037738, -0.063721456, 0.06704209, 0.023367029, 0.062291127, -0.064369015, -0.15066057, 0.05868575) * inp_4_1_2;
    result2 += M4(-0.095467605, -0.014950098, -0.14068195, -0.054416373, -0.06487351, -0.014398203, -0.0087023685, 0.09169875, -0.06743584, 0.09660204, -0.021951767, -0.08161332, 0.022165446, 0.03748225, -0.010950247, 0.027711654) * inp_4_2_2;
    result3 += M4(-0.038466305, -0.048420474, -0.0033154374, 0.03104047, -0.14385304, 0.029913161, -0.0734465, 0.09555101, 0.2754125, 0.17113881, 0.032221176, 0.31740078, 0.0839535, 0.045786478, 0.06120702, -0.0017205643) * inp_4_0_0;
    result3 += M4(0.012390657, -0.004267318, 0.004063183, -0.2136307, -0.105523914, -0.17755507, -0.0096868025, -0.23541726, -0.03151082, -0.20269887, 0.19814453, 0.092139564, 0.15225443, -0.0050363913, 0.10169794, -0.1680558) * inp_4_1_0;
    result3 += M4(0.04676184, -0.013572237, -0.034279596, 0.07275749, -0.39257613, -0.0030880494, 0.0056883907, -0.15479928, 0.20831192, -0.03783311, -0.23507376, -0.05043479, -0.20758668, 0.0149495285, 0.12896043, -0.10484679) * inp_4_2_0;
    result3 += M4(0.099481516, 0.05170134, 0.035139013, 0.08719981, -0.18515538, -0.031856973, -0.2356558, 0.24436462, -0.21255186, -0.18600169, -0.06260256, -0.026059091, 0.06655129, 0.023919744, -0.0681049, 0.07967868) * inp_4_0_1;
    result3 += M4(-0.0477429, 0.02366304, -0.19386125, -0.030050987, -0.38758782, 0.07365075, 0.048590638, -0.53434926, 0.069132045, 0.117488205, 0.09901891, 0.058560994, 0.20070566, -0.12342434, -0.11637098, 0.027316516) * inp_4_1_1;
    result3 += M4(-0.0751933, -0.031965736, -0.041705582, -0.011610203, -0.031832762, -0.04131048, 0.08148654, 0.13121375, 0.097492315, 0.10774024, -0.055765245, 0.086808115, 0.06711002, -0.0006698914, -0.0047123814, 0.094381385) * inp_4_2_1;
    result3 += M4(-0.01478517, 0.0075592627, 0.0926936, 0.0156203015, -0.0819872, 0.054843403, 0.10038314, 0.0062380396, -0.067249924, 0.049502574, 0.19474524, -0.023273556, 0.029289002, 0.011131403, -0.08716658, 0.1264829) * inp_4_0_2;
    result3 += M4(0.055194847, -0.03452229, 0.042056944, 0.09602126, -0.0829617, 0.01325802, -0.122632906, 0.009351789, -0.14460516, 0.026921807, -0.044622984, 0.17178294, 0.06557392, 0.012406226, -0.049625937, 0.10203955) * inp_4_1_2;
    result3 += M4(-0.10046821, 0.03926378, 0.015313615, 0.06510726, -0.089425966, 0.06273256, -0.06609017, -0.07925431, 0.16031544, -0.04636524, 0.103885785, 0.0010368172, 0.0483098, -0.014835474, 0.022912472, 0.02595467) * inp_4_2_2;
    result4 += M4(-0.051971354, -0.030057078, -0.00856703, 0.06857074, 0.045456238, -0.11957306, -0.08495408, -0.06332182, 0.39350715, 0.04092034, -0.41325182, -0.7284459, 0.13343152, -0.10771549, -0.08072162, -0.04080176) * inp_4_0_0;
    result4 += M4(0.104353175, 0.028828306, -0.07784826, -0.109541856, 0.29681656, 0.1431006, -0.06644197, -0.16426462, -0.47438833, 0.135164, 0.01851083, -0.0708293, 0.015924077, -0.18517691, -0.010520964, 0.037061766) * inp_4_1_0;
    result4 += M4(0.02329624, -0.059905037, -0.001158056, 0.060279515, -0.12337603, 0.1490887, 0.065391816, -0.15067191, -0.10804108, -0.08704777, 0.06263262, 0.09758856, 0.008271893, -0.19155973, -0.019344829, -0.23779051) * inp_4_2_0;
    result4 += M4(-0.040383235, 0.014539466, 0.20419535, 0.017586254, 0.21450438, -0.07598251, 0.054439906, -0.18443243, 0.7058328, -0.15478933, -0.26817232, -0.075398184, -0.12987597, -0.11180428, -0.16016415, 0.090252735) * inp_4_0_1;
    result4 += M4(-0.025003128, -0.094863825, -0.12733677, 0.22197942, -0.01753702, 0.101473205, -0.1268269, -0.012144508, -0.18404919, 0.3605211, 0.38694462, -0.14390647, -0.0114573045, -0.020211572, 0.4119293, 0.06406852) * inp_4_1_1;
    result4 += M4(0.034357924, 0.1046409, 0.040349703, 0.052726153, -0.14756201, -0.010217898, 0.053390827, -0.0069590895, 0.19165549, -0.1911627, 0.045866963, -0.06721711, -0.12114117, -0.0016680553, -0.035197895, -0.061008662) * inp_4_2_1;
    result4 += M4(0.0075863055, 0.06740563, 0.06576517, 0.053734906, -0.07547277, 0.04562575, 0.044907346, 0.096324205, 0.021987444, 0.037032384, 0.021406323, 0.3597881, -0.075514585, 0.11609543, -0.13167879, 0.040407114) * inp_4_0_2;
    result4 += M4(-0.04196798, 0.06882697, -0.045744628, -0.06511294, 0.079105124, 0.0196253, -0.01632612, -0.08244511, -0.092610024, 0.16187759, 0.15594861, 0.029257182, -0.091281414, -0.11582116, -0.011483707, -0.047566216) * inp_4_1_2;
    result4 += M4(-0.011992499, -0.039244182, -0.011173996, 0.03229294, -0.014772491, 0.0453693, 0.054795623, 0.08455781, -0.103942044, -0.3249534, -0.07013658, -0.06930721, -0.25036445, 0.109886564, -0.11417401, -0.19054131) * inp_4_2_2;
    result5 += M4(0.04523882, 0.014035597, -0.027614705, -0.01869908, -0.0038880017, -0.029801875, -0.01428979, 0.075592555, -0.13672131, -0.47235006, 0.137067, 0.25084662, -0.15039323, -0.18641676, 0.07839217, -0.045998335) * inp_4_0_0;
    result5 += M4(0.17255065, -0.1005664, -0.004146184, 0.019907631, -0.06755015, 0.10374345, 0.044869468, 0.17929927, 0.24177468, -0.46303627, 0.060272284, 0.16783129, -0.056775495, -0.20253184, 0.18542074, 0.03122791) * inp_4_1_0;
    result5 += M4(0.017796598, 0.16340849, -0.009486969, -0.045010485, 0.05688953, -0.12252728, -0.07580484, -0.35149127, 0.026533071, -0.13132745, 0.2639782, 0.2950266, 0.03707115, -0.09807798, -0.027287098, -0.12102393) * inp_4_2_0;
    result5 += M4(-0.13729733, -0.050293133, 0.045356806, 0.008545758, -0.015021369, 0.05525791, -0.031681906, 0.053325176, 0.09683326, -0.22982362, -0.022306168, 0.15918605, 0.08517965, -0.13249534, -0.2087286, 0.202616) * inp_4_0_1;
    result5 += M4(-0.059304778, -0.13179821, -0.009633437, 0.021820739, -0.019878745, 0.01507535, 0.075475365, -0.0079796715, -0.027411414, -0.18563479, 0.009816882, 0.12576964, -0.11322986, -0.42701715, -0.45415664, 0.16022944) * inp_4_1_1;
    result5 += M4(0.009407946, 0.23817445, -0.14164212, 0.10689272, 0.12767021, -0.052823093, 0.027530337, -0.20308916, -0.06494343, -0.43037236, -0.08118659, 0.15769126, -0.013227257, -0.14357683, -0.10617275, 0.1346855) * inp_4_2_1;
    result5 += M4(0.018106667, 0.107072406, 0.06468745, -0.012952368, 0.03373819, 0.12636316, 0.03807123, 0.010221058, -0.06875839, -0.08922705, 0.049337614, 0.055934556, 0.046628024, -0.21029188, 0.061625082, -0.025670966) * inp_4_0_2;
    result5 += M4(0.053393617, 0.019727977, 0.21390432, -0.07380181, -0.03237604, -0.029432891, 0.054877944, -0.13903412, 0.18150496, -0.2916059, 0.24972288, 0.083154544, 0.05077849, -0.20197356, 0.051408466, -0.013108134) * inp_4_1_2;
    result5 += M4(-0.035064958, -0.035339367, 0.0579716, -0.06826239, -0.012168386, 0.048543092, -0.09960947, 0.11088454, 0.024335926, -0.13684155, 0.0582779, -0.15724218, -0.027689664, -0.054121073, 0.25361586, -0.21427661) * inp_4_2_2;
    result6 += M4(0.022357361, 0.0060698185, -0.0018320492, 0.14084259, 0.06366229, 0.014898557, -0.03532067, 0.023723267, 0.028259428, 0.016249994, -0.0414946, -0.112299405, -0.090254374, 0.118973665, -0.0032364286, -0.07029115) * inp_4_0_0;
    result6 += M4(-0.00055133353, -0.025690123, 0.017996335, 0.085975245, -0.15294737, -0.14005744, -0.0093360115, 0.32200125, -0.5382236, -0.006003871, -0.259192, 0.03162183, 0.061575353, -0.03156504, -0.012554606, -0.12802255) * inp_4_1_0;
    result6 += M4(0.03190346, -0.04393057, -0.056694962, 0.0887951, -0.01268922, 0.10793231, 0.007995258, 0.16885896, -0.10299059, 0.07866845, -0.016785078, -0.07035226, 0.0026713533, -0.07897749, 0.06990366, -0.020543266) * inp_4_2_0;
    result6 += M4(-0.00432231, -0.023721328, -0.033426847, 0.14804222, 0.04589077, -0.044256367, -0.045301996, 0.30734175, -0.20869802, 0.27493635, 0.1513266, 0.26353687, 0.2718698, 0.08411265, -0.04304941, 0.023238488) * inp_4_0_1;
    result6 += M4(0.16267973, 0.055406436, -0.13176923, 0.12088659, -0.05925067, -0.084320255, -0.26876292, 0.33397296, 0.12649505, -0.051230896, 0.0024079205, -0.07916788, 0.009476324, 0.11380852, -0.025649177, -0.19457513) * inp_4_1_1;
    result6 += M4(0.01998658, -0.058462504, -0.021836383, 0.14079387, -0.1493189, 0.04717878, -0.1994397, 0.50931543, 0.14404601, 0.11609858, 0.07437822, -0.035634175, -0.14503942, 0.055725407, 0.03104328, -0.0119770905) * inp_4_2_1;
    result6 += M4(-0.05144618, 0.042796895, -0.034058366, -0.01944377, -0.012347231, -0.056729257, -0.034330692, -0.03312888, -0.11036136, -0.1517121, -0.064850226, -0.09936197, 0.13536645, -0.108620845, -0.028470427, 0.050970394) * inp_4_0_2;
    result6 += M4(-0.07983911, 0.14385565, 0.024980461, 0.053740017, 0.08087874, -0.02210347, -0.027124057, 0.14446402, -0.13037325, 0.08284029, 0.008163526, 0.115291335, -0.04296792, -0.026227994, -0.014239964, -0.0060221287) * inp_4_1_2;
    result6 += M4(0.07519278, 0.019318055, 0.008301063, -0.051058263, -0.0011137571, 0.03113586, -0.060188908, -0.017951312, -0.089722745, 0.059174333, 0.010727206, 0.04637245, -0.077664346, 0.025243336, -0.027247408, 0.11167757) * inp_4_2_2;
    result7 += M4(0.006203741, -0.02110408, 0.20209372, -0.02095122, 0.21632837, -0.03785608, -0.17306596, -0.11994713, 0.01091347, -0.13207142, -0.09541232, 0.39879486, 0.01821866, 0.01475871, -0.08609195, 0.083435535) * inp_4_0_0;
    result7 += M4(-0.088869065, 0.0010978857, 0.04977248, -0.094951876, -0.15034996, 0.023742203, 0.17009325, 0.13932881, -0.40013286, -0.15605721, 0.40333644, 0.5991645, 0.26204374, 0.11318307, -0.13808233, 0.09531678) * inp_4_1_0;
    result7 += M4(0.08707347, -0.06925859, -0.006174647, -0.019288087, 0.254231, -0.022443911, 0.06412335, -0.18390864, -0.08960653, 0.04263897, 0.04518323, 0.076747656, -0.0019448333, 0.04994383, -0.0018550488, 0.0064543146) * inp_4_2_0;
    result7 += M4(-0.060749203, -0.016978195, 0.08614689, -0.0072958646, 0.09098939, 0.034455206, 0.068472296, 0.040796466, -0.022059431, 0.18368547, 0.20684825, -0.023893455, 0.052206982, -0.004656217, -0.09742264, 0.17157967) * inp_4_0_1;
    result7 += M4(-0.1958774, 0.07218177, 0.7706751, 0.11935993, 0.27473527, -0.06898811, 0.25300986, -0.23753232, 0.082698904, 0.1252019, 0.23108771, 0.11356903, 0.13474822, -0.24464428, -0.85052526, -0.06608236) * inp_4_1_1;
    result7 += M4(0.047384024, 0.014548741, 0.029615993, -0.07103475, 0.111372344, -0.05102259, 0.005812755, 0.052521482, -0.046601493, 0.02358833, 0.0800267, 0.0027481634, -0.07653078, -0.012280118, -0.051867466, 0.024447776) * inp_4_2_1;
    result7 += M4(-0.016285125, -0.010190128, 0.012608241, 0.039956845, 0.021137547, 0.014445392, 0.03236076, -0.029385816, -0.14870381, -0.02327879, 0.05524119, -0.14091633, -0.05264735, -0.01982693, 0.05054639, 0.06776419) * inp_4_0_2;
    result7 += M4(-0.043195173, 0.003960533, -0.005291654, -0.05768951, 0.038294356, -0.012357226, -0.0015514195, -0.062888525, 0.043617766, 0.053043064, -0.011007097, -0.031145167, 0.094881624, 0.010962549, 0.014070046, 0.05609763) * inp_4_1_2;
    result7 += M4(0.016113803, 0.036226247, 0.00832007, 0.11428632, 0.028862275, -0.021698035, -0.021876423, 0.054209854, 0.09162774, -0.036584496, -0.03593838, -0.02002227, 0.0437548, -0.040987175, -0.13039342, -0.072240554) * inp_4_2_2;
    const V4 inp_5_0_0 = inp[5][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_5_1_0 = inp[5][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_5_2_0 = inp[5][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_5_0_1 = inp[5][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_5_1_1 = inp[5][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_5_2_1 = inp[5][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_5_0_2 = inp[5][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_5_1_2 = inp[5][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_5_2_2 = inp[5][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.060006216, 0.051338457, -0.1356379, 0.014803663, -0.09446908, -0.03108302, -0.13464461, -0.10647349, 0.19205612, -0.0961507, 0.0575336, -0.09588016, 0.062313125, -0.014277212, 0.07210714, 0.016227871) * inp_5_0_0;
    result0 += M4(0.01605507, 0.047748994, -0.065637045, 0.018965797, -0.0043973676, -0.17345327, 0.15646543, 0.019460201, -0.032860532, 0.085014574, 0.13541313, -0.030924525, 0.13577916, 0.04366319, -0.08944115, -0.03383335) * inp_5_1_0;
    result0 += M4(0.13217252, 0.036078174, 0.0551169, -0.0004884041, 0.021122403, -0.18542485, 0.07149457, 0.0813162, 0.026441269, -0.119732656, 0.057807498, 0.039701637, 0.066245064, -0.07094625, -0.21258911, 0.018908761) * inp_5_2_0;
    result0 += M4(0.0064109145, -0.01412905, 0.005273177, 0.0014270407, 0.12937778, 0.0062073544, 0.14808878, -0.049906544, -0.04502105, 0.030816633, -0.0013636154, 0.017924918, 0.025214529, -0.0035770864, -0.07324624, 0.010499369) * inp_5_0_1;
    result0 += M4(0.094881184, -0.04120402, -0.0013042297, -0.03802255, -0.0158704, 0.5480363, 0.55488527, 0.12178526, 0.05075396, -0.14601867, -0.080347635, 0.18281753, -0.077796504, -0.043192767, 0.1767789, 0.07184297) * inp_5_1_1;
    result0 += M4(0.012176277, -0.067631505, 0.10069842, -0.16373612, 0.04172929, 0.04579092, 0.097056836, -0.047553778, -0.013088972, -0.0906896, -0.032544542, 0.0656517, -0.030621838, 0.113416076, 0.064315505, -0.075269215) * inp_5_2_1;
    result0 += M4(-0.033590812, -0.11615537, -0.0068121827, 0.008983084, 0.08790214, 0.015218817, -0.056778766, 0.008706411, -0.0424072, 0.07379789, 0.010512475, -0.0146028735, 0.02604632, -0.0028200247, 0.07545342, -0.0015447485) * inp_5_0_2;
    result0 += M4(-0.04166391, -0.06870566, 0.05645295, 0.028093493, -0.09483618, 0.05526712, -0.12175449, -0.05114078, -0.06271119, 0.06552764, 0.0040071206, -0.024484295, 0.07328326, -0.07046376, -0.026500022, -0.04683562) * inp_5_1_2;
    result0 += M4(0.053035755, 0.0027118525, -0.005926529, -0.016746268, 0.05169807, -0.11545641, 0.011448179, -0.12792341, 0.044689313, 0.05326881, -0.09675487, 0.023666857, 0.1527457, -0.0701695, 0.022476312, -0.07424084) * inp_5_2_2;
    result1 += M4(0.07963828, 0.06892003, -0.040989943, -0.024331434, -0.053242322, -0.08975051, -0.109221146, 0.17058529, -0.26779938, -0.060142647, 0.09400618, 0.018028969, -0.06165632, -0.028523747, -0.0026140541, 0.06986711) * inp_5_0_0;
    result1 += M4(0.045944743, 0.13105609, -0.04195356, -0.0007813257, 0.015230692, 0.13288274, -0.15994567, 0.005690768, 0.041673247, -0.1623967, -0.09000932, 0.04531145, 0.011136315, -0.10541336, -0.17788354, 0.075350426) * inp_5_1_0;
    result1 += M4(0.22847234, -0.06160067, 0.027882792, 0.028860154, -0.05133805, 0.027869798, -0.2835544, 0.017334921, -0.122641645, 0.013515018, -0.12371319, 0.03034828, 0.027930157, -0.016684854, -0.12697375, 0.16134712) * inp_5_2_0;
    result1 += M4(0.020142756, -1.927941e-05, -0.061541397, 0.027389081, 0.037529763, -0.19625229, -0.0477085, -0.0006815424, -0.06956382, -0.30814093, 0.1093834, 0.08196954, 0.018412687, 0.0049034227, -0.071522236, -0.09906333) * inp_5_0_1;
    result1 += M4(0.0025254209, 0.08461662, -0.10863721, -0.10648646, 0.020757826, 0.23461677, 0.3915563, -0.07266677, -0.12292739, 0.00950749, 0.36608243, -0.17928578, 0.063829854, 0.14849862, 0.1552905, -0.14441542) * inp_5_1_1;
    result1 += M4(-0.024538133, 0.2035267, -0.048922636, 0.117901646, 0.039696395, 0.032752898, 0.0074409535, 0.0029823536, 0.014334805, -0.127629, 0.022846986, -0.06666283, -0.044776887, -0.089453004, 0.241991, -0.35724002) * inp_5_2_1;
    result1 += M4(0.0034636972, -0.068192266, 0.038666252, 0.11542668, -0.049290262, -0.1196329, 0.15699552, 0.021263821, 0.07279237, 0.15476139, -0.06592486, -0.14970702, 0.012623246, 0.07010027, -0.016896404, 0.015794193) * inp_5_0_2;
    result1 += M4(0.08206393, 0.010437249, -0.14232656, -0.046939917, -0.20801674, -0.032821182, 0.13762687, 0.0147099905, 0.09015858, 0.037814267, -0.020965321, 0.15296611, 0.18317442, -0.18069711, -0.18530276, 0.01653442) * inp_5_1_2;
    result1 += M4(-0.04284563, -0.08933278, -0.10247962, 0.043698315, -0.020160325, 0.083970994, -0.06662534, 0.032811698, 0.13725589, 0.26297644, 0.21419713, -0.11177229, -0.06871391, 0.048744492, 0.07220273, 0.20978945) * inp_5_2_2;
    result2 += M4(-0.07713245, -0.065418236, 0.00716299, -0.08996499, -0.01969042, 0.021268727, 0.058164973, -0.012649964, 0.12777413, 0.11470955, -0.14383401, 0.10460179, 0.073846236, 0.04608856, 0.017249532, 0.03384963) * inp_5_0_0;
    result2 += M4(-0.15096812, 0.019776974, 0.07017029, -0.13830464, 0.025664104, -0.023559364, -0.0800207, 0.16022065, 0.05991604, -0.04665741, 0.05715144, 0.036761142, -0.033327658, -0.03485016, 0.002950758, 0.067006655) * inp_5_1_0;
    result2 += M4(-0.09289502, 0.07408314, -0.016914159, -0.112667486, 0.058399368, -0.009729449, -0.15065221, 0.12780264, 0.03604736, -0.07037989, 0.041715443, 0.11206407, 0.02208501, -0.010097963, 0.056994837, 0.07080009) * inp_5_2_0;
    result2 += M4(-0.0137733305, -0.13394767, 0.011147393, -0.08214214, 0.25403088, 0.022625461, 0.04883566, 0.12970187, 0.1918868, 0.08374957, 0.06907343, 0.044979226, 0.005162989, -0.09123652, 0.05507439, 0.058144234) * inp_5_0_1;
    result2 += M4(-0.011945414, -0.43619463, 0.064639196, -0.60817975, -0.18311074, 0.20710835, 0.6122167, -0.057372365, -0.04401767, 0.1973085, -0.15356298, 0.7708075, 0.036085315, 0.1548403, -0.012312283, 0.3068632) * inp_5_1_1;
    result2 += M4(0.030310644, -0.0053522005, 0.10268526, -0.10041913, 0.07621824, -0.0510951, 0.0060293474, -0.0054940903, 0.013851128, -0.042485677, -0.050478768, 0.07992677, -0.007809865, 0.117320254, -0.47188503, 0.28303528) * inp_5_2_1;
    result2 += M4(0.048292674, -0.04386752, 0.02548576, -0.09910566, 0.063078955, -0.07744216, -0.09847889, -0.11375228, -0.06628071, 0.069998994, 0.01785976, 0.1290691, 0.04890646, -0.0022530453, -0.00619107, 0.06340019) * inp_5_0_2;
    result2 += M4(-0.09360558, -0.04043153, -0.059332956, -0.0485516, -0.11419343, -0.023961615, -0.1236366, -0.050545238, -0.028133864, 0.079955265, -0.046367828, -0.044644598, 0.010868189, -0.07178501, 0.25404808, 0.15108326) * inp_5_1_2;
    result2 += M4(0.0022142464, 0.04808157, -0.10345072, -0.12827481, -0.0071287, -0.043243516, -0.14125256, -0.0036773633, 0.0048549217, 0.04159889, 0.22169884, 0.027670724, 0.18523237, -0.07263681, 0.13853909, 0.116038784) * inp_5_2_2;
    result3 += M4(0.01717432, -0.059043147, 0.11249057, -0.05883103, 0.079433, -0.024761781, -0.0691184, -0.12085789, -0.047940716, 0.045114025, -0.12728441, 0.10848009, -0.02266582, 0.06666598, -0.12965828, 0.10718303) * inp_5_0_0;
    result3 += M4(-0.19417614, -0.0033068575, 0.124657504, 0.034090724, -0.09784054, 0.053972844, 0.12500095, 0.03298682, 0.12226574, 0.005166909, 0.017007522, -0.023043247, -0.10130208, -0.007934578, -0.2357841, -0.03450671) * inp_5_1_0;
    result3 += M4(-0.0056666774, 0.03866343, -0.06087406, 0.039166566, 0.012585813, -0.019162357, 0.26491466, 0.19254929, -0.013485724, -0.0068570212, 0.057693776, -0.023714513, -0.28954402, 0.011511472, -0.12408507, 0.15561543) * inp_5_2_0;
    result3 += M4(-0.11785917, -0.02086518, -0.12831908, -0.021049777, 0.15635872, 0.028433384, 0.18273707, -0.11933486, 0.06918442, 0.012480796, 0.029653281, 0.01967258, -0.12342627, -0.030199027, 0.053023938, -0.13182408) * inp_5_0_1;
    result3 += M4(-0.1758386, -0.025795437, -0.071409, -0.19624475, -0.35617992, -0.017861607, 0.2465045, -0.46943167, -0.16983508, 0.051218294, -0.041525893, 0.1023346, -0.13227595, 0.0724372, -0.01761062, 0.054726377) * inp_5_1_1;
    result3 += M4(-0.06767764, -0.011423712, 0.095119156, -0.050356142, 0.028599715, 0.027145555, 0.1817481, -0.15082371, -0.05114906, -0.013123228, -0.11084521, -0.06438881, -0.55457294, -0.12651418, -0.006600909, -0.15035349) * inp_5_2_1;
    result3 += M4(0.023733718, -0.012185901, 0.099888034, 0.0067554307, -0.07234834, 0.008720392, 0.08697535, -0.10199522, 0.02382701, -0.0047796927, -0.102603555, 0.017280119, -0.13637702, 0.035675284, -0.06991815, -0.06164001) * inp_5_0_2;
    result3 += M4(-0.18830544, 0.013767556, -0.10418165, -0.07870776, -0.039481793, -0.023803208, -0.16420686, -0.06678803, 0.044401105, -0.06931114, 0.0837857, -0.010615519, -0.2329233, 0.04129701, 0.15252022, -0.0571673) * inp_5_1_2;
    result3 += M4(0.05020073, 0.08359182, -0.0512533, -0.015735468, -0.16516295, -0.013045836, -0.18153533, -0.08347274, -0.134498, -0.0143158175, 0.09757725, 0.08304772, -0.3268521, -0.06872356, 0.03857117, -0.018556401) * inp_5_2_2;
    result4 += M4(-0.13398822, 0.045040496, -0.11990736, 0.14882995, 0.13264039, 0.0645091, 0.13738795, -0.23237723, 0.06696212, 0.011395548, 0.059863105, -0.060525592, -0.005734461, 0.017879449, 0.024269134, -0.07028675) * inp_5_0_0;
    result4 += M4(0.1778595, 0.103736795, -0.066252366, -0.048569024, -0.08732527, -0.047492348, 0.08553897, 0.04420672, -0.14824264, 0.012035076, -0.009419822, -0.012372238, 0.015761776, 0.13554534, -0.22324406, 0.05130929) * inp_5_1_0;
    result4 += M4(0.033530653, -0.018447803, 0.0032704365, -0.050860766, 0.0042602303, -0.0116046965, -0.00818638, -0.04652318, 0.02208079, -0.08741886, -0.02094995, 0.01638151, 0.21802767, 0.2195315, -0.13506813, 0.19101784) * inp_5_2_0;
    result4 += M4(0.044141714, -0.00449343, -0.004022785, -0.050571408, -0.052055363, 0.2631813, 0.17845911, 0.13389233, 0.16450189, 0.047811612, 0.13216323, -0.026650947, 0.024504753, -0.05025305, -0.016298095, 0.06865292) * inp_5_0_1;
    result4 += M4(-0.46852058, -0.16417654, 0.02382208, 0.04730238, 0.20489232, 0.10361384, -0.6850026, 0.29056582, 0.27040753, -0.0020886809, 0.06523787, -0.1519582, 0.14649981, 0.009111599, 0.18300433, -0.11186841) * inp_5_1_1;
    result4 += M4(-0.1124688, -0.1022959, 0.04065602, 0.050427742, 0.24764329, 0.18579854, -0.022006223, 0.010324465, -0.06318381, 0.024738666, -0.029427605, 0.020266837, 0.041670807, -0.08568493, 0.05212187, -0.130644) * inp_5_2_1;
    result4 += M4(-0.04606879, 0.026082916, 0.04425642, 0.09784094, 0.14761885, -0.16981429, 0.29392412, -0.027741868, -0.18755852, -0.06458092, -0.071278974, -0.035301212, -0.05567989, 0.07636759, 0.08086539, -0.006368391) * inp_5_0_2;
    result4 += M4(0.2636983, 0.0916876, -0.052101735, -0.056988288, -0.100290224, -0.2817294, -0.09799889, -0.16786379, -0.005241573, -0.11036578, -0.23806372, 0.070143595, -0.11044631, 0.0031729017, 0.08104728, 0.013405475) * inp_5_1_2;
    result4 += M4(-0.07240748, -0.019193437, 0.028618712, 0.040130604, 0.14539237, -0.16621837, -2.2932241e-05, 0.19882785, 0.1903116, 0.15838975, 0.006099575, -0.1345859, -0.11973544, 0.04532193, 0.25198057, -0.039118517) * inp_5_2_2;
    result5 += M4(0.10160007, -0.0409272, -0.08758475, 0.032967094, 0.034158625, 0.3661096, -0.15504639, -0.065023094, -0.05768097, -0.025777064, 0.013326563, -0.14401525, -0.06992515, -0.03765818, 0.05393158, 0.045157623) * inp_5_0_0;
    result5 += M4(0.05353364, 0.08400218, -0.044317905, 0.034080252, -0.1554515, 0.6751661, -0.21196613, -0.23417543, -0.031397052, 0.1025258, -0.08933949, -0.0041722925, -0.0606389, -0.1262364, -0.056258406, 0.03937086) * inp_5_1_0;
    result5 += M4(0.009667628, 0.027583841, -0.021644168, 0.053677138, -0.18415646, 0.36732435, -0.17994988, -0.08116993, -0.04648963, 0.0032785682, 0.03822919, -0.079822205, -0.035299074, 0.03951501, 0.0055581126, -0.20472696) * inp_5_2_0;
    result5 += M4(-0.042274594, -0.00060059363, 0.066484496, -0.033383388, 0.02002448, 0.6592751, 0.06373703, -0.12745811, -0.008014424, 0.0548892, 0.14888743, 0.067834415, 0.04274309, -0.019195383, -0.05811482, 0.0030089314) * inp_5_0_1;
    result5 += M4(-0.038117114, -0.21048737, -0.013941152, -0.061909426, 0.03201417, 1.2410605, 0.480464, 0.0036973092, -0.015715739, 0.0056875763, 0.17203212, -0.03507373, 0.036063213, -0.2320934, 0.04749117, -0.10700615) * inp_5_1_1;
    result5 += M4(0.009483963, -0.07707256, -0.068228655, -0.1518598, -0.006318204, 0.5741786, -0.093423754, 0.06681992, 0.13153887, 0.039594326, 0.016873175, 0.044870067, 0.12084339, -0.2572179, -0.04953113, 0.17855826) * inp_5_2_1;
    result5 += M4(-0.016160166, 0.00818932, 0.013923294, 0.078811616, -0.111003794, 0.32130086, -0.037595935, -0.06567015, 0.03867184, -0.11934763, -0.022738377, -0.09215811, -0.013070392, -0.0023669812, -0.0026311663, -0.0026946864) * inp_5_0_2;
    result5 += M4(-0.08933073, 0.03981249, 0.028608521, 0.11189308, -0.089736655, 0.5613718, -0.02305679, -0.026314491, -0.07142325, -0.030067312, -0.052221175, 0.07798329, -0.052400433, 0.19002117, 0.11071254, 0.10000112) * inp_5_1_2;
    result5 += M4(0.05430168, 0.06874963, -0.113561496, -0.07342253, -0.054534186, 0.35166642, -0.4279835, 0.00040821152, -0.099880114, -0.06001495, 0.12433029, 0.022602329, -0.19863832, 0.15768123, 0.08316875, -0.056680698) * inp_5_2_2;
    result6 += M4(0.04396745, -0.032179102, 0.019857962, 0.11110493, 0.069323525, -0.062142294, 0.0513787, -0.046482585, -0.12287739, 0.15062396, -0.033706684, -0.18631627, -0.004999007, -0.014654518, -0.008533977, -0.090186745) * inp_5_0_0;
    result6 += M4(0.017636957, -0.028191797, 0.086365804, 0.07621355, 0.1075214, -0.012319444, 0.029019903, -0.058231585, -0.13889089, 0.11891921, 0.027923094, -0.0823686, -0.106180474, -0.17009597, -0.097971916, -0.08217671) * inp_5_1_0;
    result6 += M4(-0.03197569, 0.03304773, -0.029919485, 0.016443394, 0.060255308, -0.05341709, -0.020935744, 0.012540056, 0.062573195, -0.04933599, 0.009330233, -0.036497086, 0.037246913, 0.031172711, -0.040498525, -0.042550884) * inp_5_2_0;
    result6 += M4(0.041485034, -0.0010558002, -0.04183039, 0.15123886, 0.07051096, -0.16674243, -0.00020806002, -0.14721352, -0.09162569, 0.04610281, 0.09941788, -0.3116322, -0.14299457, 0.009485541, -0.008458499, -0.07818155) * inp_5_0_1;
    result6 += M4(0.043327358, -0.09339998, 0.17643875, -0.04163621, 0.21807061, -0.23143324, -0.15677337, 0.25947353, -0.03192619, -0.07938315, -0.22639708, 0.1412958, 0.110910825, 0.16117749, -0.1607874, -0.094775505) * inp_5_1_1;
    result6 += M4(0.10936435, -0.0198091, -0.038519636, -0.10552097, -0.02701033, -0.039451785, 0.042806815, -0.024099771, -0.08485375, 0.046382178, 0.06354775, -0.0022612717, -0.09273428, -0.033198778, 0.005724264, -0.113244474) * inp_5_2_1;
    result6 += M4(-0.07948593, -0.062040344, 0.0706578, -0.0943274, -0.050031547, 0.090209424, 0.022237219, -0.04665414, 0.052895356, -0.062215958, -0.057567026, 0.07964553, 0.059634488, 0.013655144, -0.03534673, -0.0480554) * inp_5_0_2;
    result6 += M4(-0.034893002, -0.04421608, 0.047297053, -0.021670703, 0.08455597, 0.09163269, -0.004462279, 0.032197427, 0.09043312, -0.030472163, 0.02412043, 0.014636827, -0.0839409, 0.08212427, -0.089159325, -0.109508485) * inp_5_1_2;
    result6 += M4(0.007453848, 0.12818474, -0.024030864, -0.086025655, 0.106644794, 0.046073373, 0.058630485, -0.074429184, 0.05090478, -0.109204575, -0.01308373, 0.0736661, 0.12583348, 0.08617678, -0.01856052, -0.05379072) * inp_5_2_2;
    result7 += M4(-0.12766281, 0.016021073, 0.0865909, 0.11956309, -0.022326875, 0.033232573, -0.2454726, -0.25816867, 0.058734935, -0.014418147, -0.14357229, -0.11428162, 0.101113364, -0.018711885, -0.077563286, -0.005715511) * inp_5_0_0;
    result7 += M4(-0.010506642, 0.05798518, 0.04708602, -0.012176758, -0.055600084, 0.025422843, 0.06896623, -0.083224475, -0.039177537, 0.028706113, 0.071438566, -0.059650775, -0.07790696, -0.054761566, -0.025772005, -0.047592968) * inp_5_1_0;
    result7 += M4(0.022482507, 0.025243925, 0.007519672, -0.13438556, -0.0029496765, -0.044715684, 0.08082508, -0.1465609, 0.07353403, -0.019474024, -0.039952658, -0.0149236005, 0.056720767, -0.050660107, 0.0092651835, 0.056051232) * inp_5_2_0;
    result7 += M4(-0.060731303, -0.0074184337, 0.08416418, -0.09470244, 0.02990513, 0.03506835, -0.12711343, 0.13987108, -0.0010558577, -0.0018883292, -0.029144663, -0.12783635, -0.041724954, 0.025043169, 0.016691592, 0.030248532) * inp_5_0_1;
    result7 += M4(0.04089301, 0.29174444, 0.5162613, 0.08682666, -0.19680557, -0.27537352, 0.02959422, -0.14740811, -0.090062894, -0.31114155, -0.6071766, 0.0014867745, -0.013261365, -0.17911777, -0.23011614, 0.03779167) * inp_5_1_1;
    result7 += M4(0.10004105, 0.09157477, 0.21471123, 0.03582067, -0.10260297, 0.025994219, 0.06437832, -0.026433408, -0.089748986, -0.058038834, -0.1137816, 0.020067738, 0.08045719, -0.08715706, -0.053029597, -0.057712406) * inp_5_2_1;
    result7 += M4(-0.015363239, 0.032664306, -0.0055651185, -0.07387567, 0.11450702, 0.0028710342, 0.039371792, 0.019570272, 0.047342777, -0.03847094, 0.025803493, -0.048010312, -0.025331177, -0.020635718, -0.036168996, -0.03340886) * inp_5_0_2;
    result7 += M4(-0.1396737, 0.08666583, 0.08169268, -0.038987793, -0.35500798, 0.017643867, 0.09418417, -0.15801173, -0.015798695, -0.030158661, 0.011904652, 0.06749529, 0.17183068, -0.11409599, -0.08551782, -0.14142764) * inp_5_1_2;
    result7 += M4(0.009144551, 0.043256044, 0.13783818, 0.0035369005, -0.019409554, 0.048129804, 0.24021885, 0.06522175, -0.06811419, -0.014806173, -0.09878924, -0.044965036, -0.31675687, -0.000513752, 0.14731127, -0.11192457) * inp_5_2_2;
    const V4 inp_6_0_0 = inp[6][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_6_1_0 = inp[6][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_6_2_0 = inp[6][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_6_0_1 = inp[6][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_6_1_1 = inp[6][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_6_2_1 = inp[6][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_6_0_2 = inp[6][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_6_1_2 = inp[6][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_6_2_2 = inp[6][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.040462814, -0.024339769, 0.021413561, -0.054474153, 0.05933263, 0.025476193, 0.06695779, -0.04832442, 0.04523521, 0.061100185, -0.08939425, -0.007984827, 0.0133195305, 0.04855829, -0.1420152, 0.04178758) * inp_6_0_0;
    result0 += M4(-0.1327694, -0.04008416, 0.06835535, -0.017539399, -0.13914739, 0.10527704, -0.054120075, -0.007148084, -0.119803846, -0.08152622, -0.027807457, 0.0069850227, 0.16863738, -0.034373235, -0.019226959, -0.06814645) * inp_6_1_0;
    result0 += M4(0.06844765, -0.07259566, 0.25918603, -0.08527326, 0.04383688, 0.4720358, -0.25602505, -0.14347857, 0.020971833, 0.07554564, 0.039210528, 0.06326332, 0.024068356, -0.005530886, -0.059330493, -0.00636312) * inp_6_2_0;
    result0 += M4(-0.040138815, 0.09841651, 0.026550613, -0.0074075465, 0.06451848, 0.14974332, -0.04992221, -0.0337126, -0.038799133, -0.101282515, 0.034370605, 0.057351477, 0.012036736, 0.08812603, -0.18757668, -0.10960467) * inp_6_0_1;
    result0 += M4(0.06638068, 0.10710678, 0.040965654, -0.08124668, 0.10634146, -0.3764351, -0.004470531, -0.05897825, 0.03170006, 0.24787927, 0.22862872, -0.16903828, -0.21898033, -0.011926513, 0.073897526, -0.06944717) * inp_6_1_1;
    result0 += M4(-0.028329842, 0.09420027, -0.26092163, 0.14137979, -0.12921627, 0.1732689, 0.089857854, 0.089132465, -0.034759425, -0.058132377, -0.10264126, 0.01191087, 0.03780068, 0.023175376, 0.054817468, 0.07605407) * inp_6_2_1;
    result0 += M4(-0.06980556, -0.032848313, -0.07146062, -0.082942836, -0.11713902, -0.020826496, -0.072233476, -0.028253747, 0.23591363, 0.24789803, -0.21757245, 0.049612448, -0.075901926, -0.14317031, 0.09421988, 0.08277632) * inp_6_0_2;
    result0 += M4(-0.137034, -0.10726536, -0.030860132, -0.06599989, -0.09954242, 0.0122898575, 0.075431265, -0.1515194, -0.13026741, 0.05951265, -0.30759844, 0.13143231, 0.12930761, 0.04884657, 0.048700213, -0.09091741) * inp_6_1_2;
    result0 += M4(-0.30884537, 0.06435878, -0.16001415, 0.09870482, -0.019115973, -0.12989888, -0.015121728, -0.109920524, 0.007509987, 0.03287552, -0.14605962, 0.08380275, -0.08906046, 0.051766537, 0.039102886, -0.049122628) * inp_6_2_2;
    result1 += M4(-0.14648843, 0.0734577, 0.0748501, -0.12797248, 0.05737435, 0.0032557813, 0.06549489, 0.03980978, -0.051160518, -0.09005332, -0.09066695, -0.014092534, -0.17557573, -0.03917105, -0.13519001, -0.027090477) * inp_6_0_0;
    result1 += M4(0.100319736, 0.119441465, -0.01882651, -0.28050303, -0.20339584, -0.032790627, -0.12078931, -0.07901293, -0.061072327, 0.02261693, 0.0729885, -0.06399192, 0.2066175, -0.02290031, 0.12284697, 0.16825081) * inp_6_1_0;
    result1 += M4(-0.52791244, 0.15327848, 0.07540618, -0.2520839, 0.08680352, -0.016809903, 0.22695169, 0.048165414, 0.047355592, -0.00480857, -0.03637784, 0.016393913, 0.020222971, 0.02244744, 0.06100231, 0.013821898) * inp_6_2_0;
    result1 += M4(0.08640244, -0.14559485, 0.011496059, 0.123526104, -0.051937185, -0.07737947, -0.0549908, -0.07239142, 0.031495444, -0.006363273, 0.16137126, 0.21472214, 0.21110795, 0.07749806, -0.08967649, -0.03519603) * inp_6_0_1;
    result1 += M4(-0.08713857, 0.08168131, 0.14839828, -0.09235435, 0.41138464, 0.24105474, -0.09870033, 0.040305603, 0.32144395, -0.24083778, 0.40044835, -0.12771522, -0.2640638, -0.06857752, -0.3543689, 0.12639321) * inp_6_1_1;
    result1 += M4(0.13721627, -0.0673816, -0.10505163, 0.14619067, -0.019367607, -0.24512543, -0.17308894, 0.130894, 0.17524941, 0.14016236, -0.08593954, 0.0623012, -0.020411719, -0.031766243, -0.036939774, 0.08137602) * inp_6_2_1;
    result1 += M4(-0.08759783, 0.067338996, 0.064172134, -0.08956554, 0.03908009, -0.08249451, -0.024031114, 0.006511497, 0.013076348, -0.10823913, -0.15144663, -0.12734872, 0.090073295, 0.01762544, 0.09966374, 0.19341166) * inp_6_0_2;
    result1 += M4(-0.29657143, -0.045658246, 0.15557387, 0.109584175, -0.09728467, -0.06500571, 0.020645846, 0.1447013, -0.070764355, -0.20940232, 0.023149384, 0.00729236, 0.2900289, 0.10003316, -0.12518214, -0.14387597) * inp_6_1_2;
    result1 += M4(0.05882244, -0.031050518, 0.101200186, -0.23721348, -0.07513818, 0.16267575, 0.1825214, -0.0931608, 0.06832106, 0.10570181, -0.100216344, -0.1060184, 0.03822987, 0.025243565, -0.0992558, 0.09685126) * inp_6_2_2;
    result2 += M4(-0.10698125, 0.097363986, -0.01265533, -0.12619556, 0.068990245, -0.00072211324, -0.033299614, 0.028640382, -0.084043674, 0.04947359, 0.024753511, -0.036548927, -0.12742838, -0.1549207, 0.1070525, -0.16945131) * inp_6_0_0;
    result2 += M4(-0.06848849, 0.14529456, -0.06457037, -0.08550241, 0.05207313, -0.020290555, -0.25240162, -0.12594752, -0.047953382, 0.017648626, 0.04081248, 0.038582206, 0.043301594, 0.06877788, -0.01671566, -0.034105677) * inp_6_1_0;
    result2 += M4(0.041814953, 0.0070030275, -0.04423948, -0.14984606, 0.02110122, -0.05955555, -0.03975933, -0.1464694, -0.051536992, 0.007902254, 0.034123212, -0.018461695, -0.042469896, 0.010331104, 0.028954733, -0.044781435) * inp_6_2_0;
    result2 += M4(-0.045739457, 0.061217304, 0.08392919, -0.16035031, 0.23782988, -0.050427757, 0.05693238, -0.15016188, -0.21430095, -0.016896913, 0.12538025, 0.03905855, 0.007082239, -0.11423963, 0.031465627, -0.31035247) * inp_6_0_1;
    result2 += M4(-0.053179696, 0.27109787, -0.07200547, 0.057988074, 0.32313675, -0.19904849, 0.03256794, -0.19130757, -0.20934623, -0.20949377, 0.10293665, -0.11477504, -0.045292176, -0.24634135, 0.09587081, -0.39983156) * inp_6_1_1;
    result2 += M4(0.18770464, 0.14586905, 0.17174584, -0.42539385, 0.09207107, -0.018474692, 0.031075161, 0.018653063, -0.04442625, 0.022829928, -0.10009436, 0.047812317, 0.0056841313, -0.0030774076, 0.016106667, -0.0077994973) * inp_6_2_1;
    result2 += M4(-0.07319846, 0.07434623, -0.0693923, -0.09798424, 0.0883861, 0.07682528, -0.033767737, -0.058330286, 0.04798948, 0.054584887, 0.07694039, 0.10447283, -0.18404983, -0.03240765, -0.11619813, -0.11599491) * inp_6_0_2;
    result2 += M4(0.085296154, 0.07820159, -0.21665683, -0.14827845, 0.19526032, -0.0426152, -0.16406772, 0.119495966, -0.15560779, -0.023074662, -0.02865501, 0.12456221, -0.11786739, -0.0138244685, -0.0038003568, -0.27650848) * inp_6_1_2;
    result2 += M4(-0.30836174, 0.11960735, -0.183614, -0.15041386, 0.1324061, -0.010744548, -0.024846878, 0.0688468, -0.045627844, 0.09296088, -0.058935687, -0.016570423, 0.042303275, -0.0027982283, -0.021269575, -0.021863962) * inp_6_2_2;
    result3 += M4(0.25302246, -0.054751646, 0.014834545, 0.15796685, 0.15129708, -0.009868255, -0.025657125, 0.06274955, -0.08147568, 0.013499036, 0.005702571, -0.018632153, -0.007577916, -0.029519156, 0.22686592, -0.07515324) * inp_6_0_0;
    result3 += M4(0.4633201, -0.0896666, 0.012977237, -0.15458506, -0.19890517, -0.036445744, -0.13581818, -0.010917715, -0.029147511, -0.05999004, -0.027671812, -0.036875863, 0.06352943, -0.04536058, -0.03502863, -0.007167468) * inp_6_1_0;
    result3 += M4(0.48676738, -0.019477805, 0.36503735, -0.13001089, -0.2729817, 0.07814406, -0.11419341, -0.17827761, -0.11689191, 0.03689763, 0.01981027, -0.009592809, 0.013430917, 0.0039057592, -0.06703854, -0.06500749) * inp_6_2_0;
    result3 += M4(0.22488354, 0.01822792, 0.012126439, 0.022259966, 0.23749854, 0.038264174, 0.123576745, -0.049825642, -0.31297785, -0.06911969, -0.035435937, 0.25788325, 0.057216156, -0.08508501, -0.33725238, 0.15154785) * inp_6_0_1;
    result3 += M4(0.2096475, 0.18065058, -0.26235396, 0.24116892, -0.15351847, 0.0904921, -0.15952145, 0.42537776, -0.15247613, 0.034762017, 0.13139093, -0.11412691, 0.21176518, 0.14028029, -0.03311351, -0.047110897) * inp_6_1_1;
    result3 += M4(0.7682889, 0.109058335, -0.2964947, 0.31216007, 0.4434053, -0.07489344, -0.117909275, -0.28952384, -0.09545247, -0.056908723, 0.05527342, 0.04037257, 0.036935452, 0.035114694, 0.020761672, 0.0790602) * inp_6_2_1;
    result3 += M4(0.27767634, -0.07631825, 0.0020795865, 0.21119766, 0.0884978, -0.03646724, 0.029447306, -0.062357266, -0.17891642, 0.09402986, -0.2417734, 0.20579213, 0.101598375, -0.05778695, 0.25762305, -0.04708778) * inp_6_0_2;
    result3 += M4(0.46178383, -0.10110289, 0.19177832, 0.18094526, 0.041117962, -0.0033624268, 0.03333627, 0.05423002, 0.235581, -0.016594717, -0.07908424, 0.044352066, 0.01624695, 0.021215396, -0.12977941, 0.15363595) * inp_6_1_2;
    result3 += M4(0.39356017, 0.032767955, 0.0069126617, 0.29300633, 0.10701038, -0.042305924, 0.048049435, -0.034779787, -0.105009526, 0.024691992, -0.13108893, -0.09677029, -0.013901916, 0.003891417, -0.028966233, -0.0052953237) * inp_6_2_2;
    result4 += M4(-0.031021865, -0.1254216, -0.035095263, -0.036314324, -0.121248536, -0.06691109, 0.13539265, 0.10293671, 0.02926312, 0.037377976, 0.04407376, 0.042543303, -0.031514667, -0.083436094, -0.009064519, -0.10597672) * inp_6_0_0;
    result4 += M4(0.21358933, -0.23057556, -0.076409735, -0.20599338, 0.44811264, -0.15066157, -0.026402177, 0.09997647, -0.022112396, -0.02752328, 0.02387406, 0.027073136, 0.08424073, -0.08375956, 0.09530882, -0.13776252) * inp_6_1_0;
    result4 += M4(-0.15807204, -0.33981228, 0.075351395, -0.24548395, -0.012366701, -0.060939547, -0.087504596, -0.13664944, 0.0016176738, 0.02382357, 0.019225726, -0.043511245, -0.006020141, -0.02896326, 0.042765718, 0.001235955) * inp_6_2_0;
    result4 += M4(0.19097424, 0.13090529, 0.039143924, -0.09535315, 0.18949012, -0.030483272, 0.09759746, 0.06634751, -0.02558163, 0.21790892, 0.1865384, 0.103523135, 0.0629637, 0.225337, -0.22259314, -0.07347661) * inp_6_0_1;
    result4 += M4(0.055262037, 0.10779008, -0.060363, -0.2580668, -0.13011175, -0.04266338, 0.03384092, -0.20663257, -0.17654642, -0.22895268, -0.22453731, -0.17729166, -0.31730193, -0.18713464, -0.21319932, 0.13631113) * inp_6_1_1;
    result4 += M4(-0.076791406, 0.44399607, -0.06685413, -0.0011849483, -0.20507632, 0.09551535, -0.059949487, -0.009858683, 0.081085764, 0.09181724, -0.08001234, 0.0056760786, 0.021953527, 0.11264894, 0.02235743, -0.020969065) * inp_6_2_1;
    result4 += M4(-0.011122425, -0.1231766, -0.0852766, 0.006034165, 0.09636505, -0.11118168, -0.07811164, -0.05693014, 0.15839115, -0.06522252, 0.352321, 0.1184636, 0.007375221, 0.057602387, -0.033832993, 0.18988994) * inp_6_0_2;
    result4 += M4(0.033669986, -0.056378566, -0.05897347, 0.1118345, -0.14132863, 0.101599835, -0.11875675, 0.262222, -0.20439284, 0.09936588, -0.076417744, 0.009185294, 0.20060167, 0.20921552, 0.074415505, 0.1776991) * inp_6_1_2;
    result4 += M4(0.44258878, -0.18471193, -0.05047806, 0.06898421, 0.019103078, -0.04881953, 0.099405386, 0.0134570105, -0.037134204, -0.0571462, -0.096009836, 0.014042791, -0.06982047, -0.10799475, -0.0015116663, 0.00034633646) * inp_6_2_2;
    result5 += M4(0.021454915, 0.06013282, -0.019211916, 0.014409271, -0.008350362, 0.20456114, -0.024087977, -0.0788728, -0.078935444, -0.006298628, -0.089786015, -0.030976553, 0.12769796, 0.14777902, -0.071380585, 0.035573784) * inp_6_0_0;
    result5 += M4(-0.08444208, 0.18002443, 0.03255969, 0.18741032, -0.20096377, -0.11003544, -0.024379715, 0.034954023, 0.110752195, 0.03713649, -0.021407308, 0.06997796, 0.11025699, -0.031041428, 0.035449397, 0.18992774) * inp_6_1_0;
    result5 += M4(0.18508068, 0.34097505, 0.1364495, -0.036098767, 0.34972826, -0.2755209, 0.0896029, -0.040674593, -0.028038444, -0.054516084, -0.027956147, -0.07348674, 0.06994956, -0.018614141, 0.028764086, -0.0207729) * inp_6_2_0;
    result5 += M4(-0.06254976, 0.0072943307, 0.09082012, -0.0646733, 0.081762776, -0.007320562, -0.0133189885, -0.090598896, -0.13460045, 0.0655038, 0.029764304, 0.08424635, 0.051819354, 0.051241767, 0.087260276, -0.20680511) * inp_6_0_1;
    result5 += M4(-0.088715576, -0.10339383, 0.10809136, -0.31769162, -0.025010804, -0.22607528, -0.03087264, -0.11687762, 0.12062897, -0.38135707, 0.1413855, -0.111771315, -0.2137358, 0.3599756, -0.117952205, -0.30948263) * inp_6_1_1;
    result5 += M4(-0.109260574, 0.013039505, 0.1338449, -0.22122404, 0.14947519, 0.08396397, -0.030343022, -0.02669907, 0.006215909, 0.06564823, -0.13861209, 0.04351932, -0.10065164, 0.0031888469, 0.034575097, -0.024426071) * inp_6_2_1;
    result5 += M4(0.0526392, -0.044788826, -0.051363114, -0.024688657, 0.00299463, -0.048729293, -0.07106868, 0.055793703, 0.0560981, 0.31627864, -0.0012595681, -0.013631095, -0.11129304, -0.13224187, 0.025638366, 0.22797665) * inp_6_0_2;
    result5 += M4(0.1086407, -0.017452994, 0.0017530833, 0.08009784, 0.008506949, -0.024797406, 0.010030435, -0.13098544, -0.16046494, -0.12054389, 0.06253672, -0.005887242, 0.10856852, -0.24925128, 0.045637727, 0.09459141) * inp_6_1_2;
    result5 += M4(-0.11449636, 0.093918845, -0.09264264, 0.22673355, 0.079751275, 0.15123543, -0.03871002, 0.07078373, -0.012176157, -0.0033218567, -0.06308294, -0.02107942, 0.015265205, -0.022139, -0.03451133, -0.033276815) * inp_6_2_2;
    result6 += M4(-0.008245649, -0.048183188, 0.02944274, 0.12992084, 0.06838696, 0.006445793, 0.036113594, -0.09630437, -0.096542835, -0.0069383965, -0.037617914, 0.021151723, -0.04286256, -0.1397479, 0.053658884, 0.08161573) * inp_6_0_0;
    result6 += M4(0.06416531, 0.15113108, 0.11653285, 0.16805972, 0.056470707, -0.19324876, -0.027954753, 0.05083443, -0.0076525672, 0.07983633, 0.022190835, 0.104971394, -0.09467632, 0.10239415, 0.084429935, 0.0033971008) * inp_6_1_0;
    result6 += M4(-0.012643058, -0.038319748, 0.053345114, -0.019594051, -0.038276013, -0.052707348, 0.019172415, -0.1038929, 0.0082554035, -0.014604475, -0.0049857534, 0.028089093, -0.008687349, 0.065682106, 0.015168694, 0.004504091) * inp_6_2_0;
    result6 += M4(0.12338553, 0.059090294, 0.03646345, 0.23437129, 0.14495857, -0.0028161735, 0.013363247, 0.00772042, 0.0150899505, 0.28437757, 0.040162858, 0.0780752, 0.0049521034, 0.072424226, 0.057712663, 0.055397604) * inp_6_0_1;
    result6 += M4(0.21491297, -0.11758604, 0.14913878, 0.29707286, -0.23499136, -0.09300995, 0.08250014, -0.19135714, 0.25178966, -0.3193888, -0.10655275, 0.020853752, 0.1636201, -0.016656702, 0.18966077, 0.04476183) * inp_6_1_1;
    result6 += M4(-0.075440906, 0.027973456, 0.0020723885, 0.28203824, -0.103735566, -0.15367521, 0.07567064, -0.06823703, -0.079136446, -0.047162727, -0.041404177, 0.08203168, 0.021537067, -0.10822569, -0.03295889, 0.15999717) * inp_6_2_1;
    result6 += M4(0.012498839, -0.08270638, 0.041539382, 0.13636552, 0.11516697, 0.079689234, 0.050191965, -0.022286829, 0.13725747, 0.08658436, -0.1558415, 0.18720604, -0.09634208, -0.14748187, 0.043225814, 0.11774797) * inp_6_0_2;
    result6 += M4(-0.15206724, -0.054596383, 0.18052906, 0.054385666, 0.0057039415, -0.13098635, 0.0064813658, -0.16854739, -0.14941078, 0.17716959, -0.021718638, 0.07884621, -0.12960163, -0.20245823, 0.06639582, 0.021403626) * inp_6_1_2;
    result6 += M4(0.056859344, -0.037305627, -0.047402345, 0.30966932, 0.0911963, -0.06745449, 0.061815705, -0.11431254, -0.062382802, -0.025043493, -0.019313253, 0.06211359, -0.023673415, 0.0777699, -0.0026903409, -0.068495065) * inp_6_2_2;
    result7 += M4(0.105485626, -0.025859367, -0.03465063, -0.049131084, 0.056286205, 0.06629847, -0.11103727, -0.16318354, 0.027353836, -0.01561848, -0.07659974, -0.04716861, -0.04970574, 0.03770437, -0.002690252, -0.031567045) * inp_6_0_0;
    result7 += M4(-0.13975751, 0.05543771, 0.0990079, 0.018169653, -0.3090944, 0.013964234, -0.10256976, 0.26399827, 0.06890949, 0.009819375, 0.08298038, -0.0725947, -0.19666159, 0.032539465, 0.14778568, -0.09891346) * inp_6_1_0;
    result7 += M4(0.010287352, 0.045230333, -0.056488372, -0.18588115, 0.008610966, 0.11758991, -0.07925892, 0.14412148, 0.052172568, -0.0025876209, -0.019415554, -0.056523476, -0.05477891, 0.02019765, 0.011517594, -0.051456634) * inp_6_2_0;
    result7 += M4(0.015424201, -0.013855244, -0.008661949, 0.014357678, 0.06348731, -0.040177926, -0.07447504, 0.058070768, 0.22232033, 0.046839356, 0.057316933, -0.059794534, 0.20333855, -0.011283405, 0.20073918, -0.10398705) * inp_6_0_1;
    result7 += M4(0.20473307, 0.05371803, -0.09050381, 0.31875986, -0.26621935, 0.080395974, -0.48806638, 0.048230197, 0.20325722, 0.17379421, -0.3726188, -0.004821129, 0.08203385, 0.25997767, 0.29116657, 0.08082398) * inp_6_1_1;
    result7 += M4(-0.024022896, 0.009024036, 0.14781222, -0.117251344, 0.25150734, -0.07818896, -0.15374465, 0.13165906, -0.007870023, -0.058881942, 0.024622874, -0.07636141, 0.021704093, -0.0008467885, -0.0137808025, 0.03386251) * inp_6_2_1;
    result7 += M4(0.07962056, -0.013769294, -0.043252546, -0.046553183, -0.074989185, 0.019916417, -0.012225686, 0.0050329785, 0.3913717, -0.072376385, -0.10664802, -0.08822083, -0.29145178, 0.0059104753, 0.063813254, 0.08644493) * inp_6_0_2;
    result7 += M4(0.10533946, 0.0974363, 0.06066871, -0.2635577, 0.019977348, -0.017383037, -0.024083443, 0.07584392, -0.08816903, -0.055130467, -0.12471642, -0.06406041, 0.34162137, 0.073291615, 0.04595499, -0.06312937) * inp_6_1_2;
    result7 += M4(-0.10988775, -0.038332168, -0.040202204, 0.35451844, -0.23554428, -0.001560702, 0.10206596, 0.073377974, 0.1804329, -0.0069849296, -0.036636684, 0.09951519, 0.063653514, 0.015024471, 0.072389215, 0.00012222516) * inp_6_2_2;
    const V4 inp_7_0_0 = inp[7][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_7_1_0 = inp[7][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_7_2_0 = inp[7][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_7_0_1 = inp[7][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_7_1_1 = inp[7][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_7_2_1 = inp[7][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_7_0_2 = inp[7][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_7_1_2 = inp[7][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_7_2_2 = inp[7][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.14035666, 0.026212718, 0.09191014, 0.023129148, 0.05519419, 0.036357325, -0.17181754, -0.08859095, -0.023175808, 0.03696766, -0.13718474, -0.11831526, -0.015461064, 0.021942379, -0.016240798, -0.032551866) * inp_7_0_0;
    result0 += M4(0.11383999, 0.13373005, -0.07095509, -0.06514729, -0.14817905, -0.28805274, 0.121208034, 0.020026814, -0.028344207, -0.20914987, 0.12794386, 0.0349014, 0.07405036, 0.03590241, 0.18459131, 0.006805226) * inp_7_1_0;
    result0 += M4(-0.00057231763, 0.08110842, -0.08373391, 0.016294306, -0.02762188, 0.09072393, 0.11788116, -0.08368773, 0.025586462, 0.10544737, 0.01884721, -0.12083788, -0.008642203, -0.034986723, -0.03254971, -0.07050638) * inp_7_2_0;
    result0 += M4(0.078820184, -0.018296018, -0.07208294, 0.008866698, 0.065369524, 0.09892088, -0.15636176, -0.07120111, -0.08143803, -0.008925251, 0.27097547, -0.09927845, -0.079026476, -0.014243509, 0.24370039, -0.1531349) * inp_7_0_1;
    result0 += M4(-0.19278881, -0.12335426, 0.08663447, -0.080273785, -0.103036135, -0.05834247, 0.24787284, -0.06660496, 0.01807001, 0.11086902, -0.01293912, -0.1000216, 0.29125348, -0.20363098, -0.05427454, -0.009685624) * inp_7_1_1;
    result0 += M4(-0.0461766, -0.07733745, 0.13581152, 0.11611437, 0.043516334, -0.11995534, 0.005560871, -0.0011772584, 0.03056431, -0.039222114, -0.08758696, -0.06513427, -0.13397144, 0.009918238, -0.08711231, -0.06657255) * inp_7_2_1;
    result0 += M4(0.06610669, -0.0020710756, -0.03829409, 0.008033264, 0.03293198, -0.023976775, -0.1300249, -0.048385657, 0.043931328, -0.0005718692, 0.06973696, -0.04660289, -0.21267848, -0.2248955, 0.37774292, -0.033456616) * inp_7_0_2;
    result0 += M4(0.0810417, -0.023163918, -0.013459732, -0.018084396, 0.092156544, -0.22696565, 0.0691879, -0.009749501, 0.121157385, -0.11075538, 0.021976788, -0.010549273, 0.2674366, -0.038446788, 0.11537422, -0.24277948) * inp_7_1_2;
    result0 += M4(0.1646012, -0.082464956, 0.35236767, -0.05748259, -0.13080467, 0.10173554, -0.036733266, 0.0729165, -0.12122239, -0.01084034, 0.04189376, 0.0007903984, -0.02593264, -0.031835493, -0.08096311, -0.008451804) * inp_7_2_2;
    result1 += M4(-0.12791732, -0.08744802, 0.047097154, -0.11422871, -0.22552775, 0.0007547501, -0.18620417, 0.008449165, -0.23478487, 0.0082326755, -0.0611052, -0.050853685, -0.023731401, -0.13520129, -0.07089394, 0.021922639) * inp_7_0_0;
    result1 += M4(-0.031360425, -0.030654658, 0.026571156, -0.14053036, 0.023199914, 0.15431583, 0.13839056, 0.111622825, 0.07812258, 0.013628836, 0.0634665, 0.16185446, 0.08020485, 0.09449431, 0.052982595, 0.06292378) * inp_7_1_0;
    result1 += M4(-0.042454883, 0.083625555, -0.39087877, 0.17799127, 0.08517464, -0.05832067, 0.01980049, -0.049863238, 0.09477092, -0.14420158, -0.009411929, -0.12714039, 0.01901907, -0.076128356, -0.024207458, -0.062232066) * inp_7_2_0;
    result1 += M4(0.066530235, 0.06648962, -0.012472126, 0.13617338, -0.016321551, -0.009641107, -0.1194339, -0.12139216, -0.04191864, -0.037896525, -0.097905666, -0.02733235, -0.27580822, 0.14599855, -0.03595013, -0.110294506) * inp_7_0_1;
    result1 += M4(-0.108774275, -0.06255252, 0.31319648, 0.26002905, -0.11317954, 0.071141765, 0.016047612, -0.14251931, -0.04277266, -0.035331037, 0.21611597, 0.016464695, 0.087261386, 0.10387466, -0.20252433, -0.018015867) * inp_7_1_1;
    result1 += M4(0.52540886, 0.063744046, -0.27648482, -0.039635632, 0.07503288, 0.0049415417, -0.012373131, 0.064522296, -0.039199557, 0.042279605, -0.038414095, -0.06023218, -0.030282957, -0.19866961, -0.014337861, 0.08260442) * inp_7_2_1;
    result1 += M4(0.04714958, 0.019858105, -0.08392409, -0.056461036, -0.15654355, 0.0063124523, 0.08340242, 0.040965267, -0.08377446, 0.08950175, 0.013691652, -0.0071722777, 0.16900143, 0.05893726, 0.064100325, 0.10045612) * inp_7_0_2;
    result1 += M4(0.13407123, 0.06720747, -0.3069114, -0.1509796, 0.0887509, 0.10255851, -0.09719642, -0.23851335, -0.015740288, -0.09041531, -0.0004379691, -0.07316673, -0.2212364, -0.1083793, 0.13703369, -0.00446792) * inp_7_1_2;
    result1 += M4(0.33707908, -0.0034454775, -0.21934968, -0.21498488, 0.043104276, 0.004273958, 0.005119596, 0.012304173, 0.046766326, -0.079776436, 0.02823146, -0.0358567, -0.15289143, 0.07754418, -0.010308068, -0.15868904) * inp_7_2_2;
    result2 += M4(0.030258685, -0.0022008473, 0.00790828, -0.047843415, 0.016466936, 0.015801316, -0.15073259, 0.015192704, 0.08951226, -0.027693592, -0.09248456, 0.024282575, 0.11663834, -0.028332068, 0.041025195, 0.05698444) * inp_7_0_0;
    result2 += M4(0.055634793, -0.12700646, 0.027599221, 0.10128832, 0.031560104, -0.039944503, 0.09877907, -0.045331635, 0.12513772, -0.048778247, -0.03353606, 0.0416809, 0.08965314, 0.02348112, -0.12787122, -0.014134714) * inp_7_1_0;
    result2 += M4(-0.14336738, -0.06729082, 0.21335208, -0.032958034, 0.02402265, -0.08337007, 0.04653692, -0.09962367, 0.06602025, -0.12952346, 0.18016283, -0.11506193, 0.10078883, -0.057122353, 0.095222436, -0.04740477) * inp_7_2_0;
    result2 += M4(-0.012068043, -0.08945056, 0.0105850035, -0.07690162, 0.25275767, 0.047817226, -0.059505247, 0.021922044, 0.21531436, -0.042876367, 0.054380704, -0.1252299, 0.118750304, 0.03689545, 0.1333588, -0.14416984) * inp_7_0_1;
    result2 += M4(0.030982573, 0.033109806, -0.08213645, 0.18685038, 0.01124282, 0.087647974, 0.07027759, -0.24765895, 0.06591934, -0.03136437, 0.055910166, -0.26046056, 0.15287949, 0.11416554, -0.31173468, -0.068080604) * inp_7_1_1;
    result2 += M4(0.030308692, 0.0675454, 0.26779044, 0.1987772, 0.16349019, -0.063977726, 0.049709357, 0.12977055, 0.08657017, -0.14681831, -0.013405258, -0.02156519, -0.01764518, -0.10507199, 0.23161165, 0.0606035) * inp_7_2_1;
    result2 += M4(0.068523034, 0.02974, -0.019514201, -0.03135649, 0.085402474, -0.052738324, -0.0761231, -0.08540571, 0.13471998, -0.03363448, -0.018727727, -0.13434277, 0.019501613, 0.116065666, 0.053729873, 0.013001785) * inp_7_0_2;
    result2 += M4(0.026690962, -0.050411534, 0.095248304, 0.0729887, 0.10372894, -0.03858884, -0.07421566, -0.02109136, 0.25530288, -0.018529199, -0.058758106, -0.12219187, 0.10770971, -0.20107691, 0.096852355, -0.12343767) * inp_7_1_2;
    result2 += M4(0.122573994, 0.32669917, 0.20749046, 0.05734704, 0.06541587, 0.016707322, 0.19443323, 0.009817646, 0.104483984, -0.053529847, 0.11564548, -0.016952245, 0.15221582, -0.040294833, 0.096602276, -0.014385921) * inp_7_2_2;
    result3 += M4(-0.05996765, 0.014994332, -0.0015384173, 4.760506e-05, 0.0345263, 0.047622193, -0.12153507, -0.071122825, 0.11616992, 0.043869283, -0.04028831, -0.04086935, 0.11353888, 0.015150829, -0.014524467, -0.12295988) * inp_7_0_0;
    result3 += M4(0.0019569271, 0.08720722, -0.10675042, 0.23170757, 0.24832444, 0.056585994, 0.009760046, 0.17739055, 8.03708e-05, 0.07364775, -0.06091116, 0.059135497, 0.027355075, 0.117596224, 0.07214412, 0.15528195) * inp_7_1_0;
    result3 += M4(0.15558214, -0.078197286, -0.013402984, -0.16149148, 0.06559618, 0.013834515, 0.12121181, -0.021001842, 0.23201793, 0.049172234, 0.02570916, 0.08342496, 0.23962831, -0.011712839, 0.003962354, 0.059936337) * inp_7_2_0;
    result3 += M4(-0.05066594, -0.021283558, 0.14794259, -0.009511928, 0.122917645, -0.015375272, -0.1996329, 0.06665671, 0.2368931, -0.05523352, 0.06778116, -0.18691435, 0.16070881, -0.02906163, 0.023664014, -0.2318561) * inp_7_0_1;
    result3 += M4(-0.012940162, 0.1873856, -0.0011057489, -0.019113129, 0.19282892, -0.11994087, 0.24982049, 0.028557371, -0.12942183, -0.10326189, 0.016140625, 0.21794036, -0.024137216, 0.16451305, 0.25220245, 0.19923344) * inp_7_1_1;
    result3 += M4(0.45983177, -0.31391537, -0.016637301, -0.17955573, 0.08554245, 0.0221108, -0.008028747, -0.03548053, 0.25862256, -0.0057837055, -0.09561716, -0.0245499, 0.51341033, -0.03997168, -0.18288556, -0.057494767) * inp_7_2_1;
    result3 += M4(0.042567894, 0.023269124, -0.13832375, -0.0086152805, 0.0835464, -0.018319884, -0.02840446, 0.022287572, 0.14199273, -0.023796631, -0.056025594, 0.012634855, 0.19367653, -0.079330735, 0.21811728, -0.06833522) * inp_7_0_2;
    result3 += M4(-4.2598684e-05, 0.19847046, 0.1361947, -0.12235735, -0.04468562, 0.006398846, -0.040396225, -0.028271178, 0.108425006, -0.016232789, -0.18475384, -0.017254582, 0.19055443, -0.14113998, 0.015133829, 0.07887932) * inp_7_1_2;
    result3 += M4(0.3073101, -0.121064775, -0.051044174, -0.13344613, 0.07542749, 0.018445844, 0.12930155, 0.10700314, 0.071895055, 0.049424388, 0.018038098, 0.029169947, 0.040075872, 0.006091056, -0.07656265, 0.2698687) * inp_7_2_2;
    result4 += M4(0.11632603, 0.057896215, 0.0052918247, -0.045258302, 0.30952954, -0.01434661, -0.08242911, -0.026623454, 0.12293237, 0.10267697, -0.041961934, 0.043024004, 0.054354154, 0.16965063, 0.04531724, 0.011408402) * inp_7_0_0;
    result4 += M4(-0.117819786, -0.09417858, -0.14738417, 0.09179696, -0.023918387, 0.029122833, 0.15249826, 0.2960671, -0.13013856, 0.016630376, 0.18149827, 0.19404446, -0.104449846, -0.056444682, 0.041619036, 0.12653217) * inp_7_1_0;
    result4 += M4(0.19311067, -0.15105431, -0.15432633, -0.024805358, -0.13496178, -0.08445245, -0.03737432, -0.24612091, -0.06858696, -0.0665952, -0.15640984, -0.10512597, -0.098776765, -0.08190762, -0.055920586, -0.14482471) * inp_7_2_0;
    result4 += M4(-0.101278916, -0.041636206, 0.095103286, 0.087097734, -0.016578222, 0.06370841, -0.08547129, 0.022557456, -0.20517054, 0.062371887, -0.081136495, 0.020603001, -0.19078448, 0.108456515, 0.08658724, -0.059892125) * inp_7_0_1;
    result4 += M4(0.4590071, -0.054340247, 0.14955112, -0.12085707, -0.043076914, -0.23419134, 0.051003143, -0.004884765, 0.046780497, -0.02098295, 0.058827654, -0.079591125, -0.21461512, 0.021669623, -0.10477614, 0.09911001) * inp_7_1_1;
    result4 += M4(-0.044197217, 0.33862385, 0.044597346, -0.09548435, -0.046556044, 0.009382223, 0.021680076, -0.1140434, 0.08845368, -0.010530808, -0.14699864, 0.08937098, 0.11480829, 0.06514435, 0.031025179, 0.08884954) * inp_7_2_1;
    result4 += M4(-0.010848656, -0.006872391, -0.09151898, -0.023041824, 0.019639373, -0.07727297, 0.0080066025, 0.010130856, 0.011155599, 0.11247294, 0.0370598, 0.0627102, -0.18626045, 0.13153775, -0.117006406, 0.05910218) * inp_7_0_2;
    result4 += M4(-0.34537175, 0.1163598, 0.04919288, 0.04634622, 0.05713849, 0.1532584, 0.092097096, 0.108763054, -0.10702264, -0.03207313, 0.14868917, 0.031993706, -0.0552348, -0.16912085, -0.041152533, 0.037286114) * inp_7_1_2;
    result4 += M4(-0.05251481, -0.15066405, -0.014063789, 0.08733574, -0.064545006, -0.11338402, -0.1459582, -0.1449641, 0.053128492, 0.0014864682, -0.1010149, 0.020502003, 0.105597146, 0.15490949, 0.16679727, 0.086660504) * inp_7_2_2;
    result5 += M4(0.0410587, -0.04510027, 0.026168268, -0.01030513, -0.10623416, -0.13090275, -0.019207802, -0.109312326, 0.0590774, -0.048798673, 0.01742273, -0.10404523, 0.0076579573, -0.037557304, -0.061683044, -0.07202548) * inp_7_0_0;
    result5 += M4(0.043872166, -0.16678558, 0.17488496, -0.06593686, -0.075957865, 0.10037119, 0.04859834, -0.036637932, 0.05925795, 0.16750617, 0.03524896, -0.1255465, 0.025613606, -0.10607617, 0.054719895, -0.08444545) * inp_7_1_0;
    result5 += M4(-0.03702239, -0.21714668, 0.13047083, -0.05272383, -0.027261147, -0.12406296, 0.010970764, 0.12152492, 0.0093055405, -0.14402652, 0.028293472, 0.0876553, 0.026557423, 0.02364204, 0.068518385, 0.27820787) * inp_7_2_0;
    result5 += M4(-0.02335594, 0.0788689, 0.011236206, -0.041814096, 0.08408095, 0.09633769, 0.0072841295, 0.034595057, 0.038927455, -0.20881797, 0.16022678, -0.07483383, -0.14650415, -0.14772321, 0.07255174, 0.038419534) * inp_7_0_1;
    result5 += M4(0.0063949525, 0.14746846, -0.18333943, -0.0012230226, 0.05452702, 0.26822287, 0.114857815, 0.046010766, 0.09804125, 0.13542254, 0.044458363, -0.115871586, 0.25524268, 0.3185605, -0.04463629, -0.1601405) * inp_7_1_1;
    result5 += M4(0.19450516, 0.1350907, 0.32338887, 0.056521744, -0.050647292, -0.050754745, 0.032830425, 0.0911142, -0.0003028716, 0.021169325, 0.009997823, 0.0011083112, -0.16468105, -0.040594433, -0.045538045, 0.28174013) * inp_7_2_1;
    result5 += M4(-0.037622873, -0.047300156, 0.008017579, 0.062194135, -0.016804855, -0.006373795, -0.07422574, 0.06427026, 0.047670536, -0.080935046, -0.013553989, 0.004167727, 0.00023976518, -0.14386673, 0.07565915, 0.1826904) * inp_7_0_2;
    result5 += M4(0.029816039, 0.0029151407, -0.010435049, 0.09780553, -0.016035544, -0.060186945, 0.0035076737, 0.097718604, -0.015552177, 0.13878761, 0.04597583, -0.05412921, 0.14340626, 0.21842247, -0.11455229, -0.109752804) * inp_7_1_2;
    result5 += M4(-0.030985212, 0.27965266, -0.18898529, 0.01454216, -0.079570934, -0.017898507, 0.10025316, -0.05824175, 0.0011709456, -0.10666405, 0.0023818994, -0.021875788, -0.12307191, -0.044061266, 0.24472113, 0.018727377) * inp_7_2_2;
    result6 += M4(-0.010556821, 0.09975421, 0.0076881098, -0.00072512537, -0.08316035, 0.16734281, 0.0064350534, -0.14820512, -0.02993291, 0.18721192, 0.02204644, -0.18449302, -0.05081967, -0.097010165, 0.008450879, -0.08477673) * inp_7_0_0;
    result6 += M4(-0.08897169, -0.07602663, -0.13017388, 0.05027871, 0.045437578, 0.13072057, -0.05344233, -0.11775331, -0.009188872, -0.11127751, -0.014087078, -0.15856707, 0.009793904, 0.09589192, -0.0051718117, -0.13552295) * inp_7_1_0;
    result6 += M4(-0.10986595, -0.15962656, -0.051479336, -0.018221779, -0.015636764, 0.005517981, 0.07547564, -0.1339619, 0.10360485, -0.020415023, 0.051420357, -0.2456463, 0.031986035, -0.06191942, 0.031425714, -0.13556838) * inp_7_2_0;
    result6 += M4(-0.06835802, -0.0847027, -0.050748765, 0.06672316, 0.076418385, -0.08840635, 0.032112077, -0.24365848, -0.05533544, 0.095966555, 0.13916686, -0.3334124, -0.10648495, 0.11792798, 0.12601271, -0.22947478) * inp_7_0_1;
    result6 += M4(0.112637885, 0.17132367, -0.12437386, -0.10952937, -0.031158354, -0.1384591, 0.30871636, -0.29832786, 0.15902196, -0.13515201, 0.30355498, -0.2929977, -0.043841362, -0.22785698, 0.2442527, -0.43907377) * inp_7_1_1;
    result6 += M4(-0.3186491, 0.03919349, -0.07602581, 0.028976044, -0.1326463, 0.13674887, 0.009365226, -0.16505334, -0.03943322, 0.09674497, 0.13320577, -0.38501295, -0.118028335, 0.268203, 0.1446116, -0.20730609) * inp_7_2_1;
    result6 += M4(0.078890204, -0.00998617, 0.027467221, -0.043493874, 0.12377559, -0.031856038, 0.07574942, -0.06013667, 0.06474636, -0.08118194, 0.07613642, -0.060737595, -0.01072399, -0.041445643, 0.0432528, 0.044246994) * inp_7_0_2;
    result6 += M4(0.017147481, -0.09557517, -0.15437962, -0.07081251, 0.009510722, -0.037523415, 0.027026622, 0.10867839, 0.0585247, 0.071931, 0.048089597, 0.0014578415, -0.05432743, 0.08225931, 0.19983828, -0.02885845) * inp_7_1_2;
    result6 += M4(-0.09077407, -0.19993581, -0.12738867, -0.07406553, -0.12392993, 0.027872689, -0.020976761, -0.014237772, -0.030498156, 0.03077637, 0.014496107, -0.10823867, 0.18665576, -0.051311996, 0.06787884, 0.005319032) * inp_7_2_2;
    result7 += M4(0.01738567, -0.0124841845, 0.018266419, -0.10827431, -0.07934837, 0.050183147, -0.28362712, -0.061416183, 0.010008455, 0.037268717, -0.2699135, -0.15740047, -0.17147881, 0.007845664, -0.06506554, 0.012450599) * inp_7_0_0;
    result7 += M4(0.13672915, -0.07937093, 0.089893445, 0.029869758, 0.12912805, -0.027708286, -0.020087868, 0.055670895, 0.09781966, -0.0072113243, 0.004010393, 0.019604757, 0.13248383, -0.0951612, -0.033864465, 0.031326827) * inp_7_1_0;
    result7 += M4(0.4755403, 0.018374758, -0.22089545, -0.28499392, 0.014834464, 0.09146308, -0.105404355, -0.0299245, 0.085874684, 0.11951815, -0.21102163, 0.003799048, -0.011629549, 0.04751071, -0.044128142, 0.0918865) * inp_7_2_0;
    result7 += M4(-0.012505594, -0.025840016, 0.04369061, 0.03717479, 0.07240399, -0.031270225, -0.0857797, -0.17208333, -0.0019207475, 0.07297594, -0.2177171, -0.13301422, 0.059328485, -0.044778354, -0.06970693, 0.09058855) * inp_7_0_1;
    result7 += M4(-0.048072148, 0.09867651, -0.16009967, -0.048407797, -0.05098424, 0.11453101, -0.95359236, 0.104627065, 0.084105365, 0.22121625, -0.56511587, 0.08484243, -0.10582638, -0.04705832, -0.27494258, 0.18465999) * inp_7_1_1;
    result7 += M4(-0.10090088, -0.19530399, -0.49274582, 0.06144859, -0.105442844, 0.017185297, 0.05853463, -0.08368335, 0.021879002, 0.11523853, -0.23345207, -0.08374545, -0.095558204, 0.03075394, -0.21775135, -0.03776848) * inp_7_2_1;
    result7 += M4(-0.0122877555, -0.0067679356, -0.03284397, 0.003917509, -0.034854297, 0.021527398, -0.0024382349, 0.019985413, 0.035968773, 0.01040982, -0.0872662, -0.06776038, -0.15868089, -0.04086517, 0.08134113, 0.0013262286) * inp_7_0_2;
    result7 += M4(-0.063414656, -0.07362997, -0.15963201, 0.048891522, 0.005997416, 0.05190951, 0.055745497, 0.028873738, 0.0015090007, 0.052615993, -0.06891147, 0.084943175, -0.21016996, 0.1034267, -0.2577396, 0.121008575) * inp_7_1_2;
    result7 += M4(0.08461429, -0.1234623, -0.12555185, 0.10766685, -0.061525244, -0.016494969, -0.090014055, -0.028357085, -0.019038517, 0.011579131, -0.03595191, -0.079823375, -0.16517617, -0.0033313243, 0.04074734, 0.101529546) * inp_7_2_2;
    const ivec2 output_base = ivec2(gl_GlobalInvocationID) * ivec2(4, 2);
    imageStore(out_image, output_base + ivec2(0, 0), result0);
    imageStore(out_image, output_base + ivec2(1, 0), result1);
    imageStore(out_image, output_base + ivec2(2, 0), result2);
    imageStore(out_image, output_base + ivec2(3, 0), result3);
    imageStore(out_image, output_base + ivec2(0, 1), result4);
    imageStore(out_image, output_base + ivec2(1, 1), result5);
    imageStore(out_image, output_base + ivec2(2, 1), result6);
    imageStore(out_image, output_base + ivec2(3, 1), result7);
}

//!DESC ArtCNN C4F32 DS (Conv2D-6)
//!COMPUTE 12 16 12 16
//!HOOK LUMA
//!BIND conv2d
//!BIND conv2d_5
//!SAVE conv2d_6
//!WIDTH LUMA.w 1.0 *
//!HEIGHT LUMA.h 1.0 *
//!COMPONENTS 4
//!WHEN OUTPUT.w LUMA.w / 1.3 > OUTPUT.h LUMA.h / 1.3 > *
#extension GL_EXT_shader_explicit_arithmetic_types_float16 : enable
#ifdef GL_EXT_shader_explicit_arithmetic_types_float16
#	define V4 f16vec4
#	define M4 f16mat4
#	define F float16_t
#else
#	define V4 vec4
#	define M4 mat4
#	define F float
#endif

const ivec2 ksize = ivec2(3, 3);
const ivec2 offset = ksize / 2;
const ivec2 wg_size = ivec2(gl_WorkGroupSize);
const ivec2 isize = wg_size + ksize - 1;
shared V4 inp[8][isize.y][isize.x];
void hook() {
    const uvec2 local_xy = gl_LocalInvocationID.xy;
    ivec2 base = ivec2(gl_WorkGroupID) * wg_size;
    for (uint y = local_xy.y; y < isize.y; y += wg_size.y) {
        for (uint x = local_xy.x; x < isize.x; x += wg_size.x) {
            const ivec2 input_base = (base + ivec2(x,y) - offset) * ivec2(4, 2);
            inp[0][y][x] = V4(conv2d_5_mul * texelFetch(conv2d_5_raw, input_base + ivec2(0, 0), 0) + conv2d_mul * texelFetch(conv2d_raw, input_base + ivec2(0, 0), 0));
            inp[1][y][x] = V4(conv2d_5_mul * texelFetch(conv2d_5_raw, input_base + ivec2(1, 0), 0) + conv2d_mul * texelFetch(conv2d_raw, input_base + ivec2(1, 0), 0));
            inp[2][y][x] = V4(conv2d_5_mul * texelFetch(conv2d_5_raw, input_base + ivec2(2, 0), 0) + conv2d_mul * texelFetch(conv2d_raw, input_base + ivec2(2, 0), 0));
            inp[3][y][x] = V4(conv2d_5_mul * texelFetch(conv2d_5_raw, input_base + ivec2(3, 0), 0) + conv2d_mul * texelFetch(conv2d_raw, input_base + ivec2(3, 0), 0));
            inp[4][y][x] = V4(conv2d_5_mul * texelFetch(conv2d_5_raw, input_base + ivec2(0, 1), 0) + conv2d_mul * texelFetch(conv2d_raw, input_base + ivec2(0, 1), 0));
            inp[5][y][x] = V4(conv2d_5_mul * texelFetch(conv2d_5_raw, input_base + ivec2(1, 1), 0) + conv2d_mul * texelFetch(conv2d_raw, input_base + ivec2(1, 1), 0));
            inp[6][y][x] = V4(conv2d_5_mul * texelFetch(conv2d_5_raw, input_base + ivec2(2, 1), 0) + conv2d_mul * texelFetch(conv2d_raw, input_base + ivec2(2, 1), 0));
            inp[7][y][x] = V4(conv2d_5_mul * texelFetch(conv2d_5_raw, input_base + ivec2(3, 1), 0) + conv2d_mul * texelFetch(conv2d_raw, input_base + ivec2(3, 1), 0));
        }
    }

    barrier();
    V4 result0 = V4(0.11820534, 0.1142933, 0.11977196, 0.11703383);
    const V4 inp_0_0_0 = inp[0][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_0_1_0 = inp[0][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_0_2_0 = inp[0][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_0_0_1 = inp[0][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_0_1_1 = inp[0][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_0_2_1 = inp[0][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_0_0_2 = inp[0][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_0_1_2 = inp[0][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_0_2_2 = inp[0][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.011639688, 0.02048606, -0.016872741, -0.000332063, -0.02478374, -0.004558933, -0.011892751, 0.014026109, 0.035917312, -0.01065963, 0.0038752712, -0.018530909, -0.154547, -0.0105257835, -0.04062278, 0.008503111) * inp_0_0_0;
    result0 += M4(-0.026186898, -0.006249482, 0.009988812, 0.011725102, 0.00729088, -0.013361922, -0.034376252, -0.03987459, 0.0017388548, -0.00899437, -0.00021294334, 0.0157943, -0.058437023, -0.13277853, -0.025455806, -0.04654327) * inp_0_1_0;
    result0 += M4(0.009928752, -0.007324953, 0.014090458, 0.016872097, 0.004172834, 0.008794596, 0.014326963, 0.025480667, -0.019355264, 0.0034053575, 0.014526935, 0.009168287, -0.07187553, -0.14940883, 0.0052900095, -0.0032713867) * inp_0_2_0;
    result0 += M4(0.014688229, -0.010371164, 0.013028213, -0.011423413, -0.007853697, 0.020590456, -0.036850203, -0.009890834, 0.009544625, 0.011222157, -0.01509601, -0.008552502, -0.015421458, 0.049719747, -0.01616633, 0.026926663) * inp_0_0_1;
    result0 += M4(0.0061672432, -0.0071810735, 0.005144415, 0.00029252813, -0.05575906, 0.0022873404, 0.04526727, 0.03064057, -0.010790469, 0.035218228, 0.018768769, -0.053857297, 0.24420372, 0.2293986, 0.27929193, 0.2439709) * inp_0_1_1;
    result0 += M4(0.007939174, -0.008848642, -0.027132234, 0.015075844, 0.03115365, 0.01426379, -0.027391797, -0.012864509, 0.0071326853, -0.036053754, 0.007833054, 0.03151171, -0.010340638, -0.06035741, -0.03807813, -0.051524) * inp_0_2_1;
    result0 += M4(-0.0039380696, -0.0020302122, 0.007976842, 0.0061210827, 0.022329805, -0.017052539, 0.059439465, 0.012659825, -0.017656213, -0.011120664, -0.0036218981, 0.00963842, 0.007633209, 0.02647312, -0.11630054, 0.009429812) * inp_0_0_2;
    result0 += M4(-0.0052635237, 0.0047849515, -0.03255619, -0.025831603, 0.030008238, 0.008054239, -0.00878867, 0.014806011, 0.011210198, 0.0046204366, -0.010649766, 0.010663962, 0.03175669, 0.046167087, 0.0050017424, 0.006868128) * inp_0_1_2;
    result0 += M4(-0.013758374, 0.01919477, 0.012441475, -0.011113727, -0.004478684, -0.016755948, -0.0047034, -0.032984003, -0.0069269566, 0.014337039, -0.018556425, -0.024298329, 0.017664498, -0.01727705, -0.043154396, -0.1816669) * inp_0_2_2;
    const V4 inp_1_0_0 = inp[1][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_1_1_0 = inp[1][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_1_2_0 = inp[1][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_1_0_1 = inp[1][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_1_1_1 = inp[1][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_1_2_1 = inp[1][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_1_0_2 = inp[1][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_1_1_2 = inp[1][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_1_2_2 = inp[1][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.02252219, -0.0047125374, -0.006416192, -0.0038287635, -0.032580502, -0.025233611, -0.041233506, 0.0056920634, -0.054528248, 0.010281978, -0.02187116, 0.010545709, -0.0064479485, -0.020831725, 0.007682775, -0.0076845437) * inp_1_0_0;
    result0 += M4(0.007053002, -0.027469192, 0.0074195154, 0.0072249565, 0.015592533, -0.0074797035, 0.008220084, -0.0040634517, 0.035789624, 0.04499459, -0.012555129, 0.006032427, 0.021121265, 0.008532868, -0.004505405, -0.015027318) * inp_1_1_0;
    result0 += M4(0.0054109665, 0.010416023, -0.009464313, 0.012453712, -0.010694938, 0.024236644, 0.010212968, 0.004394467, 0.006875486, -0.0390353, -0.0026681204, -0.0029025825, 0.004755599, -0.02362536, 0.015884727, -0.018651433) * inp_1_2_0;
    result0 += M4(0.008524365, -0.0017062034, 0.02497875, 0.010814682, 0.029531382, 0.008169821, 0.031706672, 0.011933914, 0.024547044, 0.0014205717, -0.021168815, 0.0048486767, -0.0033175324, 0.024830105, -0.013203935, -0.0021330274) * inp_1_0_1;
    result0 += M4(0.009752152, -0.0066311676, -0.049736034, -0.024766266, -0.02204972, 0.016523723, -0.020377267, -0.030172471, -0.010865282, -0.039489277, 0.048779987, -0.02763279, 0.006095725, 0.014610573, -0.006970743, 0.014841053) * inp_1_1_1;
    result0 += M4(-0.021976292, -0.0012045106, 0.015767213, 0.024034027, -0.008743295, -0.01651274, -0.0020104577, 0.01387686, -0.016552003, 0.008287045, 0.008533337, 0.02113638, -0.036225308, 0.00095826265, -0.0005093255, 0.011574069) * inp_1_2_1;
    result0 += M4(-0.0056155017, -0.008430463, 0.03261786, 0.008024148, -0.0009815505, 0.012291125, -0.008050351, -0.015989376, 0.0008895027, 0.002829714, 0.06127, 0.0021904001, 0.00778281, 0.014795677, 0.00033537394, -0.009965638) * inp_1_0_2;
    result0 += M4(-0.0067922417, 0.027046926, -0.014311027, 0.011765863, 0.00570809, -0.005573521, 0.012770357, 0.02597375, -0.0027202936, -0.020361936, -0.031236501, 0.016840827, -0.025053216, -0.0048430483, -0.019158386, 0.006628034) * inp_1_1_2;
    result0 += M4(0.015617583, -0.005684133, 0.020754231, -0.031092007, -0.006861923, 0.004296511, 0.005940947, -0.007939817, -0.0059567126, 0.014643552, -0.025129901, -0.019457188, 0.0090089915, 0.0063058543, -0.0051312125, 0.04285886) * inp_1_2_2;
    const V4 inp_2_0_0 = inp[2][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_2_1_0 = inp[2][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_2_2_0 = inp[2][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_2_0_1 = inp[2][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_2_1_1 = inp[2][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_2_2_1 = inp[2][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_2_0_2 = inp[2][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_2_1_2 = inp[2][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_2_2_2 = inp[2][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.056146212, -0.029993707, -0.0029946084, 0.009597066, -0.0055773505, -0.014496897, 0.0053008134, 0.03380927, -0.00857316, -0.024327002, -0.0077396543, 0.023093903, 0.016923629, 0.011660726, -0.017473694, 0.012278802) * inp_2_0_0;
    result0 += M4(-0.015310294, 0.013877311, 0.059517235, 0.10568639, -0.023764415, -0.015286063, -0.03247092, 0.0084537985, -0.0057531856, 0.0053891786, 0.0005023959, 0.01207993, 0.06533402, -0.009690131, -0.00534058, -0.02281885) * inp_2_1_0;
    result0 += M4(0.012439267, -0.054563586, 0.014124557, -0.0042888303, -0.018063948, -0.009083126, -0.019351825, -0.045898348, 0.029926755, 0.03754743, -0.0007531956, -0.0053502675, 0.01938234, 0.108528994, -0.009830306, 0.0233738) * inp_2_2_0;
    result0 += M4(0.048626624, 0.048052464, 0.034028225, 0.011379155, 0.023351762, -0.08459995, 0.007998285, -0.048312, -0.037684888, -0.038798638, -0.015584144, -0.008591416, -0.03722752, 0.047289547, -0.014998745, 0.019503634) * inp_2_0_1;
    result0 += M4(0.009321066, -0.061791185, -0.05614355, -0.09954208, 0.15573722, 0.08033231, 0.16624257, 0.039115697, -0.015057292, 0.060993157, 0.02775619, -0.009678067, 0.08701264, -0.19015558, 0.16736636, -0.033601154) * inp_2_1_1;
    result0 += M4(0.021980716, 0.06545731, -0.0016990079, -0.012586056, -0.15282418, -0.024230676, -0.14616908, 0.0031548773, 0.007949588, -0.019104335, -0.012806655, -0.014541425, -0.093731314, 0.0055443007, -0.0916258, -0.011105488) * inp_2_2_1;
    result0 += M4(-0.0070524802, -0.015206575, -0.024987424, -0.020959001, 0.02742666, 0.026945539, 0.035545178, 0.0038931908, 0.0050143832, 0.01858862, 0.021417262, 0.0024609163, 0.012026831, 0.014149429, -0.0010996889, 0.016795497) * inp_2_0_2;
    result0 += M4(0.039286576, 0.032172065, -0.021925751, -0.01727614, 0.020542376, 0.015991235, 0.0099482415, -0.08519835, -0.003963433, -0.040830042, -0.014214168, -0.0111057935, -0.0025819058, -0.005014527, 0.06998702, -0.03203197) * inp_2_1_2;
    result0 += M4(-0.015863057, -0.004233235, 0.00781192, -0.0075804996, -0.033949554, 0.0012959616, -0.059760123, 0.061610345, 0.01659447, -0.001190921, -0.005174426, 0.0029767347, -0.039155435, -0.020467311, -0.04002659, 0.0114872) * inp_2_2_2;
    const V4 inp_3_0_0 = inp[3][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_3_1_0 = inp[3][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_3_2_0 = inp[3][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_3_0_1 = inp[3][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_3_1_1 = inp[3][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_3_2_1 = inp[3][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_3_0_2 = inp[3][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_3_1_2 = inp[3][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_3_2_2 = inp[3][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(-0.015630241, -0.004027464, -0.013040066, -0.0149316685, 0.10156493, -0.0064434395, 0.019830583, -0.020290595, -0.018034365, -5.439517e-05, -0.0033242956, 0.018719414, 0.0027127375, -0.0013970506, 0.004377301, 0.018534634) * inp_3_0_0;
    result0 += M4(0.01897502, -0.0016047776, -0.0038379368, -0.02015278, 0.18910392, 0.21818724, 0.12807204, 0.15242544, -0.00014261382, 0.005834797, -0.0014806784, 0.009478976, 0.023083616, -0.012326605, 0.005725552, -0.0015032903) * inp_3_1_0;
    result0 += M4(0.0063696583, 0.017680557, 0.0053854575, 0.017272968, 0.10214523, 0.1748511, -0.0072864625, -0.022981353, 0.028337317, -0.01251157, -0.013479139, -0.021717139, -0.008391472, 0.02182073, 0.0131572625, 0.017339678) * inp_3_2_0;
    result0 += M4(-0.024813147, 0.025906507, -0.014236887, -0.004296818, 0.21192485, 0.088715844, 0.19871712, 0.1791554, 0.013340956, -0.012025656, 0.019359834, -0.018649103, -0.023994384, -0.011616201, 0.010661006, -0.0004192406) * inp_3_0_1;
    result0 += M4(0.010812841, -0.010695332, 0.015688086, 0.04565046, 0.15837257, 0.2391036, 0.20844008, 0.20032401, -0.0027581926, 0.0021465956, -0.0054261014, -0.01258115, 0.042213958, -0.012497854, -0.010795549, 0.0071023987) * inp_3_1_1;
    result0 += M4(-0.005182465, 0.01727754, -0.0019545401, 0.022294307, -0.013030307, 0.045218915, 0.06853269, 0.1635434, -0.026300864, 0.005450505, -0.002326413, 0.029002018, -0.016995488, 0.010266647, 0.022239, -0.048940584) * inp_3_2_1;
    result0 += M4(-0.0056235567, -0.0051563876, -0.025468364, -0.01892005, 0.033918757, 0.048554055, 0.12677808, 0.04554534, -0.0005792802, -0.007605645, -0.020921718, 0.015139634, -0.0001280151, -0.01703925, -0.007870343, 0.013898517) * inp_3_0_2;
    result0 += M4(0.006422349, -0.0013431243, 0.01007344, 0.01188304, 0.21099241, 0.16251329, 0.21553156, 0.116644464, 0.0028521153, -0.0078109116, -0.0016409477, -0.009481059, -0.018767575, 0.0106040435, -0.0054676714, -0.008941914) * inp_3_1_2;
    result0 += M4(-0.005620965, -0.00614183, -0.0062092785, -0.010327982, 0.006019252, 0.036637742, 0.042170476, 0.19119851, -0.0007023228, 0.008340826, 0.03510382, -0.009136804, -0.0010402815, 0.008920552, -0.01813815, -0.017552188) * inp_3_2_2;
    const V4 inp_4_0_0 = inp[4][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_4_1_0 = inp[4][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_4_2_0 = inp[4][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_4_0_1 = inp[4][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_4_1_1 = inp[4][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_4_2_1 = inp[4][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_4_0_2 = inp[4][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_4_1_2 = inp[4][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_4_2_2 = inp[4][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.013893306, -0.0048349984, -0.031820174, 0.0025733542, 0.0004327661, 0.025202326, -0.008798959, 0.0036201512, 0.041549608, 0.0151235, -0.003692209, 0.00796003, 0.04019586, 0.0008560334, 0.008096168, 0.0019705014) * inp_4_0_0;
    result0 += M4(0.019484479, -0.0028440675, -0.014953286, 0.014799095, 0.004235063, -0.020486385, -0.00013871345, 0.025786685, -0.03914735, 0.0030271448, -0.008248057, -0.027490713, -0.015632492, -0.012201471, 0.0072807353, 0.0042271307) * inp_4_1_0;
    result0 += M4(-0.008552535, -0.021086277, -0.0065986775, 0.019903269, -0.0141293425, 0.0028152133, 0.0018514814, 8.292023e-05, 0.01095558, -0.010994559, 0.0061708805, 0.028621605, 0.016100785, -0.0174511, 0.005643692, -0.024656218) * inp_4_2_0;
    result0 += M4(0.0011575994, -0.010874013, 0.026106633, -0.0112061, -0.004384862, -0.010548679, -0.0023069258, -0.014571037, -0.026060622, -0.00032884735, 0.005964445, -0.013082362, -0.029111348, -0.014322494, -0.0045429138, 0.024419025) * inp_4_0_1;
    result0 += M4(-0.017054884, -0.014399408, 0.03411677, -0.031615075, -0.019143494, 0.021843094, -0.003341748, -0.0040443297, 0.06553026, -0.02028375, -0.052841417, 0.061606303, 0.0010276696, 0.02943414, 0.028999746, -0.030752156) * inp_4_1_1;
    result0 += M4(0.019169858, 0.030502386, -0.00076883804, -0.027899202, 0.0030963065, -0.010482338, 0.011389055, 0.012175334, -0.02820273, 0.033937227, 0.03657266, -0.04560631, 0.023403335, 0.031023378, -0.0013174645, 0.03458853) * inp_4_2_1;
    result0 += M4(0.0027003894, 0.016650828, -0.0060148695, 0.0042626765, 0.02074511, -0.0055924254, -0.0042904564, -0.0025879983, 0.008467807, 0.0075940527, 0.013750776, 0.002834788, 0.016045049, 0.015451945, -0.008224602, -0.0032328696) * inp_4_0_2;
    result0 += M4(-0.01157004, 0.009217196, -0.0013347088, 0.014203428, -0.002967931, -0.015940513, 0.009886501, -0.019522898, -0.008242241, 0.0021916782, 0.02255344, -0.032271955, -0.008016589, -0.0024743383, -0.032983776, 0.015665041) * inp_4_1_2;
    result0 += M4(-0.0051072496, -0.0040505105, 0.0046668346, 0.01679584, 0.008696501, -0.0038146232, 0.00910716, 0.012045967, -0.0050979746, -0.024309596, -0.017838743, -0.0033193233, -0.031412996, -0.016320242, -0.009205932, -0.037352126) * inp_4_2_2;
    const V4 inp_5_0_0 = inp[5][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_5_1_0 = inp[5][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_5_2_0 = inp[5][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_5_0_1 = inp[5][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_5_1_1 = inp[5][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_5_2_1 = inp[5][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_5_0_2 = inp[5][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_5_1_2 = inp[5][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_5_2_2 = inp[5][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.027820561, -0.02793066, -0.013501581, -0.015864102, -0.006547303, -0.000542337, 0.00632699, -0.001991439, 0.008934065, 0.04353298, 0.034430444, 0.017816298, 0.028953703, 0.0048226872, -0.022395154, -0.0002800703) * inp_5_0_0;
    result0 += M4(0.010114456, -0.0044486295, -0.0017837835, 0.004097175, 0.010056056, -0.012640788, -0.012164849, 0.0067934925, -0.0064451722, -0.045882974, -0.018771708, 0.0018019635, -0.009992078, 0.0054342854, 0.009569196, 0.0139673) * inp_5_1_0;
    result0 += M4(-0.021084445, 0.004841246, 0.008306838, 0.0018763344, 0.007579991, 0.002484884, -0.00039603136, 0.0094184, -0.005778353, 0.013610208, -0.010265976, 0.00017188815, -6.106693e-06, -0.010151133, 0.0013259958, -0.017957645) * inp_5_2_0;
    result0 += M4(0.039437298, -0.0054160627, -0.0516676, 0.03409567, 0.010467291, -0.004348648, 0.008717012, 0.029355792, 0.040704947, -0.0091088535, -0.024377175, 0.011594595, -0.020551365, 0.020170001, 0.0074305297, 0.021749197) * inp_5_0_1;
    result0 += M4(-0.030145658, 0.014910378, 0.016582267, 0.015885213, -0.046683863, 0.05260656, 0.034547742, -0.040888116, -0.0210968, 0.025809785, 0.034318626, -0.032854818, 0.002255369, -0.0026824852, 0.014977016, -0.011318095) * inp_5_1_1;
    result0 += M4(-0.015863132, 0.01819936, -0.0053539197, -0.012231695, -0.01731845, 0.0052841953, -0.006579735, -0.01680039, 0.0037970368, -0.022472411, 0.0017818498, -0.017422205, -0.022085179, -0.028481942, -0.01754122, 0.012227851) * inp_5_2_1;
    result0 += M4(-0.0077444427, 0.018961372, 0.028114628, 0.010600614, -0.009296155, -0.008243337, -0.020174136, -0.009059731, -0.015338912, 0.0031580075, 0.0087196985, -0.021547858, -0.016197404, -0.034627248, 0.025085533, -0.028402183) * inp_5_0_2;
    result0 += M4(-0.0057799383, -0.0028766468, 0.011339992, -0.035530515, 0.009545185, -0.0057790186, 0.018869113, 0.00044697704, 0.020529214, -0.012707417, -0.038278, 0.014514243, 0.0022907425, -0.009136833, -0.018564567, -0.0045068264) * inp_5_1_2;
    result0 += M4(0.01909896, -0.016489917, 0.010907225, -0.029027017, 0.012311409, -0.0030599562, 5.4453878e-05, 0.0009189634, -0.0019219625, 0.010293774, 0.0022204556, 0.0044286586, 0.01615638, 0.023709055, 0.015776101, 0.018450683) * inp_5_2_2;
    const V4 inp_6_0_0 = inp[6][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_6_1_0 = inp[6][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_6_2_0 = inp[6][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_6_0_1 = inp[6][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_6_1_1 = inp[6][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_6_2_1 = inp[6][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_6_0_2 = inp[6][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_6_1_2 = inp[6][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_6_2_2 = inp[6][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.05268083, 0.033097517, 0.025801184, 0.018795751, -0.00033200573, -0.010791032, 0.0145518305, -0.0023110148, 0.050155148, 0.03884765, 0.010799786, 0.030547308, 0.026148258, 0.016694741, 0.023759741, -0.0009179383) * inp_6_0_0;
    result0 += M4(-0.013379363, 0.008767712, -0.028095828, -0.034962103, -0.007935414, -0.0015933524, 0.00075986463, 0.0034700374, -0.10289263, -0.092444375, 0.017743062, 0.010768255, 0.091892675, 0.04987667, 0.14154665, 0.124061674) * inp_6_1_0;
    result0 += M4(-0.034626655, -0.03908912, 0.0028543037, -0.012291055, 0.005483564, -0.0023928608, -0.008323329, 0.01397509, 0.015025153, -0.008904496, 0.0011611413, -9.678023e-05, 0.042473014, 0.077103846, 0.011852508, 0.0045177545) * inp_6_2_0;
    result0 += M4(-0.031192627, -0.026278075, 0.0024595393, -0.03131032, -0.0069494, 0.021021659, 0.0008697018, 0.016067944, 0.065482125, 0.10283658, 0.09163405, 0.18885237, 0.031839173, 0.038518276, -0.0019447267, 0.053981863) * inp_6_0_1;
    result0 += M4(0.026757373, 0.004271728, 0.01721111, 0.008279828, 0.016287426, -0.0083296485, 0.019539686, 0.00081175024, -0.23570544, -0.21735306, -0.3797498, -0.3729832, -0.22064926, -0.1645878, -0.1528784, -0.1466617) * inp_6_1_1;
    result0 += M4(0.001866709, 0.022707518, -0.031606447, 0.048108928, 0.008432527, -0.039188948, -0.019350346, 0.004091359, -0.15355937, -0.12931582, -0.10710922, -0.07864407, -0.021158725, -0.032392215, -0.03676639, -0.0012169054) * inp_6_2_1;
    result0 += M4(-0.010820835, 0.00045228738, -0.038120262, 0.0039238287, 0.020347841, 0.0075789783, -0.0024825044, -0.007621447, 0.10844908, 0.11019733, 0.12427308, 0.12653096, 0.013490069, 0.014458239, 0.026704794, -0.0006417239) * inp_6_0_2;
    result0 += M4(-0.016052837, -0.013295947, 0.041586954, 0.03125826, -0.020099074, -0.0037973397, 0.0046050916, 0.01306901, 0.2478633, 0.20080535, 0.19864875, 0.034438323, 0.0109496955, 0.006476489, 0.006511561, 0.013280712) * inp_6_1_2;
    result0 += M4(0.029822977, 0.00036307416, 0.0015841565, -0.045890458, -0.013613035, 0.014434638, 0.01707951, -0.03145169, 0.017191472, 0.045044795, -0.0020277663, 0.058278434, -0.017361179, -0.0067594997, 0.010170386, 0.026200699) * inp_6_2_2;
    const V4 inp_7_0_0 = inp[7][local_xy.y + 0][local_xy.x + 0];
    const V4 inp_7_1_0 = inp[7][local_xy.y + 0][local_xy.x + 1];
    const V4 inp_7_2_0 = inp[7][local_xy.y + 0][local_xy.x + 2];
    const V4 inp_7_0_1 = inp[7][local_xy.y + 1][local_xy.x + 0];
    const V4 inp_7_1_1 = inp[7][local_xy.y + 1][local_xy.x + 1];
    const V4 inp_7_2_1 = inp[7][local_xy.y + 1][local_xy.x + 2];
    const V4 inp_7_0_2 = inp[7][local_xy.y + 2][local_xy.x + 0];
    const V4 inp_7_1_2 = inp[7][local_xy.y + 2][local_xy.x + 1];
    const V4 inp_7_2_2 = inp[7][local_xy.y + 2][local_xy.x + 2];
    result0 += M4(0.0012761244, -0.005859113, -0.014875662, -0.019149242, 0.023506522, -0.005479092, -0.009192156, 0.004806527, 0.053095397, -0.006551251, -0.0008956963, -0.007564479, 0.012062057, -0.0072817644, 0.0071839504, -0.009064106) * inp_7_0_0;
    result0 += M4(-0.007507553, -0.04516494, 0.009630113, 0.007888298, 0.11041952, 0.07235009, 0.047095772, 0.061580453, 0.013059472, 0.034824733, -0.0360668, -0.03198619, 0.028988112, -0.0049120826, 0.003229605, -0.018386068) * inp_7_1_0;
    result0 += M4(0.017141782, -0.002708708, -0.008868139, 0.016924277, -0.038493343, 0.002271269, -0.0052672205, -0.024071466, 0.013291609, 0.025129493, -0.007041967, 0.009878292, -0.0045442227, -0.0013242495, -0.0049446616, 0.019907102) * inp_7_2_0;
    result0 += M4(0.008044816, 0.02490562, 0.016403439, -0.0202737, 0.14466463, -0.0069114654, 0.057702485, -0.079055466, 0.018155048, 0.0025477055, 0.060546197, -0.03247536, -0.011425671, 0.0045189285, -0.006668489, 0.010629509) * inp_7_0_1;
    result0 += M4(-0.033897575, -0.008572021, 0.016179547, 0.056133516, 0.3826207, 0.44365427, 0.23990424, 0.4141474, -0.21782464, -0.051082548, 0.043562017, 0.19841976, 0.015480979, 0.041348774, -0.0076051294, 0.002175263) * inp_7_1_1;
    result0 += M4(-0.023199487, 0.026480895, 0.0028926819, -0.03454182, -0.18300375, -0.020270674, -0.21117868, -0.15674476, -0.0017998322, -0.077307075, -0.0064701913, -0.026851816, -0.02167514, 0.0024356388, 0.004492397, -0.011491041) * inp_7_2_1;
    result0 += M4(-0.00014212566, 0.001552669, -0.016092839, -0.010925387, -0.09090814, -0.07715102, 0.032531474, -0.059876975, 0.026333705, 0.0117245605, -0.00027149278, -0.0032350302, -0.0090701, -0.00029662394, 0.008493824, 0.0015323613) * inp_7_0_2;
    result0 += M4(-0.0047064396, -0.02491048, 0.021267803, 0.03389893, -0.2096372, -0.2549729, -0.0019944997, -0.03575328, 0.038211614, 0.012425436, -0.057891406, -0.050214842, 0.006670089, -0.018298648, -0.041759048, -0.015510323) * inp_7_1_2;
    result0 += M4(0.012256931, 0.024988838, -0.0046809795, -0.004015869, -0.14823508, -0.14451814, -0.17057183, -0.12097658, 0.037211217, 0.03324923, 0.01317652, -0.039975446, 0.0043269354, 0.003946722, 0.021585535, 0.0025453577) * inp_7_2_2;
    const ivec2 output_base = ivec2(gl_GlobalInvocationID) * ivec2(1, 1);
    imageStore(out_image, output_base + ivec2(0, 0), result0);
}

//!DESC ArtCNN C4F32 DS (Depth-To-Space)
//!COMPUTE 12 16 12 16
//!HOOK LUMA
//!BIND conv2d_6
//!WIDTH LUMA.w 2.0 *
//!HEIGHT LUMA.h 2.0 *
//!COMPONENTS 4
//!WHEN OUTPUT.w LUMA.w / 1.3 > OUTPUT.h LUMA.h / 1.3 > *
#extension GL_EXT_shader_explicit_arithmetic_types_float16 : enable
#ifdef GL_EXT_shader_explicit_arithmetic_types_float16
#	define V4 f16vec4
#	define M4 f16mat4
#	define F float16_t
#else
#	define V4 vec4
#	define M4 mat4
#	define F float
#endif

void hook() {
    vec4 result = vec4(0.0, 0.0, 0.0, 1.0);
    vec2 f0 = fract(conv2d_6_pos * conv2d_6_size);
    ivec2 i0 = ivec2(f0 * vec2(2.0));
    result.x = conv2d_6_tex((vec2(0.5) - f0) * conv2d_6_pt + conv2d_6_pos)[i0.y * 2 + i0.x];
    imageStore(out_image, ivec2(gl_GlobalInvocationID), clamp(result, 0.0, 1.0));
}
